#include <memory>

#include "MCS_Array.h"

namespace mcs {

// constructor
// rectangular zero offset matrix
template<class T,Uint8 R> Array<T,R>::Array() : 
	rng_(),valid_(false),data_()
{
	alloc(IndexRange<R>());
}

template<class T,Uint8 R> Array<T,R>::Array(const IndexRange<R>& rng,const T& val) : 
	rng_(),valid_(false),data_()
{
	alloc(rng);
	set(val);
}

template<class T,Uint8 R> Array<T,R>::Array(const IndexRange<R>& rng,const std::string& str_init) : 
	rng_(),valid_(false),data_()
{
	alloc(rng);
	set(str_init);
}

template<class T,Uint8 R> Array<T,R>::Array(const IndexRange<R>& rng,const T* data) :
	rng_(),valid_(false),data_()
{
	alloc(rng);
	set(data);
}

template<class T,Uint8 R> Array<T,R>::Array(const Array<T,R>& source) :
	rng_(source.rng_),valid_(source.valid_),data_(source.data_)
{ }

//destructor
template<class T,Uint8 R> Array<T,R>::~Array()
{ 
	dealloc(); 
}

template<class T,Uint8 R> Array<T,R>& Array<T,R>::operator=(const Array<T,R>& source)
{
	if (this == &source) return *this;
	
	rng_ = source.rng_;
	valid_ = source.valid_;
	data_ = source.data_;
	
	return *this;
}

// rectangular arbitrary offset matrix
template<class T,Uint8 R> void Array<T,R>::alloc(const IndexRange<R>& rng)
{
	if (all(rng.dim() >= 1))
	{
		rng_ = rng;
		
		try
		{
			const Int32	num_elem = product(rng_.dim());
			
			data_ = Vector<T>(IndexRange<1>(num_elem));
			valid_ = true;
		}
		catch (...) 
		{ 
			rng_ = IndexRange<R>(); 
			valid_ = false;
			data_ = Vector<T>();
			throw MemoryErr(); 
		}
	}
	else
	{
		rng_ = IndexRange<R>();
		valid_ = false;
		data_ = Vector<T>();
	}
}

// deallocator
template<class T,Uint8 R> void Array<T,R>::dealloc()
{ 
	try
	{
		rng_ = IndexRange<R>();
		valid_ = false; 
	}
	catch (...)
	{
		//data_ = NULL;
		throw MemoryErr();
	}
}

// reallocator
template<class T,Uint8 R> void Array<T,R>::realloc(const IndexRange<R>& rng)
{
	// check to see if new range is same as old
	if (rng == rng_)
	{
		return;
	}
	// just change range if new dimensions are same as old
	else if (all(rng.dim() == rng_.dim()))
	{
		range(rng);
		return;
	}
	// otherwise actually reallocate
	else
	{
		dealloc();
		alloc(rng);
		return;
	}
}

template<class T,Uint8 R> inline void Array<T,R>::set(const T& val)
{
	data_ = val;
}

template<class T,Uint8 R> inline void Array<T,R>::set(const std::string& str_init)
{
	data_ = str_init;
}

template<class T,Uint8 R> inline void Array<T,R>::set(const T* data)
{
	data_ = Vector<T>(IndexRange<1>(elem()),data);
}

// range assignors
template<class T,Uint8 R> inline const Array<T,R>& Array<T,R>::offset(const Index<R>& offs) const
{	
	range(IndexRange<R>(offs,offs+dim()-1));
	
	return (*this);
}

template<class T,Uint8 R> inline const Array<T,R>& Array<T,R>::range(const IndexRange<R>& rng) const
{
	if (!isValid() || any(dim() != rng.dim())) throw SizeMismatchErr();
	
	rng_ = rng;
		
	return (*this);
}

// range accessors
// access data element
template<class T,Uint8 R> inline Index<1> Array<T,R>::index(const Index<R>& idx) const
{
	return ArrayIndexer<R>::index(idx,rng_);
}

// element accessors
template<class T,Uint8 R> inline T& Array<T,R>::operator()(const Index<R>& idx)
{
	return data_[index(idx)]; 
}

template<class T,Uint8 R> inline const T& Array<T,R>::operator()(const Index<R>& idx) const
{
	return data_[index(idx)]; 
}

/*
	Utility class for Arrays which allows the offset index to be temporarily changed 
		within a local block of code, with the destructor resetting it to its original value
		on exit.  Usage :
		
			if (true)
			{
				Array<T,R>::Offset	offs(m1,o1,o2);
				
				...do some stuff...
			}
*/
template<class T,Uint8 R> class Array<T,R>::Offset
{
	public:
		Offset(const Array<T,R>& tns,const Index<R>& off) : tns_(tns) { ooff_ = tns_.offset(); tns_.offset(off); }
		~Offset() { tns_.offset(ooff_); }
		
		const Index<R>& currentOffset() const	{ return tns_.first(); }
		const Index<R>& originalOffset() const	{ return ooff_; }
		
	private:
		Offset();
		Offset(const Offset&);
		operator=(const Offset&);
		
		const Array<T,R>&	tns_;
		Index<R>			ooff_;
};		

} // namespace mcs