#ifndef MCS_ARRAY_H
#define MCS_ARRAY_H

#include <iostream>
#include <string>

#include "MCS_Ensure.h"
#include "MCS_Exceptions.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

__DECLARE_EXCEPTION_CLASS__(ArrayException,Exception)

// rank R array
template<class T,Uint8 R = 3> class Array
{
	public:	
		typedef Array<T,R>&			reference;
		typedef const Array<T,R>&	const_reference;
		
		// ctor/dtor	
		Array();
		explicit Array(const IndexRange<R>&,const T& = T());
		explicit Array(const IndexRange<R>&,const std::string&);
		explicit Array(const IndexRange<R>&,const T*);
		Array(const Array<T,R>&);
		virtual ~Array();
		
		Array<T,R>& 			operator=(const Array<T,R>&);
		
		// range accessors	
				bool			isValid() const							{ return valid_; }
				
				Uint32			elem() const							{ return data_.dim(); }
				
		const 	Index<R>& 		first() const							{ return rng_.first(); }
		const	Index<R>& 		last() const							{ return rng_.last(); }
		const	Index<R>& 		dim() const								{ return rng_.dim(); }
		const	Index<R>&		offset() const							{ return rng_.offset(); }
			
		const	IndexRange<R>&	range() const							{ return rng_; }

			 	Index<1> 		first(Uint8 d) const					{ return rng_.first()[d]; }
				Index<1> 		last(Uint8 d) const						{ return rng_.last()[d]; }
				Index<1> 		dim(Uint8 d) const						{ return rng_.dim()[d]; }
				Index<1>		offset(Uint8 d) const					{ return rng_.offset()[d]; }
			
				IndexRange<1>	range(Uint8 d) const					{ return rng_.range(d); }
				
		// range assignors
		const	Array<T,R>& 	offset(const Index<R>&) const;
		const	Array<T,R>& 	range(const IndexRange<R>&) const;
		
		// element accessors		
			 	T&				operator()(const Index<R>&);
		const	T&				operator()(const Index<R>&) const;

		// IO
				void			print(std::ostream& os) const;
				
				Array<T,R>& 	get(std::ifstream& ifs);
				Array<T,R>& 	read(std::ifstream& ifs);
				
		const 	Array<T,R>& 	put(std::ofstream& ofs) const;
		const 	Array<T,R>& 	write(std::ofstream& ofs) const;
		
		// offset class to allow offset change within local block with cleanup
		class Offset;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,ArrayException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)

	protected:
		// allocators
		virtual void 			alloc(const IndexRange<R>&);
		virtual void 			dealloc();
		virtual void 			realloc(const IndexRange<R>& rng = IndexRange<R>());
				
				void 			set(const T&);
				void			set(const std::string&);
				void			set(const T*);
				
	private:
				Index<1> 		index(const Index<R>&) const;
				
				bool			valid_;
				Vector<T>		data_;
		mutable IndexRange<R>	rng_;	 
};

// support class
template<Uint8 R> class ArrayIndexer
{
	public:
		static Index<1> index(const Index<R>& idx,const IndexRange<R>& rng);
};

template<Uint8 R>
inline Index<1> ArrayIndexer<R>::index(const Index<R>& idx,const IndexRange<R>& rng)
{
	FixedVector<Int32,R>	size(rng.dim(R-1));
	
	for (Int32 i=R-2;i>=0;i--)
		size[i] = size[i+1]*rng.dim(i);
	
	Index<1>	outidx(0);
	
	for (Int32 i=0;i<R-1;i++)
		outidx += (idx[i]-rng.first(i))*size[i+1];
		
	return (outidx += (idx[R-1]-rng.first(R-1)));
}

template<> class ArrayIndexer<1>
{
	public:
		static Index<1> index(const Index<1>& idx,const IndexRange<1>& rng);
};

template<>
inline Index<1> ArrayIndexer<1>::index(const Index<1>& idx,const IndexRange<1>& rng)
{
	return idx-rng.first(0);
}

} // namespace mcs

// implementation
#include "MCS_Array.cpp"
#include "MCS_ArrayPtrSpecialized.cpp"

#endif // MCS_ARRAY_H