#include <memory>

#include "MCS_Array.h"

namespace mcs {

template<class T,Uint8 R> class Array<T*,R>
{
	public:	
		class SmartPtr;
		
		typedef Array<T*,R>&		reference;
		typedef const Array<T*,R>&	const_reference;
		
		// ctor/dtor	
		Array();
		explicit Array(const IndexRange<R>&);
		explicit Array(const IndexRange<R>&,const T&);
		virtual ~Array();
		
		// range accessors	
				bool			isValid() const							{ return valid_; }
				
				Uint32			elem() const							{ return data_.dim(); }
				
		const 	Index<R>& 		first() const							{ return rng_.first(); }
		const	Index<R>& 		last() const							{ return rng_.last(); }
		const	Index<R>& 		dim() const								{ return rng_.dim(); }
		const	Index<R>&		offset() const							{ return rng_.offset(); }
			
		const	IndexRange<R>&	range() const							{ return rng_; }
		
		// range assignors
		const	Array<T*,R>& 	offset(const Index<R>&) const;
		const	Array<T*,R>& 	range(const IndexRange<R>&) const;
		
		// element accessors		
				SmartPtr& 		operator()(const Index<R>&);
		const 	SmartPtr&		operator()(const Index<R>&) const;

		// IO
				void			print(std::ostream& os) const;
				
				Array<T*,R>& 	get(std::ifstream& ifs);
				Array<T*,R>& 	read(std::ifstream& ifs);
				
		const 	Array<T*,R>& 	put(std::ofstream& ofs) const;
		const 	Array<T*,R>& 	write(std::ofstream& ofs) const;
		
		// offset class to allow offset change within local block with cleanup
		class Offset;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,ArrayException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)

	protected:
		// allocators
		virtual void 			alloc(const IndexRange<R>&);
		virtual void 			dealloc();
		virtual void 			realloc(const IndexRange<R>& rng = IndexRange<R>());
							
				void 			set(const T&);
				
	private:
		Array(const Array<T*,R>&);
		Array<T*,R>& operator=(const Array<T*,R>&);
		
				Index<1> 		index(const Index<R>&) const;
				
				bool				valid_;
				SmartPtr*			data_;
				//Vector<SmartPtr>	data_;
		mutable IndexRange<R>		rng_;	 
};

// constructor
// rectangular zero offset matrix
template<class T,Uint8 R> Array<T*,R>::Array() : 
	rng_(),valid_(false),data_()
{
	alloc(IndexRange<R>());
}

template<class T,Uint8 R> Array<T*,R>::Array(const IndexRange<R>& rng) : 
	rng_(),valid_(false),data_()
{
	alloc(rng);
}

template<class T,Uint8 R> Array<T*,R>::Array(const IndexRange<R>& rng,const T& val) : 
	rng_(),valid_(false),data_()
{
	alloc(rng);
	set(val);
}

//destructor
template<class T,Uint8 R> Array<T*,R>::~Array()
{ 
	dealloc(); 
}

// rectangular arbitrary offset matrix
template<class T,Uint8 R> void Array<T*,R>::alloc(const IndexRange<R>& rng)
{
	if (all(rng.dim() >= 1))
	{
		rng_ = rng;
		
		try
		{
			const Int32	num_elem = product(rng_.dim());
			
			//data_ = Vector<SmartPtr>(IndexRange<1>(num_elem));
			data_ = new SmartPtr[num_elem];
			valid_ = true;
		}
		catch (...) 
		{ 
			rng_ = IndexRange<R>(); 
			valid_ = false;
			//data_ = Vector<SmartPtr>();
			data_ = NULL;
			throw MemoryErr(); 
		}
	}
	else
	{
		rng_ = IndexRange<R>();
		valid_ = false;
		//data_ = Vector<SmartPtr>();
		data_ = NULL;
	}
}

// deallocator
template<class T,Uint8 R> void Array<T*,R>::dealloc()
{ 
	try
	{
		delete[] data_;
		rng_ = IndexRange<R>();
		valid_ = false; 
	}
	catch (...)
	{
		throw MemoryErr();
	}
}

// reallocator
template<class T,Uint8 R> void Array<T*,R>::realloc(const IndexRange<R>& rng)
{
	// check to see if new range is same as old
	if (rng == rng_)
	{
		return;
	}
	// just change range if new dimensions are same as old
	else if (all(rng.dim() == rng_.dim()))
	{
		range(rng);
		return;
	}
	// otherwise actually reallocate
	else
	{
		dealloc();
		alloc(rng);
		return;
	}
}

template<class T,Uint8 R> inline void Array<T*,R>::set(const T& val)
{
	for (Index<1> i=data_.first();i<=data_.last();++i) data_[i] = new T(val);
}

// range assignors
template<class T,Uint8 R> inline const Array<T*,R>& Array<T*,R>::offset(const Index<R>& offs) const
{	
	range(IndexRange<R>(offs,offs+dim()-1));
	
	return (*this);
}

template<class T,Uint8 R> inline const Array<T*,R>& Array<T*,R>::range(const IndexRange<R>& rng) const
{
	if (!isValid() || any(dim() != rng.dim())) throw SizeMismatchErr();
	
	rng_ = rng;
		
	return (*this);
}

// range accessors
// access data element
template<class T,Uint8 R> inline Index<1> Array<T*,R>::index(const Index<R>& idx) const
{
	return ArrayIndexer<R>::index(idx,rng_);
}

// element accessors
template<class T,Uint8 R> inline typename Array<T*,R>::SmartPtr& Array<T*,R>::operator()(const Index<R>& idx)
{
	const Index<1>	vecidx = index(idx);
	
	return data_[vecidx]; 
}

template<class T,Uint8 R> inline const typename Array<T*,R>::SmartPtr& Array<T*,R>::operator()(const Index<R>& idx) const
{
	const Index<1>	vecidx = index(idx);
	
	return data_[vecidx]; 
}

/*
	Utility class for Arrays which allows the offset index to be temporarily changed 
		within a local block of code, with the destructor resetting it to its original value
		on exit.  Usage :
		
			if (true)
			{
				Array<T*,R>::Offset	offs(m1,o1,o2);
				
				...do some stuff...
			}
*/
template<class T,Uint8 R> class Array<T*,R>::Offset
{
	public:
		Offset(const Array<T*,R>& tns,const Index<R>& off) : tns_(tns) { ooff_ = tns_.offset(); tns_.offset(off); }
		~Offset() { tns_.offset(ooff_); }
		
		const Index<R>& currentOffset() const	{ return tns_.first(); }
		const Index<R>& originalOffset() const	{ return ooff_; }
		
	private:
		Offset();
		Offset(const Offset&);
		operator=(const Offset&);
		
		const Array<T*,R>&	tns_;
		Index<R>			ooff_;
};		

template<class T,Uint8 R> class Array<T*,R>::SmartPtr
{
	public:
		SmartPtr() : ptr_(0) 					{ }
		~SmartPtr()								{ delete ptr_; }
		
		SmartPtr& operator=(const T& val)		{ delete ptr_; ptr_ = new T(val); return *this; }
		//SmartPtr& operator=(const T* ptr)		{ delete ptr_; ptr_ = ptr; return *this; }
		
		//T& operator*() const					{ return *ptr_; }
		T* operator->() const					{ return ptr_; }
	
	private:
		SmartPtr(const SmartPtr&);
		SmartPtr& operator=(const SmartPtr&);
		
		T*	ptr_;
};

} // namespace mcs