// class Date
//
//	implements double precision floating point (J.D.) date as defined in 
//	Calendrical Calculations (Dershowitz & Reingold, Cambridge University Press)
//
//	constructor guarantees that the specified date is valid, throwing an error if not
//
#ifndef MCS_DATE_H
#define MCS_DATE_H

#include <cmath>

#include "MCS_Day.h"
#include "MCS_Exceptions.h"
#include "MCS_Time.h"

namespace mcs {

class Date
{
	public:
		typedef	Float64		FloatType;
		//typedef	Float128		FloatType;
		
		// constructor/destructor
		explicit Date(const Day& = Day(),const Time& = Time());						// day and time
		explicit Date(Int32);														// fixed (RD) day
		explicit Date(Int32,Int32);													// year,calendar day
		explicit Date(Int32,Int32,Int32);											// year,month,day
		explicit Date(Int32,Int32,Int32,FloatType);									// year,month,day,seconds
		explicit Date(FloatType);													// julian (JD) day
		Date(const Date&);
		~Date() { }
		
		// assignment
		Date& operator=(const Date& val);	
		
		// accessors
		const Day& day() const					{ return DY_; }
		const Time& time() const				{ return TM_; }
				
		// assignors
		Date& day(const Day& dy)				{ return ((*this) = Date(dy,TM_)); }
		Date& time(const Time& tm)				{ return ((*this) = Date(DY_,tm)); }
		
		// forwarded members from Day
		Int32 fixed() const						{ return DY_.fixed(); }				// RD
		Int32 year() const						{ return DY_.year(); };				// year			anything but 0
		Int32 month() const						{ return DY_.month(); }				// month		1..12
		Int32 week() const						{ return DY_.week(); }				// week			1..52
		Int32 dayOfMonth() const				{ return DY_.dayOfMonth(); }		// day			1..days_in_month
		Int32 dayOfWeek() const					{ return DY_.dayOfWeek(); }			// weekday		0..6
		Int32 calendarDay() const				{ return DY_.calendarDay(); }		// calendar day	1..days_in_year

		// forwarded members from Time
		Int32 	hour() const					{ return TM_.hour(); }				// hour		0..23
		Int32 	minute() const					{ return TM_.minute(); }			// minute	0..59
		FloatType second() const				{ return TM_.second(); }			// second	0..59
		FloatType ticks() const					{ return TM_.ticks(); }				// seconds from midnight
		
		
		Int32 daysInYear() const				{ return DY_.daysInYear(); }
		Int32 daysInMonth() const				{ return DY_.daysInMonth(); }
		static Int32 daysInWeek() 				{ return Day::daysInWeek(); }
		
		bool isStartOfMonth() const				{ return DY_.isStartOfMonth(); }
		bool isEndOfMonth() const				{ return DY_.isEndOfMonth(); }
		bool isStartOfYear() const				{ return DY_.isStartOfYear(); }
		bool isEndOfYear() const				{ return DY_.isEndOfYear(); }
			
		bool isLeapYear() const					{ return DY_.isLeapYear(); }
		bool isLeapDay() const					{ return DY_.isLeapDay(); }
		
		// accessors
		FloatType julian() const				{ return JD_; }						// JD
		
		// arithmetic operations		
		Date& operator+=(FloatType val)			{ return ((*this) = Date(julian()+val)); }		
		Date& operator-=(FloatType val)			{ return ((*this) = Date(julian()-val)); }
		
		Date& operator++()						{ return ((*this) += 1); }									// prefix
		Date& operator--()						{ return ((*this) -= 1); }									// prefix
		const Date operator++(Int32)			{ const Date tdate(*this); ++(*this); return (*this); }		// postfix
		const Date operator--(Int32)			{ const Date tdate(*this); --(*this); return (*this); }		// postfix
		
		Date& addYears(Int32);
		Date& addMonths(Int32);
		Date& addDays(Int32);
		
		Date& addHours(Int32);
		Date& addMinutes(Int32);
		Date& addSeconds(FloatType);
			  
 		static Date today()						{ return Date(Day::today(),Time::UTC()); }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	private:
		static FloatType JD_START()				{ return -1721424.5; }
		
		void set_JD()							{ JD_ = DY_.fixed()+TM_.ticks()/Time::secondsInDay() - JD_START(); }
	
		FloatType	JD_;
		Day			DY_;
		Time		TM_;
};
		
// constructor/destructor
// default : set to beginning of January 1, 2000 
inline Date::Date(const Day& dy,const Time& tm) : 
	DY_(dy),TM_(tm)
{
	set_JD();
}

inline Date::Date(Int32 rd) : 
	DY_(rd),TM_()
{
	set_JD();
}

inline Date::Date(Int32 yr,Int32 jday) : 
	DY_(yr,jday),TM_()
{
	set_JD();		
}

inline Date::Date(Int32 yr,Int32 mo,Int32 dy) : 
	DY_(yr,mo,dy),TM_()
{
	set_JD();
}

inline Date::Date(Int32 yr,Int32 mo,Int32 dy,FloatType time) : 
	DY_(yr,mo,dy),TM_(time)
{
	set_JD();
}

inline Date::Date(FloatType jd) : 
	JD_(jd)
{
	DY_ = Day(static_cast<Int32>(std::floor(jd + JD_START())));
	TM_ = Time((jd + JD_START() - DY_.fixed())*Time::secondsInDay());
}

inline Date::Date(const Date& val) : 
	DY_(val.day()),TM_(val.time()),JD_(val.JD_)
{ }

inline Date& Date::operator=(const Date& val)
{ 
	if (this == &val) return (*this);
	
	DY_ = val.DY_;
	TM_ = val.TM_;
	JD_ = val.JD_;
	
	return (*this); 
}
	
// add/subtract days
inline Date& Date::addDays(Int32 days)
{ 
	return (days == 0) ? (*this) : ((*this) = Date(DY_.addDays(days),TM_)); 
}

inline Date& Date::addMonths(Int32 months)
{
	return (months == 0) ? (*this) : ((*this) = Date(DY_.addMonths(months),TM_));
}
	
inline Date& Date::addYears(Int32 years)
{
	return (years == 0) ? (*this) : ((*this) = Date(DY_.addYears(years),TM_));
}

inline Date& Date::addHours(Int32 hours)
{
	return (hours == 0) ? (*this) : ((*this) = Date(julian()+hours/Time::hoursInDay()));
}			

inline Date& Date::addMinutes(Int32 minutes)
{
	return (minutes == 0) ? (*this) : ((*this) = Date(julian()+minutes/Time::minutesInDay()));
}

inline Date& Date::addSeconds(FloatType seconds)
{
	return (seconds == 0) ? (*this) : ((*this) = Date(julian()+seconds/Time::secondsInDay()));
}		

// logical operators
inline bool operator==(const Date& lhs,const Date& rhs)						{ return (lhs.julian()==rhs.julian()); }
inline bool operator!=(const Date& lhs,const Date& rhs)						{ return (lhs.julian()!=rhs.julian()); }
inline bool operator<(const Date& lhs,const Date& rhs)						{ return (lhs.julian()<rhs.julian());  }
inline bool operator<=(const Date& lhs,const Date& rhs)						{ return (lhs.julian()<=rhs.julian()); }
inline bool operator>(const Date& lhs,const Date& rhs)						{ return (lhs.julian()>rhs.julian());  }
inline bool operator>=(const Date& lhs,const Date& rhs)						{ return (lhs.julian()>=rhs.julian()); }
		
// arithmetic operators
inline Date operator+(const Date& ep,const Date::FloatType& dt)				{ return Date(ep.julian()+dt); }
inline Date operator-(const Date& ep,const Date::FloatType& dt)				{ return Date(ep.julian()-dt); }

inline typename Date::FloatType operator-(const Date& lep,const Date& rep)	{ return lep.julian()-rep.julian(); }

} // namespace mcs

#endif // MCS_DATE_H