// Date-related utility functions
//
#ifndef MCS_DATEUTILITIES_H
#define MCS_DATEUTILITIES_H

#include <cmath>
#include <iomanip>
#include <iosfwd>
#include <string>

#include "MCS_Ensure.h"

#include "MCS_Date.h"
#include "MCS_Day.h"
#include "MCS_Time.h"

namespace mcs {

// add 1900 if short_year >= 1950, 2000 otherwise
template<class T> 
inline T Y2K(T short_year)
{
	static const EnsureInteger<T>	dummy_;
	
	return (short_year >= 50) ? short_year + 1900 : short_year + 2000; 
}

// convert to short_year
template<class T> 
inline T shortYear(T yr)
{
	static const EnsureInteger<T>	dummy_;
	
	return std::mod(yr,100); 
}

// return RD1987 as a number of days relative to (1987,1,1,0:0)
inline typename Date::FloatType toRD1987(const Date& ep)
{
	return (ep.julian()-Date(1987,1,1).julian()); 
}

// convert RD1987 to calendar date
inline Date fromRD1987(Date::FloatType rd1987)	
{ 
	return Date(rd1987+Date(1987,1,1).julian()); 
}

// return RD2000 as number of days relative to (2000,1,1,0:0)
inline typename Date::FloatType toRD2000(const Date& ep)				
{ 
	return (ep.julian()-Date(2000,1,1).julian()); 
}

// convert RD2000 to calendar date
inline Date fromRD2000(Date::FloatType rd2000)	
{ 
	return Date(Date(2000,1,1).julian()+rd2000);
}

// special days in month
inline Day beginningOfMonth(const Day& ep)
{
	return Day(ep.year(),ep.month(),1);
}

inline Day endOfMonth(const Day& ep)
{
	return Day(ep.year(),ep.month(),ep.daysInMonth());
}

inline Day middleOfMonth(const Day& ep)
{
	return Day((beginningOfMonth(ep).fixed()+endOfMonth(ep).fixed())/2);
}

inline Date beginningOfMonth(const Date& ep)
{
	return Date(Day(ep.year(),ep.month(),1),Time(0,0,0));
}

inline Date endOfMonth(const Date& ep)
{
	return Date(Day(ep.year(),ep.month(),ep.daysInMonth()),Time(23,59,59.99999999999));
}

inline Date middleOfMonth(const Date& ep)
{
	return Date(0.5*(beginningOfMonth(ep).julian()+endOfMonth(ep).julian()));
}

// determine Greenwich mean sidereal time, in radians	
inline typename Date::FloatType siderealTime(const Date& ep)
{
	static const Date::FloatType J2000_JULIAN = 0.5 + Date(2000,1,1).julian(); 
	
	const Date::FloatType	df = ep.time().ticks()/Time::secondsInDay(),	// fraction of day
							du = ep.julian()-df-J2000_JULIAN,				// wrt 2000/1/1 12h UT1
							Tu = du/36525.0,
							gmst = (24110.54841+Tu*(8640184.812866+Tu*(0.093104-Tu*6.2e-6)));
	
	return std::mod(gmst+1.00273790934*df*Time::secondsInDay(),static_cast<Date::FloatType>(Time::secondsInDay()))*_two_pi_/Time::secondsInDay();
}

// utility functions for various time intervals
typedef Int32 (*DayIntervalFunction)(const Day&);
typedef Int32 (*DateIntervalFunction)(const Date&);

// N-day interval
inline Int32 NadNumber(const Day& epoch,Float64 intervalLength)
{
	return static_cast<Int32>(std::floor(epoch.fixed()/intervalLength));
}

inline Int32 NadNumber(const Date& epoch,Float64 intervalLength)
{
	return static_cast<Int32>(std::floor(epoch.fixed()/intervalLength));
}

// half-day number
inline Int32 halfDayNumber(const Date& epoch)
{
	return epoch.time().hour() >= 12 ? epoch.fixed()*2+1 : epoch.fixed()*2;
}

// integer day number
inline Int32 dayNumber(const Day& epoch)
{
	return epoch.fixed();
}

inline Int32 dayNumber(const Date& epoch)
{
	return epoch.fixed();
}

// pentad number (5 day intervals)
inline Int32 pentadNumber(const Day& epoch)
{
	return NadNumber(epoch,5.0);
}

inline Int32 pentadNumber(const Date& epoch)
{
	return NadNumber(epoch,5.0);
}

// week number (7 day intervals)
inline Int32 weekNumber(const Day& epoch)
{
	return NadNumber(epoch,7.0);
}

inline Int32 weekNumber(const Date& epoch)
{
	return NadNumber(epoch,7.0);
}

// decad number (10 day intervals)
inline Int32 decadNumber(const Day& epoch)
{
	return NadNumber(epoch,10.0);
}

inline Int32 decadNumber(const Date& epoch)
{
	return static_cast<Int32>(std::floor(epoch.fixed()/10.0));
}

// month number (1 month intervals)
inline Int32 monthNumber(const Day& epoch)
{
	return static_cast<Int32>(epoch.year()*12 + epoch.month());
}

inline Int32 monthNumber(const Date& epoch)
{
	return static_cast<Int32>(epoch.year()*12 + epoch.month());
}

// season number (3 month intervals)
inline Int32 seasonNumber(const Day& epoch)
{
	return static_cast<Int32>(epoch.year()*4 + std::floor((epoch.month()-1)/3));
}

inline Int32 seasonNumber(const Date& epoch)
{
	return static_cast<Int32>(epoch.year()*4 + std::floor((epoch.month()-1)/3));
}

// year number 
inline Int32 yearNumber(const Day& epoch)
{
	return epoch.year();
}

inline Int32 yearNumber(const Date& epoch)
{
	return epoch.year();
}

// IO

// Time class IO
inline std::ostream& operator<<(std::ostream& ostr,const Time& tm)
{ 
	const Int32		isec(std::floor(tm.second()));
	const Float64	fsec(tm.second()-isec);
	
	//ostr << tm.hour() << ":" << tm.minute() << ":" << isec << ":" << fsec;
	ostr << std::setfill('0')
		 << std::setw(2) << tm.hour() << ":" 
		 << std::setw(2) << tm.minute() << ":"
		 << std::setw(2) << isec
		 << std::setfill(' ');

	return ostr; 
}

inline std::istream& operator>>(std::istream& istr,Time& dest)
{ 
	Int32	temp_hr,temp_min,temp_sec;
	
	istr >> temp_hr >> temp_min >> temp_sec;
	
	dest = Time(temp_hr,temp_min,temp_sec);
	
	return istr;
}

// Day class IO
inline std::string yearString(const Day& dy)			
{ 
	std::stringstream	sstr;
	
	sstr << dy.year();
	
	return sstr.str(); 
}	

inline std::string dayString(const Day& dy)
{
	static const std::string _DAY_STRING_[]   = { "Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday" };
	
	return _DAY_STRING_[dy.dayOfWeek()];
}
			
inline std::string monthString(const Day& dy)
{ 
	static const std::string _MONTH_STRING_[] = { "","January","February","March","April","May","June",
										          "July","August","September","October","November","December" };
										         
	return _MONTH_STRING_[dy.month()]; 
}

inline std::ostream& operator<<(std::ostream& ostr,const Day& dy)
{ 
	//ostr << dayString(dy) << " " << monthString(dy) << " " << dy.dayOfMonth() << ", " << dy.year();
	ostr << std::setfill('0') 
		 << std::setw(4) << dy.year() << "/"
		 << std::setw(2) << dy.month() << "/" 
		 << std::setw(2) << dy.dayOfMonth() 
		 << std::setfill(' ');

	return ostr; 
}

inline std::istream& operator>>(std::istream& istr,Day& dest)
{ 
	Int32	temp_yr,temp_mo,temp_dy;
	
	istr >> temp_yr >> temp_mo >> temp_dy;
	
	dest = Day(temp_yr,temp_mo,temp_dy);
	
	return istr;
}

// Date class IO
inline std::string yearString(const Date& ep)			
{ 
	return yearString(ep.day()); 
}	

inline std::string dayString(const Date& ep)
{
	return dayString(ep.day());
}
			
inline std::string monthString(const Date& ep)
{ 
	return monthString(ep.day()); 
}

inline std::ostream& operator<<(std::ostream& ostr,const Date& ep)
{ 
	ostr << ep.day() << " " << ep.time();

	return ostr; 
}

inline std::istream& operator>>(std::istream& istr,Date& dest)
{ 
	Int32	temp_yr,temp_mo,temp_dy;
	
	istr >> temp_yr >> temp_mo >> temp_dy;
	
	dest = Date(temp_yr,temp_mo,temp_dy);
	
	return istr;
}

} // namespace mcs

#endif // MCS_DATEUTILITIES_H