// class Day
//
//	implements year/month/day including fixed (RD) date as defined in 
//	Calendrical Calculations (Dershowitz & Reingold, Cambridge University Press)
//
//	constructor guarantees that the specified day is valid, throwing an error if not
//
//	constructors:	fixed (R.D.) day
//					year/calendar day (1..366)
//					year/month/day
//
#ifndef MCS_DAY_H
#define MCS_DAY_H

#include <cmath>

#include "MCS_Exceptions.h"
#include "MCS_MathExtensions.h"
#include "MCS_Typedefs.h"

namespace mcs {

enum DayOfWeek 	{ _SUN_ = 0, _MON_, _TUE_, _WED_, _THU_, _FRI_, _SAT_ };
enum Month 		{ _JAN_ = 1, _FEB_, _MAR_, _APR_, _MAY_, _JUN_, 
				  _JUL_, _AUG_, _SEP_, _OCT_, _NOV_, _DEC_ };
enum LongMonth 	{ JANUARY = 1, FEBRUARY, MARCH, APRIL, MAY, JUNE, 
				  JULY, AUGUST, SEPTEMBER, OCTOBER, NOVEMBER, DECEMBER };
		
// fundamental unit of time is the integer day
class Day
{
	public:
	
		// constructor/destructor
		Day();																		// Jan. 1, 1
		explicit Day(Int32);														// fixed (RD) day
		explicit Day(Int32,Int32);													// year,calendar day
		explicit Day(Int32,Int32,Int32);											// year,month,day
		Day(const Day&);
		~Day() { }
		
		// assignment
		Day& operator=(const Day& val);	
		
		// accessors
		Int32 fixed() const						{ return RD_; }						// RD
		Int32 year() const						{ return yr_; }						// year		anything but 0
		Int32 month() const						{ return mo_; }						// month	1..12
		Int32 week() const						{ return (calendarDay()-1)/7+1; }	// week		1..52 (53 in leap year)
		Int32 dayOfMonth() const				{ return dy_; }						// day		1..days_in_month
		Int32 dayOfWeek() const					{ return std::mod(RD_,7); }				// weekday	0..6
		
		Int32 calendarDay() const				{ return (fixed() - Day(year()-1,_DEC_,31).fixed()); }
		
		Int32 daysInYear() const				{ return (isLeapYear() ? 366 : 365); }
		Int32 daysInMonth() const;
		static Int32 daysInWeek() 				{ return 7; }
		
		bool isStartOfMonth() const				{ return (dayOfMonth() == 1); }
		bool isEndOfMonth() const				{ return (dayOfMonth() == daysInMonth()); }
		bool isStartOfYear() const				{ return (month() == _JAN_ && isStartOfMonth()); }
		bool isEndOfYear() const				{ return (month() == _DEC_ && isEndOfMonth()); }
			
		bool isLeapYear() const					{ return (std::mod(year(),4) == 0) && ((std::mod(year(),100) != 0) || (std::mod(year(),400) == 0)); }
		bool isLeapDay() const					{ return (isLeapYear() && month() == _FEB_ && isEndOfMonth()); }
		
		// arithmetic operations		
		Day& operator+=(Int32 val)				{ return ((*this) = Day(fixed()+val)); }		
		Day& operator-=(Int32 val)				{ return ((*this) = Day(fixed()-val)); }
		
		Day& operator++()						{ return ((*this) += 1); }								// prefix
		Day& operator--()						{ return ((*this) -= 1); }								// prefix
		const Day operator++(Int32)				{ const Day tday(*this); ++(*this); return tday; }		// postfix
		const Day operator--(Int32)				{ const Day tday(*this); --(*this); return tday; }		// postfix
		
		Day& addYears(Int32);
		Day& addMonths(Int32);
		Day& addDays(Int32);
			  
		static Day today();						// get today's date from internal clock
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		static Int32 EPOCH()					{ return 1; }
		
		void gregorian_to_fixed();
		void fixed_to_gregorian();
		Int32 year_from_fixed() const;
		
		// year 0 is allowed, as per Dershowitz/Reingold -> need to be careful
		bool is_valid() const					{ return ((month() >= _JAN_ && month() <= _DEC_) && 
											 			 (dayOfMonth() >= 1 && dayOfMonth() <= daysInMonth())); };
				
		Int32	yr_,mo_,dy_,RD_;
};

// constructor/destructor
// default : set to January 1, 1 
inline Day::Day() : 
	yr_(1),mo_(1),dy_(1)
{
	if (!is_valid()) throw Err();
		
	gregorian_to_fixed();
}

inline Day::Day(Int32 rd) : 
	RD_(rd)
{
	fixed_to_gregorian();
	
	if (!is_valid()) throw Err();	
}

inline Day::Day(Int32 yr,Int32 jday)
{
	(*this) = Day(Day(yr,1,1).fixed()+jday-1);
	
	if (!is_valid()) throw Err();		
}

inline Day::Day(Int32 yr,Int32 mo,Int32 dy) : 
	yr_(yr), mo_(mo), dy_(dy)
{
	if (!is_valid()) throw Err();
		
	gregorian_to_fixed();
}

inline Day::Day(const Day& val) : 
	yr_(val.yr_), mo_(val.mo_), dy_(val.dy_), RD_(val.RD_)
{ }

inline Int32 Day::daysInMonth() const
{ 
	static const Int32 _MONTH_LENGTH_[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	
	return (month() == 2 && isLeapYear()) ? _MONTH_LENGTH_[month()]+1 : _MONTH_LENGTH_[month()]; 
}

inline Day& Day::operator=(const Day& val)
{ 
	if (this == &val) return (*this);
	
	yr_ = val.yr_; 
	mo_ = val.mo_; 
	dy_ = val.dy_; 
	RD_ = val.RD_;
	
	return (*this); 
}
			
// add/subtract days
inline Day& Day::addDays(Int32 days)
{ 
	return ((*this) = Day(fixed()+days)); 
}

inline Day& Day::addMonths(Int32 months)
{
	if (months == 0) 
	{
		return (*this);
	}
	else
	{
		Int32 	yr = year() + months/12,
				mo = months % 12,
				dy = dayOfMonth();
		
		mo += month();
		
		if (mo < 1)
		{
			mo += 12;
			yr--;
		}
		else if (mo > 12)
		{
			mo -= 12;
			yr++;
		}
		
		return (*this) = Day(yr,mo,dy);
	}
}
	
inline Day& Day::addYears(Int32 years)
{
	return (years == 0) ? (*this) : ((*this) = Day(year() + years,month(),dayOfMonth()));
}

inline Day Day::today()
{
	std::time_t	timedata;
	
	std::time(&timedata);
	
	std::tm*	tmdata = std::localtime(&timedata);
	
	Day	today(tmdata->tm_year+1900,tmdata->tm_mon+1,tmdata->tm_mday);
	
	return today;
}

// from Calendrical Calculations, N. Dershowitz and E. Reingold, Cambridge University Press (1997)
//
inline void Day::gregorian_to_fixed()	
{ 
	RD_ = EPOCH()-1 + 365*(year()-1) + std::quotient(year()-1,4)
									 - std::quotient(year()-1,100)
									 + std::quotient(year()-1,400)
									 + std::quotient(367*month()-362,12)
									 + (month() <= 2 ? 0 : (isLeapYear() ? -1 : -2))
									 + dayOfMonth(); 
}

inline void Day::fixed_to_gregorian()
{
	yr_ = year_from_fixed();
		
	Int32	priorDays = RD_ - Day(yr_,_JAN_,1).fixed(),
			correction = RD_ < Day(yr_,_MAR_,1).fixed() ? 0 : (isLeapYear() ? 1 : 2);
			
	mo_ = std::quotient(12*(priorDays+correction)+373,367);
	dy_ = RD_-Day(yr_,mo_,1).fixed()+1;
}	

inline Int32 Day::year_from_fixed() const
{
	Int32	d0 = RD_ - EPOCH(),
			n400 = std::quotient(d0,146097),
			d1 = std::mod(d0,146097),
			n100 = std::quotient(d1,36524),
			d2 = std::mod(d1,36524),
			n4 = std::quotient(d2,1461),
			d3 = std::mod(d2,1461),
			n1 = std::quotient(d3,365),
			d4 = std::mod(d3,365) + 1,
			year = 400*n400 + 100*n100 + 4*n4 + n1;
		
	return (n100 == 4 || n1 == 4) ? year : year + 1;
}	

// logical operators
inline bool operator==(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()==rhs.fixed()); }
inline bool operator!=(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()!=rhs.fixed()); }
inline bool operator<(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()<rhs.fixed());  }
inline bool operator<=(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()<=rhs.fixed()); }
inline bool operator>(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()>rhs.fixed());  }
inline bool operator>=(const Day& lhs,const Day& rhs)		{ return (lhs.fixed()>=rhs.fixed()); }
		
// arithmetic operators
inline Day operator+(const Day& dy,const Int32& dt)			{ return Day(dy.fixed()+dt); }
inline Day operator-(const Day& dy,const Int32& dt)			{ return Day(dy.fixed()-dt); }

inline Int32 operator-(const Day& ldy,const Day& rdy)		{ return ldy.fixed()-rdy.fixed(); }

} // namespace mcs

#endif // MCS_DAY_H