// class Time
//
//	implements clock time, accurate to precision in seconds determined by FloatType
//
#ifndef MCS_TIME_H
#define MCS_TIME_H

#include <cmath>

#include "MCS_Exceptions.h"
#include "MCS_MathExtensions.h"
#include "MCS_Typedefs.h"

namespace mcs {

// fundamental unit of time is the second
class Time
{
	public:
		typedef	Float64	FloatType;
		//typedef	Float128	FloatType;
	
		// constructor/destructor
		Time();												// default time - 0:0:0.0
		explicit Time(Int32);								// hour
		explicit Time(Int32,Int32);							// hour,minute
		explicit Time(Int32,Int32,FloatType);				// hour,minute,second
		explicit Time(FloatType);							// seconds from midnight
		Time(const Time&);
		~Time() { }
		
		// assignment
		Time& operator=(const Time& val);	

		// accessors
		Int32 		hour() const					{ return hr_; }				// hour		0..23
		Int32 		minute() const					{ return min_; }			// minute	0..59
		FloatType 	second() const					{ return sec_; }			// second	0..59.9_
		FloatType	ticks() const					{ return ticks_; }			// seconds from midnight

		// arithmetic operations - operations are always in seconds	
		Time& operator+=(FloatType val)				{ return ((*this) = Time(ticks()+val)); }		
		Time& operator-=(FloatType val)				{ return ((*this) = Time(ticks()-val)); }
		
		Time& operator++()							{ return ((*this) += 1); }								// prefix
		Time& operator--()							{ return ((*this) -= 1); }								// prefix
		const Time operator++(Int32)				{ const Time tday(*this); ++(*this); return tday; }		// postfix
		const Time operator--(Int32)				{ const Time tday(*this); --(*this); return tday; }		// postfix
		
		Time& addHours(Int32 hr)					{ return ((*this) += hr*secondsInHour()); }
		Time& addMinutes(Int32 min)					{ return ((*this) += min*secondsInMinute()); }
		Time& addSeconds(FloatType sec)				{ return ((*this) += sec); }
		
		// static members
		static FloatType hoursInDay()				{ return 24; };
		static FloatType minutesInHour()			{ return 60; };
		static FloatType minutesInDay()				{ return 1440; };
		static FloatType secondsInMinute() 			{ return 60; };
		static FloatType secondsInHour()			{ return 3600; };
		static FloatType secondsInDay()				{ return 86400; };
		
		static Time now();							// get current local time
		static Time UTC();							// get current UTC (GMT) time
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		Int32		hr_,min_;
		FloatType	sec_,ticks_;
};

inline Time::Time() : 
	hr_(0),min_(0),
	sec_(0),ticks_(0)
{ }

inline Time::Time(Int32 hr) : 
	hr_(hr),min_(0),sec_(0.0)
{
	const Int32	isec = std::floor(sec_);
	
	if (hr_ < 0 || hr_ > 23 || min_ < 0 || min_ > 59 || isec < 0 || isec > 59) throw Err();
	
	ticks_ = (hour()*minutesInHour() + minute())*secondsInMinute() + second();
}

inline Time::Time(Int32 hr,Int32 min) : 
	hr_(hr),min_(min),sec_(0.0)
{
	const Int32	isec = std::floor(sec_);
	
	if (hr_ < 0 || hr_ > 23 || min_ < 0 || min_ > 59 || isec < 0 || isec > 59) throw Err();
	
	ticks_ = (hour()*minutesInHour() + minute())*secondsInMinute() + second();
}

inline Time::Time(Int32 hr,Int32 min,FloatType sec) : 
	hr_(hr),min_(min),sec_(sec)
{
	const Int32	isec = std::floor(sec_);
	
	if (hr_ < 0 || hr_ > 23 || min_ < 0 || min_ > 59 || isec < 0 || isec > 59) throw Err();
	
	ticks_ = (hour()*minutesInHour() + minute())*secondsInMinute() + second();
}

inline Time::Time(FloatType ticks) : 
	ticks_(std::mod(ticks,secondsInDay()))
{ 
	//if (ticks < static_cast<FloatType>(0) || ticks >= secondsInDay()) throw Err();
	
	const FloatType	t = ticks_/secondsInDay();
	
	hr_ = static_cast<Int32>(std::floor(t*hoursInDay()));
	min_ = static_cast<Int32>(std::floor(t*minutesInDay()));
	
	sec_ = t*secondsInDay()-min_*secondsInMinute();
	
	min_ -= hour()*minutesInHour();
}

inline Time::Time(const Time& val) : 
	hr_(val.hr_),min_(val.min_),
	sec_(val.sec_),ticks_(val.ticks_)
{ }

inline Time& Time::operator=(const Time& val)
{ 
	if (this == &val) return (*this);
	
	hr_ = val.hr_; 
	min_ = val.min_; 

	sec_ = val.sec_;
	ticks_ = val.ticks_;
		
	return (*this); 
}
			
inline Time Time::now()
{
	std::time_t	timedata;
	
	std::time(&timedata);
	
	std::tm*	tmdata = std::localtime(&timedata);
	
	return Time(tmdata->tm_hour,tmdata->tm_min,tmdata->tm_sec);
}

inline Time Time::UTC()
{
	std::time_t	timedata;
	
	std::time(&timedata);
	
	std::tm*	tmdata = std::gmtime(&timedata);
	
	return Time(tmdata->tm_hour,tmdata->tm_min,tmdata->tm_sec);
}

// logical operators (precision is limited to approx. 10^-11 sec. for FloatType double)
inline bool operator==(const Time& lhs,const Time& rhs) 						{ return (lhs.ticks()==rhs.ticks()); }
inline bool operator!=(const Time& lhs,const Time& rhs) 						{ return (lhs.ticks()!=rhs.ticks()); }
inline bool operator<(const Time& lhs,const Time& rhs) 							{ return (lhs.ticks()<rhs.ticks());  }
inline bool operator<=(const Time& lhs,const Time& rhs) 						{ return (lhs.ticks()<=rhs.ticks()); }
inline bool operator>(const Time& lhs,const Time& rhs) 							{ return (lhs.ticks()>rhs.ticks());  }
inline bool operator>=(const Time& lhs,const Time& rhs) 						{ return (lhs.ticks()>=rhs.ticks()); }

// arithmetic operators
inline Time operator+(const Time& tm,const Time::FloatType& dt)					{ return Time(tm.ticks()+dt); }
inline Time operator-(const Time& tm,const Time::FloatType& dt)					{ return Time(tm.ticks()-dt); }

inline typename Time::FloatType operator-(const Time& ltm,const Time& rtm)		{ return ltm.ticks()-rtm.ticks(); }

} // namespace mcs

#endif // MCS_TIME_H

