#include <iomanip>
#include <iostream>

#include "MCS_Array.h"
#include "MCS_Bin.h"
#include "MCS_Cache.h"
#include "MCS_Date.h"
#include "MCS_Debug.h"
#include "MCS_FileLocations.h"
#include "MCS_Geography.h"
#include "MCS_Histogram.h"
#include "MCS_IO.h"
#include "MCS_Iterator.h"
#include "MCS_Satellite.h"
#include "MCS_Vector.h"
#include "MCS_WeightedBin.h"

#include "MCS_MSULevel0.h"
#include "MCS_MSULevel1A.h"

#include "MCS_SSMIOrbit.h"
#include "MCS_SSMILevel1A.h"
#include "MCS_SSMIMap.h"

#include "MCS_TMIDataset.h"

#include "MCS_MATLAB_Engine.h"

using namespace mcs;
using namespace sat;
using namespace std;

class Footprint
{
	public:
		Footprint() : u_(_NaN_),v_(_NaN_) { }
		Footprint(const Float32& u,const Float32& v) : u_(u),v_(v) { }
		~Footprint() { }
		
		const Float32&	delta_u() const	{ return u_; }
		const Float32& 	delta_v() const	{ return v_; }
		
	private:
		Float32	u_,v_;	
};

template<class DS>
class AccumulateMap : public Vector<WeightedBin2D<Float64>*>
{
	public:
		AccumulateMap(const Int32& latres = 180,const Int32& lonres = 360);
		~AccumulateMap();
		
		void addSwath(const Swath<DS>& swath);
		void addSwath(const Swath<DS>& swath,const Vector<Footprint>& footprints);
		void addSwath(const Swath<DS>& swath,const Vector<Footprint>& footprints,const Index<1>& lo,const Index<1>& hi);
};

template<class DS>
AccumulateMap<DS>::AccumulateMap(const Int32& latres,const Int32& lonres) :
	Vector<WeightedBin2D<Float64>*>(IndexRange<1>(1,DS::numberOfChannels()))
{ 
	for (Index<1> i=first();i<=last();++i)
		(*this)[i] = new WeightedBin2D<Float64>(Bounds<Float64>(-90,90,latres),Bounds<Float64>(0,360,lonres));
}

template<class DS>
AccumulateMap<DS>::~AccumulateMap()
{ 
	for (Index<1> i=first();i<=last();++i)
		delete (*this)[i];
}

template<class DS>
void AccumulateMap<DS>::addSwath(const Swath<DS>& swath)
{
	for (Index<1> j=1;j<=swath.numberOfChannels();++j)
		for (Index<1> i=1;i<=swath.numberOfFOVs();++i)
			(*this)[j]->add(swath.latitude(i),swath.longitude(i),swath.data(j,i),1.0);
}

template<class DS>
void AccumulateMap<DS>::addSwath(const Swath<DS>& swath,const Vector<Footprint>& footprints)
{
	addSwath(swath,footprints,swath.latitude().first(),swath.latitude().last());
}

template<class DS>
void AccumulateMap<DS>::addSwath(const Swath<DS>& swath,const Vector<Footprint>& footprints,const Index<1>& lo,const Index<1>& hi)
{
	for (Index<1> j=1;j<=swath.numberOfChannels();++j)
	{
		const Float64	dlat = (*this)[j]->x_bounds().delta(),
						dlon = (*this)[j]->y_bounds().delta(),
						footlat = 1.5*footprints[j].delta_v()*360/(2*_pi_*(constants::physical::_r_Ep_/1000.0)),
						latrng = std::max(footlat,dlat);		
			
		for (Index<1> i=lo;i<=hi;++i)
		{
			const Float64	footlon = 1.5*footprints[j].delta_u()*360/(2*_pi_*(constants::physical::_r_Ee_/1000.0)*cosd(swath.latitude(i))),
							lonrng = std::max(footlon,dlon),
							rho2 = (sqr(latrng)+sqr(lonrng))/2,
							w0 = 2/rho2;		
			
			(*this)[j]->add(swath.latitude(i),swath.longitude(i),swath.data(j,i),w0);
			
			for (Float32 lat=swath.latitude(i)-latrng;lat<=swath.latitude(i)+latrng;lat+=dlat)
			{
				const Float64	deltalat = lat-swath.latitude(i);
				
				for (Float32 lon=swath.longitude(i)-lonrng;lon<=swath.longitude(i)+lonrng;lon+=dlon)
				{
					const Float64	deltalon = lon-swath.longitude(i),
									rad2 = sqr(deltalat)+sqr(deltalon),
									wt = std::exp(-(rad2/rho2))*w0;
		
					if (wt >= 0.1*w0)				
						(*this)[j]->add(lat,mod(lon,static_cast<Float32>(360.0)),swath.data(j,i),wt);
				}
			}
		}
	}
}

#define MSU_MAP		0
#define SSMI_MAP	1

#if MSU_MAP

typedef Dataset<MSU,msu::Level0,msu::v2>	Level0Dataset;
typedef OrbitData<Level0Dataset>			Level0OrbitData;

typedef Dataset<MSU,msu::Level1A<msu::Mo1995>,msu::v2>	Level1ADataset;

// map MSU swath data
int main()
{	
	AccumulateMap<Level1ADataset>	swathMap(360,720);
	
	Vector<Footprint>				footprints(IndexRange<1>(1,Level1ADataset::numberOfChannels()));
		
	footprints[1] = footprints[2] = footprints[3] = footprints[4] = Footprint(50,50);
	
	MATLAB_Engine	engine;
	
	engine.evaluate("clear;");

	Satellite	sat("NOAA-14");
	
	Level0Dataset	ds(sat.ID());
	
	OrbitData<Level0Dataset>	orb(ds);
	
	for (Level0OrbitData::iterator it=orb.begin();it!=orb.end();++it)
	{
		cout << *it << "\t" << it->numberOfSwaths() << endl;
		
		for (Orbit<Level0Dataset>::iterator sit=it->begin()+50000;sit<=it->begin()+100000;++sit)
		{
			try
			{	
				const Swath<Level1ADataset>	sw(*sit);
				
				//swathMap.addSwath(sw);
				//swathMap.addSwath(sw,footprints);
				swathMap.addSwath(sw,footprints,4,8);
				
				if (sit->swathNumber() % 100 == 0)
					cout << it->satelliteID().name() << "\t" << sit->epoch() << "\t" << sit->swathNumber() << endl;
			}
			catch (Swath<Level0Dataset>::Err& ex)
			{
				cout << ex.message() << endl;
			} 
		}
	}
	
	for (Index<1> i=swathMap.first();i<=swathMap.last();i++)
	{
		engine.putValue("i",i);
		engine.putMatrix("b",swathMap[i]->bins());
		engine.putMatrix("w",swathMap[i]->weights());
		engine.evaluate("TA(i,:,:)=b./w;");
		engine.evaluate("imagesc(flipud(squeeze(TA(i,:,:))'));colorbar;");
		engine.evaluate("clear b w;");
	}
	
	engine.evaluate("rgb=toRGB(TA(1,:,:),TA(1,:,:),TA(1,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\MSU_Ch1.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA(2,:,:),TA(2,:,:),TA(2,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\MSU_Ch2.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA(3,:,:),TA(3,:,:),TA(3,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\MSU_Ch3.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA(4,:,:),TA(4,:,:),TA(4,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\MSU_Ch2.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA(1,:,:),TA(2,:,:),TA(3,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\MSU_Ch1_Ch3.tif','tif');");
	engine.evaluate("clear rgb;");
		
	return 0;
}

#elif SSMI_MAP

typedef Dataset<SSMI,ssmi::Level1A<ssmi::LoFreq>,ssmi::v5>	LoFreqDataset;
typedef Dataset<SSMI,ssmi::Level1A<ssmi::HiFreq>,ssmi::v5>	HiFreqDataset;

typedef OrbitIterator<LoFreqDataset>	LoFreqOrbitIterator;
typedef OrbitIterator<HiFreqDataset>	HiFreqOrbitIterator;

// map SSMI swath data
int main()
{	
	AccumulateMap<LoFreqDataset>	loFreqMap(720,1440);
	AccumulateMap<HiFreqDataset>	hiFreqMap(1440,2880);
	
	Vector<Footprint>				loFreqFootprints(IndexRange<1>(1,LoFreqDataset::numberOfChannels())),
									hiFreqFootprints(IndexRange<1>(1,HiFreqDataset::numberOfChannels()));
		
	loFreqFootprints[1]	= loFreqFootprints[2] = Footprint(35,22);
	loFreqFootprints[3] = Footprint(25,20);
	loFreqFootprints[4] = loFreqFootprints[5] = Footprint(19,15);
	hiFreqFootprints[1] = hiFreqFootprints[2] = Footprint(8,7);
		
	MATLAB_Engine	engine;
	
	engine.evaluate("clear;");

	Satellite	sat("DMSP-F13");
	
	const Float32	orbits_per_day = Time::secondsInDay()/sat.period(Date(1999,8,31));
	
	LoFreqDataset	ds_lo(sat.ID());
	HiFreqDataset	ds_hi(sat.ID());	
	
	OrbitIterator<LoFreqDataset>	orb_lo(ds_lo);
	OrbitIterator<HiFreqDataset>	orb_hi(ds_hi);
	
	LoFreqOrbitIterator::iterator	lit=orb_lo.begin()+1579*orbits_per_day;
	HiFreqOrbitIterator::iterator	hit=orb_hi.begin()+1579*orbits_per_day;
	
	for (;lit<=orb_lo.begin()+1579.5*orbits_per_day;++lit,++hit)
	{
		cout << *lit << "\t" << lit->numberOfSwaths() << endl;
		
		for (Orbit<LoFreqDataset>::iterator sit=lit->begin();sit!=lit->end();++sit)
		{
			try
			{	
				const Swath<LoFreqDataset>&	sw = *sit;
				
				//loFreqMap.addSwath(sw);
				loFreqMap.addSwath(sw,loFreqFootprints);
				
				if (sit->swathNumber() % 100 == 0)
					cout << lit->satelliteID().name() << "\t" << sit->epoch() << "\t" << sit->swathNumber() << endl;
			}
			catch (Swath<LoFreqDataset>::Err& ex)
			{
				cout << ex.message() << endl;
			} 
		}
		
		for (Orbit<HiFreqDataset>::iterator sit=hit->begin();sit!=hit->end();++sit)
		{
			try
			{	
				const Swath<HiFreqDataset>&	sw = *sit;
				
				//hiFreqMap.addSwath(sw);
				hiFreqMap.addSwath(sw,hiFreqFootprints);
							
				if (sit->swathNumber() % 100 == 0)
					cout << lit->satelliteID().name() << "\t" << sit->epoch() << "\t" << sit->swathNumber() << endl;
			}
			catch (Swath<HiFreqDataset>::Err& ex)
			{
				cout << ex.message() << endl;
			} 
		}
	}
	
	for (Index<1> i=loFreqMap.first();i<=loFreqMap.last();i++)
	{
		engine.putValue("i",i);
		engine.putMatrix("b",loFreqMap[i]->bins());
		engine.putMatrix("w",loFreqMap[i]->weights());
		engine.evaluate("TA_lo(i,:,:)=b./w;");
		engine.evaluate("clear b w;");
	}
	
	for (Index<1> i=hiFreqMap.first();i<=hiFreqMap.last();i++)
	{
		engine.putValue("j",i);
		engine.putMatrix("b",hiFreqMap[i]->bins());
		engine.putMatrix("w",hiFreqMap[i]->weights());
		engine.evaluate("TA_hi(j,:,:)=b./w;");
		engine.evaluate("clear b w;");
	}
	
	engine.evaluate("rgb=toRGB(TA_lo(1,:,:),TA_lo(2,:,:),TA_lo(3,:,:));");
	engine.evaluate("imwrite(rgb,'d:\\temp\\ssmi_19_V-H_22_V.tif','tif');");
	engine.evaluate("clear rgb;");
	
	engine.evaluate("rgb=toRGB(TA_lo(3,:,:),TA_lo(4,:,:),TA_lo(5,:,:));");	
	engine.evaluate("imwrite(rgb,'d:\\temp\\ssmi_22_V_37_V-H.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA_hi(1,:,:),TA_hi(2,:,:),TA_hi(1,:,:)-TA_hi(2,:,:));");	
	engine.evaluate("imwrite(rgb,'d:\\temp\\ssmi_85_V-H.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA_hi(1,:,:),TA_hi(2,:,:),2*TA_hi(1,:,:)-TA_hi(2,:,:));");	
	engine.evaluate("imwrite(rgb,'d:\\temp\\ssmi_85_2V-H.tif','tif');");
	engine.evaluate("clear rgb;");
		
	engine.evaluate("rgb=toRGB(TA_hi(1,:,:),TA_hi(2,:,:),TA_hi(1,:,:)-2*TA_hi(2,:,:));");	
	engine.evaluate("imwrite(rgb,'d:\\temp\\ssmi_85_V-2H.tif','tif');");
	engine.evaluate("clear rgb;");
			
	return 0;
}

#endif