#include <deque>

#include "MCS_Debug.h"

#include "MCS_Date.h"
#include "MCS_Interpolation.h"
#include "MCS_MathExtensions.h"
#include "MCS_MATLAB_Engine.h"
#include "MCS_NRAliases.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_Regression.h"
#include "MCS_Sequences.h"
#include "MCS_Statistics.h"
#include "MCS_TLE.h"
#include "MCS_Vector.h"
	
using namespace std;
using namespace mcs;

static Vector<Float64> findPeakFrequency(const Vector<Float64>& x,const Vector<Float64>& y,const Int32 num = 1)
{
	using namespace numerics;
	
	Float64	ofac = 8.0,
			hifac = 0.1;
			
	const Int32	nfreqt = ofac*hifac*x.dim()*4;
	Int32		nfreq = 64;
	
	while (nfreq < nfreqt) nfreq <<= 1;
	
	nfreq <<=1;
	
	Vector<Float64>		freq(IndexRange<1>(1,nfreq)),
						amp(IndexRange<1>(1,nfreq));
	unsigned long		nout,
						jmax;
	Float64				prob;
						
	NRVector<Float64>	xnr(const_cast<Vector<Float64>&>(x)),
						ynr(const_cast<Vector<Float64>&>(y)),
						freqnr(freq),
						ampnr(amp);
	
	NR::FN<Float64>::fasper(xnr.data(),ynr.data(),x.dim(),ofac,hifac,
							freqnr.data(),ampnr.data(),freq.dim(),&nout,&jmax,&prob);
	
	// return num frequencies with largest amplitudes
	// 	sort by increasing amplitude
	NR::FN<Float64>::sort2(freq.dim(),ampnr.data(),freqnr.data());
		
	//	pick frequencies with highest amplitudes
	Vector<Float64>	freq_out(freq[IndexRange<1>(freq.last()-(num-1),freq.last())]);
	
	//	return in order of decreasing amplitude
	return reverse(freq_out);
}

static Vector<Float64> interpolatePeriodic(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& xp)
{
	// find frequency to remove
	const Vector<Float64>	freq = findPeakFrequency(x,y,5);
	
	// regress to three harmonics
	const numerics::TrigonometricSeriesFit<Float64>	trf(freq,2);	
	const numerics::FunctorRegressor<Float64>		trigReg(x,y,&trf);
	
	// median filter to remove outliers
	numerics::QuantileStatistics<Float64>	stat(trigReg.residual());
	
	VectorMask		mask(abs(trigReg.residual()-stat.median()) > 2.5*stat.IQR());
	Vector<Float64>	tx(x),
					ty(trigReg.residual());
	
	remove(tx,mask);
	remove(ty,mask);
	
	// interpolate on residuals and restore regressed periodics
	const Float64			psig = statistics::standardDeviation(ty),
							w_dc = 2.0*_two_pi_*firstValue(freq);
	Vector<Float64>			sig(tx.range(),psig/sqrt(static_cast<Float64>(tx.dim())));
	
	numerics::RPInterpolator<Float64>	interp(tx,ty,sig,psig,w_dc);
	
	return interp(xp) + trigReg(xp);
}
/*
static Float64 findPeakFrequency(const Vector<Float64>& x,const Vector<Float64>& y)
{
	using namespace numerics;
	
	Float64	ofac = 8.0,
			hifac = 0.1;
			
	const Int32	nfreqt = ofac*hifac*x.dim()*4;
	Int32		nfreq = 64;
	
	while (nfreq < nfreqt) nfreq <<= 1;
	
	nfreq <<=1;
	
	Vector<Float64>		freq(IndexRange<1>(1,nfreq)),
						amp(IndexRange<1>(1,nfreq));
	unsigned long		nout,
						jmax;
	Float64				prob;
						
	NRVector<Float64>	xnr(const_cast<Vector<Float64>&>(x)),
						ynr(const_cast<Vector<Float64>&>(y)),
						freqnr(freq),
						ampnr(amp);
	
	NR::FN<Float64>::fasper(xnr.data(),ynr.data(),x.dim(),ofac,hifac,
							freqnr.data(),ampnr.data(),freq.dim(),&nout,&jmax,&prob);
	
	// return peak frequency
	return freq[jmax];
}

static Vector<Float64> interpolatePeriodic(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& xp)
{
	// find frequency to remove
	const Float64	freq = findPeakFrequency(x,y);
	
	// regress to three harmonics
	const numerics::TrigonometricHarmonicFit<Float64>	trf(freq,3);	
	const numerics::FunctorRegressor<Float64>			trigReg(x,y,&trf);
	
	// median filter to remove outliers
	numerics::QuantileStatistics<Float64>	stat(trigReg.residual());
	
	VectorMask		mask(abs(trigReg.residual()-stat.median()) > 2.5*stat.IQR());
	Vector<Float64>	tx(x),
					ty(trigReg.residual());
	
	remove(tx,mask);
	remove(ty,mask);
	
	// interpolate on residuals and restore regressed periodics
	const Float64			psig = statistics::standardDeviation(ty),
							w_dc = 2.0*_two_pi_*freq;
	Vector<Float64>			sig(tx.range(),psig/sqrt(static_cast<Float64>(tx.dim())));
	
	numerics::RPInterpolator<Float64>	interp(tx,ty,sig,psig,w_dc);
	
	return interp(xp) + trigReg(xp);
}
*/
// chop up series into 5 chunks and interpolate separately
static Vector<Float64> interpolatePeriodicByPieces(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& xp)
{
	const Float64	start_day = firstValue(x),
					end_day = lastValue(x)+1,
					delta = (end_day-start_day)/3;
				
	Vector<Float64>	yout;
	
	for (Int32 i=1;i<=3;i++)
	{
		const Float64	lo = (i-1)*delta+start_day,
						hi = lo+delta;
						
		const VectorIndirection	ind(find(x>=lo && x<hi)),
								indp(find(xp>=lo && xp<hi));
		
		const Vector<Float64>	tx(x[ind]),
								ty(y[ind]),
								txp(xp[indp]);
			
		if (tx.isValid() && ty.isValid() && txp.isValid())
			append(yout,interpolatePeriodic(tx,ty,txp));
	}
	
	return yout;
}

// generate interpolated TLEs
int main(void)
{
	string		satellite_name;
	
	cout << "Satellite? ";
	cin >> satellite_name;
	
	const string	ifname = "E:\\DATA\\TLE\\"+satellite_name+".txt";
	ifstream		ifs(ifname.c_str(),ios::in);
	
	TLE			tle;
	deque<TLE>	tle_data;	
	
	int	counter = 0;
	
	// read in TLEs
	while (ifs.good())
	{
		try
		{
			++counter;
			
			ifs >> tle;
			
			tle_data.push_back(tle);
			
			if (!(counter % 100)) 
				cout << counter << endl << tle << endl << endl;
		}
		catch (TLE::Err& err)
		{ 
			cout << "Element#: " << counter << endl; 
			//cout << tle << endl;
			continue; 
		}
		catch (...)
		{
			return -1;
		}
	}
	
	// remove duplicates and sort in time order
	deque<TLE>::iterator	end_it = unique(tle_data.begin(),tle_data.end());

	tle_data.erase(end_it,tle_data.end());
		
	sort(tle_data.begin(),tle_data.end());

	cout << "Number of two-line elements: " << tle_data.size() << endl << endl;
	
	if (tle_data.size() == 0) return -1;
	
	IndexRange<1>	rng(1,tle_data.size());
	Vector<Float64>	epoch(rng),nd(rng),ndd(rng),bstar(rng),io(rng),
					Wo(rng),eo(rng),wo(rng),Mo(rng),no(rng),height(rng);

	// loop over all TLEs
	Index<1>	i = rng.first();
	
	for (deque<TLE>::const_iterator it = tle_data.begin();it != tle_data.end();++it,++i)
	{
		epoch[i] = it->epoch().toRD2000();
		nd[i] = it->dno_dt();
		ndd[i] = it->d2no_dt2();
		bstar[i] = it->bStarDrag();
		io[i] = it->inclinationAngle();
		Wo[i] = it->longitudeOfAscendingNode();
		eo[i] = it->eccentricity();
		wo[i] = it->argumentOfPerigee();
		Mo[i] = it->meanAnomaly();
		no[i] = it->meanMotion();
		
		// compute orbital altitude
		OrbitalPropagator<SGP4SDP4Model>	orbit(*it);
		
		orbit.propagate(0);
	
		height[i] = orbit.altitude();
	}
	
	Date		start(tle_data.begin()->epoch()),
				end(tle_data.rbegin()->epoch());
	Float64		lo(ceil(start.toRD2000())),
				hi(floor(end.toRD2000()));
				
	Vector<Float64>	i_epoch,i_nd,i_ndd,i_bstar,i_io,i_Wo,i_eo,i_wo,i_Mo,i_no;
	
	try 
	{
		// interpolate at five day intervals for gaps over ten days in length
		VectorIndirection	gaps(find(difference(epoch)>=10.0));
		
		for (Index<1> i=gaps.first();i<=gaps.last();i++)
		{
			const Int32		lo = static_cast<Int32>(ceil(epoch[gaps[i]])),
							hi = static_cast<Int32>(floor(epoch[gaps[i]+1]));
			const Float64	lod = lo,
							hid = hi;
			
			append(i_epoch,numerics::arithmeticSequence(IndexRange<1>(0,(hi-lo+1)/5),lod,hid));
		}
		/*					
		// interpolate at one day intervals for gaps over two days in length
		VectorIndirection	gaps(find(difference(epoch)>=2.0));
		
		for (Index<1> i=gaps.first();i<=gaps.last();i++)
		{
			const Int32		lo = static_cast<Int32>(ceil(epoch[gaps[i]])),
							hi = static_cast<Int32>(floor(epoch[gaps[i]+1]));
			const Float64	lod = lo,
							hid = hi;
			
			append(i_epoch,numerics::arithmeticSequence(IndexRange<1>(0,hi-lo+1),lod,hid));
		}
		*/			
		// population sigma, decorrelation frequency, and measurement sigma
		Float64				psig,w_dc;
		Vector<Float64>		sig(epoch.range());
		
		// nd,ndd,bstar are noisy but tend to zero except during solar activity so set unbiased to false
		
		// interpolate nd
		psig = 2.0*statistics::standardDeviation(nd);
		sig = psig/sqrt(static_cast<double>(sig.dim()));
		w_dc = _two_pi_/365.25;
		numerics::RPInterpolator<Float64>	nd_interp(epoch,nd,sig,psig,w_dc,false);
		i_nd = nd_interp(i_epoch);
		
		// interpolate ndd
		psig = 2.0*statistics::standardDeviation(ndd);
		sig = psig/sqrt(static_cast<double>(sig.dim()));
		w_dc = _two_pi_/365.25;
		numerics::RPInterpolator<Float64>	ndd_interp(epoch,ndd,sig,psig,w_dc,false);
		i_ndd = ndd_interp(i_epoch);
		
		// interpolate bstar
		psig = 2.0*statistics::standardDeviation(bstar);
		sig = psig/sqrt(static_cast<double>(sig.dim()));
		w_dc = _two_pi_/365.25;
		numerics::RPInterpolator<Float64>	bstar_interp(epoch,bstar,sig,psig,w_dc,false);
		i_bstar = bstar_interp(i_epoch);
		
		// interpolate io after detrending
		sig = 1.0;

		const numerics::PolynomialRegressor<Float64>	io_reg(epoch,io,sig,2);
		
		psig = statistics::standardDeviation(io_reg.residual());
		sig = psig;
		w_dc = 0.1*_two_pi_/365.25;
		
		numerics::RPInterpolator<Float64>	io_interp(epoch,io_reg.residual(),sig,psig,w_dc);
		i_io = io_interp(i_epoch)+io_reg(i_epoch);
		
		// interpolate no after detrending
		sig = 1.0;
	
		const numerics::PolynomialRegressor<Float64>	no_reg(epoch,no,sig,2);
		
		psig = statistics::standardDeviation(no_reg.residual());
		sig = psig;
		w_dc = 0.1*_two_pi_/365.25;
		
		numerics::RPInterpolator<Float64>	no_interp(epoch,no_reg.residual(),sig,psig,w_dc);
		i_no = no_interp(i_epoch)+no_reg(i_epoch);
				
		// interpolate eo after removing periodic component
		i_eo = interpolatePeriodic(epoch,eo,i_epoch);
		//i_eo = interpolatePeriodicByPieces(epoch,eo,i_epoch);
		
		// interpolate angular terms
		i_Wo = mod(atan2(interpolatePeriodic(epoch,sin(Wo),i_epoch),
						 interpolatePeriodic(epoch,cos(Wo),i_epoch)),_two_pi_);
		//i_Wo = mod(atan2(interpolatePeriodicByPieces(epoch,sin(Wo),i_epoch),
		//				 interpolatePeriodicByPieces(epoch,cos(Wo),i_epoch)),_two_pi_);
	
		i_wo = mod(atan2(interpolatePeriodic(epoch,sin(wo),i_epoch),
						 interpolatePeriodic(epoch,cos(wo),i_epoch)),_two_pi_);
		//wo = mod(atan2(interpolatePeriodicByPieces(epoch,sin(wo),i_epoch),
		//		 interpolatePeriodicByPieces(epoch,cos(wo),i_epoch)),_two_pi_);
		
		// mean anomaly is special case for NOAA POES due to weirdness
		//	starting on 1999:167 and ending on 2000:106
		const string	sat_prefix(toUpper(satellite_name.substr(0,4)));
		const Float64	startRD = Date(1999,166).toRD2000(),
						endRD = Date(2000,106).toRD2000();
						
		if (lastValue(epoch) >= startRD && (sat_prefix == "NOAA" || sat_prefix == "TIRO"))
		{
			VectorMask		mask(epoch > startRD && epoch < endRD);
			Vector<Float64>	Mo_i_epoch(epoch[find(mask==true)]),i_Mo_tmp;
			
			i_Mo_tmp = mod(atan2(interpolatePeriodic(epoch,sin(Mo),Mo_i_epoch),
								 interpolatePeriodic(epoch,cos(Mo),Mo_i_epoch)),_two_pi_);
							 
			for (Index<1> idx=mask.first(),tidx=i_Mo_tmp.first();idx<=mask.last();++idx)
			{
				if (mask[idx])
				{
					Mo[idx] = i_Mo_tmp[tidx++];

					tle.set_parameters(Date::fromRD2000(epoch[idx]),nd[idx],ndd[idx],bstar[idx],
									   io[idx],Wo[idx],eo[idx],wo[idx],Mo[idx],no[idx]);
									   
					// recompute orbital altitude
					OrbitalPropagator<SGP4SDP4Model>	orb(tle);
					
					orb.propagate(0);
				
					height[idx] = orb.altitude();	
				}
			}
		}
	
		i_Mo = mod(atan2(interpolatePeriodic(epoch,sin(Mo),i_epoch),
						 interpolatePeriodic(epoch,cos(Mo),i_epoch)),_two_pi_);
		//i_Mo = mod(atan2(interpolatePeriodicByPieces(epoch,sin(Mo),i_epoch),
		//				 interpolatePeriodicByPieces(epoch,cos(Mo),i_epoch)),_two_pi_);
	}
	catch (...)
	{ 
		cout << "Error in interpolation..." << endl;
	}

	// append new interpolated TLEs to stack
	tle = *tle_data.begin();
	
	Vector<Float64>	i_height(i_epoch.range());
	
	for (Index<1> i=i_epoch.first();i<=i_epoch.last();i++)
	{
		try
		{
			tle.set_parameters(Date::fromRD2000(i_epoch[i]),i_nd[i],i_ndd[i],i_bstar[i],
							   i_io[i],i_Wo[i],i_eo[i],i_wo[i],i_Mo[i],i_no[i]);
							   
			tle_data.push_back(tle);
			
			// compute orbital altitude
			OrbitalPropagator<SGP4SDP4Model>	orb(tle);
			
			orb.propagate(0);
		
			i_height[i] = orb.altitude();	
		}
		catch (TLE::Err&)
		{
			continue;
		}
	}
	
	MATLAB_Engine	engine;
	
	engine.evaluate("clear;");
					
	engine.putVector("epoch",epoch);
	engine.putVector("height",height);
	engine.putVector("nd",nd);
	engine.putVector("ndd",ndd);
	engine.putVector("bstar",bstar);
	engine.putVector("io",io);
	engine.putVector("Wo",Wo);
	engine.putVector("eo",eo);
	engine.putVector("wo",wo);
	engine.putVector("Mo",Mo);
	engine.putVector("no",no);

	engine.evaluate("data1(1,:)=epoch;");
	engine.evaluate("data1(2,:)=height;");
	engine.evaluate("data1(3,:)=nd;");
	engine.evaluate("data1(4,:)=ndd;");
	engine.evaluate("data1(5,:)=bstar;");
	engine.evaluate("data1(6,:)=io;");
	engine.evaluate("data1(7,:)=Wo;");
	engine.evaluate("data1(8,:)=eo;");
	engine.evaluate("data1(9,:)=wo;");
	engine.evaluate("data1(10,:)=Mo;");
	engine.evaluate("data1(11,:)=no;");
	
	engine.putVector("epoch",i_epoch);
	engine.putVector("height",i_height);
	engine.putVector("nd",i_nd);
	engine.putVector("ndd",i_ndd);
	engine.putVector("bstar",i_bstar);
	engine.putVector("io",i_io);
	engine.putVector("Wo",i_Wo);
	engine.putVector("eo",i_eo);
	engine.putVector("wo",i_wo);
	engine.putVector("Mo",i_Mo);
	engine.putVector("no",i_no);
	
	engine.evaluate("data2(1,:)=epoch;");
	engine.evaluate("data2(2,:)=height;");
	engine.evaluate("data2(3,:)=nd;");
	engine.evaluate("data2(4,:)=ndd;");
	engine.evaluate("data2(5,:)=bstar;");
	engine.evaluate("data2(6,:)=io;");
	engine.evaluate("data2(7,:)=Wo;");
	engine.evaluate("data2(8,:)=eo;");
	engine.evaluate("data2(9,:)=wo;");
	engine.evaluate("data2(10,:)=Mo;");
	engine.evaluate("data2(11,:)=no;");
	
	engine.evaluate("for i=1:11 data(i,:)=[data1(i,:) data2(i,:)]; end;");
	engine.evaluate("[y,idx] = sort(data(1,:));");
	engine.evaluate("for i=1:11 data(i,:)=data(i,idx); end;");
	
	engine.evaluate("figure(1);clf;");
	engine.evaluate("plot_TLE_data(data',data1','"+satellite_name+"');");
	engine.evaluate("set(gcf,'position',[10 50 800 600],'paperorientation','landscape','paperposition',[.25 .25 10 8]);");
	engine.evaluate("print -depsc E:\\Documents\\Figures\\"+satellite_name+"_tle.eps");
	
	// sort in time order and remove duplicates
	sort(tle_data.begin(),tle_data.end());
	//unique(tle_data.begin(),tle_data.end());
	
	// output all TLEs
	ofstream	ofs_tle((ifname.substr(0,ifname.length()-4)+".tle").c_str(),ios::out | ios::trunc);
	
	for (deque<TLE>::const_iterator it = tle_data.begin();it != tle_data.end();++it) 
		ofs_tle << *it; 
	
	ofs_tle.close();
	
	return 0;
}


