#include "MCS_BoundaryConditions.h"

namespace mcs {

// 1D fixed boundary
template<> 
Index<1> FixedBoundary<1>::mapIndex(const Index<1>& idx,const IndexRange<1>& rng)
{
	if (idx[0] < rng.first(0) || idx[0] > rng.last(0)) 
		throw Err();
	
	return idx;
}

template<> 
Index<2> FixedBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng)
{
	if (idx[0] < rng.first(0) || idx[1] < rng.first(1) || 
		idx[0] > rng.last(0) || idx[1] > rng.last(1)) 
		throw Err();
	
	return idx;
}

// 1D periodic boundary
template<> 
Index<1> PeriodicBoundary<1>::mapIndex(const Index<1>& idx,const IndexRange<1>& rng)
{
	// periodic boundaries
	if (idx < rng.first() || idx > rng.last())
		return std::mod(idx[0] - rng.first(0),rng.dim(0))+rng.first(0);
	else
		return idx;
}

// 2D periodic boundary
template<> 
Index<2> PeriodicBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng)
{
	const Int32	xidx = idx[0],
				yidx = idx[1];
				
	Int32	xret,
			yret;
			
	// outside mesh bounds : periodic boundaries
	if (xidx < rng.first(0) || xidx > rng.last(0))
	{
		xret = std::mod(xidx - rng.first(0),rng.dim(0))+rng.first(0);
	}
	else
	{
		xret = idx[0];
	}
	
	if (yidx < rng.first(1) || yidx > rng.last(1))
	{
		yret = std::mod(yidx - rng.first(1),rng.dim(1))+rng.first(1);
	}
	else
	{
		yret = idx[1];
	}
	
	return Index<2>(xret,yret);
}

// 1D even mirror boundary
template<> 
Index<1> EvenMirrorBoundary<1>::mapIndex(const Index<1>& idx,const IndexRange<1>& rng)
{
	if (idx[0] < rng.first(0) || idx[0] > rng.last(0))
	{
		Int32	offs = idx[0] - rng.first(0);
					
		// number of boundary crossings
		const Int32	num_boundaries = std::quotient(offs,rng.dim(0));
		
		if (std::even(num_boundaries))
			offs = std::mod(offs,rng.dim(0));
		else
			offs = (rng.dim(0)-1)-std::mod(offs,rng.dim(0));
			
		return offs+rng.first(0);
	}
	else
	{
		return idx;
	}
}

// 2D even mirror boundary
template<> 
Index<2> EvenMirrorBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng)
{
	const Int32	xidx = idx[0],
				yidx = idx[1];
	
	Int32		xret,
				yret;
				
	// outside mesh bounds : mirror boundaries
	if (xidx < rng.first(0) || xidx > rng.last(0))
	{
		Int32	xoffs = idx[0] - rng.first(0);
					
		// number of boundary crossings
		const Int32	num_xboundaries = std::quotient(xoffs,rng.dim(0));
		
		if (std::even(num_xboundaries))
			xoffs = std::mod(xoffs,rng.dim(0));
		else
			xoffs = (rng.dim(0)-1)-std::mod(xoffs,rng.dim(0));
			
		xret = xoffs+rng.first(0);
	}
	else
	{
		xret = idx[0];
	}

	if (yidx < rng.first(1) || yidx > rng.last(1))
	{
		Int32	yoffs = idx[1] - rng.first(1);
					
		// number of boundary crossings
		const Int32	num_yboundaries = std::quotient(yoffs,rng.dim(1));
		
		if (std::even(num_yboundaries))
			yoffs = std::mod(yoffs,rng.dim(1));
		else
			yoffs = (rng.dim(1)-1)-std::mod(yoffs,rng.dim(1));
			
		yret = yoffs+rng.first(1);
	}
	else
	{
		yret = idx[1];
	}

	return Index<2>(xret,yret);
}

// 1D odd mirror boundary
template<> 
Index<1> OddMirrorBoundary<1>::mapIndex(const Index<1>& idx,const IndexRange<1>& rng)
{
	if (idx[0] < rng.first(0) || idx[0] > rng.last(0))
	{
		Int32	offs = idx[0] - rng.first(0);
					
		// number of boundary crossings
		const Int32	period = 2*(rng.dim(0)-1),
					num_periods = std::quotient(offs,period),
					normalized_offs = offs-num_periods*period;
		
		if (normalized_offs <= rng.dim(0)-1)
			offs = normalized_offs;
		else
			offs = period-normalized_offs;
			
		return offs+rng.first(0);
	}
	else
	{
		return idx;
	}
}

// 2D odd mirror boundary
template<> 
Index<2> OddMirrorBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng)
{
	const Int32	xidx = idx[0],
				yidx = idx[1];
				
	Int32		xret,
				yret;
				
	// outside mesh bounds : mirror boundaries
	if (xidx < rng.first(0) || xidx > rng.last(0))
	{
		Int32	xoffs = idx[0] - rng.first(0),
				yoffs = idx[1] - rng.first(1);
					
		// number of boundary crossings
		const Int32	xperiod = 2*(rng.dim(0)-1),
					num_xperiods = std::quotient(xoffs,xperiod),
					normalized_xoffs = xoffs-num_xperiods*xperiod;
		
		if (normalized_xoffs <= rng.dim(0)-1)
			xoffs = normalized_xoffs;
		else
			xoffs = xperiod-normalized_xoffs;
			
		xret = xoffs+rng.first(0);
	}
	else
	{
		xret = idx[0];
	}
	
	if (yidx < rng.first(1) || yidx > rng.last(1))
	{
		Int32	yoffs = idx[1] - rng.first(1);
					
		// number of boundary crossings
		const Int32	yperiod = 2*(rng.dim(1)-1),
					num_yperiods = std::quotient(yoffs,yperiod),
					normalized_yoffs = yoffs-num_yperiods*yperiod;
		
		if (normalized_yoffs <= rng.dim(1)-1)
			yoffs = normalized_yoffs;
		else
			yoffs = yperiod-normalized_yoffs;
			
		yret = yoffs+rng.first(1);
	}
	else
	{
		yret = idx[1];
	}
	
	return Index<2>(xret,yret);
}

// 2D Cylindrical boundary
template<> 
Index<2> CylindricalBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng) 
{
	const Int32	xidx = idx[0],
				yidx = idx[1];
				
	Int32	xret,
			yret;
			
	// outside mesh bounds : periodic boundaries in x, fixed boundaries in y
	if (xidx < rng.first(0) || xidx > rng.last(0))
	{
		xret = std::mod(xidx - rng.first(0),rng.dim(0))+rng.first(0);
	}
	else
	{
		xret = idx[0];
	}
	
	if (yidx < rng.first(1) || yidx > rng.last(1))
	{
		throw Err();
	}
	else
	{
		yret = idx[1];
	}
	
	return Index<2>(xret,yret);
}

// 2D spherical boundary
template<> 
Index<2> SphericalBoundary<2>::mapIndex(const Index<2>& idx,const IndexRange<2>& rng) 
{
	const Int32	xidx = idx[0],
				yidx = idx[1];
			
	// outside mesh bounds : spherical boundaries
	if (xidx < rng.first(0) || xidx > rng.last(0) || 
		yidx < rng.first(1) || yidx > rng.last(1))
	{
		Int32	xoffs = xidx - rng.first(0),
				yoffs = yidx - rng.first(1);
					
		// number of boundary crossings
		const Int32	num_x_boundaries = std::quotient(xoffs,rng.dim(0)),
					num_y_boundaries = std::quotient(yoffs,rng.dim(1));
		
		// twist boundaries in x
		if (std::odd(num_x_boundaries))
		{
			xoffs = (rng.dim(0)-1)-std::mod(xoffs,rng.dim(0));
			yoffs = std::mod(yoffs+rng.dim(1)/2,rng.dim(1));
		}
		else
		{
			// periodic boundaries in y
			xoffs = std::mod(xoffs,rng.dim(0));
			yoffs = std::mod(yoffs,rng.dim(1));
		}
			
		return Index<2>(xoffs+rng.first(0),yoffs+rng.first(1));
	}
	else
	{
		return idx;
	}
}

} // namespace mcs