#ifndef MCS_FIELD_H
#define MCS_FIELD_H

#include "MCS_Grid.h"
#include "MCS_Interpolation.h"
#include "MCS_Matrix.h"

namespace mcs {

// data of type T represented on a 2D grid of coordinates of type M with boundary B
template<class T,class M = T,class G = UniformGrid<M,2,SphericalBoundary<2> > > 
class Field : public Matrix<T>
{
	public:
		typedef T							ValueType;
		typedef G							GridType;
		typedef typename G::BoundaryType	BoundaryType;
		typedef typename G::CoordinateType	CoordinateType;
		typedef typename G::OrdinateType	OrdinateType;
		
		Field();
		Field(const GridType&,const T& = T());
		Field(const GridType&,const Matrix<T>&);
		Field(const Field<T,M,G>&);
		virtual ~Field();
		
		Field<T,M,G>& operator=(const T&);
		Field<T,M,G>& operator=(const Matrix<T>&);
		Field<T,M,G>& operator=(const Field<T,M,G>&);
		
		bool operator==(const Field<T,M,G>& source)	{ return (grid_ == source.grid_ && Matrix<T>::operator==(source)); }
		bool operator!=(const Field<T,M,G>& source)	{ return (grid_ != source.grid_ || Matrix<T>::operator!=(source)); }
		
		CoordinateType xy(const Index<2>&) const;
		Index<2> xy(const CoordinateType&) const;
				
		T z(const Index<2>&) const;
		T z(const CoordinateType&) const;

		const GridType& grid() const;
		
		Field<T,M,G> resize(const GridType&) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		static const EnsureNumeric<T>	dummy1_;
		static const EnsureNumeric<M>	dummy2_;
				
		GridType	grid_;
};

// implementation
template<class T,class M,class G> const EnsureNumeric<T> Field<T,M,G>::dummy1_;
template<class T,class M,class G> const EnsureNumeric<M> Field<T,M,G>::dummy2_;

template<class T,class M,class G> 
inline Field<T,M,G>::Field() :
	grid_(),Matrix<T>()
{ }

template<class T,class M,class G> 
inline Field<T,M,G>::Field(const GridType& grid,const T& val) :
	grid_(grid),
	Matrix<T>(grid.range(),val)
{ }

template<class T,class M,class G> 
inline Field<T,M,G>::Field(const GridType& grid,const Matrix<T>& source) :
	grid_(grid)
{
	if (grid.range() != source.range()) throw Err(); 
	
	Matrix<T>::operator=(source);
}

template<class T,class M,class G> 
inline Field<T,M,G>::Field(const Field<T,M,G>& source) :
	grid_(source.grid_),
	Matrix<T>(source)
{ }

template<class T,class M,class G> 
inline Field<T,M,G>::~Field()
{ }

template<class T,class M,class G> 
inline Field<T,M,G>& Field<T,M,G>::operator=(const T& val)
{
	Matrix<T>::operator=(val);
	
	return (*this);
}

template<class T,class M,class G> 
inline Field<T,M,G>& Field<T,M,G>::operator=(const Matrix<T>& source)
{
	if ((*this).range() != source.range()) throw Err();
	
	Matrix<T>::operator=(source);
	
	return (*this);
}

template<class T,class M,class G> 
inline Field<T,M,G>& Field<T,M,G>::operator=(const Field<T,M,G>& source)
{
	if (this == &source) return (*this);
	
	grid_ = source.grid_;
	Matrix<T>::operator=(source);
	
	return (*this);
}

// return coordinates of point, dealing with boundary conditions
template<class T,class M,class G> 
inline typename Field<T,M,G>::CoordinateType Field<T,M,G>::xy(const Index<2>& idx) const
{
	return grid_.BCvalue(idx);
}

template<class T,class M,class G> 
inline Index<2> Field<T,M,G>::xy(const CoordinateType& val) const
{
	return grid_.nearest(val);
}

// return value of Field point, dealing with boundary conditions
template<class T,class M,class G> 
inline T Field<T,M,G>::z(const Index<2>& idx) const 
{
	return (*this)(grid_.index(idx));
}

// interpolate value of Field point, dealing with boundary conditions
// policy for real maps with NaNs is to replace them with the mean of good pixels
template<class T,class M,class G> 
inline T Field<T,M,G>::z(const CoordinateType& val) const 
{
	const typename GridType::IndexBracketType	NoBCbox(grid_.NoBCbracket(val)),
												BCbox(grid_.BCbracket(val));
									
	const CoordinateType	xy11 = grid_.NoBCvalue(lowerLeft(NoBCbox)),		// lower left
							xy22 = grid_.NoBCvalue(upperRight(NoBCbox));	// upper right

	FixedVector<typename PromoteFloat<T>::promote_t,4>	zv;
	
	zv[0] = static_cast<typename PromoteFloat<T>::promote_t>(z(lowerLeft(BCbox)));
	zv[1] = static_cast<typename PromoteFloat<T>::promote_t>(z(lowerRight(BCbox)));
	zv[2] = static_cast<typename PromoteFloat<T>::promote_t>(z(upperRight(BCbox)));
	zv[3] = static_cast<typename PromoteFloat<T>::promote_t>(z(upperLeft(BCbox)));
	
	const typename PromoteFloat<T>::promote_t	mn = statistics::ignoreNaN::mean(zv);
	
	if (!isfinite(zv[0])) zv[0] = mn;
	if (!isfinite(zv[1])) zv[1] = mn;
	if (!isfinite(zv[2])) zv[2] = mn;
	if (!isfinite(zv[3])) zv[3] = mn;
	 	
	// bilinear interpolate
	return static_cast<T>(num::bilinearInterpolate(zv,xy11,xy22,val[0],val[1]));
}

template<class T,class M,class G> 
inline const typename Field<T,M,G>::GridType& Field<T,M,G>::grid() const
{ return grid_; }

template<class T,class M,class G> 
inline Field<T,M,G> Field<T,M,G>::resize(const G& grid) const
{
	if (grid_ == grid) return (*this);
	
	Field<T,M,G> tmap(grid);
	
	// if new field is higher resolution than original use interpolation
	if (any(tmap.dim() > dim()))
	{
		for (Index<1> xidx = tmap.first(0);xidx <= tmap.last(0);++xidx)
		{
			for (Index<1> yidx = tmap.first(1);yidx <= tmap.last(1);++yidx)
			{
				const Index<2>	tidx(xidx,yidx);
				
				tmap(tidx) = z(grid.BCvalue(tidx));
			}
		}
	}
	// if new field is lower resolution than original use supersampling
	else
	{
		const Index<2>	scale = dim()/tmap.dim(),
						diff = tmap.dim()*scale;
		
		// if new field is integral fraction of original use straight averaging
		if (all(dim() == diff))	
		{
			for (Index<1> xidx = first(0),txidx = tmap.first(0);txidx <= tmap.last(0);xidx += scale[0],++txidx)
			{
				for (Index<1> yidx = first(1),tyidx = tmap.first(1);tyidx <= tmap.last(1);yidx += scale[1],++tyidx)
				{
					T		val = 0;
					Int32	num = 0;
					
					for (Index<1> idx1=0;idx1<scale[0];++idx1)
					{
						for (Index<1> idx2=0;idx2<scale[1];++idx2)
						{
							if (isfinite((*this)(xidx+idx1,yidx+idx2)))
							{
								val += (*this)(xidx+idx1,yidx+idx2);
								++num;
							}
						}
					}
							
					const Index<2>	tidx(txidx,tyidx);
					
					tmap(tidx) = val/num;
				}
			}
		}
		else
		{
			throw Err();
		}
	}
	
	return tmap;
}

} // namespace mcs

#endif // MCS_FIELD_H
