#ifndef MCS_GRID_H
#define MCS_GRID_H

#include "MCS_Bracket.h"
#include "MCS_BoundaryConditions.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Index.h"
#include "MCS_IndexRange.h"
#include "MCS_Vector.h"

namespace mcs {

// Grid class representing a D-dimensional grid with boundary conditions
// evenly spaced grid
template<class T,Uint8 D,class B = FixedBoundary<D> > 
class UniformGrid
{
	public:
		enum { Rank = D };
		
		typedef B					BoundaryType;
		typedef FixedVector<T,D>	CoordinateType;
		typedef Bracket<Int32,D>	IndexBracketType;
		typedef T					OrdinateType;
		
		UniformGrid();
		UniformGrid(const IndexRange<D>& rng,const CoordinateType& lo,const CoordinateType& hi);
		virtual ~UniformGrid() { }
		
		bool operator==(const UniformGrid<T,D,B>&) const;
		bool operator!=(const UniformGrid<T,D,B>&) const;
		
		const IndexRange<D>& range() const					{ return rng_; }
				  
		Vector<T> axis(Int32) const;

		// returns grid value at given index, accounting for boundary conditions
		CoordinateType BCvalue(const Index<D>&) const;
		
		// returns grid value at given index, disregarding boundary conditions
		CoordinateType NoBCvalue(const Index<D>&) const;
		
		// returns indices bracketing given value, accounting for boundary conditions
		IndexBracketType BCbracket(const CoordinateType&) const;
		
		// returns indices bracketing given value, accounting for boundary conditions
		IndexBracketType NoBCbracket(const CoordinateType&) const;
		
		// returns index transformed to account for boundary conditions
		Index<D> index(const Index<D>& idx) const			{ return B::mapIndex(idx,range()); }
		
		// returns index nearest to point, accounting for boundary conditions
		Index<D> nearest(const CoordinateType& coord) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		const CoordinateType& lo() const					{ return lo_; }
		const CoordinateType& hi() const					{ return hi_; }
		const CoordinateType& delta()	const				{ return delta_; }
		
		Index<D> loIndex(const CoordinateType&) const;
	
		IndexRange<D>	rng_;		
		CoordinateType	lo_,hi_,delta_;
		
		static const EnsureNumeric<T>	dummy_;
};

template<class T,Uint8 D,class B> const EnsureNumeric<T>	UniformGrid<T,D,B>::dummy_;

// implementation
template<class T,Uint8 D,class B> 
inline UniformGrid<T,D,B>::UniformGrid() :
	rng_(),
	lo_(),
	hi_(),
	delta_()
{ }

template<class T,Uint8 D,class B> 
inline UniformGrid<T,D,B>::UniformGrid(const IndexRange<D>& rng,const CoordinateType& clo,const CoordinateType& chi) :
	rng_(rng),
	lo_(clo),
	hi_(chi)
{
	delta_ = (hi()-lo())/construct<CoordinateType>::from(range().dim()-1);
}

template<class T,Uint8 D,class B> 
inline bool UniformGrid<T,D,B>::operator==(const UniformGrid<T,D,B>& source) const
{
	return (all(lo() == source.lo()) && all(hi() == source.hi()) && range() == source.range());
}

template<class T,Uint8 D,class B> 
inline bool UniformGrid<T,D,B>::operator!=(const UniformGrid<T,D,B>& source) const
{
	return (any(lo() != source.lo()) || any(hi() != source.hi()) || range() != source.range());
}

template<class T,Uint8 D,class B> 
inline Vector<T> UniformGrid<T,D,B>::axis(Int32 rnk) const
{
	Vector<T>	tmp(IndexRange<1>(range().first().rank(rnk),range().last().rank(rnk)));
	
	for (Index<1> i=tmp.first();i<=tmp.last();i++)
		tmp[i] = lo()[rnk]+(i-tmp.first())*delta()[rnk];
	
	return tmp;
}

template<class T,Uint8 D,class B> 
inline typename UniformGrid<T,D,B>::CoordinateType UniformGrid<T,D,B>::BCvalue(const Index<D>& idx) const
{
	return CoordinateType(lo() + construct< FixedVector<T,D> >::from(index(idx)-range().first())*delta());
}

template<class T,Uint8 D,class B> 
inline typename UniformGrid<T,D,B>::CoordinateType UniformGrid<T,D,B>::NoBCvalue(const Index<D>& idx) const
{
	return CoordinateType(lo() + construct< FixedVector<T,D> >::from(idx-range().first())*delta());
}

template<class T,Uint8 D,class B> 
inline typename UniformGrid<T,D,B>::IndexBracketType UniformGrid<T,D,B>::BCbracket(const CoordinateType& val) const
{
	const Index<D>	loidx = loIndex(val),
					hiidx = loidx+1;
	
	return IndexBracketType(index(loidx),index(hiidx));
}

template<class T,Uint8 D,class B> 
inline typename UniformGrid<T,D,B>::IndexBracketType UniformGrid<T,D,B>::NoBCbracket(const CoordinateType& val) const
{
	const Index<D>	loidx = loIndex(val),
					hiidx = loidx+1;
	
	return IndexBracketType(loidx,hiidx);
}

template<class T,Uint8 D,class B> 
inline Index<D> UniformGrid<T,D,B>::nearest(const CoordinateType& val) const
{
	const IndexBracketType	brak(BCbracket(val));
	const CoordinateType	loVal(BCvalue(brak.lo())),
							hiVal(BCvalue(brak.hi()));
	const T					loDist(norm(val-loVal)),
							hiDist(norm(val-hiVal));
						
	if (loDist < hiDist)
	{
		return brak.lo();
	}
	else
	{
		return brak.hi();
	}
}

template<class T,Uint8 D,class B> 
inline Index<D> UniformGrid<T,D,B>::loIndex(const CoordinateType& val) const
{
	return Index<D>(construct< FixedVector<Int32,D> >::from(floor((val-lo())/delta()))+range().first());
}

} // namespace mcs

#endif // MCS_GRID_H