#ifndef MCS_FIXEDMATRIX_H
#define MCS_FIXEDMATRIX_H

#include <cmath>
#include <complex>
#include <string>

#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Promote.h"
#include "MCS_Typedefs.h"

namespace mcs {

// FixedMatrix of type T, dimension N, offset 0
template<class T,Uint8 N> class FixedMatrix 
{
	public:	
		typedef FixedMatrix<Binary,N>	Mask;
		
		// ctor/dtor		
		FixedMatrix();
		explicit FixedMatrix(const T&);
		explicit FixedMatrix(const T*);
		explicit FixedMatrix(const std::string&);
		FixedMatrix(const FixedMatrix<T,N>&);
		~FixedMatrix();
		
		// range accessors
				bool isValid() const			{ return true; }
							 	
				Int32 elem() const				{ return N; }
				 	
				Int32 first() const				{ return 0; }
				Int32 last() const				{ return N-1; }
				Int32 dim() const				{ return N; }			
				Int32 offset() const			{ return 0; }
		
		// assignment
				FixedMatrix<T,N>& operator=(const T&);
				FixedMatrix<T,N>& operator=(const std::string&);
				FixedMatrix<T,N>& operator=(const FixedMatrix<T,N>&);
				 
		// element accessors		
				T& 					operator()(const Int32& idx0,const Int32& idx1)			{ return mat_data_[idx0][idx1]; }
		const 	T& 					operator()(const Int32& idx0,const Int32& idx1) const	{ return mat_data_[idx0][idx1]; }
			 	
		// arithmetic operations		
			 	FixedMatrix<T,N>& 	operator+=(const T&);		
			 	FixedMatrix<T,N>& 	operator-=(const T&);
			 	FixedMatrix<T,N>& 	operator*=(const T&);
			 	FixedMatrix<T,N>& 	operator/=(const T&);

			 	FixedMatrix<T,N>& 	operator+=(const FixedMatrix<T,N>&);
			 	FixedMatrix<T,N>& 	operator-=(const FixedMatrix<T,N>&);
			 	FixedMatrix<T,N>& 	operator*=(const FixedMatrix<T,N>&);
			 	FixedMatrix<T,N>& 	operator/=(const FixedMatrix<T,N>&);
				
	private:
		static const EnsureNumeric<T>	dummy_;
		
		// element assignors			
				void			set();
				void			set(const T&);
				void			set(const T*);
				void			set(const std::string&);
				void			set(const FixedMatrix<T,N>& source);	
									
		T		mat_data_[N][N];
};

// logical operators		
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator==(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator!=(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<=(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>=(const FixedMatrix<T,N>& lmat,const T& rscal);

template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator==(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator!=(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<=(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>=(const T& lscal,const FixedMatrix<T,N>& rmat);

template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator==(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator!=(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator<=(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator>=(const FixedMatrix<T,N>& lmat,const FixedMatrix<T,N>& rmat);

// matrix non-member operators
template<class T,Uint8 N> const FixedMatrix<T,N> operator+(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const FixedMatrix<T,N> operator-(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const FixedMatrix<T,N> operator*(const T& lscal,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const FixedMatrix<T,N> operator/(const T& lscal,const FixedMatrix<T,N>& rmat);

template<class T,Uint8 N> const FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lmat,const T& rscal);
template<class T,Uint8 N> const FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lmat,const T& rscal);

template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator&&(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> const typename FixedMatrix<T,N>::Mask operator||(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);

template<class T,Uint8 N> const FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> const FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> const FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> const FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);

template<class T,Uint8 N> const FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& source);

template<class T,Uint8 N> const FixedMatrix<T,N> operator!(const FixedMatrix<T,N>& source);

template<class T,Uint8 N> const FixedVector<T,N> operator*(const FixedVector<T,N>& lvec,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> const FixedVector<T,N> operator*(const FixedMatrix<T,N>& lmat,const FixedVector<T,N>& rvec);

// in place
// arithmetic functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Abs(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Ceil(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Exp(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Floor(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Log(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Pow(FixedMatrix<T,N>& source,const T& expon);
template<class T,Uint8 N> const FixedMatrix<T,N>& Round(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sqr(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sqrt(FixedMatrix<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Acos(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Asin(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Atan(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Cos(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sin(FixedMatrix<T,N>& source); 
template<class T,Uint8 N> const FixedMatrix<T,N>& Tan(FixedMatrix<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Acosh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Asinh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Atanh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Cosh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sinh(FixedMatrix<T,N>& source); 
template<class T,Uint8 N> const FixedMatrix<T,N>& Tanh(FixedMatrix<T,N>& source);

// complex vectors
template<class T,Uint8 N> const FixedMatrix<T,N>& Conj(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Imag(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Real(FixedMatrix<T,N>& source);

// not in place 
// arithmetic functions
template<class T,Uint8 N> FixedMatrix<T,N> abs(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> ceil(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> exp(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> floor(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> log(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> pow(const FixedMatrix<T,N>& source,const T& expon);
template<class T,Uint8 N> FixedMatrix<T,N> round(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sqr(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sqrt(const FixedMatrix<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> FixedMatrix<T,N> acos(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> asin(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atan(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atan2(const FixedMatrix<T,N>& y,const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<T,N> cos(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sin(const FixedMatrix<T,N>& source); 
template<class T,Uint8 N> FixedMatrix<T,N> tan(const FixedMatrix<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> FixedMatrix<T,N> acosh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> asinh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atanh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> cosh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sinh(const FixedMatrix<T,N>& source); 
template<class T,Uint8 N> FixedMatrix<T,N> tanh(const FixedMatrix<T,N>& source);
 
// complex vectors
template<class T,Uint8 N> FixedMatrix<T,N> conj(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<typename DemoteComplex<T>::demote_t,N> imag(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<typename DemoteComplex<T>::demote_t,N> real(const FixedMatrix<T,N>& source);

// miscellaneous
template<class T,Uint8 N> FixedMatrix<T,N> bound(const FixedMatrix<T,N>& x,const T& min_val,const T& max_val);
template<class T,Uint8 N> FixedMatrix<Binary,N> even(const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<Binary,N> odd(const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const T& y);
template<class T,Uint8 N> FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y);
template<class T,Uint8 N> FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const T& y);
template<class T,Uint8 N> FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y);

// vector specific
template<class T,Uint8 N> bool all(const FixedMatrix<T,N>&);
template<class T,Uint8 N> bool any(const FixedMatrix<T,N>&);

template<class T,Uint8 N> T sum(const FixedMatrix<T,N>& source);

} // namespace mcs

// implementation
#include "MCS_FixedMatrix.cpp"
#include "MCS_FixedMatrixOperators.cpp"
#include "MCS_FixedMatrixFunctions.cpp"

//#include "MCS_FixedMatrix1.h"
//#include "MCS_FixedMatrix2.h"
//#include "MCS_FixedMatrix3.h"
//#include "MCS_FixedMatrix4.h"

//#include "MCS_FixedMatrixOperators.h"
//#include "MCS_FixedMatrixFunctions.h"
//#include "MCS_FixedMatrixIO.h"
//#include "MCS_FixedMatrixStatistics.h"
//#include "MCS_FixedMatrixTypecasts.h"

#endif // MCS_FIXEDMATRIX_H