#ifndef MCS_FIXEDVECTOR_H
#define MCS_FIXEDVECTOR_H

#include <string>

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

// FixedVector of type T, length N, offset 0
template<class T,Uint8 N> class FixedVector 
{
	public:	
		typedef FixedVector<Binary,N>	Mask;
		
		// ctor/dtor		
		FixedVector()									{ set(); }
		explicit FixedVector(const T& val)				{ set(val); }
		explicit FixedVector(const T* ptr)				{ set(ptr); }
		FixedVector(const std::string& str)				{ set(str); }
		FixedVector(const FixedVector<T,N>& source)		{ set(source); }
		~FixedVector() 									{ }
		
		// range accessors
		static bool isValid() 							{ return true; }
		
		static Int32 size()								{ return N; }				 	
		static Int32 elem() 							{ return N; }			 	
		static Int32 first() 							{ return 0; }
		static Int32 last() 							{ return N-1; }
		static Int32 dim() 								{ return N; }			
		static Int32 offset() 							{ return 0; }
		
		// assignment
				FixedVector<T,N>& operator=(T);
				FixedVector<T,N>& operator=(const std::string&);
				FixedVector<T,N>& operator=(const FixedVector<T,N>&);
				 
		// element accessors		
				T& 					operator[](const Int32& idx)		{ return vec_data_[idx]; }
		const 	T& 					operator[](const Int32& idx) const	{ return vec_data_[idx]; }
			 	
				T& 					rank(const Int32& idx)				{ return vec_data_[idx]; }
		const 	T& 					rank(const Int32& idx) const		{ return vec_data_[idx]; }
		
		// arithmetic operations		
			 	FixedVector<T,N>& 	operator+=(T);		
			 	FixedVector<T,N>& 	operator-=(T);
			 	FixedVector<T,N>& 	operator*=(T);
			 	FixedVector<T,N>& 	operator/=(T);
			 	FixedVector<T,N>& 	operator%=(T);
			 	FixedVector<T,N>& 	operator&=(T);
			 	FixedVector<T,N>& 	operator|=(T);
			 	FixedVector<T,N>& 	operator^=(T);
			 	FixedVector<T,N>& 	operator<<=(T);
			 	FixedVector<T,N>& 	operator>>=(T);

			 	FixedVector<T,N>& 	operator+=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator-=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator*=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator/=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator%=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator&=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator|=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator^=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator<<=(const FixedVector<T,N>&);
			 	FixedVector<T,N>& 	operator>>=(const FixedVector<T,N>&);
				
	private:
		static const EnsureNumeric<T>	dummy_;
		
		// element assignors			
				void			set();
				void			set(const T&);
				void			set(const T*);
				void			set(const std::string&);
				void			set(const FixedVector<T,N>& source);	
									
		T		vec_data_[N];
};

template<class T,Uint8 N> const EnsureNumeric<T> FixedVector<T,N>::dummy_;

// assignment
template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator=(T val)
{
	 set(val); 
	 return (*this); 
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator=(const std::string& str)
{ 
	set(str); 
	return (*this); 
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator=(const FixedVector<T,N>& source)
{
	if (this == &source) return *this;

	set(source);	
	
	return *this;
}		

// element assignors
// initialize with value
template<class T,Uint8 N> 
inline void FixedVector<T,N>::set()
{ 
	for (Int32 i=first();i<=last();++i) vec_data_[i] = T();
}

template<class T,Uint8 N> 
inline void FixedVector<T,N>::set(const T& val)
{ 
	for (Int32 i=first();i<=last();++i) vec_data_[i] = val;
}

// initialize with array
template<class T,Uint8 N> 
inline void FixedVector<T,N>::set(const T* data)
{ 
	for (Int32 i=first(),j=0;i<=last();vec_data_[i++] = data[j++]) { };
}

template<class T,Uint8 N> 
inline void FixedVector<T,N>::set(const std::string& str)
{	
	std::stringstream	temp(str);
	
	for (Int32 i=first();i<=last();++i) temp >> vec_data_[i];
}

template<class T,Uint8 N> 
inline void FixedVector<T,N>::set(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();++i) vec_data_[i] = source.vec_data_[i];
}

// arithmetic operators
// scalar member assign/equals operators
template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator+=(T val)
{
	if (val == T(0)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] += val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator-=(T val)
{
	if (val == T(0)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] -= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator*=(T val)
{
	if (val == T(1)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] *= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator/=(T val)
{
	if (val == T(1)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] /= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator%=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] %= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator&=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] &= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator|=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] |= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator^=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] ^= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator<<=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] <<= val;
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator>>=(T val)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] >>= val;
	
	return *this;
}

// FixedVector member assign/equals operators
template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator+=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] += source.vec_data_[i];
		
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator-=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] -= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator*=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] *= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator/=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] /= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator%=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] %= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator&=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] &= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator|=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] |= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator^=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] ^= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator<<=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] <<= source.vec_data_[i];
	
	return *this;
}

template<class T,Uint8 N> 
inline FixedVector<T,N>& FixedVector<T,N>::operator>>=(const FixedVector<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		vec_data_[i] >>= source.vec_data_[i];
	
	return *this;
}

} // namespace mcs

#include "MCS_FixedVectorOperatorsNew.h"

#include "MCS_FixedVectorIO.h"

#include "MCS_FixedVectorStatistics.h"
#include "MCS_FixedVectorTypecasts.h"

#include "MCS_FixedVector1.h"
#include "MCS_FixedVector2.h"
#include "MCS_FixedVector3.h"
#include "MCS_FixedVector4.h"
#include "MCS_FixedVector5.h"
#include "MCS_FixedVector6.h"
#include "MCS_FixedVector7.h"
#include "MCS_FixedVector8.h"
#include "MCS_FixedVector9.h"
#include "MCS_FixedVector10.h"

#endif // MCS_FIXEDVECTOR_H