#ifndef MCS_BENCHMARK_H
#define MCS_BENCHMARK_H

#include "afx.h"

#include "MCS_Exceptions.h"

// local utility functions
namespace {

volatile void DunselFunction() { return; }

} // namespace

namespace mcs {

template<bool On = true> class Benchmark
{
	public:
		typedef ::LARGE_INTEGER	large_integer_type;
		
		Benchmark();
		~Benchmark();
		
		void reset() { tStart = ticks(); }
		
		large_integer_type ticks() const;
		double seconds() const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		enum { ttuUnknown = 0, ttuHiRes, ttuClock } TimerType;
		
		static void DetermineTimer();
		
							
		static large_integer_type 	PerfFreq;      		// ticks per second
		static int					TimerToUse,			// is hi res timer available
									PerfFreqAdjust, 	// in case Freq is too big
									OverheadTicks;		// overhead in calling timer

		mutable large_integer_type	tStart;
};

template<bool On> typename Benchmark<On>::large_integer_type Benchmark<On>::PerfFreq;
template<bool On> int Benchmark<On>::TimerToUse = Benchmark<On>::ttuUnknown;
template<bool On> int Benchmark<On>::PerfFreqAdjust = 0;
template<bool On> int Benchmark<On>::OverheadTicks = 0;

template<bool On>
inline Benchmark<On>::Benchmark()
{
	using namespace std;
	
	DetermineTimer();
	
	// Let any other stuff happen before we start
	MSG msg;
	PeekMessage(&msg,NULL,NULL,NULL,PM_NOREMOVE);
	Sleep(0);

	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL);
	
	tStart = ticks();
}

template<bool On>
inline Benchmark<On>::~Benchmark()
{
	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL);
}

template<bool On>
inline void Benchmark<On>::DetermineTimer()
{
	using namespace std;
	
	if (TimerToUse == ttuUnknown)
	{
		volatile void (*pFunc)() = DunselFunction;

		// Assume the worst
		TimerToUse = ttuClock;

		if (QueryPerformanceFrequency(&PerfFreq))
		{
			// We can use hires timer, determine overhead
			TimerToUse = ttuHiRes;
			OverheadTicks = 200;
			
			for (int i=0;i<20;i++)
			{
				large_integer_type b,e;
				int Ticks;
				
				QueryPerformanceCounter(&b);
				(*pFunc)();
				QueryPerformanceCounter(&e);
				
				Ticks = e.LowPart - b.LowPart;
				
				if ( Ticks >= 0 && Ticks < OverheadTicks )
					OverheadTicks = Ticks;
			}
			
			// See if Freq fits in 32 bits; if not lose some precision
			PerfFreqAdjust = 0;
			
			int High32 = PerfFreq.HighPart;
			
			while (High32)
			{
				High32 >>= 1;
				PerfFreqAdjust++;
			}
		}
	}
	
	return;
}

template<bool On>
inline typename Benchmark<On>::large_integer_type Benchmark<On>::ticks() const
{
	using namespace std;
	
	large_integer_type tck;
	
	if ( TimerToUse == ttuHiRes )
	{
		QueryPerformanceCounter(&tck);
	}
	else if (TimerToUse == ttuClock)
	{
		tck.LowPart = clock();
	}
	else throw Err();
	
	return tck;
}

template<bool On>
inline double Benchmark<On>::seconds() const
{
	using namespace std;
	
	large_integer_type	tStop = ticks();
	
	if (TimerToUse == ttuHiRes)
	{
		large_integer_type	Freq = PerfFreq;

		int	Oht = OverheadTicks,
		ReduceMag = 0;

		// Results are 64 bits but we only do 32
		unsigned int	High32 = tStop.HighPart-tStart.HighPart;

		while (High32)
		{
			High32 >>= 1;
			ReduceMag++;
		}

		if (PerfFreqAdjust || ReduceMag)
		{
			if (PerfFreqAdjust > ReduceMag)
				ReduceMag = PerfFreqAdjust;

			tStart.QuadPart = Int64ShrlMod32(tStart.QuadPart, ReduceMag);
			tStop.QuadPart = Int64ShrlMod32(tStop.QuadPart, ReduceMag);
			Freq.QuadPart = Int64ShrlMod32(Freq.QuadPart, ReduceMag);
			Oht >>= ReduceMag;
		}

		// Reduced numbers to 32 bits, now can do the math
		if (Freq.LowPart == 0)
			return 0.0;
		else
			return (static_cast<double>(tStop.LowPart - tStart.LowPart  - Oht))/Freq.LowPart;
	}
	else
	{
		return static_cast<double>(tStop.LowPart-tStart.LowPart)/CLOCKS_PER_SEC;
	}
}

} // namespace mcs

#endif // MCS_BENCHMARK_H

/*
//Original code off MSDN
#include "afx.h"

enum { ttuUnknown, ttuHiRes, ttuClock } TimerToUse = ttuUnknown;
LARGE_INTEGER PerfFreq;      // ticks per second
int PerfFreqAdjust; // in case Freq is too big
int OverheadTicks;   // overhead  in calling timer

void DunselFunction() { return; }

void DetermineTimer()
{
   void (*pFunc)() = DunselFunction;

   // Assume the worst
   TimerToUse = ttuClock;
   if ( QueryPerformanceFrequency(&PerfFreq) )
      {
      // We can use hires timer, determine overhead
      TimerToUse = ttuHiRes;
      OverheadTicks = 200;
      for ( int i=0; i < 20; i++ )
         {
         LARGE_INTEGER b,e;
         int Ticks;
         QueryPerformanceCounter(&b);
         (*pFunc)();
         QueryPerformanceCounter(&e);
         Ticks = e.LowPart - b.LowPart;
         if ( Ticks >= 0 && Ticks < OverheadTicks )
            OverheadTicks = Ticks;
         }
      // See if Freq fits in 32 bits; if not lose some precision
      PerfFreqAdjust = 0;
      int High32 = PerfFreq.HighPart;
      while ( High32 )
         {
         High32 >>= 1;
         PerfFreqAdjust++;
         }
      }
   return;
}

double DoBench(void(*funcp)())
{
   double time;      // 

   // Let any other stuff happen before we start
   MSG msg;
   PeekMessage(&msg,NULL,NULL,NULL,PM_NOREMOVE);
   Sleep(0);

   if ( TimerToUse == ttuUnknown )
      DetermineTimer();
      
   if ( TimerToUse == ttuHiRes )
      {
      LARGE_INTEGER tStart, tStop;
      LARGE_INTEGER Freq = PerfFreq;
      int Oht = OverheadTicks;
      int ReduceMag = 0;
      SetThreadPriority(GetCurrentThread(), 
         THREAD_PRIORITY_TIME_CRITICAL);
      QueryPerformanceCounter(&tStart);
        (*funcp)();   //call the actual function being timed
      QueryPerformanceCounter(&tStop);
      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
      // Results are 64 bits but we only do 32
      unsigned int High32 = tStop.HighPart - tStart.HighPart;
      while ( High32 )
         {
         High32 >>= 1;
         ReduceMag++;
         }
      if ( PerfFreqAdjust || ReduceMag )
         {
         if ( PerfFreqAdjust > ReduceMag )
            ReduceMag = PerfFreqAdjust;
         tStart.QuadPart = Int64ShrlMod32(tStart.QuadPart, ReduceMag);
         tStop.QuadPart = Int64ShrlMod32(tStop.QuadPart, ReduceMag);
         Freq.QuadPart = Int64ShrlMod32(Freq.QuadPart, ReduceMag);
         Oht >>= ReduceMag;
         }

      // Reduced numbers to 32 bits, now can do the math
      if ( Freq.LowPart == 0 )
         time = 0.0;
      else
         time = ((double)(tStop.LowPart - tStart.LowPart 
            - Oht))/Freq.LowPart;
      }
   else
      {
        long stime, etime;
      SetThreadPriority(GetCurrentThread(),
          THREAD_PRIORITY_TIME_CRITICAL);
      stime = clock();
      (*funcp)();
      etime = clock();
        SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
      time = ((double)(etime - stime)) / CLOCKS_PER_SEC;
      }

  return (time);
}
*/
