#ifndef MCS_BOUNDS_H
#define MCS_BOUNDS_H

#include "MCS_Ensure.h"
#include "MCS_Exceptions.h"
#include "MCS_Vector.h"

namespace mcs {

// evenly spaced bins
template<class T> 
class Bounds : public Vector< std::pair<T,T> >
{
	public:
		Bounds(const T& lo,const T& hi,const Int32& nbins);
		~Bounds() { }
		
		Index<1> find(const T& val) const;
		
		Vector<T> centers() const;
		
		const T& min() const	{ return min_; }
		const T& max() const	{ return max_; }
		const T& delta() const	{ return delta_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		static const EnsureReal<T>	dummy_;
	
		T	min_,max_,delta_;
};

// implementation
template<class T> const EnsureReal<T> Bounds<T>::dummy_;

template<class T> 
Bounds<T>::Bounds(const T& lo,const T& hi,const Int32& nbins) : 
	Vector< std::pair<T,T> >(IndexRange<1>(0,nbins+1)),
	min_(lo),
	max_(hi),
	delta_((hi-lo)/nbins)
{
	static const T	min_val = std::numeric_limits<T>::is_integer ? std::numeric_limits<T>::min() : _minus_Inf_,
					max_val = std::numeric_limits<T>::is_integer ? std::numeric_limits<T>::max() : _plus_Inf_;
					
	if (lo < min_val || hi > max_val) throw Err();
	if (hi <= lo) throw Err();
	
	(*this)[first()].first = min_val;
	(*this)[first()].second = min_;
	
	for (Index<1> i=first()+1;i<=last()-1;i++)
	{
		(*this)[i].first = min_+(i-1)*delta_;
		(*this)[i].second = (*this)[i].first+delta_;
		
		if ((*this)[i].second <= (*this)[i].first) throw Err();
	}
	
	(*this)[last()].first = max_;
	(*this)[last()].second = max_val;
}

template<class T> 
inline Index<1> Bounds<T>::find(const T& val) const
{
	if (val < min_) 
		return first();
	else if (val > max_) 
		return last();
	else
		return static_cast< Index<1> >(std::round((val-min_)/delta_));
}

template<class T> 
inline Vector<T> Bounds<T>::centers() const
{
	Vector<T>	bc(range());
	
	for (Index<1> i=first();i<=last();++i)
		bc[i] = ((*this)[i].first + (*this)[i].second)/2;
		
	return bc;
}

} // namespace mcs

#endif // MCS_BOUNDS_H