#ifndef MCS_CACHE_H
#define MCS_CACHE_H

#include <deque>
#include <map>

#include "MCS_Exceptions.h"

namespace mcs {

template<class Idx,class Val> class Cache
{
	public:
		Cache(const Int32& max_size = 1);
		~Cache() { }
		
		bool inCache(const Idx&) const;
		Val& get(const Idx&);
		
		Val& add(const Idx&,const Val& = Val());
		
		void size(const Int32& max_size);
		
		const Int32& size() const	{ return max_size_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void shrink(const Int32& n);
		
		Int32				max_size_;
		std::deque<Idx>		idx_cache_;
		std::map<Idx,Val>	val_cache_;
};

// implementation
template<class Idx,class Val>
inline Cache<Idx,Val>::Cache(const Int32& max_size) :
	max_size_(std::max(max_size,1))
{ }

template<class Idx,class Val>
inline bool Cache<Idx,Val>::inCache(const Idx& idx) const
{
	return (std::find(idx_cache_.begin(),idx_cache_.end(),idx) != idx_cache_.end());
}

template<class Idx,class Val>
inline Val& Cache<Idx,Val>::get(const Idx& idx)
{
	// if idx is not in cache
	if (!inCache(idx)) throw Err();
	
	return val_cache_[idx];	
}

template<class Idx,class Val>
inline Val& Cache<Idx,Val>::add(const Idx& idx,const Val& val)
{
	// if idx is not in cache
	if (!inCache(idx))
	{
		// if cache is full, pop off last 
		if (idx_cache_.size() == max_size_) 
			shrink(1);
					
		idx_cache_.push_front(idx); 
		val_cache_[idx] = val;
	}

	return get(idx);
}

// set cache size
//	- minimum cache size is one element
template<class Idx,class Val>
inline void Cache<Idx,Val>::size(const Int32& max_size)
{
	if (max_size < 1) throw Err();
	
	// if new cache size is smaller, shrink
	if (max_size < max_size_)
		shrink(max_size_-max_size);
		
	max_size_ = max_size;
}

// remove last n elements in cache
//	- minimum cache size is one element
template<class Idx,class Val>
void Cache<Idx,Val>::shrink(const Int32& n)
{
	if (n <= 0) return;
	
	for (Int32 i=1;i<=std::min(n,max_size_-1);++i)
	{
		val_cache_.erase(idx_cache_.back());
		idx_cache_.pop_back();
	}
}

// specialize for pointer types
template<class Idx,class Val> class Cache<Idx,Val*>
{
	public:
		Cache(const Int32& max_size = 1);
		~Cache();
		
		bool inCache(const Idx&) const;
		Val& get(const Idx&);
		
		Val& add(const Idx&);
		Val& add(const Idx&,const Val&);
		
		void size(const Int32& max_size);
		
		const Int32& size() const	{ return max_size_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void shrink(const Int32& n);
		
		Int32				max_size_;
		std::deque<Idx>		idx_cache_;
		std::map<Idx,Val*>	val_cache_;
};

// implementation
template<class Idx,class Val>
inline Cache<Idx,Val*>::Cache(const Int32& max_size) :
	max_size_(std::max(max_size,1))
{ }

template<class Idx,class Val>
Cache<Idx,Val*>::~Cache()
{
	for (typename std::map<Idx,Val*>::iterator it=val_cache_.begin();it!=val_cache_.end();++it)
		delete it->second;
}

template<class Idx,class Val>
inline bool Cache<Idx,Val*>::inCache(const Idx& idx) const
{
	return (std::find(idx_cache_.begin(),idx_cache_.end(),idx) != idx_cache_.end());
}

template<class Idx,class Val>
inline Val& Cache<Idx,Val*>::get(const Idx& idx)
{
	// if idx is not in cache
	if (!inCache(idx)) throw Err();
	
	return *(val_cache_[idx]);	
}

template<class Idx,class Val>
inline Val& Cache<Idx,Val*>::add(const Idx& idx)
{
	// if idx is not in cache
	if (!inCache(idx))
	{
		// if cache is full, pop off last 
		if (idx_cache_.size() == max_size_) 
			shrink(1);
					
		idx_cache_.push_front(idx); 
		val_cache_[idx] = new Val();
	}

	return get(idx);
}

template<class Idx,class Val>
inline Val& Cache<Idx,Val*>::add(const Idx& idx,const Val& val)
{
	// if idx is not in cache
	if (!inCache(idx))
	{
		// if cache is full, pop off last 
		if (idx_cache_.size() == max_size_) 
			shrink(1);
					
		idx_cache_.push_front(idx); 
		val_cache_[idx] = new Val(val);
	}

	return get(idx);
}

// set cache size
//	- minimum cache size is one element
template<class Idx,class Val>
inline void Cache<Idx,Val*>::size(const Int32& max_size)
{
	if (max_size < 1) throw Err();
	
	// if new cache size is smaller, shrink
	if (max_size < max_size_)
		shrink(max_size_-max_size);
		
	max_size_ = max_size;
}

// remove last n elements in cache
//	- minimum cache size is one element
template<class Idx,class Val>
void Cache<Idx,Val*>::shrink(const Int32& n)
{
	if (n <= 0) return;
	
	for (Int32 i=1;i<=std::min(n,max_size_-1);++i)
	{
		delete val_cache_[idx_cache_.back()];
		
		val_cache_.erase(idx_cache_.back());
		idx_cache_.pop_back();
	}
}

} // namespace mcs

#endif // MCS_CACHE_H