#ifndef MCS_ENSURE_H
#define MCS_ENSURE_H

#include "MCS_Typedefs.h"

namespace mcs {

/*
	These classes enable an assertion to restrict instantiated classes/functions/etc..
		to a specific set of acceptable types.  Usage is normally to enclose a dummy
		static const variable, either in the private members of the class or in the 
		function definition : 
		
			static const Ensure<T>	dummy_;
			
		Since the constructor doesn't exist for non-allowed T, this will fail.  
		The technique is good for preventing unintended consequences in templated code.
*/

// require a plain-old-data type (POD)
template<class T> class EnsurePOD				{ private: EnsurePOD(); };

template<> class EnsurePOD<Binary>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Int8>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Uint8>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Int16>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Uint16>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Int32>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Uint32>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Float32>				{ public: EnsurePOD() { } };
template<> class EnsurePOD<Float64>				{ public: EnsurePOD() { } };

// require a numeric type (real or complex)
template<class T> class EnsureNumeric			{ private: EnsureNumeric(); };

template<> class EnsureNumeric<Binary>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Int8>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Uint8>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Int16>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Uint16>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Int32>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Uint32>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Float32>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Float64>			{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Complex32>		{ public: EnsureNumeric() { } };
template<> class EnsureNumeric<Complex64>		{ public: EnsureNumeric() { } };

// require an integer type
template<class T> class EnsureInteger			{ private: EnsureInteger(); };

template<> class EnsureInteger<Int8>			{ public: EnsureInteger() { } };
template<> class EnsureInteger<Uint8>			{ public: EnsureInteger() { } };
template<> class EnsureInteger<Int16>			{ public: EnsureInteger() { } };
template<> class EnsureInteger<Uint16>			{ public: EnsureInteger() { } };
template<> class EnsureInteger<Int32>			{ public: EnsureInteger() { } };
template<> class EnsureInteger<Uint32>			{ public: EnsureInteger() { } };

// require a real valued type
template<class T> class EnsureReal				{ private: EnsureReal(); };

template<> class EnsureReal<Int8>				{ public: EnsureReal() { } };
template<> class EnsureReal<Uint8>				{ public: EnsureReal() { } };
template<> class EnsureReal<Int16>				{ public: EnsureReal() { } };
template<> class EnsureReal<Uint16>				{ public: EnsureReal() { } };
template<> class EnsureReal<Int32>				{ public: EnsureReal() { } };
template<> class EnsureReal<Uint32>				{ public: EnsureReal() { } };
template<> class EnsureReal<Float32>			{ public: EnsureReal() { } };
template<> class EnsureReal<Float64>			{ public: EnsureReal() { } };

// require a floating point type
template<class T> class EnsureFloat				{ private: EnsureFloat(); };

template<> class EnsureFloat<Float32>			{ public: EnsureFloat() { } };
template<> class EnsureFloat<Float64>			{ public: EnsureFloat() { } };
template<> class EnsureFloat<Complex32>			{ public: EnsureFloat() { } };
template<> class EnsureFloat<Complex64>			{ public: EnsureFloat() { } };

// require a real floating point type
template<class T> class EnsureRealFloat			{ private: EnsureRealFloat(); };

template<> class EnsureRealFloat<Float32>		{ public: EnsureRealFloat() { } };
template<> class EnsureRealFloat<Float64>		{ public: EnsureRealFloat() { } };

// require a complex valued type
template<class T> class EnsureComplex			{ private: EnsureComplex(); };

template<> class EnsureComplex<Int8>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Uint8>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Int16>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Uint16>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Int32>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Uint32>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Float32>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Float64>			{ public: EnsureComplex() { } };
template<> class EnsureComplex<Complex32>		{ public: EnsureComplex() { } };
template<> class EnsureComplex<Complex64>		{ public: EnsureComplex() { } };

// require a complex valued type
template<class T> class EnsureComplexFloat		{ private: EnsureComplexFloat(); };

template<> class EnsureComplexFloat<Complex32>	{ public: EnsureComplexFloat() { } };
template<> class EnsureComplexFloat<Complex64>	{ public: EnsureComplexFloat() { } };

} // namespace mcs

#endif // MCS_ENSURE_H
