#ifndef MCS_INDEXRANGE_H
#define MCS_INDEXRANGE_H

#include <iostream>

#include "MCS_Signature.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<Uint8 R>
class IndexRange
{
	public:
		typedef typename Signature<R>::IndexType	IndexType;
		
		IndexRange();
		explicit IndexRange(const IndexType&);
		explicit IndexRange(const IndexType&,const IndexType&);
		IndexRange(const IndexRange<R>&);
		virtual ~IndexRange();

		IndexRange<R>& operator=(const IndexRange<R>&);

		const Int32 elem() const						{ return elem_; }

		const IndexType& first() const					{ return first_; }
		const IndexType& last() const					{ return last_; }
		const IndexType& dim() const					{ return dim_; }
		//const IndexType& offset() const					{ return first_; }

		Int32 first(Int32 rnk) const					{ return first_[rnk]; }
		Int32 last(Int32 rnk) const						{ return last_[rnk]; }
		Int32 dim(Int32 rnk) const						{ return dim_[rnk]; }
		//const Int32& offset(const Uint8& rnk) const		{ return first_[rnk]; }

		void setOffset(const IndexType&);
		
		bool operator==(const IndexRange<R>&) const;
		bool operator!=(const IndexRange<R>&) const;

		IndexRange<1> range(Int32 d) const				{ return IndexRange<1>(first_[d],last_[d]); }

		void permute(const Signature<R>&);
		void inverse_permute(const Signature<R>&);
		void transpose(const Uint8&,const Uint8&);
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	protected:
		void getNumberOfElements();

		IndexType	first_,last_,dim_;
		Int32		elem_;
};

template<Uint8 R>
inline IndexRange<R>::IndexRange() :
	first_(0),
	last_(-1),
	dim_(-1)
{
	getNumberOfElements();
}

template<Uint8 R>
inline IndexRange<R>::IndexRange(const IndexType& last) :
	first_(0),
	last_(last-1),
	dim_(last)
{
	getNumberOfElements();
}

template<Uint8 R>
inline IndexRange<R>::IndexRange(const IndexType& first,const IndexType& last) :
	first_(first),
	last_(last),
	dim_(last-first+1)
{
	getNumberOfElements();
}

template<Uint8 R>
inline IndexRange<R>::IndexRange(const IndexRange<R>& source) :
	first_(source.first_),
	last_(source.last_),
	dim_(source.dim_),
	elem_(source.elem_)
{ }

template<Uint8 R>
inline IndexRange<R>::~IndexRange()
{ }

template<Uint8 R>
inline IndexRange<R>& IndexRange<R>::operator=(const IndexRange<R>& source)
{
	if (this == &source) return *this;

	first_ = source.first_;
	last_ = source.last_;
	dim_ = source.dim_;
	elem_ = source.elem_;

	return *this;
}

template<Uint8 R>
inline void IndexRange<R>::setOffset(const IndexType& idx)
{
	first_ = idx;
	last_ = idx+dim_-1;
}

template<Uint8 R>
inline void IndexRange<R>::permute(const Signature<R>& sig)
{
	first_ = sig.permute(first_);
	last_ = sig.permute(last_);
	
	*this = IndexRange<R>(first_,last_);
}

template<Uint8 R>
inline void IndexRange<R>::inverse_permute(const Signature<R>& sig)
{
	first_ = sig.inverse_permute(first_);
	last_ = sig.inverse_permute(last_);
	
	*this = IndexRange<R>(first_,last_);
}

template<Uint8 R>
inline void IndexRange<R>::transpose(const Uint8& rank1,const Uint8& rank2)
{
	if (rank1 > R-1 || rank2 > R-1) throw Err();
	
	std::swap(first_[rank1],first_[rank2]);
	std::swap(last_[rank1],last_[rank2]);
	std::swap(dim_[rank1],dim_[rank2]);
}

template<Uint8 R>
inline void IndexRange<R>::deserialize(std::istream& is)
{
	IndexType	first,
				last;
				
	read(first,is);
	read(last,is);
	
	*this = IndexRange<R>(first,last);
}

template<Uint8 R>
inline void IndexRange<R>::serialize(std::ostream& os) const
{
	write(first_,os);
	write(last_,os);
}

template<Uint8 R>
inline void IndexRange<R>::getNumberOfElements()
{
	if (dim_[dim_.first()] <= 0)
	{
		elem_ = 0;
		return;
	}
	else
	{
		elem_ = 1;
	}

	for (Int32 i=dim_.first();i<=dim_.last();i++)
	{
		if (dim_[i] <= 0) 
		{
			elem_ = 0;
			return;
		}
		
		elem_ *= dim_[i];
	}
}

template<Uint8 R>
inline bool IndexRange<R>::operator==(const IndexRange<R>& source) const
{
	return (all(first() == source.first()) && all(last() == source.last()));
}

template<Uint8 R>
inline bool IndexRange<R>::operator!=(const IndexRange<R>& source) const
{
	return (any(first() != source.first()) || any(last() != source.last()));
}

template<Uint8 R>
inline std::ostream& operator<<(std::ostream& ostr,const IndexRange<R>& source)
{
	ostr << "{ " << source.first() << ".." << source.last() << " }";

	return ostr;
}

template<Uint8 R>
inline std::istream& operator>>(std::istream& istr,IndexRange<R>& dest);

} // namespace mcs

#endif // MCS_INDEXRANGE_H