#ifndef MCS_ITERATOR_H
#define MCS_ITERATOR_H

#include "MCS_Dereferencer.h"
#include "MCS_Exceptions.h"

namespace mcs {

template<class P,class Imp> class Iterator;
template<class P,class Imp> class ConstIterator;
template<class P,class Imp> class ReverseIterator;
template<class P,class Imp> class ConstReverseIterator;

// generalized Iterator class
//
//	class Imp must support appropriate arithmetic and comparison operators
//	and class P must be initializable with Imp or must specialize Dereferencer<P,Imp>
template<class P,class Imp = Int32>
class Iterator
{
	public:
		typedef P	PointeeType;
		
		explicit Iterator(const Imp& imp);
		Iterator(const ConstIterator<P,Imp>& it);
		Iterator(const Iterator<P,Imp>& source);
		~Iterator() { }
				
		Iterator<P,Imp>& operator=(const Iterator<P,Imp>&);
		Iterator<P,Imp>& operator=(const Imp&);
		
		bool		operator==(const Iterator<P,Imp>& it) const		{ return imp_ == it.imp_; }
		bool		operator!=(const Iterator<P,Imp>& it) const		{ return imp_ != it.imp_; }
		bool		operator<(const Iterator<P,Imp>& it) const		{ return imp_ < it.imp_; }
		bool		operator<=(const Iterator<P,Imp>& it) const		{ return imp_ <= it.imp_; }
		bool		operator>(const Iterator<P,Imp>& it) const		{ return imp_ > it.imp_; }
		bool		operator>=(const Iterator<P,Imp>& it) const		{ return imp_ >= it.imp_; }
		
		Iterator<P,Imp>&	operator+=(const Int32& num)			{ imp_ += num; return *this; }
		Iterator<P,Imp>&	operator-=(const Int32& num)			{ imp_ -= num; return *this; }
		
		Int32		operator-(const Iterator<P,Imp>& it) const		{ return imp_-it.imp_; }
		
		Iterator<P,Imp>&		operator++() 						{ ++imp_; return *this; }
		Iterator<P,Imp>&		operator--()						{ --imp_; return *this; }
		const Iterator<P,Imp>	operator++(int)						{ Iterator<P,Imp> tmp(*this); ++imp_; return tmp; }
		const Iterator<P,Imp>	operator--(int)						{ Iterator<P,Imp> tmp(*this); --imp_; return tmp; }
		
		Iterator<P,Imp>	operator+(const Int32& num)	const			{ Iterator<P,Imp> tmp(*this); tmp += num; return tmp; }
		Iterator<P,Imp>	operator-(const Int32& num)	const			{ Iterator<P,Imp> tmp(*this); tmp -= num; return tmp; }
		
		const P&	operator*()										{ return deref_.dereference(imp_); }
		const P&	operator[](const Int32& num)					{ imp_ = num; return deref_.dereference(imp_); }
		
		const P*	operator->()									{ deref_.dereference(imp_); return deref_.operator->(); }
		
		const Imp& 	operator()() const								{ return imp_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:		
		Iterator();
		
		Imp					imp_;
		Dereferencer<P,Imp>	deref_;
};

template<class P,class Imp>
inline Iterator<P,Imp>::Iterator(const Imp& imp) :
	imp_(imp),
	deref_(imp)
{ }

template<class P,class Imp> 
inline Iterator<P,Imp>::Iterator(const ConstIterator<P,Imp>& it) :
	imp_(it()),
	deref_(it())
{ }

template<class P,class Imp> 
inline Iterator<P,Imp>::Iterator(const Iterator<P,Imp>& source) :
	imp_(source.imp_),
	deref_(source.deref_)
{ }

template<class P,class Imp> 
inline Iterator<P,Imp>& Iterator<P,Imp>::operator=(const Iterator<P,Imp>& source)
{
	if (this == &source) return *this;
	
	imp_ = source.imp_;
	deref_ = source.deref_;
	
	return *this;
}

template<class P,class Imp> 
inline Iterator<P,Imp>& Iterator<P,Imp>::operator=(const Imp& imp)
{
	imp_ = imp;
	deref_ = Dereferencer<P,Imp>(imp_);
	
	return *this;
}

template<class P,class Imp = Int32>
class ConstIterator
{
	public:
		typedef P	PointeeType;
		
		explicit ConstIterator(const Imp& imp);
		ConstIterator(const Iterator<P,Imp>& it);
		ConstIterator(const ConstIterator<P,Imp>& source);
		~ConstIterator() { }
				
		ConstIterator<P,Imp>& operator=(const ConstIterator<P,Imp>&);
		ConstIterator<P,Imp>& operator=(const Imp&);
		
		bool		operator==(const ConstIterator<P,Imp>& it) const	{ return imp_ == it.imp_; }
		bool		operator!=(const ConstIterator<P,Imp>& it) const	{ return imp_ != it.imp_; }
		bool		operator<(const ConstIterator<P,Imp>& it) const		{ return imp_ < it.imp_; }
		bool		operator<=(const ConstIterator<P,Imp>& it) const	{ return imp_ <= it.imp_; }
		bool		operator>(const ConstIterator<P,Imp>& it) const		{ return imp_ > it.imp_; }
		bool		operator>=(const ConstIterator<P,Imp>& it) const	{ return imp_ >= it.imp_; }
		
		ConstIterator<P,Imp>&	operator+=(const Int32& num)			{ imp_ += num; return *this; }
		ConstIterator<P,Imp>&	operator-=(const Int32& num)			{ imp_ -= num; return *this; }
		
		Int32		operator-(const ConstIterator<P,Imp>& it) const		{ return imp_-it.imp_; }
		
		ConstIterator<P,Imp>&		operator++() 						{ ++imp_; return *this; }
		ConstIterator<P,Imp>&		operator--()						{ --imp_; return *this; }
		const ConstIterator<P,Imp>	operator++(int)						{ ConstIterator<P,Imp> tmp(*this); ++imp_; return tmp; }
		const ConstIterator<P,Imp>	operator--(int)						{ ConstIterator<P,Imp> tmp(*this); --imp_; return tmp; }
		
		ConstIterator<P,Imp>	operator+(const Int32& num)	const		{ ConstIterator<P,Imp> tmp(*this); tmp += num; return tmp; }
		ConstIterator<P,Imp>	operator-(const Int32& num)	const		{ ConstIterator<P,Imp> tmp(*this); tmp -= num; return tmp; }
			
		const P&	operator*()											{ return deref_.dereference(imp_); }
		const P&	operator[](const Int32& num)						{ imp_ = num; return deref_.dereference(imp_); }
		
		const P*	operator->()										{ deref_.dereference(imp_); return deref_.operator->(); }
		
		const Imp& 	operator()() const									{ return imp_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:		
		ConstIterator();
		
		Imp					imp_;
		Dereferencer<P,Imp>	deref_;
};

template<class P,class Imp>
inline ConstIterator<P,Imp>::ConstIterator(const Imp& imp) :
	imp_(imp),
	deref_(imp)
{ }

template<class P,class Imp> 
inline ConstIterator<P,Imp>::ConstIterator(const Iterator<P,Imp>& it) :
	imp_(it()),
	deref_(it())
{ }

template<class P,class Imp> 
inline ConstIterator<P,Imp>::ConstIterator(const ConstIterator<P,Imp>& source) :
	imp_(source.imp_),
	deref_(source.deref_)
{ }

template<class P,class Imp> 
inline ConstIterator<P,Imp>& ConstIterator<P,Imp>::operator=(const ConstIterator<P,Imp>& source)
{
	if (this == &source) return *this;
	
	imp_ = source.imp_;
	deref_ = source.deref_;
	
	return *this;
}

template<class P,class Imp> 
inline ConstIterator<P,Imp>& ConstIterator<P,Imp>::operator=(const Imp& imp)
{
	imp_ = imp;
	deref_ = Dereferencer<P,Imp>(imp_);
	
	return *this;
}

// generalized ReverseIterator class
//
//	class Imp must support appropriate arithmetic and comparison operators
//	and class P must be initializable with Imp or must specialize Dereferencer<P,Imp>
template<class P,class Imp = Int32>
class ReverseIterator
{
	public:
		typedef P	PointeeType;
		
		explicit ReverseIterator(const Imp& imp);
		ReverseIterator(const ConstReverseIterator<P,Imp>& it);
		ReverseIterator(const ReverseIterator<P,Imp>& source);
		~ReverseIterator() { }
				
		ReverseIterator<P,Imp>& operator=(const ReverseIterator<P,Imp>&);
		ReverseIterator<P,Imp>& operator=(const Imp&);
		
		bool		operator==(const ReverseIterator<P,Imp>& it) const		{ return imp_ == it.imp_; }
		bool		operator!=(const ReverseIterator<P,Imp>& it) const		{ return imp_ != it.imp_; }
		bool		operator<(const ReverseIterator<P,Imp>& it) const		{ return imp_ > it.imp_; }
		bool		operator<=(const ReverseIterator<P,Imp>& it) const		{ return imp_ >= it.imp_; }
		bool		operator>(const ReverseIterator<P,Imp>& it) const		{ return imp_ < it.imp_; }
		bool		operator>=(const ReverseIterator<P,Imp>& it) const		{ return imp_ <= it.imp_; }
		
		ReverseIterator<P,Imp>&	operator+=(const Int32& num)				{ imp_ -= num; return *this; }
		ReverseIterator<P,Imp>&	operator-=(const Int32& num)				{ imp_ += num; return *this; }
		
		Int32		operator-(const ReverseIterator<P,Imp>& it) const		{ return it.imp_-imp_; }
		
		ReverseIterator<P,Imp>&			operator++() 						{ --imp_; return *this; }
		ReverseIterator<P,Imp>&			operator--()						{ ++imp_; return *this; }
		const ReverseIterator<P,Imp>	operator++(int)						{ ReverseIterator<P,Imp> tmp(*this); --imp_; return tmp; }
		const ReverseIterator<P,Imp>	operator--(int)						{ ReverseIterator<P,Imp> tmp(*this); ++imp_; return tmp; }
		
		ReverseIterator<P,Imp>	operator+(const Int32& num)	const			{ ReverseIterator<P,Imp> tmp(*this); tmp += num; return tmp; }
		ReverseIterator<P,Imp>	operator-(const Int32& num)	const			{ ReverseIterator<P,Imp> tmp(*this); tmp -= num; return tmp; }
		
		const P&	operator*()												{ return deref_.dereference(imp_); }
		const P&	operator[](const Int32& num)							{ imp_ = num; return deref_.dereference(imp_); }
		
		const P*	operator->()											{ deref_.dereference(imp_); return deref_.operator->(); }
			
		const Imp& 	operator()() const										{ return imp_; }
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:		
		ReverseIterator();
		
		Imp					imp_;
		Dereferencer<P,Imp>	deref_;
};

template<class P,class Imp>
inline ReverseIterator<P,Imp>::ReverseIterator(const Imp& imp) :
	imp_(imp),
	deref_(imp)
{ }

template<class P,class Imp> 
inline ReverseIterator<P,Imp>::ReverseIterator(const ConstReverseIterator<P,Imp>& it) :
	imp_(it()),
	deref_(it())
{ }

template<class P,class Imp> 
inline ReverseIterator<P,Imp>::ReverseIterator(const ReverseIterator<P,Imp>& source) :
	imp_(source.imp_),
	deref_(source.deref_)
{ }

template<class P,class Imp> 
inline ReverseIterator<P,Imp>& ReverseIterator<P,Imp>::operator=(const ReverseIterator<P,Imp>& source)
{
	if (this == &source) return *this;
	
	imp_ = source.imp_;
	deref_ = source.deref_;
	
	return *this;
}

template<class P,class Imp> 
inline ReverseIterator<P,Imp>& ReverseIterator<P,Imp>::operator=(const Imp& imp)
{
	imp_ = imp;
	deref_ = Dereferencer<P,Imp>(imp_);
	
	return *this;
}

template<class P,class Imp = Int32>
class ConstReverseIterator
{
	public:
		typedef P	PointeeType;
		
		explicit ConstReverseIterator(const Imp& imp);
		ConstReverseIterator(const ReverseIterator<P,Imp>& source);
		ConstReverseIterator(const ConstReverseIterator<P,Imp>& source);
		~ConstReverseIterator() { }
				
		ConstReverseIterator<P,Imp>& operator=(const ConstReverseIterator<P,Imp>&);
		ConstReverseIterator<P,Imp>& operator=(const Imp&);
		
		bool		operator==(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ == it.imp_; }
		bool		operator!=(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ != it.imp_; }
		bool		operator<(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ > it.imp_; }
		bool		operator<=(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ >= it.imp_; }
		bool		operator>(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ < it.imp_; }
		bool		operator>=(const ConstReverseIterator<P,Imp>& it) const		{ return imp_ <= it.imp_; }
		
		ConstReverseIterator<P,Imp>&	operator+=(const Int32& num)			{ imp_ -= num; return *this; }
		ConstReverseIterator<P,Imp>&	operator-=(const Int32& num)			{ imp_ += num; return *this; }
		
		Int32		operator-(const ConstReverseIterator<P,Imp>& it) const		{ return it.imp_-imp_; }
		
		ConstReverseIterator<P,Imp>&		operator++() 						{ --imp_; return *this; }
		ConstReverseIterator<P,Imp>&		operator--()						{ ++imp_; return *this; }
		const ConstReverseIterator<P,Imp>	operator++(int)						{ ConstReverseIterator<P,Imp> tmp(*this); --imp_; return tmp; }
		const ConstReverseIterator<P,Imp>	operator--(int)						{ ConstReverseIterator<P,Imp> tmp(*this); ++imp_; return tmp; }
		
		ConstReverseIterator<P,Imp>	operator+(const Int32& num)	const			{ ConstReverseIterator<P,Imp> tmp(*this); tmp += num; return tmp; }
		ConstReverseIterator<P,Imp>	operator-(const Int32& num)	const			{ ConstReverseIterator<P,Imp> tmp(*this); tmp -= num; return tmp; }
		
		const P&	operator*()													{ return deref_.dereference(imp_); }
		const P&	operator[](const Int32& num)								{ imp_ = num; return deref_.dereference(imp_); }
		
		const P*	operator->()												{ deref_.dereference(imp_); return deref_.operator->(); }
				
		const Imp& 	operator()() const											{ return imp_; }
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:		
		ConstReverseIterator();
		
		Imp					imp_;
		Dereferencer<P,Imp>	deref_;
};

template<class P,class Imp>
inline ConstReverseIterator<P,Imp>::ConstReverseIterator(const Imp& imp) :
	imp_(imp),
	deref_(imp)
{ }

template<class P,class Imp> 
inline ConstReverseIterator<P,Imp>::ConstReverseIterator(const ReverseIterator<P,Imp>& it) :
	imp_(it()),
	deref_(it())
{ }

template<class P,class Imp> 
inline ConstReverseIterator<P,Imp>::ConstReverseIterator(const ConstReverseIterator<P,Imp>& source) :
	imp_(source.imp_),
	deref_(source.deref_)
{ }

template<class P,class Imp> 
inline ConstReverseIterator<P,Imp>& ConstReverseIterator<P,Imp>::operator=(const ConstReverseIterator<P,Imp>& source)
{
	if (this == &source) return *this;
	
	imp_ = source.imp_;
	deref_ = source.deref_;
	
	return *this;
}

template<class P,class Imp> 
inline ConstReverseIterator<P,Imp>& ConstReverseIterator<P,Imp>::operator=(const Imp& imp)
{
	imp_ = imp;
	deref_ = Dereferencer<P,Imp>(imp_);
	
	return *this;
}

} // namespace mcs

#endif // MCS_ITERATOR_H