#ifndef MCS_MATHEXTENSIONS_H
#define MCS_MATHEXTENSIONS_H

#include <complex>

#include "MCS_Promote.h"
#include "MCS_Typedefs.h"

namespace mcs {

// mathematical constants
static const Float64	_e_(2.71828182845904523536),
						_pi_(3.14159265358979323846),
						_pi_over_two_(_pi_/2.0),
						_two_pi_(2.0*_pi_),
						#ifdef __MWERKS__
						_NaN_(NAN),
						#else
						_NaN_(0.0/0.0),
						#endif
						_minus_Inf_(-1.0/0.0),
						_plus_Inf_(1.0/0.0),
						_rad_per_deg_(_pi_/180.0),
						_deg_per_rad_(180.0/_pi_);

static const Complex64	_i_(0.0,1.0),
						_j_(0.0,1.0);	

// angle conversions
template<class T> inline T toDegrees(const T& source)							{ return source*(180/_pi_); }
template<class T> inline T toRadians(const T& source)							{ return source*(_pi_/180); }

template<class T> inline T angularSeparationDegrees(const T& a1,const T& a2)	{ return std::abs(mod(a1-a2,T(360))); }
template<class T> inline T angularSeparationRadians(const T& a1,const T& a2)	{ return std::abs(mod(a1-a2,_two_pi_)); }

} // namespace mcs

namespace std {

// unary
template<class T> inline T acosd(const T& x)				{ return mcs::toDegrees(std::acos(x)); }
template<class T> inline T asind(const T& x)				{ return mcs::toDegrees(std::asin(x)); }
template<class T> inline T atand(const T& x)				{ return mcs::toDegrees(std::atan(x)); }
template<class T> inline T cosd(const T& x)					{ return std::cos(mcs::toRadians(x)); }
template<class T> inline T sind(const T& x)					{ return std::sin(mcs::toRadians(x)); }
template<class T> inline T tand(const T& x)					{ return std::tan(mcs::toRadians(x)); }

template<class T> inline T acosr(const T& x)				{ return std::acos(x); }
template<class T> inline T asinr(const T& x)				{ return std::asin(x); }
template<class T> inline T atanr(const T& x)				{ return std::atan(x); }
template<class T> inline T cosr(const T& x)					{ return std::cos(x); }
template<class T> inline T sinr(const T& x)					{ return std::sin(x); }
template<class T> inline T tanr(const T& x)					{ return std::tan(x); }

template<class T> inline T mag(const T& x)					{ return std::abs(x); }
template<class T> inline T norm(const T& x)					{ return x*x; }
template<class T> inline T product(const T& x)				{ return x; }
template<class T> inline T sign(const T& x)					{ return (x >= 0 ? 1 : -1); }
template<class T> inline T sum(const T& x)					{ return x; }
template<class T> inline T sqr(const T& x)					{ return x*x; }

template<class T> inline bool all(const T& x)				{ return x; }
template<class T> inline bool any(const T& x)				{ return x; }
template<class T> inline bool is_finite(const T& x)			{ return isfinite(x); }
template<class T> inline bool even(const T& x)				{ return !(x % 2); }
template<class T> inline bool odd(const T& x)				{ return x % 2; }

// binary
template<class T> inline T dot(const T& x,const T& y);
template<class T> inline T quotient(const T& x,const T& y)	{ return static_cast<T>(std::floor(static_cast<double>(x)/y)); }
template<class T> inline T mod(const T& x,const T& y)		{ return x - y*quotient(x,y); }
template<class T> inline T sign(const T& x,const T& y)		{ return (y >= 0 ? std::abs(x) : -std::abs(x)); }
template<class T> inline T step(const T& x,const T& y)		{ return x < y ? 1 : 0; }

//template<class T> inline T conj(const T& x)					{ return std::conj(x); }

//template<class T> 
//inline std::complex<T> conj(const std::complex<T>& x)
//{
//	return std::conj(x);
//}

template<class T> inline typename mcs::DemoteComplex<T>::demote_t imag(const T& x)	{ return T(0); }

template<class T> 
inline typename mcs::DemoteComplex< std::complex<T> >::demote_t imag(const std::complex<T>& val)
{
	return val.imag();
}

template<class T> inline typename mcs::DemoteComplex<T>::demote_t real(const T& x)	{ return x; }

template<class T> 
inline typename mcs::DemoteComplex< std::complex<T> >::demote_t real(const std::complex<T>& val)
{
	return val.real();
}

// ternary
template<class T> 
inline T bound(const T& x,const T& min_val,const T& max_val)
{ 
	if (x <= min_val)
		return min_val;
	else if (x >= max_val)
		return max_val;
	else
		return x;
}

template<class T>
inline T where(bool cond,T trueval,T falseval)
{
	return cond ? trueval : falseval;
}

} // namespace std

#endif // MCS_MATHEXTENSIONS_H