#ifndef MCS_ONEWAYDIRECTORY_H
#define MCS_ONEWAYDIRECTORY_H

#include <map>

#include "MCS_Exceptions.h"

namespace mcs {

template<class T,class U> class OneWayDirectory
{
	public:
		typedef T	KeyType;
		typedef U	ValueType;
		
		typedef typename std::map<T,U>::const_iterator 			const_iterator;
		typedef typename std::map<T,U>::const_reverse_iterator 	const_reverse_iterator;

		OneWayDirectory() { }
		OneWayDirectory(const OneWayDirectory<T,U>&);
		virtual ~OneWayDirectory() { }
		
		OneWayDirectory<T,U>& operator=(const OneWayDirectory<T,U>&);
		
		U& operator[](const T& key);
		const U& operator[](const T& key) const;
		
		bool has(const T& key) const;
		
		void add(const T& key,const U& val);
		void insert(const T& key,const U& val);
		void replace(const T& key,const U& val);
		
		void clear(const T& key);
		void clearAll();
		
		Int32 size() const						{ return dir_.size(); }
		
		const_iterator begin() const			{ return dir_.begin(); }
		const_iterator end() const				{ return dir_.end(); }
		
		const_reverse_iterator rbegin() const	{ return dir_.rbegin(); }
		const_reverse_iterator rend() const		{ return dir_.rend(); }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:	
		std::map<T,U>	dir_;
		
		typedef typename std::map<T,U>::iterator				iterator;
		typedef typename std::map<T,U>::reverse_iterator		reverse_iterator;
};

// implementation
template<class T,class U>
inline OneWayDirectory<T,U>::OneWayDirectory(const OneWayDirectory<T,U>& source)
{
	clearAll();
	
	for (const_iterator it=source.begin();it!=source.end();++it)
		add(it->first,it->second);
}

template<class T,class U>
inline OneWayDirectory<T,U>& OneWayDirectory<T,U>::operator=(const OneWayDirectory<T,U>& source)
{
	if (&source == this) return *this;
	
	clearAll();
	
	for (const_iterator it=source.begin();it!=source.end();++it)
		add(it->first,it->second);
		
	return *this;
}

// return reference to specified value
template<class T,class U>
inline U& OneWayDirectory<T,U>::operator[](const T& key)
{
	iterator	it=dir_.find(key);
	
	if (it == dir_.end()) throw Err();
	
	return it->second;
}

template<class T,class U>
inline const U& OneWayDirectory<T,U>::operator[](const T& key) const
{
	const_iterator	it=dir_.find(key);
	
	if (it == dir_.end()) throw Err();
	
	return it->second;
}

template<class T,class U>
inline bool OneWayDirectory<T,U>::has(const T& key) const
{
	return (dir_.find(key) != dir_.end());
}

template<class T,class U>
inline void OneWayDirectory<T,U>::add(const T& key,const U& val)
{
	if (!has(key))
	{
		dir_[key] = val;
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void OneWayDirectory<T,U>::insert(const T& key,const U& val)
{
	if (has(key))
	{
		replace(key,val);
	}
	else
	{
		add(key,val);
	}
	
	return;
}

template<class T,class U>
inline void OneWayDirectory<T,U>::replace(const T& key,const U& val)
{
	if (has(key))
	{
		clear(key);	
		add(key,val);
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void OneWayDirectory<T,U>::clear(const T& key)
{
	if (has(key)) dir_.erase(key);
}

template<class T,class U>
inline void OneWayDirectory<T,U>::clearAll()
{
	dir_.clear();
}

} // namespace mcs

#endif // MCS_ONEWAYDIRECTORY_H