#ifndef MCS_SIGNATURE_H
#define MCS_SIGNATURE_H

#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_Index.h"

namespace mcs {

template<Uint8 R> class Signature
{
	public:
		typedef FixedVector<Int32,R>	IndexType;
		//typedef Index<R>				IndexType;
		
		// ctor/dtor
		Signature();	
		explicit Signature(const IndexType&);
		Signature(const Signature<R>&);
		~Signature() { }
		
		// assignment
		Signature<R>& operator=(const Signature<R>&);
		
		operator IndexType() const								{ return sig_; }
		
		// element accessors		
		const Int32& operator[](const Int32& idx) const			{ return sig_[idx]; }
		
		IndexType permute(const IndexType&) const;
		IndexType inverse_permute(const IndexType&) const;
		
		void transpose(const Uint8& rank1,const Uint8& rank2);
		
		void deserialize(std::istream& is)						{ return read(sig_,is); }
		void serialize(std::ostream& os) const					{ return write(sig_,os); }

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		IndexType	sig_;
};

template<Uint8 R>
inline Signature<R>::Signature()
{
	for (Int32 i=0;i<R;++i) sig_[i] = i;
}

template<Uint8 R>
inline Signature<R>::Signature(const IndexType& idx) :
	sig_(idx)
{
	FixedVector<bool,R>	valid(false);
	
	for (Int32 i=0;i<R;++i)
	{
		if (sig_[i] < 0 || sig_[i] > R-1) throw Err();
		
		valid[sig_[i]] = true;
	}
	
	if (any(valid == false)) throw Err();
}

template<Uint8 R>
inline Signature<R>::Signature(const Signature<R>& source) :
	sig_(source.sig_)
{ }

template<Uint8 R>
inline Signature<R>& Signature<R>::operator=(const Signature<R>& source)
{
	if (this == &source) return *this;
	
	sig_ = source.sig_;
	
	return *this;
}

// return dim permuted by sig
template<Uint8 R>
inline typename Signature<R>::IndexType Signature<R>::permute(const IndexType& source) const
{
	IndexType	dimout;
	
	for (Int32 i=0;i<R;++i) dimout[i] = source[(*this)[i]];
	
	return dimout;
}

// if dim is the result of permutation sig, return unpermuted dim
template<Uint8 R>
inline typename Signature<R>::IndexType Signature<R>::inverse_permute(const IndexType& source) const
{
	IndexType	dimout = source;
	
	for (Int32 i=0;i<R;++i)
	{
		if ((*this)[i] != i)
		{
			for (Int32 j=i+1;j<R;++j)
			{
				if ((*this)[j] == i)
				{
					std::swap(dimout[i],dimout[j]);
					break;
				}
			}
		}
	}
	
	return dimout;
}

template<Uint8 R>
inline void Signature<R>::transpose(const Uint8& rank1,const Uint8& rank2)
{
	if (rank1 > R-1 || rank2 > R-1) throw Err();
	
	std::swap(sig_[rank1],sig_[rank2]);
}

template<Uint8 R>
inline std::ostream& operator<<(std::ostream& ostr,const Signature<R>& source)
{
	ostr << "S [ ";
	
	for (Int32 i=0;i<R;++i)
		ostr << source[i] << " ";
		
	ostr << "]";

	return ostr;
}

template<Uint8 R>
inline std::istream& operator>>(std::istream& istr,Signature<R>& dest)
{
	IndexType	idx;
	
	istr >> idx;
	
	dest = Signature<R>(idx);
}

} // namespace mcs

#endif // MCS_SIGNATURE_H