#ifndef MCS_TRACE_H
#define MCS_TRACE_H

#include <iostream>
#include <string>

namespace mcs {

// should partially specialize this to enable tracing for specific classes
// 
//	e.g. template<> struct Tracing<MyDumbClass> { enum { active = true }; };
//
template<class T> struct Tracing { enum { active = true }; };

class TraceOn 	{ static bool value() { return true; } };
class TraceOff 	{ static bool value() { return false; } };

template<bool On = true> class Trace;

// do-nothing version, for tracing completely off
template<> class Trace<false>
{
	public:
		static bool isOn()										{ return false; }
		
		static void on()										{ }
		static void off()										{ }
		
		static void truncate_name(int)							{ }
		
		static void trace(const std::string& = "\n") 			{ }

		template<class T> static void trace_name(const T&) 		{ }
		template<class T> static void trace_value(const T&) 	{ }
		
		static std::ostream& get_output_stream()				{ return std::cerr; }
		static void set_output_stream(std::ostream& )			{ }
		
	private:
		Trace();
		~Trace();	
};

// lightweight tracing class
template<> class Trace<true>
{
	public:
		static bool isOn()										{ return on_; }
		
		static void on()										{ on_ = true; }
		static void off()										{ on_ = false; }
		
		static void truncate_name(int len)						{ len_ = len; }
		
		static void trace(const std::string& = "\n");

		template<class T> static void trace_name(const T&);
		template<class T> static void trace_value(const T&);
		
		static std::ostream& get_output_stream()				{ return *osptr_; }
		static void set_output_stream(std::ostream& os)			{ osptr_ = &os; }

	private:
		Trace();
		~Trace();
		
		static bool				on_;
		static int				len_;
		static std::ostream*	osptr_;
};

template<>
inline void Trace<true>::trace(const std::string& mesg)
{
	if (on_) std::cerr << mesg;
}

template<>
template<class T>
inline void Trace<true>::trace_name(const T& t)
{
	if (len_ == 0)
		trace(typeid(t).name());
	else
		trace(std::string(typeid(t).name()).substr(0,len_));
}

template<>
template<class T>
inline void Trace<true>::trace_value(const T& t)
{
	std::stringstream	sstr;
	
	sstr << t;
	
	trace(sstr.str());
}

template<class T> class TraceType
{
	public:
		TraceType(const T&,const std::string& = "");
		~TraceType();
		
	private:
		static int	tracenum_;
		
		const T&	t_;
		int			this_tracenum_;
		bool		tracing_on_;
};

template<class T> int TraceType<T>::tracenum_ = 0;

template<class T>
inline TraceType<T>::TraceType(const T& t,const std::string& mesg) :
	t_(t),
	this_tracenum_(tracenum_)
{
	tracing_on_ = Trace<>::isOn() && Tracing<T>::active;
	
	if (tracing_on_)
	{
		++tracenum_;
	
		Trace<>::trace("\nTracing #");
		Trace<>::trace_value(this_tracenum_);
		Trace<>::trace(" : ");
		Trace<>::trace_name(t_);
		Trace<>::trace(" : "+mesg+" ");
	}
}

template<class T>
inline TraceType<T>::~TraceType()
{
	if (tracing_on_)
	{
		Trace<>::trace("\nCompleted #");
		Trace<>::trace_value(this_tracenum_);
		Trace<>::trace("\n");
	}
}

} // namespace mcs

#endif // MCS_TRACE_H