#ifndef MCS_TWOWAYDIRECTORY_H
#define MCS_TWOWAYDIRECTORY_H

#include <map>

#include "MCS_Distinct.h"
#include "MCS_Exceptions.h"

namespace mcs {

template<class T,class U> class TwoWayDirectory
{
	public:
		typedef T	ForwardType;
		typedef U	ReverseType;

		TwoWayDirectory() { }
		TwoWayDirectory(const TwoWayDirectory<T,U>&);
		virtual ~TwoWayDirectory() { }
		
		TwoWayDirectory<T,U>& operator=(const TwoWayDirectory<T,U>&);
		
		//U& operator[](const T& key);
		const U& operator[](const T& key) const;
		
		//T& operator[](const U& key);
		const T& operator[](const U& key) const;
		
		bool has(const T& x) const;
		bool has(const U& y) const;
		
		void add(const T& x,const U& y);
		void add(const U& y,const T& x);
		
		void insert(const T& x,const U& y);
		void insert(const U& y,const T& x);
		
		void replace(const T& x,const U& y);
		void replace(const U& y,const T& x);
		
		void clear(const T& x);
		void clear(const U& y);
		
		void clearAll();
				
		Int32 size() const						{ return fdir_.size(); }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:	
		static const Distinct<T,U>	dummy_;
		
		std::map<T,U>	fdir_;
		std::map<U,T>	rdir_;
};

// implementation
template<class T,class U> const Distinct<T,U>	TwoWayDirectory<T,U>::dummy_;

template<class T,class U>
inline TwoWayDirectory<T,U>::TwoWayDirectory(const TwoWayDirectory<T,U>& source)
{
	clearAll();
	
	for (typename std::map<T,U>::const_iterator it=source.fdir_.begin();it!=source.fdir_.end();++it)
		add(it->first,it->second);
}

template<class T,class U>
inline TwoWayDirectory<T,U>& TwoWayDirectory<T,U>::operator=(const TwoWayDirectory<T,U>& source)
{
	if (&source == this) return *this;
	
	clearAll();
	
	for (typename std::map<T,U>::const_iterator it=source.fdir_.begin();it!=source.fdir_.end();++it)
		add(it->first,it->second);
		
	return *this;
}

// return reference to specified value
template<class T,class U>
inline const U& TwoWayDirectory<T,U>::operator[](const T& key) const
{
	typename std::map<T,U>::const_iterator	it=fdir_.find(key);
	
	if (it == fdir_.end()) throw Err();
	
	return it->second;
}

template<class T,class U>
inline const T& TwoWayDirectory<T,U>::operator[](const U& val) const
{
	typename std::map<U,T>::const_iterator	it=rdir_.find(val);
	
	if (it == rdir_.end()) throw Err();
	
	return it->second;
}

template<class T,class U>
inline bool TwoWayDirectory<T,U>::has(const T& key) const
{
	return (fdir_.find(key) != fdir_.end());
}

template<class T,class U>
inline bool TwoWayDirectory<T,U>::has(const U& val) const
{
	return (rdir_.find(val) != rdir_.end());
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::add(const T& key,const U& val)
{
	if (!has(key) && !has(val))
	{
		fdir_[key] = val;
		rdir_[val] = key;
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::add(const U& val,const T& key)
{
	add(key,val);
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::insert(const T& key,const U& val)
{
	if (has(key) && has(val))
	{
		replace(key,val);
	}
	else if (!has(key) && !has(val))
	{
		add(key,val);
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::insert(const U& val,const T& key)
{
	insert(key,val);
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::replace(const T& key,const U& val)
{
	if (has(key))
	{
		clear(key);	
		add(key,val);
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::replace(const U& val,const T& key)
{
	if (has(key))
	{
		clear(key);	
		add(key,val);
	}
	else throw Err();
	
	return;
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::clear(const T& key)
{
	typename std::map<T,U>::const_iterator	it = fdir_.find(key);
		
	if (it != fdir_.end())
	{
		fdir_.erase(it->first);
		rdir_.erase(it->second);
	}
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::clear(const U& val)
{
	typename std::map<U,T>::const_iterator	it = rdir_.find(val);
		
	if (it != rdir_.end())
	{
		rdir_.erase(it->first);
		fdir_.erase(it->second);
	}
}

template<class T,class U>
inline void TwoWayDirectory<T,U>::clearAll()
{
	fdir_.clear();
	rdir_.clear();
}

} // namespace mcs

#endif // MCS_TWOWAYDIRECTORY_H