#ifndef MCS_EARTHMAP_H
#define MCS_EARTHMAP_H

#include "MCS_Exceptions.h"
#include "MCS_Field.h"
#include "MCS_Grid.h"
#include "MCS_IndexRange.h"

namespace mcs {

namespace geo {

//class Err : public Exception { };
__DECLARE_EXCEPTION_CLASS__(GeographicErr,Exception)

template<class T = Float32> 
class CenteredEarthGrid : public UniformGrid<T,2,SphericalBoundary<2> >
{
	public:
		CenteredEarthGrid(const IndexRange<2>& rng = IndexRange<2>());
};

template<class T = Float32> 
class CornerEarthGrid : public UniformGrid<T,2,SphericalBoundary<2> >
{
	public:
		CornerEarthGrid(const IndexRange<2>& rng = IndexRange<2>());
};

template<class T = Float32> 
class EdgeEarthGrid : public UniformGrid<T,2,SphericalBoundary<2> >
{
	public:
		EdgeEarthGrid(const IndexRange<2>& rng = IndexRange<2>());
};

template<class T = Float32> 
class CCM3Grid : public UniformGrid<T,2,SphericalBoundary<2> >
{
	public:
		CCM3Grid(const IndexRange<2>& rng = IndexRange<2>());
};

template<class T = Float32> 
class NCEPGrid : public UniformGrid<T,2,SphericalBoundary<2> >
{
	public:
		NCEPGrid(const IndexRange<2>& rng = IndexRange<2>());
};

// map with full Earth coverage
template<class T,class M = Float32,class G = CenteredEarthGrid<M> > 
class EarthMap : public Field<T,M,G>
{
	public:
		typedef Field<T,M,G>						FieldType;
		typedef typename FieldType::BoundaryType	BoundaryType;
		typedef typename FieldType::CoordinateType	CoordinateType;
		typedef typename FieldType::GridType		GridType;
		typedef typename FieldType::OrdinateType	OrdinateType;
		typedef typename FieldType::ValueType		ValueType;
		
		EarthMap();
		EarthMap(const Matrix<T>&);
		EarthMap(const IndexRange<2>&,const T& = T());
		EarthMap(const GridType&,const T& = T());
		EarthMap(const GridType&,const Matrix<T>&);
		EarthMap(const EarthMap<T,M,G>&);
		virtual ~EarthMap() { }
		
		EarthMap& operator=(const T&);
		EarthMap& operator=(const Matrix<T>&);
		EarthMap& operator=(const EarthMap<T,M,G>&);
		
		CoordinateType coordinate(const Index<1>& xidx,const Index<1>& yidx) const	{ return coordinate(Index<2>(xidx,yidx)); }
		CoordinateType coordinate(const Index<2>& idx) const						{ return CoordinateType((*this).xy(idx)); }	
		CoordinateType resolution() const											{ return CoordinateType(180.0/(*this).dim(0),360.0/(*this).dim(1)); }

		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,GeographicErr)
};

typedef EarthMap<Binary,Float32,CenteredEarthGrid<Float32> >	CenteredSurfaceMask;
typedef EarthMap<Int8,Float32,CenteredEarthGrid<Float32> >		CenteredSurfaceMapI8;
typedef EarthMap<Uint8,Float32,CenteredEarthGrid<Float32> >		CenteredSurfaceMapUI8;
typedef EarthMap<Int16,Float32,CenteredEarthGrid<Float32> >		CenteredSurfaceMapI16;
typedef EarthMap<Uint16,Float32,CenteredEarthGrid<Float32> >	CenteredSurfaceMapUI16;
typedef EarthMap<Int32,Float32,CenteredEarthGrid<Float32> >		CenteredSurfaceMapI32;
typedef EarthMap<Uint32,Float32,CenteredEarthGrid<Float32> >	CenteredSurfaceMapUI32;
typedef EarthMap<Float32,Float32,CenteredEarthGrid<Float32> >	CenteredSurfaceMapF32;
typedef EarthMap<Float64,Float64,CenteredEarthGrid<Float32> >	CenteredSurfaceMapF64;

typedef EarthMap<Binary,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMask;
typedef EarthMap<Int8,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapI8;
typedef EarthMap<Uint8,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapUI8;
typedef EarthMap<Int16,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapI16;
typedef EarthMap<Uint16,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapUI16;
typedef EarthMap<Int32,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapI32;
typedef EarthMap<Uint32,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapUI32;
typedef EarthMap<Float32,Float32,CornerEarthGrid<Float32> >		CornerSurfaceMapF32;
typedef EarthMap<Float64,Float64,CornerEarthGrid<Float32> >		CornerSurfaceMapF64;

typedef EarthMap<Binary,Float32,EdgeEarthGrid<Float32> >		EdgeSurfaceMask;
typedef EarthMap<Int8,Float32,EdgeEarthGrid<Float32> >			EdgeSurfaceMapI8;
typedef EarthMap<Uint8,Float32,EdgeEarthGrid<Float32> >			EdgeSurfaceMapUI8;
typedef EarthMap<Int16,Float32,EdgeEarthGrid<Float32> >			EdgeSurfaceMapI16;
typedef EarthMap<Uint16,Float32,EdgeEarthGrid<Float32> >		EdgeSurfaceMapUI16;
typedef EarthMap<Int32,Float32,EdgeEarthGrid<Float32> >			EdgeSurfaceMapI32;
typedef EarthMap<Uint32,Float32,EdgeEarthGrid<Float32> >		EdgeSurfaceMapUI32;
typedef EarthMap<Float32,Float32,EdgeEarthGrid<Float32> >		EdgeSurfaceMapF32;
typedef EarthMap<Float64,Float64,EdgeEarthGrid<Float32> >		EdgeSurfaceMapF64;

} // namespace geo

} // namespace mcs

// implementation
#include "MCS_EarthMap.cpp"

#endif // MCS_EARTHMAP_H
