#include <cmath>

#include "MCS_Constants.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_GeographicCoordinates.h"
#include "MCS_MathExtensions.h"

namespace mcs {

namespace geo {

// generalized geographic coordinate
template<class T> const EnsureReal<T> GeographicCoordinate<T>::dummy_;

template<class T> 
inline GeographicCoordinate<T>::GeographicCoordinate(const T& lat,const T& lon) :
	FixedVector<T,2>(lat,std::mod(lon,T(360)))
{ }	

template<class T> 
inline GeographicCoordinate<T>::GeographicCoordinate(const FixedVector<T,2>& coord) :
	FixedVector<T,2>(coord[0],std::mod(coord[1],T(360)))
{ }	

template<class T> 
inline GeographicCoordinate<T>::GeographicCoordinate(const GeographicCoordinate<T>& source) :
	FixedVector<T,2>(source.latitude(),source.longitude())
{ }	

// Earth centered inertial reference frame
template<class T> const EnsureReal<T> Inertial<T>::dummy_;

template<class T> 
inline Geocentric<T> Inertial<T>::toGeocentric(const Date& epoch) const
{
	static const T	rE = constants::physical::_r_Ee_*1.0e-3;
	
	const T	Rp = std::sqrt(std::sqr(x())+std::sqr(y())+std::sqr(z())),
			thetap = std::atan2(y(),x())-siderealTime(epoch);
			
	return Geocentric<T>(GeographicCoordinate<T>(toDegrees(std::asin(z()/Rp)),toDegrees(thetap)),Rp-rE);
}

template<class T> 
inline Geodetic<T> Inertial<T>::toGeodetic(const Date& epoch) const
{
	// Reference:  The 1992 Astronomical Almanac, page K12. 
	static const T	rE = constants::physical::_r_Ee_*1.0e-3,
					LLA_EPS = 1.0e-10;
		
	const T	theta = std::atan2(y(),x())-siderealTime(epoch),
			Rp = std::hypot(x(),y()),
			f = constants::physical::_oblateness_,							
			e2 = f*(2.0-f);
					
	T	phi_old,phi,C;
	
	phi = std::atan2(z(),Rp);
	
	do	
	{
		phi_old = phi;
		
		const T	sinphiold = std::sin(phi_old);
		
		C = 1.0/std::sqrt(1.0-e2*std::sqr(sinphiold));
		phi = std::atan2(z() + rE*C*e2*sinphiold,Rp);
	} 
	while (std::abs(phi - phi_old) > LLA_EPS);
	
	return Geodetic<T>(GeographicCoordinate<T>(toDegrees(phi),toDegrees(theta)),Rp/std::cos(phi) - rE*C);
}

template<class T> 
inline FixedVector<T,3> Inertial<T>::toCartesian(const Date& epoch) const
{
	return toGeocentric(epoch).toCartesian();
}

// geocentric earth location
//	- latitude (deg), longitude (deg), height (km)
//	- longitude from 0 to 360 E
template<class T> const EnsureReal<T> Geocentric<T>::dummy_;

// height above spherical Earth with radius equal to equatorial radius
template<class T> 
inline T Geocentric<T>::r() const
{
	static const T	rE = constants::physical::_r_Ee_*1.0e-3;
	
	return (h()+rE);
}

template<class T> 
inline Inertial<T> Geocentric<T>::toInertial(const Date& epoch) const
{
	const T		Rp = r()*cosd(latitude()),
				thetap = toRadians(longitude())+siderealTime(epoch);
			
	return Inertial<T>(Rp*std::cos(thetap),Rp*std::sin(thetap),r()*std::sind(latitude()));
}

template<class T> 
inline FixedVector<T,3> Geocentric<T>::toCartesian() const
{
	const T		Rp = r()*std::cosd(latitude());
			
	return FixedVector<T,3>(Rp*std::cosd(longitude()),Rp*std::sind(longitude()),r()*std::sind(latitude()));
}

template<class T> 
inline Geocentric<T> Geocentric<T>::toGeocentric() const
{ 
	return (*this); 
}

template<class T> 
inline Geodetic<T> Geocentric<T>::toGeodetic() const
{
	static const Date	j2000(2000,1,1);
	
	// geocentric -> inertial -> geodetic
	return toInertial(j2000).toGeodetic(j2000);
}

template<class T> 
inline std::ostream& operator<<(std::ostream& os,const Geocentric<T>& source)
{
	const T		tlon = toEWLongitude(source.longitude()),
				tlat = source.latitude();
	const char	lon_suffix = (tlon < 0) ? 'W' : 'E',
				lat_suffix = (tlat < 0) ? 'S' : 'N';
	
	os << "alt: " << source.h() << " " 
	   << "lon: " << std::abs(tlon) << lon_suffix << " " 
	   << "lat: " << std::abs(tlat) << lat_suffix << " ";
	   
	return os;
}

// geodetic earth location
//	- latitude (deg), longitude (deg), height (km)
//	- longitude from 0 to 360 E
template<class T> const EnsureReal<T> Geodetic<T>::dummy_;

template<class T> 
inline T Geodetic<T>::r() const
{
	static const T	rE = constants::physical::_r_Ee_*1.0e-3;
	
	const T		rE2 = std::sqr(rE),
				f = constants::physical::_oblateness_,
				q = f*(f-2.0),
				qt = q+2.0,
				qs = 2.0*rE*h()*std::sqrt(1.0+q*std::sqr(std::sind(latitude()))),
				qc = q*std::cosd(2.0*latitude()),
				qh = std::sqr(h())+q*rE2+qs;
				
	return std::sqrt(qh*(qc-qt) + 2.0*rE2*(qc-1.0))/(qc-qt);		
}
					
template<class T> 
inline Inertial<T> Geodetic<T>::toInertial(const Date& epoch) const
{
	static const T	rE = constants::physical::_r_Ee_*1.0e-3;
	
	const T		a = rE,
				f = constants::physical::_oblateness_,
				C = 1.0/std::sqrt(1.0+f*(f-2.0)*std::sqr(std::sind(latitude()))),
				S = std::sqr(1.0-f)*C,
				Rp = (a*C + h())*std::cosd(latitude()),
				thetap = toRadians(longitude())+siderealTime(epoch);
					
	return Inertial<T>(Rp*std::cos(thetap),Rp*std::sin(thetap),(a*S + h())*std::sind(latitude()));
}

template<class T> 
inline FixedVector<T,3> Geodetic<T>::toCartesian() const
{
	static const T	rE = constants::physical::_r_Ee_*1.0e-3;
	
	const Float64	a = rE,
					f = constants::physical::_oblateness_,
					C = 1.0/std::sqrt(1.0+f*(f-2.0)*std::sqr(std::sind(latitude()))),
					S = std::sqr(1.0-f)*C,
					Rp = (a*C + h())*std::cosd(latitude());
					
	return FixedVector<T,3>(Rp*std::cosd(longitude()),Rp*std::sind(longitude()),(a*S + h())*std::sind(latitude()));
}

template<class T> 
inline Geocentric<T> Geodetic<T>::toGeocentric() const
{
	static const Date	j2000(2000,1,1);
	
	// geodetic -> inertial -> geocentric
	return toInertial(j2000).toGeocentric(j2000);
}

template<class T> 
inline Geodetic<T> Geodetic<T>::toGeodetic() const
{ return (*this); }

template<class T> 
inline std::ostream& operator<<(std::ostream& os,const Geodetic<T>& source)
{
	const T		tlon = toEWLongitude(source.longitude()),
				tlat = source.latitude();
	const char	lon_suffix = (tlon < 0) ? 'W' : 'E',
				lat_suffix = (tlat < 0) ? 'S' : 'N';
	
	os << "alt: " << source.h() << " " 
	   << "lon: " << std::abs(tlon) << lon_suffix << " " 
	   << "lat: " << std::abs(tlat) << lat_suffix << " ";
	   
	return os;
}

// topocentric earth location
//	- range (km),elevation (deg),azimuth (deg)
template<class T> const EnsureReal<T> Topocentric<T>::dummy_;

template<class T> inline std::ostream& operator<<(std::ostream& os,const Topocentric<T>& source)
{
	os << "rng: " << source.range() << " " 
	   << "elv: " << source.elevation() << " " 
	   << "azm: " << source.azimuth();
	
	return os;
}

// convert between 0..360 and -180..+180 longitudes
template<class T> 
inline T toEWLongitude(const T& theta)
{
	return static_cast<T>((theta > 180.0) ? theta - 360.0 : theta);
}

// convert between -180..+180 and 0..360 longitudes
template<class T> 
inline T fromEWLongitude(const T& theta)
{
	return static_cast<T>((theta < 0.0) ? theta + 360.0 : theta);
}

// convert UTC time to local time
template<class T> 
Time localTime(const Time& utc,const T& lon)
{
	if (!isfinite(lon) || lon < 0 || lon > 360) throw Time::Err();
	
	return Time(std::mod(utc.ticks()/Time::secondsInDay()+lon/360.0,1.0)*Time::secondsInDay());
}

template<class T> 
Time localTime(const Time& utc,const GeographicCoordinate<T>& coord)
{
	if (!isfinite(coord.longitude())) throw Time::Err();
	
	return Time(std::mod(utc.ticks()/Time::secondsInDay()+coord.longitude()/360.0,1.0)*Time::secondsInDay());
}

// angular separation between two positions
template<class T> 
inline T cosTheta(const Geodetic<T>& x,const Geodetic<T>& y)
{
	const FixedVector<T,3>	xc(x.toCartesian()),yc(y.toCartesian());
	
	return dot(xc,yc)/std::sqrt(dot(xc,xc)*dot(yc,yc));
}

// elevation and azimuth angles for an observer located at obs_pos	
template<class T> 
inline Topocentric<T> look(const Date& epoch,const Geodetic<T>& observer,const Geodetic<T>& target)
{
	const Inertial<T>	ob = observer.toInertial(epoch),
						tg = target.toInertial(epoch),
						dr = tg-ob;
	const T				theta = toRadians(observer.longitude())+siderealTime(epoch),
						phi = toRadians(observer.latitude()),
						sintheta = std::sin(theta),
						costheta = std::cos(theta),
						sinphi = std::sin(phi),
						cosphi = std::cos(phi),
						rs = sinphi*costheta*dr.x()+sinphi*sintheta*dr.y()-cosphi*dr.z(),
						re = -sintheta*dr.x()+costheta*dr.y(),
						rz = cosphi*costheta*dr.x()+cosphi*sintheta*dr.y()+sinphi*dr.z(),
						rng = std::sqrt(sqr(rs)+sqr(re)+sqr(rz)),
						elev = std::asin(rz/rng);
						
	T					azim = std::atan2(-re,rs);
	
	azim = (rs > 0) ? azim+_pi_ : azim+_two_pi_;
							
	return Topocentric<T>(rng,toDegrees(elev),toDegrees(azim));
}

} // namespace geo

} // namespace mcs
