#ifndef MCS_GEOGRAPHIC_COORDINATES_H
#define MCS_GEOGRAPHIC_COORDINATES_H

#include <iostream>

#include "MCS_Constants.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"

namespace mcs {

namespace geo {

template<class T> class Inertial;
template<class T> class Geocentric;
template<class T> class Geodetic;
template<class T> class Topocentric;

// generalized geographic coordinate
template<class T> 
class GeographicCoordinate : public FixedVector<T,2>
{
	public:
		GeographicCoordinate(const T& lat = _NaN_,const T& lon = _NaN_);
		GeographicCoordinate(const FixedVector<T,2>& coord);
		GeographicCoordinate(const GeographicCoordinate<T>& coord);
		~GeographicCoordinate() { }
		
		const T& latitude() const					{ return (*this)[0]; }
		const T& longitude() const					{ return (*this)[1]; }
		
	private:
		static const EnsureReal<T> dummy_;
};

// Earth centered inertial reference frame
template<class T> 
class Inertial : public FixedVector<T,3>
{
	public:
		Inertial() : FixedVector<T,3>() { }
		Inertial(const T& x,const T& y,const T& z) : FixedVector<T,3>(x,y,z) { }
		Inertial(const FixedVector<T,3>& source) : FixedVector<T,3>(source) { }
		Inertial(const Inertial<T>& source) : FixedVector<T,3>(source) { }
		
		//assignors
		void x(const T& val)	{ (*this)[0] = val; }
		void y(const T& val)	{ (*this)[1] = val; }
		void z(const T& val)	{ (*this)[2] = val; }
		
		// accessors
		const T& x() const		{ return (*this)[0]; }
		const T& y() const		{ return (*this)[1]; }
		const T& z() const		{ return (*this)[2]; }

		inline Geocentric<T> toGeocentric(const Date&) const;
		inline Geodetic<T> toGeodetic(const Date&) const;
		
		inline FixedVector<T,3> toCartesian(const Date&) const;
		
	private:
		static const EnsureReal<T> dummy_;
};

// geocentric earth location
//	- latitude (deg), longitude (deg), height (km)
//	- longitude from 0 to 360 E
template<class T> 
class Geocentric
{
	public:
		Geocentric() : coord_(_NaN_,_NaN_),h_(_NaN_) { }
		Geocentric(const T& lat,const T& lon,const T& h) : coord_(lat,lon),h_(h) { }
		Geocentric(const GeographicCoordinate<T>& coord,const T& h) : coord_(coord),h_(h) { }
		Geocentric(const Geocentric<T>& source) : coord_(source.coord_),h_(source.h_) { }
		~Geocentric() { }
		
		const T& h() const							{ return h_; }
		const T& latitude() const					{ return coord_.latitude(); }
		const T& longitude() const					{ return coord_.longitude(); }
		
		GeographicCoordinate<T> coordinate() const	{ return coord_; }

		T r() const;
		
		Inertial<T> toInertial(const Date&) const;
		FixedVector<T,3> toCartesian() const;
		
		Geocentric<T> toGeocentric() const;
		Geodetic<T> toGeodetic() const;
		
	private:
		static const EnsureReal<T>	dummy_;

		GeographicCoordinate<T>		coord_;
		T							h_;
};

template<class T> std::ostream& operator<<(std::ostream& os,const Geocentric<T>& source);

// geodetic earth location
//	- latitude (deg), longitude (deg), height (km)
//	- longitude from 0 to 360 E
template<class T> 
class Geodetic
{
	public:
		Geodetic() : coord_(_NaN_,_NaN_),h_(_NaN_) { }
		Geodetic(const T& lat,const T& lon,const T& h) : coord_(lat,lon),h_(h) { }
		Geodetic(const GeographicCoordinate<T>& coord,const T& h) : coord_(coord),h_(h) { }
		Geodetic(const Geodetic<T>& source) : coord_(source.coord_),h_(source.h_) { }
		~Geodetic() { }
		
		const T& h() const							{ return h_; }
		const T& height() const						{ return h_; }
		const T& latitude() const					{ return coord_.latitude(); }
		const T& longitude() const					{ return coord_.longitude(); }
		
		GeographicCoordinate<T> coordinate() const	{ return coord_; }

		T r() const;
		
		Inertial<T> toInertial(const Date&) const;
		FixedVector<T,3> toCartesian() const;
		
		Geocentric<T> toGeocentric() const;
		Geodetic<T> toGeodetic() const;
		
	private:
		static const EnsureReal<T>	dummy_;

		GeographicCoordinate<T>		coord_;
		T							h_;
};

template<class T> std::ostream& operator<<(std::ostream& os,const Geodetic<T>& source);

// topocentric earth location
//	- range (km),elevation (deg),azimuth (deg)
template<class T> 
class Topocentric
{
	public:
		Topocentric(const T& rng,const T& elev,const T& azim) : vec_(rng,elev,mod(azim,T(360))) { }
		Topocentric(const Topocentric& source) : vec_(source.vec_) { }
		~Topocentric() { }
		
		// accessors
		T range() const					{ return vec_[0]; }
		T elevation() const				{ return vec_[1]; }
		T azimuth() const				{ return vec_[2]; }
		
		inline Inertial<T> toInertial(const Date&) const;
		
	private:
		static const EnsureReal<T>	dummy_;
		
		Topocentric();
		
		FixedVector<T,3>	vec_;
};

template<class T> inline std::ostream& operator<<(std::ostream& os,const Topocentric<T>& source);

// convert between -180..+180 and 0..360 longitudes
template<class T> T toEWLongitude(const T& theta);
template<class T> T fromEWLongitude(const T& theta);

// convert UTC time to local time
template<class T> Time localTime(const Time& utc,const T& lon);
template<class T> Time localTime(const Time& utc,const GeographicCoordinate<T>& coord);

// angular separation between two positions
template<class T> T cosTheta(const Geodetic<T>& x,const Geodetic<T>& y);

// elevation and azimuth angles for an observer located at obs_pos	
template<class T> Topocentric<T> look(const Date& epoch,const Geodetic<T>& observer,const Geodetic<T>& target);

} // namespace geo

} // namespace mcs

// implementation
#include "MCS_GeographicCoordinates.cpp"

#endif // MCS_GEOGRAPHIC_COORDINATES_H