#ifndef MCS_GEOGRAPHY_H
#define MCS_GEOGRAPHY_H

#include "MCS_EarthMap.h"
#include "MCS_Exceptions.h"
#include "MCS_GeographicCoordinates.h"

namespace mcs {

// constants
static const Int32	LAND = 0,
					OCEAN = 1;
					
namespace geo {

__DECLARE_EXCEPTION_CLASS__(GeographyErr,Exception)

Float32 						getEllipsoid(const GeographicCoordinate<Float32>& coord);

Float32							getGeoid(const GeographicCoordinate<Float32>& coord);
CenteredSurfaceMapF32			getGeoid(const CenteredSurfaceMapF32::GridType& grid);
CenteredSurfaceMapF32&			getGeoid(CenteredSurfaceMapF32& map);

Float32							getInsolation(const GeographicCoordinate<Float32>& coord,const Date& today);

bool 							isLand(const GeographicCoordinate<Float32>& coord,const Float32& thresh = 0.1);
bool							isOcean(const GeographicCoordinate<Float32>& coord,const Float32& thresh = 0.1);

CenteredSurfaceMapF32			getLand(const CenteredSurfaceMapF32::GridType& grid,const Float32& thresh = 0.1);
CenteredSurfaceMapF32&			getLand(CenteredSurfaceMapF32& map,const Float32& thresh = 0.1);

CenteredSurfaceMapF32			getOcean(const CenteredSurfaceMapF32::GridType& grid,const Float32& thresh = 0.1);
CenteredSurfaceMapF32&			getOcean(CenteredSurfaceMapF32& map,const Float32& thresh = 0.1);

Float32							getLandFraction(const GeographicCoordinate<Float32>& coord);
CenteredSurfaceMapF32			getLandFraction(const CenteredSurfaceMapF32::GridType& grid);
CenteredSurfaceMapF32&			getLandFraction(CenteredSurfaceMapF32& map);

Float32							getOceanFraction(const GeographicCoordinate<Float32>& coord);
CenteredSurfaceMapF32			getOceanFraction(const CenteredSurfaceMapF32::GridType& grid);
CenteredSurfaceMapF32&			getOceanFraction(CenteredSurfaceMapF32& map);

CenteredSurfaceMapF32&			landMask(CenteredSurfaceMapF32& map,const Float32& thresh = 0.1);
CenteredSurfaceMapF32&			oceanMask(CenteredSurfaceMapF32& map,const Float32& thresh = 0.1);

Int8							getLandCover(const GeographicCoordinate<Float32>& coord);
CenteredSurfaceMapI8			getLandCover(const CenteredSurfaceMapI8::GridType& grid);
CenteredSurfaceMapI8&			getLandCover(CenteredSurfaceMapI8& map);

GeographicCoordinate<Float32> 	getSun(const Date& today);

Float32							getTopography(const GeographicCoordinate<Float32>& coord);
CenteredSurfaceMapF32			getTopography(const CenteredSurfaceMapF32::GridType& grid);
CenteredSurfaceMapF32&			getTopography(CenteredSurfaceMapF32& map);

template<class T> FixedVector<T,3> earthNormal(const GeographicCoordinate<T>& coord);

template<class T> T earthIncidenceAngle(const Geodetic<T>& loc,const T& nadir_angle);
template<class T> T earthIncidenceAngle(const GeographicCoordinate<T>& coord,const T& height,const T& nadir_angle);
template<class T> T nadirAngle(const GeographicCoordinate<T>& coord,const T& height,const T& incidence_angle);

// return normal vector at Earth surface
template<class T> 
inline FixedVector<T,3> earthNormal(const GeographicCoordinate<T>& coord)
{ 
	return unit(Geodetic<T>(coord,0).toCartesian()); 
}

template<class T> 
inline T earthIncidenceAngle(const GeographicCoordinate<T>& loc,const T& height,const T& nadir_angle)
{
	// calculate earth incidence angle
	//	- approximation from SSM/I Antenna Temperature Tape User's Manual, Rev. 2, pg. 17
	const T	earthr = getEllipsoid(GeographicCoordinate<Float32>(loc.latitude(),loc.longitude()))/1000;
	
	return std::abs(std::asind(std::sind(nadir_angle)*(earthr+height)/earthr));
}

template<class T> 
inline T earthIncidenceAngle(const Geodetic<T>& pos,const T& nadir_angle)
{		
	return earthIncidenceAngle(pos.coordinate(),pos.height(),nadir_angle);
}

template<class T> 
inline T nadirAngle(const GeographicCoordinate<T>& loc,const T& height,const T& incidence_angle)
{
	// calculate earth incidence angle
	//	- approximation from SSM/I Antenna Temperature Tape User's Manual, Rev. 2, pg. 17
	const T	earthr = getEllipsoid(GeographicCoordinate<Float32>(loc.latitude(),loc.longitude()))/1000;
	
	return std::abs(std::asind(std::sind(incidence_angle)*earthr/(earthr+height)));
}

template<class T> 
inline T nadirAngle(const Geodetic<T>& pos,const T& incidence_angle)
{
	return nadirAngle(pos.coordinate(),pos.height(),incidence_angle);
}

// utility routines
CenteredSurfaceMapF32& getLoResGeoidData();
CenteredSurfaceMapF32& getMedResGeoidData();

CenteredSurfaceMask& getLoResIceMaskData();
CenteredSurfaceMask& getMedResIceMaskData();

CenteredSurfaceMapI8& getLoResLandCoverData();
CenteredSurfaceMapI8& getMedResLandCoverData();
CenteredSurfaceMapI8& getHiResLandCoverData();
CenteredSurfaceMapI8& getHi2ResLandCoverData();
CenteredSurfaceMapI8& getHi3ResLandCoverData();
CenteredSurfaceMapI8& getHi4ResLandCoverData();

CenteredSurfaceMapF32& getLoResLandFractionData();
CenteredSurfaceMapF32& getMedResLandFractionData();
CenteredSurfaceMapF32& getHiResLandFractionData();

CenteredSurfaceMapF32& getLoResTopographyData();
CenteredSurfaceMapF32& getMedResTopographyData();
CenteredSurfaceMapF32& getHiResTopographyData();
CenteredSurfaceMapF32& getHi2ResTopographyData();

} // namespace geo

} // namespace mcs

#endif // MCS_GEOGRAPHY_H
