#include "MCS_FileLocations.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

// returns true if pixel has land fraction exceeding threshold
bool isLand(const GeographicCoordinate<Float32>& coord,const Float32& thresh)
{
	return (getLandFraction(coord) > thresh);
}

bool isOcean(const GeographicCoordinate<Float32>& coord,const Float32& thresh)
{
	return !isLand(coord,1-thresh);
}

CenteredSurfaceMapF32& getLand(CenteredSurfaceMapF32& map,const Float32& thresh)
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = isLand(map.coordinate(idx),thresh) ? 0 : 1;
				
	return map;
}

CenteredSurfaceMapF32& getOcean(CenteredSurfaceMapF32& map,const Float32& thresh)
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = isOcean(map.coordinate(idx),thresh) ? 1 : 0;
				
	return map;
}

Float32 getLandFraction(const GeographicCoordinate<Float32>& loc)
{
	const CenteredSurfaceMapF32&	data = getMedResLandFractionData();
	
	return data.z(loc);
}

CenteredSurfaceMapF32 getLandFraction(const CenteredSurfaceMapF32::GridType& grid)
{
	CenteredSurfaceMapF32	map(grid);
	
	getLandFraction(map);

	return map;
}

CenteredSurfaceMapF32& getLandFraction(CenteredSurfaceMapF32& map)
{
	CenteredSurfaceMapF32*	dataptr;
	
	const Float32	res = statistics::normal::min(map.resolution());
	
	if (res <= (180.0/2700))
		dataptr = &getHiResLandFractionData();
	else if (res <= (180.0/675))
		dataptr = &getMedResLandFractionData();
	else
		dataptr = &getLoResLandFractionData();
		
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = dataptr->z(map.coordinate(idx));
						
	return map;
}

Float32 getOceanFraction(const GeographicCoordinate<Float32>& loc)
{
	const CenteredSurfaceMapF32&	data = getMedResLandFractionData();
	
	return 1-data.z(loc);
}

CenteredSurfaceMapF32 getOceanFraction(const CenteredSurfaceMapF32::GridType& grid)
{
	const CenteredSurfaceMapF32*	dataptr = &getHiResLandFractionData();
	CenteredSurfaceMapF32			map(grid);
	
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = 1-dataptr->z(map.coordinate(idx));
		
	return map;
}

CenteredSurfaceMapF32& getOceanFraction(CenteredSurfaceMapF32& map)
{
	CenteredSurfaceMapF32*	dataptr;
	
	const Float32	res = statistics::normal::min(map.resolution());
	
	if (res <= (180.0/2700))
		dataptr = &getHiResLandFractionData();
	else if (res <= (180.0/675))
		dataptr = &getMedResLandFractionData();
	else
		dataptr = &getLoResLandFractionData();
		
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = 1-dataptr->z(map.coordinate(idx));
						
	return map;
}

// multiply map by land mask to remove land areas
CenteredSurfaceMapF32& landMask(CenteredSurfaceMapF32& map,const Float32& thresh)
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) *= (isLand(map.coordinate(idx),thresh) ? _NaN_ : 1);
				
	return map;
}

// multiply map by ocean mask to remove ocean areas
CenteredSurfaceMapF32& oceanMask(CenteredSurfaceMapF32& map,const Float32& thresh)
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) *= (isOcean(map.coordinate(idx),thresh) ? _NaN_ : 1);
				
	return map;
}

} // namespace geo

} // namespace mcs