#ifndef MCS_MATLAB_ENGINE_H
#define MCS_MATLAB_ENGINE_H

#include <cstring>
#include <map>
#include <string>
#include <vector>

#include "MCS_PlatformDependent.h"

#include "MCS_Ensure.h"
#include "MCS_Matrix.h"
#include "MCS_Tensor.h"
#include "MCS_Vector.h"

#ifndef DISABLE_MATLAB_ENGINE
	#include "engine.h"
#else
	typedef void* mxArray;
	typedef void* Engine;
#endif

namespace mcs {

class MATLAB_Engine
{
	public:
		MATLAB_Engine(const std::string& str = "\0");
		~MATLAB_Engine();
		
		Engine* engine() const	{ return engine_ptr_; }
		
		Matrix<Float64> get(const std::string&) const;
		
		template<class T> void putValue(const std::string& name,const T& source)			{ put(name,source); }
		template<class T> void putVector(const std::string& name,const Vector<T>& source)	{ put(name,source); }
		template<class T> void putMatrix(const std::string& name,const Matrix<T>& source)	{ put(name,source); }
		
		template<class T> void put(const std::string&,const T&);
		template<class T> void put(const std::string&,const std::vector<T>&);
		
		template<Uint8 R> void put(const std::string&,const Index<R>&);
		template<class T,Uint8 R> void put(const std::string&,const FixedVector<T,R>&);
		
		template<class T> void put(const std::string&,const Vector<T>&);
		template<class T> void put(const std::string&,const Matrix<T>&);
		template<class T,Uint8 R,class Imp> void put(const std::string&,const Tensor<T,R,Imp>&);

		std::string buffer() const;
		std::string className(const std::string&) const;
		void evaluate(const std::string&);
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		typedef std::map<std::string,mxArray*>::const_iterator	MI;
		typedef std::map<std::string,mxArray*>::const_iterator	CMI;
		
		MATLAB_Engine(const MATLAB_Engine&);
		MATLAB_Engine& operator=(const MATLAB_Engine&);
		
		static const int	BUFFER_SIZE = 1024;
		
		static Engine*	engine_ptr_;
		static Int32	engine_ref_count_;
		
		mutable std::map<std::string,mxArray*>	array_list_;
		char									output_buffer_[BUFFER_SIZE];
};

inline MATLAB_Engine::MATLAB_Engine(const std::string& str)
{
#ifndef DISABLE_MATLAB_ENGINE
	if (engine_ref_count_ == 0)
	{
		if (!(engine_ptr_ = engOpen(str.c_str()))) throw Err();
	
		engOutputBuffer(engine_ptr_,output_buffer_,BUFFER_SIZE);
	}
	
	++engine_ref_count_;
#endif
}

inline MATLAB_Engine::~MATLAB_Engine()
{
#ifndef DISABLE_MATLAB_ENGINE
	for (CMI it=array_list_.begin();it!=array_list_.end();++it)
		mxDestroyArray(it->second);
		
	array_list_.clear();
	
	--engine_ref_count_;
		
	if (engine_ref_count_ == 0)
		engClose(engine_ptr_);
#endif
}

inline Matrix<Float64> MATLAB_Engine::get(const std::string& name) const
{
#ifndef DISABLE_MATLAB_ENGINE
	mxArray*	array_ptr = engGetVariable(engine_ptr_,name.c_str());
	Int32		xlen = mxGetM(array_ptr),
				ylen = mxGetN(array_ptr);
	
	if (xlen != 0 && ylen != 0) array_list_[name] = array_ptr;
	
	const IndexRange<2>	rng(Index<2>(1,1),Index<2>(ylen,xlen));

	return Matrix<Float64>(rng,mxGetPr(array_ptr));
#else
	return Matrix<Float64>();
#endif
}

template<class T> 
inline void MATLAB_Engine::put(const std::string& name,const T& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);
		
	array_ptr = mxCreateDoubleMatrix(1,1,mxREAL);
	array_list_[name] = array_ptr;
	
	Float64	tval(source);
	
	//mxSetName(array_ptr,name.c_str());
	
	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(&tval),sizeof(Float64)));
	
	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<class T> 
inline void MATLAB_Engine::put(const std::string& name,const std::vector<T>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	if (source.size() == 0) return;
	
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);

	array_ptr = mxCreateDoubleMatrix(1,source.size(),mxREAL);
	array_list_[name] = array_ptr;
	
	std::vector<Float64>	tvec;
	
	for (typename std::vector<T>::const_iterator it=source.begin();it!=source.end();++it)
		tvec.push_back(*it);
	
	//mxSetName(array_ptr,name.c_str());

	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(&tvec[0]),tvec.size()*sizeof(Float64)));
	
	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<Uint8 R> 
inline void MATLAB_Engine::put(const std::string& name,const Index<R>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);

	array_ptr = mxCreateDoubleMatrix(1,source.dim(),mxREAL);
	array_list_[name] = array_ptr;
	
	FixedVector<Float64,R>	tvec(construct< FixedVector<Float64,R> >::from(source));
	
	//mxSetName(array_ptr,name.c_str());
	
	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(&tvec[0]),tvec.dim()*sizeof(Float64)));
	
	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<class T,Uint8 R> 
inline void MATLAB_Engine::put(const std::string& name,const FixedVector<T,R>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);

	array_ptr = mxCreateDoubleMatrix(1,source.dim(),mxREAL);
	array_list_[name] = array_ptr;
	
	FixedVector<Float64,R>	tvec(construct< FixedVector<Float64,R> >::from(source));
	
	//mxSetName(array_ptr,name.c_str());
	
	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(&tvec[0]),tvec.dim()*sizeof(Float64)));
	
	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<class T> 
inline void MATLAB_Engine::put(const std::string& name,const Vector<T>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	if (!source.isValid()) return;
	
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);

	array_ptr = mxCreateDoubleMatrix(1,source.dim(),mxREAL);
	array_list_[name] = array_ptr;
	
	Vector<Float64>	tvec(construct< Vector<Float64> >::from(source));
	
	//mxSetName(array_ptr,name.c_str());

	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(tvec.dataPointer()),tvec.dim()*sizeof(Float64)));
	
	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<class T> 
inline void MATLAB_Engine::put(const std::string& name,const Matrix<T>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	if (!source.isValid()) return;
	
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);
	
	array_ptr = mxCreateDoubleMatrix(source.dim(1),source.dim(0),mxREAL);
	array_list_[name] = array_ptr;
	
	Matrix<Float64>	tmat(construct< Matrix<Float64> >::from(source));
	
	//mxSetName(array_ptr,name.c_str());

	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(tmat.dataPointer()),tmat.elem()*sizeof(Float64)));

	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<class T,Uint8 R,class Imp> 
inline void MATLAB_Engine::put(const std::string& name,const Tensor<T,R,Imp>& source)
{
#ifndef DISABLE_MATLAB_ENGINE
	if (source.size() <= 0) return;
	
	Vector<Float64>	tmp(IndexRange<1>(source.size()));
	
	for (typename Tensor<T,R,Imp>::const_iterator it=source.begin();it!=source.end();++it)
		tmp[it.offset()] = *it;
	
	put(name,tmp);

	// recover original dimension
	const Index<R>	tdim = source.signature().inverse_permute(source.range().dim());
	
	put("MATLAB_Engine_dim",tdim);
	put("MATLAB_Engine_sig",Index<R>(source.signature())+1);
	
	evaluate(name+" = reshape("+name+",MATLAB_Engine_dim(length(MATLAB_Engine_dim):-1:1));"+
			 name+" = ipermute("+name+",MATLAB_Engine_sig(size(MATLAB_Engine_sig,2):-1:1));"+
			 "clear MATLAB_Engine_dim MATLAB_Engine_sig;");
#endif
}

template<> inline void MATLAB_Engine::put(const std::string& name,const Vector<Float64>& vec)
{
#ifndef DISABLE_MATLAB_ENGINE
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);

	array_ptr = mxCreateDoubleMatrix(1,vec.dim(),mxREAL);
	array_list_[name] = array_ptr;
	
	//mxSetName(array_ptr,name.c_str());

	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(vec.dataPointer()),vec.dim()*sizeof(Float64)));

	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

template<> inline void MATLAB_Engine::put(const std::string& name,const Matrix<Float64>& mat)
{
#ifndef DISABLE_MATLAB_ENGINE
	MI			it = array_list_.find(name);
	mxArray*	array_ptr;
	
	if (it != array_list_.end())
		mxDestroyArray(it->second);
	
	array_ptr = mxCreateDoubleMatrix(mat.dim(1),mat.dim(0),mxREAL);
	array_list_[name] = array_ptr;
	
	//mxSetName(array_ptr,name.c_str());

	STD_(memcpy(reinterpret_cast<void*>(mxGetPr(array_ptr)),reinterpret_cast<void*>(mat.dataPointer()),mat.elem()*sizeof(Float64)));

	//engPutArray(engine_ptr_,array_ptr);
	engPutVariable(engine_ptr_,name.c_str(),array_ptr);
#endif
}

inline std::string MATLAB_Engine::buffer() const
{
#ifndef DISABLE_MATLAB_ENGINE
	return std::string(output_buffer_);
#else
	return "";
#endif
}

inline std::string MATLAB_Engine::className(const std::string& name) const
{
#ifndef DISABLE_MATLAB_ENGINE
	// check for variable "name" in MATLAB workspace
	mxArray*	array_ptr = engGetVariable(engine_ptr_,name.c_str());

	// check for variable "name" in internal record
	CMI	it = array_list_.find(name);
	
	// if variable exists but is not in internal record, add
	//		this usually happens when a variable is manually added 
	//		in MATLAB rather than through the C++ interface
	if (it == array_list_.end()) array_list_[name] = array_ptr;
		
	// return the class name of "name"
	return std::string(mxGetClassName(array_ptr));
#else
	return "";
#endif
}

inline void MATLAB_Engine::evaluate(const std::string& str)
{
#ifndef DISABLE_MATLAB_ENGINE
	engEvalString(engine_ptr_,str.c_str());
#endif
}

} // namespace mcs

#endif // MCS_MATLAB_ENGINE_H