#include "MCS_Ensure.h"
#include "MCS_IndexRange.h"
#include "MCS_Index.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// constructor
// rectangular zero offset matrix
template<class T> 
inline Matrix<T>::Matrix() : 
	rng_(),valid_(false),mat_ptr_()
{
	alloc(IndexRange<2>());
}

template<class T> 
inline Matrix<T>::Matrix(const IndexRange<2>& rng) : 
	rng_(),valid_(false),mat_ptr_()
{
	alloc(rng);
	set(T());
}

template<class T> 
inline Matrix<T>::Matrix(const IndexRange<2>& rng,const T& val) : 
	rng_(),valid_(false),mat_ptr_()
{
	alloc(rng);
	set(val);
}

template<class T> 
inline Matrix<T>::Matrix(const IndexRange<2>& rng,const T* ptr) : 
	rng_(),valid_(false),mat_ptr_()
{
	alloc(rng);
	set(ptr);
}

template<class T> 
inline Matrix<T>::Matrix(const IndexRange<2>& rng,const Vector<T>& vec) : 
	rng_(),valid_(false),mat_ptr_()
{
	alloc(rng);
	set(vec.dataPointer());
}

template<class T> 
inline Matrix<T>::Matrix(const IndexRange<2>& rng,const std::string& str_init) : 
	rng_(),valid_(false),mat_ptr_()
{	
	alloc(rng);
	set(str_init);
}

template<class T> 
inline Matrix<T>::Matrix(const MatrixRangeSlice<T>& rng) : 
	rng_(),valid_(false),mat_ptr_()
{	
	alloc(rng.rng_);
	set(rng);
}

// copy constructor
template<class T> 
inline Matrix<T>::Matrix(const Matrix<T>& source) : 
	rng_(),valid_(false),mat_ptr_()
{
	realloc(source.range());
	
	mat_ptr_ = source.mat_ptr_;
}

//destructor
template<class T> 
inline Matrix<T>::~Matrix()
{ 
	dealloc(); 
}

// rectangular arbitrary offset matrix
template<class T> 
inline void Matrix<T>::alloc(const IndexRange<2>& rng)
{
	if (all(rng.dim() >= 1))
	{
		rng_ = rng;
		
		try
		{
			mat_ptr_ = IndexRange<1>(elem());
			valid_ = mat_ptr_.isValid();
		}
		catch (...) 
		{ 
			rng_ = IndexRange<2>(); 
			valid_ = false;
			mat_ptr_ = IndexRange<1>();
			throw MemoryErr(); 
		}
	}
	else
	{
		rng_ = IndexRange<2>();
		valid_ = false;
		mat_ptr_ = IndexRange<1>();
	}
}

// deallocator
template<class T> 
inline void Matrix<T>::dealloc()
{ 
	rng_ = IndexRange<2>();
	valid_ = false; 
}

// reallocator
template<class T> 
inline void Matrix<T>::realloc(const IndexRange<2>& rng)
{
	// check to see if new range is same as old
	if (rng == rng_)
	{
		return;
	}
	// just change range if new dimensions are same as old
	else if (all(rng.dim() == rng_.dim()))
	{
		range(rng);
		return;
	}
	// otherwise actually reallocate
	else
	{
		dealloc();
		alloc(rng);
		set(T());
		return;
	}
}

// assignment
template<class T> 
inline Matrix<T>& Matrix<T>::operator=(const T& val)
{ 
	set(val); 
	return (*this); 
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator=(const std::string& str_init)
{ 
	set(str_init); 
	return (*this); 
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator=(const Matrix<T>& source)
{
	if (this == &source) return *this;
	
	realloc(source.range());
	mat_ptr_ = source.mat_ptr_;
	
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator=(const MatrixRangeSlice<T>& rng)
{
	if (this == &(rng.mat_)) throw IllegalOperationErr();
	
	realloc(rng.rng_);
	set(rng);
	
	return *this;
}		

// pretend Matrix is located at ptr, with specified size
// 	- only allowed on default allocated, invalid Matrices
//	- N.B. need to call unplace before destructor is reached or aliased memory can be released accidentally
template<class T> 
inline void Matrix<T>::place(T* ptr,const IndexRange<2>& rng)
{
	if (isValid() || ptr == NULL) throw IllegalOperationErr();
	
	valid_ = true;
	rng_ = rng;
	mat_ptr_.place(ptr,IndexRange<1>(elem()));
}

template<class T> 
inline void Matrix<T>::unplace()
{
	valid_ = false;
	rng_ = IndexRange<2>();
	mat_ptr_ = Vector<T>();
}

// safe clone of data located at pointer
//	- does not realloc
template<class T> 
inline void Matrix<T>::clone(const T* ptr)
{
	set(ptr);
}

// element assignors
// initialize with value
template<class T> 
inline void Matrix<T>::set(const T& val) 					
{ 
	if (isValid()) 
		mat_ptr_.set(val); 
}

// initialize with array
template<class T> 
inline void Matrix<T>::set(const T* ptr)
{ 
	if (isValid() && ptr != NULL) 
		mat_ptr_.set(ptr); 
}

template<class T> 
inline void Matrix<T>::set(const Vector<T>& data)
{ 
	if (isValid()) 
		mat_ptr_ = data; 
}

template<class T> 
void Matrix<T>::set(const std::string& str_init)
{
	std::stringstream 	temp1,temp2;	
	T					temp_val;
	Uint32 				temp_idx = 0;
	
	temp1 << str_init;
	
	// count number of elements in initializer string
	while (temp1 >> temp_val) temp_idx++;
	
	if (temp_idx != elem()) throw SizeMismatchErr();

	temp2 << str_init;
	
	for (Int32 i=first(0);i<=last(0);i++)
	{
		for (Int32 j=first(1);j<=last(1);j++)
		{
			temp2 >> temp_val;
			
			(*this)(i,j) = temp_val;
		}
	}
}

template<class T> 
void Matrix<T>::set(const MatrixRangeSlice<T>& rng)
{
	if (isValid() && range() == rng.rng_)
	{
		for (Int32 i=first(0);i<=last(0);i++)
			for (Int32 j=first(1);j<=last(1);j++)
				(*this)(i,j) = rng.mat_(i,j);
	}
}

// range assignors
template<class T> 
inline const Matrix<T>& Matrix<T>::offset(const Index<2>& offs) const
{
	range(IndexRange<2>(Index<2>(offs[0],offs[1]),Index<2>(offs[0]+dim(0)-1,offs[1]+dim(1)-1)));
	
	return (*this);
}

template<class T> 
inline const Matrix<T>& Matrix<T>::range(const IndexRange<2>& rng) const
{
	if (!isValid() || any(dim() != rng.dim())) throw SizeMismatchErr();
	
	rng_ = rng;
		
	return (*this);
}

// range accessors
// checks validity w/o throwing errors
template<class T> 
inline bool Matrix<T>::isValid() const 	
{
	return valid_;
}

template<class T> 
inline Int32 Matrix<T>::elem() const
{
	return rng_.elem();
}

template<class T> 
inline const Index<2>& Matrix<T>::first() const 	
{ 
	return rng_.first(); 
}

template<class T> 
inline const Index<2>& Matrix<T>::last() const 	
{
	return rng_.last(); 
}

template<class T> 
inline const Index<2>& Matrix<T>::dim() const 		
{ 
	return rng_.dim(); 
}

template<class T> 
inline const Index<2>& Matrix<T>::offset() const	
{ 
	return rng_.offset(); 
}

template<class T> 
inline const IndexRange<2>& Matrix<T>::range() const 	
{ 
	return rng_; 
}

// transform matrix indices to vector index
template<class T> 
inline Int32 Matrix<T>::transformIndex(const Index<2>& idx) const
#if MATRIX_RANGE_CHECKING
#if CERR_DEBUG
{ 
	if (any(idx < first()) || any(idx > last()))
		std::cerr << "Access error : " << idx << " in " << __FILE__ << ", line: " << __LINE__ << std::endl; 
		
	return Int32((idx[0]-rng_.first(0))*rng_.dim(1)+(idx[1]-rng_.first(1)));
}
#elif THROW_DEBUG
{ 
	if (any(idx < first()) || any(idx > last()))
		throw RangeErr(); 
	
	return Int32((idx[0]-rng_.first(0))*rng_.dim(1)+(idx[1]-rng_.first(1)));
}
#else
{
	return Int32((idx[0]-rng_.first(0))*rng_.dim(1)+(idx[1]-rng_.first(1)));
}
#endif
#else
{
	return Int32((idx[0]-rng_.first(0))*rng_.dim(1)+(idx[1]-rng_.first(1)));
}
#endif

// access data element
template<class T> 
inline T& Matrix<T>::getElement(const Int32& idx)
{
	return mat_ptr_[idx];
}

template<class T> inline const T& Matrix<T>::getElement(const Int32& idx) const
{
	return mat_ptr_[idx];
}

// element accessors
template<class T> 
inline T& Matrix<T>::operator()(const Index<2>& idx)
{
	return getElement(transformIndex(idx)); 
}

template<class T> 
inline const T& Matrix<T>::operator()(const Index<2>& idx) const
{
	return getElement(transformIndex(idx));
}
				
template<class T> 
inline T& Matrix<T>::operator()(const Int32& idx1,const Int32& idx2)
{
	return getElement(transformIndex(Index<2>(idx1,idx2))); 
}

template<class T> 
inline const T& Matrix<T>::operator()(const Int32& idx1,const Int32& idx2) const
{
	return getElement(transformIndex(Index<2>(idx1,idx2))); 
}

template<class T> 
inline MatrixIndirectionSlice<T> Matrix<T>::operator()(const MatrixIndirection& ind)
{
	return MatrixIndirectionSlice<T>((*this),ind);
}

template<class T> 
inline MatrixMaskSlice<T> Matrix<T>::operator()(const MatrixMask& msk)
{
	return MatrixMaskSlice<T>((*this),msk);
}

template<class T> 
inline MatrixRangeSlice<T> Matrix<T>::operator()(const IndexRange<2>& rng)
{
	return MatrixRangeSlice<T>((*this),rng);
}

template<class T> 
inline const MatrixIndirectionSlice<T> Matrix<T>::operator()(const MatrixIndirection& ind) const
{
	return MatrixIndirectionSlice<T>((*this),ind);
}

template<class T> 
inline const MatrixMaskSlice<T> Matrix<T>::operator()(const MatrixMask& msk) const
{
	return MatrixMaskSlice<T>((*this),msk);
}

template<class T> inline const MatrixRangeSlice<T> Matrix<T>::operator()(const IndexRange<2>& rng) const
{
	return MatrixRangeSlice<T>((*this),rng);
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> 
inline Matrix<T>& Matrix<T>::operator+=(const T& val)
{
	mat_ptr_ += val;
			
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator-=(const T& val)
{
	mat_ptr_ -= val;
			
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator*=(const T& val)
{
	mat_ptr_ *= val;
				
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator/=(const T& val)
{	
	mat_ptr_ /= val;
	
	return *this;
}

// Matrix member assign/equals operators
template<class T> 
inline Matrix<T>& Matrix<T>::operator+=(const Matrix<T>& source)
{
	if (!isValid()) return (*this = source);
	 
	if (range() != source.range()) throw RangeErr();

	mat_ptr_ += source.mat_ptr_;		
	
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator-=(const Matrix<T>& source)
{
	if (!isValid()) return (*this = -source);
	 
	if (range() != source.range()) throw RangeErr();
				
	mat_ptr_ -= source.mat_ptr_;
	
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator*=(const Matrix<T>& source)
{
	if (!isValid()) return (*this = Matrix<T>(source.range()));
	
	if (range() != source.range()) throw RangeErr();
			
	mat_ptr_ *= source.mat_ptr_;
	
	return *this;
}

template<class T> 
inline Matrix<T>& Matrix<T>::operator/=(const Matrix<T>& source)
{	
	if (!isValid()) return (*this = Matrix<T>(source.range()));
	
	if (range() != source.range()) throw RangeErr();
			
	mat_ptr_ /= source.mat_ptr_;
	
	return *this;
}	

} // namespace mcs
