#ifndef MCS_MATRIX_H
#define MCS_MATRIX_H

#include "MCS_IndexRange.h"
#include "MCS_Index.h"
#include "MCS_Vector.h"

namespace mcs {

template<class T> class Matrix;

class MatrixIndirection;
class MatrixMask;

template<class T> class MatrixIndirectionSlice;
template<class T> class MatrixMaskSlice;
template<class T> class MatrixRangeSlice;

__DECLARE_EXCEPTION_CLASS__(MatrixException,Exception)

// matrix
template<class T> class Matrix
{
	public:	
		typedef Matrix<T>&			reference;
		typedef const Matrix<T>&	const_reference;
		typedef Matrix<T>*			pointer;
		typedef const Matrix<T>*	const_pointer;
		
		// ctor/dtor	
		Matrix();
		explicit Matrix(const IndexRange<2>&);
		explicit Matrix(const IndexRange<2>&,const T&);
		explicit Matrix(const IndexRange<2>&,const T*);
		explicit Matrix(const IndexRange<2>&,const Vector<T>&);
		explicit Matrix(const IndexRange<2>&,const std::string&);
		explicit Matrix(const MatrixMaskSlice<T>&);
		explicit Matrix(const MatrixRangeSlice<T>&);
		Matrix(const Matrix<T>&);
		virtual ~Matrix();
		
		// range accessors	
				bool			isValid() const;
				
				Int32			elem() const;
				
				Int32	 		first(Uint8 r) const	{ return first()[r]; }
				Int32	 		last(Uint8 r) const		{ return last()[r]; }
				Int32			dim(Uint8 r) const		{ return dim()[r]; }
				Int32	 		offset(Uint8 r) const	{ return offset()[r]; }
		
		const 	Index<2>& 		first() const;
		const	Index<2>& 		last() const;
		const	Index<2>&		dim() const;
		const	Index<2>& 		offset() const;
		
		const	IndexRange<2>&	range() const;

			 	IndexRange<1> 	range(Uint8 d) const	{ return rng_.range(d); }
		
		// range assignors
		const	Matrix<T>& 		offset(const Index<2>&) const;
		const	Matrix<T>& 		range(const IndexRange<2>&) const;
		
		// assignment		
				Matrix<T>& 		operator=(const T&);
				Matrix<T>& 		operator=(const std::string&);
				Matrix<T>& 		operator=(const Matrix<T>&);
				 
				Matrix<T>& 		operator=(const MatrixRangeSlice<T>&);
		
		// element accessors		
			 	T&				operator()(const Index<2>&);
		const	T&				operator()(const Index<2>&) const;
	
				T&				operator()(const Int32&,const Int32&);
		const	T&				operator()(const Int32&,const Int32&) const;
		
			 	MatrixRangeSlice<T>			operator()(const IndexRange<2>&);
			 	MatrixMaskSlice<T>			operator()(const MatrixMask&);
			 	MatrixIndirectionSlice<T>	operator()(const MatrixIndirection&);
				
		const	MatrixRangeSlice<T>			operator()(const IndexRange<2>&) const;
		const	MatrixMaskSlice<T>			operator()(const MatrixMask&) const;
		const	MatrixIndirectionSlice<T>	operator()(const MatrixIndirection&) const;

		// arithmetic operations		
			 	Matrix<T>& 		operator+=(const T&);
			 	Matrix<T>& 		operator-=(const T&);
			 	Matrix<T>& 		operator*=(const T&);
			 	Matrix<T>& 		operator/=(const T&);
		
			 	Matrix<T>&		operator+=(const Matrix<T>&);
			 	Matrix<T>&		operator-=(const Matrix<T>&);
			 	Matrix<T>&		operator*=(const Matrix<T>&);
			 	Matrix<T>& 		operator/=(const Matrix<T>&);
		
		// low level access
				T* const		dataPointer() const		{ return mat_ptr_.dataPointer(); }
				
			 	Vector<T>&		asVector() 				{ return mat_ptr_; }
		const 	Vector<T>&		asVector() const		{ return mat_ptr_; }

		// set mat_ptr_ to specified pointer - potentially dangerous
				void			place(T*,const IndexRange<2>&);
				void			unplace();
				
		// copy data from specified pointer - safe
				void			clone(const T*);
				
		// element assignors		
				void 			set(const T&);
				void			set(const T*);
				void			set(const std::string&);
		// IO
				void			print(std::ostream& os) const;
				
		virtual	Matrix<T>& 		get(std::istream& ifs);
		virtual	Matrix<T>&		getNth(std::istream&,Int32 N = 1);
		virtual	Matrix<T>&		getNext(std::istream&,Int32 N = 1);
				
		virtual	Matrix<T>& 		read(std::istream& ifs);
		virtual	Matrix<T>&		readNth(std::istream& ifs,Int32 N = 1);
		virtual	Matrix<T>&		readNext(std::istream& ifs,Int32 N = 1);
				
		virtual const Matrix<T>& 	put(std::ostream& ofs) const;
		virtual const Matrix<T>& 	write(std::ostream& ofs) const;
		
		// offset class to allow offset change within local block with cleanup
		class Offset;
					
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)

	protected:
		// allocators
				void 			alloc(const IndexRange<2>&);
				void 			dealloc();
				void 			realloc(const IndexRange<2>& rng = IndexRange<2>());
							
		// element assignors		
				void 			set(const Vector<T>&);
				void 			set(const MatrixRangeSlice<T>&);
				
		// index transforms converting matrix indices to internal vector index
				Int32		transformIndex(const Index<2>&) const;
				
		// access element data
				T&				getElement(const Int32&);
		const	T&				getElement(const Int32&) const;
		
	private:
				bool			valid_;
				Vector<T>		mat_ptr_;
		
		mutable IndexRange<2>	rng_;	 
};

/*
	Utility class for Matrices which allows the offset index to be temporarily changed 
		within a local block of code, with the destructor resetting it to its original value
		on exit.  Usage :
		
			if (true)
			{
				Matrix<T>::Offset	offs(m1,o1,o2);
				
				...do some stuff...
			}
*/
template<class T> class Matrix<T>::Offset
{
	public:
		Offset(const Matrix<T>& mat,const Index<2>& off) : mat_(mat) { ooff_ = mat_.offset(); mat_.offset(off); }
		~Offset() { mat_.offset(ooff_); }
		
		const Index<2>& currentOffset() const	{ return mat_.first(); }
		const Index<2>& originalOffset() const	{ return ooff_; }
		
	private:
		Offset();
		Offset(const Offset&);
		Offset& operator=(const Offset&);
		
		const Matrix<T>&	mat_;
		Index<2>			ooff_;
};		

// matrix non-member operators

// logical operations		
template<class T> MatrixMask operator==(const Matrix<T>& lmat,const T& rscal);
template<class T> MatrixMask operator!=(const Matrix<T>& lmat,const T& rscal);
template<class T> MatrixMask operator<(const Matrix<T>& lmat,const T& rscal);
template<class T> MatrixMask operator<=(const Matrix<T>& lmat,const T& rscal);
template<class T> MatrixMask operator>(const Matrix<T>& lmat,const T& rscal);
template<class T> MatrixMask operator>=(const Matrix<T>& lmat,const T& rscal);

template<class T> MatrixMask operator==(const Matrix<T>& lmat,const Matrix<T>& rmat);
template<class T> MatrixMask operator!=(const Matrix<T>& lmat,const Matrix<T>& rmat);
template<class T> MatrixMask operator<(const Matrix<T>& lmat,const Matrix<T>& rmat);
template<class T> MatrixMask operator<=(const Matrix<T>& lmat,const Matrix<T>& rmat);
template<class T> MatrixMask operator>(const Matrix<T>& lmat,const Matrix<T>& rmat);
template<class T> MatrixMask operator>=(const Matrix<T>& lmat,const Matrix<T>& rmat);

// arithmetic operators
template<class T> Matrix<T> operator+(const T& lscal,const Matrix<T>& rmat);
template<class T> Matrix<T> operator-(const T& lscal,const Matrix<T>& rmat);
template<class T> Matrix<T> operator*(const T& lscal,const Matrix<T>& rmat);
template<class T> Matrix<T> operator/(const T& lscal,const Matrix<T>& rmat);

template<class T> Matrix<T> operator+(const Matrix<T>& lmat,const T& rscal);
template<class T> Matrix<T> operator-(const Matrix<T>& lmat,const T& rscal);
template<class T> Matrix<T> operator*(const Matrix<T>& lmat,const T& rscal);
template<class T> Matrix<T> operator/(const Matrix<T>& lmat,const T& rscal);

template<class T> Matrix<T> operator+(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> operator-(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> operator*(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> operator/(const Matrix<T>& lhs,const Matrix<T>& rhs);

template<class T> Vector<T> operator*(const Vector<T>& lvec,const Matrix<T>& rmat);
template<class T> Vector<T> operator*(const Matrix<T>& lmat,const Vector<T>& rvec);

template<class T> MatrixMask operator&&(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> MatrixMask operator||(const Matrix<T>& lhs,const Matrix<T>& rhs);

template<class T> Matrix<T> operator+(const Matrix<T>& source);
template<class T> Matrix<T> operator-(const Matrix<T>& source);

template<class T> Matrix<T> operator!(const Matrix<T>& source);

// in-place
// arithmetic functions
template<class T> const Matrix<T>& Abs(Matrix<T>& source);
template<class T> const Matrix<T>& Ceil(Matrix<T>& source);
template<class T> const Matrix<T>& Exp(Matrix<T>& source);
template<class T> const Matrix<T>& Floor(Matrix<T>& source);
template<class T> const Matrix<T>& Log(Matrix<T>& source);
template<class T> const Matrix<T>& Pow(Matrix<T>& source,const T& expon);
template<class T> const Matrix<T>& Round(Matrix<T>& source);
template<class T> const Matrix<T>& Sqr(Matrix<T>& source);
template<class T> const Matrix<T>& Sqrt(Matrix<T>& source);

// trigonometric functions
template<class T> const Matrix<T>& Acos(Matrix<T>& source);
template<class T> const Matrix<T>& Asin(Matrix<T>& source);
template<class T> const Matrix<T>& Atan(Matrix<T>& source);
template<class T> const Matrix<T>& Cos(Matrix<T>& source);
template<class T> const Matrix<T>& Sin(Matrix<T>& source);
template<class T> const Matrix<T>& Tan(Matrix<T>& source);

// hyperbolic functions
template<class T> const Matrix<T>& Acosh(Matrix<T>& source);
template<class T> const Matrix<T>& Asinh(Matrix<T>& source);
template<class T> const Matrix<T>& Atanh(Matrix<T>& source);
template<class T> const Matrix<T>& Cosh(Matrix<T>& source);
template<class T> const Matrix<T>& Sinh(Matrix<T>& source);
template<class T> const Matrix<T>& Tanh(Matrix<T>& source);

// complex matrices
template<class T> const Matrix<T>& Conj(Matrix<T>& source);
template<class T> const Matrix<T>& Imag(Matrix<T>& source);
template<class T> const Matrix<T>& Real(Matrix<T>& source);

// not in-place
// arithmetic functions
template<class T> Matrix<T> abs(const Matrix<T>& source);
template<class T> Matrix<T> ceil(const Matrix<T>& source);
template<class T> Matrix<T> exp(const Matrix<T>& source);
template<class T> Matrix<T> floor(const Matrix<T>& source);
template<class T> Matrix<T> log(const Matrix<T>& source);
template<class T> Matrix<T> pow(const Matrix<T>& source,const T& expon);
template<class T> Matrix<T> round(const Matrix<T>& source);
template<class T> Matrix<T> sqr(const Matrix<T>& source);
template<class T> Matrix<T> sqrt(const Matrix<T>& source);

// trigonometric functions
template<class T> Matrix<T> acos(const Matrix<T>& source);
template<class T> Matrix<T> asin(const Matrix<T>& source);
template<class T> Matrix<T> atan(const Matrix<T>& source);
template<class T> Matrix<T> atan2(const Matrix<T>& y,const Matrix<T>& x);
template<class T> Matrix<T> cos(const Matrix<T>& source);
template<class T> Matrix<T> sin(const Matrix<T>& source);
template<class T> Matrix<T> tan(const Matrix<T>& source);

// hyperbolic functions
template<class T> Matrix<T> acosh(const Matrix<T>& source);
template<class T> Matrix<T> asinh(const Matrix<T>& source);
template<class T> Matrix<T> atanh(const Matrix<T>& source);
template<class T> Matrix<T> cosh(const Matrix<T>& source);
template<class T> Matrix<T> sinh(const Matrix<T>& source);
template<class T> Matrix<T> tanh(const Matrix<T>& source);

// complex vectors
template<class T> Matrix<T> conj(const Matrix<T>& source);
template<class T> Matrix<typename DemoteComplex<T>::demote_t> imag(const Matrix<T>& source);
template<class T> Matrix<typename DemoteComplex<T>::demote_t> real(const Matrix<T>& source);

// miscellaneous
template<class T> Matrix<T> bound(const Matrix<T>& x,const T& min_val,const T& max_val);
template<class T> Matrix<Binary> even(const Matrix<T>& x);
template<class T> Matrix<Binary> odd(const Matrix<T>& x);
template<class T> Matrix<T> mod(const Matrix<T>& x,const T& y);
template<class T> Matrix<T> mod(const Matrix<T>& x,const Matrix<T>& y);
template<class T> Matrix<T> quotient(const Matrix<T>& x,const T& y);
template<class T> Matrix<T> quotient(const Matrix<T>& x,const Matrix<T>& y);

// Matrix specific
template<class T> bool all(const Matrix<T>&);
template<class T> bool any(const Matrix<T>&);

// matrix specific
template<class T> bool		isSquare(const Matrix<T>& source);

template<class T> T			prod(const Matrix<T>& source);
template<class T> T			sum(const Matrix<T>& source);
template<class T> T 		trace(const Matrix<T>& source);

template<class T> Vector<T> diagonal(const Matrix<T>& source);

template<class T> Matrix<T> product(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T>	transpose(const Matrix<T>& source);

} // namespace mcs

#include "MCS_MatrixIndirection.h"
#include "MCS_MatrixMask.h"

#include "MCS_MatrixIndirectionSlice.h"
#include "MCS_MatrixMaskSlice.h"
#include "MCS_MatrixRangeSlice.h"

// implementation
#include "MCS_Matrix.cpp"
#include "MCS_MatrixOperators.cpp"
#include "MCS_MatrixFunctions.cpp"

#include "MCS_MatrixIO.h"
#include "MCS_MatrixStatistics.h"
#include "MCS_MatrixTypecasts.h"
#include "MCS_MatrixUtilities.h"

#endif // MCS_MATRIX_H