#ifndef MCS_MATRIXIO_H
#define MCS_MATRIXIO_H

#include <iostream>
#include <string>
#include <valarray>

#include "MCS_IO.h"
#include "MCS_Matrix.h"
#include "MCS_TypeFlags.h"

namespace mcs {

//namespace matrix {

template<class T> std::ostream& operator<<(std::ostream& os,const Matrix<T>& source);

// stream input
/*
template<class T> inline std::istream& operator>>(std::istream& is,Matrix<T>& dest)
{ 
	const std::string	lineread(is.getline());
	
	dest = Matrix<T>(lineread);
	
	return is; 
}
*/
// stream output
template<class T> inline std::ostream& operator<<(std::ostream& os,const Matrix<T>& source)
{
	source.print(os);
	
	return os;
}

// I/O
// output in text format
template<class T> inline void Matrix<T>::print(std::ostream& os) const
{
	// allow this to be changed
	static const int 	MAX_ROW_ELEM = 10,
						MAX_COL_ELEM = 10;
	
	os << "Matrix : " << range() << std::endl;
	
	if (dim(0) > MAX_ROW_ELEM)
	{
		for (Int32 i=first(0);i<=first(0)+MAX_ROW_ELEM-1;i++)
		{
			os << "[ ";
			
			if (dim(1) > MAX_COL_ELEM)
			{
				for (Int32 j=first(1);j<=first(1)+MAX_COL_ELEM-1;j++)
					os << (*this)(i,j) << " ";
					
				os << " ... " << (*this)(i,last(1));
			}
			else
			{
				for (Int32 j=first(1);j<=last(1);j++)
					os << (*this)(i,j) << " ";
			}
			
			os << "]" << std::endl;
		}
	}
	else
	{
		for (Int32 i=first(0);i<=last(0);i++)
		{
			os << "[ ";
			
			if (dim(1) > MAX_COL_ELEM)
			{
				for (Int32 j=first(1);j<=first(1)+MAX_COL_ELEM-1;j++)
					os << (*this)(i,j) << " ";
					
				os << " ... " << (*this)(i,last(1));
			}
			else
			{
				for (Int32 j=first(1);j<=last(1);j++)
					os << (*this)(i,j) << " ";
			}
			
			os << "]" << std::endl;
		}
	}
		
	return;
}

// flat read from file stream
template<class T> Matrix<T>& Matrix<T>::get(std::istream& ifs)
{
	if (!isValid()) throw IOErr();
	
	try 
	{
		mat_ptr_.get(ifs);
	} 
	catch (std::ios_base::failure&) 
	{ 
		throw IOErr(); 
	}
	catch (typename Vector<T>::Err&)
	{
		throw IOErr();
	}
	
	return (*this);
}

// flat read nth matrix from file stream
template<class T> Matrix<T>& Matrix<T>::getNth(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(elem()*sizeof(T)));

		ifs.seekg(offs,std::ios::beg);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(elem()*sizeof(T)));

		ifs.seekg(offs,std::ios::end);
	}
					
	return get(ifs);
}

// skip n-1, then flat read nth matrix from file stream
template<class T> Matrix<T>& Matrix<T>::getNext(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(elem()*sizeof(T)));
		
		ifs.seekg(offs,std::ios::cur);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(elem()*sizeof(T)));
		
		ifs.seekg(offs,std::ios::cur);
	}
				
	return get(ifs);
}

// read from file stream, with dimensions
template<class T> Matrix<T>& Matrix<T>::read(std::istream& ifs)
{
	if (!ifs.good()) throw IOErr();
	
	Int32	lo_x_idx,hi_x_idx,lo_y_idx,hi_y_idx;
	Uint8		flags[4];
	
	try
	{
		if (!ifs.read(reinterpret_cast<char*>(&lo_x_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char*>(&hi_x_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char*>(&lo_y_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char*>(&hi_y_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char*>(&flags),4*sizeof(Uint8)))
			throw IOErr();
		
		if (TypeFlags<T>::flags().to_ulong() != flags[1]) throw IOErr();

		const IndexRange<2>	rng(Index<2>(lo_x_idx,lo_y_idx),Index<2>(hi_x_idx,hi_y_idx));
		
		realloc(rng);
		
		mat_ptr_.get(ifs);
	}
	catch (std::ios_base::failure&)
	{
		throw IOErr();
	}
	catch (typename Vector<T>::IOErr&)
	{
		throw IOErr();
	}
			
	return (*this);
}

// read nth from file stream, with dimensions
template<class T> Matrix<T>& Matrix<T>::readNth(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
			
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(elem()*sizeof(T)+4*sizeof(Int32)+4*sizeof(Uint8)));

		ifs.seekg(offs,std::ios::beg);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(elem()*sizeof(T)+4*sizeof(Int32)+4*sizeof(Uint8)));

		ifs.seekg(offs,std::ios::end);
	}

	return read(ifs);
}

// skip n-1, then read nth from file stream, with dimensions
template<class T> Matrix<T>& Matrix<T>::readNext(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(elem()*sizeof(T)+4*sizeof(Int32)+4*sizeof(Uint8)));
			
		ifs.seekg(offs,std::ios::cur);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(elem()*sizeof(T)+4*sizeof(Int32)+4*sizeof(Uint8)));
			
		ifs.seekg(offs,std::ios::cur);
	}
	
	return read(ifs);
}

// flat write to file stream
template<class T> const Matrix<T>& Matrix<T>::put(std::ostream& ofs) const
{
	if (!ofs.good()) throw IOErr();
	
	try 
	{
		mat_ptr_.put(ofs);
	} 
	catch (std::ios_base::failure&)
	{
		throw IOErr();
	}
	
	return (*this);
}

// write to file stream, with dimensions
template<class T> const Matrix<T>& Matrix<T>::write(std::ostream& ofs) const
{
	if (!ofs.good()) throw IOErr();
	
	Int32	lo_x_idx = rng_.first(0),
			hi_x_idx = rng_.last(0),
			lo_y_idx = rng_.first(1),
			hi_y_idx = rng_.last(1);
			
	Uint32	flags = TypeFlags<T>::flags().to_ulong() << 8;

	try 
	{
		// output lo and hi indices and data
		if (!ofs.write(reinterpret_cast<char*>(&lo_x_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char*>(&hi_x_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char*>(&lo_y_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char*>(&hi_y_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char*>(&flags),sizeof(Uint32)))
			throw IOErr();
			
		mat_ptr_.put(ofs);
	} 
	catch (std::ios_base::failure&)
	{ 
		throw IOErr(); 
	}
	catch (typename Vector<T>::IOErr&)
	{
		throw IOErr();
	}
	
	return (*this);
}

template<class T> void byteSwap(Matrix<T>& mat)
{
	byteSwap(mat.asVector());
}

//} // namespace matrix

} // namespace mcs

#endif // MCS_MATRIXIO_H
