#ifndef MCS_MATRIXINDIRECTIONSLICE_H
#define MCS_MATRIXINDIRECTIONSLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

template<class T> class Matrix;

// IndirectionSlice
template<class T> class IndirectionSlice
{
	public:	
		// dtor	
		~IndirectionSlice();
		
		// assignment		
		IndirectionSlice<T>& 	operator=(const T&);
		IndirectionSlice<T>& 	operator=(const Vector<T>&);
		IndirectionSlice<T>& 	operator=(const IndirectionSlice<T>&);
		
		// logical operations		
	 	MatrixMask				operator==(const T&) const;
	 	MatrixMask				operator!=(const T&) const;
	 	MatrixMask 				operator<(const T&) const;
	 	MatrixMask				operator<=(const T&) const;
	 	MatrixMask				operator>(const T&) const;
	 	MatrixMask 				operator>=(const T&) const;

	 	MatrixMask 				operator==(const IndirectionSlice<T>&) const;
	 	MatrixMask 				operator!=(const IndirectionSlice<T>&) const;
	 	MatrixMask				operator<(const IndirectionSlice<T>&) const;
	 	MatrixMask				operator<=(const IndirectionSlice<T>&) const;
	 	MatrixMask				operator>(const IndirectionSlice<T>&) const;
	 	MatrixMask 				operator>=(const IndirectionSlice<T>&) const;
		
		// arithmetic operations		
	 	IndirectionSlice<T>& 	operator+=(const T&);
	 	IndirectionSlice<T>& 	operator-=(const T&);
	 	IndirectionSlice<T>& 	operator*=(const T&);
	 	IndirectionSlice<T>& 	operator/=(const T&);

	 	IndirectionSlice<T>&	operator+=(const IndirectionSlice<T>&);
	 	IndirectionSlice<T>&	operator-=(const IndirectionSlice<T>&);
	 	IndirectionSlice<T>&	operator*=(const IndirectionSlice<T>&);
	 	IndirectionSlice<T>& 	operator/=(const IndirectionSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Matrix<T>;
		
		// ctor
		explicit IndirectionSlice(Matrix<T>&,const MatrixIndirection&);
		
		IndirectionSlice(const IndirectionSlice<T>&);
		
		void checkBounds() const;
		
				Matrix<T>&		mat_;
		const	MatrixIndirection&	ind_;		
};

// constructor
// rectangular zero offset matrix
template<class T> IndirectionSlice<T>::IndirectionSlice(Matrix<T>& mat,const MatrixIndirection& ind) : 
	mat_(mat),ind_(ind)
{
	// ensure all indirected indices are in bounds
	checkBounds();
}

template<class T> void IndirectionSlice<T>::checkBounds() const
{
#if MATRIX_RANGE_CHECKING
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		if (ind_[i][0] < mat_.first(0) || 
			ind_[i][0] > mat_.last(0) ||
			ind_[i][1] < mat_.first(1) ||
			ind_[i][1] > mat_.last(1))
			throw MatrixException::RangeErr();
#endif
	
	return;
}

//destructor
template<class T> IndirectionSlice<T>::~IndirectionSlice()
{ }

// assignment
template<class T> IndirectionSlice<T>& IndirectionSlice<T>::operator=(const T& val)
{ 
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		mat_(ind_[i]) = val;
			
	return *this;
}

template<class T> IndirectionSlice<T>& IndirectionSlice<T>::operator=(const Vector<T>& source)
{
	if (ind_.dim() != source.dim()) throw RangeErr();
		
	if (&mat_ == &source) return *this;
	
	for (Int32 i=ind_.first(),ii=source.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) = source[ii];
				
	return *this;
}

template<class T> IndirectionSlice<T>& IndirectionSlice<T>::operator=(const IndirectionSlice<T>& source)
{
	if (ind_.dim() != source.ind_.dim()) throw RangeErr();
		
	if (&mat_ == &source.mat_) return *this;
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) = source.mat_(source.ind_[ii]);
				
	return *this;
}

// logical operators
template<class T> MatrixMask IndirectionSlice<T>::operator==(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) == val);
		
	return tmp;
}
	
template<class T> MatrixMask IndirectionSlice<T>::operator!=(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) != val);
		
	return tmp;
}
	
template<class T> MatrixMask IndirectionSlice<T>::operator<(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) < val);
		
	return tmp;
}
	
template<class T> MatrixMask IndirectionSlice<T>::operator<=(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) <= val);
		
	return tmp;
}
	
template<class T> MatrixMask IndirectionSlice<T>::operator>(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) > val);
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator>=(const T& val) const
{
	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp(ind_[i]) = (mat_(ind_[i]) >= val);
		
	return tmp;
}
/*
template<class T> MatrixMask IndirectionSlice<T>::operator==(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) == source.mat_(source_.ind_[ii]));
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator!=(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) != source.mat_(source_.ind_[ii]));
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator<(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) < source.mat_(source_.ind_[ii]));
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator<=(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) <= source.mat_(source_.ind_[ii]));
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator>(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) > source.mat_(source_.ind_[ii]));
		
	return tmp;
}

template<class T> MatrixMask IndirectionSlice<T>::operator>=(const IndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw MatrixException::RangeErr();

	MatrixMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		tmp[i] = (mat_(ind_[i]) >= source.mat_(source_.ind_[ii]));
		
	return tmp;
}
*/
// arithmetic operators
// scalar member assign/equals operators
template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=ind_.first();i<=ind_.last();i++)
		mat_(ind_[i]) += val;
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=ind_.first();i<=ind_.last();i++)
		mat_(ind_[i]) -= val;
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=ind_.first();i<=ind_.last();i++)
		mat_(ind_[i]) *= val;
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator/=(const T& val)
{	
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=ind_.first();i<=ind_.last();i++)
		mat_(ind_[i]) /= val;
		
	return *this;
}

// IndirectionSlice member assign/equals operators
template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator+=(const IndirectionSlice<T>& source)
{
	if (source.dim() != ind_.dim()) throw RangeErr();

	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) += source.mat_(source.ind_[ii]);
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator-=(const IndirectionSlice<T>& source)
{
	if (source.dim() != ind_.dim()) throw RangeErr();

	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) -= source.mat_(source.ind_[ii]);
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator*=(const IndirectionSlice<T>& source)
{
	if (source.dim() != ind_.dim()) throw RangeErr();

	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) *= source.mat_(source.ind_[ii]);
		
	return *this;
}

template<class T> inline IndirectionSlice<T>& IndirectionSlice<T>::operator/=(const IndirectionSlice<T>& source)
{	
	if (source.dim() != ind_.dim()) throw RangeErr();

	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		mat_(ind_[i]) /= source.mat_(source.ind_[ii]);
		
	return *this;
}

} // namespace mcs

#endif // MCS_MATRIXINDIRECTIONSLICE_H