#ifndef MCS_MATRIXMASKSLICE_H
#define MCS_MATRIXMASKSLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> class Matrix;

// MatrixMaskSlice
template<class T> class MatrixMaskSlice
{
	public:	
		// dtor	
		~MatrixMaskSlice();
		
		// assignment		
		MatrixMaskSlice<T>& 	operator=(const T&);
		MatrixMaskSlice<T>& 	operator=(const Matrix<T>&);
		
		// logical operations		
	 	MatrixMask 		operator==(const T&) const;
	 	MatrixMask 		operator!=(const T&) const;
	 	MatrixMask 		operator<(const T&) const;
	 	MatrixMask 		operator<=(const T&) const;
	 	MatrixMask 		operator>(const T&) const;
	 	MatrixMask 		operator>=(const T&) const;

	 	MatrixMask 		operator==(const MatrixMaskSlice<T>&) const;
	 	MatrixMask 		operator!=(const MatrixMaskSlice<T>&) const;
	 	MatrixMask 		operator<(const MatrixMaskSlice<T>&) const;
	 	MatrixMask		operator<=(const MatrixMaskSlice<T>&) const;
	 	MatrixMask		operator>(const MatrixMaskSlice<T>&) const;
	 	MatrixMask 		operator>=(const MatrixMaskSlice<T>&) const;
		
		// arithmetic operations		
	 	MatrixMaskSlice<T>& 	operator+=(const T&);
	 	MatrixMaskSlice<T>& 	operator-=(const T&);
	 	MatrixMaskSlice<T>& 	operator*=(const T&);
	 	MatrixMaskSlice<T>& 	operator/=(const T&);

	 	MatrixMaskSlice<T>&	operator+=(const MatrixMaskSlice<T>&);
	 	MatrixMaskSlice<T>&	operator-=(const MatrixMaskSlice<T>&);
	 	MatrixMaskSlice<T>&	operator*=(const MatrixMaskSlice<T>&);
	 	MatrixMaskSlice<T>& 	operator/=(const MatrixMaskSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Matrix<T>;
		
		// ctor
		explicit MatrixMaskSlice(Matrix<T>&,const MatrixMask&);
		
		MatrixMaskSlice(const MatrixMaskSlice<T>&);
		MatrixMaskSlice<T>& operator=(const MatrixMaskSlice<T>&);
		
				Matrix<T>&	mat_;
		const	MatrixMask&		msk_;		
};

// constructor
// rectangular zero offset matrix
template<class T> MatrixMaskSlice<T>::MatrixMaskSlice(Matrix<T>& mat,const MatrixMask& msk) : 
	mat_(mat),msk_(msk)
{
	if (msk_.range() != mat_.range()) throw SizeMismatchErr(); 
}

//destructor
template<class T> MatrixMaskSlice<T>::~MatrixMaskSlice()
{ }

// assignment
template<class T> MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator=(const T& val)
{ 
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			if (msk_(i,j)) mat_(i,j) = val;
			
	return *this;
}

template<class T> MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator=(const Matrix<T>& source)
{
	if (msk_.range() != source.range()) throw RangeErr();
		
	if (&mat_ == &source) return *this;
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			if (msk_(i,j)) mat_(i,j) = source(i,j);
				
	return *this;
}

// logical operators
template<class T> MatrixMask MatrixMaskSlice<T>::operator==(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) == val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixMaskSlice<T>::operator!=(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) != val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixMaskSlice<T>::operator<(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) < val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixMaskSlice<T>::operator<=(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) <= val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixMaskSlice<T>::operator>(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) > val);
		
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator>=(const T& val) const
{
	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) >= val);
		
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator==(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) == source.mat_(i,j));
			
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator!=(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) != source.mat_(i,j));
			
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator<(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) < source.mat_(i,j));
			
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator<=(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) <= source.mat_(i,j));
			
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator>(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) > source.mat_(i,j));
			
	return tmp;
}

template<class T> MatrixMask MatrixMaskSlice<T>::operator>=(const MatrixMaskSlice<T>& source) const
{
	if (source.range() != msk_.range()) throw RangeErr();

	MatrixMask tmp(msk_);
	
	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			tmp(i,j) = (mat_(i,j) >= source.mat_(i,j));
			
	return tmp;
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) += val;
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) -= val;
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) *= val;
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator/=(const T& val)
{	
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) /= val;
			
	return *this;
}

// MatrixMaskSlice member assign/equals operators
template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator+=(const MatrixMaskSlice<T>& source)
{
	if (source.range() != msk_.range()) throw RangeErr();

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) += source(i,j);
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator-=(const MatrixMaskSlice<T>& source)
{
	if (source.range() != msk_.range()) throw RangeErr();

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) -= source(i,j);
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator*=(const MatrixMaskSlice<T>& source)
{
	if (source.range() != msk_.range()) throw RangeErr();

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) *= source(i,j);
			
	return *this;
}

template<class T> inline MatrixMaskSlice<T>& MatrixMaskSlice<T>::operator/=(const MatrixMaskSlice<T>& source)
{	
	if (source.range() != msk_.range()) throw RangeErr();

	for (Int32 i=msk_.first(0);i<=msk_.last(0);i++)
		for (Int32 j=msk_.first(1);j<=msk_.last(1);j++)
			mat_(i,j) /= source(i,j);
			
	return *this;
}

} // namespace mcs

#endif // MCS_MATRIXMASKSLICE_H