#ifndef MCS_MATRIXRANGESLICE_H
#define MCS_MATRIXRANGESLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> class Matrix;

// MatrixRangeSlice
template<class T> class MatrixRangeSlice
{
	public:	
		// dtor	
		~MatrixRangeSlice();
		
		// assignment		
		MatrixRangeSlice<T>& 	operator=(const T&);
		MatrixRangeSlice<T>& 	operator=(const Matrix<T>&);
		
		// logical operations		
	 	MatrixMask 		operator==(const T&) const;
	 	MatrixMask 		operator!=(const T&) const;
	 	MatrixMask 		operator<(const T&) const;
	 	MatrixMask 		operator<=(const T&) const;
	 	MatrixMask 		operator>(const T&) const;
	 	MatrixMask 		operator>=(const T&) const;

	 	MatrixMask 		operator==(const MatrixRangeSlice<T>&) const;
	 	MatrixMask 		operator!=(const MatrixRangeSlice<T>&) const;
	 	MatrixMask 		operator<(const MatrixRangeSlice<T>&) const;
	 	MatrixMask		operator<=(const MatrixRangeSlice<T>&) const;
	 	MatrixMask		operator>(const MatrixRangeSlice<T>&) const;
	 	MatrixMask 		operator>=(const MatrixRangeSlice<T>&) const;
		
		// arithmetic operations		
	 	MatrixRangeSlice<T>& 	operator+=(const T&);
	 	MatrixRangeSlice<T>& 	operator-=(const T&);
	 	MatrixRangeSlice<T>& 	operator*=(const T&);
	 	MatrixRangeSlice<T>& 	operator/=(const T&);

	 	MatrixRangeSlice<T>&	operator+=(const MatrixRangeSlice<T>&);
	 	MatrixRangeSlice<T>&	operator-=(const MatrixRangeSlice<T>&);
	 	MatrixRangeSlice<T>&	operator*=(const MatrixRangeSlice<T>&);
	 	MatrixRangeSlice<T>& 	operator/=(const MatrixRangeSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Matrix<T>;
		
		// ctor
		explicit MatrixRangeSlice(Matrix<T>&,const IndexRange<2>&);
		
		MatrixRangeSlice(const MatrixRangeSlice<T>&);
		MatrixRangeSlice<T>& operator=(const MatrixRangeSlice<T>&);
		
				Matrix<T>&		mat_;
		const	IndexRange<2>&	rng_;		
};

// constructor
// rectangular zero offset matrix
template<class T> MatrixRangeSlice<T>::MatrixRangeSlice(Matrix<T>& mat,const IndexRange<2>& rng) : 
	mat_(mat),rng_(rng)
{ }

//destructor
template<class T> MatrixRangeSlice<T>::~MatrixRangeSlice()
{ }

// assignment
template<class T> MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator=(const T& val)
{ 
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			mat_(i,j) = val;
			
	return *this;
}

template<class T> MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator=(const Matrix<T>& source)
{
	if (any(rng_.dim() != source.dim())) throw RangeErr();
	
	if (&mat_ == &source) return *this;
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			mat_(i,j) = source(ii,jj);
				
	return *this;
}

// logical operators
template<class T> MatrixMask MatrixRangeSlice<T>::operator==(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) == val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixRangeSlice<T>::operator!=(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) != val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixRangeSlice<T>::operator<(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) < val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixRangeSlice<T>::operator<=(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) <= val);
		
	return tmp;
}
	
template<class T> MatrixMask MatrixRangeSlice<T>::operator>(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) > val);
		
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator>=(const T& val) const
{
	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			tmp(i,j) = (mat_(i,j) >= val);
		
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator==(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) == source.mat_(ii,jj));
			
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator!=(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) != source.mat_(ii,jj));
			
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator<(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) < source.mat_(ii,jj));
			
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator<=(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) <= source.mat_(ii,jj));
			
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator>(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) > source.mat_(ii,jj));
			
	return tmp;
}

template<class T> MatrixMask MatrixRangeSlice<T>::operator>=(const MatrixRangeSlice<T>& source) const
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	MatrixMask tmp(rng_);
	
	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			tmp(i,j) = (mat_(i,j) >= source.mat_(ii,jj));
			
	return tmp;
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			mat_(i,j) += val;
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;

	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			mat_(i,j) -= val;
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			mat_(i,j) *= val;
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator/=(const T& val)
{	
	if (val == static_cast<T>(1)) return *this;

	for (Int32 i=rng_.first(0);i<=rng_.last(0);i++)
		for (Int32 j=rng_.first(1);j<=rng_.last(1);j++)
			mat_(i,j) /= val;
			
	return *this;
}

// MatrixRangeSlice member assign/equals operators
template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator+=(const MatrixRangeSlice<T>& source)
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			mat_(i,j) += source(ii,jj);
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator-=(const MatrixRangeSlice<T>& source)
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			mat_(i,j) -= source(ii,jj);
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator*=(const MatrixRangeSlice<T>& source)
{
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			mat_(i,j) *= source(ii,jj);
			
	return *this;
}

template<class T> inline MatrixRangeSlice<T>& MatrixRangeSlice<T>::operator/=(const MatrixRangeSlice<T>& source)
{	
	if (any(source.dim() != rng_.dim())) throw RangeErr();

	for (Int32 i=rng_.first(0),ii=source.first(0);i<=rng_.last(0);i++,ii++)
		for (Int32 j=rng_.first(1),jj=source.first(1);j<=rng_.last(1);j++,jj++)
			mat_(i,j) /= source(ii,jj);
			
	return *this;
}

} // namespace mcs

#endif // MCS_MATRIXRANGESLICE_H