#ifndef MCS_MATRIXSTATISTICS_H
#define MCS_MATRIXSTATISTICS_H

#include "MCS_Matrix.h"
#include "MCS_Promote.h"
#include "MCS_VectorStatistics.h"

namespace mcs {

namespace statistics {

template<class T> T 			max(const Matrix<T>& source);
template<class T> Matrix<T> 	max(const Matrix<T>& lhs,const T& rhs);
template<class T> Matrix<T> 	max(const T& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> 	max(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> T				min(const Matrix<T>& source);
template<class T> Matrix<T> 	min(const Matrix<T>& lhs,const T& rhs);
template<class T> Matrix<T> 	min(const T& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> 	min(const Matrix<T>& lhs,const Matrix<T>& rhs);

template<class T> T 			product(const Matrix<T>& source); 		
template<class T> T 			sum(const Matrix<T>& source); 		

template<class T> typename PromoteFloat<T>::promote_t	mean(const Matrix<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const Matrix<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const Matrix<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const Matrix<T>& source,const Matrix<T>& weight);

template<class T> Vector<T> 	averageColumns(const Matrix<T>& source);
template<class T> Vector<T> 	averageRows(const Matrix<T>& source);
template<class T> Vector<T> 	sumColumns(const Matrix<T>& source);
template<class T> Vector<T> 	sumRows(const Matrix<T>& source);

// min/max 
template<class T> inline T max(const Matrix<T>& source)
{
	return max(source.asVector());
}

template<class T> inline Matrix<T> max(const Matrix<T>& lhs,const T& rhs)
{
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs));
}

template<class T> inline Matrix<T> max(const T& lhs,const Matrix<T>& rhs)
{
	return Matrix<T>(rhs.range(),max(lhs,rhs.asVector()));
}

template<class T> inline Matrix<T> max(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Matrix<T>::RangeErr();
			
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs.asVector()));
}

template<class T> inline T min(const Matrix<T>& source)
{
	return min(source.asVector());
}

template<class T> inline Matrix<T> min(const Matrix<T>& lhs,const T& rhs)
{
	return Matrix<T>(lhs.range(),min(lhs.asVector(),rhs));
}

template<class T> inline Matrix<T> min(const T& lhs,const Matrix<T>& rhs)
{
	return Matrix<T>(rhs.range(),min(lhs,rhs.asVector()));
}

template<class T> inline Matrix<T> min(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Matrix<T>::RangeErr();
			
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs.asVector()));
}

// product of all elements
template<class T> inline T product(const Matrix<T>& source)
{
	return mcs::product(source);
}

// sum of all elements
template<class T> inline T sum(const Matrix<T>& source)
{
	return mcs::sum(source);
}

// other statistics
template<class T> inline typename PromoteFloat<T>::promote_t mean(const Matrix<T>& source)
{ 
	return mean(source.asVector()); 
}
 
template<class T> inline typename PromoteFloat<T>::promote_t standardDeviation(const Matrix<T>& source)
{ 
	return standardDeviation(source.asVector());
}

// corrected two-pass calculation (NR pp. 613)
template<class T> inline typename PromoteFloat<T>::promote_t variance(const Matrix<T>& source)
{ 
	return variance(source.asVector());
}

template<class T> inline typename PromoteFloat<T>::promote_t weightedMean(const Matrix<T>& source,const Matrix<T>& weight)
{
	return weightedMean(source.asVector(),weight.asVector());
}

template<class T> inline Vector<T> averageColumns(const Matrix<T>& source)
{
	Vector<T>	sm(IndexRange<1>(source.first(0),source.last(0))),
				nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[i] += source(idx);
				nm[i]++;
			}
		}
	}
	
	return sm/nm;
}

template<class T> inline Vector<T> averageRows(const Matrix<T>& source)
{
	Vector<T>	sm(IndexRange<1>(source.first(1),source.last(1))),
				nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[j] += source(idx);
				nm[j]++;
			}
		}
	}
	
	return sm/nm;
}

template<class T> inline Vector<T> sumColumns(const Matrix<T>& source)
{
	Vector<T>	sm(IndexRange<1>(source.first(0),source.last(0)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
				sm[i] += source(idx);
		}
	}
	
	return sm;
}

template<class T> inline Vector<T> sumRows(const Matrix<T>& source)
{
	Vector<T>	sm(IndexRange<1>(source.first(1),source.last(1)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			sm[j] += source(idx);
		}
	}
	
	return sm;
}

namespace ignoreNaN {


template<class T> T 			max(const Matrix<T>& source);
template<class T> Matrix<T> 	max(const Matrix<T>& lhs,const T& rhs);
template<class T> Matrix<T> 	max(const T& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> 	max(const Matrix<T>& lhs,const Matrix<T>& rhs);
template<class T> T				min(const Matrix<T>& source);
template<class T> Matrix<T> 	min(const Matrix<T>& lhs,const T& rhs);
template<class T> Matrix<T> 	min(const T& lhs,const Matrix<T>& rhs);
template<class T> Matrix<T> 	min(const Matrix<T>& lhs,const Matrix<T>& rhs);

template<class T> T 			product(const Matrix<T>& source); 		
template<class T> T 			sum(const Matrix<T>& source); 		

template<class T> typename PromoteFloat<T>::promote_t	mean(const Matrix<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const Matrix<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const Matrix<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const Matrix<T>& source,const Matrix<T>& weight);

template<class T> Vector<T> 	averageColumns(const Matrix<T>& source);
template<class T> Vector<T> 	averageRows(const Matrix<T>& source);
template<class T> Vector<T> 	sumColumns(const Matrix<T>& source);
template<class T> Vector<T> 	sumRows(const Matrix<T>& source);

// min/max 
template<class T> inline T max(const Matrix<T>& source)
{
	return max(source.asVector());
}

template<class T> inline Matrix<T> max(const Matrix<T>& lhs,const T& rhs)
{
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs));
}

template<class T> inline Matrix<T> max(const T& lhs,const Matrix<T>& rhs)
{
	return Matrix<T>(rhs.range(),max(lhs,rhs.asVector()));
}

template<class T> inline Matrix<T> max(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Matrix<T>::RangeErr();
			
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs.asVector()));
}

template<class T> inline T min(const Matrix<T>& source)
{
	return min(source.asVector());
}

template<class T> inline Matrix<T> min(const Matrix<T>& lhs,const T& rhs)
{
	return Matrix<T>(lhs.range(),min(lhs.asVector(),rhs));
}

template<class T> inline Matrix<T> min(const T& lhs,const Matrix<T>& rhs)
{
	return Matrix<T>(rhs.range(),min(lhs,rhs.asVector()));
}

template<class T> inline Matrix<T> min(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Matrix<T>::RangeErr();
			
	return Matrix<T>(lhs.range(),max(lhs.asVector(),rhs.asVector()));
}

// product of all elements
template<class T> inline T product(const Matrix<T>& source)
{
	return product(source.asVector());
}

// sum of all elements
template<class T> inline T sum(const Matrix<T>& source)
{
	return sum(source.asVector());
}

// other statistics
template<class T> inline typename PromoteFloat<T>::promote_t mean(const Matrix<T>& source)
{ 
	return mean(source.asVector()); 
}
 
template<class T> inline typename PromoteFloat<T>::promote_t standardDeviation(const Matrix<T>& source)
{ 
	return standardDeviation(source.asVector());
}

// corrected two-pass calculation (NR pp. 613)
template<class T> inline typename PromoteFloat<T>::promote_t variance(const Matrix<T>& source)
{ 
	return variance(source.asVector());
}

template<class T> inline typename PromoteFloat<T>::promote_t weightedMean(const Matrix<T>& source,const Matrix<T>& weight)
{
	return weightedMean(source.asVector(),weight.asVector());
}

template<class T> inline Vector<T> averageColumns(const Matrix<T>& source)
{
	return statistics::averageColumns(source);
}

template<> inline Vector<Float32> averageColumns(const Matrix<Float32>& source)
{
	Vector<Float32>	sm(IndexRange<1>(source.first(0),source.last(0))),
					nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[i] += source(idx);
				nm[i]++;
			}
		}
	}
	
	return sm/nm;
}

template<> inline Vector<Float64> averageColumns(const Matrix<Float64>& source)
{
	Vector<Float64>	sm(IndexRange<1>(source.first(0),source.last(0))),
					nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[i] += source(idx);
				nm[i]++;
			}
		}
	}
	
	return sm/nm;
}

template<class T> inline Vector<T> averageRows(const Matrix<T>& source)
{
	return statistics::averageRows(source);
}

template<> inline Vector<Float32> averageRows(const Matrix<Float32>& source)
{
	Vector<Float32>	sm(IndexRange<1>(source.first(1),source.last(1))),
					nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[j] += source(idx);
				nm[j]++;
			}
		}
	}
	
	return sm/nm;
}

template<> inline Vector<Float64> averageRows(const Matrix<Float64>& source)
{
	Vector<Float64>	sm(IndexRange<1>(source.first(1),source.last(1))),
					nm(sm.range());
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
			{
				sm[j] += source(idx);
				nm[j]++;
			}
		}
	}
	
	return sm/nm;
}

template<class T> inline Vector<T> sumColumns(const Matrix<T>& source)
{
	return statistics::sumColumns(source);
}

template<> inline Vector<Float32> sumColumns(const Matrix<Float32>& source)
{
	Vector<Float32>	sm(IndexRange<1>(source.first(0),source.last(0)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
				sm[i] += source(idx);
		}
	}
	
	return sm;
}

template<> inline Vector<Float64> sumColumns(const Matrix<Float64>& source)
{
	Vector<Float64>	sm(IndexRange<1>(source.first(0),source.last(0)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
				sm[i] += source(idx);
		}
	}
	
	return sm;
}

template<class T> inline Vector<T> sumRows(const Matrix<T>& source)
{
	return statistics::sumRows(source);
}

template<> inline Vector<Float32> sumRows(const Matrix<Float32>& source)
{
	Vector<Float32>	sum(IndexRange<1>(source.first(1),source.last(1)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
				sum[j] += source(idx);
		}
	}
	
	return sum;
}

template<> inline Vector<Float64> sumRows(const Matrix<Float64>& source)
{
	Vector<Float64>	sum(IndexRange<1>(source.first(1),source.last(1)));
	
	for (Int32 i=source.first(0);i<=source.last(0);++i)
	{
		for (Int32 j=source.first(1);j<=source.last(1);++j)
		{
			const Index<2>	idx(i,j);
			
			if (isfinite(source(idx)))
				sum[j] += source(idx);
		}
	}
	
	return sum;
}

} // namespace ignoreNaN

} // namespace statistics

} // namespace mcs

#endif // MCS_MATRIXSTATISTICS_H