// MemoryManager.cpp

// Howard E. Hinnant	 1995 - 2000

/*
	This code manages memory by allocated large blocks by malloc, and then allocating
	sub-blocks from the large blocks to satisfy memory requests.  Performance should
	be fairly acceptable considering all the error checking that is going on.
	
	On the performance side:  sub-blocks are allocated from large blocks using a first-fit
	algorithm.  The sub-blocks are stored in a circular, doubly linked list.  The start of
	the search starts where the last search left off.  New large blocks are automatically
	added when existing ones are depleted.  The size of the large block is set by the
	arguments to the MemoryManager constructor.  If more memory is asked for than the size
	of the large block, this is handled without problem.  Upon freeing, physically adjacent
	sub-blocks are merged into one larger sub-block.
	
	Error checking:  Each sub-block is bordered on the low and high ends with information
	about the sub-block.  Upon freeing, if this memory is overwritten, MemoryManager will
	print out a diagnostic (to the console) and exit.  Diagonostics are also printed if you
	try to delete the same pointer more than once, or if you try to use array delete on a
	non-array new'd	pointer or vice versa.  Also, upon program end, all sub-blocks that
	have not been freed	will generate an error message, optionally with the file name and
	line number of where they were new'd.
	
	To use:  Put this file in your project.  That's it.  You will now get out-
	of-bounds memory checking, and leak checking.  However any leaks you have will say
	something like:
	Leaking memory (8 bytes) at ??
	To get specific information	about where the leaked memory was new'd, replace your
	"new" with "NEW".  You will also need to #include "MemoryManager.h" in each file that
	you put "NEW" into.  Reports are then printed to the console window that look like
	"Leaking memory (8 bytes) at HelloWorld.cp, Line 8".
	
	If you are using the nothrow versions of new, and want information about where leaked
	memory was allocated, you must use the NEW_NOTHROW(std::nothrow) macro instead of
	the NEW macro.
	
	If there are no memory overwrites, double deletes, leaks, etc., then no output is
	produced from the MemoryManager.  It will be a silent friend.
	
	See MemoryManager.cpp on how to redirect errors to a log file.  The default error
	reporting goes to stderr (which may or may not be operational in your environment).
	
	Known bugs:  This code has not been made multi-thread safe.  To do this, the new
	and delete operators should be wrapped up in a "critical section".
	
	MemoryManager attempts to diagnose mismatched new/delete[] and new[]/delete.  When
	calling	delete[] on a new with a class that has a non-trivial destructor, at best 
	MemoryManager will miss-diagnose this as a "block underwrite" (client writes below
	allowed address).  At worst, your app will crash before it even gets to MemoryManager's
	operator delete [] (in which case it would also crash without MemoryManager).
	
	This code inspired by Donald Knuth's "Fundamental Algorithms."
*/

#include "MemoryManager.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <new.h>

#define MinBlockSize 1024
#define MinSubBlockSize (6*sizeof(size_t))

#ifndef _MSL_NO_CPP_NAMESPACE
	using namespace Metrowerks;
#endif

// Customize your memory manager here:
//	1.	You can give the mm constructor a default block size (in bytes).
//		The default is 65536 bytes.
//	2.	You can send errors to stdout or stderr by setting log_open to true and 
//		error_log to the desired FILE*.
//		Or you can send errors to a file of your choice by setting log_open to
//		false and setting error_file to the filename.
//  3.  #define MANAGE_MALLOC to include malloc checking as well.  You will get
//      linker warnings when you do this.  That is normal.
//#define MANAGE_MALLOC

namespace
{
	MemoryManager&
	get_mm()
	{
		static MemoryManager mm;                 // set default block size, e.g. mm(1048576);
		return mm;
	}
	bool log_open = true;                        // true if reporting to a standard stream, false if to a file
	const char* const error_file = "errors.out"; // if !log_open, report to this file
	FILE* error_log = stderr;                    // if log_open report to this already open FILE*

	bool aborting = false;

	void report(const char* f);
	void report(const char* f, void* p);
	void report(const char* f, void* p, const char* s);
	void report(const char* f, size_t size);
	void report(const char* f, size_t size, const char* s);

	inline void open_log()
	{
		error_log = fopen(error_file, "w");
		log_open = true;
	}

	void report(const char* f)
	{
		if (!log_open)
			open_log();
		fprintf(error_log, f);
	}

	void report(const char* f, void* p)
	{
		if (!log_open)
			open_log();
		fprintf(error_log, f, p);
	}

	void report(const char* f, void* p, const char* s)
	{
		if (!log_open)
			open_log();
		fprintf(error_log, f, p, s);
	}

	void report(const char* f, size_t size)
	{
		if (!log_open)
			open_log();
		fprintf(error_log, f, size);
	}

	void report(const char* f, size_t size, const char* s)
	{
		if (!log_open)
			open_log();
		fprintf(error_log, f, size, s);
	}

	const size_t ARRAY_FLAG = 0x00000100;

	void* new_dance(size_t size, const char* file, int line, bool array = false, bool throwOnError = true);
	void* new_dance(size_t size, const char* file, int line, bool array, bool throwOnError)
	{
		char buf[128];
		size_t len;
		if (file != 0)
		{
			sprintf(buf, "%s, Line %d\n", file, line);
			len = strlen(buf) + 1;
			while ((len + sizeof(size_t)) % 8 != 0)
				len++;
		}
		else
		{
			buf[0] = buf[1] = '?';
			buf[2] = '\n';
			buf[3] = '\0';
			len = 4;
		}
		char* result;
		while (true)
		{
			result = (char*)get_mm().malloc(len + sizeof(size_t) + size);
			if (result != 0)
				break;
			new_handler f = set_new_handler(0);
			set_new_handler(f);
			if (f)
			{
				if (throwOnError)
					f();
				else
				{
					try
					{
						f();
					}
					catch (bad_alloc)
					{
						return 0;
					}
				}
			}
			else if (throwOnError)
				throw bad_alloc();
			else
				return 0;
		}
		if (array)
			*(size_t*)(result - sizeof(size_t)) |= ARRAY_FLAG;
		strcpy(result, buf);
		char* ptr = result + len + sizeof(size_t);
		result += strlen(buf) + 1;
		while (result + sizeof(size_t) < ptr)
			*result++ = char(0xff);
		*(size_t*)result = len;
		return ptr;
	}
}

void* operator new(size_t size) throw(_STD::bad_alloc)
{
	return new_dance(size, 0, 0);
}

void* operator new(_CSTD::size_t size, const _STD::nothrow_t&) throw()
{
	return new_dance(size, 0, 0, false, false);
}

void* operator new[](size_t size) throw(_STD::bad_alloc)
{
	return new_dance(size, 0, 0, true);
}

void* operator new[](_CSTD::size_t size, const _STD::nothrow_t&) throw()
{
	return new_dance(size, 0, 0, true, false);
}

void* operator new(size_t size, const char* file, int line) throw(_STD::bad_alloc)
{
	return new_dance(size, file, line);
}

void* operator new(_CSTD::size_t size, const _STD::nothrow_t&, const char* file, int line) throw()
{
	return new_dance(size, file, line, false, false);
}

void* operator new[](size_t size, const char* file, int line) throw(_STD::bad_alloc)
{
	return new_dance(size, file, line, true);
}

void* operator new[](_CSTD::size_t size, const _STD::nothrow_t&, const char* file, int line) throw()
{
	return new_dance(size, file, line, true, false);
}

#if __dest_os == __mac_os

	#include <Carbon.h>

#elif __dest_os == __win32_os

	#define WIN32_LEAN_AND_MEAN
	#include <windows.h>

#endif

namespace
{

#if __dest_os == __mac_os

	inline
	void*
	sys_alloc(_CSTD::size_t size)
	{
		return NewPtr((long)size);
	}

	inline
	void
	sys_free(void* p)
	{
		if (p)
			DisposePtr((Ptr)p);
	}

#elif __dest_os == __win32_os

	void*
	sys_alloc(_CSTD::size_t size)
	{
		char* ptr;
		size += 8;
	#if __dest_os==__win32_os
		ptr = (char*)GlobalAlloc(GMEM_FIXED, size);
	#else
		ptr = (char*)LocalAlloc(LMEM_FIXED, size);
	#endif
		if (ptr == 0)
			return 0;
		if ((size_t)ptr % 8 == 0)
		{
			ptr += 4;
			*(size_t*)ptr = 0;
		}
		else
			*(size_t*)ptr = 1;
		ptr += 4;
		return (void*)ptr;
	}

	inline
	void
	sys_free(void* p)
	{
		if (p)
		{
			char* ptr = (char*)p;
			ptr -= 4;
			if (*(size_t*)ptr == 0)
				ptr -= 4;
			GlobalFree((HGLOBAL) ptr);
		}
	}

#endif

bool
has_no_item_count(char* ptr)
{
	ptr -= sizeof(size_t);
	size_t len = *(size_t*)ptr;
	if (len >= 128)
		return false;
	ptr -= len;
	size_t ar = *(size_t*)(ptr - sizeof(size_t));
	ar &= 0xff;
	return ar == '+' || ar == '-';
}

bool
has_item_count(char* ptr)
{
	ptr -= 3*sizeof(size_t);
	size_t len = *(size_t*)ptr;
	if (len >= 128)
		return false;
	ptr -= len;
	size_t ar = *(size_t*)(ptr - sizeof(size_t));
	ar &= 0xff;
	return ar == '+' || ar == '-';
}

}

void operator delete(void* ptr) throw()
{
	if (ptr != 0)
	{
		char* p = (char*)ptr;
		if (has_no_item_count(p))
			p -= sizeof(size_t);
		else if (has_item_count(p))
			p -= 3*sizeof(size_t);
		else
		{
			report("MemoryManager detects block underwrite\n");
			aborting = true;
			exit(1);
		}
		size_t len = *(size_t*)p;
		if (len >= 128)
		{
			report("MemoryManager detects block underwrite\n");
			aborting = true;
			exit(1);
		}
		p -= len;
		size_t& ar = *(size_t*)(p - sizeof(size_t));
		if ((ar & 0xff) == '-')
		{
			report("MemoryManager detects double delete on %p\n", ptr);
			aborting = true;
			exit(1);
		}
		if (ar & ARRAY_FLAG)
		{
			report("MemoryManager detects non-array delete on array new %p allocated at %s\n", ptr, p);
			aborting = true;
			exit(1);
		}
		get_mm().free(p);
	}
}

void operator delete(void* ptr, const _STD::nothrow_t&) throw()
{
	operator delete(ptr);
}

void operator delete[](void* ptr) throw()
{
	if (ptr != 0)
	{
		char* p = (char*)ptr;
		if (has_item_count(p))
			p -= 3*sizeof(size_t);
		else if (has_no_item_count(p))
			p -= sizeof(size_t);
		else
		{
			report("MemoryManager detects block underwrite\n");
			aborting = true;
			exit(1);
		}
		size_t len = *(size_t*)p;
		if (len >= 128)
		{
			report("MemoryManager detects block underwrite\n");
			aborting = true;
			exit(1);
		}
		p -= len;
		size_t& ar = *(size_t*)(p - sizeof(size_t));
		if ((ar & 0xff) == '-')
		{
			report("MemoryManager detects double delete on %p\n", ptr);
			aborting = true;
			exit(1);
		}
		if (!(ar & ARRAY_FLAG))
		{
			report("MemoryManager detects array delete on non-array new %p allocated at %s\n", ptr, p);
			aborting = true;
			exit(1);
		}
		ar &= ~ARRAY_FLAG;
		get_mm().free(p);
	}
}

void operator delete[](void* ptr, const _STD::nothrow_t&) throw()
{
	operator delete[](ptr);
}

void operator delete(void* ptr, const char*, int) throw()
{
	operator delete(ptr);
}

void operator delete(void* ptr, const _STD::nothrow_t&, const char*, int) throw()
{
	operator delete(ptr);
}

void operator delete[](void* ptr, const char*, int) throw()
{
	operator delete[](ptr);
}

void operator delete[](void* ptr, const _STD::nothrow_t&, const char*, int) throw()
{
	operator delete[](ptr);
}

#ifdef MANAGE_MALLOC

#ifdef _MSL_USING_NAMESPACE
	namespace std {
#endif

void* malloc(size_t s)
{
	return operator new(s);
}

void free(void* p)
{
	operator delete(p);
}

void*
realloc(void* ptr, size_t size)
{
	if (ptr == 0)
		return malloc(size);
	if (size == 0)
	{
		free(ptr);
		return 0;
	}
	size_t orig_size = get_mm().get_size(ptr);
	void* p = malloc(size);
	if (p != 0)
	{
		orig_size = orig_size < size ? orig_size : size;
		memcpy(p, ptr, orig_size);
		free(ptr);
	}
	return p;
}

void*
calloc(size_t nmemb, size_t size)
{
	void* result;

	size *= nmemb;
	result = malloc(size);
	if (result != 0)
		memset(result, 0, size);
	return result;
}

#ifdef _MSL_USING_NAMESPACE
	}
#endif

#endif // MANAGE_MALLOC

MemoryManager::MemoryManager(size_t blocksize)
	: default_blocksize_(blocksize),
	  masterlen_(0),
	  master_(0),
	  start_(0)
{
	if (default_blocksize_ < MinBlockSize)
	{
		report("MemoryManager default blocksize is too small (< %d)\n", MinBlockSize);
		aborting = true;
		exit(1);
	}
}

MemoryManager::~MemoryManager()
{
	if (!aborting)
		screamIfInUse();
	for (size_t i = 0; i < masterlen_; ++i)
		if (master_[i] != 0)
			sys_free(master_[i]);
	sys_free(master_);
}

void
MemoryManager::screamIfInUse() const
{
	size_t totalwaste = 0;
	for (int i = 0; i < masterlen_; i++)
	{
		if (master_[i] != 0)
		{
			Block::iterator e = master_[i]->end();
			for (Block::iterator j = master_[i]->begin(); j != e; ++j)
			{
				if (j->not_free())
				{
					size_t size = j->size() - SubBlock::overhead();
					const char* p = (const char*)(&*j) + 2*sizeof(size_t)
					              + strlen((const char*)&*j + 2*sizeof(size_t)) + 1;
					while (*p == (char)0xff)
						p++;
					size -= *(size_t*)p + sizeof(size_t);
					totalwaste += size;
					report("Leaking memory (%d bytes) at %s", size, (const char*)&*j + 2*sizeof(size_t));
				}
			}
		}
	}
	if (totalwaste > 0)
		report("Total memory leaked is %d bytes\n", totalwaste);
}

namespace
{
	int dummy;
}

void*
MemoryManager::malloc(size_t size)
{
	if (size == 0)
		return &dummy;
	while (size % 8)                    // Convert size to 8-byte aligned
		size++;
	size += SubBlock::overhead();           // Add space for end-of-block markers
	if (size < MinSubBlockSize)			// Avoid tiny blocks
		size = MinSubBlockSize;
	SubBlock* ptr;
	if (start_)
		ptr = start_;
	else
		ptr = link_new_block(size);
	if (ptr == 0)
		return 0;
	size_t recievedSize = ptr->size();
	while (size > recievedSize)
	{
		ptr = ptr->next();
		recievedSize = ptr->size();
		if (ptr == start_)
		{
			ptr = link_new_block(size);
			if (ptr == 0)
				return 0;
			recievedSize = ptr->size();
		}
	}
	if (recievedSize - size < MinSubBlockSize)		// Avoid tiny blocks
		size = recievedSize;
	if (size < recievedSize)
		ptr->split(size);
	start_ = ptr->unlink();
	return ptr->client_space();
}

MemoryManager::SubBlock*
MemoryManager::link_new_block(size_t size)
{
	if (default_blocksize_ < MinBlockSize)
	{
		report("MemoryManager is being asked to allocate memory before it is constructed.\n"
		       "Put MemoryManager.cpp first in the \"Link Order\" list.\n");
		aborting = true;
		exit(1);
	}
	size_t i = 0;
	while (i < masterlen_ && master_[i] != 0)
		++i;
	if (i == masterlen_)
	{
		size_t newsize = masterlen_;
		if (newsize == 0)
			newsize = 1;
		else
			newsize *= 2;
		Block** tmp = (Block**)sys_alloc(newsize*sizeof(Block*));
		if (tmp == 0)
			return 0;
		memcpy(tmp, master_, masterlen_*sizeof(Block*));
		sys_free(master_);
		master_ = tmp;
		for (size_t j = masterlen_; j < newsize; ++j)
			master_[j] = 0;
		masterlen_ = newsize;
	}
	while (size % 8)                    // Convert size to 8-byte aligned
		size++;
	size += Block::overhead();           // Add space for end-of-block markers
	if (size < default_blocksize_)
		size = default_blocksize_;
	master_[i] = (Block*)sys_alloc(size);
	if (master_[i] == 0)
		return 0;
	new (master_[i]) Block(size);
	Block::iterator b = master_[i]->begin();
	start_ = b->link(start_);
	return start_;
}

void
MemoryManager::free(void* ptr)
{
	if (ptr == 0 || ptr == &dummy)
		return;
	SubBlock* sb = (SubBlock*)((char*)ptr - 2*sizeof(size_t));
	sb->check_not_free();
	new (sb) SubBlock;
	sb->link(start_);
	sb = sb->merge_prev();
	start_ = sb->merge_next();
}

_CSTD::size_t
MemoryManager::get_size(void* ptr) const
{
	SubBlock* sb = (SubBlock*)((char*)ptr - 2*sizeof(size_t));
	return sb->size();
}

MemoryManager::Block::Block(_CSTD::size_t sz)
{
	size_ = sz;
	flag_ = '(';
	*(size_t*)((char*)this + sz - 2*sizeof(size_t)) = ')';
	*(size_t*)((char*)this + sz - sizeof(size_t)) = sz;
	new ((char*)this + 2*sizeof(size_t)) SubBlock(sz - overhead());
}

MemoryManager::SubBlock::SubBlock(size_t sz)
{
	size_ = sz;
	flag_ = '-';
	*(size_t*)((char*)this + sz - 2*sizeof(size_t)) = '-';
	*(size_t*)((char*)this + sz - sizeof(size_t)) = sz;
}

void
MemoryManager::SubBlock::split(size_t sz)
{
	size_t origsize = size();
	this->~SubBlock();
	new (this) SubBlock(sz);
	SubBlock* np = (SubBlock*)((char*)this + sz);
	new (np) SubBlock(origsize - sz);
	np->set_next(next());
	np->next()->set_prev(np);
	np->set_prev(this);
	set_next(np);
}

// insert this after p and return pointer this
// if p == 0, then link to self
MemoryManager::SubBlock*
MemoryManager::SubBlock::link(SubBlock* p)
{
	if (p == 0)
	{
		prev_ = next_ = this;
	}
	else
	{
		set_next(p->next());
		next()->set_prev(this);
		set_prev(p);
		p->set_next(this);
	}
	return this;
}

// return pointer to next
// if next == this, return 0
MemoryManager::SubBlock*
MemoryManager::SubBlock::unlink()
{
	prev()->set_next(next());
	next()->set_prev(prev());
	this->~SubBlock();
	if (next() == this)
		return 0;
	return next();
}

void
MemoryManager::SubBlock::check_not_free() const
{
	if (flag_ == '-')
	{
		report("MemoryManager freeing already free block at %p\n", (char*)this + 2*sizeof(size_t));
		aborting = true;
		exit(1);
	}
	if (flag_ != '+')
	{
		report("MemoryManager underwrite at %p\n", (char*)this + 2*sizeof(size_t));
		aborting = true;
		exit(1);
	}
	if (*(size_t*)((char*)this + size() - 2*sizeof(size_t)) != '+')
	{
		report("MemoryManager overwrite at %p\n", (char*)this + 2*sizeof(size_t));
		aborting = true;
		exit(1);
	}
}

MemoryManager::SubBlock*
MemoryManager::SubBlock::merge_prev()
{
	size_t sn = *(size_t*)((char*)this - 2*sizeof(size_t));
	if (sn == '-')
	{
		size_t prevsz = *(size_t*)((char*)this - sizeof(size_t));
		SubBlock* p = (SubBlock*)((char*)this - prevsz);
		p->prev()->set_next(p->next());
		p->next()->set_prev(p->prev());
		p->set_prev(prev());
		p->set_next(next());
		p->prev()->set_next(p);
		p->next()->set_prev(p);
		p->set_size(prevsz + size_);
		return p;
	}
	return this;
}

MemoryManager::SubBlock*
MemoryManager::SubBlock::merge_next()
{
	size_t sn = *(size_t*)((char*)this + size_ + sizeof(size_t));
	if (sn == '-')
	{
		SubBlock* p = (SubBlock*)((char*)this + size_);
		size_t nextsz = p->size();
		p->prev()->set_next(p->next());
		p->next()->set_prev(p->prev());
		set_size(size_ + nextsz);
	}
	return this;
}
