#ifndef MEMORYMANAGER_H
#define MEMORYMANAGER_H

// MemoryManager.h

// Howard E. Hinnant	 1995-2000
/*
	This code manages memory by allocated large blocks by malloc, and then allocating
	sub-blocks from the large blocks to satisfy memory requests.  Performance should
	be fairly acceptable considering all the error checking that is going on.
	
	On the performance side:  sub-blocks are allocated from large blocks using a first-fit
	algorithm.  The sub-blocks are stored in a circular, doubly linked list.  The start of
	the search starts where the last search left off.  New large blocks are automatically
	added when existing ones are depleted.  The size of the large block is set by the
	arguments to the MemoryManager constructor.  If more memory is asked for than the size
	of the large block, this is handled without problem.  Upon freeing, physically adjacent
	sub-blocks are merged into one larger sub-block.
	
	Error checking:  Each sub-block is bordered on the low and high ends with information
	about the sub-block.  Upon freeing, if this memory is overwritten, MemoryManager will
	print out a diagnostic (to the console) and exit.  Diagonostics are also printed if you
	try to delete the same pointer more than once, or if you try to use array delete on a
	non-array new'd	pointer or vice versa.  Also, upon program end, all sub-blocks that
	have not been freed	will generate an error message, optionally with the file name and
	line number of where they were new'd.
	
	To use:  Put this file in your project.  That's it.  You will now get out-
	of-bounds memory checking, and leak checking.  However any leaks you have will say
	something like:
	Leaking memory (8 bytes) at ??
	To get specific information	about where the leaked memory was new'd, replace your
	"new" with "NEW".  You will also need to #include "MemoryManager.h" in each file that
	you put "NEW" into.  Reports are then printed to the console window that look like
	"Leaking memory (8 bytes) at HelloWorld.cp, Line 8".
	
	If you are using the nothrow versions of new, and want information about where leaked
	memory was allocated, you must use the NEW_NOTHROW(std::nothrow) macro instead of
	the NEW macro.
	
	If there are no memory overwrites, double deletes, leaks, etc., then no output is
	produced from the MemoryManager.  It will be a silent friend.
	
	See MemoryManager.cpp on how to redirect errors to a log file.  The default error
	reporting goes to stderr (which may or may not be operational in your environment).
	
	Known bugs:  This code has not been made multi-thread safe.  To do this, the new
	and delete operators should be wrapped up in a "critical section".
	
	MemoryManager attempts to diagnose mismatched new/delete[] and new[]/delete.  When
	calling	delete[] on a new with a class that has a non-trivial destructor, at best 
	MemoryManager will miss-diagnose this as a "block underwrite" (client writes below
	allowed address).  At worst, your app will crash before it even gets to MemoryManager's
	operator delete [] (in which case it would also crash without MemoryManager).
	
	This code inspired by Donald Knuth's "Fundamental Algorithms."
*/

#include <new>
#include <iterator>
#include <cstdio>
#include <cstdlib>

void* operator new     (_CSTD::size_t size)                         throw(_STD::bad_alloc);
void* operator new     (_CSTD::size_t size, const _STD::nothrow_t&) throw();
void  operator delete  (void* ptr)                                  throw();
void  operator delete  (void* ptr, const _STD::nothrow_t&)          throw();
void* operator new   [](_CSTD::size_t size)                         throw(_STD::bad_alloc);
void* operator new   [](_CSTD::size_t size, const _STD::nothrow_t&) throw();
void  operator delete[](void* ptr)                                  throw();
void  operator delete[](void* ptr, const _STD::nothrow_t&)          throw();

void* operator new     (_CSTD::size_t,                         const char*, int) throw(_STD::bad_alloc);
void* operator new     (_CSTD::size_t, const _STD::nothrow_t&, const char*, int) throw();
void  operator delete  (void*,                                 const char*, int) throw();
void  operator delete  (void*, const _STD::nothrow_t&,         const char*, int) throw();
void* operator new   [](_CSTD::size_t,                         const char*, int) throw(_STD::bad_alloc);
void* operator new   [](_CSTD::size_t, const _STD::nothrow_t&, const char*, int) throw();
void  operator delete[](void*,                                 const char*, int) throw();
void  operator delete[](void*, const _STD::nothrow_t&,         const char*, int) throw();

namespace Metrowerks
{

class MemoryManager
{
public:
	MemoryManager(_CSTD::size_t blocksize = 65536u);
	~MemoryManager();
	void* malloc(_CSTD::size_t size);
	void free(void* ptr);
private:
	class Block;
	class SubBlock;
	_CSTD::size_t default_blocksize_;
	_CSTD::size_t masterlen_;
	Block** master_;
	SubBlock* start_;

	MemoryManager(const MemoryManager&);             // Not defined
	MemoryManager& operator=(const MemoryManager&);  // Not defined
	void screamIfInUse() const;
	_CSTD::size_t get_size(void* p) const;
	SubBlock* link_new_block(_CSTD::size_t size);

	friend void* _CSTD::realloc(void* ptr, _CSTD::size_t size);
};

class MemoryManager::Block
{
public:
	class iterator;
	Block(_CSTD::size_t sz);
	_CSTD::size_t size() const;
	static _CSTD::size_t overhead();
	iterator begin();
	iterator end();
private:
	_CSTD::size_t size_;
	_CSTD::size_t flag_;

	Block(const Block&);              // Not defined
	Block& operator=(const Block&);   // Not defined
};

class MemoryManager::SubBlock
{
public:
	SubBlock();
	SubBlock(_CSTD::size_t sz);
	~SubBlock();
	_CSTD::size_t size() const;
	SubBlock* next() const;
	SubBlock* prev() const;
	void split(_CSTD::size_t sz);
	SubBlock* link(SubBlock* p);  // insert this after p and return pointer this
	SubBlock* unlink();  // return pointer to next
	void* client_space() const;
	bool not_free() const;
	bool free() const;
	void check_not_free() const;
	SubBlock* merge_prev();
	SubBlock* merge_next();
	static _CSTD::size_t overhead();
private:
	_CSTD::size_t size_;
	_CSTD::size_t flag_;
	SubBlock* prev_;
	SubBlock* next_;

	void set_next(SubBlock* n);
	void set_prev(SubBlock* p);
	void set_size(_CSTD::size_t sz);
	SubBlock(const SubBlock&);              // Not defined
	SubBlock& operator=(const SubBlock&);   // Not defined
};

class MemoryManager::Block::iterator
	: public _STD::iterator<_STD::bidirectional_iterator_tag, MemoryManager::SubBlock>
{
public:
	iterator(SubBlock*);
	iterator& operator++();
	iterator operator++(int);
	iterator& operator--();
	iterator operator--(int);
	SubBlock& operator*() const;
	SubBlock* operator->() const;
	friend bool operator == (const iterator lhs, const iterator rhs);
	friend bool operator != (const iterator lhs, const iterator rhs);
private:
	SubBlock* ptr_;
};

inline
bool
operator == (const MemoryManager::Block::iterator lhs, const MemoryManager::Block::iterator rhs)
{
	return static_cast<bool>(lhs.ptr_ == rhs.ptr_);
}

inline
bool
operator != (const MemoryManager::Block::iterator lhs, const MemoryManager::Block::iterator rhs)
{
	return static_cast<bool>(lhs.ptr_ != rhs.ptr_);
}

inline
MemoryManager::Block::iterator::iterator(SubBlock* p)
	: ptr_(p)
{
}

inline
MemoryManager::Block::iterator&
MemoryManager::Block::iterator::operator++()
{
	ptr_ = (SubBlock*)((char*)ptr_ + ptr_->size());
	return *this;
}

inline
MemoryManager::Block::iterator
MemoryManager::Block::iterator::operator++(int)
{
	iterator tmp(*this);
	++(*this);
	return tmp;
}

inline
MemoryManager::Block::iterator&
MemoryManager::Block::iterator::operator--()
{
	_CSTD::size_t p = *(_CSTD::size_t*)((char*)ptr_ - sizeof(_CSTD::size_t));
	ptr_ = (SubBlock*)((char*)ptr_ - p);
	return *this;
}

inline
MemoryManager::Block::iterator
MemoryManager::Block::iterator::operator--(int)
{
	iterator tmp(*this);
	--(*this);
	return tmp;
}

inline
MemoryManager::SubBlock&
MemoryManager::Block::iterator::operator*() const
{
	return *ptr_;
}

inline
MemoryManager::SubBlock*
MemoryManager::Block::iterator::operator->() const
{
	return ptr_;
}

inline
_CSTD::size_t
MemoryManager::Block::size() const
{
	return size_;
}

inline
_CSTD::size_t
MemoryManager::Block::overhead()
{
	return 4*sizeof(_CSTD::size_t);
}

inline
MemoryManager::Block::iterator
MemoryManager::Block::begin()
{
	return (SubBlock*)((char*)this + 2*sizeof(_CSTD::size_t));
}

inline
MemoryManager::Block::iterator
MemoryManager::Block::end()
{
	return (SubBlock*)((char*)this + size() - 2*sizeof(_CSTD::size_t));
}

inline
MemoryManager::SubBlock::SubBlock()
{
	flag_ = '-';
	*(_CSTD::size_t*)((char*)this + size() - 2*sizeof(_CSTD::size_t)) = '-';
}

inline
MemoryManager::SubBlock::~SubBlock()
{
	flag_ = '+';
	*(_CSTD::size_t*)((char*)this + size() - 2*sizeof(_CSTD::size_t)) = '+';
}

inline
MemoryManager::SubBlock*
MemoryManager::SubBlock::next() const
{
	return next_;
}

inline
void
MemoryManager::SubBlock::set_next(SubBlock* n)
{
	next_ = n;
}

inline
MemoryManager::SubBlock*
MemoryManager::SubBlock::prev() const
{
	return prev_;
}

inline
void
MemoryManager::SubBlock::set_prev(SubBlock* p)
{
	prev_ = p;
}

inline
_CSTD::size_t
MemoryManager::SubBlock::size() const
{
	return size_;
}

inline
void*
MemoryManager::SubBlock::client_space() const
{
	return (void*)&prev_;
}

inline
bool
MemoryManager::SubBlock::free() const
{
	return static_cast<bool>(flag_ == '-');
}

inline
bool
MemoryManager::SubBlock::not_free() const
{
	return static_cast<bool>(!free());
}

inline
_CSTD::size_t
MemoryManager::SubBlock::overhead()
{
	return 4*sizeof(_CSTD::size_t);
}

inline
void
MemoryManager::SubBlock::set_size(_CSTD::size_t sz)
{
	size_ = *(_CSTD::size_t*)((char*)this + sz - sizeof(_CSTD::size_t)) = sz;
}

}

#define NEW new(__FILE__, __LINE__)
#define NEW_NOTHROW(NOTHROW) new(NOTHROW, __FILE__, __LINE__)

#endif
