#ifndef MCS_BIN_H
#define MCS_BIN_H

#include "MCS_Bounds.h"
#include "MCS_Exceptions.h"
#include "MCS_Matrix.h"
#include "MCS_Triple.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// bin class
template<class T> 
class Bin
{
	public:
		Bin(const Bounds<T>& bds);
		Bin(const Bounds<T>& bds,const Int32& buf_siz);
		
		void add(const T& x,const T& z);
		void add(const Vector<T>& x,const Vector<T>& z);
		
		void clear();
		
		const Bounds<T>& bounds() const	{ return bounds_; }
		
		const Vector<T>& bins();
		const Vector<T>& weights();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x,const T& z);
		void finalize();
		
		Bounds<T>	bounds_;
		Vector<T>	bins_,
					weights_;
		
		Vector< std::pair<T,T> >	buf_;	
		Index<1>					buf_idx_;
		bool						pending_;
		
		static const Index<1>		DEFAULT_BUF_SIZ_;
};

// implementation
template<class T> const Index<1> Bin<T>::DEFAULT_BUF_SIZ_ = 1000;

template<class T> 
Bin<T>::Bin(const Bounds<T>& bds) : 
	bounds_(bds),
	bins_(bds.range(),T()),
	weights_(bds.range()),
	buf_(IndexRange<1>(1,DEFAULT_BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
Bin<T>::Bin(const Bounds<T>& bds,const Int32& buf_siz) : 
	bounds_(bds),
	bins_(bds.range(),T()),
	weights_(bds.range()),
	buf_(IndexRange<1>(1,buf_siz)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void Bin<T>::add(const T& x,const T& z)
{
	add_to_buffer(x,z);
}

template<class T> 
inline void Bin<T>::add(const Vector<T>& x,const Vector<T>& z)
{
	if (x.range() != z.range()) throw Err();
	
	for (Index<1> i=x.first();i<=x.last();i++)
		add_to_buffer(x[i],z[i]);
}

template<class T> 
inline void Bin<T>::clear()
{
	// clear data
	bins_ = weights_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Vector<T>& Bin<T>::bins() 
{
	if (pending_) finalize();
	
	return bins_;
}

template<class T> 
inline const Vector<T>& Bin<T>::weights()
{
	if (pending_) finalize();
	
	return weights_;
}

template<class T> 
inline void Bin<T>::add_to_buffer(const T& x,const T& z)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x,z);
	}
	else
	{
		buf_[buf_idx_].first = x;
		buf_[buf_idx_].second = z;
		buf_idx_++;
	}
	
	pending_ = true;
}

template<class T> 
void Bin<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (isfinite(buf_[i].first) && isfinite(buf_[i].second)) 
		{
			const Index<1>	idx = bounds_.find(buf_[i].first);
							
			bins_[idx] += buf_[i].second;
			weights_[idx]++;
		}
	}
	
	pending_ = false;
	buf_idx_ = buf_.first();
}

// 2D bin class
template<class T> 
class Bin2D
{
	public:
		Bin2D(const Bounds<T>& xbds,const Bounds<T>& ybds);
		Bin2D(const Bounds<T>& xbds,const Bounds<T>& ybds,const Int32& buf_siz);
		
		void add(const T& x,const T& y,const T& z);
		void add(const Vector<T>& x,const Vector<T>& y,const Vector<T>& z);
		
		void clear();
		
		const Bounds<T>& x_bounds() const		{ return x_bounds_; }
		const Bounds<T>& y_bounds() const		{ return y_bounds_; }
		
		const Matrix<T>& bins();
		const Matrix<T>& weights();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x,const T& y,const T& z);
		void finalize();
		
		Bounds<T>	x_bounds_,
					y_bounds_;
		Matrix<T>	bins_,
					weights_;
		
		Vector< triple<T> >		buf_;	
		Index<1>				buf_idx_;
		bool					pending_;
		
		static const Index<1>	DEFAULT_BUF_SIZ_;
};

// implementation
template<class T> const Index<1> Bin2D<T>::DEFAULT_BUF_SIZ_ = 1000;

template<class T> 
Bin2D<T>::Bin2D(const Bounds<T>& xbds,const Bounds<T>& ybds) : 
	x_bounds_(xbds),y_bounds_(ybds),
	bins_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	weights_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	buf_(IndexRange<1>(1,DEFAULT_BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
Bin2D<T>::Bin2D(const Bounds<T>& xbds,const Bounds<T>& ybds,const Int32& buf_siz) : 
	x_bounds_(xbds),y_bounds_(ybds),
	bins_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	weights_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	buf_(IndexRange<1>(1,buf_siz)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void Bin2D<T>::add(const T& x,const T& y,const T& z)
{
	add_to_buffer(x,y,z);
}

template<class T> 
inline void Bin2D<T>::add(const Vector<T>& x,const Vector<T>& y,const Vector<T>& z)
{
	if (x.range() != z.range() || y.range() != z.range()) throw Err();
	
	for (Index<1> i=z.first();i<=z.last();i++)
		add_to_buffer(x[i],y[i],z[i]);
}

template<class T> 
inline void Bin2D<T>::clear()
{
	// clear data
	bins_ = weights_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Matrix<T>& Bin2D<T>::bins() 
{
	if (pending_) finalize();
	
	return bins_;
}

template<class T> 
inline const Matrix<T>& Bin2D<T>::weights() 
{
	if (pending_) finalize();
	
	return weights_;
}

template<class T> 
inline void Bin2D<T>::add_to_buffer(const T& x,const T& y,const T& z)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x,y,z);
	}
	else 
	{
		buf_[buf_idx_].first = x;
		buf_[buf_idx_].second = y;
		buf_[buf_idx_].third = z;
		buf_idx_++;
	}
	
	pending_ = true;
}

template<class T> 
void Bin2D<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (isfinite(buf_[i].first) && isfinite(buf_[i].second) && isfinite(buf_[i].third))
		{
			const Index<2>	idx(x_bounds_.find(buf_[i].first),
								y_bounds_.find(buf_[i].second));
							
			bins_(idx) += buf_[i].third;
			weights_(idx)++;
		}
	}

	pending_ = false;
		buf_idx_ = buf_.first();
}

} // namespace mcs

#endif // MCS_BIN_H