#ifndef MCS_HISTOGRAM_H
#define MCS_HISTOGRAM_H

#include "MCS_Bounds.h"
#include "MCS_Exceptions.h"
#include "MCS_Matrix.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// histogram class
template<class T> 
class Histogram
{
	public:
		Histogram(const Bounds<T>& bds);
		
		void add(const T& x);
		void add(const Vector<T>& x);
		
		void clear();
		
		const Bounds<T>& bounds() const	{ return bounds_; }
		
		const Vector<Uint32>& histogram();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x);
		void finalize();
		
		Bounds<T>		bounds_;
		Vector<Uint32>	hist_;
		
		Vector<T>		buf_;	
		Index<1>		buf_idx_;
		bool			pending_;
		
		static const Index<1>	BUF_SIZ_;
};

// implementation
template<class T> const Index<1> Histogram<T>::BUF_SIZ_ = 1000;

template<class T> 
Histogram<T>::Histogram(const Bounds<T>& bds) : 
	bounds_(bds),
	hist_(bds.range()),
	buf_(IndexRange<1>(1,BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void Histogram<T>::add(const T& x)
{
	add_to_buffer(x);
}

template<class T> 
inline void Histogram<T>::add(const Vector<T>& x)
{
	for (Index<1> i=x.first();i<=x.last();i++)
		add_to_buffer(x[i]);
}

template<class T> 
inline void Histogram<T>::clear()
{
	// clear histogram
	hist_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Vector<Uint32>& Histogram<T>::histogram() 
{
	if (pending_) finalize();
	
	return hist_;
}

template<class T> 
inline void Histogram<T>::add_to_buffer(const T& x)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x);
	}
	else
	{
		buf_[buf_idx_++] = x;
	}
	
	pending_ = true;
}

template<class T> 
void Histogram<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (!isfinite(buf_[i])) continue;
		
		const Index<1>	idx = bounds_.find(buf_[i]);
						
		hist_[idx]++;
	}
	
	pending_ = false;
	buf_idx_ = buf_.first();
}

// 2D histogram class
template<class T> 
class Histogram2D
{
	public:
		Histogram2D(const Bounds<T>& xbds,const Bounds<T>& ybds);
		
		void add(const T& x,const T& y);
		void add(const Vector<T>& x,const Vector<T>& y);
		
		void clear();
		
		const Bounds<T>& x_bounds() const		{ return x_bounds_; }
		const Bounds<T>& y_bounds() const		{ return y_bounds_; }
		
		const Matrix<Uint32>& histogram();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x,const T& y);
		void finalize();
		
		Bounds<T>		x_bounds_,
						y_bounds_;
		Matrix<Uint32>	hist_;
		
		Vector< std::pair<T,T> >	buf_;	
		Index<1>					buf_idx_;
		bool						pending_;
		
		static const Index<1>		BUF_SIZ_;
};

// implementation
template<class T> const Index<1> Histogram2D<T>::BUF_SIZ_ = 1000;

template<class T> 
Histogram2D<T>::Histogram2D(const Bounds<T>& xbds,const Bounds<T>& ybds) : 
	x_bounds_(xbds),
	y_bounds_(ybds),
	hist_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	buf_(IndexRange<1>(1,BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void Histogram2D<T>::add(const T& x,const T& y)
{
	add_to_buffer(x,y);
}

template<class T> 
inline void Histogram2D<T>::add(const Vector<T>& x,const Vector<T>& y)
{
	if (x.range() != y.range()) throw Err();
	
	for (Index<1> i=x.first();i<=x.last();i++)
		add_to_buffer(x[i],y[i]);
}

template<class T> 
inline void Histogram2D<T>::clear()
{
	// clear histogram
	hist_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Matrix<Uint32>& Histogram2D<T>::histogram() 
{
	if (pending_) finalize();
	
	return hist_;
}

template<class T> 
inline void Histogram2D<T>::add_to_buffer(const T& x,const T& y)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x,y);
	}
	else
	{
		buf_[buf_idx_].first = x;
		buf_[buf_idx_].second = y;
		buf_idx_++;
	}
	
	pending_ = true;
}

template<class T> 
void Histogram2D<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (!isfinite(buf_[i].first) || !isfinite(buf_[i].second)) continue;
		
		const Index<2>	idx(x_bounds_.find(buf_[i].first),
							y_bounds_.find(buf_[i].second));
						
		hist_(idx)++;
	}

	pending_ = false;
	buf_idx_ = buf_.first();
}

} // namespace mcs

#endif // MCS_HISTOGRAM_H