#ifndef MCS_INTEGRATION_H
#define MCS_INTEGRATION_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"

namespace mcs {

namespace num {

template<class T> class FuncRefiner
{
	public:
		typedef T (*IntegrandPtr)(T);
		typedef T (*RefinerPtr)(IntegrandPtr,T,T,int);
		
		FuncRefiner(const RefinerPtr);
		virtual ~FuncRefiner() { }
		
		T operator()(const IntegrandPtr,const T&,const T&,const int&) const;
		
		const RefinerPtr   refiner() const		{ return ref_; }
		
	private:
		const RefinerPtr	ref_;
};

template<class T> inline FuncRefiner<T>::FuncRefiner(const RefinerPtr ref) :
	ref_(ref)
{ }

template<class T> inline T FuncRefiner<T>::operator()(const IntegrandPtr itg,const T& lo,const T& hi,const int& n) const
{
	return (*ref_)(*itg,lo,hi,n);
}

// 1D integrator
template<class T> class Integrator : public Functor2<const T&,const T&,T>
{ };

template<class T> class FuncIntegrator : public Integrator<T>
{
	public:
		typedef T (*IntegrandPtr)(T);
		typedef T (*RefinerPtr)(IntegrandPtr,T,T,int);
		typedef T (*IntegratorPtr)(IntegrandPtr,T,T,RefinerPtr);
		
		FuncIntegrator(const IntegrandPtr,const IntegratorPtr,const RefinerPtr);
		virtual ~FuncIntegrator() { }
		
		T operator()(const T&,const T&) const;
	
	private:
		static const EnsureReal<T>	dummy_;
		
		const IntegrandPtr	itg_;
		const IntegratorPtr	itr_;
		const RefinerPtr 	ref_;
};

template<class T> inline FuncIntegrator<T>::FuncIntegrator(const IntegrandPtr itg,const IntegratorPtr itr,const RefinerPtr ref) :
	itg_(itg),itr_(itr),ref_(ref)
{ }

template<class T> inline T FuncIntegrator<T>::operator()(const T& lo,const T& hi) const
{
	return (*itr_)(*itg_,lo,hi,*ref_);
}

// closed interval trapezoid rule
template<class T> class TrapezoidIntegrator : public FuncIntegrator<T>
{
	public:
		TrapezoidIntegrator(const FuncIntegrator<T>::IntegrandPtr);
		~TrapezoidIntegrator() { }
};

template<class T> inline TrapezoidIntegrator<T>::TrapezoidIntegrator(const FuncIntegrator<T>::IntegrandPtr itg) :
	FuncIntegrator<T>(itg,NR::FN<T>::qtrapr,NR::FN<T>::trapzd)
{ }

// closed interval Simpson's method
template<class T> class SimpsonIntegrator : public FuncIntegrator<T>
{
	public:
		SimpsonIntegrator(const FuncIntegrator<T>::IntegrandPtr);
		~SimpsonIntegrator() { }
};

template<class T> inline SimpsonIntegrator<T>::SimpsonIntegrator(const FuncIntegrator<T>::IntegrandPtr itg) :
	FuncIntegrator<T>(itg,NR::FN<T>::qsimpr,NR::FN<T>::trapzd)
{ }

// closed interval Romberg method
template<class T> class RombergIntegrator : public FuncIntegrator<T>
{
	public:
		RombergIntegrator(const FuncIntegrator<T>::IntegrandPtr);
		~RombergIntegrator() { }
};

template<class T> inline RombergIntegrator<T>::RombergIntegrator(const FuncIntegrator<T>::IntegrandPtr itg) :
	FuncIntegrator<T>(itg,NR::FN<T>::qrombr,NR::FN<T>::trapzd)
{ }

} // namespace num

} // namespace mcs

#endif // MCS_INTEGRATION_H