#ifndef MCS_INTERPOLATION_H
#define MCS_INTERPOLATION_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"
#include "MCS_RPFastInterpolation.h"

namespace mcs {

namespace num {

// 1D interpolator implementation
template<class T> class InterpolatorImp
{
	public:
		typedef std::auto_ptr< InterpolatorImp<T> >	Ptr;
		
		virtual ~InterpolatorImp() { };
		
		virtual T interpolate(const T&) const = 0;
		virtual Vector<T> interpolate(const Vector<T>&) const = 0;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)
		
	private:
		static const EnsureNumeric<T>	dummy_;
};

template<class T> const EnsureNumeric<T> InterpolatorImp<T>::dummy_;

// linear 1D interpolation
template<class T> class LinearInterpolatorImp : public InterpolatorImp<T>
{
	public:
		~LinearInterpolatorImp() { }
		
		T interpolate(const T&) const;
		Vector<T> interpolate(const Vector<T>&) const;
		
		static typename InterpolatorImp<T>::Ptr create(const Vector<T>&,const Vector<T>&);
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		LinearInterpolatorImp(const Vector<T>&,const Vector<T>&);
		
		Vector<T>	x_,y_;
};

template<class T> inline LinearInterpolatorImp<T>::LinearInterpolatorImp(const Vector<T>& x,const Vector<T>& y) :
	x_(x),y_(y)
{ 
	if (x.range() != y.range()) throw Err();
}

template<class T> inline T LinearInterpolatorImp<T>::interpolate(const T& xp) const
{
	const Index<1>	i = locateIndex(x_,xp);

	if (i == x_.first()-1)
		return linearInterpolate(xp,x_[i+1],y_[i+1],x_[i+2],y_[i+2]);
	else if (i== x_.last())
		return linearInterpolate(xp,x_[i-1],y_[i-1],x_[i],y_[i]);
	else
		return linearInterpolate(xp,x_[i],y_[i],x_[i+1],y_[i+1]);
}

template<class T> inline Vector<T> LinearInterpolatorImp<T>::interpolate(const Vector<T>& xp) const
{
	Vector<T>	yp(xp.range());	

	for (Index<1> i=xp.first();i<=xp.last();i++)
	{
		const Index<1>	ii = huntIndex(x_,xp[i]);

		if (ii == x_.first()-1)
			yp[i] = linearInterpolate(xp[i],x_[ii+1],y_[ii+1],x_[ii+2],y_[ii+2]);
		else if (ii== x_.last())
			yp[i] = linearInterpolate(xp[i],x_[ii-1],y_[ii-1],x_[ii],y_[ii]);
		else
			yp[i] = linearInterpolate(xp[i],x_[ii],y_[ii],x_[ii+1],y_[ii+1]);
	}
	
	return yp;
}

template<class T> inline typename InterpolatorImp<T>::Ptr LinearInterpolatorImp<T>::create(const Vector<T>& x,const Vector<T>& y) 
{ 
	return InterpolatorImp<T>::Ptr(new LinearInterpolatorImp<T>(x,y)); 
}

// spline 1D interpolation
template<class T> class SplineInterpolatorImp : public InterpolatorImp<T>
{
	public:
		~SplineInterpolatorImp();
		
		T interpolate(const T&) const;
		Vector<T> interpolate(const Vector<T>&) const;

		static typename InterpolatorImp<T>::Ptr create(const Vector<T>&,const Vector<T>&,const T&,const T&);
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		SplineInterpolatorImp(const Vector<T>&,const Vector<T>&,const T&,const T&);
		
		Vector<T>			x_,y_;
		mutable T			lod_,hid_;
		Vector<T>			y2data_;
		
		mutable NRVector<T>	*xptr_,
							*yptr_,
							*y2ptr_;
};

template<class T> inline SplineInterpolatorImp<T>::SplineInterpolatorImp(const Vector<T>& x,const Vector<T>& y,const T& loderiv,const T& hideriv) :
	x_(x),y_(y),
	y2data_(x.range()),lod_(loderiv),hid_(hideriv)
{
	if (x.range() != y.range()) throw Err();
	
	// check that input abscissa is monotonic
	if (!isMonotonic(x)) throw NumericException();
	
	// if x is monotonically decreasing, reverse vectors
	if (nthValue(x_,2) < nthValue(x_,1))
	{
		reverse(x_);
		reverse(y_);
	}	
		
	if (!isfinite(lod_)) lod_ = 1.0e38;
	if (!isfinite(hid_)) hid_ = 1.0e38;
	
	xptr_ = new NRVector<T>(x_);
	yptr_ = new NRVector<T>(y_);
	y2ptr_ = new NRVector<T>(y2data_);
	
	NR::FN<T>::spline(xptr_->data(),yptr_->data(),y2ptr_->dim(),lod_,hid_,y2ptr_->data());
}

template<class T> inline SplineInterpolatorImp<T>::~SplineInterpolatorImp()
{
	delete xptr_;
	delete yptr_;
	delete y2ptr_;
}

template<class T> inline T SplineInterpolatorImp<T>::interpolate(const T& xp) const
{
	T	yp;
	
	NR::FN<T>::splint(xptr_->data(),yptr_->data(),y2ptr_->data(),y2ptr_->dim(),xp,&yp);
	
	return yp;
}

template<class T> inline Vector<T> SplineInterpolatorImp<T>::interpolate(const Vector<T>& xp) const
{
	Vector<T>	yp(xp.range());	

	for (Index<1> i=xp.first();i<=xp.last();i++)
	{
		NR::FN<T>::splint(xptr_->data(),yptr_->data(),y2ptr_->data(),y2ptr_->dim(),xp[i],&yp[i]);
	}
	
	return yp;
}

template<class T> inline typename InterpolatorImp<T>::Ptr SplineInterpolatorImp<T>::create(const Vector<T>& x,const Vector<T>& y,const T& loderiv,const T& hideriv) 
{ 
	return InterpolatorImp<T>::Ptr(new SplineInterpolatorImp<T>(x,y,loderiv,hideriv)); 
}

// Rybicki and Press 1D power spectrum interpolation
template<class T> class RPInterpolatorImp : public InterpolatorImp<T>
{
	public:
		~RPInterpolatorImp();
		
		T interpolate(const T&) const;
		Vector<T> interpolate(const Vector<T>&) const;

		static typename InterpolatorImp<T>::Ptr create(const Vector<T>&,const Vector<T>&,const Vector<T>&,const T&,const T&,bool = true);
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		RPInterpolatorImp(const Vector<T>&,const Vector<T>&,const Vector<T>&,const T&,const T&,bool = true);
		
		Vector<T>	x_,y_,sig_;
		T			psig_,w_;
		bool		unbiased_;
};

template<class T> inline RPInterpolatorImp<T>::RPInterpolatorImp(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,const T& psig,const T& w,bool unbiased) :
	x_(x),y_(y),sig_(sig),
	psig_(psig),w_(w),unbiased_(unbiased)
{
	if (x.range() != y.range() || x.range() != sig.range()) throw Err();

	// make sure data are increasing
	// check that input abscissa is monotonic
	if (!isMonotonic(x_)) throw Err();
	
	// if x is monotonically decreasing, reverse vectors
	if (nthValue(x_,2) < nthValue(x_,1))
	{
		reverse(x_);
		reverse(y_);
		reverse(sig_);
	}	
		
}

template<class T> inline RPInterpolatorImp<T>::~RPInterpolatorImp()
{ }

template<class T> inline T RPInterpolatorImp<T>::interpolate(const T& xp) const
{
	return firstValue(remap(x_,y_,sig_,psig_,w_,unbiased_,Vector<T>(IndexRange<1>(1,1),xp)));
}

template<class T> inline Vector<T> RPInterpolatorImp<T>::interpolate(const Vector<T>& xp) const
{
	return remap(x_,y_,sig_,psig_,w_,unbiased_,xp);
}

template<class T> inline typename InterpolatorImp<T>::Ptr RPInterpolatorImp<T>::create(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,const T& psig,const T& w,bool unbiased) 
{ 
	return InterpolatorImp<T>::Ptr(new RPInterpolatorImp<T>(x,y,sig,psig,w,unbiased)); 
}

// support routines for finding indices straddling sought value
template<class T> inline Index<1> locateIndex(const Vector<T>& vec,const T& val)
{
	NRVector<T>	vnr(const_cast<Vector<T>&>(vec));
	
	return Index<1>(locateIndex(vnr,val)-vnr.offset()+vec.offset());
}

template<class T> inline Int32 locateIndex(NRBaseVector<T>& vec,const T& val)
{
	unsigned long	idx;
	
	NR::FN<T>::locate(vec.data(),vec.dim(),val,&idx);
		
	return idx;
}

template<class T> inline Index<1> huntIndex(const Vector<T>& vec,const T& val)
{
	NRVector<T>	vnr(const_cast<Vector<T>&>(vec));
	
	return Index<1>(huntIndex(vnr,val)-vnr.offset()+vec.offset());
}

template<class T> inline Index<1> huntIndex(NRBaseVector<T>& vec,const T& val)
{
	unsigned long	idx;
	
	NR::FN<T>::hunt(vec.data(),vec.dim(),val,&idx);
		
	return idx;
}

// 1D linear interpolation for a single point
template<class T> T linearInterpolate(const T& x,const T& x1,const T& y1,const T& x2,const T& y2)
{
	return y1 + (y2-y1)*((x-x1)/(x2-x1));
}

// 2D bilinear interpolation for a single point
//	-> lo is lower left, hi is upper right
//	-> z is ordered LL, LR, UR, UL
template<class T> inline T bilinearInterpolate(const FixedVector<T,4>& z,
										const FixedVector<T,2>& lo,
						  				const FixedVector<T,2>& hi,
				   		   				const T& x_val,const T& y_val)
{
	const T	d1=hi[0]-lo[0],
			d2=hi[1]-lo[1],
			t=(x_val-lo[0])/d1,
			u=(y_val-lo[1])/d2,
			omt=1-t,
			omu=1-u;

	return omt*omu*z[0]+t*omu*z[1]+t*u*z[2]+omt*u*z[3];
}

template<class T> class Interpolator : public Functor1<const T&,T>,public Functor1< const Vector<T>&, Vector<T> >
{ 
	public:
		Interpolator(typename InterpolatorImp<T>::Ptr);
		virtual ~Interpolator() { }
		
		T operator()(const T&) const;
		Vector<T> operator()(const Vector<T>&) const;
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		static const EnsureReal<T>	dummy_;
		
		typename InterpolatorImp<T>::Ptr	impptr_;
};

template<class T> inline Interpolator<T>::Interpolator(typename InterpolatorImp<T>::Ptr impptr) :
	impptr_(impptr)
{ }

template<class T> inline T Interpolator<T>::operator()(const T& xint) const
{
	return impptr_->interpolate(xint);
}

template<class T> inline Vector<T> Interpolator<T>::operator()(const Vector<T>& xint) const
{
	return impptr_->interpolate(xint);
}

template<class T> class LinearInterpolator : public Interpolator<T>
{
	public:
		LinearInterpolator(const Vector<T>&,const Vector<T>&);
		~LinearInterpolator() { }
};

template<class T> inline LinearInterpolator<T>::LinearInterpolator(const Vector<T>& x,const Vector<T>& y) :
	Interpolator<T>(LinearInterpolatorImp<T>::create(x,y))
{ }

template<class T> class SplineInterpolator : public Interpolator<T>
{
	public:
		SplineInterpolator(const Vector<T>&,const Vector<T>&,const T& = _plus_Inf_,const T& = _plus_Inf_);
		~SplineInterpolator() { }
};

template<class T> inline SplineInterpolator<T>::SplineInterpolator(const Vector<T>& x,const Vector<T>& y,const T& loderiv,const T& hideriv) :
	Interpolator<T>(SplineInterpolatorImp<T>::create(x,y,loderiv,hideriv))
{ }

template<class T> class RPInterpolator : public Interpolator<T>
{
	public:
		RPInterpolator(const Vector<T>&,const Vector<T>&,const Vector<T>&,const T&,const T&,bool = true);
		~RPInterpolator() { }
};

template<class T> inline RPInterpolator<T>::RPInterpolator(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,const T& psig,const T& w,bool unbiased) :
	Interpolator<T>(RPInterpolatorImp<T>::create(x,y,sig,psig,w,unbiased))
{ }

} // namespace num

} // namespace mcs

#endif // MCS_INTERPOLATION_H