#ifndef MCS_MINIMIZATION_H
#define MCS_MINIMIZATION_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"

namespace mcs {

namespace num {

// NR 10.1 Golden section search in one dimension
template<class T> FixedVector<T,3> bracketMinimum(const Functor1<const T&,T>& func,const T& lo,const T& hi)
{
	static const T	GOLD = 1.618034,
					GLIMIT = 100.0,
					TINY = 1.0e-20;

	T 	ulim,u,r,q,fu,dum;
	
	FixedVector<T,3>	x,f;
	
	x[0] = lo;
	x[1] = hi;

	f[0] = func(x[0]);
	f[1] = func(x[1]);
	
	if (f[1] > f[0]) 
	{
		NR::shft(dum,x[0],x[1],dum);
		NR::shft(dum,f[1],f[0],dum);
	}
	
	x[2] = x[1]+GOLD*(x[1]-x[0]);
	f[2] = func(x[2]);
	
	while (f[1] > f[2]) 
	{
		r = (x[1]-x[0])*(f[1]-f[2]);
		q = (x[1]-x[2])*(f[1]-f[0]);
		u = x[1]-((x[1]-x[2])*q-(x[1]-x[0])*r)/(2*std::sign(std::max(std::abs(q-r),TINY),q-r));
		ulim = x[1]+GLIMIT*(x[2]-x[1]);
		
		if ((x[1]-u)*(u-x[2]) > 0) 
		{
			fu = func(u);
			
			if (fu < f[2]) 
			{
				x[0] = x[1];
				x[1] = u;
				f[0] = f[1];
				f[1] = fu;
				return x;
			} 
			else if (fu > f[1]) 
			{
				x[2] = u;
				f[2] = fu;
				return x;
			}
			
			u = x[2]+GOLD*(x[2]-x[1]);
			fu = func(u);
		} 
		else if ((x[2]-u)*(u-ulim) > 0) 
		{
			fu = func(u);
			
			if (fu < f[2]) 
			{
				NR::shft(x[1],x[2],u,x[2]+GOLD*(x[2]-x[1]));
				NR::shft(f[1],f[2],fu,func(u));
			}
		} 
		else if ((u-ulim)*(ulim-x[2]) >= 0) 
		{
			u = ulim;
			fu = func(u);
		} 
		else 
		{
			u = x[2]+GOLD*(x[2]-x[1]);
			fu = func(u);
		}
		
		NR::shft(x[0],x[1],x[2],u);
		NR::shft(f[0],f[1],f[2],fu);
	}
	
	return x;
}

// 1D minimizer - takes bracketing points as arguments and returns minimum location and value
template<class T> class Minimizer : public Functor1<const FixedVector<T,3>&,FixedVector<T,2> >
{
	public:
		typedef Functor1<const T&,T> MinFunctor;
		
		Minimizer(const MinFunctor& func);
		Minimizer(const MinFunctor& func,const T& tol) : func_(func),tol_(tol) { }
		virtual ~Minimizer() { }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	protected:
		const MinFunctor& functor() const	{ return func_; }
		const T& tolerance() const			{ return tol_; }
		
	private:
		static const EnsureReal<T>	dummy_;
	
		const MinFunctor&	func_;
		const T				tol_;
};

template<class T> inline Minimizer<T>::Minimizer(const MinFunctor& func) :
	func_(func),tol_(std::sqrt(std::numeric_limits<T>::epsilon()))
{ }

// 1D Brent's method minimization
template<class T> class BrentsMethodMinimizer : public Minimizer<T>
{
	public:
		typedef typename Minimizer<T>::MinFunctor	MinFunctor;
			
		BrentsMethodMinimizer(const MinFunctor&);
		BrentsMethodMinimizer(const MinFunctor&,const T& tol);
		~BrentsMethodMinimizer() { }
		
		FixedVector<T,2> operator()(const FixedVector<T,3>&) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		static FixedVector<T,2> findMinimum(const MinFunctor&,const FixedVector<T,3>& x,const T& tol);
};

template<class T> inline BrentsMethodMinimizer<T>::BrentsMethodMinimizer(const MinFunctor& func) :
	Minimizer<T>(func)
{ }

template<class T> inline BrentsMethodMinimizer<T>::BrentsMethodMinimizer(const MinFunctor& func,const T& tol) :
	Minimizer<T>(func,tol)
{ }

template<class T> inline FixedVector<T,2> BrentsMethodMinimizer<T>::operator()(const FixedVector<T,3>& x) const
{
	return findMinimum((*this).functor(),x,(*this).tolerance());
}

template<class T> inline FixedVector<T,2> BrentsMethodMinimizer<T>::findMinimum(const MinFunctor& func,const FixedVector<T,3>& x,const T& tol) 
{
	const Int32	ITMAX = 100;
	const T		CGOLD = 0.3819660,
				ZEPS = 1.0e-10;

	T		a,b,d,etemp,
			fu,fv,fw,fx,
			p,q,r,tol1,tol2,
			u,v,w,xx,xm,e=0.0;

	a = (x[0] < x[2] ? x[0] : x[2]);
	b = (x[0] > x[2] ? x[0] : x[2]);
	
	xx=w=v=x[1];
	fw=fv=fx=func(xx);
	
	for (Int32 iter=1;iter<=ITMAX;iter++) 
	{
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*std::abs(xx)+ZEPS);
		
		if (std::abs(xx-xm) <= (tol2-0.5*(b-a))) 
		{
			return FixedVector<T,2>(xx,fx);
		}
		
		if (std::abs(e) > tol1) 
		{
			r=(xx-w)*(fx-fv);
			q=(xx-v)*(fx-fw);
			p=(xx-v)*q-(xx-w)*r;
			q=2.0*(q-r);
			if (q > 0.0) p = -p;
			q=std::abs(q);
			etemp=e;
			e=d;
		
			if (std::abs(p) >= std::abs(0.5*q*etemp) || p <= q*(a-xx) || p >= q*(b-xx))
				d=CGOLD*(e=(xx >= xm ? a-xx : b-xx));
			else 
			{
				d=p/q;
				u=xx+d;
				if (u-a < tol2 || b-u < tol2)
					d=std::sign(tol1,xm-xx);
			}
		} 
		else 
		{
			d=CGOLD*(e=(xx >= xm ? a-xx : b-xx));
		}
		
		u=(std::abs(d) >= tol1 ? xx+d : xx+std::sign(tol1,d));
		fu=func(u);
		
		if (fu <= fx) 
		{
			if (u >= xx) a=xx; else b=xx;
			NR::shft(v,w,xx,u);
			NR::shft(fv,fw,fx,fu);
		} 
		else 
		{
			if (u < xx) a=u; else b=u;
			
			if (fu <= fw || w == xx) 
			{
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} 
			else if (fu <= fv || v == xx || v == w) 
			{
				v=u;
				fv=fu;
			}
		}
	}
	
	throw Err("BrentsMethodMinimizer<T>::Too many iterations in brent");

	return FixedVector<T,2>();
}

} // namespace num

} // namespace mcs

#endif // MCS_MINIMIZATION_H