/*
********file reestimate.f90*****************************

! This module contains the user-callable routines remap and reest.

! Remap "rectifies" irregularly spaced data (y) known (with errors sig) on
! one set of x's to data (yout) on a different, generally interleaved,
! set of xout's.  The data is characterized by a single scalar population
! variance (psig**2) and a single inverse decorrelation distance w.
! All quantities are input, except yout, which is output.

! Reest is a lower-level routine that reestimates a single data set
! (x,y,sig) onto a set of improved y values yout, located at the same
! data points x.  Now psig and w are general vectors.  The optional
! argument ybar, if present, causes an unbiased (Gauss-Markov) estimate
! to be used (essentially removing and then restoring the mean of the
! data) and returns the value of the data's mean.  The optional argument
! chisq, if present, causes the chi-square of the data's internal
! consistency to be returned as chisq.

! A trivial test routine is appended (commented out).
*/
#include "MCS_Constants.h"
#include "MCS_Matrix.h"
#include "MCS_NumericExceptions.h"
#include "MCS_RPFastInterpolation.h"
#include "MCS_Vector.h"
	
namespace mcs {

namespace num {

// single precision
static Vector<Float32> symtrimul(const Vector<Float32>& a,const Vector<Float32>& b,const Vector<Float32>& u)
{
	Vector<Float32>::Offset	a_offs(a,1),
							b_offs(b,1),
							u_offs(u,1);

	IndexRange<1>	rng(b.range());
	
	if (u.range() != rng || a.last() != rng.last()-1) throw Vector<Float32>::SizeMismatchErr();
	
	Index<1>	nlo = rng.first(),
				nhi = rng.last();
	
	Vector<Float32>	tmp(rng);
	
	tmp[nlo]=b[nlo]*u[nlo]+a[nlo]*u[nlo+1];
	
	for (Index<1> i=nlo+1;i<=nhi-1;i++)
		tmp[i]=a[i-1]*u[i-1]+b[i]*u[i]+a[i]*u[i+1];
	
	tmp[nhi]=a[nhi-1]*u[nhi-1]+b[nhi]*u[nhi];
	
	return tmp;
}

static Vector<Float32> tridag(const Vector<Float32>& a,const Vector<Float32>& b,
							  const Vector<Float32>& c,const Vector<Float32>& r)
{
	IndexRange<1>	drng(b.range()),					// diagonal
					udrng(drng.first(),drng.last()-1),	// upper diag
					ldrng(drng.first(),drng.last()-1);	// lower diag
	
	if (a.range() != udrng || c.range() != ldrng || r.range() != drng) 
		throw Vector<Float32>::SizeMismatchErr();
		
	Vector<Float32>	gam(drng),
					u(drng);
	Index<1>		nlo=drng.first(),
					nhi=drng.last();
	
	Float32 		bet = b[nlo];
	
	if (bet == 0.0) throw NumericException();
	
	u[nlo]=r[nlo]/bet;
	
	for (Index<1> j=nlo+1;j<=nhi;j++) 
	{
		gam[j]=c[j-1]/bet;
		bet=b[j]-a[j-1]*gam[j];
		
		if (bet == 0.0) throw NumericException();
		
		u[j]=(r[j]-a[j-1]*u[j-1])/bet;
	}
	
	for (Index<1> j=nhi-1;j>=nlo;j--)
		u[j] -= gam[j+1]*u[j+1];
		
	return u;
}

Vector<Float32> remap(const Vector<Float32>& x,const Vector<Float32>& y,const Vector<Float32>& sig,
					  Float32 psig,Float32 w,const Vector<Float32>& xout)
{
	return remap(x,y,sig,psig,w,true,xout);
}

Vector<Float32> remap(const Vector<Float32>& x,const Vector<Float32>& y,const Vector<Float32>& sig,
					  Float32 psig,Float32 w,bool unbiased,const Vector<Float32>& xout)
{
	const Vector<Float32>::Offset	x_offs(x,1),
									y_offs(y,1),
									sig_offs(sig,1),
									xout_offs(xout,1);
	
	const Int32				n = x.dim(),
							nout = xout.dim(),
							ntot = n+nout;
	const IndexRange<1>		in_rng(x.range()),
							out_rng(xout.range()),
							tot_rng(x.first(),x.first()+ntot-1);
	
	if (y.range() != in_rng || sig.range() != in_rng) throw Vector<Float32>::SizeMismatchErr();

	static const Float32	ZERO = 0.0,
							ONE = 1.0;
	
	// check that input abscissa is monotonic
	//if (!isMonotonicallyIncreasing(x)) throw NumericException();
	
	// constant value
	if (all(difference(y)==ZERO)) return Vector<Float32>(xout.range(),firstValue(y));
	
	const Float32	ave = statistics::mean(y);
	
	Vector<Float32>	xx(tot_rng),
					yy(tot_rng),
					ppsig(tot_rng),
					ssig(tot_rng),
					ww(tot_rng);

	Vector<Int32>	iflag(tot_rng,0);
	
	Index<1>	j = tot_rng.first(),
				k = tot_rng.first(),
				m;
	
	for (Index<1> mm=tot_rng.first();mm<=tot_rng.last();mm++)
	{	
		if (j > n || k > nout) 
		{
			m = mm;
			break;
		}
		
		if (x[j] <= xout[k])
		{
			iflag[mm] = j;
			j++;
		}
		else
		{
			iflag[mm] = -k;
			k++;
		}
		
		m = mm;
	}
	
	if (j<=n)
		for (Index<1> idx=m,i=j;idx<=ntot;idx++,i++)
			iflag[idx] = i;	

	if (k<=nout)
		for (Index<1> idx=m,i=k;idx<=ntot;idx++,i++)
			iflag[idx] = -i;	
	
	for (Index<1> idx=iflag.first();idx<=iflag.last();idx++)
	{
		if (iflag[idx] > 0)
		{
			xx[idx] = x[iflag[idx]];
			yy[idx] = y[iflag[idx]];
			ssig[idx] = sig[iflag[idx]];
			ppsig[idx] = psig;
			ww[idx] = w;
		}
		else
		{
			xx[idx] = xout[-iflag[idx]];
			yy[idx] = ave;
			ssig[idx] = 100.0*psig;
			ppsig[idx] = psig;
			ww[idx] = w;
		}
	}
	
	Float32	chisq;
	
	const Vector<Float32>	tww(ww[IndexRange<1>(ww.first(),ww.last()-1)]);
	Vector<Float32>			yyout = reest(xx,yy,ppsig,ssig,tww,chisq,unbiased);
	
	remove(yyout,iflag>0);
	
	return yyout.offset(xout_offs.originalOffset());
}

Vector<Float32> reest(const Vector<Float32>& x,const Vector<Float32>& y,const Vector<Float32>& psig,const Vector<Float32>& sig,
			   		  const Vector<Float32>& w,Float32& chisq,bool unbiased)
{
	static const Float32	_TINY_ = 1.0e-4;
	
	const Vector<Float32>::Offset	x_offs(x,1),
									y_offs(y,1),
									psig_offs(psig,1),
									sig_offs(sig,1),
									w_offs(w,1);
			
	IndexRange<1>	rng1(x.range()),
					rng2(x.first(),x.last()-1);
	Vector<Float32>	b(rng1),
					temp(rng1),
					bb(rng1),
					e(rng1),
					a(rng2),
					c(rng2),
					r(rng2),
					aa(rng2),
					cc(rng2);
			
	Index<1>		n = x.dim();
	
	static const Float32	ZERO = 0.0,
							ONE = 1.0;
	
	if (y.range() != rng1 || psig.range() != rng1 || sig.range() != rng1 || w.range() != rng2)
		throw Vector<Float32>::SizeMismatchErr();
		
	// check that input abscissa is monotonic
	if (!isMonotonicallyIncreasing(x)) throw NumericException();
		
	temp = sqr(sig);
	
	//! compute Rybicki's tridiagonal elements
	r = exp(-abs(w*difference(x)));
	a = -ONE/statistics::max(ONE/r-r,_TINY_);
	r *= a;
	
	b[b.first()] = ONE-firstValue(r);
	
	for (Index<1> idx=b.first()+1;idx<=b.last()-1;idx++)
		b[idx] = ONE-r[idx]-r[idx-1];
		
	b[b.last()] = ONE-lastValue(r);
	
	for (Index<1> idx=a.first();idx<=a.last();idx++)
		a[idx] /= psig[idx]*psig[idx+1];
		
	b /= sqr(psig);
	
	for (Index<1> idx=a.first();idx<=a.last();idx++)
	{
		aa[idx] = a[idx]*temp[idx+1];
		cc[idx] = a[idx]*temp[idx];
	}
	
	bb = b*temp + ONE;
	
	temp = tridag(aa,bb,cc,Vector<Float32>(rng1,ONE));
	e = symtrimul(a,b,temp);
	
	Float32	ybar = unbiased ? dot(e,y)/sum(e) : ZERO;
	
	temp = y-ybar;
	
	Vector<Float32>	yout = tridag(aa,bb,cc,temp);
		
	chisq = dot(temp,symtrimul(a,b,yout));
	yout += ybar;
	
	return yout.offset(x_offs.originalOffset());
}

// double precision
static Vector<Float64> symtrimul(const Vector<Float64>& a,const Vector<Float64>& b,const Vector<Float64>& u)
{
	Vector<Float64>::Offset	a_offs(a,1),
							b_offs(b,1),
							u_offs(u,1);

	IndexRange<1>	rng(b.range());
	
	if (u.range() != rng || a.last() != rng.last()-1) throw Vector<Float64>::SizeMismatchErr();
	
	Index<1>	nlo = rng.first(),
				nhi = rng.last();
	
	Vector<Float64>	tmp(rng);
	
	tmp[nlo]=b[nlo]*u[nlo]+a[nlo]*u[nlo+1];
	
	for (Index<1> i=nlo+1;i<=nhi-1;i++)
		tmp[i]=a[i-1]*u[i-1]+b[i]*u[i]+a[i]*u[i+1];
	
	tmp[nhi]=a[nhi-1]*u[nhi-1]+b[nhi]*u[nhi];
	
	return tmp;
}

static Vector<Float64> tridag(const Vector<Float64>& a,const Vector<Float64>& b,
					   		  const Vector<Float64>& c,const Vector<Float64>& r)
{
	IndexRange<1>	drng(b.range()),					// diagonal
					udrng(drng.first(),drng.last()-1),	// upper diag
					ldrng(drng.first(),drng.last()-1);	// lower diag
	
	if (a.range() != udrng || c.range() != ldrng || r.range() != drng) 
		throw Vector<Float64>::SizeMismatchErr();
		
	Vector<Float64>	gam(drng),
					u(drng);
	Index<1>		nlo=drng.first(),
					nhi=drng.last();
	
	Float64 		bet = b[nlo];
	
	if (bet == 0.0) throw NumericException();
	
	u[nlo]=r[nlo]/bet;
	
	for (Index<1> j=nlo+1;j<=nhi;j++) 
	{
		gam[j]=c[j-1]/bet;
		bet=b[j]-a[j-1]*gam[j];
		
		if (bet == 0.0) throw NumericException();
		
		u[j]=(r[j]-a[j-1]*u[j-1])/bet;
	}
	
	for (Index<1> j=nhi-1;j>=nlo;j--)
		u[j] -= gam[j+1]*u[j+1];
		
	return u;
}

Vector<Float64> remap(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& sig,
					  Float64 psig,Float64 w,const Vector<Float64>& xout)
{
	return remap(x,y,sig,psig,w,true,xout);
}

Vector<Float64> remap(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& sig,
					  Float64 psig,Float64 w,bool unbiased,const Vector<Float64>& xout)
{
	const Vector<Float64>::Offset	x_offs(x,1),
									y_offs(y,1),
									sig_offs(sig,1),
									xout_offs(xout,1);
	
	const Int32				n = x.dim(),
							nout = xout.dim(),
							ntot = n+nout;
	const IndexRange<1>		in_rng(x.range()),
							out_rng(xout.range()),
							tot_rng(x.first(),x.first()+ntot-1);
	
	if (y.range() != in_rng || sig.range() != in_rng) throw Vector<Float64>::SizeMismatchErr();

	static const Float64	ZERO = 0.0,
							ONE = 1.0;
	
	// must be monotonically increasing and unique
	//if (!isMonotonicallyIncreasing(x)) throw NumericException();
	
	
	// constant value
	if (all(difference(y)==ZERO)) return Vector<Float64>(xout.range(),firstValue(y));
	
	const Float64	ave = statistics::mean(y);
	
	Vector<Float64>	xx(tot_rng),
					yy(tot_rng),
					ppsig(tot_rng),
					ssig(tot_rng),
					ww(tot_rng);

	Vector<Int32>	iflag(tot_rng,0);
	
	Index<1>	j = tot_rng.first(),
				k = tot_rng.first(),
				m;
	
	for (Index<1> mm=tot_rng.first();mm<=tot_rng.last();mm++)
	{	
		if (j > n || k > nout) 
		{
			m = mm;
			break;
		}
		
		if (x[j] <= xout[k])
		{
			iflag[mm] = j;
			j++;
		}
		else
		{
			iflag[mm] = -k;
			k++;
		}
		
		m = mm;
	}
	
	if (j<=n)
		for (Index<1> idx=m,i=j;idx<=ntot;idx++,i++)
			iflag[idx] = i;	

	if (k<=nout)
		for (Index<1> idx=m,i=k;idx<=ntot;idx++,i++)
			iflag[idx] = -i;	
	
	for (Index<1> idx=iflag.first();idx<=iflag.last();idx++)
	{
		if (iflag[idx] > 0)
		{
			xx[idx] = x[iflag[idx]];
			yy[idx] = y[iflag[idx]];
			ssig[idx] = sig[iflag[idx]];
			ppsig[idx] = psig;
			ww[idx] = w;
		}
		else
		{
			xx[idx] = xout[-iflag[idx]];
			yy[idx] = ave;
			ssig[idx] = 100.0*psig;
			ppsig[idx] = psig;
			ww[idx] = w;
		}
	}
	
	Float64	chisq;
	
	const Vector<Float64>	tww(ww[IndexRange<1>(ww.first(),ww.last()-1)]);
	Vector<Float64>			yyout = reest(xx,yy,ppsig,ssig,tww,chisq,unbiased);
	
	remove(yyout,iflag>0);
	
	return yyout.offset(xout_offs.originalOffset());
}

Vector<Float64> reest(const Vector<Float64>& x,const Vector<Float64>& y,const Vector<Float64>& psig,const Vector<Float64>& sig,
			   		  const Vector<Float64>& w,Float64& chisq,bool unbiased)
{
	static const Float64	_TINY_ = 1.0e-8;
	
	const Vector<Float64>::Offset	x_offs(x,1),
									y_offs(y,1),
									psig_offs(psig,1),
									sig_offs(sig,1),
									w_offs(w,1);
			
	IndexRange<1>	rng1(x.range()),
					rng2(x.first(),x.last()-1);
	Vector<Float64>	b(rng1),
					temp(rng1),
					bb(rng1),
					e(rng1),
					a(rng2),
					c(rng2),
					r(rng2),
					aa(rng2),
					cc(rng2);
			
	Index<1>		n = x.dim();
	
	static const Float64	ZERO = 0.0,
							ONE = 1.0;
	
	if (y.range() != rng1 || psig.range() != rng1 || sig.range() != rng1 || w.range() != rng2)
		throw Vector<Float64>::SizeMismatchErr();
		
	// check that input abscissa is monotonic
	if (!isMonotonicallyIncreasing(x)) throw NumericException();
		
	temp = sqr(sig);
	
	//! compute Rybicki's tridiagonal elements
	r = exp(-abs(w*difference(x)));
	a = -ONE/statistics::max(ONE/r-r,_TINY_);
	r *= a;
	
	b[b.first()] = ONE-firstValue(r);
	
	for (Index<1> idx=b.first()+1;idx<=b.last()-1;idx++)
		b[idx] = ONE-r[idx]-r[idx-1];
		
	b[b.last()] = ONE-lastValue(r);
	
	for (Index<1> idx=a.first();idx<=a.last();idx++)
		a[idx] /= psig[idx]*psig[idx+1];
		
	b /= sqr(psig);
	
	for (Index<1> idx=a.first();idx<=a.last();idx++)
	{
		aa[idx] = a[idx]*temp[idx+1];
		cc[idx] = a[idx]*temp[idx];
	}
	
	bb = b*temp + ONE;
	
	temp = tridag(aa,bb,cc,Vector<Float64>(rng1,ONE));
	e = symtrimul(a,b,temp);
	
	Float64	ybar = unbiased ? dot(e,y)/sum(e) : ZERO;
	
	temp = y-ybar;
	
	Vector<Float64>	yout = tridag(aa,bb,cc,temp);
		
	chisq = dot(temp,symtrimul(a,b,yout));
	yout += ybar;
	
	return yout.offset(x_offs.originalOffset());
}

} // namespace num

} // namespace mcs

/*
*************file lopast.f90***********************************

! The function lopast(y,x,f) returns a vector that is the low-pass
! filtered version of the data points y, at the abcissas x, with a 3dB
! cutoff frequency of f.  This is pretty much exactly as described in
! the paper.

! High pass filtering is obtained by changing the line
!	lopast=y-real(ans)
! to
!	lopast=real(ans)
! and (if you want f to be the 3dB point again) changing the
! constant 5.5380744 to 3.56427.  (If the filters were characterized
! by 6dB cutoffs instead of 3dB cutoffs, the constants would of course
! be the same -- this is only semantics in the definition of cutoff.)

! A trivial test routine is appended (commented out).

MODULE lopast_module
CONTAINS
	FUNCTION lopast(y,x,f)
	IMPLICIT NONE
	REAL, DIMENSION(:) :: y,x
	REAL :: f
	REAL, DIMENSION(size(y)) :: lopast
	INTEGER :: n
	COMPLEX :: fac
	COMPLEX, DIMENSION(size(y)) :: b,rhs,ans
	COMPLEX, DIMENSION(size(y)-1) :: r,e,w,re
	n=size(y)
	if (size(x) /= n) pause 'bad sizes in lopast'
	fac=abs(f)*5.5380744*cmplx(1.,1.)
	w=fac*abs(x(2:n)-x(1:n-1))
	r=exp(-w)  ! note this is a complex exponential !
	e=1./(1./r - r)
	re=r*e
	b=1.
	b(1:n-1)=b(1:n-1)+re
	b(2:n)=b(2:n)+re
	w = 0.5/w
	rhs(1:n-1)=w*(y(1:n-1)-y(2:n))
	rhs(n)=0.
	rhs(2:n)=rhs(2:n)+w*(y(2:n)-y(1:n-1))
	e=-e
	call ctridag(e,b,e,rhs,ans)
	lopast=y-real(ans)
	END FUNCTION lopast

	SUBROUTINE ctridag(a,b,c,r,u)
	IMPLICIT NONE
	COMPLEX, DIMENSION(:) :: a,b,c,r,u
	INTEGER :: n,j
	COMPLEX, DIMENSION(size(b)) :: gam
	COMPLEX :: bet
	n=size(b)
	if (size(a) /= n-1 .or. size(c) /= n-1 .or. size(r) /= n &
		.or. size(u) /= n) pause 'bad sizes in ctridag'
	bet=b(1)
	if(bet==(0.,0.)) pause 'Error 1 in ctridag'
	u(1)=r(1)/bet
	do j=2,n
		gam(j)=c(j-1)/bet
		bet=b(j)-a(j-1)*gam(j)
		if(bet==(0.,0.)) pause 'Error 2 in ctridag'
		u(j)=(r(j)-a(j-1)*u(j-1))/bet
	enddo
	do j=n-1,1,-1
		u(j)=u(j)-gam(j+1)*u(j+1)
	enddo
	END SUBROUTINE ctridag
END MODULE

!	PROGRAM test_lopast
!	USE lopast_module
!	INTEGER, PARAMETER :: n=200
!	REAL, DIMENSION(n) :: x,y,a,b,c,d
!	x = (/ (j+0.5*sin(0.1*real(j)),j=1,n) /)
!	y = 0.
!	y(1:20)=1.
!	a=lopast(y,x,0.01)
!	b=lopast(y,x,0.03)
!	c=lopast(y,x,0.1)
!	d=lopast(y,x,0.3)
!	do j=1,n
!		write (*,*) x(j),a(j),b(j),c(j),d(j)
!	enddo
!	END

*******************end file************************
*/