#ifndef MCS_RANDOM_H
#define MCS_RANDOM_H

#include <cstdlib>
#include <memory>

#include "MCS_Ensure.h"
#include "MCS_Functor.h"

namespace mcs {

namespace num {

class RandomDeviate : public Functor0<double>
{
	public:
		typedef std::auto_ptr< Functor0<double> >	Ptr;
		
		virtual ~RandomDeviate() { };
		
		virtual long seed() const = 0;
};

// use ANSI standard random number generator initialized with long seed
class SystemDeviate : public RandomDeviate
{
	public:        
		~SystemDeviate() { }
		
		long seed() const	{ return static_cast<long>(seed_); }
		
		double operator()() const;
		
		static RandomDeviate::Ptr create(long seed);
		
	private:
		SystemDeviate(long);
		
		mutable unsigned int	seed_;
};

inline SystemDeviate::SystemDeviate(long seed) : 
	seed_(static_cast<unsigned int>(seed))
{ }

inline RandomDeviate::Ptr SystemDeviate::create(long seed)
{ 
	return RandomDeviate::Ptr(new SystemDeviate(seed)); 
}

inline double SystemDeviate::operator()() const	
{ 
	static bool seed_initialized = false;
	
	if (!seed_initialized)
	{
		STD_(srand(seed_));
		
		seed_initialized = true;
	} 
		
	return static_cast<double>(STD_(rand()))/RAND_MAX; 
}

// random deviate generator initialized with long seed
class LDeviate : public RandomDeviate
{
	public:        
		typedef double (*Generator)(long*);
		
		~LDeviate() { }
		
		long seed() const	{ return static_cast<long>(seed_); }
		
		double operator()() const;
		
		static RandomDeviate::Ptr create(long,const Generator);
		
	private:
		LDeviate(long,const Generator);
		
		mutable long	seed_;
		const Generator	gen_;
};

inline LDeviate::LDeviate(long seed,const Generator gen) : 
	gen_(gen),seed_(seed)
{ }

inline RandomDeviate::Ptr LDeviate::create(long seed,const Generator gen)
{ 
	return RandomDeviate::Ptr(new LDeviate(seed,gen)); 
}
			
inline double LDeviate::operator()() const	
{ 
	return gen_(&seed_); 
}

// random deviate generator initialized with int parameter and long seed
class ILDeviate : public RandomDeviate
{
	public:        
		typedef double (*Generator)(int,long*);
		
		~ILDeviate() { }
		
		long seed() const	{ return static_cast<long>(seed_); }
		
		double operator()() const;
		
		static RandomDeviate::Ptr create(long,const Generator,int);
		
	private:
		ILDeviate(long,const Generator,int);
		
		mutable long	seed_;
		const Generator	gen_;
		const int		val_;
};

inline ILDeviate::ILDeviate(long seed,const Generator gen,int val) : 
	gen_(gen),val_(val),seed_(seed)
{ }

inline RandomDeviate::Ptr ILDeviate::create(long seed,const Generator gen,int val)
{ 
	return RandomDeviate::Ptr(new ILDeviate(seed,gen,val)); 
}
			
inline double ILDeviate::operator()() const	
{ 
	return gen_(val_,&seed_); 
}

// random deviate generator initialized with double parameter and long seed
class DLDeviate : public RandomDeviate
{
	public:        
		typedef double (*Generator)(double,long*);
		
		~DLDeviate() { }
		
		long seed() const	{ return static_cast<long>(seed_); }
		
		double operator()() const;
		
		static RandomDeviate::Ptr create(long,const Generator,double);
		
	private:
		DLDeviate(long,const Generator,double);
		
		mutable long	seed_;
		const Generator	gen_;
		const double	val_;
};

inline DLDeviate::DLDeviate(long seed,const Generator gen,double val) : 
	gen_(gen),val_(val),seed_(seed)
{ }

inline RandomDeviate::Ptr DLDeviate::create(long seed,const Generator gen,double val)
{ 
	return RandomDeviate::Ptr(new DLDeviate(seed,gen,val)); 
}
			
inline double DLDeviate::operator()() const	
{ 
	return gen_(val_,&seed_); 
}

// random deviate generator initialized with double and int parameters and long seed
class DILDeviate : public RandomDeviate
{
	public:        
		typedef double (*Generator)(double,int,long*);
		
		~DILDeviate() { }
		
		long seed() const	{ return static_cast<long>(seed_); }
		
		double operator()() const;
		
		static RandomDeviate::Ptr create(long,const Generator,double,int);
		
	private:
		DILDeviate(long,const Generator,double,int);
		
		mutable long	seed_;
		const Generator	gen_;
		const double	dval_;
		const int		ival_;
};

inline DILDeviate::DILDeviate(long seed,const Generator gen,double dval,int ival) : 
	gen_(gen),dval_(dval),ival_(ival),seed_(seed)
{ }

inline RandomDeviate::Ptr DILDeviate::create(long seed,const Generator gen,double dval,int ival)
{ 
	return RandomDeviate::Ptr(new DILDeviate(seed,gen,dval,ival)); 
}
			
inline double DILDeviate::operator()() const	
{ 
	return gen_(dval_,ival_,&seed_); 
}

typedef LDeviate	UniformDeviate;
typedef ILDeviate	GammaDeviate;
typedef DLDeviate	PoissonDeviate;
typedef DILDeviate	BinomialDeviate;

template<class T> class RandomNumber : public Functor0<T>,public Functor2<const T&,const T&,T>
{ 
	public:
		RandomNumber(long);
		RandomNumber(RandomDeviate::Ptr);
		virtual ~RandomNumber() { }
		
		long seed() const 	{ return randptr_->seed(); }
		
		T operator()() const;
		T operator()(const T&,const T&) const;
	
	private:
		static const EnsureReal<T>	dummy_;
		
		RandomDeviate::Ptr	randptr_;
};

template<class T> inline RandomNumber<T>::RandomNumber(long seed) :
	randptr_(SystemDeviate::create(seed))
{ }

template<class T> inline RandomNumber<T>::RandomNumber(RandomDeviate::Ptr randptr) :
	randptr_(randptr)
{ }

// return random number between 0 and 1
//	-> round for integer types
template<class T> inline T RandomNumber<T>::operator()() const
{
	return static_cast<T>(std::round((*randptr_)()));
}

//	-> return deviate for floating point types
template<> inline float RandomNumber<float>::operator()() const
{
	return static_cast<float>((*randptr_)());
}

template<> inline double RandomNumber<double>::operator()() const
{
	return static_cast<double>((*randptr_)());
}

// return random number in [lo,hi)
template<class T> inline T RandomNumber<T>::operator()(const T& lo,const T& hi) const
{
	return lo+static_cast<T>((hi-lo)*(*randptr_)());
}

} // namespace num

} // namespace mcs

#endif // MCS_RANDOM_H