#ifndef MCS_REGRESSION_H
#define MCS_REGRESSION_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_Matrix.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"

namespace mcs {

namespace num {

// regressor
template<class T> class Regressor : public Functor1<const T&,T>,public Functor1< const Vector<T>&,Vector<T> >
{ 
	public:
		Regressor(const Vector<T>&,const Vector<T>&);
		Regressor(const Vector<T>&,const Vector<T>&,const Vector<T>&);
		virtual ~Regressor() { }
				
		const Vector<T>&	x() const				{ return x_; }
		const Vector<T>&	y() const				{ return y_; }
		const Vector<T>&	std() const				{ return sig_; }
		
		const Vector<T>&	coefficients() const	{ return coef_; }
		const Matrix<T>&	covariance() const		{ return covar_; }
		const Vector<T>&	fit() const				{ return fit_; }
		const Vector<T>&	residual() const		{ return residual_; }
		
		T	chiSquare() const						{ return chisq_; }
		T	Q() const								{ return Q_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)
		
	protected:
		virtual void regress() = 0;
		
		Vector<T>	x_,
					y_,
					sig_,
					coef_,
					fit_,
					residual_;
		Matrix<T>	covar_;
		T			chisq_,Q_;	
};

template<class T> Regressor<T>::Regressor(const Vector<T>& x,const Vector<T>& y) :
	x_(x),y_(y),sig_(x.range(),T(1))
{ }

template<class T> Regressor<T>::Regressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig) :
	x_(x),y_(y),sig_(sig)
{ }

// regress to a parametric function
template<class T> class FunctionRegressor : public Regressor<T>
{ 
	public:
		typedef Func3<T,T*,int,void>::FunctionPtr	FunctionPtr;
		
		FunctionRegressor(const Vector<T>&,const Vector<T>&,FunctionPtr,Int32);
		FunctionRegressor(const Vector<T>&,const Vector<T>&,const Vector<T>&,FunctionPtr,Int32);
		virtual ~FunctionRegressor() { }
		
		T operator()(const T&) const;
		Vector<T> operator()(const Vector<T>& x) const;

		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)
		
	private:	
		void alloc();
		void regress();
		
		FunctionPtr		funcptr_;
		Int32			param_;
};

template<class T> inline FunctionRegressor<T>::FunctionRegressor(const Vector<T>& x,const Vector<T>& y,FunctionPtr funcptr,Int32 param) :
	Regressor<T>(x,y),funcptr_(funcptr),param_(param)
{
	if (x.range() != y.range()) throw Err();
	
	alloc();
	regress();
}

template<class T> inline FunctionRegressor<T>::FunctionRegressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,FunctionPtr funcptr,Int32 param) :
	Regressor<T>(x,y,sig),funcptr_(funcptr),param_(param)
{
	if (x.range() != y.range() || x.range() != sig.range()) throw Err();
	
	alloc();
	regress();
}

template<class T> inline void FunctionRegressor<T>::alloc()
{
	coef_ = Vector<T>(IndexRange<1>(1,param_));
	covar_ = Matrix<T>(IndexRange<2>(Index<2>(1,1),Index<2>(param_,param_)));
}

template<class T> inline void FunctionRegressor<T>::regress()
{
	NRVector<T>	xnr(const_cast<Vector<T>&>(x_)),
				ynr(const_cast<Vector<T>&>(y_)),
				signr(const_cast<Vector<T>&>(sig_));
	
	Matrix<T>	u(IndexRange<2>(Index<2>(1,1),Index<2>(x_.dim(),param_))),
				v(covar_.range());
	Vector<T>	w(coef_.range());
	
	NRMatrix<T>	nru(u),nrv(v);
	NRVector<T>	nrw(w);
	
	NRVector<T>	nrcoef(coef_);
	NRMatrix<T>	nrcovar(covar_);	
	
	NR::FN<T>::svdfit(xnr.data(),ynr.data(),signr.data(),xnr.dim(),
					  nrcoef.data(),param_,nru.data(),nrv.data(),nrw.data(),
					  &chisq_,funcptr_);
					  
	NR::FN<T>::svdvar(nrv.data(),param_,nrw.data(),nrcovar.data());	
	
	fit_ = (*this)(x_);
	residual_ = y_-fit_;
}

template<class T> inline T FunctionRegressor<T>::operator()(const T& x) const
{
	Vector<T>	funcval(coef_.range());	
	NRVector<T>	nrfuncval(funcval);
	
	(*funcptr_)(x,nrfuncval.data(),nrfuncval.dim());
	
	return dot(coef_,funcval);
}

template<class T> inline Vector<T> FunctionRegressor<T>::operator()(const Vector<T>& x) const
{
	Vector<T>	y(x.range());
				
	for (Index<1> i=y.first();i<=y.last();i++)
	{
		y[i] = (*this)(x[i]);
	}
	
	return y;
}

// container class for actual fitting functions
template<class T> class FitFunctions
{
	public:
		static void linearFit(T,T*,int);
		static void polynomialFit(T,T*,int);
		static void trigonometricFit(T,T*,int);
};

template<class T> inline void FitFunctions<T>::linearFit(T x,T* afunc,int)
{
	afunc[1] = 1;
	afunc[2] = x;
}

template<class T> inline void FitFunctions<T>::polynomialFit(T x,T* afunc,int ma)
{
	afunc[1] = 1;
	
	for (Int32 i=2;i<=ma;i++)
		afunc[i] = afunc[i-1]*x;
}

template<class T> inline void FitFunctions<T>::trigonometricFit(T x,T* afunc,int ma)
{
	afunc[1] = 1;
	afunc[2] = x;
	
	for (Int32 i=3;i<=ma;i+=2)
	{
		afunc[i] = std::sin(_two_pi_*static_cast<T>((i-1)/2)*x);
		afunc[i+1] = std::cos(_two_pi_*static_cast<T>((i-1)/2)*x);
	}
}

// shortcut classes
template<class T> class LinearRegressor : public FunctionRegressor<T>
{
	public:
		LinearRegressor(const Vector<T>&,const Vector<T>&);
		LinearRegressor(const Vector<T>&,const Vector<T>&,const Vector<T>&);
		~LinearRegressor() { }
};

template<class T> inline LinearRegressor<T>::LinearRegressor(const Vector<T>& x,const Vector<T>& y) :
	FunctionRegressor<T>(x,y,FitFunctions<T>::linearFit,2)
{ }

template<class T> inline LinearRegressor<T>::LinearRegressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig) :
	FunctionRegressor<T>(x,y,sig,FitFunctions<T>::linearFit,2)
{ }

template<class T> class PolynomialRegressor : public FunctionRegressor<T>
{
	public:
		PolynomialRegressor(const Vector<T>&,const Vector<T>&,Int32);
		PolynomialRegressor(const Vector<T>&,const Vector<T>&,const Vector<T>&,Int32);
		~PolynomialRegressor() { }
		
	private:
		PolynomialRegressor();
};

template<class T> inline PolynomialRegressor<T>::PolynomialRegressor(const Vector<T>& x,const Vector<T>& y,Int32 order) :
	FunctionRegressor<T>(x,y,FitFunctions<T>::polynomialFit,order+1)
{ }

template<class T> inline PolynomialRegressor<T>::PolynomialRegressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,Int32 order) :
	FunctionRegressor<T>(x,y,sig,FitFunctions<T>::polynomialFit,order+1)
{ }

template<class T> class TrigonometricRegressor : public FunctionRegressor<T>
{
	public:
		TrigonometricRegressor(const Vector<T>&,const Vector<T>&,const Vector<T>&,Int32);
		~TrigonometricRegressor() { }
		
	private:
		TrigonometricRegressor();
};

template<class T> inline TrigonometricRegressor<T>::TrigonometricRegressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,Int32 harmonics) :
	FunctionRegressor<T>(x,y,sig,FitFunctions<T>::trigonometricFit,2*harmonics+2)
{ }

// regress to a functor
template<class T> class FitFunctor : public Functor1<const T&,Vector<T> >
{
	public:
		FitFunctor(const Int32& numParam) : numParam_(numParam) { }
		virtual ~FitFunctor() { }
		
		virtual Int32 numberOfParameters() const 	{ return numParam_; }
		
	private:
		Int32	numParam_;
};

// series of frequencies
template<class T> class TrigonometricSeriesFit : public FitFunctor<T>
{
	public:
		TrigonometricSeriesFit(const Vector<T>& freq,const Int32& polyOrder = 1);
		~TrigonometricSeriesFit() { }
		
		Vector<T> operator()(const T&) const;
		
	private:
		Vector<T>	omega_;
		Int32		polyOrder_;
};

template<class T> inline TrigonometricSeriesFit<T>::TrigonometricSeriesFit(const Vector<T>& freq,const Int32& polyOrder) :
	FitFunctor<T>(freq.dim()*2+polyOrder+1),polyOrder_(polyOrder),omega_(_two_pi_*freq)
{ }

template<class T> inline Vector<T> TrigonometricSeriesFit<T>::operator()(const T& x) const
{
	Vector<T>	out(IndexRange<1>(1,numberOfParameters()));
	
	for (Index<1> i=out.first();i<=polyOrder_+1;++i)
		out[i] = std::pow(x,i-1);
	
	for (Index<1> i=polyOrder_+2,ii=omega_.first();i<=out.last();i+=2,ii++)
	{
		out[i] = std::sin(omega_[ii]*x);
		out[i+1] = std::cos(omega_[ii]*x);
	}
	
	return out;
}

// series of harmonics
template<class T> class TrigonometricHarmonicFit : public FitFunctor<T>
{
	public:
		TrigonometricHarmonicFit(const T& freq,const Int32& numHarmonics,const Int32& polyOrder = 1);
		~TrigonometricHarmonicFit() { }
		
		Vector<T> operator()(const T&) const;
		
	private:
		T		omega_;
		Int32	polyOrder_;
};

template<class T> inline TrigonometricHarmonicFit<T>::TrigonometricHarmonicFit(const T& freq,const Int32& numHarmonics,const Int32& polyOrder) :
	FitFunctor<T>(2*numHarmonics+polyOrder+1),omega_(_two_pi_*freq),polyOrder_(polyOrder)
{ }

template<class T> inline Vector<T> TrigonometricHarmonicFit<T>::operator()(const T& x) const
{
	Vector<T>	out(IndexRange<1>(1,numberOfParameters()));
	
	for (Index<1> i=out.first();i<=polyOrder_+1;++i)
		out[i] = std::pow(x,i-1);
		
	for (Index<1> i=3;i<=out.last();i+=2)
	{
		out[i] = std::sin(omega_*static_cast<T>((i-1)/2)*x);
		out[i+1] = std::cos(omega_*static_cast<T>((i-1)/2)*x);
	}
	
	return out;
}

template<class T> class FunctorRegressor : public Regressor<T>
{ 
	public:
		//typedef Functor1<const T&,Vector<T> > const *	FunctorPtr;
		typedef FitFunctor<T> const *	FunctorPtr;
		
		FunctorRegressor(const Vector<T>&,const Vector<T>&,FunctorPtr);
		FunctorRegressor(const Vector<T>&,const Vector<T>&,const Vector<T>&,FunctorPtr);
		virtual ~FunctorRegressor();
		
		T operator()(const T&) const;
		Vector<T> operator()(const Vector<T>& x) const;

		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:	
		void alloc();
		void regress();
		
		const FunctorPtr	func_;
};

template<class T> inline FunctorRegressor<T>::FunctorRegressor(const Vector<T>& x,const Vector<T>& y,FunctorPtr func) :
	Regressor<T>(x,y),func_(func)
{
	if (x.range() != y.range()) throw Err();
	
	alloc();
	regress();
}

template<class T> inline FunctorRegressor<T>::FunctorRegressor(const Vector<T>& x,const Vector<T>& y,const Vector<T>& sig,FunctorPtr func) :
	Regressor<T>(x,y,sig),func_(func)
{
	if (x.range() != y.range() || x.range() != sig.range()) throw Err();
	
	alloc();
	regress();
}

template<class T> inline FunctorRegressor<T>::~FunctorRegressor()
{ }

template<class T> inline void FunctorRegressor<T>::alloc()
{
	coef_ = Vector<T>(IndexRange<1>(1,func_->numberOfParameters()));
	covar_ = Matrix<T>(IndexRange<2>(Index<2>(1,1),Index<2>(func_->numberOfParameters(),func_->numberOfParameters())));
}

template<class T> inline void FunctorRegressor<T>::regress()
{
	NRVector<T>	xnr(const_cast<Vector<T>&>(x_)),
				ynr(const_cast<Vector<T>&>(y_)),
				signr(const_cast<Vector<T>&>(sig_));
	
	const IndexRange<2>	rng(Index<2>(x_.first(),1),Index<2>(x_.last(),func_->numberOfParameters()));
	Matrix<T>			afunc(rng),
				u(rng),
				v(covar_.range());
	Vector<T>	w(coef_.range());
	
	// set up fit function matrix
	for (Index<1> i=afunc.first().rank(0);i<=afunc.last().rank(0);++i)
	{
		const Vector<T>	val = (*func_)(x_[i]);
		
		for (Index<1> j=afunc.first().rank(1);j<=afunc.last().rank(1);++j)
		{
			afunc(Index<2>(i,j)) = val[j];
		}
	}
	
	NRMatrix<T>	nrafunc(afunc),nru(u),nrv(v);
	NRVector<T>	nrw(w);
	
	NRVector<T>	nrcoef(coef_);
	NRMatrix<T>	nrcovar(covar_);	
	
	NR::FN<T>::svdfitm(nrafunc.data(),ynr.data(),signr.data(),xnr.dim(),
					   nrcoef.data(),func_->numberOfParameters(),nru.data(),nrv.data(),nrw.data(),
					   &chisq_);
					  
	NR::FN<T>::svdvar(nrv.data(),func_->numberOfParameters(),nrw.data(),nrcovar.data());	
	
	fit_ = (*this)(x_);
	residual_ = y_-fit_;
}

template<class T> inline T FunctorRegressor<T>::operator()(const T& x) const
{
	return dot(coef_,(*func_)(x));
}

template<class T> inline Vector<T> FunctorRegressor<T>::operator()(const Vector<T>& x) const
{
	Vector<T>	y(x.range());
				
	for (Index<1> i=y.first();i<=y.last();i++)
	{
		y[i] = (*this)(x[i]);
	}
	
	return y;
}

} // namespace num

} // namespace mcs

#endif // MCS_REGRESSION_H