#ifndef MCS_ROOTFINDER_H
#define MCS_ROOTFINDER_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"

namespace mcs {

namespace num {

template<class T> void bracketRoot(const Functor1<const T&,T>& func,T& x1,T& x2,const T& FACTOR = 1.6,const Int32& NTRY = 50)
{
	if (x1 == x2) throw NumericException("bracketRoot - initial values not distinct");
	
	T	f1 = func(x1),
		f2 = func(x2);

	for (Int32 j=1;j<=NTRY;j++) 
	{
		if (f1*f2 < 0.0) return;
		
		if (std::abs(f1) < std::abs(f2))
		{
			x1 += FACTOR*(x1-x2);
			f1 = func(x1);
		}
		else
		{
			x2 += FACTOR*(x2-x1);
			f2 = func(x2);
		}
	}
	
	// unable to find bracket
	throw NumericException("bracketRoot - unable to find bracket");
	
	return;
}

// 1D root finder - takes bracketing points as arguments and returns root
template<class T> class RootFinder : public Functor2<const T&,const T&,T>
{
	public:
		typedef Functor1<const T&,T> RootFunctor;
		
		RootFinder(const RootFunctor& func) : func_(func) { }
		virtual ~RootFinder() { }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	protected:
		const RootFunctor& functor() const	{ return func_; }
		
	private:
		static const EnsureReal<T>	dummy_;
	
		const RootFunctor&	func_;
};

// 1D bisection root finder
template<class T> class BisectionRootFinder : public RootFinder<T>
{
	public:
		typedef typename RootFinder<T>::RootFunctor	RootFunctor;
			
		BisectionRootFinder(const RootFunctor&);
		BisectionRootFinder(const RootFunctor&,const T& tol);
		~BisectionRootFinder() { }
		
		T operator()(const T&,const T&) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		static T findRoot(const RootFunctor&,const T&,const T&,const T&);
		
		T	tol_;
};

template<class T> inline BisectionRootFinder<T>::BisectionRootFinder(const RootFunctor& func) :
	RootFinder<T>(func)
{
	tol_ = std::numeric_limits<T>::epsilon();
}

template<class T> inline BisectionRootFinder<T>::BisectionRootFinder(const RootFunctor& func,const T& tol) :
	RootFinder<T>(func),tol_(tol)
{ }

template<class T> inline T BisectionRootFinder<T>::operator()(const T& lo,const T& hi) const
{
	return findRoot((*this).functor(),lo,hi,tol_);
}

template<class T> inline T BisectionRootFinder<T>::findRoot(const RootFunctor& func,const T& lo,const T& hi,const T& tol) 
{
	T	f = func(lo),
		fmid = func(hi);
	
	if (f*fmid > 0) throw Err("BisectionRootFinder<T>::findRoot - root not bracketed");
	
	T	dx,rtb,xmid;
	
	rtb = f < 0 ? (dx = hi-lo,lo) : (dx = lo-hi,hi);
	
	static const Int32	JMAX = 40;
	
	for (Int32 j=1;j<=JMAX;++j)
	{
		fmid = func(xmid = rtb+(dx *= 0.5));
		
		if (fmid < 0) rtb = xmid;
		
		if (std::abs(dx) < tol || fmid == 0)
			return rtb;
	}
	
	throw Err("BisectionRootFinder<T>::findRoot - root not found");
	
	return 0;
}

// 1D Brent's method root finder
template<class T> class BrentsMethodRootFinder : public RootFinder<T>
{
	public:
		typedef typename RootFinder<T>::RootFunctor	RootFunctor;
			
		BrentsMethodRootFinder(const RootFunctor&);
		BrentsMethodRootFinder(const RootFunctor&,const T&);
		~BrentsMethodRootFinder() { }
		
		T operator()(const T&,const T&) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,NumericException)

	private:
		static T findRoot(const RootFunctor&,const T&,const T&,const T&);
		
		T	tol_;
};

template<class T> inline BrentsMethodRootFinder<T>::BrentsMethodRootFinder(const RootFunctor& func) :
	RootFinder<T>(func)
{
	tol_ = std::numeric_limits<T>::epsilon();
}

template<class T> inline BrentsMethodRootFinder<T>::BrentsMethodRootFinder(const RootFunctor& func,const T& tol) :
	RootFinder<T>(func),tol_(tol)
{ }

template<class T> inline T BrentsMethodRootFinder<T>::operator()(const T& lo,const T& hi) const
{
	return findRoot((*this).functor(),lo,hi,tol_);
}

template<class T> inline T BrentsMethodRootFinder<T>::findRoot(const RootFunctor& func,const T& lo,const T& hi,const T& tol) 
{
	// based on NR zbrent routine
	static const Int32	ITMAX = 100;
	static const T		EPS = std::numeric_limits<T>::epsilon();
	
	T	a = lo,
		b = hi,
		c = hi,
		d,e,min1,min2,
		fa = func(a),
		fb = func(b),
		fc = fb,
		p,q,r,s,tol1,xm;
		
	if ((fa > 0 && fb > 0) || (fa < 0 && fb < 0)) throw Err("BrentsMethodRootFinder<T>::findRoot - root not bracketed");
		
	for (Int32 iter=1;iter<=ITMAX;iter++) 
	{
		if ((fb > 0 && fc > 0) || (fb < 0 && fc < 0)) 
		{
			c = a;
			fc = fa;
			e = d = b-a;
		}
		
		if (std::abs(fc) < std::abs(fb)) 
		{
			a = b;
			b = c;
			c = a;
			fa = fb;
			fb = fc;
			fc = fa;
		}
		
		tol1 = 2*EPS*std::abs(b)+0.5*tol;
		xm = 0.5*(c-b);
		
		if (std::abs(xm) <= tol1 || fb == 0) return b;
		
		if (std::abs(e) >= tol1 && std::abs(fa) > std::abs(fb)) 
		{
			s = fb/fa;
			
			if (a == c) 
			{
				p = 2*xm*s;
				q = 1-s;
			} 
			else 
			{
				q = fa/fc;
				r = fb/fc;
				p = s*(2*xm*q*(q-r)-(b-a)*(r-1));
				q = (q-1)*(r-1)*(s-1);
			}
			
			if (p > 0) q = -q;
			
			p = std::abs(p);
			min1 = 3*xm*q-std::abs(tol1*q);
			min2 = std::abs(e*q);
			
			if (2*p < (min1 < min2 ? min1 : min2)) 
			{
				e = d;
				d = p/q;
			} 
			else 
			{
				d = xm;
				e = d;
			}
		} 
		else 
		{
			d = xm;
			e = d;
		}
		
		a = b;
		fa = fb;
		
		if (std::abs(d) > tol1)
		{
			b += d;
		}
		else
		{
			b += std::sign(tol1,xm);
		}
		
		fb = func(b);
	}
	
	throw Err("BrentsMethodRootFinder<T>::findRoot - root not found");
	
	return 0;
}

} // namespace num

} // namespace mcs

#endif // MCS_ROOTFINDER_H