#ifndef MCS_STATISTICS_H
#define MCS_STATISTICS_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_NRAliases.h"
#include "MCS_NRWrappers.h"
#include "MCS_NumericalRecipes.h"
#include "MCS_NumericExceptions.h"

namespace mcs {

namespace num {

// standard Gaussian statistics
template<class T> class GaussianStatistics
{
	public:
		GaussianStatistics(Int32 order = 2);
		GaussianStatistics(const Vector<T>&,Int32 order = 2);
		~GaussianStatistics() { }
		
		void add(const T&);
		
		T mean() const					{ return moments_[1]/num_; }
		T standardDeviation() const		{ return std::sqrt(variance()); }
		T variance() const				{ return moments_[2]/num_-sqr(moments_[1]/num_); }
		T skewness() const;
		T kurtosis() const;
		
		T moment(Int32 order) const;
	
	private:
		static const EnsureFloat<T>	dummy_;
		
		Uint32		num_;
		Vector<T>	moments_;
};

template<class T> const EnsureFloat<T> GaussianStatistics<T>::dummy_;

template<class T> inline GaussianStatistics<T>::GaussianStatistics(Int32 order) :
	num_(0),moments_(IndexRange<1>(1,order))
{ }

template<class T> inline GaussianStatistics<T>::GaussianStatistics(const Vector<T>& x,Int32 order) :
	num_(0),moments_(IndexRange<1>(1,order))
{ 
	for (Index<1> i x.first();i<=x.last();++i) add(x[i]);
}

template<class T> inline void GaussianStatistics<T>::add(const T& val)
{
	if (isfinite(val))
	{
		num_++;
			
		for (Index<1> j=moments_.first();j<=moments_.last();++j)
			moments_[j] += std::pow(val,j);
	}
}

template<class T> class QuantileStatistics
{
	public:
		QuantileStatistics(const Vector<T>&);
		~QuantileStatistics() { }
		
		T median() const;
		T IQR() const;
		Vector<T> quantile(const Vector<T>&) const;
		
	private:
		Vector<T>::const_reference	x_;
};

template<class T> QuantileStatistics<T>::QuantileStatistics(const Vector<T>& x) :
	x_(x)
{ }

template<class T> T QuantileStatistics<T>::median() const
{
	NRVector<T>	xnr_(const_cast<Vector<T>&>(x_));
	
	if (xnr_.dim() > 100)
	{
		return NR::FN<T>::select(xnr_.dim()/2,xnr_.dim(),xnr_.data());
	}
	else
	{
		if (odd(xnr_.dim()))
		{
			return NR::FN<T>::select((xnr_.dim()+1)/2,xnr_.dim(),xnr_.data());
		}
		else
		{
			return 0.5*(NR::FN<T>::select(xnr_.dim()/2,xnr_.dim(),xnr_.data())+
					    NR::FN<T>::select(xnr_.dim()/2+1,xnr_.dim(),xnr_.data()));
		}
	}
}

template<class T> T QuantileStatistics<T>::IQR() const
{
	NRVector<T>	xnr_(const_cast<Vector<T>&>(x_));
	
	return NR::FN<T>::select(0.75*xnr_.dim(),xnr_.dim(),xnr_.data())-
		   NR::FN<T>::select(0.25*xnr_.dim(),xnr_.dim(),xnr_.data());
}

template<class T> Vector<T> QuantileStatistics<T>::quantile(const Vector<T>& q_in) const
{
	NRVector<T>	xnr_(const_cast<Vector<T>&>(x_));
	
	if (min(q_in) < 0.0 || max(q_in) > 1.0) throw NumericException::Err();
	
	Vector<T>	q_out(q_in.range());
	for (Index<1> i=q_in.first();i<=q_in.last();i++)
		q_out[i] = NR::FN<T>::select(q_in[i]*xnr_.dim(),xnr_.dim(),xnr_.data());
		
	return q_out;
}

} // namespace num

} // namespace mcs

#endif // MCS_STATISTICS_H