#ifndef MCS_WEIGHTEDBIN_H
#define MCS_WEIGHTEDBIN_H

#include "MCS_Bounds.h"
#include "MCS_Exceptions.h"
#include "MCS_Matrix.h"
#include "MCS_Triple.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// bin class
template<class T> 
class WeightedBin
{
	public:
		WeightedBin(const Bounds<T>& bds);
		WeightedBin(const Bounds<T>& bds,const Int32& buf_siz);
		virtual ~WeightedBin() { }
		
		void add(const T& x,const T& z,const T& w);
		void add(const Vector<T>& x,const Vector<T>& z,const Vector<T>& w);
		
		void clear();
		
		const Bounds<T>& bounds() const	{ return bounds_; }
		
		const Vector<T>& bins();
		const Vector<T>& weights();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x,const T& z,const T& w);
		void finalize();
		
		Bounds<T>	bounds_;
		Vector<T>	bins_,
					weights_;
		
		Vector< FixedVector<T,3> >	buf_;	
		Index<1>					buf_idx_;
		bool						pending_;
		
		static const Index<1>		DEFAULT_BUF_SIZ_;
};

//implementation
template<class T> const Index<1> WeightedBin<T>::DEFAULT_BUF_SIZ_ = 1000;

template<class T> 
WeightedBin<T>::WeightedBin(const Bounds<T>& bds) : 
	bounds_(bds),
	bins_(bds.range(),T()),
	weights_(bds.range()),
	buf_(IndexRange<1>(1,DEFAULT_BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
WeightedBin<T>::WeightedBin(const Bounds<T>& bds,const Int32& buf_siz) : 
	bounds_(bds),
	bins_(bds.range(),T()),
	weights_(bds.range()),
	buf_(IndexRange<1>(1,buf_siz)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void WeightedBin<T>::add(const T& x,const T& z,const T& w)
{
	add_to_buffer(x,z,w);
}

template<class T> 
inline void WeightedBin<T>::add(const Vector<T>& x,const Vector<T>& z,const Vector<T>& w)
{
	if (x.range() != z.range()) throw Err();
	
	for (Index<1> i=x.first();i<=x.last();i++)
		add_to_buffer(x[i],z[i],w[i]);
}

template<class T> 
inline void WeightedBin<T>::clear()
{
	// clear data
	bins_ = weights_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Vector<T>& WeightedBin<T>::bins() 
{
	if (pending_) finalize();
	
	return bins_;
}

template<class T> 
inline const Vector<T>& WeightedBin<T>::weights()
{
	if (pending_) finalize();
	
	return weights_;
}

template<class T> 
inline void WeightedBin<T>::add_to_buffer(const T& x,const T& z,const T& w)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x,z,w);
	}
	else
	{
		buf_[buf_idx_][0] = x;
		buf_[buf_idx_][1] = z;
		buf_[buf_idx_][2] = w;
		buf_idx_++;
	}
	
	pending_ = true;
}

template<class T> 
void WeightedBin<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (isfinite(buf_[i][0]) && isfinite(buf_[i][1]) && isfinite(buf_[i][2])) 
		{
			const Index<1>	idx = bounds_.find(buf_[i][0]);
							
			bins_[idx] += buf_[i][1]*buf_[i][2];
			weights_[idx] += buf_[i][2];
		}
	}
	
	pending_ = false;
	buf_idx_ = buf_.first();
}

// 2D bin class
template<class T> 
class WeightedBin2D
{
	public:
		WeightedBin2D(const Bounds<T>& xbds,const Bounds<T>& ybds);
		WeightedBin2D(const Bounds<T>& xbds,const Bounds<T>& ybds,const Int32& buf_siz);
		virtual ~WeightedBin2D() { }
		
		void add(const T& x,const T& y,const T& z,const T& w);
		void add(const Vector<T>& x,const Vector<T>& y,const Vector<T>& z,const Vector<T>& w);
		
		void clear();
		
		const Bounds<T>& x_bounds() const		{ return x_bounds_; }
		const Bounds<T>& y_bounds() const		{ return y_bounds_; }
		
		const Matrix<T>& bins();
		const Matrix<T>& weights();
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		void add_to_buffer(const T& x,const T& y,const T& z,const T& w);
		void finalize();
		
		Bounds<T>	x_bounds_,
					y_bounds_;
		Matrix<T>	bins_,
					weights_;
		
		Vector< FixedVector<T,4> >	buf_;	
		Index<1>					buf_idx_;
		bool						pending_;
		
		static const Index<1>		DEFAULT_BUF_SIZ_;
};

// implementation
template<class T> const Index<1> WeightedBin2D<T>::DEFAULT_BUF_SIZ_ = 1000;

template<class T> 
WeightedBin2D<T>::WeightedBin2D(const Bounds<T>& xbds,const Bounds<T>& ybds) : 
	x_bounds_(xbds),y_bounds_(ybds),
	bins_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	weights_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	buf_(IndexRange<1>(1,DEFAULT_BUF_SIZ_)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
WeightedBin2D<T>::WeightedBin2D(const Bounds<T>& xbds,const Bounds<T>& ybds,const Int32& buf_siz) : 
	x_bounds_(xbds),y_bounds_(ybds),
	bins_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	weights_(IndexRange<2>(Index<2>(xbds.first(),ybds.first()),Index<2>(xbds.last(),ybds.last()))),
	buf_(IndexRange<1>(1,buf_siz)),
	buf_idx_(1),pending_(false)
{ }

template<class T> 
inline void WeightedBin2D<T>::add(const T& x,const T& y,const T& z,const T& w)
{
	add_to_buffer(x,y,z,w);
}

template<class T> 
inline void WeightedBin2D<T>::add(const Vector<T>& x,const Vector<T>& y,const Vector<T>& z,const Vector<T>& w)
{
	if (x.range() != z.range() || y.range() != z.range()) throw Err();
	
	for (Index<1> i=z.first();i<=z.last();i++)
		add_to_buffer(x[i],y[i],z[i],w[i]);
}

template<class T> 
inline void WeightedBin2D<T>::clear()
{
	// clear data
	bins_ = weights_ = 0;
	
	// clear buffer
	buf_idx_ = buf_.first();
	pending_ = false;
}

template<class T> 
inline const Matrix<T>& WeightedBin2D<T>::bins() 
{
	if (pending_) finalize();
	
	return bins_;
}

template<class T> 
inline const Matrix<T>& WeightedBin2D<T>::weights() 
{
	if (pending_) finalize();
	
	return weights_;
}

template<class T> 
inline void WeightedBin2D<T>::add_to_buffer(const T& x,const T& y,const T& z,const T& w)
{
	if (buf_idx_ > buf_.last())
	{
		finalize();
		add_to_buffer(x,y,z,w);
	}
	else 
	{
		buf_[buf_idx_][0] = x;
		buf_[buf_idx_][1] = y;
		buf_[buf_idx_][2] = z;
		buf_[buf_idx_][3] = w;
		buf_idx_++;
	}
	
	pending_ = true;
}

template<class T> 
void WeightedBin2D<T>::finalize()
{
	for (Index<1> i=buf_.first();i<=buf_idx_-1;i++)
	{
		if (isfinite(buf_[i][0]) && isfinite(buf_[i][1]) && isfinite(buf_[i][2]) && isfinite(buf_[i][3]))
		{
			const Index<2>	idx(x_bounds_.find(buf_[i][0]),
								y_bounds_.find(buf_[i][1]));
							
			bins_(idx) += buf_[i][2]*buf_[i][3];
			weights_(idx) += buf_[i][3];
		}
	}

	pending_ = false;
		buf_idx_ = buf_.first();
}

} // namespace mcs

#endif // MCS_WEIGHTEDBIN_H