#ifndef MCS_NRALIASES_H
#define MCS_NRALIASES_H

#include "MCS_FixedMatrix.h"
#include "MCS_FixedVector.h"
#include "MCS_Matrix.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

namespace num {

// Numerical Recipes vector alias base class
template<class T> class NRBaseVector
{
	public:
		NRBaseVector(T*,Int32,Int32);
		virtual ~NRBaseVector() { }

		T* data() 				{ return data_; }
		Int32 dim() const		{ return dim_; }
		Int32 offset() const	{ return offs_; }
		
		class Err { };

	protected:	
		T*			data_;
		Int32		dim_,
					offs_;
	
	private:
		NRBaseVector();
		NRBaseVector<T>& operator=(const NRBaseVector<T>&);
};

template<class T> inline NRBaseVector<T>::NRBaseVector(T* data,Int32 dim,Int32 offs) :
	data_(data),dim_(dim),offs_(offs)
{
	if (data_ == NULL || dim_ < 1) throw Err();
}

// Numerical Recipes vector alias to FixedVector<T,N>
template<class T,Uint8 N> class NRFixedVector : public NRBaseVector<T>
{
	public:
		NRFixedVector(FixedVector<T,N>&,Int32 offs = 1);
		~NRFixedVector() { }

	private:
		NRFixedVector();
		NRFixedVector<T,N>& operator=(const NRFixedVector<T,N>&);
};

template<class T,Uint8 N> inline NRFixedVector<T,N>::NRFixedVector(FixedVector<T,N>& vec,Int32 offs) :
	NRBaseVector<T>(&vec[0]-offs,vec.dim(),offs)
{ }

// Numerical Recipes vector alias to Vector<T>
template<class T> class NRVector : public NRBaseVector<T>
{
	public:
		NRVector(Vector<T>&,Int32 offs = 1);
		~NRVector() { }
		
	private:
		NRVector();
		NRVector<T>& operator=(const NRVector<T>&);
};

template<class T> inline NRVector<T>::NRVector(Vector<T>& vec,Int32 offs) :
	NRBaseVector<T>(vec.dataPointer()-offs,vec.dim(),offs)
{ }

// Numerical Recipes matrix alias base class
template<class T> class NRBaseMatrix
{
	public:
		NRBaseMatrix(T*,Int32,Int32,Int32,Int32);
		virtual ~NRBaseMatrix();

		T** data() 				{ return data_; }
		Int32 rows() const		{ return rdim_; }
		Int32 cols() const		{ return cdim_; }
		Int32 rowOffset() const	{ return roffs_; }
		Int32 colOffset() const	{ return coffs_; }
		
		class Err { };

	protected:	
		T**		data_;
		Int32	rdim_,cdim_,
				roffs_,coffs_;
	
	private:
		NRBaseMatrix();
		NRBaseMatrix<T>& operator=(const NRBaseMatrix<T>&);
};

template<class T> inline NRBaseMatrix<T>::NRBaseMatrix(T* data_ptr,Int32 rdim,Int32 roffs,Int32 cdim,Int32 coffs) :
	rdim_(rdim),roffs_(roffs),cdim_(cdim),coffs_(coffs)
{ 
	if (data_ptr == NULL || rdim_ < 1 || cdim_ < 1) throw Err();
	
	data_ = new T*[rows()];
	
	data_ -= rowOffset();
	
	for (Index<1> i=1;i<=rows();i++)
		data_[i] = &data_ptr[(i-1)*cols()]-colOffset();
}

template<class T> inline NRBaseMatrix<T>::~NRBaseMatrix()
{
	data_ += rowOffset();
	
	delete[] data_;
}

// Numerical Recipes matrix alias to FixedMatrix<T,N>
template<class T,Uint8 N> class NRFixedMatrix : public NRBaseMatrix<T>
{
	public:
		NRFixedMatrix(FixedMatrix<T,N>&,Int32 roffs = 1,Int32 coffs = 1);
		~NRFixedMatrix() { }
		
	private:
		NRFixedMatrix();
		NRFixedMatrix<T,N>& operator=(const NRFixedMatrix<T,N>&);
};

template<class T,Uint8 N> inline NRFixedMatrix<T,N>::NRFixedMatrix(FixedMatrix<T,N>& mat,Int32 roffs,Int32 coffs) :
	NRBaseMatrix<T>(&(mat[0][0]),N,roffs,N,coffs)
{ }

// Numerical Recipes matrix alias to Matrix<T>
template<class T> class NRMatrix : public NRBaseMatrix<T>
{
	public:
		NRMatrix(Matrix<T>&,Int32 roffs = 1,Int32 coffs = 1);
		~NRMatrix() { }
	
	private:
		NRMatrix();
		NRMatrix<T>& operator=(const NRMatrix<T>&);
};

template<class T> inline NRMatrix<T>::NRMatrix(Matrix<T>& mat,Int32 roffs,Int32 coffs) :
	NRBaseMatrix<T>(mat.dataPointer(),mat.dim(0),roffs,mat.dim(1),coffs)
{ }

} // namespace num

} // namespace mcs

#endif // MCS_NRALIASES_H