#include "MCS_NumericalRecipes.h"
	
namespace NR {

namespace SP {

using namespace std;

// NR 4.2 Elementary algorithms
float trapzd(float (*func)(float), float a, float b, int n)
{
	#define FUNC(x) ((*func)(x))

	float x,tnm,sum,del;
	static float s;
	int it,j;

	if (n == 1) {
		return (s=0.5*(b-a)*(FUNC(a)+FUNC(b)));
	} else {
		for (it=1,j=1;j<n-1;j++) it <<= 1;
		tnm=it;
		del=(b-a)/tnm;
		x=a+0.5*del;
		for (sum=0.0,j=1;j<=it;j++,x+=del) sum += FUNC(x);
		s=0.5*(s+(b-a)*sum/tnm);
		return s;
	}
	
	#undef FUNC
}

float qtrap(float (*func)(float), float a, float b)
{
	return qtrapr(func,a,b,trapzd);
}

float qtrapr(float (*func)(float), float a, float b,
			float (*choose)(float(*)(float), float, float, int))
{
	#define EPS 1.0e-5
	#define JMAX 20

	int j;
	float s,olds;

	olds = -1.0e30;
	for (j=1;j<=JMAX;j++) {
		s=choose(func,a,b,j);
		if (fabs(s-olds) < EPS*fabs(olds)) return s;
		olds=s;
	}
	nrerror("Too many steps in routine qtrap");
	return 0.0;

	#undef EPS
	#undef JMAX
}

float qsimp(float (*func)(float), float a, float b)
{
	return qsimpr(func,a,b,trapzd);
}

float qsimpr(float (*func)(float), float a, float b,
			float (*choose)(float(*)(float), float, float, int))
{
	#define EPS 1.0e-6
	#define JMAX 20

	int j;
	float s,st,ost,os;

	ost = os = -1.0e30;
	for (j=1;j<=JMAX;j++) {
		st=choose(func,a,b,j);
		s=(4.0*st-ost)/3.0;
		if (fabs(s-os) < EPS*fabs(os)) return s;
		os=s;
		ost=st;
	}
	nrerror("Too many steps in routine qsimp");
	return 0.0;

	#undef EPS
	#undef JMAX
}

// NR 4.3 Romberg integration
float qromb(float (*func)(float), float a, float b)
{
	return qrombr(func,a,b,trapzd);
}

float qrombr(float (*func)(float), float a, float b,
			float (*choose)(float(*)(float), float, float, int))
{
	#define EPS 1.0e-6
	#define JMAX 20
	#define JMAXP (JMAX+1)
	#define K 5

	float ss,dss;
	float s[JMAXP+1],h[JMAXP+1];
	int j;

	h[1]=1.0;
	for (j=1;j<=JMAX;j++) {
		s[j]=choose(func,a,b,j);
		if (j >= K) {
			polint(&h[j-K],&s[j-K],K,0.0,&ss,&dss);
			if (fabs(dss) < EPS*fabs(ss)) return ss;
		}
		s[j+1]=s[j];
		h[j+1]=0.25*h[j];
	}
	nrerror("Too many steps in routine qromb");
	return 0.0;

	#undef EPS
	#undef JMAX
	#undef JMAXP
	#undef K
}

// NR 4.4 Improper integrals
float midpnt(float (*func)(float), float a, float b, int n)
{
	#define FUNC(x) ((*func)(x))

	float x,tnm,sum,del,ddel;
	static float s;
	int it,j;

	if (n == 1) {
		return (s=(b-a)*FUNC(0.5*(a+b)));
	} else {
		for(it=1,j=1;j<n-1;j++) it *= 3;
		tnm=it;
		del=(b-a)/(3.0*tnm);
		ddel=del+del;
		x=a+0.5*del;
		sum=0.0;
		for (j=1;j<=it;j++) {
			sum += FUNC(x);
			x += ddel;
			sum += FUNC(x);
			x += del;
		}
		s=(s+(b-a)*sum/tnm)/3.0;
		return s;
	}

	#undef FUNC
}

float qromo(float (*func)(float), float a, float b,
	float (*choose)(float(*)(float), float, float, int))
{
	#define EPS 1.0e-6
	#define JMAX 14
	#define JMAXP (JMAX+1)
	#define K 5

	int j;
	float ss,dss,h[JMAXP+1],s[JMAXP+1];

	h[1]=1.0;
	for (j=1;j<=JMAX;j++) {
		s[j]=(*choose)(func,a,b,j);
		if (j >= K) {
			polint(&h[j-K],&s[j-K],K,0.0,&ss,&dss);
			if (fabs(dss) < EPS*fabs(ss)) return ss;
		}
		s[j+1]=s[j];
		h[j+1]=h[j]/9.0;
	}
	nrerror("Too many steps in routing qromo");
	return 0.0;

	#undef EPS
	#undef JMAX
	#undef JMAXP
	#undef K
}

float midinf(float (*funk)(float), float aa, float bb, int n)
{
	#define FUNC(x) ((*funk)(1.0/(x))/((x)*(x)))

	float x,tnm,sum,del,ddel,b,a;
	static float s;
	int it,j;

	b=1.0/aa;
	a=1.0/bb;
	if (n == 1) {
		return (s=(b-a)*FUNC(0.5*(a+b)));
	} else {
		for(it=1,j=1;j<n-1;j++) it *= 3;
		tnm=it;
		del=(b-a)/(3.0*tnm);
		ddel=del+del;
		x=a+0.5*del;
		sum=0.0;
		for (j=1;j<=it;j++) {
			sum += FUNC(x);
			x += ddel;
			sum += FUNC(x);
			x += del;
		}
		return (s=(s+(b-a)*sum/tnm)/3.0);
	}

	#undef FUNC
}

float midsql(float (*funk)(float), float aa, float bb, int n)
{
	#define FUNC(x) (2.0*(x)*(*funk)(aa+(x)*(x)))

	float x,tnm,sum,del,ddel,a,b;
	static float s;
	int it,j;

	b=sqrt(bb-aa);
	a=0.0;
	if (n == 1) {
		return (s=(b-a)*FUNC(0.5*(a+b)));
	} else {
		for(it=1,j=1;j<n-1;j++) it *= 3;
		tnm=it;
		del=(b-a)/(3.0*tnm);
		ddel=del+del;
		x=a+0.5*del;
		sum=0.0;
		for (j=1;j<=it;j++) {
			sum += FUNC(x);
			x += ddel;
			sum += FUNC(x);
			x += del;
		}
		s=(s+(b-a)*sum/tnm)/3.0;
		return s;
	}

	#undef FUNC
}

float midsqu(float (*funk)(float), float aa, float bb, int n)
{
	#define FUNC(x) (2.0*(x)*(*funk)(bb-(x)*(x)))

	float x,tnm,sum,del,ddel,a,b;
	static float s;
	int it,j;

	b=sqrt(bb-aa);
	a=0.0;
	if (n == 1) {
		return (s=(b-a)*FUNC(0.5*(a+b)));
	} else {
		for(it=1,j=1;j<n-1;j++) it *= 3;
		tnm=it;
		del=(b-a)/(3.0*tnm);
		ddel=del+del;
		x=a+0.5*del;
		sum=0.0;
		for (j=1;j<=it;j++) {
			sum += FUNC(x);
			x += ddel;
			sum += FUNC(x);
			x += del;
		}
		s=(s+(b-a)*sum/tnm)/3.0;
		return s;
	}

	#undef FUNC
}

float midexp(float (*funk)(float), float aa, float bb, int n)
{
	#define FUNC(x) ((*funk)(-log(x))/(x))

	float x,tnm,sum,del,ddel,a,b;
	static float s;
	int it,j;

	b=exp(-aa);
	a=0.0;
	if (n == 1) {
		return (s=(b-a)*FUNC(0.5*(a+b)));
	} else {
		for(it=1,j=1;j<n-1;j++) it *= 3;
		tnm=it;
		del=(b-a)/(3.0*tnm);
		ddel=del+del;
		x=a+0.5*del;
		sum=0.0;
		for (j=1;j<=it;j++) {
			sum += FUNC(x);
			x += ddel;
			sum += FUNC(x);
			x += del;
		}
		s=(s+(b-a)*sum/tnm)/3.0;
		return s;
	}

	#undef FUNC
}

// NR 4.5 Gaussian quadratures and orthogonal polynomials
float qgaus(float (*func)(float), float a, float b)
{
	int j;
	float xr,xm,dx,s;
	static float x[]={0.0,0.1488743389,0.4333953941,
		0.6794095682,0.8650633666,0.9739065285};
	static float w[]={0.0,0.2955242247,0.2692667193,
		0.2190863625,0.1494513491,0.0666713443};

	xm=0.5*(b+a);
	xr=0.5*(b-a);
	s=0;
	for (j=1;j<=5;j++) {
		dx=xr*x[j];
		s += w[j]*((*func)(xm+dx)+(*func)(xm-dx));
	}
	return s *= xr;
}

void gauleg(float x1, float x2, float x[], float w[], int n)
{
	#define EPS 3.0e-11

	int m,j,i;
	double z1,z,xm,xl,pp,p3,p2,p1;

	m=(n+1)/2;
	xm=0.5*(x2+x1);
	xl=0.5*(x2-x1);
	for (i=1;i<=m;i++) {
		z=cos(3.141592654*(i-0.25)/(n+0.5));
		do {
			p1=1.0;
			p2=0.0;
			for (j=1;j<=n;j++) {
				p3=p2;
				p2=p1;
				p1=((2.0*j-1.0)*z*p2-(j-1.0)*p3)/j;
			}
			pp=n*(z*p1-p2)/(z*z-1.0);
			z1=z;
			z=z1-p1/pp;
		} while (fabs(z-z1) > EPS);
		x[i]=xm-xl*z;
		x[n+1-i]=xm+xl*z;
		w[i]=2.0*xl/((1.0-z*z)*pp*pp);
		w[n+1-i]=w[i];
	}

	#undef EPS
}

void gaulag(float x[], float w[], int n, float alf)
{
	#define EPS 3.0e-14
	#define MAXIT 10

	int i,its,j;
	float ai;
	double p1,p2,p3,pp,z,z1;

	for (i=1;i<=n;i++) {
		if (i == 1) {
			z=(1.0+alf)*(3.0+0.92*alf)/(1.0+2.4*n+1.8*alf);
		} else if (i == 2) {
			z += (15.0+6.25*alf)/(1.0+0.9*alf+2.5*n);
		} else {
			ai=i-2;
			z += ((1.0+2.55*ai)/(1.9*ai)+1.26*ai*alf/
				(1.0+3.5*ai))*(z-x[i-2])/(1.0+0.3*alf);
		}
		for (its=1;its<=MAXIT;its++) {
			p1=1.0;
			p2=0.0;
			for (j=1;j<=n;j++) {
				p3=p2;
				p2=p1;
				p1=((2*j-1+alf-z)*p2-(j-1+alf)*p3)/j;
			}
			pp=(n*p1-(n+alf)*p2)/z;
			z1=z;
			z=z1-p1/pp;
			if (fabs(z-z1) <= EPS) break;
		}
		if (its > MAXIT) nrerror("too many iterations in gaulag");
		x[i]=z;
		w[i] = -exp(gammln(alf+n)-gammln((float)n))/(pp*n*p2);
	}

	#undef EPS
	#undef MAXIT
}

void gauher(float x[], float w[], int n)
{
	#define EPS 3.0e-14
	#define PIM4 0.7511255444649425
	#define MAXIT 10

	int i,its,j,m;
	double p1,p2,p3,pp,z,z1;

	m=(n+1)/2;
	for (i=1;i<=m;i++) {
		if (i == 1) {
			z=sqrt((double)(2*n+1))-1.85575*pow((double)(2*n+1),-0.16667);
		} else if (i == 2) {
			z -= 1.14*pow((double)n,0.426)/z;
		} else if (i == 3) {
			z=1.86*z-0.86*x[1];
		} else if (i == 4) {
			z=1.91*z-0.91*x[2];
		} else {
			z=2.0*z-x[i-2];
		}
		for (its=1;its<=MAXIT;its++) {
			p1=PIM4;
			p2=0.0;
			for (j=1;j<=n;j++) {
				p3=p2;
				p2=p1;
				p1=z*sqrt(2.0/j)*p2-sqrt(((double)(j-1))/j)*p3;
			}
			pp=sqrt((double)2*n)*p2;
			z1=z;
			z=z1-p1/pp;
			if (fabs(z-z1) <= EPS) break;
		}
		if (its > MAXIT) nrerror("too many iterations in gauher");
		x[i]=z;
		x[n+1-i] = -z;
		w[i]=2.0/(pp*pp);
		w[n+1-i]=w[i];
	}

	#undef EPS
	#undef PIM4
	#undef MAXIT
}

void gaujac(float x[], float w[], int n, float alf, float bet)
{
	#define EPS 3.0e-14
	#define MAXIT 10

	int i,its,j;
	float alfbet,an,bn,r1,r2,r3;
	double a,b,c,p1,p2,p3,pp,temp,z,z1;

	for (i=1;i<=n;i++) {
		if (i == 1) {
			an=alf/n;
			bn=bet/n;
			r1=(1.0+alf)*(2.78/(4.0+n*n)+0.768*an/n);
			r2=1.0+1.48*an+0.96*bn+0.452*an*an+0.83*an*bn;
			z=1.0-r1/r2;
		} else if (i == 2) {
			r1=(4.1+alf)/((1.0+alf)*(1.0+0.156*alf));
			r2=1.0+0.06*(n-8.0)*(1.0+0.12*alf)/n;
			r3=1.0+0.012*bet*(1.0+0.25*fabs(alf))/n;
			z -= (1.0-z)*r1*r2*r3;
		} else if (i == 3) {
			r1=(1.67+0.28*alf)/(1.0+0.37*alf);
			r2=1.0+0.22*(n-8.0)/n;
			r3=1.0+8.0*bet/((6.28+bet)*n*n);
			z -= (x[1]-z)*r1*r2*r3;
		} else if (i == n-1) {
			r1=(1.0+0.235*bet)/(0.766+0.119*bet);
			r2=1.0/(1.0+0.639*(n-4.0)/(1.0+0.71*(n-4.0)));
			r3=1.0/(1.0+20.0*alf/((7.5+alf)*n*n));
			z += (z-x[n-3])*r1*r2*r3;
		} else if (i == n) {
			r1=(1.0+0.37*bet)/(1.67+0.28*bet);
			r2=1.0/(1.0+0.22*(n-8.0)/n);
			r3=1.0/(1.0+8.0*alf/((6.28+alf)*n*n));
			z += (z-x[n-2])*r1*r2*r3;
		} else {
			z=3.0*x[i-1]-3.0*x[i-2]+x[i-3];
		}
		alfbet=alf+bet;
		for (its=1;its<=MAXIT;its++) {
			temp=2.0+alfbet;
			p1=(alf-bet+temp*z)/2.0;
			p2=1.0;
			for (j=2;j<=n;j++) {
				p3=p2;
				p2=p1;
				temp=2*j+alfbet;
				a=2*j*(j+alfbet)*(temp-2.0);
				b=(temp-1.0)*(alf*alf-bet*bet+temp*(temp-2.0)*z);
				c=2.0*(j-1+alf)*(j-1+bet)*temp;
				p1=(b*p2-c*p3)/a;
			}
			pp=(n*(alf-bet-temp*z)*p1+2.0*(n+alf)*(n+bet)*p2)/(temp*(1.0-z*z));
			z1=z;
			z=z1-p1/pp;
			if (fabs(z-z1) <= EPS) break;
		}
		if (its > MAXIT) nrerror("too many iterations in gaujac");
		x[i]=z;
		w[i]=exp(gammln(alf+n)+gammln(bet+n)-gammln(n+1.0)-
			gammln(n+alfbet+1.0))*temp*pow(2.0,alfbet)/(pp*p2);
	}

	#undef EPS
	#undef MAXIT
}

void gaucof(int n, float a[], float b[], float amu0, float x[], float w[])
{
	int i,j;
	float **z;

	z=matrix(1,n,1,n);
	for (i=1;i<=n;i++) {
		if (i != 1) b[i]=sqrt(b[i]);
		for (j=1;j<=n;j++) z[i][j]=(float)(i == j);
	}
	tqli(a,b,n,z);
	eigsrt(a,z,n);
	for (i=1;i<=n;i++) {
		x[i]=a[i];
		w[i]=amu0*z[1][i]*z[1][i];
	}
	free_matrix(z,1,n,1,n);
}

void orthog(int n, float anu[], float alpha[], float beta[], float a[],
	float b[])
{
	int k,l;
	float **sig;
	int looptmp;

	sig=matrix(1,2*n+1,1,2*n+1);
	looptmp=2*n;
	for (l=3;l<=looptmp;l++) sig[1][l]=0.0;
	looptmp++;
	for (l=2;l<=looptmp;l++) sig[2][l]=anu[l-1];
	a[1]=alpha[1]+anu[2]/anu[1];
	b[1]=0.0;
	for (k=3;k<=n+1;k++) {
		looptmp=2*n-k+3;
		for (l=k;l<=looptmp;l++) {
			sig[k][l]=sig[k-1][l+1]+(alpha[l-1]-a[k-2])*sig[k-1][l]-
				b[k-2]*sig[k-2][l]+beta[l-1]*sig[k-1][l-1];
		}
		a[k-1]=alpha[k-1]+sig[k][k+1]/sig[k][k]-sig[k-1][k]/sig[k-1][k-1];
		b[k-1]=sig[k][k]/sig[k-1][k-1];
	}
	free_matrix(sig,1,2*n+1,1,2*n+1);
}

// NR 4.6 Multidimensional integrals
float quad3d(float (*func)(float, float, float), float x1, float x2)
{
	extern float xsav,ysav;
	extern float (*nrfunc)(float,float,float);

	nrfunc=func;
	return qgaus(f1,x1,x2);
}

float f1(float x)
{
	extern float xsav;
	extern float (*yy1)(float);
	extern float (*yy2)(float);

	xsav=x;
	return qgaus(f2,yy1(x),yy2(x));
}

float f2(float y)
{
	extern float xsav,ysav;
	extern float (*z1)(float,float);
	extern float (*z2)(float,float);

	ysav=y;
	return qgaus(f3,z1(xsav,y),z2(xsav,y));
}

float f3(float z)
{
	extern float xsav,ysav;
	extern float (*nrfunc)(float,float,float);
	
	return (*nrfunc)(xsav,ysav,z);
}

} // namespace SP

} // namespace NR