//#define DEBUG
#ifndef DEBUG
	#include "MCS_Headers++.h"
#else
	#include <MSLHeaders++.h>
	
	#include "MCS_Vector.h"
	#include "MCS_Matrix.h"
	
	#include "NR.h"
#endif

using namespace NR;

// single precision
VF32 NR::Time_Series_Regression(const VF32& x,const VF32& y,Regression<Float32>& rData,Int32 order)
{
	if (x.range() != y.range()) throw Err(__FILE__,__LINE__);
	
	try
	{
		Vector<float>	s(x.range());
		
		// determine weighting coefficients by spacing between data points
		s[s.first()] = std::sqrt(fabs(x[x.first()+1]-x[x.first()]));
		s[s.last()] = std::sqrt(fabs(x[x.last()]-x[x.last()-1]));
		for (v_idx i=s.first()+1;i<=s.last()-1;i++)
			s[i] = std::sqrt(0.5*(fabs(x[i+1]-x[i])+fabs(x[i]-x[i-1])));
		s /= mean(s);
									
		return Time_Series_Regression(x,y,s,rData,order);
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF32 NR::Time_Series_Regression(const VF32& x,const VF32& y,const VF32& s,Regression<Float32>& rData,Int32 order)
{
	if (x.range() != y.range() || x.range() != s.range()) throw Err(__FILE__,__LINE__);
	
	try
	{
		Regression<double>	rDataT;
		
		Vector<double>	coef = Time_Series_Regression(x.type_cast(Float64()),y.type_cast(Float64()),s.type_cast(Float64()),rDataT,order);
		
		rData.coefficients((rDataT.coefficients()).type_cast(Float32()));
		rData.fit((rDataT.fit()).type_cast(Float32()));
		rData.residual((rDataT.residual()).type_cast(Float32()));
		rData.covariance((rDataT.covariance()).type_cast(Float32()));
		rData.chiSquare(rDataT.chiSquare());
		
		return coef.type_cast(Float32());
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

// double precision
VF64 NR::Time_Series_Regression(const VF64& x,const VF64& y,Regression<Float64>& rData,Int32 order)
{
	if (x.range() != y.range()) throw Err(__FILE__,__LINE__);
	
	try
	{
		Vector<Float64>	s(x.range());
		
		// determine weighting coefficients by spacing between data points
		s[s.first()] = std::sqrt(fabs(x[x.first()+1]-x[x.first()]));
		s[s.last()] = std::sqrt(fabs(x[x.last()]-x[x.last()-1]));
		for (v_idx i=s.first()+1;i<=s.last()-1;i++)
			s[i] = std::sqrt(0.5*(fabs(x[i+1]-x[i])+fabs(x[i]-x[i-1])));
		s /= mean(s);
									
		return (Time_Series_Regression(x,y,s,rData,order));
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF64 NR::Time_Series_Regression(const VF64& x,const VF64& y,const VF64& s,Regression<Float64>& rData,Int32 order)
{
	if (x.range() != y.range() || x.range() != s.range()) throw Err(__FILE__,__LINE__);
	
	try
	{
		rData = Regression<Float64>(2+2*order);
		
		rData.fit_function(Trig_Fit);
	
		Vector<bool> 	mask = (x.is_nan() || y.is_nan());
		Vector<Float64>	xd(x),yd(y),sd(s);
						
		xd.remove(mask);
		yd.remove(mask);
		sd.remove(mask);
							
		sd /= mean(sd);			
		
		int 	num_fit_param = 2 + 2*order;
		
		if (xd.dim() < 4*num_fit_param) throw Err(__FILE__,__LINE__);
	
		Matrix<Float64>	covar;
		Vector<Float64>	coef(v_range(1,num_fit_param));
		Float64			chisq;
	
		Matrix<Float64>	u,v;
		Vector<Float64>	w;
	
		// calculate regression coefficients
		rData.fit(svdfit(xd,yd,sd,coef,u,v,w,chisq,Trig_Fit));
		rData.coefficients(coef);
		rData.residual(y-rData.fit());
		rData.chiSquare(chisq);
		
		svdvar(v,w,covar);
		rData.covariance(covar);
		
		//Q = sqrt(1-chisq/sum(sqr((yd-mean(yd))/sd)));				// r^2
		//R = stddev(yd-fit);
		
		// MCS 2000/03/08 changed
		//rData.coefficients(Trig_Fit_Param(rData.coefficients()));
		
		return rData.coefficients();
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

