/*
********file reestimate.f90*****************************

! This module contains the user-callable routines remap and reest.

! Remap "rectifies" irregularly spaced data (y) known (with errors sig) on
! one set of x's to data (yout) on a different, generally interleaved,
! set of xout's.  The data is characterized by a single scalar population
! variance (psig**2) and a single inverse decorrelation distance w.
! All quantities are input, except yout, which is output.

! Reest is a lower-level routine that reestimates a single data set
! (x,y,sig) onto a set of improved y values yout, located at the same
! data points x.  Now psig and w are general vectors.  The optional
! argument ybar, if present, causes an unbiased (Gauss-Markov) estimate
! to be used (essentially removing and then restoring the mean of the
! data) and returns the value of the data's mean.  The optional argument
! chisq, if present, causes the chi-square of the data's internal
! consistency to be returned as chisq.

! A trivial test routine is appended (commented out).
*/
//#define DEBUG
#ifndef DEBUG
	#include "MCS_Headers++.h"
#else
	#include <MSLHeaders++.h>
	
	#include "MCS_Vector.h"
	#include "MCS_Matrix.h"
	
	#include "NR.h"
#endif

using namespace NR;
using namespace Constants::Mathematical;

VF32 NR::remap(const VF32& x,const VF32& y,const VF32& sig,
			   Float32 psig,Float32 w,const VF32& xout)
{
	return remap(x,y,sig,psig,w,true,xout);
}

VF32 NR::remap(const VF32& x,const VF32& y,const VF32& sig,
			   Float32 psig,Float32 w,bool unbiased,const VF32& xout)
{
	try
	{
		VF32::Offset	x_offs(x,1),y_offs(y,1),sig_offs(sig,1),xout_offs(xout,1);
		
		const int		n = x.dim(),
						nout = xout.dim(),
						ntot = n+nout;
		const v_range	in_rng(x.range()),
						out_rng(xout.range()),
						tot_rng(x.first(),x.first()+ntot-1);
		
		if (y.range() != in_rng || sig.range() != in_rng) throw Err(__FILE__,__LINE__);
	
		// must be monotonically increasing and unique
		if (any(diff(x)<=0.0)) throw Err(__FILE__,__LINE__);
		
		// constant value
		if (all(diff(y)==0.0)) return VF32(xout.range(),y.first_val());
		
		const Float32	ave = mean(y);
		
		VF32	xx(tot_rng),yy(tot_rng),ppsig(tot_rng),
				ssig(tot_rng),ww(tot_rng);
	
		Vector<v_idx>	iflag(tot_rng,0);
		
		v_idx	j = tot_rng.first(),k = tot_rng.first(),m;
		
		for (v_idx mm=tot_rng.first();mm<=tot_rng.last();mm++)
		{	
			if (j > n || k > nout) 
			{
				m = mm;
				break;
			}
			
			if (x[j] <= xout[k])
			{
				iflag[mm] = j;
				j++;
			}
			else
			{
				iflag[mm] = -k;
				k++;
			}
			
			m = mm;
		}
		
		if (j<=n)
			for (v_idx idx=m,i=j;idx<=ntot;idx++,i++)
				iflag[idx] = i;	
	
		if (k<=nout)
			for (v_idx idx=m,i=k;idx<=ntot;idx++,i++)
				iflag[idx] = -i;	
		
		for (v_idx idx=iflag.first();idx<=iflag.last();idx++)
		{
			if (iflag[idx] > 0)
			{
				xx[idx] = x[iflag[idx]];
				yy[idx] = y[iflag[idx]];
				ssig[idx] = sig[iflag[idx]];
				ppsig[idx] = psig;
				ww[idx] = w;
			}
			else
			{
				xx[idx] = xout[-iflag[idx]];
				yy[idx] = ave;
				ssig[idx] = 100.0*psig;
				ppsig[idx] = psig;
				ww[idx] = w;
			}
		}
		
		Float32	chisq;
		
		VF32	yyout = reest(xx,yy,ppsig,ssig,ww[v_range(ww.first(),ww.last()-1)],chisq,unbiased);
		
		return yyout.remove(iflag>0);
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF32 NR::reest(const VF32& x,const VF32& y,const VF32& psig,const VF32& sig,
			   const VF32& w,Float32& chisq,bool unbiased)
{
	static const Float32	_TINY_ = 1.0e-4;
	
	try
	{
		VF32::Offset	x_offs(x,1),y_offs(y,1),psig_offs(psig,1),sig_offs(sig,1),w_offs(w,1);
				
		v_range	rng1(x.first(),x.last()),
				rng2(x.first(),x.last()-1);
		VF32	b(rng1),temp(rng1),bb(rng1),e(rng1),
				a(rng2),c(rng2),r(rng2),aa(rng2),cc(rng2);
				
		v_idx	n = x.dim();
		
		if (y.range() != rng1 || psig.range() != rng1 || sig.range() != rng1 || w.range() != rng2)
			throw Err(__FILE__,__LINE__);
			
		if (any(diff(x)<0.0)) throw Err(__FILE__,__LINE__);
		
		temp = sqr(sig);
		
		//! compute Rybicki's tridiagonal elements
		r = exp(-abs(w*diff(x)));
		a = -static_cast<Float32>(_one_)/max((static_cast<Float32>(_one_)/r)-r,_TINY_);
		r *= a;
		
		b.first_val() = _one_-r.first_val();
		
		for (v_idx idx=b.first()+1;idx<=b.last()-1;idx++)
			b[idx] = _one_-r[idx]-r[idx-1];
			
		b.last_val() = _one_-r.last_val();
		
		for (v_idx idx=a.first();idx<=a.last();idx++)
			a[idx] /= psig[idx]*psig[idx+1];
			
		b /= sqr(psig);
		
		for (v_idx idx=a.first();idx<=a.last();idx++)
		{
			aa[idx] = a[idx]*temp[idx+1];
			cc[idx] = a[idx]*temp[idx];
		}
		
	
		bb = b*temp + static_cast<Float32>(_one_);
		
		temp = tridag(aa,bb,cc,VF32(rng1,_one_));
		e = symtrimul(a,b,temp);
		
		Float32	ybar = unbiased ? dot(e,y)/sum(e) : 0.0;
		
		temp = y-ybar;
		
		VF32	yout = tridag(aa,bb,cc,temp);
		
		chisq = dot(temp,symtrimul(a,b,yout));
		yout += ybar;
		
		return yout;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF64 NR::remap(const VF64& x,const VF64& y,const VF64& sig,
			   		  Float64 psig,Float64 w,const VF64& xout)
{
	return remap(x,y,sig,psig,w,true,xout);
}

VF64 NR::remap(const VF64& x,const VF64& y,const VF64& sig,
			   Float64 psig,Float64 w,bool unbiased,const VF64& xout)
{
	try
	{
		VF64::Offset	x_offs(x,1),y_offs(y,1),sig_offs(sig,1),xout_offs(xout,1);
							
		const int		n = x.dim(),
						nout = xout.dim(),
						ntot = n+nout;
		const v_range	in_rng(x.range()),
						out_rng(xout.range()),
						tot_rng(x.first(),x.first()+ntot-1);
		
		if (y.range() != in_rng || sig.range() != in_rng) throw Err(__FILE__,__LINE__);
		
		if (any(diff(x)<=0.0)) throw Err(__FILE__,__LINE__);
		
		if (all(diff(y)==0.0)) return VF64(xout.range(),y.first_val());
		
		const Float64	ave = mean(y);
		
		VF64	xx(tot_rng),yy(tot_rng),ppsig(tot_rng),
				ssig(tot_rng),ww(tot_rng);
	
		Vector<v_idx>	iflag(tot_rng,0);
		
		v_idx	j = tot_rng.first(),k = tot_rng.first(),m;
		
		for (v_idx mm=tot_rng.first();mm<=tot_rng.last();mm++)
		{	
			if (j > n || k > nout) 
			{
				m = mm;
				break;
			}
			
			if (x[j] <= xout[k])
			{
				iflag[mm] = j;
				j++;
			}
			else
			{
				iflag[mm] = -k;
				k++;
			}
			
			m = mm;
		}
		
		if (j<=n)
			for (v_idx idx=m,i=j;idx<=ntot;idx++,i++)
				iflag[idx] = i;	
	
		if (k<=nout)
			for (v_idx idx=m,i=k;idx<=ntot;idx++,i++)
				iflag[idx] = -i;	
		
		for (v_idx idx=iflag.first();idx<=iflag.last();idx++)
		{
			if (iflag[idx] > 0)
			{
				xx[idx] = x[iflag[idx]];
				yy[idx] = y[iflag[idx]];
				ssig[idx] = sig[iflag[idx]];
				ppsig[idx] = psig;
				ww[idx] = w;
			}
			else
			{
				xx[idx] = xout[-iflag[idx]];
				yy[idx] = ave;
				ssig[idx] = 100.0*psig;
				ppsig[idx] = psig;
				ww[idx] = w;
			}
		}
		
		Float64	chisq;
		
		VF64	yyout = reest(xx,yy,ppsig,ssig,ww[v_range(ww.first(),ww.last()-1)],chisq,unbiased);
		
		yyout.remove(iflag>0);
		
		return yyout;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

//SUBROUTINE reest(x,y,psig,sig,w,yout,ybar,chisq)
VF64 NR::reest(const VF64& x,const VF64& y,const VF64& psig,const VF64& sig,
			   const VF64& w,Float64& chisq,bool unbiased)
{
	static const Float64	_TINY_ = 1.0e-8;
	
	try
	{
		VF64::Offset	x_offs(x,1),y_offs(y,1),psig_offs(psig,1),sig_offs(sig,1),w_offs(w,1);
						
		v_range	rng1(x.first(),x.last()),
				rng2(x.first(),x.last()-1);
		VF64	b(rng1),temp(rng1),bb(rng1),e(rng1),
				a(rng2),c(rng2),r(rng2),aa(rng2),cc(rng2);
				
		v_idx	n = x.dim();
		
		if (y.range() != rng1 || psig.range() != rng1 || sig.range() != rng1 || w.range() != rng2)
			throw Err(__FILE__,__LINE__);
			
		if (any(diff(x)<0.0)) throw Err(__FILE__,__LINE__);
		
		temp = sqr(sig);
		
		r = exp(-abs(w*diff(x)));
		a = -_one_/max((_one_/r)-r,_TINY_);
		r *= a;
		
		b.first_val() = _one_-r.first_val();
		
		for (v_idx idx=b.first()+1;idx<=b.last()-1;idx++)
			b[idx] = _one_-r[idx]-r[idx-1];
			
		b.last_val() = _one_-r.last_val();
		
		for (v_idx idx=a.first();idx<=a.last();idx++)
			a[idx] /= psig[idx]*psig[idx+1];
			
		b /= sqr(psig);
		
		for (v_idx idx=a.first();idx<=a.last();idx++)
		{
			aa[idx] = a[idx]*temp[idx+1];
			cc[idx] = a[idx]*temp[idx];
		}
		
		bb = b*temp + _one_;
		
		temp = tridag(aa,bb,cc,VF64(rng1,_one_));
		e = symtrimul(a,b,temp);
		
		Float64	ybar = unbiased ? dot(e,y)/sum(e) : 0.0;
		
		temp = y-ybar;
		
		VF64	yout = tridag(aa,bb,cc,temp);
		
		chisq = dot(temp,symtrimul(a,b,yout));
		yout += ybar;
		
		return yout;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}
/*
*************file lopast.f90***********************************

! The function lopast(y,x,f) returns a vector that is the low-pass
! filtered version of the data points y, at the abcissas x, with a 3dB
! cutoff frequency of f.  This is pretty much exactly as described in
! the paper.

! High pass filtering is obtained by changing the line
!	lopast=y-real(ans)
! to
!	lopast=real(ans)
! and (if you want f to be the 3dB point again) changing the
! constant 5.5380744 to 3.56427.  (If the filters were characterized
! by 6dB cutoffs instead of 3dB cutoffs, the constants would of course
! be the same -- this is only semantics in the definition of cutoff.)

! A trivial test routine is appended (commented out).

MODULE lopast_module
CONTAINS
	FUNCTION lopast(y,x,f)
	IMPLICIT NONE
	REAL, DIMENSION(:) :: y,x
	REAL :: f
	REAL, DIMENSION(size(y)) :: lopast
	INTEGER :: n
	COMPLEX :: fac
	COMPLEX, DIMENSION(size(y)) :: b,rhs,ans
	COMPLEX, DIMENSION(size(y)-1) :: r,e,w,re
	n=size(y)
	if (size(x) /= n) pause 'bad sizes in lopast'
	fac=abs(f)*5.5380744*cmplx(1.,1.)
	w=fac*abs(x(2:n)-x(1:n-1))
	r=exp(-w)  ! note this is a complex exponential !
	e=1./(1./r - r)
	re=r*e
	b=1.
	b(1:n-1)=b(1:n-1)+re
	b(2:n)=b(2:n)+re
	w = 0.5/w
	rhs(1:n-1)=w*(y(1:n-1)-y(2:n))
	rhs(n)=0.
	rhs(2:n)=rhs(2:n)+w*(y(2:n)-y(1:n-1))
	e=-e
	call ctridag(e,b,e,rhs,ans)
	lopast=y-real(ans)
	END FUNCTION lopast

	SUBROUTINE ctridag(a,b,c,r,u)
	IMPLICIT NONE
	COMPLEX, DIMENSION(:) :: a,b,c,r,u
	INTEGER :: n,j
	COMPLEX, DIMENSION(size(b)) :: gam
	COMPLEX :: bet
	n=size(b)
	if (size(a) /= n-1 .or. size(c) /= n-1 .or. size(r) /= n &
		.or. size(u) /= n) pause 'bad sizes in ctridag'
	bet=b(1)
	if(bet==(0.,0.)) pause 'Error 1 in ctridag'
	u(1)=r(1)/bet
	do j=2,n
		gam(j)=c(j-1)/bet
		bet=b(j)-a(j-1)*gam(j)
		if(bet==(0.,0.)) pause 'Error 2 in ctridag'
		u(j)=(r(j)-a(j-1)*u(j-1))/bet
	enddo
	do j=n-1,1,-1
		u(j)=u(j)-gam(j+1)*u(j+1)
	enddo
	END SUBROUTINE ctridag
END MODULE

!	PROGRAM test_lopast
!	USE lopast_module
!	INTEGER, PARAMETER :: n=200
!	REAL, DIMENSION(n) :: x,y,a,b,c,d
!	x = (/ (j+0.5*sin(0.1*real(j)),j=1,n) /)
!	y = 0.
!	y(1:20)=1.
!	a=lopast(y,x,0.01)
!	b=lopast(y,x,0.03)
!	c=lopast(y,x,0.1)
!	d=lopast(y,x,0.3)
!	do j=1,n
!		write (*,*) x(j),a(j),b(j),c(j),d(j)
!	enddo
!	END

*******************end file************************
*/