#include <cmath>
	
#include "MCS_Constants.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_TLE.h"
#include "MCS_Typedefs.h"

namespace mcs {

using namespace mcs::geo;

// update position, velocity, and geodetic info
void SGPupdate(Inertial<Float64>& pos,
				Inertial<Float64>& vel,
				Float64 rk,Float64 rdot,Float64 rvdot,
				Float64 uk,Float64 Wk,Float64 ik)
{
	using namespace SGPConstants;
	
	const Float64	cosuk = std::cos(uk),
					sinuk = std::sin(uk),
					cosWk = std::cos(Wk),
					sinWk = std::sin(Wk),
					cosik = std::cos(ik),
					sinik = std::sin(ik),
	
					Mx = -sinWk*cosik,
					My = cosWk*cosik,
					Mz = sinik,
					
					Nx = cosWk,
					Ny = sinWk,
					Nz = _zero_,
					
					Ux = Mx*sinuk + Nx*cosuk,
					Uy = My*sinuk + Ny*cosuk,
					Uz = Mz*sinuk + Nz*cosuk,
					
					Vx = Mx*cosuk - Nx*sinuk,
					Vy = My*cosuk - Ny*sinuk,
					Vz = Mz*cosuk - Nz*sinuk;

	// convert to km
	static const Float64	pos_scal = rE/aE;
	
	pos.x(rk*Ux*pos_scal);
	pos.y(rk*Uy*pos_scal);
	pos.z(rk*Uz*pos_scal);
	
	// convert to km/min
	static const Float64	vel_scal = (rE/aE)*(Time::minutesInDay()/Time::secondsInDay());
	
	vel.x((rdot*Ux+rvdot*Vx)*vel_scal);
	vel.y((rdot*Uy+rvdot*Vy)*vel_scal);
	vel.z((rdot*Uz+rvdot*Vz)*vel_scal);
	
	return;
}

KeplerModel::KeplerModel(const ElementType& elem) : elem_(elem)
{
	using namespace SGPConstants;
	
	nodover2_ = elem_.dno_dt();
	noddover6_ = elem_.d2no_dt2();
	bstar_ = elem_.bStarDrag()/aE;
	io_ = elem_.inclinationAngle();
	Wo_ = elem_.longitudeOfAscendingNode();
	eo_ = elem_.eccentricity();
	wo_ = elem_.argumentOfPerigee();
	Mo_ = elem_.meanAnomaly();
	no_ = elem_.meanMotion();
	
	// initialize various common quantities
	a1 = std::pow(ke/no_,_two_thirds_);
	delta1 = _three_fourths_*J2*std::sqr(aE/a1)*(_three_*std::sqr(std::cos(io_))-_one_)/std::pow(_one_-std::sqr(eo_),_three_halves_);
	ao = a1*(_one_+delta1*(-_one_third_+delta1*(-_one_-delta1*(134.0/81.0))));
		
	deltao = delta1*std::sqr(a1/ao);
	nopp = no_/(1+deltao);
	aopp = ao/(1-deltao);

	common_elements_initialized_ = false;
	
	return;		
}

// perigee (km)
Float64 KeplerModel::perigee() const
{
	using namespace SGPConstants; 
	
	return (aopp*(1.0-eo_)-aE)*rE;
}

// period (sec)
Float64 KeplerModel::period() const
{
	using namespace SGPConstants; 
	
	return (_two_pi_/nopp)*Time::secondsInMinute();
}

void KeplerModel::initializeCommon()
{
	using namespace SGPConstants;
	
	if (common_elements_initialized_) return;
	
	cosio = std::cos(io_);
	cosio2 = std::sqr(cosio);
	
	x3thm1 = _three_*cosio2-_one_;
	
	sinio = std::sin(io_);
	sinio2 = std::sqr(sinio);
	
	eo2 = std::sqr(eo_);
	omeo = _one_-eo_;
	omeo2 = _one_-eo2;
	
	po = ao*omeo2;
	qo = ao*omeo;
	Lo = Mo_+wo_+Wo_;	
	dWdt = -_three_halves_*J2*std::sqr(aE/po)*no_*cosio;
	dwdt = _three_fourths_*J2*std::sqr(aE/po)*no_*(_five_*cosio2-_one_);
	
	common_elements_initialized_ = true;
	
	return;
}

void KeplerModel::propagate(Float64 dts,Inertial<Float64>& pos,Inertial<Float64>& vel)
{
	using namespace SGPConstants;
	
	initializeCommon();
		
	// convert dt to minutes
	const Float64 	dt = dts/Time::secondsInMinute();
	
	// secular effects
	const Float64	a = ao,
					e = (a > qo) ? _one_-(qo/a) : 1.0e-6,
					e2 = std::sqr(e),
					ome = _one_-e,
					ome2 = _one_-e2,
					p = a*ome2,
					Wso = Wo_+dWdt*dt,
					wso = wo_+dwdt*dt,
					Ls = Lo+dt*((no_+dwdt+dWdt));
	
	// long-period periodics
	const Float64	axNL = e*std::cos(wso),
					ayNL = e*std::sin(wso)-(_one_/_two_)*(J3/J2)*(aE/p)*sinio,
					L = Ls-(_one_/_four_)*(J3/J2)*(aE/p)*axNL*sinio*(_three_+_five_*cosio)/(_one_+cosio);

	// solve Kepler's equation
	const Float64	U = L-Wso;
	Float64			Epw = U,
					dEpw = _one_,
					cosEpw,
					sinEpw;
	int				iter = 0;
			
	while (true)
	{
		cosEpw = std::cos(Epw);
		sinEpw = std::sin(Epw);
		
		const Float64	abs_dEpw = std::abs(dEpw);
		
		if (abs_dEpw < _KEPLER_EPS_ || ++iter > _KEPLER_ITER_) break;
		
		Float64	dEpw = (U+axNL*sinEpw-ayNL*cosEpw-Epw)/(_one_-axNL*cosEpw-ayNL*sinEpw);
		
		if (abs_dEpw > _one_) dEpw = abs_dEpw/dEpw;
	
		Epw += dEpw;
	}
	
	const Float64	ecosE = axNL*cosEpw+ayNL*sinEpw,
					esinE = axNL*sinEpw-ayNL*cosEpw,
					eL2 = std::sqr(axNL)+std::sqr(ayNL),
					omeL2 = _one_-eL2,
					pL = a*omeL2,
					r = a*(_one_-ecosE),
					rdot = ke*(std::sqrt(a)/r)*esinE,
					rvdot = ke*(std::sqrt(pL)/r),
					sinu = (a/r)*(sinEpw-ayNL-axNL*(esinE/(_one_+std::sqrt(omeL2)))),
					cosu = (a/r)*(cosEpw-axNL+ayNL*(esinE/(_one_+std::sqrt(omeL2)))),
					u = atan2mod2pi(sinu,cosu);
	
	// short period perturbations
	const Float64	cos2u = _two_*std::sqr(cosu)-_one_,
					sin2u = _two_*sinu*cosu,
					rk = r+(_one_/_four_)*J2*(std::sqr(aE)/pL)*sinio2*cos2u,
					uk = u-(_one_/_eight_)*J2*std::sqr(aE/pL)*(_seven_*cosio2-_one_)*sin2u,
					Wk = Wso+(_three_/_four_)*J2*std::sqr(aE/pL)*cosio*sin2u,
					ik = io_+(_three_/_four_)*J2*std::sqr(aE/pL)*sinio*cosio*cos2u;
					
	SGPupdate(pos,vel,rk,rdot,rvdot,uk,Wk,ik);
	
	return;
}

SGPModel::SGPModel(const ElementType& elem) : elem_(elem)
{
	using namespace SGPConstants;
	
	nodover2_ = elem_.dno_dt();
	noddover6_ = elem_.d2no_dt2();
	bstar_ = elem_.bStarDrag()/aE;
	io_ = elem_.inclinationAngle();
	Wo_ = elem_.longitudeOfAscendingNode();
	eo_ = elem_.eccentricity();
	wo_ = elem_.argumentOfPerigee();
	Mo_ = elem_.meanAnomaly();
	no_ = elem_.meanMotion();
	
	// initialize various common quantities
	a1 = std::pow(ke/no_,_two_thirds_);
	delta1 = _three_fourths_*J2*std::sqr(aE/a1)*(_three_*std::sqr(std::cos(io_))-_one_)/std::pow(_one_-std::sqr(eo_),_three_halves_);
	ao = a1*(_one_+delta1*(-_one_third_+delta1*(-_one_-delta1*(134.0/81.0))));
		
	deltao = delta1*std::sqr(a1/ao);
	nopp = no_/(1+deltao);
	aopp = ao/(1-deltao);

	common_elements_initialized_ = false;
	
	return;		
}

// perigee (km)
Float64 SGPModel::perigee() const
{
	using namespace SGPConstants; 
	
	return (aopp*(1.0-eo_)-aE)*rE;
}

// period (sec)
Float64 SGPModel::period() const
{
	using namespace SGPConstants; 
	
	return (_two_pi_/nopp)*Time::secondsInMinute();
}

void SGPModel::initializeCommon()
{
	using namespace SGPConstants;
	
	if (common_elements_initialized_) return;
	
	cosio = std::cos(io_);
	cosio2 = std::sqr(cosio);
	
	x3thm1 = _three_*cosio2-_one_;
	
	sinio = std::sin(io_);
	sinio2 = std::sqr(sinio);
	
	eo2 = std::sqr(eo_);
	omeo = _one_-eo_;
	omeo2 = _one_-eo2;
	
	po = ao*omeo2;
	qo = ao*omeo;
	Lo = Mo_+wo_+Wo_;	
	dWdt = -_three_halves_*J2*std::sqr(aE/po)*no_*cosio;
	dwdt = _three_fourths_*J2*std::sqr(aE/po)*no_*(_five_*cosio2-_one_);
	
	common_elements_initialized_ = true;
	
	return;
}

void SGPModel::propagate(Float64 dts,Inertial<Float64>& pos,Inertial<Float64>& vel)
{
	using namespace SGPConstants;
	
	initializeCommon();
		
	// convert dt to minutes
	const Float64 	dt = dts/Time::secondsInMinute();
	
	// secular effects
	const Float64	a = ao*std::pow(no_/(no_+dt*(_two_*(nodover2_)+dt*_three_*(noddover6_))),_two_/_three_),
					e = (a > qo) ? _one_-(qo/a) : 1.0e-6,
					e2 = std::sqr(e),
					ome = _one_-e,
					ome2 = _one_-e2,
					p = a*ome2,
					Wso = Wo_+dWdt*dt,
					wso = wo_+dwdt*dt,
					Ls = Lo+dt*((no_+dwdt+dWdt)+dt*(nodover2_+dt*noddover6_));
	
	// long-period periodics
	const Float64	axNL = e*std::cos(wso),
					ayNL = e*std::sin(wso)-(_one_/_two_)*(J3/J2)*(aE/p)*sinio,
					L = Ls-(_one_/_four_)*(J3/J2)*(aE/p)*axNL*sinio*(_three_+_five_*cosio)/(_one_+cosio);

	// solve Kepler's equation
	const Float64	U = L-Wso;
	Float64			Epw = U,
					dEpw = _one_,
					cosEpw,
					sinEpw;
	int				iter = 0;
			
	while (true)
	{
		cosEpw = std::cos(Epw);
		sinEpw = std::sin(Epw);
		
		const Float64	abs_dEpw = std::abs(dEpw);
		
		if (abs_dEpw < _KEPLER_EPS_ || ++iter > _KEPLER_ITER_) break;
		
		Float64	dEpw = (U+axNL*sinEpw-ayNL*cosEpw-Epw)/(_one_-axNL*cosEpw-ayNL*sinEpw);
		
		if (abs_dEpw > _one_) dEpw = abs_dEpw/dEpw;
	
		Epw += dEpw;
	}
	
	const Float64	ecosE = axNL*cosEpw+ayNL*sinEpw,
					esinE = axNL*sinEpw-ayNL*cosEpw,
					eL2 = std::sqr(axNL)+std::sqr(ayNL),
					omeL2 = _one_-eL2,
					pL = a*omeL2,
					r = a*(_one_-ecosE),
					rdot = ke*(std::sqrt(a)/r)*esinE,
					rvdot = ke*(std::sqrt(pL)/r),
					sinu = (a/r)*(sinEpw-ayNL-axNL*(esinE/(_one_+std::sqrt(omeL2)))),
					cosu = (a/r)*(cosEpw-axNL+ayNL*(esinE/(_one_+std::sqrt(omeL2)))),
					u = atan2mod2pi(sinu,cosu);
	
	// short period perturbations
	const Float64	cos2u = _two_*std::sqr(cosu)-_one_,
					sin2u = _two_*sinu*cosu,
					rk = r+(_one_/_four_)*J2*(std::sqr(aE)/pL)*sinio2*cos2u,
					uk = u-(_one_/_eight_)*J2*std::sqr(aE/pL)*(_seven_*cosio2-_one_)*sin2u,
					Wk = Wso+(_three_/_four_)*J2*std::sqr(aE/pL)*cosio*sin2u,
					ik = io_+(_three_/_four_)*J2*std::sqr(aE/pL)*sinio*cosio*cos2u;
					
	SGPupdate(pos,vel,rk,rdot,rvdot,uk,Wk,ik);
	
	return;
}

static Float64 thetag(const Float64& ep)
{
	const Float64	yr = std::round((ep+2.0E-7)*1.0E-3),
					d = ep-yr*1.0E3;
					
	Int32		jy = static_cast<Int32>(yr);
					
	if (jy < 10) jy += 80;
	
	const Int32	n = (jy < 70) ? (jy-72)/4 : (jy-69)/4;
	
	const Float64	ds50 = 7305 + 365*(jy-70)+n+d,
					theta = 1.72944494 + 6.3003880987*ds50;
					
	Float64	temp = std::round(theta/_two_pi_);
	
	return std::mod(theta-temp*_two_pi_,_two_pi_);
}

SGP4SDP4Model::SGP4SDP4Model(const ElementType& elem) : elem_(elem)
{
	using namespace SGPConstants;
	
	nodover2_ = elem_.dno_dt();
	noddover6_ = elem_.d2no_dt2();
	bstar_ = elem_.bStarDrag()/aE;
	io_ = elem_.inclinationAngle();
	Wo_ = elem_.longitudeOfAscendingNode();
	eo_ = elem_.eccentricity();
	wo_ = elem_.argumentOfPerigee();
	Mo_ = elem_.meanAnomaly();
	no_ = elem_.meanMotion();
	
	// initialize constants necessary for computing period/perigee/etc...
	const Float64	a1 = std::pow(ke/no_,_two_thirds_),
					delta1 = _three_fourths_*J2*std::sqr(aE/a1)*(_three_*std::sqr(std::cos(io_))-_one_)/std::pow(_one_-std::sqr(eo_),_three_halves_),
					ao = a1*(_one_+delta1*(-_one_third_+delta1*(-_one_-delta1*(134.0/81.0)))),
					deltao = delta1*std::sqr(a1/ao);
	
	nopp_ = no_/(1+deltao);
	aopp_ = ao/(1-deltao);
	
	//thetag_ = siderealTime(epoch());
	thetag_ = thetag(1.0e3*(epoch().year()-1900)+epoch().calendarDay()+epoch().ticks()/Time::secondsInDay());
	
	// choose appropriate propagator model
	//	- orbital periods over 225 minutes use SDP4, else SGP4
	if (period() < 225*Time::secondsInMinute())
	{
		deep_space_propagator_ = false;
		sgp4_ = std::auto_ptr<SGP4>(new SGP4(*this));
	}
	else
	{
		deep_space_propagator_ = true;
		sdp4_ = std::auto_ptr<SDP4>(new SDP4(*this));
	}

	return;
}

// perigee (km)
Float64 SGP4SDP4Model::perigee() const
{
	using namespace SGPConstants; 
	
	return (aopp_*(1.0-eo_)-aE)*rE;
}

// period (sec)
Float64 SGP4SDP4Model::period() const
{
	using namespace SGPConstants; 
	
	return (_two_pi_/nopp_)*Time::secondsInMinute();
}

void SGP4SDP4Model::propagate(Float64 dts,Inertial<Float64>& pos,Inertial<Float64>& vel)
{
	// choose appropriate propagator model
	if (!deep_space_propagator_)
		sgp4_->propagate(dts,pos,vel);
	else
		sdp4_->propagate(dts,pos,vel);
}

} // namespace mcs