/********************************************************************************************/
//																							//
//	MCS_OrbitalPropagator.h																	//
//																							//	
//																							//
//																							//
//	Copyright (C) 2001 Matthias Christian Schabel											//
//																							//
//	This program is free software; you can redistribute it and/or modify it under the 		//
//	terms of the GNU General Public License as published by the Free Software Foundation; 	//
//	either version 2 of the License, or (at your option) any later version.  This program 	//
//	is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 	//
//	even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 		//
//	See the GNU General Public License for more details.									//
//																							//	
//	You should have received a copy of the GNU General Public License along with this 		//
//	program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, 			//
//	Suite 330, Boston, MA 02111-1307 USA													//
//																							//
/********************************************************************************************/

#ifndef MCS_ORBITAL_PROPAGATOR_H
#define MCS_ORBITAL_PROPAGATOR_H

#include "MCS_Date.h"
#include "MCS_Exceptions.h"
#include "MCS_GeographicCoordinates.h"
#include "MCS_MathExtensions.h"
#include "MCS_TLE.h"
#include "MCS_Typedefs.h"

namespace mcs {

namespace SGPConstants	{

static const Float64	_zero_ = 0.0,
						_one_ = 1.0,
						_two_ = 2.0,
						_three_ = 3.0,
						_four_ = 4.0,
						_five_ = 5.0,
						_six_ = 6.0,
						_seven_ = 7.0,
						_eight_ = 8.0,
						_nine_ = 9.0,
						_ten_ = 10.0,
						_one_half_ = 1.0/2.0,
						_three_halves_ = 3.0/2.0,
						_seven_halves_ = 7.0/2.0,
						_one_third_ = 1.0/3.0,
						_two_thirds_ = 2.0/3.0,
						_four_thirds_ = 4.0/3.0,
						_three_fourths_ = 3.0/4.0;

//! WGS-72 PHYSICAL AND GEOPOTENTIAL CONSTANTS
static const Float64	aE = 1.0,
						rE = 6378.135,
						//OrbitalPropagator::rE = _r_Ee_,
						ke = 7.43669161E-2,
						q_zero = 120.0,
						s_zero = 78.0,
						s = aE*(1.0+s_zero/rE),
						J2 = 1.082616E-3,
						J3 = -0.253881E-5,
						J4 = -1.65597E-6,
						k2 = J2*std::sqr(aE)/2.0,
						k4 = (-3.0/8.0)*J4*std::pow(aE,4.0),
						A30 = -J3*std::pow(aE,3.0),
						_KEPLER_EPS_ = 1.0e-6;
						
static const Uint32		_KEPLER_ITER_ = 10;

} // namespace SGPConstants

inline Float64 atan2mod2pi(const Float64& y,const Float64& x)
{
	return std::mod(std::atan2(y,x),_two_pi_);
}

// update position, velocity, and geodetic info
void SGPupdate(geo::Inertial<Float64>& pos,
				geo::Inertial<Float64>& vel,
				Float64 rk,Float64 rdot,Float64 rvdot,
				Float64 uk,Float64 Wk,Float64 ik);

// Kepler (SGP minus n. & n: terms), delta t in seconds
class KeplerModel
{
	public:
		typedef TLE ElementType;
		
		KeplerModel(const ElementType&);

		const Date& epoch() const	{ return elem_.epoch(); }
		
		Float64 perigee() const;
		Float64 period() const;
		
		void propagate(Float64 dt,geo::Inertial<Float64>& pos,geo::Inertial<Float64>& vel);
		
	private:
		void initializeCommon();
		
		ElementType			elem_;
		
		Float64				nodover2_,
							noddover6_,
							bstar_,
							io_,
							Wo_,
							eo_,
							wo_,
							Mo_,
							no_;	
		bool				common_elements_initialized_;
		
		// various working variables	
		Float64	ao,a1,aopp,
				cosio,cosio2,
				deltao,delta1,dWdt,dwdt,
				eo2,
				Lo,
				nopp,
				omeo,omeo2,
				po,
				qo,
				sinio,sinio2,
				x3thm1;
};

// SGP, delta t in seconds
class SGPModel
{
	public:
		typedef TLE ElementType;
		
		SGPModel(const ElementType&);

		const Date& epoch() const	{ return elem_.epoch(); }
		
		Float64 perigee() const;
		Float64 period() const;
		
		void propagate(Float64 dt,geo::Inertial<Float64>& pos,geo::Inertial<Float64>& vel);
		
	private:	
		void initializeCommon();
			
		ElementType			elem_;
				
		Float64				nodover2_,
							noddover6_,
							bstar_,
							io_,
							Wo_,
							eo_,
							wo_,
							Mo_,
							no_;
		bool				common_elements_initialized_;
		
		// various working variables	
		Float64	ao,a1,aopp,
				cosio,cosio2,
				deltao,delta1,dWdt,dwdt,
				eo2,
				Lo,
				nopp,
				omeo,omeo2,
				po,
				qo,
				sinio,sinio2,
				x3thm1;
};

// SGP4/SDP4, delta t in seconds
class SGP4SDP4Model
{
	public:
		typedef TLE ElementType;
		
		SGP4SDP4Model(const ElementType&);

		const Date& epoch() const	{ return elem_.epoch(); }
		
		Float64 perigee() const;
		Float64 period() const;
		
		void propagate(Float64 dt,geo::Inertial<Float64>& pos,geo::Inertial<Float64>& vel);
		
	private:
		class SGP4;	
		class SDP4;
		class DeepSpace;
		
		friend class SGP4;
		friend class SDP4;
		
		ElementType	elem_;
		
		Float64		nodover2_,
					noddover6_,
					bstar_,
					io_,
					Wo_,
					eo_,
					wo_,
					Mo_,
					no_,
					nopp_,
					aopp_,
					thetag_;
		bool		deep_space_propagator_;
					
		std::auto_ptr<SGP4>	sgp4_;
		std::auto_ptr<SDP4>	sdp4_;
};

class SGP4SDP4Model::SGP4
{
	public:
		SGP4(const SGP4SDP4Model& model);
		
		void propagate(Float64 dt,geo::Inertial<Float64>& pos,geo::Inertial<Float64>& vel);

	private:
		void initializeCommon();
		
		const SGP4SDP4Model&	model_;
		
		Float64		nodover2_,
					noddover6_,
					bstar_,
					io_,
					Wo_,
					eo_,
					wo_,
					Mo_,
					no_,
					nopp_,
					aopp_,
					thetag_;
					
		Float64		aopp2,aopp4,
					betao,betao2,betao3,betao4,betao7,betao8,
					cosio,cosio2,cosio3,cosio4,C1,C2,C3,C4,C5,
					D2,D3,D4,
					eo2,eo3,eta,eta2,
					Mdot,
					omeo,omeo2,ometa2,ometa2_72,
					qoms4,
					sinio,sinio2,sstar,
					wdot,Wdot,
					xi,xi4,x3thm1;		
};

class SGP4SDP4Model::SDP4
{
	public:
		SDP4(const SGP4SDP4Model& model);
		
		void propagate(Float64 dt,geo::Inertial<Float64>& pos,geo::Inertial<Float64>& vel);

	private:
		class DeepSpace;
		
		friend class DeepSpace;
		
		void initializeCommon();		
		
		const SGP4SDP4Model&	model_;
		
		Float64		nodover2_,
					noddover6_,
					bstar_,
					io_,
					Wo_,
					eo_,
					wo_,
					Mo_,
					no_,
					nopp_,
					aopp_,
					thetag_;		
							
		std::auto_ptr<DeepSpace>	ds_;
							
		Float64		aopp2,aopp4,
					betao,betao2,betao3,betao4,betao7,betao8,
					cosio,cosio2,cosio3,cosio4,C1,C2,C4,
					eo2,eo3,eta,eta2,
					Mdot,
					omeo,omeo2,ometa2,ometa2_72,
					qoms4,
					sinio,sinio2,sstar,
					wdot,Wdot,
					xi,xi4,x3thm1;		
};

class SGP4SDP4Model::SDP4::DeepSpace
{
	public:
		DeepSpace(const SGP4SDP4Model::SDP4& sdp4);
		
		void initialize();
		void secular(Float64& M_DF,Float64& w_DF,Float64& W_DF,Float64& e_DF,Float64& i_DF,
					 const Float64& dt);
		void periodic(Float64& M_DS,Float64& w_DS,Float64& W_DS,Float64& e_DS,Float64& i_DS,
					  const Float64& dt);
		
	private:
		static const Float64	C1L,C1SS,
								ZEL,ZES,ZNL,ZNS,
								G22,G32,G44,G52,G54,
								Q22,Q31,Q33,
								ROOT22,ROOT32,ROOT44,ROOT52,ROOT54,
								THDT,	
								ZCOSGS,ZSINGS,
								ZCOSIS,ZSINIS,
								ZCOSHS,ZSINHS;
		
		const SGP4SDP4Model::SDP4&	sdp4_;
		
		bool	iresfl,isynfl;
		Float64	atime,delt,
				d2201,d2211,
				d3210,d3222,
				d4410,d4422,
				d5220,d5232,
				d5421,d5433,
				del1,del2,del3,
				ee2,e3,
				fasx2,fasx4,fasx6,
				omegaq,
				pe,pinc,pl,
				savtsn,
				stepn,stepp,step2,
				se2,se3,si2,si3,sh2,sh3,
				sgh2,sgh3,sgh4,
				shs,shl,sghs,sghl,
				sl2,sl3,sl4,
				sse,ssg,ssh,ssi,ssl,
				xi2,xi3,xh2,xh3,
				xl2,xl3,xl4,
				xgh2,xgh3,xgh4,
				xfact,xlamo,xli,xldot,xni,xnddt,xndot,xnq,
				zm,zmol,zmos;
};

template<class M = SGP4SDP4Model> 
class OrbitalPropagator
{
	public:
		OrbitalPropagator(const typename M::ElementType&);
		~OrbitalPropagator();
		
		geo::Inertial<Float64> position() const			{ return pos_; }
		geo::Inertial<Float64> velocity() const			{ return vel_; }
		geo::Geodetic<Float64> location() const			{ updateLocation(); return geod_; }
		
		const Date& epoch() const						{ return model_.epoch(); }
		
		Float64 x() const								{ return pos_.x(); }
		Float64 y() const								{ return pos_.y(); }
		Float64 z() const								{ return pos_.z(); }
	
		Float64 vx() const								{ return vel_.x(); }
		Float64 vy() const								{ return vel_.y(); }
		Float64 vz() const								{ return vel_.z(); }
	
		Float64 altitude() const						{ updateLocation(); return geod_.h(); }
		Float64 latitude() const						{ updateLocation(); return geod_.latitude(); }
		Float64 longitude() const						{ updateLocation(); return geod_.longitude(); }
		
		Float64 perigee() const							{ return model_.perigee(); }
		Float64 period() const							{ return model_.period(); }
		
		void propagate(Float64 dt);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		// must construct new orbital propagator for each new set of line elements
		// 	otherwise static initializers in SGP algorithms will be corrupted
		OrbitalPropagator();
		OrbitalPropagator(const OrbitalPropagator<M>&);
		operator=(const OrbitalPropagator<M>&);
		
		void updateLocation() const;
				
		// propagator model
		M	model_;
		
		// position and velocity state vectors
		geo::Inertial<Float64>	pos_,
								vel_;
		Float64					last_dt_;
		
		// use lazy evaluation
		//	- conversion of Inertial->Geodetic is expensive
		mutable bool					geod_current_;
		mutable geo::Geodetic<Float64>	geod_;
};

template<class M> 
inline OrbitalPropagator<M>::OrbitalPropagator(const typename M::ElementType& element) :
	model_(element),
	pos_(_NaN_,_NaN_,_NaN_),
	vel_(_NaN_,_NaN_,_NaN_),
	geod_(geo::GeographicCoordinate<Float64>(_NaN_,_NaN_),_NaN_),
	last_dt_(_NaN_),
	geod_current_(false)
{ }

template<class M> 
inline OrbitalPropagator<M>::~OrbitalPropagator()
{ }

template<class M> 
inline void OrbitalPropagator<M>::propagate(Float64 dt) 
{ 
	if (dt != last_dt_) 
	{
		model_.propagate(dt,pos_,vel_); 
		last_dt_ = dt;
		geod_current_ = false;
	}
}

template<class M>
inline void OrbitalPropagator<M>::updateLocation() const
{
	if (geod_current_ == false)
	{
		geod_ = position().toGeodetic(Date(epoch().julian()+last_dt_/Time::secondsInDay()));
		geod_current_ = true;
	}
}

} // namespace mcs

#endif // MCS_ORBITAL_PROPAGATOR_H