#include <cmath>
	
#include "MCS_Constants.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_TLE.h"
#include "MCS_Typedefs.h"

namespace mcs {

using namespace mcs::geo;

SGP4SDP4Model::SDP4::SDP4(const SGP4SDP4Model& model) :
	model_(model)
{
	nodover2_ = model_.nodover2_;
	noddover6_ = model_.noddover6_;
	bstar_ = model_.bstar_;
	io_ = model_.io_;
	Wo_ = model_.Wo_;
	eo_ = model_.eo_;
	wo_ = model_.wo_;
	Mo_ = model_.Mo_;
	no_ = model_.no_;
	nopp_ = model_.nopp_;
	aopp_ = model_.aopp_;
	thetag_ = model_.thetag_;
	
	initializeCommon();
	
	ds_ = std::auto_ptr<DeepSpace>(new DeepSpace(*this));
	
	// initialize deep space coefficients
	ds_->initialize();
}

void SGP4SDP4Model::SDP4::initializeCommon()
{
	using namespace SGPConstants;
	
	const Float64	perige = model_.perigee();
	
	if (perige >= 156.0)
		sstar = s_zero;
	else if (perige > 98.0)
		sstar = perige-78.0;
	else 
		sstar = 20.0;	
		
	qoms4 = std::pow((q_zero-sstar)*(aE/rE),_four_);
	sstar = sstar/rE+aE;
	
	aopp2 = std::sqr(aopp_);
	aopp4 = std::sqr(aopp2);

	cosio = std::cos(io_);
	sinio = std::sin(io_);
	
	cosio2 = std::sqr(cosio);
	cosio3 = cosio2*cosio;
	cosio4 = std::sqr(cosio2);
	sinio2 = std::sqr(sinio);
		
	x3thm1 = _three_*cosio2-_one_;
	
	eo2 = std::sqr(eo_);
	eo3 = eo_*eo2;
	omeo = _one_-eo_;
	omeo2 = _one_-eo2;
	
	betao = std::sqrt(omeo2);
	betao2 = std::sqr(betao);
	betao3 = betao2*betao;
	betao4 = std::sqr(betao2);
	betao7 = betao4*betao3;
	betao8 = std::sqr(betao4);
	
	xi = _one_/(aopp_-sstar);
	xi4 = std::pow(xi,_four_);
	
	eta = aopp_*eo_*xi;
	eta2 = std::sqr(eta);
	ometa2 = _one_-eta2;
	ometa2_72 = std::pow(ometa2,-_seven_halves_);
	
	C2 = qoms4*xi4*nopp_*ometa2_72*
			(aopp_*(_one_+ eta*(_four_*eo_+eta*(_three_halves_+eta*eo_)))+
			 ((_three_halves_*k2*xi)/ometa2)*(-_one_half_+_three_halves_*cosio2)*(8.0+eta2*(24.0+eta2*3.0)));
			 
	C1 = bstar_*C2;
	
	C4 = _two_*nopp_*qoms4*xi4*aopp_*betao2*ometa2_72*
			 ((_one_half_*eo_+eta*(_two_*(_one_+eo_*eta)+_one_half_*eta2))-
			 ((_two_*k2*xi)/(aopp_*ometa2))*
			  (_three_*(-x3thm1)*(_one_+eta*(-_two_*eo_+eta*(_three_halves_-_one_half_*eta*eo_)))+
			   _three_fourths_*(_one_-cosio2)*eta*(-eo_+eta*(_two_-eta*eo_))*(_two_*std::sqr(std::cos(wo_))-_one_)));
			 
	const Float64	a2b3 = aopp2*betao*betao2,
					a2b4 = a2b3*betao,
					a4b7 = std::sqr(a2b3)*betao,
					a4b8 = a4b7*betao;

	Mdot = 	(_one_+
			(_three_halves_*k2*x3thm1)/a2b3+
			(_three_*std::sqr(k2)*(13.0+cosio2*(-78.0+cosio2*137.0)))/(16.0*a4b7))*nopp_;

	wdot = 	((-_three_halves_*k2*(_one_-_five_*cosio2))/a2b4+
			(_three_*std::sqr(k2)*(7.0+cosio2*(-114.0+cosio2*395.0)))/(16.0*a4b8)+
			(_five_*k4*(3.0+cosio2*(-36.0+cosio2*49.0)))/(_four_*a4b8))*nopp_;
			  
	Wdot = 	((-_three_*k2*cosio)/a2b4+
			(_three_halves_*std::sqr(k2)*cosio*(4.0-cosio2*19.0))/a4b8+
			(_five_*k4*cosio*(_three_-_seven_*cosio2))/(_two_*a4b8))*nopp_;
			  
	return;
}

void SGP4SDP4Model::SDP4::propagate(Float64 dts,Inertial<Float64>& pos,Inertial<Float64>& vel)
{
	using namespace SGPConstants;
			
	// convert dt to minutes
	const Float64 	dt = dts/Time::secondsInMinute(),
					dt2 = std::sqr(dt);
	
	// secular gravity and atmospheric drag
	Float64	M_DF = Mo_+Mdot*dt,
			w_DF = wo_+wdot*dt,
			W_DF = Wo_+Wdot*dt,
			e_DF = 0,
			i_DF = 0;
									  							  
	ds_->secular(M_DF,w_DF,W_DF,e_DF,i_DF,dt);
	/*
	Float64	L = M_DF+w_DF+W_DF+nopp_*_three_halves_*C1*dt2,
			w = w_DF,
			W = W_DF,
			e = e_DF-bstar_*C4*dt,
			a = aopp_*std::sqr(_one_-C1*dt),
			i = i_DF;
			*/
	Float64	M_DS = M_DF+nopp_*_three_halves_*C1*dt2,
			w_DS = w_DF,
			W_DS = W_DF,
			e_DS = e_DF-bstar_*C4*dt,
			a_DS = std::pow(ke/nopp_,_two_thirds_)*std::sqr(_one_-C1*dt),
			i_DS = i_DF;
			 
			 
	ds_->periodic(M_DS,w_DS,W_DS,e_DS,i_DS,dt);
	
	sinio = std::sin(i_DS);
	cosio = std::cos(i_DS);
	
	Float64	L = M_DS+w_DS+W_DS,
			w = w_DS,
			W = W_DS,
			e = e_DS,
			a = a_DS,
			i = i_DS;
						
	// identical to SGP4 from here on out...	 
	const Float64	b = std::sqrt(1-std::sqr(e)),
					n = ke/std::pow(a,_three_halves_);
					
	// long-period periodics
	const Float64	axN = e*std::cos(w),
					ayNL = (A30*sinio)/(4*k2*a*std::sqr(b)),
					LL = (ayNL/_two_)*axN*((_three_+_five_*cosio)/(_one_+cosio)),
					LT = L+LL,
					ayN = e*std::sin(w)+ayNL;

	// solve Kepler's equation
	// - slightly different from iteration in SGP...
	const Float64	U = LT-W;
	Float64			Epw = U,
					dEpw = _one_,
					cosEpw,
					sinEpw;
	int				iter = 0;
	
	while (true)
	{
		cosEpw = std::cos(Epw);
		sinEpw = std::sin(Epw);
		
		const Float64	dEpw = (U+axN*sinEpw-ayN*cosEpw-Epw)/(_one_-axN*cosEpw-ayN*sinEpw),
						abs_dEpw = std::abs(dEpw);
				
		Epw += dEpw;
		
		if (abs_dEpw < _KEPLER_EPS_ || ++iter > _KEPLER_ITER_) break;
	}

	const Float64	ecosE = axN*cosEpw+ayN*sinEpw,
					esinE = axN*sinEpw-ayN*cosEpw,
					eL2 = std::sqr(axN)+std::sqr(ayN),
					omeL2 = _one_-eL2,
					pL = a*omeL2,
					r = a*(_one_-ecosE),
					sinu = (a/r)*(sinEpw-ayN-axN*(esinE/(_one_+std::sqrt(omeL2)))),
					cosu = (a/r)*(cosEpw-axN+ayN*(esinE/(_one_+std::sqrt(omeL2)))),
					u = atan2mod2pi(sinu,cosu);
	
	Float64			rdot = ke*(std::sqrt(a)/r)*esinE,
					rvdot = ke*(std::sqrt(pL)/r);
					
	// short period perturbations
	const Float64	cos2u = _two_*std::sqr(cosu)-_one_,
					sin2u = _two_*sinu*cosu,
					dr = (k2/(_two_*pL))*(_one_-cosio2)*cos2u,
					du = (-k2/(_four_*std::sqr(pL)))*(_seven_*cosio2-_one_)*sin2u,
					dW = ((_three_*k2*cosio)/(_two_*std::sqr(pL)))*sin2u,
					di = ((_three_*k2*cosio)/(_two_*std::sqr(pL)))*sinio*cos2u,
					drdot = ((-k2*n)/pL)*(_one_-cosio2)*sin2u,
					drvdot = ((k2*n)/pL)*((_one_-cosio2)*cos2u-(_three_/_two_)*(_one_-_three_*cosio2)),
					
					rk = r*(_one_-(_three_/_two_)*(k2*std::sqrt(omeL2)/std::sqr(pL))*(_three_*cosio2-_one_))+dr,
					uk = u+du,
					Wk = W+dW,
					ik = i+di;
					
	rdot += drdot;
	rvdot += drvdot;
											
	SGPupdate(pos,vel,rk,rdot,rvdot,uk,Wk,ik);
	
	return;
}

} // namespace mcs