#include <cmath>
	
#include "MCS_Constants.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_TLE.h"
#include "MCS_Typedefs.h"

namespace mcs {

using namespace mcs::geo;

SGP4SDP4Model::SGP4::SGP4(const SGP4SDP4Model& model) :
	model_(model)
{
	nodover2_ = model_.nodover2_;
	noddover6_ = model_.noddover6_;
	bstar_ = model_.bstar_;
	io_ = model_.io_;
	Wo_ = model_.Wo_;
	eo_ = model_.eo_;
	wo_ = model_.wo_;
	Mo_ = model_.Mo_;
	no_ = model_.no_;
	nopp_ = model_.nopp_;
	aopp_ = model_.aopp_;
	thetag_ = model_.thetag_;
	
	initializeCommon();
}

void SGP4SDP4Model::SGP4::initializeCommon()
{
	using namespace SGPConstants;
	
	const Float64	perige = model_.perigee();
	
	if (perige >= 156.0)
		sstar = s_zero;
	else if (perige > 98.0)
		sstar = perige-78.0;
	else 
		sstar = 20.0;	
		
	qoms4 = std::pow((q_zero-sstar)*(aE/rE),_four_);
	sstar = sstar/rE+aE;
	
	aopp2 = std::sqr(aopp_);
	aopp4 = std::sqr(aopp2);

	cosio = std::cos(io_);
	sinio = std::sin(io_);
	
	cosio2 = std::sqr(cosio);
	cosio3 = cosio2*cosio;
	cosio4 = std::sqr(cosio2);
	sinio2 = std::sqr(sinio);
		
	x3thm1 = _three_*cosio2-_one_;
	
	eo2 = std::sqr(eo_);
	eo3 = eo_*eo2;
	omeo = _one_-eo_;
	omeo2 = _one_-eo2;
	
	betao = std::sqrt(omeo2);
	betao2 = std::sqr(betao);
	betao3 = betao2*betao;
	betao4 = std::sqr(betao2);
	betao7 = betao4*betao3;
	betao8 = std::sqr(betao4);
	
	xi = _one_/(aopp_-sstar);
	xi4 = std::pow(xi,_four_);
	
	eta = aopp_*eo_*xi;
	eta2 = std::sqr(eta);
	ometa2 = _one_-eta2;
	ometa2_72 = std::pow(ometa2,-_seven_halves_);
	
	C2 = qoms4*xi4*nopp_*ometa2_72*
			(aopp_*(_one_+ eta*(_four_*eo_+eta*(_three_halves_+eta*eo_)))+
			 ((_three_halves_*k2*xi)/ometa2)*(-_one_half_+_three_halves_*cosio2)*(8.0+eta2*(24.0+eta2*3.0)));
			 
	C1 = bstar_*C2;
	
	C3 = (qoms4*xi*xi4*A30*nopp_*aE*sinio)/(k2*eo_);
	
	C4 = _two_*nopp_*qoms4*xi4*aopp_*betao2*ometa2_72*
			 ((_one_half_*eo_+eta*(_two_*(_one_+eo_*eta)+_one_half_*eta2))-
			 ((_two_*k2*xi)/(aopp_*ometa2))*
			  (_three_*(-x3thm1)*(_one_+eta*(-_two_*eo_+eta*(_three_halves_-_one_half_*eta*eo_)))+
			   _three_fourths_*(_one_-cosio2)*eta*(-eo_+eta*(_two_-eta*eo_))*(_two_*std::sqr(std::cos(wo_))-_one_)));
			 
	C5 = _two_*qoms4*xi4*aopp_*betao2*std::pow(ometa2,-_seven_halves_)*(_one_+eta*((11.0/4.0)*(eta+eo_)+eta2*eo_));
	
	D2 = _four_*aopp_*xi*std::sqr(C1);
	D3 = _four_thirds_*aopp_*eta2*(17.0*aopp_+sstar)*std::pow(C1,_three_);
	D4 = _two_thirds_*aopp_*eta*eta2*(221.0*aopp_+31.0*sstar)*std::pow(C1,_four_);

	const Float64	a2b3 = aopp2*betao*betao2,
					a2b4 = a2b3*betao,
					a4b7 = std::sqr(a2b3)*betao,
					a4b8 = a4b7*betao;

	Mdot = 	(_one_+
			(_three_halves_*k2*x3thm1)/a2b3+
			(_three_*std::sqr(k2)*(13.0+cosio2*(-78.0+cosio2*137.0)))/(16.0*a4b7))*nopp_;

	wdot = 	((-_three_halves_*k2*(_one_-_five_*cosio2))/a2b4+
			(_three_*std::sqr(k2)*(7.0+cosio2*(-114.0+cosio2*395.0)))/(16.0*a4b8)+
			(_five_*k4*(3.0+cosio2*(-36.0+cosio2*49.0)))/(_four_*a4b8))*nopp_;
			  
	Wdot = 	((-_three_*k2*cosio)/a2b4+
			(_three_halves_*std::sqr(k2)*cosio*(4.0-cosio2*19.0))/a4b8+
			(_five_*k4*cosio*(_three_-_seven_*cosio2))/(_two_*a4b8))*nopp_;
			  
	return;
}

void SGP4SDP4Model::SGP4::propagate(Float64 dts,Inertial<Float64>& pos,Inertial<Float64>& vel)
{
	using namespace SGPConstants;
			
	// convert dt to minutes
	const Float64 	dt = dts/Time::secondsInMinute();
	
	// secular effects
	const Float64	dt2 = std::sqr(dt),
					dt3 = dt*dt2,
					dt4 = std::sqr(dt2),
					dt5 = dt*dt4,
					M_DF = Mo_+Mdot*dt,
					w_DF = wo_+wdot*dt,
					W_DF = Wo_+Wdot*dt;
							  
	Float64			w,W,e,a,L,i = io_;
	
	//! FOR PERIGEE LESS THAN 220 KILOMETERS, THE ISIMP FLAG IS SET AND
	//! THE EQUATIONS ARE TRUNCATED TO LINEAR VARIATION IN SQRT A AND
	//! QUADRATIC VARIATION IN MEAN ANOMALY. ALSO, THE C3 TERM, THE
	//! DELTA OMEGA TERM, AND THE DELTA M TERM ARE DROPPED.
	const bool	simple = (model_.perigee() < 220.0) ? true : false;
	
	if (!simple)
	{
		const Float64	dw = bstar_*C3*std::cos(wo_)*dt,
						dM = -_two_thirds_*qoms4*bstar_*xi4*(aE/(eo_*eta))*(std::pow(_one_+eta*std::cos(M_DF),_three_)-std::pow(_one_+eta*std::cos(Mo_),_three_)),
						Mp = M_DF+dw+dM;
						
		w = w_DF-dw-dM;
		W = W_DF-(21.0/_two_)*(nopp_*k2*cosio*C1*dt2)/(aopp2*betao2);
		e = eo_+bstar_*(-C4*dt-C5*(std::sin(Mp)-std::sin(Mo_)));
		a = aopp_*std::sqr(_one_-C1*dt-D2*dt2-D3*dt3-D4*dt4);
		L = Mp+w+W+nopp_*
			(_three_halves_*C1*dt2+
			 (D2+_two_*std::sqr(C1))*dt3+
			 (3.0*D3+12.0*C1*D2+10.0*std::pow(C1,_three_))*dt4/_four_+
			 (3.0*D4+12.0*C1*D3+6.0*std::sqr(D2)+30.0*std::sqr(C1)*D2+15.0*std::pow(C1,_four_))*dt5/_five_);
	}
	else
	{
		const Float64	Mp = M_DF;
						
		w = w_DF;
		W = W_DF-(21.0/_two_)*(nopp_*k2*cosio*C1*dt2)/(aopp2*betao2);
		e = eo_+bstar_*(-C4*dt);
		a = aopp_*std::sqr(_one_-C1*dt);
		L = Mp+w+W+nopp_*_three_halves_*C1*dt2;
	}
			 
	const Float64	b = std::sqrt(1-std::sqr(e)),
					n = ke/std::pow(a,_three_halves_);
						
	// long-period periodics
	const Float64	axN = e*std::cos(w),
					ayNL = (A30*sinio)/(4*k2*a*std::sqr(b)),
					LL = (ayNL/_two_)*axN*((_three_+_five_*cosio)/(_one_+cosio)),
					LT = L+LL,
					ayN = e*std::sin(w)+ayNL;

	// solve Kepler's equation
	// - slightly different from iteration in SGP...
	const Float64	U = LT-W;
	Float64			Epw = U,
					dEpw = _one_,
					cosEpw,
					sinEpw;
	int				iter = 0;
	
	while (true)
	{
		cosEpw = std::cos(Epw);
		sinEpw = std::sin(Epw);
		
		const Float64	dEpw = (U+axN*sinEpw-ayN*cosEpw-Epw)/(_one_-axN*cosEpw-ayN*sinEpw),
						abs_dEpw = std::abs(dEpw);
				
		Epw += dEpw;
		
		if (abs_dEpw < _KEPLER_EPS_ || ++iter > _KEPLER_ITER_) break;
	}

	const Float64	ecosE = axN*cosEpw+ayN*sinEpw,
					esinE = axN*sinEpw-ayN*cosEpw,
					eL2 = std::sqr(axN)+std::sqr(ayN),
					omeL2 = _one_-eL2,
					pL = a*omeL2,
					r = a*(_one_-ecosE),
					sinu = (a/r)*(sinEpw-ayN-axN*(esinE/(_one_+std::sqrt(omeL2)))),
					cosu = (a/r)*(cosEpw-axN+ayN*(esinE/(_one_+std::sqrt(omeL2)))),
					u = atan2mod2pi(sinu,cosu);
	
	Float64			rdot = ke*(std::sqrt(a)/r)*esinE,
					rvdot = ke*(std::sqrt(pL)/r);
					
	// short period perturbations
	const Float64	cos2u = _two_*std::sqr(cosu)-_one_,
					sin2u = _two_*sinu*cosu,
					dr = (k2/(_two_*pL))*(_one_-cosio2)*cos2u,
					du = (-k2/(_four_*std::sqr(pL)))*(_seven_*cosio2-_one_)*sin2u,
					dW = ((_three_*k2*cosio)/(_two_*std::sqr(pL)))*sin2u,
					di = ((_three_*k2*cosio)/(_two_*std::sqr(pL)))*sinio*cos2u,
					drdot = ((-k2*n)/pL)*(_one_-cosio2)*sin2u,
					drvdot = ((k2*n)/pL)*((_one_-cosio2)*cos2u-(_three_/_two_)*(_one_-_three_*cosio2)),
					
					rk = r*(_one_-(_three_/_two_)*(k2*std::sqrt(omeL2)/std::sqr(pL))*(_three_*cosio2-_one_))+dr,
					uk = u+du,
					Wk = W+dW,
					ik = i+di;
					
	rdot += drdot;
	rvdot += drvdot;
											
	SGPupdate(pos,vel,rk,rdot,rvdot,uk,Wk,ik);
	
	return;
}

} // namespace mcs