#include "MCS_FileLocations.h"
#include "MCS_FixedVector.h"
#include "MCS_Functor.h"
#include "MCS_Minimization.h"
#include "MCS_RootFinder.h"
#include "MCS_Satellite.h"
#include "MCS_Singleton.h"
#include "MCS_StringExtensions.h"

namespace mcs {

// Satellite
Satellite::Satellite(Int32 satID) : 
	SatelliteID(satID)
{
	initializeLineElements();
}

Satellite::Satellite(const SatelliteID& satID) : 
	SatelliteID(satID)
{
	initializeLineElements();
}

Satellite::Satellite(const std::string& sat_name) : 
	SatelliteID(sat_name)
{
	initializeLineElements();
}

Satellite::Satellite(const Satellite& sat) : 
	SatelliteID(sat)
{
	initializeLineElements();
}

Satellite::~Satellite()
{ }

const typename Satellite::TLEInventoryType& Satellite::getTLEInventory() const
{
	TLECacheType&	TLE_cache = Singleton<TLECacheType>::instance();
	
	TLE_cache.size(std::max(TLE_cache.size(),10));
	
	initializeTLECache(TLE_cache);
	
	return TLE_cache.get(number());
};

void Satellite::initializeTLECache(TLECacheType& cache) const
{
	if (!cache.inCache(number()))
	{
		cache.add(number());
		
		TLEInventoryType&	TLE_stack = cache.get(number());
	
		const std::string	ifname = FileLocations["TLE"]+name()+".txt";
		std::ifstream		ifs(ifname.c_str(),std::ios::in);
		TLE					tle;
		
		Int32	counter = 0;
		
		while (ifs)
		{
			try
			{
				++counter;
				
				ifs >> tle;
				
				if (tle.satelliteNumber() != number()) continue;
				
				TLE_stack.push_back(tle);
			}
			catch (TLE::Err&)
			{
				std::cout << "Line element #" << counter << " rejected..." << std::endl;
				continue; 
			}
			catch (...)
			{ break; }
		}
		
		ifs.close();
		
		// remove duplicates and sort in time order
		TLEInventoryType::iterator	end_it = unique(TLE_stack.begin(),TLE_stack.end());

		TLE_stack.erase(end_it,TLE_stack.end());
			
		sort(TLE_stack.begin(),TLE_stack.end());
		
		if (TLE_stack.size() == 0) throw Err();
	}
}

void Satellite::initializeLineElements() const
{
	const TLEInventoryType&	TLE_stack = getTLEInventory();
	
	// set to first two TLEs
	TLE_before_ = TLE_stack.begin();
	
	if (TLE_stack.size() == 1)
		TLE_after_ = TLE_before_;
	else
		TLE_after_ = TLE_before_+1;
	
	// set propagators
	propagator_before_ = std::auto_ptr<  OrbitalPropagator<SGP4SDP4Model> >(new OrbitalPropagator<SGP4SDP4Model>(*TLE_before_));
	propagator_after_ = std::auto_ptr<  OrbitalPropagator<SGP4SDP4Model> >(new OrbitalPropagator<SGP4SDP4Model>(*TLE_after_));
	
	// time interval between adjacent TLEs
	deltat_ = (TLE_before_->epoch().julian()-TLE_after_->epoch().julian());	
	
	interp_before_ = 1;
	interp_after_ = 0;		
}

// interpolate between two propagators
geo::Inertial<Float64> Satellite::position(const Date& ep) const
{
	propagate(ep);

	if (deltat_ > 1)
		return interp_before_*propagator_before_->position()+interp_after_*propagator_after_->position();
	else
		return propagator_before_->position();
}

geo::Inertial<Float64> Satellite::velocity(const Date& ep) const
{
	propagate(ep);

	if (deltat_ > 1)
		return interp_before_*propagator_before_->velocity()+interp_after_*propagator_after_->velocity();
	else
		return propagator_before_->velocity();
}

geo::Geodetic<Float64> Satellite::location(const Date& ep) const
{
	return position(ep).toGeodetic(ep);
}

Float64 Satellite::period(const Date& ep) const
{
	propagate(ep);

	if (deltat_ > 1)
		return interp_before_*propagator_before_->period()+interp_after_*propagator_after_->period();
	else
		return propagator_before_->period();
}
		
bool Satellite::ascending(const Date& ep) const
{
	const Date	epoch_dt(ep.julian()+0.1/Time::secondsInDay());
	
	return (location(ep).latitude()-location(epoch_dt).latitude() < 0);
}

bool Satellite::descending(const Date& ep) const
{
	return !ascending(ep);
}

void Satellite::propagate(const Date& ep) const
{
	// don't need to recalculate if called epoch is unchanged
	if (ep == prev_epoch_) 
		return;
	else
		prev_epoch_ = ep;
		
	const Date	start_date = TLE_before_->epoch(),
				end_date = TLE_after_->epoch();
	
	// need to get new line elements			
	if (ep < start_date || ep >= end_date)
	{
		const TLEInventoryType&	TLE_stack = getTLEInventory();
	
		// ep before first TLE -> set to first
		if (ep < TLE_stack.begin()->epoch())
		{
			TLE_before_ = TLE_after_ = TLE_stack.begin();
		}
		// ep after last TLE -> set to last
		else if (ep >= TLE_stack.rbegin()->epoch())
		{
			TLE_before_ = TLE_after_ = TLE_stack.end()-1;
		}
		// find appropriate bracketing TLEs
		else
		{
			TLE_before_ = upper_bound(TLE_stack.begin(),TLE_stack.end(),ep)-1;
			TLE_after_ = upper_bound(TLE_stack.begin(),TLE_stack.end(),ep);
		}
		
		//if (TLE_before_->epoch() > epoch || TLE_after_->epoch() < epoch) throw Err();
		
		// for ep before first or after last TLE, deltat_ == 0
		deltat_ = TLE_after_->epoch().julian()-TLE_before_->epoch().julian();
		
		if (deltat_ > 1)
			interpolate_ = true;
		else
			interpolate_ = false;
				
		// prevent memory leak	
		propagator_before_ = std::auto_ptr<  OrbitalPropagator<SGP4SDP4Model> >(new OrbitalPropagator<SGP4SDP4Model>(*TLE_before_));
		propagator_after_ = std::auto_ptr<  OrbitalPropagator<SGP4SDP4Model> >(new OrbitalPropagator<SGP4SDP4Model>(*TLE_after_));
	}
	
	// if time gap between line elements is more than 1 day
	// 	interpolate between forward and reverse propagators	
	if (interpolate_)
	{
		const Float64	deltat_before = (ep.julian()-TLE_before_->epoch().julian()),
						deltat_after = (ep.julian()-TLE_after_->epoch().julian());
		
		interp_before_ = 1-deltat_before/deltat_;
		interp_after_ = 1+deltat_after/deltat_;			
			
		propagator_before_->propagate(deltat_before*Time::secondsInDay());
		propagator_after_->propagate(deltat_after*Time::secondsInDay());
	}
	//	else just use one propagator
	else
	{
		const Float64	deltat_before = (ep.julian()-TLE_before_->epoch().julian());
		
		interp_before_ = 1;
		interp_after_ = 0;			
			
		propagator_before_->propagate(deltat_before*Time::secondsInDay());
	}
		
	return;
}

Date equatorCrossingTime(const Satellite& sat,const Date& ep)
{
	// find crossing at equator
	const Float64	RD = toRD2000(ep),
					period = sat.period(ep)/Time::secondsInDay(),
					satelliteLat = sat.location(ep).latitude();
	const bool		asc = sat.ascending(ep);
	
	if (satelliteLat == 0) return ep;

	Float64	lowerBound,
			upperBound;
			
	// get nearest equator crossing
	if ((asc && satelliteLat > 0) || (!asc && satelliteLat < 0))
	{
		lowerBound = RD-period/4;
		upperBound = RD;
	}
	else
	{
		lowerBound = RD;
		upperBound = RD+period/4;
	}
			
	// crossing time tolerance 
	static const Float64	TOL = 1e-12;

	const SatelliteLatitude	tgtLat(sat,0);
	
	num::BrentsMethodRootFinder<Float64>	tgtXingTime(tgtLat,TOL);

	num::bracketRoot(tgtLat,lowerBound,upperBound);
	
	const Float64	xing = tgtXingTime(lowerBound,upperBound);
	/*
	const SatelliteSqrLatitude	tgtLat(sat,0);
	
	num::BrentsMethodMinimizer<Float64>	tgtXingTime(tgtLat,TOL);

	const FixedVector<Float64,3>	x(lowerBound,0.5*(lowerBound+upperBound),upperBound);
	const FixedVector<Float64,2>	min = tgtXingTime(x);
	const Float64					xing = min[0];
	*/
	return fromRD2000(xing);
}

Date crossingTime(const Satellite& sat,const Date& ep,const Float64& targetLat)
{
	// find crossing at target latitude
	const Float64	RD = toRD2000(ep),
					period = sat.period(ep)/Time::secondsInDay(),
					satelliteLat = sat.location(ep).latitude();
	
	// if satellite already at right latitude, just return
	if (satelliteLat == targetLat) return ep;
	
	const Float64	eqRD = toRD2000(equatorCrossingTime(sat,ep));

	Float64	lowerBound = eqRD-period/4,
			upperBound = eqRD+period/4;
	
	// crossing time tolerance 
	static const Float64	TOL = 1e-12;

	const SatelliteLatitude	tgtLat(sat,targetLat);
	
	num::BrentsMethodRootFinder<Float64>	tgtXingTime(tgtLat,TOL);

	num::bracketRoot(tgtLat,lowerBound,upperBound);
	
	const Float64	xing = tgtXingTime(lowerBound,upperBound);
	/*
	static const Float64	TOL = 1e-8;

	const SatelliteSqrLatitude	tgtLat(sat,targetLat);
	
	num::BrentsMethodMinimizer<Float64>	tgtXingTime(tgtLat,TOL);

	const FixedVector<Float64,3>	x(lowerBound,0.5*(lowerBound+upperBound),upperBound);
	const FixedVector<Float64,2>	min = tgtXingTime(x);
	const Float64					xing = min[0];
	*/
	return fromRD2000(xing);
}

Date nearestApproach(const Satellite& sat,const Date& ep,const geo::Geodetic<Float64>& targetLoc)
{
	const Float64	RD = toRD2000(ep),
					per = sat.period(ep)/Time::secondsInDay(),
					dt = 0.5*per;
	
	const SatelliteSeparation	tgtSep(sat,targetLoc);
	
	// if satellite is already at targetLoc, just return
	if (std::sqrt(tgtSep(RD)) < 1) return ep;
		
	num::BrentsMethodMinimizer<Float64>	tgtApproach(tgtSep);
	
	const FixedVector<Float64,3>	x = num::bracketMinimum(tgtSep,RD-dt,RD+dt);		
	const FixedVector<Float64,2>	min = tgtApproach(x);
	
	// check previous and next orbits
	const Int32	orbitsPerDay(std::round(1/(2*dt)));
	Float64		currRD = min[0],
				currDist = min[1],
				newRD,
				newDist;
	bool		revised = false;
	
	//cout << currRD << "\t" << currDist << endl;
	
	// try previous orbits
	if (!revised)
	{
		for (Int32 i=-1;i>=-orbitsPerDay/2;--i)
		{
			newRD = RD+i*per;
			newDist = tgtSep(newRD);
							
			//cout << newRD << "\t" << newDist << endl << endl;
		
			if (newDist > currDist)
			{
				break;
			}
			else
			{
				currRD = newRD;
				currDist = newDist;
				revised = true;
			}
		}	
	}
	
	// try successive orbits
	if (!revised)
	{
		for (Int32 i=1;i<=orbitsPerDay/2;++i)
		{
			newRD = RD+i*per;
			newDist = tgtSep(newRD);
							
			//cout << newRD << "\t" << newDist << endl << endl;
		
			if (newDist > currDist)
			{
				break;
			}
			else
			{
				currRD = newRD;
				currDist = newDist;
			}
		}
	}	
		
	return fromRD2000(currRD);
}

} // namespace mcs

