#ifndef MCS_SATELLITE_H
#define MCS_SATELLITE_H

#include <deque>
#include <map>
#include <string>

#include "MCS_Cache.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Exceptions.h"
#include "MCS_GeographicCoordinates.h"
#include "MCS_Geography.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_SatelliteID.h"
#include "MCS_TLE.h"
#include "MCS_TwoWayDirectory.h"
#include "MCS_Typedefs.h"

namespace mcs {

// constants
static const Int32	ASC = 0,
					DSC = 1;
					
class Satellite : public SatelliteID
{
	public:
		typedef std::deque<TLE>		TLEInventoryType;
		
		Satellite(Int32 satID = 0);
		Satellite(const std::string& sat_name);
		Satellite(const SatelliteID& satID);
		~Satellite();
		
		const SatelliteID& ID() const				{ return *this; }
		
		inline geo::Inertial<Float64> position(const Date& ep) const;
		inline geo::Inertial<Float64> velocity(const Date& ep) const;
		inline geo::Inertial<Float64> attitude(const Date& ep) const;
		
		inline geo::Geodetic<Float64> location(const Date& ep) const;
		
		Float64 period(const Date& ep) const;
		
		bool ascending(const Date&) const;
		bool descending(const Date&) const;
		bool interpolated(const Date&) const;
		
		const TLE& TLEbefore(const Date& ep) const	{ propagate(ep); return *TLE_before_; }
		const TLE& TLEafter(const Date& ep) const	{ propagate(ep); return *TLE_after_; }
		
		const TLEInventoryType& getTLEInventory() const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		typedef Cache<Int32,TLEInventoryType>	TLECacheType;
		
		void initializeTLECache(TLECacheType&) const;
		void initializeLineElements() const;
		
		void propagate(const Date&) const;
				
		mutable TLEInventoryType::const_iterator	TLE_before_,
													TLE_after_;

		mutable std::auto_ptr< OrbitalPropagator<SGP4SDP4Model>	>	propagator_before_,
																	propagator_after_;
		mutable Float64	deltat_before_,
						deltat_after_,
						interp_before_,
						interp_after_,
						deltat_;
		mutable Date	prev_epoch_;
		mutable bool	interpolate_;
		
		Satellite(const Satellite& sat);
		Satellite& operator=(const Satellite& sat);
};

inline bool Satellite::interpolated(const Date& ep) const
{
	propagate(ep);
	
	return TLE_before_->interpolated() || TLE_after_->interpolated() || interpolate_;
}

// utility class
class SatelliteLatitude : public Functor1<const Float64&,Float64>
{
	public:
		SatelliteLatitude(const Satellite& sat,const Float64& lat = 0) : sat_(sat),lat_(lat) { }

		// returns latitude difference as function of time
		Float64 operator()(const Float64& rd2000) const;
		
	private:
		const Satellite&	sat_;
		const Float64		lat_;
};

inline Float64 SatelliteLatitude::operator()(const Float64& rd2000) const
{
	const Date	ep = fromRD2000(rd2000);
	
	return sat_.location(ep).latitude()-lat_;
}

class SatelliteAbsLatitude : public Functor1<const Float64&,Float64>
{
	public:
		SatelliteAbsLatitude(const Satellite& sat,const Float64& lat = 0) : sat_(sat),lat_(lat) { }

		// returns abs of latitude difference as function of time
		Float64 operator()(const Float64& rd2000) const;
		
	private:
		const Satellite&	sat_;
		const Float64		lat_;
};

inline Float64 SatelliteAbsLatitude::operator()(const Float64& rd2000) const
{
	const Date	ep = fromRD2000(rd2000);
	
	return std::abs(sat_.location(ep).latitude()-lat_);
}

class SatelliteSqrLatitude : public Functor1<const Float64&,Float64>
{
	public:
		SatelliteSqrLatitude(const Satellite& sat,const Float64& lat = 0) : sat_(sat),lat_(lat) { }

		// returns square of latitude difference as function of time
		Float64 operator()(const Float64& rd2000) const;
		
	private:
		const Satellite&	sat_;
		const Float64		lat_;
};

inline Float64 SatelliteSqrLatitude::operator()(const Float64& rd2000) const
{
	const Date	ep = fromRD2000(rd2000);
	
	return std::sqr(sat_.location(ep).latitude()-lat_);
}

class SatelliteSeparation : public Functor1<const Float64&,Float64>
{
	public:
		SatelliteSeparation(const Satellite& sat,const geo::Geodetic<Float64>& loc) : sat_(sat),loc_(loc) { }

		// returns cartesian norm of separation as function of time
		Float64 operator()(const Float64& rd2000) const;
		
	private:
		const Satellite&				sat_;
		const geo::Geodetic<Float64>&	loc_;
};

inline Float64 SatelliteSeparation::operator()(const Float64& rd2000) const
{
	const Date	ep(fromRD2000(rd2000));
	
	return norm(sat_.location(ep).toCartesian()-loc_.toCartesian());
}

Date equatorCrossingTime(const Satellite& sat,const Date& ep);
Date crossingTime(const Satellite& sat,const Date& ep,const Float64& targetLat);
Date nearestApproach(const Satellite& sat,const Date& epoch,const geo::Geodetic<Float64>& target);

} // namespace mcs

#endif // MCS_SATELLITE_H