#include <cmath>
#include <iomanip>
#include <iostream>
#include <string>

#include "MCS_Constants.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Ordered.h"
#include "MCS_StringExtensions.h"
#include "MCS_TLE.h"

namespace mcs {

const int TLE::_TLE_STRING_LENGTH_ = 69;

TLE::TLE() : 
	Ordered<Date>()
{
	line1_ = line2_ = "Invalid";
	satellite_number_ = launch_year_ = launch_number_ = element_number_ = revolution_number_ = -1;
	classification_ = ephemeris_type_ = checksum1_ = checksum2_ = '\0';
	launch_component_ = "";
	nodover2_ = noddover6_ = bstar_ = io_ = Wo_ = eo_ = wo_ = Mo_ = no_ = _NaN_;
}

TLE::TLE(const std::string& line1,const std::string& line2) : 
	Ordered<Date>(),
	line1_(line1),
	line2_(line2)
{
	if (line1_.length() != _TLE_STRING_LENGTH_ || line2_.length() != _TLE_STRING_LENGTH_) throw Err();
	
	std::string	s1 = line1_.substr(0,1),		// line number = 1	
				s2 = line1_.substr(2,5),		// satellite number
				s3 = line1_.substr(7,1),		// classification
				s4 = line1_.substr(9,2),		// international designator	: launch year
				s5 = line1_.substr(11,6),		// 							: launch number and component
				s7 = line1_.substr(18,2),		// epoch year
				s8 = line1_.substr(20,12),		// epoch fractional day
				s9 = line1_.substr(33,10),		// first time derivative of mean motion
				s10m = "."+line1_.substr(45,5),	// second time derivative of mean motion : mantissa
				s10e = line1_.substr(50,2),		//										 : exponent
				s11m = "."+line1_.substr(54,5),	// bstar drag term : mantissa
				s11e = line1_.substr(59,2),		// 				   : exponent
				s12 = line1_.substr(62,1),		// ephemeris type
				s13 = line1_.substr(64,4),		// element number
				s14 = line1_.substr(68,1),		// checksum
				t1 = line2_.substr(0,1),		// line number = 2
				t2 = line2_.substr(2,5),		// satellite number
				t3 = line2_.substr(8,8),		// inclination angle (degrees)
				t4 = line2_.substr(17,8),		// right ascension of ascending node (degrees)
				t5 = "."+line2_.substr(26,7),	// eccentricity
				t6 = line2_.substr(34,8),		// argument of perigee (degrees)
				t7 = line2_.substr(43,8),		// mean anomaly (degrees)
				t8 = line2_.substr(52,11),		// mean motion (revolutions/day)
				t9 = line2_.substr(63,5),		// revolution number at epoch
				t10 = line2_.substr(68,1);		// checksum
	
	// make sure we are looking at the right lines	
	if (s1[0] != '1' || t1[0] != '2') throw Err();
	
	// deal with vagaries in format NNN A/NNNA
	std::string::iterator	it;
	
	it = std::remove(s5.begin(),s5.end(),' ');			s5.erase(it,s5.end());
	
	// launch component
	std::string	s6;
	
	if (s5.size() == 0)
	{
		s5 = "00";
		s6 = "X";
	}
	else
	{
		s6 = s5.substr(s5.length()-1,1);
		s5 = s5.substr(0,s5.length()-1);
	}
	
	// check for possible blank strings
	it = std::remove(s2.begin(),s2.end(),' ');			s2.erase(it,s2.end());
	it = std::remove(s7.begin(),s7.end(),' ');			s7.erase(it,s7.end());
	it = std::remove(s8.begin(),s8.end(),' ');			s8.erase(it,s8.end());
	
	if (s2.size() == 0 || s7.size() == 0 || s8.size() == 0) throw Err();	

	std::replace(s3.begin(),s3.end(),' ','0');
	std::replace(s4.begin(),s4.end(),' ','0');
	std::replace(s5.begin(),s5.end(),' ','0');
	std::replace(s6.begin(),s6.end(),' ','0');
	std::replace(s9.begin(),s9.end(),' ','0');
	std::replace(s10m.begin(),s10m.end(),' ','0');
	std::replace(s10e.begin(),s10e.end(),' ','0');
	std::replace(s11m.begin(),s11m.end(),' ','0');
	std::replace(s11e.begin(),s11e.end(),' ','0');
	std::replace(s12.begin(),s12.end(),' ','0');
	std::replace(s13.begin(),s13.end(),' ','0');
	std::replace(s14.begin(),s14.end(),' ','0');
	
	it = std::remove(t2.begin(),t2.end(),' ');			t2.erase(it,t2.end());
	it = std::remove(t3.begin(),t3.end(),' ');			t3.erase(it,t3.end());
	it = std::remove(t4.begin(),t4.end(),' ');			t4.erase(it,t4.end());
	it = std::remove(t5.begin(),t5.end(),' ');			t5.erase(it,t5.end());
	it = std::remove(t6.begin(),t6.end(),' ');			t6.erase(it,t6.end());
	it = std::remove(t7.begin(),t7.end(),' ');			t7.erase(it,t7.end());
	it = std::remove(t8.begin(),t8.end(),' ');			t8.erase(it,t8.end());
	
	std::replace(t9.begin(),t9.end(),' ','0');
	std::replace(t10.begin(),t10.end(),' ','0');
	
	if (t2.size() == 0 || t3.size() == 0 || t4.size() == 0) throw Err();	
	if (t5.size() == 0 || t6.size() == 0 || t7.size() == 0 || t8.size() == 0) throw Err();	

	// need to deal with potential for blank string cases
	std::stringstream	in_string;
	
	in_string << s2 << " " << s4 << " " << s5 << " " 
			  << s6 << " " << s7 << " " << s8 << " " 
			  << s9 << " "
			  << s10m << " " << s10e << " "
			  << s11m << " " << s11e << " "
			  << s13 << " "
			  
			  << t2 << " " << t3 << " " << t4 << " "
			  << t5 << " " << t6 << " " << t7 << " "
			  << t8 << " " << t9 << " " << t10 << " ";
	
	int		epoch_yr,iexp,bexp;
	Float64	epoch_dy;
	
	try
	{
		in_string >> satellite_number_ >> launch_year_ >> launch_number_
				  >> launch_component_ >> epoch_yr >> epoch_dy
				  >> nodover2_ >> noddover6_ >> iexp >> bstar_ >> bexp
				  >> element_number_ >> satellite_number_ 
				  >> io_ >> Wo_ >> eo_ >> wo_ >> Mo_ >> no_
				  >> revolution_number_;
	}
	catch (...)
	{ throw Err(); }
	
	const Int32	epoch_dy_int = static_cast<Int32>(std::floor(epoch_dy));
	
	try
	{
		epoch(Date(Day(Y2K(epoch_yr),epoch_dy_int),Time((epoch_dy-epoch_dy_int)*Time::secondsInDay())));
	} 
	catch (Exception&)
	{ throw Err(); }

	static const Date	begin(Date(1950,1,1)),	// well before first artificial objects in orbit
						today(Date::today());
	
	// check date bounds
	if (epoch() < begin || epoch() > today) throw Err();
	
	classification_ = s3[0];
	ephemeris_type_ = s12[0];
	checksum1_ = s14[0];
	checksum2_ = t10[0];
	
	noddover6_ *= std::pow(10.0,iexp);
	bstar_ *= std::pow(10.0,bexp);
	
	// convert angular quantities to radians
	Wo_ = toRadians(Wo_);
	wo_ = toRadians(wo_);
	Mo_ = toRadians(Mo_);
	io_ = toRadians(io_);
	
	// convert units
	const Float64	conv1 = _two_pi_/Time::minutesInDay(),
					conv2 = _two_pi_/std::sqr(Time::minutesInDay()),
					conv3 = _two_pi_/std::pow(Time::minutesInDay(),3);
	
	no_ *= conv1;
	nodover2_ *= conv2;
	noddover6_ *= conv3;
	
	// need to change if aE != 1 -> e.g. for planets other than Earth
	//bstar_ /= aE;	
}

void TLE::set_parameters(const Date& ep,Float64 nodover2,Float64 noddover6,Float64 bstar,
		 				 Float64 io,Float64 Wo,Float64 eo,Float64 wo,Float64 Mo,Float64 no)
{
	epoch(ep);
	
	element_number_ = -1;
	nodover2_ = nodover2;
	noddover6_ = noddover6;
	bstar_ = bstar;
	io_ = io;
	Wo_ = Wo;
	eo_ = eo;
	wo_ = wo;
	Mo_ = Mo;
	no_ = no;
	revolution_number_ = -1;
	
	std::stringstream	tstr;
	
	tstr << (*this);
	
	std::getline(tstr,line1_);
	std::getline(tstr,line2_);
}

// output in TLE format
void TLE::print(std::ostream& os) const
{
	static const Float64	conv1 = _two_pi_/Time::minutesInDay(),
							conv2 = _two_pi_/std::sqr(Time::minutesInDay()),
							conv3 = _two_pi_/std::pow(Time::minutesInDay(),3);
		
	std::stringstream	line1,
						line2;
	
	Int32			epoch_yr = shortYear(epoch().year()),
					epoch_dy = epoch().calendarDay(),
					epoch_tm = static_cast<Int32>(1.0e8*epoch().time().ticks()/Time::secondsInDay());
	Float64			no = no_/conv1,
					nodover2 = nodover2_/conv2,
					noddover6 = noddover6_/conv3;
	Int32			noddover6_exp,
					noddover6_man,
					bstar_exp,
					bstar_man;
	char			nod_sign = nodover2 < 0 ? '-' : '+',
					nodd_sign = noddover6 < 0 ? '-' : '+',
					bstar_sign = bstar_ < 0 ? '-' : '+';
					
	noddover6_exp = noddover6 != 0.0 ? static_cast<Int32>(std::ceil(std::log10(std::abs(noddover6)))) : 0;
	noddover6_man = noddover6 != 0.0 ? static_cast<Int32>(1.0e5*std::abs(noddover6)/std::pow(10.0,noddover6_exp)) : 0;
	bstar_exp = bstar_ != 0.0 ? static_cast<Int32>(std::ceil(std::log10(std::abs(bstar_)))) : 0;
	bstar_man = bstar_ != 0.0 ? static_cast<Int32>(1.0e5*std::abs(bstar_)/std::pow(10.0,bstar_exp)) : 0;
	
	// exponents can't be more than 1 digit, mantissas 5 digits
	if (noddover6_exp <= -10) 
	{
		noddover6_exp = 0;
		noddover6_man = 0;
	}
	
	if (std::abs(noddover6_man) > 99999)
	{
		noddover6_exp -= 1;
		noddover6_man /= 10;
	}
	
	if (bstar_exp <= -10)
	{
		bstar_exp = 0;
		bstar_man = 0;
	}

	if (std::abs(bstar_man) > 99999)
	{
		bstar_exp -= 1;
		bstar_man /= 10;
	}
	
	{ using namespace std;
		
	line1 << "1 " << setw(5) << satellite_number_ << classification_ << " "
		  << setw(2) << setfill('0') << launch_year_ << setw(3) << setfill('0') << launch_number_ 
		  << setw(3) << setfill(' ') << std::right << launch_component_ << " "
		  << setw(2) << setfill('0') << epoch_yr << setw(3) << setfill('0') << epoch_dy << "." << setw(8) << setfill('0') << fixed << epoch_tm << " "
		  << nod_sign << "." << setw(8) << setfill('0') << fixed << static_cast<Int32>(std::abs(nodover2*1.0e8)) << " " 
		  << nodd_sign << setw(5) << setfill('0') << noddover6_man << "-" << setw(1) << std::abs(noddover6_exp) << " "
		  << bstar_sign << setw(5) << setfill('0') << bstar_man << "-" << setw(1) << std::abs(bstar_exp) << " " << ephemeris_type_ << " "
		  << setw(4) << setfill(' ') << element_number_ << checksum1_;
		  
	line2 << "2 " << setw(5) << satellite_number_ << " " 
		  << setw(8) << setprecision(4) << setfill(' ') << fixed << showpoint << toDegrees(io_) << " "
		  << setw(8) << setprecision(4) << setfill(' ') << fixed << showpoint << toDegrees(Wo_) << " "
		  << setw(7) << setfill('0') << static_cast<Int32>(1.0e7*eo_) << " " 
		  << setw(8) << setprecision(4) << setfill(' ') << fixed << showpoint << toDegrees(wo_) << " "
		  << setw(8) << setprecision(4) << setfill(' ') << fixed << showpoint << toDegrees(Mo_) << " "
		  << setw(11) << setprecision(8) << setfill(' ') << fixed << showpoint << no
		  << setw(5) << setfill(' ') << revolution_number_ << checksum2_;

		  
	if (line1.str().length() != _TLE_STRING_LENGTH_ || line2.str().length() != _TLE_STRING_LENGTH_) 
		throw Err();
		  
	os << line1.str() << endl << line2.str() << endl;  

	} // using namespace std
}

std::ostream& operator<<(std::ostream& ostr,const TLE& source)
{ source.print(ostr); return ostr; }

std::istream& operator>>(std::istream& istr,TLE& dest)
{ 
	std::string	line1,line2;
	
	std::getline(istr,line1);	//if (line1=="") std::getline(istr,line1);
	std::getline(istr,line2);
	
	dest = TLE(line1,line2);
	
	return istr;
}

} // namespace mcs
