#ifndef MCS_DATABASE_H
#define MCS_DATABASE_H

#include <fstream>
#include <iostream>
#include <string>

#include <boost/type_traits.hpp>

#include "MCS_Exceptions.h"
#include "MCS_IO.h"
#include "MCS_StringExtensions.h"

namespace mcs {

namespace db {

enum { ELEMENT = 0, DATA, ATTRIBUTE };

class LabelBase { };

template<Int32 T> class Label; 

// serialize dispatcher 
//
//	this class just decides whether to write (for POD types) or invoke 
//	the deserialize/serialize member functions (for classes)
//
template<class T,bool IsPOD> class Serializer;

template<class T> class Serializer<T,true>
{
	public:
		template<class Fmt> static void deserialize(T& item,const Fmt& fmt,std::istream& is)		{ fmt.read<db::DATA>(&item,is); }
		template<class Fmt> static void serialize(const T& item,const Fmt& fmt,std::ostream& os)	{ fmt.write<db::DATA>(&item,os); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,IOErr)
};

template<class T> class Serializer<T,false>
{
	public:
		template<class Fmt> static void deserialize(T& item,const Fmt& fmt,std::istream& is)		{ item.deserialize(fmt,is); }
		template<class Fmt> static void serialize(const T& item,const Fmt& fmt,std::ostream& os) 	{ item.serialize(fmt,os); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,IOErr)
};

// basic database engine
//
//	not very sophisticated at the moment...
//
template<class Fmt> class Database
{
	public:
		typedef Database<Fmt>	this_type;
		typedef Fmt				formatter_type;
		
		Database(const std::string&);
		~Database();
		
		template<class T> void store(const T& item);
		template<class T> void restore(T& item);
		
		formatter_type formatter()	{ return fmt_; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,IOErr)
		
	private:
		Database(const this_type&);
		this_type& operator=(const this_type&);
		
		void openForRead();
		void openForWrite();
		
		formatter_type	fmt_;
		std::string		filename_;
		std::ifstream	ifs_;
		std::ofstream	ofs_;
		bool			open_for_read_,
						open_for_write_;
};

template<class Fmt>
inline Database<Fmt>::Database(const std::string& filename) :
	fmt_(),
	filename_(filename),
	open_for_read_(false),
	open_for_write_(false)
{ }

template<class Fmt>
inline Database<Fmt>::~Database()
{
	ifs_.close();
	ofs_.close();
}

template<class Fmt>
template<class T>
inline void Database<Fmt>::openForRead()
{
	if (!open_for_read_)
	{
		if (open_for_write_) 
		{
			ofs_.close();
			open_for_write_ = false;
		}
		
		ifs_.open(filename_.c_str(),std::ios::binary | std::ios::in);

		if (ifs_.good())
		{
			open_for_read_ = true;
		}
		else
		{
			throw Err();
		}
	}
	
	if (!ifs_.good()) throw Err();
	
	return;
}

template<class Fmt>
template<class T>
inline void Database<Fmt>::openForWrite()
{
	if (!open_for_write_)
	{
		if (open_for_read_) 
		{
			ifs_.close();
			open_for_read_ = false;
		}
		
		//ofs_.open(filename_.c_str(),std::ios::binary | std::ios::out | std::ios::app);
		ofs_.open(filename_.c_str(),std::ios::binary | std::ios::out | std::ios::trunc);

		if (ofs_.good())
		{
			open_for_write_ = true;
		}
		else
		{
			throw Err();
		}
	}
	
	if (!ofs_.good()) throw Err();
	
	return;
}

template<class Fmt>
template<class T>
inline void Database<Fmt>::store(const T& item)
{
	openForWrite();
	
	Serializer<T,boost::is_POD<T>::value>::serialize(item,fmt_,ofs_);
	
	ofs_.flush();
}

template<class Fmt>
template<class T>
inline void Database<Fmt>::restore(T& item)
{
	openForRead();
	
	Serializer<T,boost::is_POD<T>::value>::deserialize(item,fmt_,ifs_);
}

} // namespace db

} // namespace mcs

#endif // MCS_DATABASE_H