#ifndef MCS_XMLFORMATTER_H
#define MCS_XMLFORMATTER_H

#include "MCS_ByteSwap.h"

namespace mcs {

namespace db {

class XMLReader
{
	public:
		XMLReader(std::istream& is);
		~XMLReader();
		
		void delimiter(const char c)	{ delimiter_ = c; }	

		// required methods
		void binary(const bool mode)	{ binary_mode_ = mode; }

		void openElement(const std::string& tag);
		void closeElement();
		
		template<class T> void attribute(const std::string& attribute,const T& val);
		template<class T> void data(const T& val);
		template<class T> void data(const T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		std::istream& 				is_;
		std::map<Int32,std::string>	elements_;
		
		char						delimiter_;
		bool						binary_mode_;
};

inline XMLReader::XMLReader(std::istream& is) : 
	is_(is),
	delimiter_(' '),
	binary_mode_(false)
{ }
/*
inline void XMLReader::openElement(const std::string& tag)
{
	std::string	line;
	
	std::getline(is_,line);
	
	if (line.front() != '<' || line.back() != '>') throw Err();
	
	const Int32	element_num = elements_.size()+1;
	
	elements_[element_num] = tag;
	
}

inline void XMLReader::closeElement()
{
	const Int32	element_num = elements_.size();
	
	if (prev_open_) 
	{
		os_ << ">" << std::endl;
		prev_open_ = false;
	}
	
	os_ <<  "</" << elements_[element_num] << ">" << std::endl;
	
	elements_.erase(element_num);
}

template<class T>
inline void XMLReader::readAttribute(const std::string& attribute,const T& val)
{
	if (!prev_open_) throw Err();
	
	os_ << " " << attribute << " = \"" << val << "\"";
}

template<class T>
inline void XMLReader::readData(const T& val)
{
	if (prev_open_)
	{
		os_ << ">" << std::endl;
		prev_open_ = false;
	}
	else
		throw Err();
	
	os_ << std::endl << val << std::endl;
}

template<class T>
inline void XMLReader::readData(const T vec[],Int32 sz)
{
	if (prev_open_)
	{
		os_ << ">" << std::endl;
		prev_open_ = false;
	}
	else
		throw Err();
	
	for (Int32 i=0;i<sz;++i)
		os_ << vec[i] << delimiter_;
		
	os_ << std::endl;
}

inline XMLReader::~XMLReader()
{
	while (elements_.size() > 0)
		closeElement();
}
*/

class XMLFormatter
{
	public:
		class Reader;
		class Writer;
		
		typedef Reader	reader_type;
		typedef Writer	writer_type;

		reader_type reader(std::istream& is) const;
		writer_type writer(std::ostream& os) const;	
};

class XMLFormatter::Writer
{
	public:
		Writer(std::ostream& os);
		~Writer();
		
		void delimiter(const char c)	{ delimiter_ = c; }
		
		// required methods
		void binary(const bool mode)	{ binary_mode_ = mode; }
		
		void openElement(const std::string& tag);
		void closeElement();
		
		template<class T> void attribute(const std::string& attribute,const T& val);
		template<class T> void data(const T& val);
		template<class T> void data(const T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		void closeElementTag()			{ os_ << ">" << std::endl; prev_open_ = false; }
		void closeElementTagNoEndl()	{ os_ << ">"; prev_open_ = false; }
		
		std::ostream& 				os_;
		std::map<Int32,std::string>	elements_;
		bool						prev_open_;
		
		char						delimiter_;
		bool						binary_mode_;
};

inline XMLFormatter::Writer::Writer(std::ostream& os) : 
	os_(os),
	prev_open_(false),
	delimiter_(' '),
	binary_mode_(false)
{ }

inline void XMLFormatter::Writer::openElement(const std::string& tag)
{
	const Int32	element_num = elements_.size()+1;
	
	elements_[element_num] = tag;
	
	if (prev_open_) closeElementTag();
	
	os_ << "<" << tag;
	
	prev_open_ = true;
}

inline void XMLFormatter::Writer::closeElement()
{
	const Int32	element_num = elements_.size();
	
	if (prev_open_) closeElementTag();
	
	os_ <<  "</" << elements_[element_num] << ">" << std::endl;
	
	elements_.erase(element_num);
}

template<class T>
inline void XMLFormatter::Writer::attribute(const std::string& attribute,const T& val)
{
	if (!prev_open_) throw Err();
	
	os_ << " " << attribute << " = \"" << val << "\"";
}

template<class T>
inline void XMLFormatter::Writer::data(const T& val)
{
	if (prev_open_)
		closeElementTag();
	else
		throw Err();
	
	if (binary_mode_)
	{
		static const std::string	endian = bigEndian() ? "BE" : "LE";
		
		openElement("Binary");
		
		attribute("Type",typeid(T).name());
		attribute("Size",sizeof(T));
		attribute("ByteOrder",endian);
		
		closeElementTagNoEndl();
		
		os_.write(reinterpret_cast<const char*>(&val),sizeof(T));
		
		closeElement();
	}
	else
	{
		os_ << val << std::endl;
	}
}

template<class T>
inline void XMLFormatter::Writer::data(const T vec[],Int32 sz)
{
	if (prev_open_)
		closeElementTag();
	else
		throw Err();
	
	if (binary_mode_)
	{
		static const std::string	endian = bigEndian() ? "BE" : "LE";
		
		openElement("Binary");
		
		attribute("Type",typeid(T).name());
		attribute("Size",sizeof(T));
		attribute("ByteOrder",endian);
		attribute("Number",sz);
		
		closeElementTagNoEndl();
		
		os_.write(reinterpret_cast<const char*>(&vec[0]),sz*sizeof(T));
		
		closeElement();
	}
	else
	{
		for (Int32 i=0;i<sz;++i)
			os_ << vec[i] << delimiter_;
			
		os_ << std::endl;
	}
}

inline XMLFormatter::Writer::~Writer()
{
	while (elements_.size() > 0)
		closeElement();
}
/*
inline typename XMLFormatter::reader_type XMLFormatter::reader(std::istream& is) const		
{ 
	return reader_type(is); 
}
*/
inline typename XMLFormatter::writer_type XMLFormatter::writer(std::ostream& os) const		
{ 
	return writer_type(os); 
}
		
} // namespace db

} // namespace mcs

#endif // MCS_XMLFORMATTER_H