#ifndef MCS_FETCH_ITERATOR
#define MCS_FETCH_ITERATOR

#include <iterator>

#include "MCS_Exceptions.h"

namespace mcs {

template<class C> class fetch_iterator;
template<class C> class const_fetch_iterator;

template<class C>
class fetch_iterator : 
	public std::iterator<std::random_access_iterator_tag,typename C::value_type,typename C::difference_type,typename C::pointer,typename C::reference>
{
	public:
		typedef fetch_iterator<C>				this_type;
		typedef C								container_type;
		typedef typename C::iterator_value_type	iterator_value_type;
		
		typedef typename C::value_type			value_type;
		typedef typename C::difference_type		difference_type;
		typedef typename C::pointer				pointer;
		typedef typename C::const_pointer		const_pointer;
		typedef typename C::reference			reference;
		typedef typename C::const_reference		const_reference;
				
		fetch_iterator(container_type& ref,iterator_value_type iter) : ref_(ref),iter_(iter) { }
		fetch_iterator(const this_type& source) : ref_(source.ref_),iter_(source.iter_) { }
		~fetch_iterator() { }
		
		this_type& operator=(iterator_value_type iter)							{ iter_ = iter; return *this; }		
		this_type& operator=(const this_type& it)								{ if (this == &it) return *this; ref_=it.ref_; iter_=it.iter_; return *this; }
		
		bool operator==(const this_type& it) const								{ return iter_ == it.iter_; }
		bool operator!=(const this_type& it) const								{ return iter_ != it.iter_; }
		bool operator<(const this_type& it) const								{ return iter_ < it.iter_; }
		bool operator<=(const this_type& it) const								{ return iter_ <= it.iter_; }
		bool operator>(const this_type& it) const								{ return iter_ > it.iter_; }
		bool operator>=(const this_type& it) const								{ return iter_ >= it.iter_; }
		
		this_type& operator+=(iterator_value_type num)							{ iter_ += num; return *this; }
		this_type& operator-=(iterator_value_type num)							{ iter_ -= num; return *this; }
		
		difference_type operator-(const this_type& it) const					{ return iter_-it.iter_; }
		
		this_type& operator++() 												{ ++iter_; return *this; }
		this_type& operator--()													{ --iter_; return *this; }
		const this_type operator++(int)											{ this_type tmp(*this); ++iter_; return tmp; }
		const this_type operator--(int)											{ this_type tmp(*this); --iter_; return tmp; }
		
		this_type operator+(difference_type num) const							{ this_type tmp(*this); tmp += num; return tmp; }
		this_type operator-(difference_type num) const							{ this_type tmp(*this); tmp -= num; return tmp; }
		
		reference operator*()													{ return ref_.fetch(iter_); }
		reference operator[](iterator_value_type num)							{ iter_ = num; return ref_.fetch(iter_); }
		
		pointer operator->()													{ return &ref_.fetch(iter_); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		friend class const_fetch_iterator<C>;
		
		container_type& container() const										{ return ref_; }
		iterator_value_type value() const										{ return iter_; }
		
		container_type&			ref_;
		iterator_value_type		iter_;
};

template<class C>
class const_fetch_iterator : 
	public std::iterator<std::random_access_iterator_tag,typename C::value_type,typename C::difference_type,typename C::const_pointer,typename C::const_reference>
{
	public:	
		typedef const_fetch_iterator<C>			this_type;
		typedef C								container_type;
		typedef typename C::iterator_value_type	iterator_value_type;
		
		typedef fetch_iterator<C>				fetch_iterator_type;
		
		typedef typename C::value_type			value_type;
		typedef typename C::difference_type		difference_type;
		typedef typename C::pointer				pointer;
		typedef typename C::const_pointer		const_pointer;
		typedef typename C::reference			reference;
		typedef typename C::const_reference		const_reference;
				
		const_fetch_iterator(const container_type& ref,iterator_value_type iter) : ref_(const_cast<container_type&>(ref)),iter_(iter) { }
		const_fetch_iterator(const fetch_iterator_type& it) : ref_(it.container()),iter_(it.value()) { }
		const_fetch_iterator(const this_type& source) : ref_(source.ref_),iter_(source.iter_) { }
		~const_fetch_iterator() { }
				
		this_type& operator=(iterator_value_type iter)							{ iter_ = iter; return *this; }		
		this_type& operator=(const this_type& it)								{ if (this == &it) return *this; const_cast<container_type&>(ref_)=it.ref_; iter_=it.iter_; return *this; }
		this_type& operator=(const fetch_iterator_type& it)						{ const_cast<container_type&>(ref_)=it.container();iter_=it.value(); }
		
		bool operator==(const this_type& it) const								{ return iter_ == it.iter_; }
		bool operator!=(const this_type& it) const								{ return iter_ != it.iter_; }
		bool operator<(const this_type& it) const								{ return iter_ < it.iter_; }
		bool operator<=(const this_type& it) const								{ return iter_ <= it.iter_; }
		bool operator>(const this_type& it) const								{ return iter_ > it.iter_; }
		bool operator>=(const this_type& it) const								{ return iter_ >= it.iter_; }
		
		this_type&	operator+=(iterator_value_type num)							{ iter_ += num; return *this; }
		this_type&	operator-=(iterator_value_type num)							{ iter_ -= num; return *this; }
		
		difference_type operator-(const this_type& it) const					{ return iter_-it.iter_; }
		
		this_type&	operator++() 												{ ++iter_; return *this; }
		this_type&	operator--()												{ --iter_; return *this; }
		const this_type operator++(int)											{ this_type tmp(*this); ++iter_; return tmp; }
		const this_type operator--(int)											{ this_type tmp(*this); --iter_; return tmp; }
		
		this_type operator+(difference_type num) const							{ this_type tmp(*this); tmp += num; return tmp; }
		this_type operator-(difference_type num) const							{ this_type tmp(*this); tmp -= num; return tmp; }
		
		const_reference operator*()	const										{ return ref_.fetch(iter_); }
		const_reference operator[](const iterator_value_type& num)				{ iter_ = num; return ref_.fetch(iter_); }
		
		const_pointer operator->() const										{ return &ref_.fetch(iter_); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		const container_type& container() const									{ return ref_; }
		iterator_value_type value() const										{ return iter_; }
		
		const container_type&	ref_;
		iterator_value_type		iter_;
};

} // namespace mcs

#endif // MCS_FETCH_ITERATOR