#include "MCS_ReynoldsOISSTMap.h"

#include "MCS_ByteSwap.h"
#include "MCS_Constants.h"
#include "MCS_FileLocations.h"
#include "MCS_IO.h"
#include "MCS_LandMask.h"
#include "MCS_Trace.h"

namespace mcs {

const Int32	ReynoldsOISSTMap::LAT_RANGE = 180,
			ReynoldsOISSTMap::LON_RANGE = 360,
			ReynoldsOISSTMap::MAP_SIZE = LAT_RANGE*LON_RANGE*sizeof(Float32);

// cache up to 5 weekly maps
Cache<typename ReynoldsOISSTMap::iterator_value_type,Vector<Uint8> >	ReynoldsOISSTMap::cache_(5);

ReynoldsOISSTMap::ReynoldsOISSTMap(const std::string& data_type)
{
	if (data_type == "SST") 						
	{
		data_type_ = SST;
		offset_ = constants::physical::_T_MP_;
	}
	else if (data_type == "error covariance") 		
	{
		data_type_ = ERROR_COVARIANCE;
		offset_ = 0.0;
	}
	else if (data_type == "sea ice") 				
	{
		data_type_ = SEA_ICE;
		offset_ = 0.0;
	}
	else throw Err();
}

std::string ReynoldsOISSTMap::getOISSTFilename(const Date& ep) const
{
	using namespace std;
	
	stringstream tmp;
	
	tmp << FileLocations["Reynolds OISST"] 
		<< ep.year() << "\\"
		<< "oisst." << ep.year() 
		<< std::setw(2) << std::setfill('0') << ep.month()
		<< std::setw(2) << std::setfill('0') << ep.dayOfMonth()
		<< ".gz";
		
	return tmp.str();
}

void ReynoldsOISSTMap::fetch(value_type& element,iterator_value_type iter) const
{
	if (!cache_.inCache(iter))
	{
		Vector<Uint8>&	vec = cache_.add(iter,Vector<Uint8>());

		const Date	ep = dateFromIterator(iter);
		
		const std::string	fname = getOISSTFilename(ep);
			 
		Trace<>::trace("Reading..."+fname);
		
		gz::readGZIP(fname,vec);
		
		Trace<>::trace("...done\n");
	}
	
	Vector<Uint8>&	vec = cache_.get(iter);
	
	// weird UNIX FORTRAN "feature" adds 4 bytes of crap at beginning and end of each write
	Int32	offs = 0;
	
	// 4 byte begin write
	offs += 4;
	
	const Int32	yrst = byteSwap(memoryAs<Int32>(&vec[offs])),
				mst = byteSwap(memoryAs<Int32>(&vec[offs+4])),
				dst = byteSwap(memoryAs<Int32>(&vec[offs+8])),
				yrnd = byteSwap(memoryAs<Int32>(&vec[offs+12])),
				mnd = byteSwap(memoryAs<Int32>(&vec[offs+16])),
				dnd = byteSwap(memoryAs<Int32>(&vec[offs+20])),
				ndays = byteSwap(memoryAs<Int32>(&vec[offs+24])),
				index = byteSwap(memoryAs<Int32>(&vec[offs+28]));
	
	offs = 32;
	
	// 4 byte end write
	offs += 4;
	
	element = value_type(iter);
	
	// select SST, error covariance, or sea ice and determine offset in file,
	//	taking care of 4 byte problem
	switch (data_type_)
	{
		case SST				:	break;
		case ERROR_COVARIANCE	:	offs += 4+MAP_SIZE+4; break;
		case SEA_ICE			:	offs += 2*(4+MAP_SIZE+4); break;
		default					:	throw Err();
	};
	
	// 4 byte begin write
	offs += 4;
		
	if (data_type_ == SEA_ICE)
	{
		for (Int32 i=0;i<LAT_RANGE;++i)
		{
			for (Int32 j=0;j<LON_RANGE;++j)
			{
				const Int8	val = byteSwap(memoryAs<Int8>(&vec[offs]));
				
				offs+=1;
				
				const geo::GeographicCoordinate<Float32>	coord(i-89.5,j+0.5);
				const Index<2>								idx(i,j);
				
				// land flag
				if (val == 122)
					element(idx) = _NaN_;
				else
					element(idx) = val/100.0;
			}
		}
	}
	else
	{
		for (Int32 i=0;i<LAT_RANGE;++i)
		{
			for (Int32 j=0;j<LON_RANGE;++j)
			{
				const Float32	val = byteSwap(memoryAs<Float32>(&vec[offs]));
				
				offs+=4;
				
				// multiply by land mask to remove interpolated values
				static const geo::LandMask	land_mask(1);
				
				const geo::GeographicCoordinate<Float32>	coord(i-89.5,j+0.5);
				
				const Index<2>	idx(i,j);
				
				if (land_mask(coord) == true)
					element(idx) = _NaN_;
				else
					element(idx) = val+offset_;
			}
		}
	}
	
	return;
}

} // namespace mcs