#ifndef MCS_REYNOLDSOISSTMAP_H
#define MCS_REYNOLDSOISSTMAP_H

#include "MCS_Cache.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Exceptions.h"
#include "MCS_Tensor.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

template<class DS> class Map : 
	public Tensor<Float32,2>
{
	public:
		typedef Map<DS>	this_type;
		typedef DS		dataset_type;
		
		typedef typename dataset_type::iterator_value_type	iterator_value_type;
		
		Map() : Tensor<Float32,2>(dataset_type::range())							{ epoch_ = Date(); }
		Map(iterator_value_type iter) : Tensor<Float32,2>(dataset_type::range())	{ epoch_ = dataset_type::dateFromIterator(iter); }
		
		const Date& epoch() const													{ return epoch_; }
		iterator_value_type interval() const										{ return dataset_type::iteratorFromDate(epoch_); }
		
	private:
		Map(const this_type&);
		//this_type& operator=(const this_type&);
		
		Date	epoch_;
};

class ReynoldsOISSTMap
{
	public:
		typedef Map<ReynoldsOISSTMap>	value_type;
		typedef Int32						iterator_value_type;	// week number
		
		ReynoldsOISSTMap(const std::string& variable = "SST");
		~ReynoldsOISSTMap() { }
		
		static IndexRange<2> range()									{ return IndexRange<2>(Index<2>(LAT_RANGE,LON_RANGE)); }
		
		static iterator_value_type iteratorFromDate(const Date& ep)		{ return (ep.year() < 1990) ? weekNumber(ep) : weekNumber(ep-3); }
		static Date dateFromIterator(iterator_value_type iter)			{ return (iter < iteratorFromDate(Date(1990,1,1))) ? Date(iter*7) : Date(iter*7)+3; }

		static iterator_value_type beginIteratorValue() 				{ static const Date	BEGIN(1981,11,8); return iteratorFromDate(BEGIN); }
		static iterator_value_type endIteratorValue()					{ static const Date END(Day::today().fixed()-7); return iteratorFromDate(END)+1; }		

		void fetch(value_type& element,iterator_value_type iterator_value) const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		enum { SST = 0, ERROR_COVARIANCE, SEA_ICE };
		
		static const Int32 	LAT_RANGE,
							LON_RANGE,
							MAP_SIZE; 
							
		static Cache< iterator_value_type,Vector<Uint8> >	cache_;
		
		std::string getOISSTFilename(const Date& ep) const;
		
		Int32	data_type_;
		Float32	offset_;
};

} // namespace mcs

#endif // MCS_REYNOLDSOISSTMAP_H
