#include "MCS_SSMIEDRMap.h"

#include "MCS_ByteSwap.h"
#include "MCS_Constants.h"
#include "MCS_FileLocations.h"
#include "MCS_FixedVector.h"
#include "MCS_IO.h"
#include "MCS_Trace.h"

namespace mcs {

const Int32	SSMIEDRMap::LAT_RANGE = 720,
			SSMIEDRMap::LON_RANGE = 1440,
			SSMIEDRMap::MAP_SIZE = LAT_RANGE*LON_RANGE*sizeof(Uint8),
			SSMIEDRMap::NUM_EDRS = 5;
			
// values are for C,V,T,R,W
const FixedVector<Int32,5>		SSMIEDRMap::MAP_OFFSETS(3*MAP_SIZE,2*MAP_SIZE,0,4*MAP_SIZE,MAP_SIZE);
const FixedVector<Float32,5>	SSMIEDRMap::EDR_SCALE_FACTORS(0.01,0.3,6.0/Time::minutesInDay(),0.1,0.15),
								SSMIEDRMap::EDR_OFFSETS(0.0,0.0,0.0,0.0,0.0);

// cache up to 3 SSMI daily bmaps
Cache<typename SSMIEDRMap::iterator_value_type,Vector<Uint8> >	SSMIEDRMap::cache_(3);

SSMIEDRMap::SSMIEDRMap(const SatelliteID& satID,const std::string& EDR) :
	satID_(satID),
	satname_(satID.name().substr(satID.name().length()-3,3))
{
	static const std::string	root_directory = FileLocations["SSMI bytemaps"];
	
	if (satID == SatelliteID("DMSP-F08")) 						
	{
		satnum_ = 1;
		EDR_directory_ = root_directory+"D\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1987,7,9),Time(6,0,0));
		end_date_ = Date(Day(1991,12,30),Time(18,0,0));
	}
	else if (satID == SatelliteID("DMSP-F10")) 		
	{
		satnum_ = 2;
		EDR_directory_ = root_directory+"E\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1991,1,7),Time(6,0,0));
		end_date_ = Date(Day(1997,11,14),Time(18,0,0));
	}
	else if (satID == SatelliteID("DMSP-F11")) 		
	{
		satnum_ = 3;
		EDR_directory_ = root_directory+"F\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1992,1,1),Time(6,0,0));
		end_date_ = Date(Day(2000,5,16),Time(18,0,0));
	}
	else if (satID == SatelliteID("DMSP-F13")) 		
	{
		satnum_ = 4;
		EDR_directory_ = root_directory+"G\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1995,5,3),Time(6,0,0));
		end_date_ = Date(Day::today(),Time(18,0,0));
	}
	else if (satID == SatelliteID("DMSP-F14")) 		
	{
		satnum_ = 5;
		EDR_directory_ = root_directory+"H\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1997,5,8),Time(6,0,0));
		end_date_ = Date(Day::today(),Time(18,0,0));
	}
	else if (satID == SatelliteID("DMSP-F15")) 		
	{
		satnum_ = 6;
		EDR_directory_ = root_directory+"K\\"+satname_+"\\bmaps";
		begin_date_ = Date(Day(1999,12,18),Time(6,0,0));
		end_date_ = Date(Day::today(),Time(18,0,0));
	}
	else throw Err();
	
	begin_iterator_value_ = iteratorFromDate(begin_date_);
	end_iterator_value_ = iteratorFromDate(end_date_);
	
	if (EDR == "column vapor")				EDR_ = COLUMN_VAPOR;
	else if (EDR == "cloud water")			EDR_ = CLOUD_WATER;
	else if (EDR == "observation time")		EDR_ = OBSERVATION_TIME;
	else if (EDR == "rain rate")			EDR_ = RAIN_RATE;
	else if (EDR == "wind speed")			EDR_ = WIND_SPEED;
	else throw Err();
}

std::string SSMIEDRMap::getEDRFilename(const Date& ep) const
{
	using namespace std;
	
	stringstream tmp;
	
	tmp << EDR_directory_;
		
	tmp << "\\y" << setw(2) << setfill('0') << shortYear(ep.year());
	tmp << "\\m" << setw(2) << setfill('0') << ep.month();

	tmp << "\\";
	tmp << setw(2) << setfill('0') << shortYear(ep.year());
	tmp << setw(2) << setfill('0') << ep.month();
	tmp << setw(2) << setfill('0') << ep.dayOfMonth();
	tmp << setw(2) << setfill('0') << satname_.substr(1,2) << ".gz";
		
	return tmp.str();
}

void SSMIEDRMap::fetch(value_type& element,iterator_value_type iter) const
{
	const Date	ep = dateFromIterator(iter);
	
	const Int32	day_number = iter/2;
	
	// since each file contains both half-days of data, index cache by day number, not half day number
	if (!cache_.inCache(day_number))
	{
		Vector<Uint8>&	vec = cache_.add(day_number,Vector<Uint8>());

		const std::string	fname = getEDRFilename(ep);
			 
		Trace<>::trace("Reading..."+fname);
		
		gz::readGZIP(fname,vec);
		
		Trace<>::trace("...done\n");
	}
	
	Vector<Uint8>&	vec = cache_.get(day_number);
	
	element = value_type(iter);
	
	// get appropriate file offset, skipping to second half of GZIP buffer for evening crossings
	const Int32	offset = std::even(iter) ? MAP_OFFSETS[EDR_] : MAP_OFFSETS[EDR_] + MAP_SIZE*NUM_EDRS;
	
	typedef Tensor<Uint8,2,TensorImplementation<PtrAliasAllocator<Uint8>,BasicTensorAccessor<2> > >	ptr_alias_tensor_type;
	
	ptr_alias_tensor_type	alias(range(),&(vec[offset]));
	
	const Float32	delta = (EDR_ == OBSERVATION_TIME) ? EDR_OFFSETS[EDR_]+toRD2000(ep) : EDR_OFFSETS[EDR_];
	
	for (typename ptr_alias_tensor_type::iterator it=alias.begin();it!=alias.end();++it)
	{
		const Int32 data_val = static_cast<Int32>(*it);
		
		if (data_val <= 250)
			element(it.index()) = static_cast<Float32>(data_val)*EDR_SCALE_FACTORS[EDR_]+delta;
		else
			element(it.index()) = _NaN_;
	}
			
	return;
}

} // namespace mcs