#ifndef MCS_BASICGEOLOCATION_H
#define MCS_BASICGEOLOCATION_H

#include "MCS_Exceptions.h"
#include "MCS_GeographicCoordinates.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

namespace sat {

// 	policy base class for basic geolocation data with F fields of view
//
//	specific implementations of geolocation classes should be subclasses
//
class BasicGeolocation
{
	public:
		const IndexRange<1>& fovRange() const								{ return inc_.range(); }
		
		// accessors
		const Float32& height() const										{ return ht_; }

		const Float32& incidenceAngle(Int32 FOVidx) const					{ return inc_[FOVidx]; }
		const Float32& latitude(Int32 FOVidx) const							{ return lat_[FOVidx]; }
		const Float32& longitude(Int32 FOVidx) const						{ return lon_[FOVidx]; }

		const Vector<Float32>& incidenceAngle() const						{ return inc_; }
		const Vector<Float32>& latitude() const								{ return lat_; }
		const Vector<Float32>& longitude() const							{ return lon_; }
		
		// mutators
		void height(const Float32& ht)										{ ht_ = ht; }
		
		void incidenceAngle(Int32 FOVidx,const Float32& inc)				{ inc_[FOVidx] = inc; }
		void latitude(Int32 FOVidx,const Float32& lat)						{ lat_[FOVidx] = lat; }
		void longitude(Int32 FOVidx,const Float32& lon)						{ lon_[FOVidx] = lon; }
		
		geo::GeographicCoordinate<Float32> coordinate(Int32 FOVidx) const;

		class Err : public Exception { };		

	protected:
		BasicGeolocation(const Int32& numFOVs);
		BasicGeolocation(const BasicGeolocation&);
		virtual ~BasicGeolocation() { }
		
		Int32			num_fovs_;
		Float32			ht_;
		Vector<Float32>	lat_,lon_,inc_;
};

inline BasicGeolocation::BasicGeolocation(const Int32& num_fovs) :
	num_fovs_(num_fovs),
	ht_(_NaN_),
	lat_(IndexRange<1>(1,num_fovs),_NaN_),
	lon_(IndexRange<1>(1,num_fovs),_NaN_),
	inc_(IndexRange<1>(1,num_fovs),_NaN_)
{ }

inline BasicGeolocation::BasicGeolocation(const BasicGeolocation& source) :
	num_fovs_(source.num_fovs_),
	ht_(source.ht_),
	lat_(source.lat_),
	lon_(source.lon_),
	inc_(source.inc_)
{ }

inline geo::GeographicCoordinate<Float32> BasicGeolocation::coordinate(Int32 FOVidx) const
{
	return geo::GeographicCoordinate<Float32>(latitude(FOVidx),longitude(FOVidx));
}

// utility routine to determine if swath is ascending or descending
//	- assumes L to R scan direction
// determine if ascending/descending	
inline bool isAscending(const BasicGeolocation& geolocation)
{
	Int32	num_pos = 0,
			num_neg = 0;

	for (Int32 i=geolocation.longitude().first()+1;i<=geolocation.longitude().last();i++)
	{
		const Float32	sep = geolocation.longitude(i)-geolocation.longitude(i-1);
		
		if (!isfinite(sep)) continue;
		
		if (sep >= 0)
			num_pos++;
		else
			num_neg++;		
	}
	
	if (num_pos > num_neg)
	{
		return true;
	}
	else if (num_pos < num_neg)
	{
		return false;
	}
	else
	{
		if (std::even(num_pos))
			return false;
		else
			return true;
	}
}

// policy class encapsulating implementation specific geolocation
//
//		SatelliteMetadata specializations should be templatized
//			on a specific Dataset<Inst,Lev,Ver>
//
template<class DS> class SatelliteGeolocation { };

} // namespace sat

} // namespace mcs

#endif // MCS_BASICGEOLOCATION_H