#include <iomanip>
#include <iostream>

#include "MemoryManager.h"

#include "MCS_Bin.h"
#include "MCS_Debug.h"
#include "MCS_MSULevel0.h"
#include "MCS_MSULevel1A.h"
#include "MCS_MSULevel1B.h"
#include "MCS_Satellite.h"

using namespace mcs;
using namespace mcs::geography;
using namespace sat;

#include "MCS_MATLAB_Engine.h"

int main()
{
	//typedef Dataset<MSU,msu::Level0,msu::v2>	DatasetType;
	typedef Dataset<MSU,msu::Level1A<msu::Mo1995>,msu::v2>	DatasetType;
	
	std::cout << "Start and end satellites (1..9) ? ";
	
	Int32	sat_start,sat_end;
	
	std::cin >> sat_start >> sat_end;
	
	if (sat_start < 1 || sat_end > 9 || sat_end < sat_start) return -1;
	
	for (Int32 i=sat_start;i<=sat_end;++i)
	{
		std::string	sat_name;
		
		switch (i)
		{
			case	1	:	sat_name = "TIROS-N"; break;
			case	2	:	sat_name = "NOAA-06"; break;
			case	3	:	sat_name = "NOAA-07"; break;
			case	4	:	sat_name = "NOAA-08"; break;
			case	5	:	sat_name = "NOAA-09"; break;
			case	6	:	sat_name = "NOAA-10"; break;
			case	7	:	sat_name = "NOAA-11"; break;
			case	8	:	sat_name = "NOAA-12"; break;
			case	9	:	sat_name = "NOAA-14"; break;
		}
		
		std::ofstream	log;
		
		SatelliteID	sat_ID(sat_name);
		
		Satellite	sat(sat_ID);
		
		DatasetType	ds(sat_ID);
		
		OrbitIterator<DatasetType>	orbdat(ds);
		
		// map data
		Bounds<Float32>			lat_bds(-90,90,72),
								lon_bds(0,360,144);
		Bin2D<Float32>			asc_map(lat_bds,lon_bds),
								dsc_map(lat_bds,lon_bds);
					
		bool	first_day = true;	
		Int32	num_swaths = 0;		
		Day		last_day;
		
		MATLAB_Engine	engine;
		
		for (OrbitIterator<DatasetType>::iterator it=orbdat.begin();it!=orbdat.end();++it)
		{
			try
			{
				std::cout << "Read orbit #" << it->orbitNumber() << std::endl;
				
				const Orbit<DatasetType>&	orb(*it);
				
				for (Orbit<DatasetType>::iterator sit=orb.begin();sit!=orb.end();++sit)
				{
					try
					{
						const Date	ep = sit->epoch();
						
						if (!sit->passedQC()) continue;
						
						Swath< Dataset<MSU,msu::Level1A<msu::Mo1995>,msu::v2> >	sw1A(*sit);
						
						if (!sw1A.passedQC()) continue;
						
						Swath< Dataset<MSU,msu::Level1B<msu::Mo1995>,msu::v2> >	sw(sw1A);
						
						sw.TB(sw.TB()+sw.limb());
						
						if (ep.day() != last_day)
						{ 
							if (!first_day)
							{
								engine.putMatrix("asc",asc_map.bins()/asc_map.weights());
								engine.putMatrix("dsc",dsc_map.bins()/dsc_map.weights());
								
								engine.evaluate("figure(1);imagesc(flipud(asc'),[200 280]);colorbar;");
								engine.evaluate("figure(2);imagesc(flipud(dsc'),[200 280]);colorbar;");

								asc_map.clear();
								dsc_map.clear();
							}
							
							first_day = false;
							last_day = ep.day();
						}
						
						// add data to map
						static const Int32	lofov = 3,
											hifov = 9;
										
						static const Int32	chan = sw.CH_2;
							
						if (sit->ascending())
							for (Index<1> fov=lofov;fov<=hifov;++fov)
								asc_map.add(sw.latitude(fov),sw.longitude(fov),sw.data(chan,fov));
						else
							for (Index<1> fov=lofov;fov<=hifov;++fov)
								dsc_map.add(sw.latitude(fov),sw.longitude(fov),sw.data(chan,fov));
						
					}
					catch (Orbit<DatasetType>::BadSwathErr& ex)
					{
						//std::cout << "Swath " << sit().value() << " bad..." << ex.message() << std::endl;
						continue;
					}
					catch (Orbit<DatasetType>::MissingSwathErr& ex)
					{
						//std::cout << "Swath " << sit().value() << " missing..." << ex.message() << std::endl;
						continue;
					}
				}
			}
			catch (Orbit<DatasetType>::BadOrbitErr& ex)
			{
				//std::cout << "Orbit " << it().value() << " bad..." << ex.message() << std::endl;
				continue;
			}
			catch (Orbit<DatasetType>::MissingOrbitErr& ex)
			{
				//std::cout << "Orbit " << it().value() << " missing..." << ex.message() << std::endl;
				continue;
			}
		}
	}

	return 0;
}	
