#include <iomanip>
#include <map>

#include "MCS_Array.h"
#include "MCS_Constants.h"
#include "MCS_FileLocations.h"
#include "MCS_Matrix.h"
#include "MCS_MSUSeries.h"

namespace mcs {

static const Index<1>		LAT_RANGE(90),
							LON_RANGE(180);
static const IndexRange<2>	GRID_RANGE(Index<2>(LAT_RANGE,LON_RANGE));

static const geo::CenteredEarthGrid<Float32>	MSUGRID(GRID_RANGE);

static const Int32			CLIMATOLOGY_YEAR = 2000;

static std::string getClimatologyFilename(const Metadata& meta)
{
	using namespace std;
	
	stringstream	tmp;
	
	static const string	directory = FileLocations["MSU climatology"];
	
	tmp << directory;
	//tmp << MSU_CLIMATOLOGY_DATA_DIRECTORY;
	
	const string&	channel = meta["channel"];
		
	tmp << "MSU_all_satellites_ch_" << channel << "_test_OffsetAndTargetCorr_symmetrized_fit.mcs";
	
	return tmp.str();	
}

MSUClimatology::MSUClimatology(const std::string& metainit) :
	MapSeries<MapType>(metainit),
	climatologyData_(IndexRange<2>(Index<2>(0,1),Index<2>(5,12))),
	initialized_(false)
{		
	curr_ = MapType(MSUGRID);
	first(); 
}

MSUClimatology::MSUClimatology(const std::string& metainit,const Date& ep) :
	MapSeries<MapType>(metainit),
	climatologyData_(IndexRange<2>(Index<2>(0,1),Index<2>(5,12))),
	initialized_(false)
{
	curr_ = MapType(MSUGRID);
	find(ep); 
}

MSUClimatology::~MSUClimatology()
{ }

void MSUClimatology::read()
{
	using namespace std;
	
	if (metadata() == last_meta_ && findNearestInterval(epoch()) == findNearestInterval(last_epoch_)) return;
	
	//static Array<Matrix<Float32>,2>	climatologyData(IndexRange<2>(Index<2>(0,1),Index<2>(5,12)));
	//static bool						initialized = false;
	
	if (!initialized_ || metadata() != last_meta_)
	{
		ifstream		ifs(getClimatologyFilename(metadata()).c_str(),ios::binary | ios::in);
		bool			firstread = true;
		Matrix<Float32>	data;
		IndexRange<2>	rng;
		
		for (Index<1> fovidx=climatologyData_.first().rank(0);fovidx<=climatologyData_.last().rank(0);++fovidx)
		{
			for (Index<1> monidx=climatologyData_.first().rank(1);monidx<=climatologyData_.last().rank(1);++monidx)
			{
				const Index<2>	idx(fovidx,monidx);
				
				if (firstread)
				{
					data.read(ifs);
					rng = IndexRange<2>(data.first()+Index<2>(1,1),data.last()-Index<2>(1,1));
					firstread = false;
					
					climatologyData_(idx) = data(rng);
					climatologyData_(idx).offset(Index<2>(0,0));
				}
				else
				{
					data.readNext(ifs,8);
					
					climatologyData_(idx) = data(rng);
					climatologyData_(idx).offset(Index<2>(0,0));
				}
			}
		}
		
		initialized_ = true;
	}
	
	const Int32	FOV = 5-std::abs(std::atoi(metadata("FOV").c_str()));
	
	curr_ = climatologyData_(Index<2>(FOV,epoch().month()));
	
	last_meta_ = metadata();
	last_epoch_ = epoch();
}

Date MSUClimatology::begin() const
{
	return Date(CLIMATOLOGY_YEAR,1,1);
	//return Date(-std::numeric_limits<Float64>::max());
}

Date MSUClimatology::end() const
{
	return Date(CLIMATOLOGY_YEAR,12,31);
	//return Date(std::numeric_limits<Float64>::max());
}

Date MSUClimatology::findNearestInterval(const Date& ep) const
{
	Date	bom(CLIMATOLOGY_YEAR,ep.month(),1),
			eom(CLIMATOLOGY_YEAR,ep.month(),1);
			
	eom.addMonths(1);
	
	return Date((bom.julian()+eom.julian())/2.0);
}

Date MSUClimatology::incrementInterval(Uint32 num) const
{
	return Date(epoch()).addMonths(num);
}
	
} // namespace mcs
