#ifndef MCS_MSUDATASET_H
#define MCS_MSUDATASET_H

#include <string>

#include "MCS_FileLocations.h"
#include "MCS_Iterator.h"
#include "MCS_MSUConstants.h"
#include "MCS_OneWayDirectory.h"
#include "MCS_Ordered.h"
#include "MCS_Satellite.h"
#include "MCS_SatelliteDataset.h"

namespace mcs {

namespace sat {

namespace msu {

// placeholder classes for different MSU calibration methods
class Uncalibrated 
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "Uncalibrated"; return tag_; }
};

class Linear 
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "Linear"; return tag_; }
};

class Mo1995 
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "Mo1995"; return tag_; }
};

class Mo2001 
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "Mo2001"; return tag_; }
};

// placeholder classes for different swath data types
class Level0 
{ 	
	public:
		typedef Int16	SwathDataType;
		
		static Int32 numberOfChannels()		{ return 4; }
		static Int32 numberOfFOVs()			{ return 11; }

		static const std::string& tag()		{ static const std::string tag_ = "Level 0"; return tag_; }
		static const std::string& suffix()	{ static const std::string sfx_ = "L0"; return sfx_; }
};

template<class Cal> class Level1A
{ 	
	public:
		typedef Cal		CalibrationType;
		typedef Float32	SwathDataType;
		
		static Int32 numberOfChannels()		{ return 4; }
		static Int32 numberOfFOVs()			{ return 11; }

		static const std::string& tag()		{ static const std::string tag_ = "Level 1A"; return tag_; }
		static const std::string& suffix()	{ static const std::string sfx_ = "L1A"; return sfx_; }
};

// placeholder classes for different MSU algorithm versions
class v1
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "v1"; return tag_; }
};

class v2
{
	public:
		static const std::string& tag()	{ static const std::string tag_ = "v2"; return tag_; }
};

// orbit inventory data
class OrbitInventoryInfo : public Ordered<Int32>
{
	public:
		OrbitInventoryInfo() : Ordered<Int32>(-1) { };
		~OrbitInventoryInfo() { };
		
		const Int32& orbitNumber() const	{ return value(); }
		const Int32& firstSwath() const		{ return first_swath_; }
		const Int32& lastSwath() const		{ return last_swath_; }
		const Float64& firstRD2000() const	{ return first_rd2000_; }
		const Float64& lastRD2000() const	{ return last_rd2000_; }
		
		void read(std::ifstream& ifs);
		
		static Int32 sizeOf() 				{ return 28; }
	
	private:
		Int32	first_swath_,
				last_swath_;
		Float64	first_rd2000_,
				last_rd2000_;
};

inline void OrbitInventoryInfo::read(std::ifstream& ifs)
{
	Int32	orbit_number;
	
	ifs.read(reinterpret_cast<char*>(&orbit_number),sizeof(Int32));
	
	value(orbit_number);
	
	ifs.read(reinterpret_cast<char*>(&first_swath_),sizeof(Int32));
	ifs.read(reinterpret_cast<char*>(&first_rd2000_),sizeof(Float64));
	ifs.read(reinterpret_cast<char*>(&last_swath_),sizeof(Int32));
	ifs.read(reinterpret_cast<char*>(&last_rd2000_),sizeof(Float64));
}

inline SatelliteID getSatelliteID(const Int32& num)
{
	switch (num)
	{
		case	TIROS_N	:	return SatelliteID("TIROS-N"); break;
		case	NOAA_06	:	return SatelliteID("NOAA-06"); break;
		case	NOAA_07	:	return SatelliteID("NOAA-07"); break;
		case	NOAA_08	:	return SatelliteID("NOAA-08"); break;
		case	NOAA_09	:	return SatelliteID("NOAA-09"); break;
		case	NOAA_10	:	return SatelliteID("NOAA-10"); break;
		case	NOAA_11	:	return SatelliteID("NOAA-11"); break;
		case	NOAA_12	:	return SatelliteID("NOAA-12"); break;
		case	NOAA_14	:	return SatelliteID("NOAA-14"); break;
		default			: 	return SatelliteID();
	}
}

inline std::string getSatelliteName(const Int32& num)
{
	switch (num)
	{
		case	TIROS_N	: return "TIROS-N"; break;
		case	NOAA_06	: return "NOAA-06"; break;
		case	NOAA_07	: return "NOAA-07"; break;
		case	NOAA_08	: return "NOAA-08"; break;
		case	NOAA_09	: return "NOAA-09"; break;
		case	NOAA_10	: return "NOAA-10"; break;
		case	NOAA_11	: return "NOAA-11"; break;
		case	NOAA_12	: return "NOAA-12"; break;
		case	NOAA_14	: return "NOAA-14"; break;
		default			: return "";
	}
}

inline Int32 getSatelliteNumber(const SatelliteID& sat_ID)
{
	if (sat_ID == SatelliteID("TIROS-N"))		return TIROS_N;
	else if (sat_ID == SatelliteID("NOAA-06"))	return NOAA_06;
	else if (sat_ID == SatelliteID("NOAA-07"))	return NOAA_07;
	else if (sat_ID == SatelliteID("NOAA-08"))	return NOAA_08;
	else if (sat_ID == SatelliteID("NOAA-09"))	return NOAA_09;
	else if (sat_ID == SatelliteID("NOAA-10"))	return NOAA_10;
	else if (sat_ID == SatelliteID("NOAA-11"))	return NOAA_11;
	else if (sat_ID == SatelliteID("NOAA-12"))	return NOAA_12;
	else if (sat_ID == SatelliteID("NOAA-14"))	return NOAA_14;
	else										return -1;
}

} // namespace msu

// placeholder class for MSU instruments, templated on data type
template<class DT,class Lev,class Ver> class MSU :
	public DT,
	public Lev,
	public Ver
{
	public:
		typedef DT	DataType;
		typedef Lev	LevelType;
		typedef Ver	VersionType;
		
		MSU(const SatelliteID& sat_ID);
		MSU(const MSU<DT,Lev,Ver>&);
		~MSU() { }
		
		const SatelliteID& satelliteID() const					{ return sat_ID_; }
		const Int32& satelliteNumber() const					{ return sat_num_; }
		
		class Err : public Exception { };
		
	private:
		SatelliteID	sat_ID_;
		Int32		sat_num_;
};

template<class DT,class Lev,class Ver>
inline MSU<DT,Lev,Ver>::MSU(const SatelliteID& sat_ID) : 
	sat_ID_(sat_ID),
	sat_num_(-1)
{ 
	sat_num_ = msu::getSatelliteNumber(sat_ID_);
}

template<class DT,class Lev,class Ver>
inline MSU<DT,Lev,Ver>::MSU(const MSU<DT,Lev,Ver>& source) :
	sat_ID_(source.sat_ID_),
	sat_num_(source.sat_num_)
{ }

template<class Ver>
class Inventory< Dataset< MSU< SwathData,msu::Level0,Ver> > > : 
	public std::deque<msu::OrbitInventoryInfo>
{
	public:
		static std::string filename(const SatelliteID& sat_ID)	
		{ return FileLocations["MSU "+msu::Level0::tag()]+sat_ID.name()+"\\"+sat_ID.name()+"_"+msu::Level0::suffix()+".inv"; }
};

template<class Cal,class Ver>
class Inventory< Dataset< MSU< SwathData,msu::Level1A<Cal>,Ver> > > : 
	public std::deque<msu::OrbitInventoryInfo>
{
	public:
		static std::string filename(const SatelliteID& sat_ID)	
		{ return FileLocations["MSU "+msu::Level1A<Cal>::tag()]+sat_ID.name()+"\\"+Cal::tag()+"\\"+sat_ID.name()+"_"+msu::Level1A<Cal>::suffix()+".inv"; }
};

// MSU dataset, iterator iterates over orbits
template<class Lev,class Ver> 
class Dataset< MSU<SwathData,Lev,Ver> > : public MSU<SwathData,Lev,Ver> 
{ 
	public:
		typedef Inventory< Dataset< MSU<SwathData,Lev,Ver> > >	InventoryType;
		typedef OneWayDirectory<Int32,InventoryType>			InventoryDirectoryType;
		
		Dataset();
		Dataset(const SatelliteID& sat_ID);
		
		const Int32& firstOrbit() const							{ return info_stack_ptr_->begin()->orbitNumber(); }
		const Int32& lastOrbit() const							{ return info_stack_ptr_->rbegin()->orbitNumber(); }
		
		Date startDate() const									{ return fromRD2000(info_stack_ptr_->begin()->firstRD2000()); }
		Date endDate() const									{ return fromRD2000(info_stack_ptr_->rbegin()->lastRD2000()); }
		
		std::string directory() const							{ return FileLocations["MSU "+Lev::tag()]+satelliteID().name()+"\\"; }

		const InventoryType& inventory() const					{ return *info_stack_ptr_; }
		
	private:
		InventoryType*	info_stack_ptr_;
};

template<class Lev,class Ver> 
inline class Dataset< MSU<SwathData,Lev,Ver> >::Dataset() : MSU<SwathData,Lev,Ver>(SatelliteID())
{ }

template<class Lev,class Ver> 
inline class Dataset< MSU<SwathData,Lev,Ver> >::Dataset(const SatelliteID& sat_ID) : MSU<SwathData,Lev,Ver>(sat_ID)
{
	InventoryDirectoryType&	inv = Singleton<InventoryDirectoryType>::instance();
	
	// read satellite inventory
	if (!inv.has(satelliteID().number()))
	{
		inv.add(satelliteID().number(),InventoryType());
		
		InventoryType&	info_stack = inv[satelliteID().number()];
		
		info_stack_ptr_ = &info_stack;
		
		const std::string	filename = InventoryType::filename(satelliteID());
		
		std::cerr << "Reading " << filename;
		
		std::ifstream	ifs(filename.c_str(),std::ios::binary | std::ios::in);
		
		try
		{
			const Int32	num_to_read = fileLength(ifs)/msu::OrbitInventoryInfo::sizeOf();
			
			msu::OrbitInventoryInfo	info;
			
			for (Int32 i=1;i<=num_to_read;++i)
			{
				info.read(ifs);
				
				info_stack.push_back(info);
			}
		}
		catch (...)
		{ }
		
		std::cerr << "...done" << std::endl;
	}
}

} // namespace sat

} // namespace mcs

#endif // MCS_MSUDATASET_H
