#ifndef MCS_MSULEVEL0_H
#define MCS_MSULEVEL0_H

#include <bitset>

#include "MCS_BasicMetadata.h"
#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_MSUConstants.h"
#include "MCS_MSUDataset.h"
#include "MCS_MSUGeolocation.h"
#include "MCS_MSUSwathData.h"
#include "MCS_MSUUtilities.h"
#include "MCS_Tensor.h"

namespace mcs {

namespace sat { 

// utility class to do bounds checking
class Check
{
	public:
		// map OOB integer counts to -1
		static inline Int16 Int(const Int16& counts)	
		{ 
			using namespace mcs::sat::msu;
			
			return (counts >= MIN_COUNTS && counts <= MAX_COUNTS) ? counts : -1;
		}

		// map OOB integer counts to NaN
		static inline Float32 Float(const Int16& counts)	
		{ 
			using namespace mcs::sat::msu;
			
			return (counts >= MIN_COUNTS && counts <= MAX_COUNTS) ? counts : _NaN_;
		}
};

// MSU Level 0 SatelliteMetadata
template<class Ver> 
class SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > > :	
	public Dataset< MSU<SwathData,msu::Level0,Ver> >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level0,Ver> >	DatasetType;
		
		// swath should be rejected if bits 1,3,12, or 15 are set or if 13 & 14 are set
		enum QC_Flags
		{					
			EARTH_LOCATION_DELTA = 0,
			NO_EARTH_LOCATION = 1,
			DATA_GAP = 2,
			FATAL = 3,
			CALIBRATION = 4,
			ASCENDING = 8,
			TLE_INTERPOLATED = 9,
			SUSPECT_GEOLOCATION = 10,
			SUSPECT_HEIGHT = 11,
			BAD_GEOLOCATION = 12,
			BAD_TARGET1_T = 13,
			BAD_TARGET2_T = 14,
			BAD = 15,
			FOV1 = 16,
			FOV2 = 17,
			FOV3 = 18,
			FOV4 = 19,
			FOV5 = 20,
			FOV6 = 21,
			FOV7 = 22,
			FOV8 = 23,
			FOV9 = 24,
			FOV10 = 25,
			FOV11 = 26,
			FOV_SPACE = 27,
			FOV_BLACKBODY = 28,
			FOV_SCAN = 29			
		};					
		
		// FOV should be rejected if bits 0 or 2 are set
		enum FOV_QC_Flags 
		{ 			
			BAD_FOV = 0,
			MIRROR_SEQ = 1,
			SCAN_SEQ = 2,
			SCAN_DISABLED = 3,
			DACS = 4,
			DWELL = 5,
			MISSING_DATA = 6,
			TIME_ERROR = 7
		};

		const DatasetType& dataset() const											{ return *this; }
		
		bool ascending() const														{ return asc_; }
		bool descending() const														{ return !asc_; }

		// accessors
		const std::bitset<32>& QC() const											{ return QC_; }
		const bool QC(Int32 idx) const												{ return QC_[idx]; }
		
		const Vector< std::bitset<8> >& FOV_QC() const								{ return FOV_QC_; }
		const std::bitset<8>& FOV_QC(Int32 fov) const								{ return FOV_QC_[fov]; }
		const bool FOV_QC(Int32 fov,Int32 idx) const								{ return FOV_QC_[fov][idx]; }
		
		static Uint32 FOVFlag(Int32 i)												{ return i+FOV1-1; }
		
		// mutators
		std::bitset<32>& QC() 														{ return QC_; }
		std::bitset<32>::reference QC(Int32 idx)									{ return QC_[idx]; }
		
		std::bitset<8>& FOV_QC(Int32 fov)											{ return FOV_QC_[fov]; }
		std::bitset<8>::reference FOV_QC(Int32 fov,Int32 idx)						{ return FOV_QC_[fov][idx]; }
		
		bool passedQC() const;
		bool passedFOV_QC(Int32 idx) const;
		
		const Int32& swathNumber() const											{ return swath_number_; }

		// miscellaneous quantities
		const std::pair<Float32,Float32>& T_Target1() const							{ return T_target1_; }
		const std::pair<Float32,Float32>& T_Target2() const							{ return T_target2_; }
		
		const std::pair<Float32,Float32>& target1_Calibration() const				{ return T1_cal_; }
		const std::pair<Float32,Float32>& target2_Calibration() const				{ return T2_cal_; }
		
		const Float32& spaceView(Int32 idx) const									{ return space_view_[idx-1]; }
		const Float32& blackbodyView(Int32 idx) const								{ return blackbody_view_[idx-1]; }
		const Float32& scanView(Int32 idx) const									{ return scan_to_spot1_view_[idx-1]; }
		
		const std::pair<Float32,Float32>& T_RFchassis() const						{ return RF_chassis_temp_; }
		
		const FixedVector<Float32,4>& T_LocalOscillator() const						{ return LO_temp_; }
		const FixedVector<Float32,4>& T_DickeLoad() const							{ return Dicke_load_temp_; }
		
		Float32 T_LocalOscillator(Int32 chidx) const								{ return LO_temp_[chidx-1]; }
		Float32 T_DickeLoad(Int32 chidx) const										{ return Dicke_load_temp_[chidx-1]; }
		
	protected:
		SatelliteMetadata(const DatasetType& ds = DatasetType());
		SatelliteMetadata(const SatelliteMetadata<DatasetType>& source);
		~SatelliteMetadata() { }
		
		void QCConsistencyCheck();
		
		std::bitset<32>				QC_;
		Vector< std::bitset<8> >	FOV_QC_;
		
		bool						asc_;
		Int32						swath_number_;
		
		Float32						local_zenith_angle_,
									earth_location_delta_;
		Int16						set_coef_;	
		Int16						serial_number_;
		Float32						E_cal_lo_,
									F_cal_hi_,
									minus15_plus5_V_,
									plus5_V_,
									E_zero_;
		std::pair<Float32,Float32>	T_target1_,
									T_target2_,
									T1_cal_,
									T2_cal_,
									other_temp_,
									xtal1_,
									xtal2_,
									xtal3_,
									xtal4_,
									antenna_bearing_temp_,
									motor_temp_,
									RF_chassis_temp_;
		FixedVector<Float32,4>		space_view_,
									blackbody_view_,
									scan_to_spot1_view_,
									LO_temp_,
									Dicke_load_temp_,
									prog_temp_;		

	private:
		SatelliteMetadata<DatasetType>& operator=(const SatelliteMetadata<DatasetType>&);
};

// MSU Level 0 Swath
template<class Ver> 
class Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > > : 	
	public Ordered<Date>,
	public SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >,
	public SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level0,Ver> > >,
	public SatelliteSwathData< Dataset< MSU<SwathData,msu::Level0,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level0,Ver> >	DatasetType;
		typedef typename DatasetType::SwathDataType			Type;
		
		Swath(const DatasetType& ds);
		Swath(const Swath<DatasetType>& source);
		~Swath() { }
		
		// accessors
		const Matrix<Type>& counts() const											{ return data(); }
		const Type& counts(const Index<2>& idx) const								{ return data(idx); }
		const Type& counts(Int32 chidx,Int32 fovidx) const							{ return data(chidx,fovidx); }

		// mutators
		void counts(const Type& val)												{ data(val); }
		void counts(const Matrix<Type>& cts)										{ data(cts); }
		void counts(const Index<2>& idx,const Type& val)							{ data(idx,val); }
		void counts(Int32 chidx,Int32 fovidx,const Type& val)						{ data(chidx,fovidx,val); }
				
		void get(const Vector<Uint8>&);
		
		const Swath<DatasetType>& read(const std::istream&);
		void write(std::ostream&) const;

		class Err : public Exception { };
		
		class GeolocationErr : public Err { };
		class IOErr : public Err { };
		
	private:
		void checkGeolocation();
		void calibrateInstrumentTemperatures();
		
		void calibrateDickeLoadTemperatures();
		void calibrateTargetTemperatures();
};

// implementation
#include "MCS_MSULevel0.cpp"

} // namespace sat

} // namespace mcs

#endif // MCS_MSULEVEL0_H