#ifndef MCS_MSULEVEL1A_H
#define MCS_MSULEVEL1A_H

#include <bitset>

#include "MCS_Array.h"
#include "MCS_BasicMetadata.h"
#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_Interpolation.h"
#include "MCS_LandFraction.h"
#include "MCS_MSUConstants.h"
#include "MCS_MSUDataset.h"
#include "MCS_MSUGeolocation.h"
#include "MCS_MSUSwathData.h"
#include "MCS_MSUUtilities.h"
#include "MCS_Radiation.h"
#include "MCS_StringExtensions.h"

namespace mcs {

namespace sat { 

// MSU Level 1A SatelliteMetadata
template<class Cal,class Ver> 
class SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > > :
	public Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >	DatasetType;
		
		// swath should be rejected if bits 1,3,12, or 15 are set or if 13 & 14 are set
		enum QC_Flags
		{					
			EARTH_LOCATION_DELTA = 0,
			NO_EARTH_LOCATION = 1,
			DATA_GAP = 2,
			FATAL = 3,
			CALIBRATION = 4,
			ASCENDING = 8,
			TLE_INTERPOLATED = 9,
			SUSPECT_GEOLOCATION = 10,
			SUSPECT_HEIGHT = 11,
			BAD_GEOLOCATION = 12,
			BAD_TARGET1_T = 13,
			BAD_TARGET2_T = 14,
			BAD = 15,
			FOV1 = 16,
			FOV2 = 17,
			FOV3 = 18,
			FOV4 = 19,
			FOV5 = 20,
			FOV6 = 21,
			FOV7 = 22,
			FOV8 = 23,
			FOV9 = 24,
			FOV10 = 25,
			FOV11 = 26
		};					
		
		// FOV should be rejected if bits 0 or 2 are set
		enum FOV_QC_Flags 
		{ 			
			BAD_FOV = 0,
			MIRROR_SEQ = 1,
			SCAN_SEQ = 2,
			SCAN_DISABLED = 3,
			DACS = 4,
			DWELL = 5,
			MISSING_DATA = 6,
			TIME_ERROR = 7
		};

		const DatasetType& dataset() const											{ return *this; }
		
		bool ascending() const														{ return asc_; }
		bool descending() const														{ return !asc_; }

		// accessors
		const std::bitset<32>& QC() const											{ return QC_; }
		const bool QC(Int32 idx) const												{ return QC_[idx]; }

		const Vector< std::bitset<8> >& FOV_QC() const								{ return FOV_QC_; }
		const std::bitset<8>& FOV_QC(Int32 idx) const								{ return FOV_QC_[idx]; }
		const bool FOV_QC(Int32 fov,Int32 idx) const								{ return FOV_QC_[fov][idx]; }
		
		static Uint32 FOVFlag(Int32 i)												{ return i+FOV1-1; }
	
		// mutators
		std::bitset<32>& QC() 														{ return QC_; }
		std::bitset<32>::reference QC(Int32 idx)									{ return QC_[idx]; }
		
		std::bitset<8>& FOV_QC(Int32 fov)											{ return FOV_QC_[fov]; }
		std::bitset<8>::reference FOV_QC(Int32 fov,Int32 idx)						{ return FOV_QC_[fov][idx]; }
		
		bool passedQC() const;
		bool passedFOV_QC(Int32 idx) const;
	
		const Int32& swathNumber() const											{ return swath_number_; }

		// miscellaneous quantities
		const std::pair<Float32,Float32>& T_target() const							{ return T_target_; }
		
	protected:
		SatelliteMetadata(const DatasetType& ds = DatasetType());
		SatelliteMetadata(const SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >& meta);
		SatelliteMetadata(const SatelliteMetadata<DatasetType>& source);
		~SatelliteMetadata() { }
		
		void QCConsistencyCheck();
		
		std::bitset<32>				QC_;
		Vector< std::bitset<8> >	FOV_QC_;
		
		bool						asc_;
		Int32						swath_number_;
		std::pair<Float32,Float32>	T_target_;
		
	private:
		SatelliteMetadata<DatasetType>& operator=(const SatelliteMetadata<DatasetType>&);
};

// MSU Level 1A Swath with linear calibration
template<class Ver> 
class Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > > : 	
	public Ordered<Date>,
	public SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >,
	public SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >,
	public SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> >	DatasetType;
		typedef typename DatasetType::SwathDataType						Type;
		
		Swath();
		Swath(const DatasetType& ds);
		explicit Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >&);
		~Swath() { }
		
		// accessors
		const Matrix<Type>& TA() const											{ return data(); }
		const Type& TA(const Index<2>& idx) const								{ return data(idx); }
		const Type& TA(Int32 chidx,Int32 fovidx) const							{ return data(chidx,fovidx); }
		
		// mutators
		void TA(const Type& val)												{ data(val); }
		void TA(const Matrix<Type>& ta)											{ data(ta); }
		void TA(const Index<2>& idx,const Type& val)							{ data(idx,val); }
		void TA(Int32 chidx,Int32 fovidx,const Type& val)						{ data(chidx,fovidx,val); }

		void get(const Vector<Uint8>&);
		
		const Swath<DatasetType>& read(const std::istream&);
		void write(std::ostream&) const;
		
		class Err : public Exception { };
		
		class GeolocationErr : public Err { };
		class IOErr : public Err { };
		
	private:
		void checkGeolocation();

		Float32 countConversion(Int32 chidx,const Float32& raw_counts) const;
};

// MSU Level 1A swath with Mo 1995 calibration
template<class Ver> 
class Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > > : 	
	public Ordered<Date>,
	public SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >,
	public SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >,
	public SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> >	DatasetType;
		typedef typename DatasetType::SwathDataType						Type;
		
		Swath();
		Swath(const DatasetType& ds);
		Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >&);
		~Swath() { }
		
		// accessors 
		const Matrix<Type>& TA() const											{ return data(); }
		const Type& TA(const Index<2>& idx) const								{ return data(idx); }
		const Type& TA(Int32 chidx,Int32 fovidx) const							{ return data(chidx,fovidx); }
		
		// mutators
		void TA(const Type& val)												{ data(val); }
		void TA(const Matrix<Type>& ta)											{ data(ta); }
		void TA(const Index<2>& idx,const Type& val)							{ data(idx,val); }
		void TA(Int32 chidx,Int32 fovidx,const Type& val)						{ data(chidx,fovidx,val); }
				
		void get(const Vector<Uint8>&);
		
		const Swath<DatasetType>& read(const std::istream&);
		void write(std::ostream&) const;

		class Err : public Exception { };
		
		class GeolocationErr : public Err { };
		class IOErr : public Err { };
		
	private:
		void checkGeolocation();

		Float32 countConversion(Int32 chidx,const Float32& raw_counts) const;
};

// MSU Level 1A swath with Mo 2001 calibration
template<class Ver> 
class Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > > : 	
	public Ordered<Date>,
	public SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >,
	public SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >,
	public SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> >	DatasetType;
		typedef typename DatasetType::SwathDataType						Type;
		
		Swath();
		Swath(const DatasetType& ds);
		Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >&);
		~Swath() { }
		
		// accessors 
		const Matrix<Type>& TA() const											{ return data(); }
		const Type& TA(const Index<2>& idx) const								{ return data(idx); }
		const Type& TA(Int32 chidx,Int32 fovidx) const							{ return data(chidx,fovidx); }
		
		// mutators
		void TA(const Type& val)												{ data(val); }
		void TA(const Matrix<Type>& ta)											{ data(ta); }
		void TA(const Index<2>& idx,const Type& val)							{ data(idx,val); }
		void TA(Int32 chidx,Int32 fovidx,const Type& val)						{ data(chidx,fovidx,val); }

		void get(const Vector<Uint8>&);
		
		const Swath<DatasetType>& read(const std::istream&);
		void write(std::ostream&) const;

		class Err : public Exception { };
		
		class GeolocationErr : public Err { };
		class IOErr : public Err { };
		
	private:
		void checkGeolocation();

		Float32 countConversion(Int32 chidx,const Float32& DickeTemperature) const;
};

// MSU Level 1A SatelliteMetadata
template<class Cal,class Ver> 
inline SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::SatelliteMetadata(const DatasetType& ds) :
	Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >(ds),
	QC_(0xFFFFFFFF),
	FOV_QC_(IndexRange<1>(1,DatasetType::numberOfFOVs()),0xFF),
	asc_(false),
	swath_number_(-1),
	T_target_(_NaN_,_NaN_)
{ }

template<class Cal,class Ver> 
inline SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::SatelliteMetadata(const SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >& meta) :
	Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >(meta.satelliteID()),
	QC_(meta.QC()),
	FOV_QC_(IndexRange<1>(1,DatasetType::numberOfFOVs()),0xFF),
	asc_(meta.ascending()),
	swath_number_(meta.swathNumber()),
	T_target_(_NaN_,_NaN_)
{
	if (!meta.passedQC()) QC(BAD) = true;
	
	// can't just copy because Level0 has 14 FOV QC flags
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i) FOV_QC(i) = meta.FOV_QC(i); 
	
	QCConsistencyCheck();
}

template<class Cal,class Ver> 
inline SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::SatelliteMetadata(const SatelliteMetadata<DatasetType>& source) :
	Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >(source),
	QC_(source.QC_),
	FOV_QC_(source.FOV_QC_),
	asc_(source.asc_),
	swath_number_(source.swath_number_),
	T_target_(source.T_target_)
{ }

template<class Cal,class Ver> 
inline bool SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::passedQC() const
{
	return !(QC().test(NO_EARTH_LOCATION) || QC().test(FATAL) || QC().test(BAD) || (QC().test(BAD_TARGET1_T) && QC().test(BAD_TARGET2_T)));
}

template<class Cal,class Ver> 
inline bool SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::passedFOV_QC(Int32 idx) const
{
	return !(FOV_QC(idx).test(BAD_FOV) || FOV_QC(idx).test(SCAN_SEQ));
}

template<class Cal,class Ver> 
inline void SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::QCConsistencyCheck()
{
	if (!passedQC()) QC(BAD) = true;
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
	{
		if (QC(BAD) || QC(FOVFlag(i)) == true || !passedFOV_QC(i))
		{
			QC(FOVFlag(i)) = true;
			FOV_QC(i,BAD_FOV) = true;
		}
	}
}

// MSU Level 1A Swath
inline Float32 checkTargetTemperature(const std::pair<Float32,Float32>& T_target)
{
	static const Float32	T_TARGET_LO = 260.0,
							T_TARGET_HI = 300.0,
							DELTA_T_MAX = 2.0;
							
	std::pair<bool,bool>	good;
		
	// target A for channels 1&2 
	good.first = (T_target.first >= T_TARGET_LO && T_target.first <= T_TARGET_HI) ? true : false; 
	good.second = (T_target.second >= T_TARGET_LO && T_target.second <= T_TARGET_HI) ? true : false; 
		
	if (good.first && good.second)
	{
		// check that two temperatures are consistent to 2K
		if (std::abs(T_target.first-T_target.second) > DELTA_T_MAX) 
			return _NaN_;
		else
			return (T_target.first+T_target.second)/2;
	}
	else if (good.first && !good.second)
		return T_target.first;
	else if (!good.first && good.second)
		return T_target.second;
	else
		return _NaN_;
}

// linear calibration of counts to temperature
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::Swath() :
	Ordered<Date>(),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >()
{ }

template<class Ver>
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::Swath(const DatasetType& ds) :
	Ordered<Date>(),
	SatelliteMetadata< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(ds),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >()
{ }

// construct Level 1A swath from Level 0 swath
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >& ob) :
	Ordered<Date>(ob),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(ob),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >(ob),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >()
{
	using namespace constants::physical;
	using namespace msu;
	
	T_target_.first = checkTargetTemperature(ob.T_Target1());
	T_target_.second = checkTargetTemperature(ob.T_Target2());
	
	// keep track of number of bad obs
	Vector<Int8>	num_bad(IndexRange<1>(0,NUM_FOVS));
	
	// loop over channels
	for (Int32 chidx=TA().first(0);chidx<=TA().last(0);chidx++)
	{
		// nonlinear correction to counts
		const Float32	counts_Space = ob.spaceView(chidx),
						counts_Blackbody = ob.blackbodyView(chidx),
						counts_Scan = ob.scanView(chidx);
		Float32			target_T;
				
		// target A for channels 1&2
		if (chidx == CH_1 || chidx == CH_2)
			target_T = T_target_.first;
		// target B for channels 3&4
		else if (chidx == CH_3 || chidx == CH_4)
			target_T = T_target_.second;
				
		// loop over FOVs
		for (Int32 fovidx=TA().first(1);fovidx<=TA().last(1);fovidx++)
		{
			if (QC(FOVFlag(fovidx)) == true || !isfinite(target_T) || ob.counts(chidx,fovidx) == -1)
			{
				TA(chidx,fovidx,_NaN_);
			}
			else
			{
				// nonlinear correction to counts
				const Float32	counts_Earth = ob.counts(chidx,fovidx);
						
				// linear conversion to TA
				// eventually do inversion of Planck equation ???
				TA(chidx,fovidx,num::linearInterpolate(counts_Earth,counts_Space,Float32(_T_BC_),counts_Blackbody,target_T));
			}
								
			// bounds on reasonable values of TA based on histograms of all satellites
			static const Vector<Float32>	min_TA(IndexRange<1>(1,NUM_CHANNELS)," 160.0  192.0  185.0  175.0 "),
											max_TA(IndexRange<1>(1,NUM_CHANNELS)," 310.0  270.0  245.0  250.0 ");
											
			if (!isfinite(TA(chidx,fovidx)) || TA(chidx,fovidx) < min_TA[chidx] || TA(chidx,fovidx) > max_TA[chidx])
			{
				num_bad[0]++;
				num_bad[fovidx]++;
				
				TA(chidx,fovidx,_NaN_);
			}
		}
	}
	
	// update QC flags
	if (num_bad[0] == NUM_CHANNELS*NUM_FOVS) QC(BAD) = true;
	
	for (Int32 i=num_bad.first();i<=num_bad.last();++i)
		if (num_bad[i] == 4) FOV_QC(i,BAD_FOV) = true;
	
	QCConsistencyCheck();
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::get(const Vector<Uint8>& swath_data)
{
	swath_number_ = *(reinterpret_cast<const Int32*>(&swath_data[0]));
	
	epoch(fromRD2000(*(reinterpret_cast<const Float64*>(&swath_data[4]))));
	
	QC() = std::bitset<32>(*(reinterpret_cast<const Int32*>(&swath_data[12])));
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
		FOV_QC(i) = std::bitset<8>(*(reinterpret_cast<const Int8*>(&swath_data[16+i-FOV_QC().first()])));
		
	height(*(reinterpret_cast<const Float32*>(&swath_data[28])));
	
	lat_.clone(reinterpret_cast<const Float32*>(&swath_data[32]));
	lon_.clone(reinterpret_cast<const Float32*>(&swath_data[76]));
	inc_.clone(reinterpret_cast<const Float32*>(&swath_data[120]));
	
	T_target_.first = *(reinterpret_cast<const Float32*>(&swath_data[164]));
	T_target_.second = *(reinterpret_cast<const Float32*>(&swath_data[168]));
	
	data_.clone(reinterpret_cast<const Float32*>(&swath_data[172]));
}

template<class Ver> 
inline const Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >& Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::read(const std::istream&)
{
	return *this;
}

template<class Ver> 
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,Ver> > >::write(std::ostream& os) const
{
	os.write(reinterpret_cast<const char*>(&swathNumber()),sizeof(Int32));				// 4 bytes
	
	const Float64	rd2000 = toRD2000(epoch());								
	
	os.write(reinterpret_cast<const char*>(&rd2000),sizeof(Float64));					// 8 bytes
	
	// metadata
	const Int32		qc = QC().to_ulong();
	
	os.write(reinterpret_cast<const char*>(&qc),sizeof(Int32));							// 4 bytes

	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
	{	
		const Int8	fovqc = FOV_QC(i).to_ulong();
		
		os.write(reinterpret_cast<const char*>(&fovqc),sizeof(Int8));					// 11 bytes
	}
	
	const Int8	fill_char = 0;
	
	os.write(&fill_char,sizeof(Int8));													// 1 byte
	
	// geolocation
	const Float32	ht = height();
	
	os.write(reinterpret_cast<const char*>(&ht),sizeof(Float32));						// 4 bytes
	
	latitude().put(os);																	// 44 bytes
	longitude().put(os);																// 44 bytes
	incidenceAngle().put(os);															// 44 bytes
	
	// target temperature
	os.write(reinterpret_cast<const char*>(&(T_target_.first)),sizeof(Float32));		// 4 bytes
	os.write(reinterpret_cast<const char*>(&(T_target_.second)),sizeof(Float32));		// 4 bytes
	
	// data
	data().put(os);																		// 176 bytes
	
	// total 																	   		// 348 bytes/swath
}

// nonlinear conversion of counts to temperature using Mo 1995 method
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::Swath() :
	Ordered<Date>(),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >()
{ }

template<class Ver>
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::Swath(const DatasetType& ds) :
	Ordered<Date>(),
	SatelliteMetadata<Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(ds),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >()
{ }

// construct Level 1A swath from Level 0 swath
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >& ob) :
	Ordered<Date>(ob),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(ob),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >(ob),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >()
{
	using namespace constants::physical;
	using namespace msu;
	
	T_target_.first = checkTargetTemperature(ob.T_Target1());
	T_target_.second = checkTargetTemperature(ob.T_Target2());
					
	// loop over channels
	Vector<Int8>	num_bad(IndexRange<1>(0,NUM_FOVS));
	
	for (Int32 chidx=TA().first(0);chidx<=TA().last(0);chidx++)
	{
		// nonlinear correction to counts
		const Float32	counts_Space = countConversion(chidx,ob.spaceView(chidx)),
						counts_Blackbody = countConversion(chidx,ob.blackbodyView(chidx)),
						counts_Scan = countConversion(chidx,ob.scanView(chidx));
		Float32			target_T;
				
		// target A for channels 1&2
		if (chidx == CH_1 || chidx == CH_2) 
			target_T = T_target_.first;
		// target B for channels 3&4
		else if (chidx == CH_3 || chidx == CH_4)
			target_T = T_target_.second;
				
		// loop over FOVs
		for (Int32 fovidx=TA().first(1);fovidx<=TA().last(1);fovidx++)
		{
			if (QC(FOVFlag(fovidx)) == true || !isfinite(target_T) || ob.counts(chidx,fovidx) == -1)
			{
				TA(chidx,fovidx,_NaN_);
			}
			else
			{
				// linear correction to counts and conversion to TA
				// eventually do inversion of Planck equation ???
				const Float32	counts_Earth = countConversion(chidx,ob.counts(chidx,fovidx));
							
				TA(chidx,fovidx,num::linearInterpolate(counts_Earth,counts_Space,Float32(_T_BC_),counts_Blackbody,target_T));
			}
			
			// bounds on reasonable values of TA based on histograms of all satellites
			static const Vector<Float32>	min_TA(IndexRange<1>(1,NUM_CHANNELS)," 160.0  192.0  185.0  175.0 "),
											max_TA(IndexRange<1>(1,NUM_CHANNELS)," 310.0  270.0  245.0  250.0 ");
								
			if (!isfinite(TA(chidx,fovidx)) || TA(chidx,fovidx) < min_TA[chidx] || TA(chidx,fovidx) > max_TA[chidx])
			{
				num_bad[0]++;
				num_bad[fovidx]++;
				
				TA(chidx,fovidx,_NaN_);
			}
		}
	}
	
	// update QC flags
	if (num_bad[0] == NUM_CHANNELS*NUM_FOVS) QC(BAD) = true;
	
	for (Int32 i=num_bad.first();i<=num_bad.last();++i)
		if (num_bad[i] == 4) FOV_QC(i,BAD_FOV) = true;
	
	QCConsistencyCheck();
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::get(const Vector<Uint8>& swath_data)
{
	swath_number_ = *(reinterpret_cast<const Int32*>(&swath_data[0]));
	
	epoch(fromRD2000(*(reinterpret_cast<const Float64*>(&swath_data[4]))));
	
	QC() = std::bitset<32>(*(reinterpret_cast<const Int32*>(&swath_data[12])));
	
	asc_ = QC(ASCENDING);
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
		FOV_QC(i) = std::bitset<8>(*(reinterpret_cast<const Int8*>(&swath_data[16+i-FOV_QC().first()])));
		
	height(*(reinterpret_cast<const Float32*>(&swath_data[28])));
	
	lat_.clone(reinterpret_cast<const Float32*>(&swath_data[32]));
	lon_.clone(reinterpret_cast<const Float32*>(&swath_data[76]));
	inc_.clone(reinterpret_cast<const Float32*>(&swath_data[120]));
	
	T_target_.first = *(reinterpret_cast<const Float32*>(&swath_data[164]));
	T_target_.second = *(reinterpret_cast<const Float32*>(&swath_data[168]));
	
	data_.clone(reinterpret_cast<const Float32*>(&swath_data[172]));
}

// correct for radiometer nonlinearities based on prelaunch thermal-vac tests
//	- NOAA-12 is a special case (see Mo 1995)
//
// coefficients in \\dell-p333a\f\msu_data\msu_calibration_data.txt
// 	see cal_coef_notes.txt in \\micron-p400c\e\msu\memos 
//	and \\micron-p266a\o\msu\stage1_proc\cal_coef.lis
//	for discussion of the selction of these coeffs
//
template<class Ver>
inline Float32 Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::countConversion(Int32 chidx,const Float32& raw_counts) const
{
	using namespace msu;
	
	static const Int32				NUM_COEF = 3;
	static const Array<Float32,3>	nl_coef(IndexRange<3>(Index<3>(1,1,0),Index<3>(NUM_SATELLITES,NUM_CHANNELS,NUM_COEF-1)),
											// TIROS-N
											       " 35.5369720459         .9302600026         .00001966600 "
											       " 26.8499755859         .9399644136         .00001726420 "
											       " 20.4720458984         .9639211297         .00000994080 "
											       " 22.2316589355         .9649070501         .00000965910 "
											// NOAA-06
											       " 56.2125549316         .9039525390         .00002132180 "
											       " 22.0407867432         .9275690913         .00001679100 "
											       " 15.9049997330         .9487079978         .00001500020 "
											       " 36.5099029541         .9355639815         .00001692300 "
											// NOAA-07
											       " 24.3630065918         .9488000274         .00001487000 "
											       " 26.0610046387         .9437999725         .00001626000 "
											       " 15.9359998703         .9629999995         .00001074000 "
											       " 21.7359924316         .9598000050         .00001124000 "
											// NOAA-08
											       " 22.4400024414         .9488999844         .00001630250 "
											       " 18.7100067139         .9588999748         .00001280040 "
											       " 25.8699951172         .9441999793         .00001636510 "
											       " 15.1400003433         .9532999992         .00001465280 "
											// NOAA-09
											       " 28.1499938965         .9593999982         .00001115140 "
											       " 20.5899963379         .9661999941         .00000966190 "
											       "103.2500000000         .9060999751         .00001931890 "
											       "102.9100036621         .9081000090         .00001860080 "
											// NOAA-10
											       " 36.0514068604         .9552710056         .00001170400 "
											       " 38.4689941406         .9546149969         .00001106220 "
											       "106.7239990234         .9076740146         .00001884020 "
											       "105.2270050049         .9087409973         .00001887950 "
											// NOAA-11
											       " 71.5310974121         .9319769740         .00001539720 "
											       " 58.5272064209         .9424229860         .00001348500 "
											       "129.7660064697         .8966810107         .00002063540 "
											       "138.3099975586         .8903009892         .00002123370 "
											// NOAA-12
											       " 68.9759000000         .9319730000         .00001501008 "
											       "200.0448000000         .8551300000         .00002593500 "
											       " 45.4182000000         .9474700000         .00001251516 "
											       " 69.3806000000         .9336700000         .00001461501 "
											// NOAA-14
											       " 90.6752777100         .9274770021         .00001377150 "
											       " 75.3933563232         .9399530292         .00001161060 "
											       " 44.2309112549         .9581810236         .00000890540 "
											       " 41.3986968994         .9579740167         .00000901370 ");
		
	return nl_coef(Index<3>(satelliteNumber(),chidx,0)) + raw_counts*(nl_coef(Index<3>(satelliteNumber(),chidx,1)) + raw_counts*(nl_coef(Index<3>(satelliteNumber(),chidx,2))));
}

template<class Ver> 
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,Ver> > >::write(std::ostream& os) const
{
	os.write(reinterpret_cast<const char*>(&swathNumber()),sizeof(Int32));				// 4 bytes
	
	const Float64	rd2000 = toRD2000(epoch());								
	
	os.write(reinterpret_cast<const char*>(&rd2000),sizeof(Float64));					// 8 bytes
	
	// metadata
	const Int32		qc = QC().to_ulong();
	
	os.write(reinterpret_cast<const char*>(&qc),sizeof(Int32));							// 4 bytes

	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
	{	
		const Int8	fovqc = FOV_QC(i).to_ulong();
		
		os.write(reinterpret_cast<const char*>(&fovqc),sizeof(Int8));					// 11 bytes
	}
	
	const Int8	fill_char = 0;
	
	os.write(&fill_char,sizeof(Int8));													// 1 byte
	
	// geolocation
	const Float32	ht = height();
	
	os.write(reinterpret_cast<const char*>(&ht),sizeof(Float32));						// 4 bytes
	
	latitude().put(os);																	// 44 bytes
	longitude().put(os);																// 44 bytes
	incidenceAngle().put(os);															// 44 bytes
	
	// target temperature
	os.write(reinterpret_cast<const char*>(&(T_target_.first)),sizeof(Float32));		// 4 bytes
	os.write(reinterpret_cast<const char*>(&(T_target_.second)),sizeof(Float32));		// 4 bytes
		
	// data
	data().put(os);																		// 176 bytes
	
	// total 																	  	 	// 348 bytes/swath
}

// nonlinear conversion of counts to temperature using Mo 2001 method
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::Swath() :
	Ordered<Date>(),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >()
{ }

template<class Ver>
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::Swath(const DatasetType& ds) :
	Ordered<Date>(),
	SatelliteMetadata<Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(ds),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >()
{ }

// construct Level 1A swath from Level 0 swath
template<class Ver> 
inline Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::Swath(const Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >& ob) :
	Ordered<Date>(ob),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(ob),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >(ob),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >()
{
	using namespace constants::physical;
	using namespace msu;
	
	T_target_.first =  checkTargetTemperature(ob.T_Target1());
	T_target_.second = checkTargetTemperature(ob.T_Target2());
	
	// loop over channels
	Vector<Int8>	num_bad(IndexRange<1>(0,NUM_FOVS));
	
	for (Int32 chidx=TA().first(0);chidx<=TA().last(0);chidx++)
	{
		// nonlinear correction to counts
		const Float32	counts_Space = ob.spaceView(chidx),
						counts_Blackbody = ob.blackbodyView(chidx),
						counts_Scan = ob.scanView(chidx);
		Float32			target_T;
								
		// target A for channels 1&2
		if (chidx == CH_1 || chidx == CH_2) 
			target_T = T_target_.first;
		// target B for channels 3&4
		else if (chidx == CH_3 || chidx == CH_4)
			target_T = T_target_.second;
				
		static const FixedVector<Float64,4>	FREQ(50.30,53.74,54.96,57.95),
											RC(radiation::PlanckRadiance(_T_BC_,FREQ[0])*1e5,
											   radiation::PlanckRadiance(_T_BC_,FREQ[1])*1e5,
											   radiation::PlanckRadiance(_T_BC_,FREQ[2])*1e5,
											   radiation::PlanckRadiance(_T_BC_,FREQ[3])*1e5);
		
		const Float64	DickeTemperature = ob.T_DickeLoad(chidx),
						freq = FREQ[chidx-1],
						Rc = RC[chidx-1];
						
		const bool	badDickeTemperature = (DickeTemperature < 260 || DickeTemperature > 320);
		
		// loop over FOVs
		for (Int32 fovidx=TA().first(1);fovidx<=TA().last(1);fovidx++)
		{
			if (!isfinite(target_T) || badDickeTemperature || QC(FOVFlag(fovidx)) == true || ob.counts(chidx,fovidx) == -1)
			{
				TA(chidx,fovidx,_NaN_);
			}
			else
			{
				const Float64	counts_Earth = ob.counts(chidx,fovidx),
								Rw = radiation::PlanckRadiance(target_T,freq)*1e5,					
								Cs_minus_Cw = counts_Earth-counts_Blackbody,
								Cs_minus_Cc = counts_Earth-counts_Space,
								Cw_minus_Cc = counts_Blackbody-counts_Space,
								G = Cw_minus_Cc/(Rw-Rc),
								u = countConversion(chidx,DickeTemperature)*1e-5,
								Rs_minus_Rw = (Cs_minus_Cw/G)*(1+u*Cs_minus_Cc/G),
								Ts = radiation::PlanckBrightnessTemperature((Rw+Rs_minus_Rw)*1e-5,freq);

				TA(chidx,fovidx,Ts);
			}
			
			// bounds on reasonable values of TA based on histograms of all satellites
			static const Vector<Float32>	min_TA(IndexRange<1>(1,NUM_CHANNELS)," 160.0  192.0  185.0  175.0 "),
											max_TA(IndexRange<1>(1,NUM_CHANNELS)," 310.0  270.0  245.0  250.0 ");
								
			if (!isfinite(TA(chidx,fovidx)) || TA(chidx,fovidx) < min_TA[chidx] || TA(chidx,fovidx) > max_TA[chidx])
			{
				num_bad[0]++;
				num_bad[fovidx]++;
				
				TA(chidx,fovidx,_NaN_);
			}					
		}
	}
	
	// update QC flags
	if (num_bad[0] == NUM_CHANNELS*NUM_FOVS) QC(BAD) = true;
	
	for (Int32 i=num_bad.first();i<=num_bad.last();++i)
		if (num_bad[i] == 4) FOV_QC(i,BAD_FOV) = true;
	
	QCConsistencyCheck();
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::get(const Vector<Uint8>& swath_data)
{
	swath_number_ = *(reinterpret_cast<const Int32*>(&swath_data[0]));
	
	epoch(fromRD2000(*(reinterpret_cast<const Float64*>(&swath_data[4]))));
	
	QC() = std::bitset<32>(*(reinterpret_cast<const Int32*>(&swath_data[12])));
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
		FOV_QC(i) = std::bitset<8>(*(reinterpret_cast<const Int8*>(&swath_data[16+i-FOV_QC().first()])));
		
	height(*(reinterpret_cast<const Float32*>(&swath_data[28])));
	
	lat_.clone(reinterpret_cast<const Float32*>(&swath_data[32]));
	lon_.clone(reinterpret_cast<const Float32*>(&swath_data[76]));
	inc_.clone(reinterpret_cast<const Float32*>(&swath_data[120]));
	
	T_target_.first = *(reinterpret_cast<const Float32*>(&swath_data[164]));
	T_target_.second = *(reinterpret_cast<const Float32*>(&swath_data[168]));
	
	data_.clone(reinterpret_cast<const Float32*>(&swath_data[172]));
}

// correct for radiometer nonlinearities based on AMSU-like calibration equation
//	(see Mo, et al. JGR 106 (10145) 2001)
//
//		returns u in m^2-sr-cm^-1/mW
template<class Ver>
inline Float32 Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::countConversion(Int32 chidx,const Float32& DickeTemperature) const
{
	using namespace msu;
	
	static const Int32				NUM_TEMP = 3;
	static const Array<Float32,3>	nl_coef(IndexRange<3>(Index<3>(1,0,1),Index<3>(NUM_SATELLITES,NUM_CHANNELS,NUM_TEMP)),
											// TIROS-N
													"	11.25			21.43			35.97		"
													"	6.459871		7.300734		6.359798	"
													"	4.679162		5.300233		4.614049	"
													"	5.201920		5.757293		5.780381	"
													"	4.253104		4.594762		4.756880	"
											// NOAA-06
											//	missing values - use NOAA-07
													"	10.00			10.50			39.53		"
													"	7.029300		7.029300		6.389682	"
													"	5.923696		5.923696		5.965380	"
													"	4.505891		4.505891		5.904314	"
													"	4.540941		4.540941		4.718846	"
											// NOAA-07
													"	10.00			10.50			39.53		"
													"	7.029300		7.029300		6.389682	"
													"	5.923696		5.923696		5.965380	"
													"	4.505891		4.505891		5.904314	"
													"	4.540941		4.540941		4.718846	"
											// NOAA-08
													"	10.35			15.84			25.52		"
													"	5.551043		5.249078		5.783429	"
													"	5.449507		5.100740		5.914635	"
													"	6.730589		6.623566		6.716581	"
													"	6.328403		6.156016		6.223986	"
											// NOAA-09
													"	9.46			10.15			39.27		"
													"	4.355292		5.966425		9.013816	"
													"	3.057721		5.197594		7.865034	"
													"	11.059461		8.772592		10.018774	"
													"	9.964882		7.607219		8.582250	"
											// NOAA-10
													"	9.58			19.26			25.77		"
													"	6.632992		6.156464		6.993056	"
													"	4.629639		4.953639		6.163996	"
													"	7.973084		6.845809		10.346873	"
													"	7.716055		5.591154		8.456595	"
											// NOAA-11
													"	8.99			18.87			39.04		"
													"	4.666636		8.293536		12.660619	"
													"	2.687082		7.702598		10.849120	"
													"	5.476822		10.795112		12.608739	"
													"	5.275250		9.085936		10.032414	"
											// NOAA-12
													"	10.00			20.00			35.00		"
													"	5.562017		4.894181		5.433871	"
													"	3.812639		3.114963		4.174880	"
													"	4.366952		4.092390		3.711945	"
													"	4.238008		4.073233		3.528564	"
											// NOAA-14
													"	10.00			20.00			35.00		"
													"	5.159236		5.300803		4.814198	"
													"	3.797210		3.150422		3.616177	"
													"	2.585159		2.844725		2.454696	"
													"	2.402867		2.485380		2.217051	");

	Vector<Float32>	T(IndexRange<1>(1,3)),
					C(IndexRange<1>(1,3));
					
	T[1] = nl_coef(Index<3>(satelliteNumber(),0,1))+constants::physical::_T_MP_;
	T[2] = nl_coef(Index<3>(satelliteNumber(),0,2))+constants::physical::_T_MP_;
	T[3] = nl_coef(Index<3>(satelliteNumber(),0,3))+constants::physical::_T_MP_;
		
	C[1] = nl_coef(Index<3>(satelliteNumber(),chidx,1));
	C[2] = nl_coef(Index<3>(satelliteNumber(),chidx,2));
	C[3] = nl_coef(Index<3>(satelliteNumber(),chidx,3));
	
	const num::LinearInterpolator<Float32>	Cint(T,C);
	
	// convert Dicke load counts to temperature
	return Cint(DickeTemperature);
}

template<class Ver> 
inline void Swath< Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,Ver> > >::write(std::ostream& os) const
{
	os.write(reinterpret_cast<const char*>(&swathNumber()),sizeof(Int32));				// 4 bytes
	
	const Float64	rd2000 = toRD2000(epoch());								
	
	os.write(reinterpret_cast<const char*>(&rd2000),sizeof(Float64));					// 8 bytes
	
	// metadata
	const Int32		qc = QC().to_ulong();
	
	os.write(reinterpret_cast<const char*>(&qc),sizeof(Int32));							// 4 bytes

	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
	{	
		const Int8	fovqc = FOV_QC(i).to_ulong();
		
		os.write(reinterpret_cast<const char*>(&fovqc),sizeof(Int8));					// 11 bytes
	}
	
	const Int8	fill_char = 0;
	
	os.write(&fill_char,sizeof(Int8));													// 1 byte
	
	// geolocation
	const Float32	ht = height();
	
	os.write(reinterpret_cast<const char*>(&ht),sizeof(Float32));						// 4 bytes
	
	latitude().put(os);																	// 44 bytes
	longitude().put(os);																// 44 bytes
	incidenceAngle().put(os);															// 44 bytes
	
	// target temperature
	os.write(reinterpret_cast<const char*>(&(T_target_.first)),sizeof(Float32));		// 4 bytes
	os.write(reinterpret_cast<const char*>(&(T_target_.second)),sizeof(Float32));		// 4 bytes
	
	// data
	data().put(os);																		// 176 bytes
	
	// total 																		   	// 348 bytes/swath
}

template<class Cal,class Ver>
inline void computeDiurnalCorrection(const Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw,const Int32& ch,Vector<Float32>& diurnal)
{
	const IndexRange<1>	fovrng(sw.TA().range(1));
	
	diurnal = Vector<Float32>(fovrng,0.0);
	
	if (!sw.passedQC())
	{
		diurnal = _NaN_;
		return;
	}
	
	for (Int32 fov=fovrng.first();fov<=fovrng.last();++fov)
	{
		if (!sw.passedFOV_QC(fov))
		{
			diurnal[fov] = _NaN_;
			continue;
		}
		
		const geo::GeographicCoordinate<Float32>	coord(sw.coordinate(fov));
		
		const Time	tim(localTime(sw.epoch().time(),coord));
		
		static const Time	new_tim(12,0,0);
			
		diurnal[fov] = msu::getMSU_CCM3DiurnalClimatology(ch,fov,Date(sw.epoch().day(),new_tim),coord)-
					   msu::getMSU_CCM3DiurnalClimatology(ch,fov,Date(sw.epoch().day(),tim),coord);
	}
	
	return;
}

template<class Cal,class Ver>
inline void applyDiurnalCorrection(const Int32& orbit_number,const Int32& chan,Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw)
{
	const Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >&	ds = sw.dataset();

	// generate orbit file name
	std::stringstream	sstr,
						fstr;
	
	const std::string	sat_name = ds.satelliteID().name();
	
	const Int32			folder_idx = (orbit_number-1)/5000;
	
	sstr << ds.directory()
		 << "Diurnal_Correction\\"
		 << "Ch_"+toString(chan)+"\\"
		 << "r"
		 << std::setw(5) << std::setfill('0') << 5000*folder_idx+1
		 << "_"
		 << std::setw(5) << std::setfill('0') << 5000*(folder_idx+1) 
		 << "\\";
		 
	fstr << sat_name+"_r" << std::setw(5) << std::setfill('0') << orbit_number;

	// flat diurnal files
	const std::string	prefix = sstr.str(),
						filename = prefix+fstr.str()+"_Ch"+toString(chan)+".dnl";
		 
	// read orbit header and cache decompressed data
	static Cache< std::string,Vector<Uint8> >	file_cache;
	
	file_cache.size(std::max(file_cache.size(),20));
	
	if (!file_cache.inCache(filename))
	{
		try
		{
			file_cache.add(filename,Vector<Uint8>());
			
			Vector<Uint8>&	orb_data = file_cache.get(filename);
		
			std::ifstream	ifs(filename.c_str(),std::ios::binary | std::ios::in);
			
			const Int32	num = fileLength(ifs)/sizeof(Uint8);
			
			orb_data = Vector<Uint8>(IndexRange<1>(num));
			
			orb_data.get(ifs);	
			
			ifs.close();
		}
		catch (Exception& ex)
		{
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::MissingOrbitErr();
		}
	}
	
	const Vector<Uint8>&	orb_data = file_cache.get(filename);

	// get number of swaths
	static const Int32	BYTES_PER_SWATH = 48;
	
	const Int32	num_scans = orb_data.dim()/BYTES_PER_SWATH-1;

	if (num_scans == -1) throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
	
	static Int32 last_orbit_read = -1;
	
	static std::map<Int32,Int32>	swath_number_to_offset;
	
	// read orbit header data
	if (last_orbit_read != orbit_number)
	{
		const char*	sat_name = reinterpret_cast<const char*>(&orb_data[0]);
		
		const Int32	orbit_num = *(reinterpret_cast<const Int32*>(&orb_data[8])),
					nscan = *(reinterpret_cast<const Int32*>(&orb_data[12]));

		if (orbit_num != orbit_number || nscan != num_scans) 
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
		
		const Int32	start_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[16])),
					end_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[20]));
			
		const Float64	rd2000_start = *(reinterpret_cast<const Float64*>(&orb_data[24])),
						rd2000_end = *(reinterpret_cast<const Float64*>(&orb_data[32]));
		
		// generate map indexing swath number to offset in orb_data
		swath_number_to_offset.clear();
		
		for (Int32 i=1;i<=num_scans;++i)
		{
			const Int32	offs = i*BYTES_PER_SWATH,
						swath_num = *(reinterpret_cast<const Int32*>(&orb_data[offs]));
			
			swath_number_to_offset[swath_num] = offs;
		}
		
		last_orbit_read = orbit_number;
	}
	
	const unsigned char*	addr = &(orb_data[swath_number_to_offset[sw.swathNumber()]]);
	
	const Int32				swath_number = *reinterpret_cast<const Int32*>(addr);
	const Vector<Float32>	deltaT(sw.fovRange(),reinterpret_cast<const Float32*>(addr+sizeof(Int32)));
	
	for (Int32 fov=sw.fovRange().first();fov<=sw.fovRange().last();++fov)
		sw.TA(chan,fov,sw.TA(chan,fov)+deltaT[fov]);

	return;
}

template<class Cal,class Ver>
inline Float32 getRollAngle(const Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw,Int32 chan)
{
	static bool	initialized = false;
	
	static Vector< Matrix<Float64> >	coef(IndexRange<1>(msu::CH_1,msu::CH_4));
	
	if (!initialized)
	{
		for (Int32 ch=msu::CH_1;ch<=msu::CH_4;++ch)
		{
			const std::string	fname = FileLocations["MSU Level 1A"]+"Roll_Error\\MSU_Roll_Error_Fit_Coefficients_Ch"+toString(ch)+".txt";
			
			std::ifstream		ifs(fname.c_str(),std::ios::in);
			
			std::stringstream	sstr;
			
			while (ifs) 
			{ 
				std::string	line;
				
				getline(ifs,line); 
				
				sstr << line; 
			}
				
			coef[ch] = Matrix<Float64>(IndexRange<2>(Index<2>(msu::TIROS_N,1),Index<2>(msu::NOAA_14,4)),sstr.str());
		}
		
		initialized = true;
	}
	
	// coefficients are fit using years from 1990
	const Int32		satnum = sw.satelliteNumber();
	const Float64	dt = (sw.epoch().julian()-Date(1990,1,1).julian())/365.25;
	
	return coef[chan](satnum,1)+dt*(coef[chan](satnum,2)+dt*(coef[chan](satnum,3)+dt*coef[chan](satnum,4)));
}

template<class Cal,class Ver> 
inline void computeAngleCorrection(const Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw,Int32 ch,Int32 fov,Float32& inc,Float32& limb,bool output) 
{
	using namespace mcs::sat::msu;
	
	if (!sw.passedFOV_QC(fov)) 
	{
		inc = limb = _NaN_;
		return;
	}
	
	// compute corrections to measured TB to nominal incidence angle 
	// viewidx = 0->nadir, 5->outer view
	const geo::GeographicCoordinate<Float32>	loc(sw.coordinate(fov));
	const geo::Geodetic<Float32>				pos(loc,sw.height());
	
	const Float32	sgn = fov > msu::SUBTRACK ? -1 : 1,
					roll_angle = sgn*static_cast<Float32>(getRollAngle(sw,ch)),
					nominal_incidence_angle = msu::INCIDENCE_ANGLES[fov],
					reported_incidence_angle = std::abs(sw.incidenceAngle(fov)),
					nadir_angle = nadirAngle(pos,reported_incidence_angle),
					actual_incidence_angle = earthIncidenceAngle(pos,nadir_angle+roll_angle);
	
	const FixedVector<Float32,2>	TB_nominal = getMSUClimatologyTB(ch,fov,nominal_incidence_angle,sw.epoch(),loc),
									TB_actual = getMSUClimatologyTB(ch,fov,actual_incidence_angle,sw.epoch(),loc),
									TB_nadir = getMSUClimatologyTB(ch,msu::SUBTRACK,0,sw.epoch(),loc);
	
	static const LandFraction	landFrac(0.5);
	
	const Float32	lf = landFrac(sw.coordinate(fov)),
					of = 1-lf;

	inc = (TB_nominal[LAND]-TB_actual[LAND])*lf + (TB_nominal[OCEAN]-TB_actual[OCEAN])*of,
	limb = (TB_nadir[LAND]-TB_nominal[LAND])*lf + (TB_nadir[OCEAN]-TB_nominal[OCEAN])*of;

	if (output)
	{
		std::cout << sw.epoch() << " " 
				  << std::setw(3) << fov << " " 
				  << std::setw(7) << std::setprecision(5) << roll_angle << " " 
				  << std::setw(8) << std::setprecision(7) << reported_incidence_angle << " "
				  << std::setw(8) << std::setprecision(7) << actual_incidence_angle << " "
				  << std::setw(8) << std::setprecision(7) << sw.TA(ch,fov) << " ";
	}
	
	return;
}

template<class Cal,class Ver>
inline void applyIncidenceCorrection(const Int32& orbit_number,const Int32& chan,Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw)
{
	const Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >&	ds = sw.dataset();

	// generate orbit file name
	std::stringstream	sstr,
						fstr;
	
	const std::string	sat_name = ds.satelliteID().name();
	
	const Int32			folder_idx = (orbit_number-1)/5000;
	
	sstr << ds.directory()
		 << "Incidence_Correction\\"
		 << "Ch_"+toString(chan)+"\\"
		 << "r"
		 << std::setw(5) << std::setfill('0') << 5000*folder_idx+1
		 << "_"
		 << std::setw(5) << std::setfill('0') << 5000*(folder_idx+1) 
		 << "\\";
		 
	fstr << sat_name+"_r" << std::setw(5) << std::setfill('0') << orbit_number;

	const std::string	prefix = sstr.str(),
						filename = prefix+fstr.str()+"_Ch"+toString(chan)+".inc";
		 
	// read orbit header and cache decompressed data
	static Cache< std::string,Vector<Uint8> >	file_cache;
	
	file_cache.size(std::max(file_cache.size(),20));
	
	if (!file_cache.inCache(filename))
	{
		try
		{
			file_cache.add(filename,Vector<Uint8>());
			
			Vector<Uint8>&	orb_data = file_cache.get(filename);
		
			std::ifstream	ifs(filename.c_str(),std::ios::binary | std::ios::in);
			
			const Int32	num = fileLength(ifs)/sizeof(Uint8);
			
			orb_data = Vector<Uint8>(IndexRange<1>(num));
			
			orb_data.get(ifs);	
			
			ifs.close();
		}
		catch (Exception& ex)
		{
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::MissingOrbitErr();
		}
	}
	
	const Vector<Uint8>&	orb_data = file_cache.get(filename);

	// get number of swaths
	static const Int32	BYTES_PER_SWATH = 48;
	
	const Int32	num_scans = orb_data.dim()/BYTES_PER_SWATH-1;

	if (num_scans == -1) throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
	
	static Int32 last_orbit_read = -1;
	
	static std::map<Int32,Int32>	swath_number_to_offset;
	
	// read orbit header data
	if (last_orbit_read != orbit_number)
	{
		const char*	sat_name = reinterpret_cast<const char*>(&orb_data[0]);
		
		const Int32	orbit_num = *(reinterpret_cast<const Int32*>(&orb_data[8])),
					nscan = *(reinterpret_cast<const Int32*>(&orb_data[12]));

		if (orbit_num != orbit_number || nscan != num_scans) 
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
		
		const Int32	start_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[16])),
					end_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[20]));
			
		const Float64	rd2000_start = *(reinterpret_cast<const Float64*>(&orb_data[24])),
						rd2000_end = *(reinterpret_cast<const Float64*>(&orb_data[32]));
		
		// generate map indexing swath number to offset in orb_data
		swath_number_to_offset.clear();
		
		for (Int32 i=1;i<=num_scans;++i)
		{
			const Int32	offs = i*BYTES_PER_SWATH,
						swath_num = *(reinterpret_cast<const Int32*>(&orb_data[offs]));
			
			swath_number_to_offset[swath_num] = offs;
		}
		
		last_orbit_read = orbit_number;
	}
	
	const unsigned char*	addr = &(orb_data[swath_number_to_offset[sw.swathNumber()]]);
	
	const Int32				swath_number = *reinterpret_cast<const Int32*>(addr);
	const Vector<Float32>	deltaT(sw.fovRange(),reinterpret_cast<const Float32*>(addr+sizeof(Int32)));
	
	for (Int32 fov=sw.fovRange().first();fov<=sw.fovRange().last();++fov)
		sw.TA(chan,fov,sw.TA(chan,fov)+deltaT[fov]);
		
	return;
}

template<class Cal,class Ver>
inline void applyLimbCorrection(const Int32& orbit_number,const Int32& chan,Swath< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >& sw)
{
	const Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >&	ds = sw.dataset();

	// generate orbit file name
	std::stringstream	sstr,
						fstr;
	
	const std::string	sat_name = ds.satelliteID().name();
	
	const Int32			folder_idx = (orbit_number-1)/5000;
	
	sstr << ds.directory()
		 << "Limb_Correction\\"
		 << "Ch_"+toString(chan)+"\\"
		 << "r"
		 << std::setw(5) << std::setfill('0') << 5000*folder_idx+1
		 << "_"
		 << std::setw(5) << std::setfill('0') << 5000*(folder_idx+1) 
		 << "\\";
		 
	fstr << sat_name+"_r" << std::setw(5) << std::setfill('0') << orbit_number;

	const std::string	prefix = sstr.str(),
						filename = prefix+fstr.str()+"_Ch"+toString(chan)+".lmb";
		 
	// read orbit header and cache decompressed data
	static Cache< std::string,Vector<Uint8> >	file_cache;
	
	file_cache.size(std::max(file_cache.size(),20));
	
	if (!file_cache.inCache(filename))
	{
		try
		{
			file_cache.add(filename,Vector<Uint8>());
			
			Vector<Uint8>&	orb_data = file_cache.get(filename);
		
			std::ifstream	ifs(filename.c_str(),std::ios::binary | std::ios::in);
			
			const Int32	num = fileLength(ifs)/sizeof(Uint8);
			
			orb_data = Vector<Uint8>(IndexRange<1>(num));
			
			orb_data.get(ifs);	
			
			ifs.close();
		}
		catch (Exception& ex)
		{
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::MissingOrbitErr();
		}
	}
	
	const Vector<Uint8>&	orb_data = file_cache.get(filename);

	// get number of swaths
	static const Int32	BYTES_PER_SWATH = 48;
	
	const Int32	num_scans = orb_data.dim()/BYTES_PER_SWATH-1;

	if (num_scans == -1) throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
	
	static Int32 last_orbit_read = -1;
	
	static std::map<Int32,Int32>	swath_number_to_offset;
	
	// read orbit header data
	if (last_orbit_read != orbit_number)
	{
		const char*	sat_name = reinterpret_cast<const char*>(&orb_data[0]);
		
		const Int32	orbit_num = *(reinterpret_cast<const Int32*>(&orb_data[8])),
					nscan = *(reinterpret_cast<const Int32*>(&orb_data[12]));

		if (orbit_num != orbit_number || nscan != num_scans) 
			throw typename Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::BadOrbitErr();
		
		const Int32	start_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[16])),
					end_swath_num = *(reinterpret_cast<const Int32*>(&orb_data[20]));
			
		const Float64	rd2000_start = *(reinterpret_cast<const Float64*>(&orb_data[24])),
						rd2000_end = *(reinterpret_cast<const Float64*>(&orb_data[32]));
		
		// generate map indexing swath number to offset in orb_data
		swath_number_to_offset.clear();
		
		for (Int32 i=1;i<=num_scans;++i)
		{
			const Int32	offs = i*BYTES_PER_SWATH,
						swath_num = *(reinterpret_cast<const Int32*>(&orb_data[offs]));
			
			swath_number_to_offset[swath_num] = offs;
		}
		
		last_orbit_read = orbit_number;
	}
	
	const unsigned char*	addr = &(orb_data[swath_number_to_offset[sw.swathNumber()]]);
	
	const Int32				swath_number = *reinterpret_cast<const Int32*>(addr);
	const Vector<Float32>	deltaT(sw.fovRange(),reinterpret_cast<const Float32*>(addr+sizeof(Int32)));
	
	for (Int32 fov=sw.fovRange().first();fov<=sw.fovRange().last();++fov)
		sw.TA(chan,fov,sw.TA(chan,fov)+deltaT[fov]);
		
	return;
}

} // namespace sat

} // namespace mcs

#endif // MCS_MSULEVEL1A_H