#include <iomanip>

#include "MCS_MSUOrbit.h"

#include "MCS_FileLocations.h"
#include "MCS_IO.h"

namespace mcs {

namespace sat {

// Level 0 orbit
template<class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >::Orbit() :
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ }

template<class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >::Orbit(const DatasetType& ds,const Int32& orbit_number) : 
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ 
	getOrbit(OrbitIteratorImp<DatasetType>(ds,orbit_number));
}

template<class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >::Orbit(const OrbitIteratorImp<DatasetType>& imp) : 
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ 
	getOrbit(imp);
}

template<class Ver>
inline void Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >::getOrbit(const OrbitIteratorImp<DatasetType>& imp)
{
	ds_ = imp.dataset();
	orbit_number_ = imp.orbitNumber();

	// check inventory for orbit
	if (find(ds_.inventory().begin(),ds_.inventory().end(),orbit_number_) == ds_.inventory().end())
		throw MissingOrbitErr();
		
	// generate orbit file name
	std::stringstream	sstr;
	
	//const std::string&	sat_name = ds_.satelliteID().name();
	const std::string	sat_name = ds_.satelliteID().name();
	
	const Int32			folder_idx = (orbit_number_-1)/5000;
	
	sstr << ds_.directory()
		 << "r"
		 << std::setw(5) << std::setfill('0') << 5000*folder_idx+1
		 << "_"
		 << std::setw(5) << std::setfill('0') << 5000*(folder_idx+1) 
		 << "\\" << sat_name+"_r"
		 << std::setw(5) << std::setfill('0') << orbit_number_ << ".L0.gz";
		 
	filename_ = sstr.str();
	
	// read orbit header and cache decompressed data
	Cache< std::string,Vector<Uint8> >&	gz_cache = Singleton< Tag< Cache<std::string,Vector<Uint8> >,DatasetType > >::instance();
	
	gz_cache.size(std::max(gz_cache.size(),5));
	
	if (!gz_cache.inCache(filename_))
	{
		try
		{
			gz_cache.add(filename_,Vector<Uint8>());
			
			Vector<Uint8>&	orb_data = gz_cache.get(filename_);
		
			gz::readGZIP(filename_,orb_data);	
		}
		catch (Exception& ex)
		{
			throw MissingOrbitErr();
		}
	}
	
	orbit_data_ptr_ = &(gz_cache.get(filename_));
	
	const Vector<Uint8>&	orb_data = *orbit_data_ptr_;
	
	// get number of swaths
	num_scans_ = orb_data.dim()/bytesPerSwath()-1;

	if (num_scans_ == -1) throw BadOrbitErr();
	
	// read orbit header data
	if (last_orbit_read_ != orbit_number_)
	{
		const char*	sat_name = reinterpret_cast<const char*>(&orb_data[0]);
		
		const Int32	orbit_num = *(reinterpret_cast<const Int32*>(&orb_data[8])),
					nscan = *(reinterpret_cast<const Int32*>(&orb_data[12]));

		if (orbit_num != orbit_number_ || nscan != num_scans_) throw BadOrbitErr();
		
		start_swath_num_ = *(reinterpret_cast<const Int32*>(&orb_data[16]));
		end_swath_num_ = *(reinterpret_cast<const Int32*>(&orb_data[20]));
			
		const Float64	rd2000_start = *(reinterpret_cast<const Float64*>(&orb_data[24])),
						rd2000_end = *(reinterpret_cast<const Float64*>(&orb_data[32]));
		
		start_epoch_ = fromRD2000(rd2000_start);
		end_epoch_ = fromRD2000(rd2000_end);
		
		// generate map indexing swath number to offset in orb_data
		swath_number_to_offset_.clear();
		
		for (Int32 i=1;i<=num_scans_;++i)
		{
			const Int32	offs = i*bytesPerSwath(),
						swath_num = *(reinterpret_cast<const Int32*>(&orb_data[offs]));
			
			swath_number_to_offset_[swath_num] = offs;
		}
		
		last_orbit_read_ = orbit_number_;
	}
}

template<class Ver>
inline void Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >::getSwath(Swath<DatasetType>& swath,const SwathIteratorImp<DatasetType>& imp) const
{
	const Int32	swath_number = imp.swathNumber();
	
	// get data for swath
	std::map<Int32,Int32>::const_iterator	it = swath_number_to_offset_.find(swath_number);
	
	// swath not found
	if (it == swath_number_to_offset_.end()) throw MissingSwathErr();

	const Vector<Uint8>&	orb_data = *orbit_data_ptr_;
	
	const Vector<Uint8>	swath_data(IndexRange<1>(bytesPerSwath()),&(orb_data[it->second]));
	
	swath.get(swath_data);
}

// Level 1A orbit
template<class Cal,class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::Orbit() :
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ }

template<class Cal,class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::Orbit(const DatasetType& ds,const Int32& orbit_number) : 
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ 
	getOrbit(OrbitIteratorImp<DatasetType>(ds,orbit_number));
}

template<class Cal,class Ver>
inline Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::Orbit(const OrbitIteratorImp<DatasetType>& imp) : 
	ds_(),
	orbit_number_(),
	last_orbit_read_(defaultOrbitNumber())
{ 
	getOrbit(imp);
}

template<class Cal,class Ver>
inline void Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::getOrbit(const OrbitIteratorImp<DatasetType>& imp)
{
	ds_ = imp.dataset();
	orbit_number_ = imp.orbitNumber();

	// generate orbit file name
	std::stringstream	sstr;
	
	//const std::string&	sat_name = ds_.satelliteID().name();
	const std::string	sat_name = ds_.satelliteID().name();
	
	const Int32			folder_idx = (orbit_number_-1)/5000;
	
	sstr << ds_.directory()
		 << Cal::tag() << "\\"
		 << "r"
		 << std::setw(5) << std::setfill('0') << 5000*folder_idx+1
		 << "_"
		 << std::setw(5) << std::setfill('0') << 5000*(folder_idx+1) 
		 << "\\" << sat_name+"_r"
		 << std::setw(5) << std::setfill('0') << orbit_number_ << ".L1A.gz";
		 
	filename_ = sstr.str();
	
	// read orbit header and cache decompressed data
	Cache< std::string,Vector<Uint8> >&	gz_cache = Singleton< Tag< Cache< std::string,Vector<Uint8> >,DatasetType> >::instance();
	
	gz_cache.size(std::max(gz_cache.size(),5));
	
	if (!gz_cache.inCache(filename_))
	{
		try
		{
			gz_cache.add(filename_,Vector<Uint8>());
			
			Vector<Uint8>&	orb_data = gz_cache.get(filename_);
		
			gz::readGZIP(filename_,orb_data);	
		}
		catch (Exception& ex)
		{
			throw MissingOrbitErr();
		}
	}
	
	orbit_data_ptr_ = &(gz_cache.get(filename_));
	
	const Vector<Uint8>&	orb_data = *orbit_data_ptr_;
	
	// get number of swaths
	num_scans_ = orb_data.dim()/bytesPerSwath()-1;

	if (num_scans_ == -1) throw MissingOrbitErr();
	
	// read orbit header data
	if (last_orbit_read_ != orbit_number_)
	{
		const char*	sat_name = reinterpret_cast<const char*>(&orb_data[0]);
		
		const Int32	orbit_num = *(reinterpret_cast<const Int32*>(&orb_data[8])),
					nscan = *(reinterpret_cast<const Int32*>(&orb_data[12]));

		if (orbit_num != orbit_number_ || nscan != num_scans_) throw BadOrbitErr();
		
		start_swath_num_ = *(reinterpret_cast<const Int32*>(&orb_data[16]));
		end_swath_num_ = *(reinterpret_cast<const Int32*>(&orb_data[20]));
			
		const Float64	rd2000_start = *(reinterpret_cast<const Float64*>(&orb_data[24])),
						rd2000_end = *(reinterpret_cast<const Float64*>(&orb_data[32]));
		
		start_epoch_ = fromRD2000(rd2000_start);
		end_epoch_ = fromRD2000(rd2000_end);
		
		// generate map indexing swath number to offset in orb_data
		swath_number_to_offset_.clear();
		
		for (Int32 i=1;i<=num_scans_;++i)
		{
			const Int32	offs = i*bytesPerSwath(),
						swath_num = *(reinterpret_cast<const Int32*>(&orb_data[offs]));
			
			swath_number_to_offset_[swath_num] = offs;
		}
		
		last_orbit_read_ = orbit_number_;
	}
}

template<class Cal,class Ver>
inline void Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >::getSwath(Swath<DatasetType>& swath,const SwathIteratorImp<DatasetType>& imp) const
{
	const Int32	swath_number = imp.swathNumber();
	
	// get data for swath
	std::map<Int32,Int32>::const_iterator	it = swath_number_to_offset_.find(swath_number);
	
	// swath not found
	if (it == swath_number_to_offset_.end()) throw MissingSwathErr();

	const Vector<Uint8>&	orb_data = *orbit_data_ptr_;
	
	const Vector<Uint8>	swath_data(IndexRange<1>(bytesPerSwath()),&(orb_data[it->second]));
	
	swath.get(swath_data);
}

} // namespace sat

} // namespace mcs
