#ifndef MCS_MSUORBIT_H
#define MCS_MSUORBIT_H

#include <map>

#include "MCS_Exceptions.h"
#include "MCS_MSUOrbitData.h"
#include "MCS_SatelliteOrbitIterator.h"
#include "MCS_SatelliteSwathIterator.h"
#include "MCS_Vector.h"

namespace mcs {

namespace sat {

// Level 0 orbit
template<class Ver> 
class Orbit< Dataset< MSU<SwathData,msu::Level0,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level0,Ver> >					DatasetType;
		typedef Orbit<DatasetType>											OrbitType;
		typedef Swath<DatasetType>											SwathType;
		typedef SwathIteratorImp<DatasetType>								IteratorImpType;			
		typedef Iterator<SwathType,SwathIteratorImp<DatasetType> >			iterator;
		typedef ReverseIterator<SwathType,SwathIteratorImp<DatasetType> >	reverse_iterator;
		
		Orbit();
		explicit Orbit(const DatasetType& ds,const Int32& orbit_number);
		explicit Orbit(const OrbitIteratorImp<DatasetType>& imp);
		~Orbit() { }
		
		const DatasetType& dataset() const							{ return ds_; }
		const SatelliteID& satelliteID() const						{ return ds_.satelliteID(); }
		
		const std::string& filename() const							{ return filename_; }
		const Int32& orbitNumber() const							{ return orbit_number_; }
		
		void getOrbit(const OrbitIteratorImp<DatasetType>& imp);
		
		Int32 firstSwath() const									{ return start_swath_num_; }
		Int32 lastSwath() const										{ return end_swath_num_; }
		
		const Date& startDate() const								{ return start_epoch_; }
		const Date& endDate() const									{ return end_epoch_; }
		
		const Int32& numberOfSwaths() const							{ return num_scans_; }
		
		// swath iterators
		void getSwath(Swath<DatasetType>& swath,const SwathIteratorImp<DatasetType>& imp) const;
		
		iterator begin() const										{ return iterator(IteratorImpType(*this,firstSwath())); }
		iterator number(const Int32& num) const						{ return iterator(IteratorImpType(*this,num)); }
		iterator end() const										{ return iterator(IteratorImpType(*this,lastSwath()+1)); }
			
		reverse_iterator rbegin() const								{ return reverse_iterator(IteratorImpType(*this,lastSwath())); }
		reverse_iterator rnumber(const Int32& num) const			{ return reverse_iterator(IteratorImpType(*this,num)); }
		reverse_iterator rend()	const								{ return reverse_iterator(IteratorImpType(*this,firstSwath()-1)); }

		class Err : public Exception { };
		
		class BadOrbitErr : public Err { };
		class MissingOrbitErr : public Err { };
		class BadSwathErr : public Err { };
		class MissingSwathErr : public Err { };
		
	private:
		static Int32 defaultOrbitNumber()	{ return -1; }
		static Int32 bytesPerSwath()		{ return 340; }
		
		DatasetType		ds_;
		Int32			orbit_number_;
		std::string		filename_;
		
		mutable Int32	last_orbit_read_,
						num_scans_,
						start_swath_num_,
						end_swath_num_;
		mutable Date	start_epoch_,
						end_epoch_;
						
		mutable Vector<Uint8>*	orbit_data_ptr_;
						
		mutable std::map<Int32,Int32>	swath_number_to_offset_;	
};

// Level 1A orbit
template<class Cal,class Ver> 
class Orbit< Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> > >
{
	public:
		typedef Dataset< MSU<SwathData,msu::Level1A<Cal>,Ver> >				DatasetType;
		typedef Orbit<DatasetType>											OrbitType;
		typedef Swath<DatasetType>											SwathType;
		typedef SwathIteratorImp<DatasetType>								IteratorImpType;			
		typedef Iterator<SwathType,SwathIteratorImp<DatasetType> >			iterator;
		typedef ReverseIterator<SwathType,SwathIteratorImp<DatasetType> >	reverse_iterator;
		
		Orbit();
		explicit Orbit(const DatasetType& ds,const Int32& orbit_number);
		explicit Orbit(const OrbitIteratorImp<DatasetType>& imp);
		~Orbit() { }
		
		const DatasetType& dataset() const							{ return ds_; }
		const SatelliteID& satelliteID() const						{ return ds_.satelliteID(); }
		
		const std::string& filename() const							{ return filename_; }
		const Int32& orbitNumber() const							{ return orbit_number_; }
		
		void getOrbit(const OrbitIteratorImp<DatasetType>& imp);
		
		Int32 firstSwath() const									{ return start_swath_num_; }
		Int32 lastSwath() const										{ return end_swath_num_; }
			
		const Date& beginDate() const								{ return start_epoch_; }
		const Date& endDate() const									{ return end_epoch_; }
		
		const Int32& numberOfSwaths() const							{ return num_scans_; }
		
		// swath iterators
		void getSwath(Swath<DatasetType>& swath,const SwathIteratorImp<DatasetType>& imp) const;
		
		iterator begin() const										{ return iterator(IteratorImpType(*this,firstSwath())); }
		iterator number(const Int32& num) const						{ return iterator(IteratorImpType(*this,num)); }
		iterator end() const										{ return iterator(IteratorImpType(*this,lastSwath()+1)); }
			
		reverse_iterator rbegin() const								{ return reverse_iterator(IteratorImpType(*this,lastSwath())); }
		reverse_iterator rnumber(const Int32& num) const			{ return reverse_iterator(IteratorImpType(*this,num)); }
		reverse_iterator rend()	const								{ return reverse_iterator(IteratorImpType(*this,firstSwath()-1)); }

		class Err : public Exception { };
		
		class BadOrbitErr : public Err { };
		class MissingOrbitErr : public Err { };
		class BadSwathErr : public Err { };
		class MissingSwathErr : public Err { };
		
	private:
		static Int32 defaultOrbitNumber()	{ return -1; }
		static Int32 bytesPerSwath()		{ return 348; }
		
		DatasetType		ds_;
		Int32			orbit_number_;
		std::string		filename_;
		
		mutable Int32	last_orbit_read_,
						num_scans_,
						start_swath_num_,
						end_swath_num_;
		mutable Date	start_epoch_,
						end_epoch_;
						
		mutable Vector<Uint8>*	orbit_data_ptr_;
						
		mutable std::map<Int32,Int32>	swath_number_to_offset_;	
};

} // namespace sat

} // namespace mcs

// implementation
#include "MCS_MSUOrbit.cpp"

#endif // MCS_MSUORBIT_H