#include <iomanip>
#include <map>

#include "MCS_Constants.h"
#include "MCS_FileLocations.h"
#include "MCS_Matrix.h"
#include "MCS_MSUSeries.h"

namespace mcs {

static const Index<1>		LAT_RANGE(72),
							LON_RANGE(144);
static const IndexRange<2>	GRID_RANGE(Index<2>(LAT_RANGE,LON_RANGE));

static const geo::EdgeEarthGrid<Float32>	MSUGRID(GRID_RANGE);

static std::string getMGAnnualCycleFilename(const Date& ep,const Metadata& meta)
{
	using namespace std;
	
	stringstream	tmp;
	
	static const string	directory = FileLocations["MSU S&G gridded"];
	
	tmp << directory;
	//tmp << MSU_SC_GRIDDED_DATA_DIRECTORY;
	
	if (meta["version"] == "d3")
	{
		const string&	channel = meta["channel"];
		
		tmp << "MSUd3\\";
		
		if (channel == "2LT")
			tmp << "t2lt\\t2lt";
		else if (channel == "2")
			tmp << "t2\\t2";
		else if (channel == "4")
			tmp << "t4\\t4";
		else
			throw MSUSeries::Err();

		tmp << "monacg.d";
	}
	else throw MSUSeries::Err();
	
	return tmp.str();	
}

static std::string getMGAnomalyFilename(const Date& ep,const Metadata& meta)
{
	using namespace std;
	
	stringstream	tmp;
	
	static const string	directory = FileLocations["MSU S&G gridded"];
	
	tmp << directory;
	//tmp << MSU_SC_GRIDDED_DATA_DIRECTORY;
	
	if (meta["version"] == "d3")
	{
		tmp << "MSUd3\\";
		
		const string&	channel = meta["channel"];
		
		if (channel == "2LT")
			tmp << "t2lt\\t2lt";
		else if (channel == "2")
			tmp << "t2\\t2";
		else if (channel == "4")
			tmp << "t4\\t4";
		else
			throw MSUSeries::Err();
	
		tmp << "monamg." << setw(4) << setfill('0') << ep.year() << "d";		
	}
	else throw MSUSeries::Err();

	return tmp.str();	
}

static std::string getMGClimatologyFilename(const Date& ep,const Metadata& meta)
{
	using namespace std;
	
	stringstream	tmp;
	
	static const string	directory = FileLocations["MSU S&G gridded"];
	
	tmp << directory;
	//tmp << MSU_SC_GRIDDED_DATA_DIRECTORY;
	
	if (meta["version"] == "d3")
	{
		const string&	channel = meta["channel"];
		
		tmp << "MSUd3\\";
		
		if (channel == "2LT")
			tmp << "t2lt\\t2lt";
		else if (channel == "2")
			tmp << "t2\\t2";
		else if (channel == "4")
			tmp << "t4\\t4";
		else
			throw MSUSeries::Err();

		tmp << "monacg.d";
	}
	else throw MSUSeries::Err();
	
	return tmp.str();	
}

static void ReadMap(std::ifstream& ifs,Matrix<Float32>& data)
{	
	using namespace std;
	
	static const Float32	TEMP_SCALE_FACTOR = 0.01;
	
	string	buf;
			
	getline(ifs,buf);
	
	int 	imon,idate1,idate2;

	imon = atoi(buf.substr(0,12).c_str());
	idate1 = atoi(buf.substr(12,12).c_str());
	idate2 = atoi(buf.substr(24,12).c_str());
					
	for (Index<1> line_idx=0;line_idx<LAT_RANGE*LON_RANGE/16;line_idx++)
	{
		getline(ifs,buf);
						
		for (Index<1> sub_idx=0;sub_idx<16;sub_idx++)
		{
			const string	sval = buf.substr(sub_idx*5,5);
			
			const Float32 	val = atoi(sval.c_str())*TEMP_SCALE_FACTOR;
			
			const Index<1> 	lat_idx = line_idx/9,
							lon_idx = (line_idx % 9)*16 + sub_idx;
			
			data(lat_idx,lon_idx) = val;
		}
	}
	
	// swap longitude halves to have Pacific in center of map
	Index<1>	middle = std::floor(data.dim().rank(1)/2.0);
	
	for (Index<1> j=data.first().rank(1);j<=data.first().rank(1)+middle-1;j++)
		swapColumns(data,j,j+middle);
		
	return;
}

MSUSeries::MSUSeries(const std::string& metainit) :
	MapSeries<MapType>(metainit)
{		
	curr_ = MapType(MSUGRID);
	first(); 
}

MSUSeries::MSUSeries(const std::string& metainit,const Date& ep) :
	MapSeries<MapType>(metainit)
{
	curr_ = MapType(MSUGRID);
	find(ep); 
}

MSUSeries::~MSUSeries()
{ }

void MSUSeries::addMGAnnualCycle()
{
	using namespace std;
	
	static Vector<MapType>	AC_data(IndexRange<1>(1,12));
	static bool 			AC_initialized = false;
	
	if (!AC_initialized)
	{
		ifstream	ifs(getMGAnnualCycleFilename(epoch(),metadata()).c_str(),ios::in);
		
		// loop over months
		for (Index<1> month=AC_data.first();month<=AC_data.last();++month)
		{	
			AC_data[month] = MapType(GRID_RANGE,_NaN_);
			
			ReadMap(ifs,AC_data[month]);			
		}
		
		ifs.close();
		AC_initialized = true;
	}	

	curr_ += AC_data[epoch().month()];
}

void MSUSeries::addMGAnomaly()
{
	using namespace std;
	
	static const std::streamoff	OFFSET = 53186;

	Matrix<Float32>	AN_data(GRID_RANGE,_NaN_);
		
	//anifs_.filename(getMGAnomalyFilename(epoch(),metadata()));
	anifs_.close();
	anifs_.open(getMGAnomalyFilename(epoch(),metadata()).c_str(),std::ios::binary | std::ios::in);
	
	anifs_.seekg((epoch().month()-1)*OFFSET);
		
	ReadMap(anifs_,AN_data);	

	curr_ += AN_data;
}

void MSUSeries::addMGClimatology()
{
	using namespace std;
	
	static Vector<MapType>	clim_data(IndexRange<1>(1,12));
	static bool 			clim_initialized = false;
	
	if (!clim_initialized)
	{
		ifstream	ifs(getMGClimatologyFilename(epoch(),metadata()).c_str(),ios::binary | ios::in);
		
		// loop over months
		for (Index<1> month=clim_data.first();month<=clim_data.last();++month)
			clim_data[month].read(ifs);		
		
		ifs.close();
		clim_initialized = true;
	}	

	curr_ += clim_data[epoch().month()];
}

void MSUSeries::read()
{
	using namespace std;
	
	if (metadata() == last_meta_ && findNearestInterval(epoch()) == findNearestInterval(last_epoch_)) return;
	
	curr_ = static_cast<Float32>(0);
	
	const string&	variable = metadata("variable");
	
	if (variable == "TB-annual-cycle")
	{
		addMGAnnualCycle();
	}
	else if (variable == "TB-anomaly")
	{
		addMGAnomaly();
	}
	else if (variable == "TB")
	{
		addMGAnnualCycle(); 
		addMGAnomaly();
	}
	else if (variable == "TB-climatology")
	{
		addMGClimatology();
	}
	else
	{
		throw Err();
	}
	
	last_meta_ = metadata();
	last_epoch_ = epoch();
}

Date MSUSeries::begin() const
{
	return Date(1979,1,1);
}

Date MSUSeries::end() const
{
	using namespace std;
	
	const string&	channel = metadata("channel");
	
	if (channel == "2LT")
	{
		return Date(2000,11,30);
	}
	else if (channel == "2")
	{
		return Date(1999,12,31);
	}
	else if (channel == "4")
	{
		return Date(1999,12,31);
	}
	else
	{
		throw Err(); 
		return Date();
	}
}

Date MSUSeries::findNearestInterval(const Date& ep) const
{
	Date	bom(ep.year(),ep.month(),1),
			eom(ep.year(),ep.month(),1);
			
	eom.addMonths(1);
	
	return Date((bom.julian()+eom.julian())/2.0);
}

Date MSUSeries::incrementInterval(Uint32 num) const
{
	return Date(epoch()).addMonths(num);
}
	
} // namespace mcs
