#include <iomanip>

#include "MCS_Date.h"
#include "MCS_FileLocations.h"
#include "MCS_FixedVector.h"
#include "MCS_Geography.h"
#include "MCS_MSUConstants.h"
#include "MCS_MSUUtilities.h"
#include "MCS_Satellite.h"
#include "MCS_Tensor.h"
#include "MCS_Vector.h"

namespace mcs {

namespace sat {

namespace msu {

namespace {

// evaluate derivative term dTB/dtheta_inc
inline Float32 evaluateTB(const FixedVector<Float32,3>& taylor,Float32 dtheta)
{
	return taylor[0]+dtheta*(taylor[1]+dtheta*(taylor[2]/2));
}

} // namespace

// compute NCEP climatology TB for specified time/location/nadir angle
FixedVector<Float32,2> getMSUClimatologyTB(Int32 channel,Int32 fov,Float32 incidence_angle,const Date& ep,const geo::GeographicCoordinate<Float32>& loc) 
{
	using namespace mcs::sat;
	
	static const Int32	VPOL = 0,
						HPOL = 1,
						NUM_CHANNELS = 4,			// 1..4
						NUM_MONTHS = 12,			// 1..12
						NUM_SURFACES = 2,			// land = 0,ocean = 1
						NUM_POLARIZATIONS = 2,		// vpol = 0, hpol = 1
						NUM_ANGLES = 6,				// nominal incidence angles 0.0,10.71,21.51,32.51,43.91,56.19
						NUM_VARIABLES = 3;			// TB/TB'/TB'' = 0/1/2 where TB(Q)=TB+TB'*(Q-Q0)+(TB''/2)*(Q-Q0)^2
												
	// nominal incidence angles computed as average values of zero-roll-error EIA from 750-900km
	typedef geo::NCEPGrid<Float32>					GridType;
	typedef geo::EarthMap<Float32,Float32,GridType>	MapType;
	
	static const GridType	GRID(IndexRange<2>(Index<2>(73,144)));
	
	static const IndexRange<6>	datarng(Index<6>(1,1,0,0,0,0),Index<6>(NUM_CHANNELS,NUM_MONTHS,NUM_SURFACES-1,NUM_POLARIZATIONS-1,NUM_ANGLES-1,NUM_VARIABLES-1));	
	static Tensor<MapType*,6>	data(datarng);

	static Matrix<bool>	already_read(IndexRange<2>(Index<2>(1,1),Index<2>(NUM_CHANNELS,NUM_MONTHS)),false);
	
	// determine which months are needed for interpolation
	Date	before_ep(ep.year(),ep.month(),1),
			after_ep(before_ep);

	if (ep.dayOfMonth() < ep.daysInMonth()/2)
		before_ep.addMonths(-1);
	else
		after_ep.addMonths(1);
	
	Day	bom,eom;
	
	bom = Day(before_ep.year(),before_ep.month(),1);
	eom = Day(before_ep.year(),before_ep.month(),before_ep.daysInMonth());
				
	before_ep = Date(0.5*(Date(bom,Time(0,0,0)).julian()+Date(eom,Time(23,59,59)).julian()));
	
	bom = Day(after_ep.year(),after_ep.month(),1);
	eom = Day(after_ep.year(),after_ep.month(),after_ep.daysInMonth());			

	after_ep = Date(0.5*(Date(bom,Time(0,0,0)).julian()+Date(eom,Time(23,59,59)).julian()));			

	// load new NCEP TB climatology
	static const Int32	BEFORE = 0,
						AFTER = 1;
	
	for (Int32 intidx=BEFORE;intidx<=AFTER;++intidx)
	{
		const Date&		curr_ep = (intidx == BEFORE) ? before_ep : after_ep;
		const Int32		mon = curr_ep.month();
		const Index<2>	ch_mon_idx(channel,mon);
	
		if (!already_read(ch_mon_idx))
		{
			std::cerr << "Reading " << monthString(curr_ep) << " NCEP dTB/dq data for MSU channel " << channel << "..."; 
			
			static const std::string	prefix = FileLocations["NCEP MSU TB"]+"tbdata_CH",
										suffix = "_topo_theta.dat";
								
			std::stringstream	lstr,ostr;
			
			const Int32	ichan = channel;
			
			lstr << prefix << std::setw(1) << ichan << "_S2_1996_" 
				 << std::setw(2) << std::setfill('0') << mon << suffix;
			ostr << prefix << std::setw(1) << ichan << "_S1_1996_" 
				 << std::setw(2) << std::setfill('0') << mon << suffix;
				
			std::ifstream	ifsl(lstr.str().c_str(),std::ios::binary | std::ios::in),
							ifso(ostr.str().c_str(),std::ios::binary | std::ios::in);
					
			Index<6>	idx;
			
			idx[0] = channel;
			idx[1] = mon;
			
			Matrix<Float32>	tmp(IndexRange<2>(Index<2>(144,73)));
				
			for (idx[2]=data.first(2);idx[2]<=data.last(2);++idx[2])
			{
				std::ifstream&	ifs = (idx[2] == LAND) ? ifsl : ifso;
				
				for (idx[3]=datarng.first(3);idx[3]<=datarng.last(3);++idx[3])
				{
					for (idx[4]=datarng.first(4);idx[4]<=datarng.last(4);++idx[4])
					{
						//engine.put("view",idx[4]+1);
						
						for (idx[5]=datarng.first(5);idx[5]<=datarng.last(5);++idx[5])
						{
							tmp.get(ifs);
							
							data(idx) = new MapType(GRID,transpose(tmp));
						}
					}
				}
			}
			
			ifsl.close();
			ifso.close();	
			
			already_read(ch_mon_idx) = true;
			
			std::cerr << "done" << std::endl;
		}
	}

	// compute corrections to measured TB to nominal incidence angle 
	// viewidx = 0->nadir, 5->outer view
	// first figure out which view we are closest to
	incidence_angle = std::abs(incidence_angle);
	
	const Index<1>	viewidx = std::abs(fov-6);
		
	// nominal incidence angle (for polarization mixing)
	const Float32	dtheta_inc_d = incidence_angle-INCIDENCE_ANGLES[fov],
					thetan_r = toRadians(NADIR_ANGLES[fov]),
					cos2thetan = std::sqr(std::cos(thetan_r)),
					sin2thetan = std::sqr(std::sin(thetan_r));
	
	// interpolate in time
	FixedVector<Float64,2>	TB_before(_NaN_),
							TB_after(_NaN_);
		
	for (Int32 intidx=BEFORE;intidx<=AFTER;++intidx)
	{
		Float32	TBV,TBH;
		
		const Int32	tmon = (intidx == BEFORE) ? before_ep.month() : after_ep.month();
		
		FixedVector<Float64,2>&	TB = (intidx == BEFORE) ? TB_before : TB_after;
		
		// compute land TB
		const FixedVector<Float32,3>	NCEP_VPOL_LAND_PAR(data(Index<6>(channel,tmon,LAND,VPOL,viewidx,0))->z(loc),
											  			   data(Index<6>(channel,tmon,LAND,VPOL,viewidx,1))->z(loc),
											  			   data(Index<6>(channel,tmon,LAND,VPOL,viewidx,2))->z(loc)),
										NCEP_HPOL_LAND_PAR(data(Index<6>(channel,tmon,LAND,HPOL,viewidx,0))->z(loc),
													  	   data(Index<6>(channel,tmon,LAND,HPOL,viewidx,1))->z(loc),
													  	   data(Index<6>(channel,tmon,LAND,HPOL,viewidx,2))->z(loc));

		TBV = evaluateTB(NCEP_VPOL_LAND_PAR,dtheta_inc_d);
		TBH = evaluateTB(NCEP_HPOL_LAND_PAR,dtheta_inc_d);
		
		// channels 1 and 3 are H pol, 2 and 4 are V pol
		TB[LAND] = (channel == msu::CH_1 || channel == msu::CH_3) ? TBH*cos2thetan+TBV*sin2thetan : TBV*cos2thetan+TBH*sin2thetan;
								
		// compute ocean TB
		const FixedVector<Float32,3>	NCEP_VPOL_OCEAN_PAR(data(Index<6>(channel,tmon,OCEAN,VPOL,viewidx,0))->z(loc),
											  				data(Index<6>(channel,tmon,OCEAN,VPOL,viewidx,1))->z(loc),
											  				data(Index<6>(channel,tmon,OCEAN,VPOL,viewidx,2))->z(loc)),
										NCEP_HPOL_OCEAN_PAR(data(Index<6>(channel,tmon,OCEAN,HPOL,viewidx,0))->z(loc),
													  		data(Index<6>(channel,tmon,OCEAN,HPOL,viewidx,1))->z(loc),
													  		data(Index<6>(channel,tmon,OCEAN,HPOL,viewidx,2))->z(loc));
											  		
		TBV = evaluateTB(NCEP_VPOL_OCEAN_PAR,dtheta_inc_d);
		TBH = evaluateTB(NCEP_HPOL_OCEAN_PAR,dtheta_inc_d);
		
		// channels 1 and 3 are H pol, 2 and 4 are V pol
		TB[OCEAN] = (channel == msu::CH_1 || channel == msu::CH_3) ? TBH*cos2thetan+TBV*sin2thetan : TBV*cos2thetan+TBH*sin2thetan;
	}
	
	FixedVector<Float32,2>	TB_out;
	
	const Float64	prev = before_ep.julian()-ep.julian(),
					next = after_ep.julian()-ep.julian();
	
	TB_out[LAND] = num::linearInterpolate(0.0,prev,TB_before[LAND],next,TB_after[LAND]);
	TB_out[OCEAN] = num::linearInterpolate(0.0,prev,TB_before[OCEAN],next,TB_after[OCEAN]);
	
	return TB_out;
}

} // namespace msu

} // namespace sat

} // namespace mcs
