#include <iomanip>

#include "MCS_Date.h"
#include "MCS_FileLocations.h"
#include "MCS_FixedVector.h"
#include "MCS_Geography.h"
#include "MCS_Interpolation.h"
#include "MCS_MSUUtilities.h"
#include "MCS_Satellite.h"
#include "MCS_Tensor.h"
#include "MCS_Vector.h"

namespace mcs {

namespace sat {

namespace msu {

// compute NCEP climatology TB for specified time/location/nadir angle
Float32 getMSU_CCM3DiurnalClimatology(Int32 channel,Int32 fov,const Date& ep,const geo::GeographicCoordinate<Float32>& loc) 
{
	static const Int16	NUM_CHANNELS = 4,			// 1..4
						NUM_HOURS = 24,
						NUM_MONTHS = 12,				
						NUM_ANGLES = 6;				// nominal incidence angles 0.0,10.71,21.51,32.51,43.91,56.19
												
	// nominal incidence angles computed as average values of zero-roll-error EIA from 750-900km
	typedef geo::CCM3Grid<Float32>					GridType;
	typedef geo::EarthMap<Float32,Float32,GridType>	MapType;
	
	static const GridType	GRID(IndexRange<2>(Index<2>(64,128)));
	
	static Vector<bool>	ch_read(IndexRange<1>(1,NUM_CHANNELS),false);
	
	static const IndexRange<4>	datarng(Index<4>(1,0,1,0),Index<4>(NUM_CHANNELS,NUM_ANGLES-1,NUM_MONTHS,NUM_HOURS-1));
	static Tensor<MapType*,4>	data(datarng);

	// load new NCEP TB climatology
	if (!ch_read[channel])
	{
		std::cerr << "Reading CCM3 diurnal cycle data for MSU channel " << channel << "..."; 
		
		static const std::string	prefix = FileLocations["CCM3 MSU diurnal cycle"]+"mn_diur_cycles_chan_",
									suffix = "_ccm3_128x64_local_time.mcs";
		
		std::stringstream	sstr;
		
		const Int32	ichan = channel;
		
		sstr << prefix << std::setw(1) << ichan << suffix;
			
		std::ifstream	ifs(sstr.str().c_str(),std::ios::binary | std::ios::in);
				
		Index<4>	idx;
		
		idx[0] = channel;
		
		for (idx[1]=datarng.first(1);idx[1]<=datarng.last(1);++idx[1])
		{
			for (idx[2]=datarng.first(2);idx[2]<=datarng.last(2);++idx[2])
			{
				for (idx[3]=datarng.first(3);idx[3]<=datarng.last(3);++idx[3])
				{
					data(idx) = new MapType(GRID);
						
					data(idx)->read(ifs);	
				}
			}
		}
		
		ifs.close();
		
		ch_read[channel] = true;
		
		std::cerr << "done" << std::endl;
	}
	
	// determine which months are needed for interpolation
	Date	before_ep(ep.year(),ep.month(),1),
			after_ep(before_ep);

	if (ep.dayOfMonth() < ep.daysInMonth()/2)
		before_ep.addMonths(-1);
	else
		after_ep.addMonths(1);
	
	Day	bom,eom;
	
	bom = Day(before_ep.year(),before_ep.month(),1);
	eom = Day(before_ep.year(),before_ep.month(),before_ep.daysInMonth());
				
	before_ep = Date(0.5*(Date(bom,Time(0,0,0)).julian()+Date(eom,Time(23,59,59)).julian()));
	
	bom = Day(after_ep.year(),after_ep.month(),1);
	eom = Day(after_ep.year(),after_ep.month(),after_ep.daysInMonth());			

	after_ep = Date(0.5*(Date(bom,Time(0,0,0)).julian()+Date(eom,Time(23,59,59)).julian()));			

	const Time	local_time = ep.time();
		
	Time	before_time,
			after_time;
			
	if (local_time.minute() <= Time::minutesInHour()/2)
	{
		before_time = Time(std::floor(local_time.hour()),30,0);
		before_time.addHours(-1);
	}
	else
	{
		before_time = Time(std::floor(local_time.hour()),30,0);
	}
	
	after_time = before_time;
	after_time.addHours(1);
	
	// Carl's files are 0 == limb, 5 == nadir
	const Index<1>	fovidx = 5-std::abs(fov-6);
	const Index<4>	LL(channel,fovidx,before_ep.month(),before_time.hour()),
					LR(channel,fovidx,after_ep.month(),before_time.hour()),
					UR(channel,fovidx,after_ep.month(),after_time.hour()),
					UL(channel,fovidx,before_ep.month(),after_time.hour());
	
	const FixedVector<Float32,2>	lo(before_ep.fixed()-ep.fixed(),before_time.ticks()/Time::secondsInDay()),
									hi(after_ep.fixed()-ep.fixed(),after_time.ticks()/Time::secondsInDay());
	const FixedVector<Float32,4>	z(data(LL)->z(loc),
									  data(LR)->z(loc),
									  data(UR)->z(loc),
									  data(UL)->z(loc));
	const Float32					x = 0,
									y = local_time.ticks()/Time::secondsInDay();	
																	
	return num::bilinearInterpolate(z,lo,hi,x,y);
}

} // namespace msu

} // namespace sat

} // namespace mcs
