#ifndef MCS_TENSOR_H
#define MCS_TENSOR_H

#include <cmath>
#include <iostream>
#include <string>

#include "MCS_Exceptions.h"
#include "MCS_IndexRange.h"
#include "MCS_Typedefs.h"
#include "MCS_TypeFlags.h"

#include "MCS_TensorImplementation.h"
#include "MCS_TensorIndirection.h"

//#define USE_PETE_OPERATORS

#ifdef USE_PETE_OPERATORS
#include "PETE/PETE.h"
#endif

namespace mcs {

template<class T,Uint8 R> class TensorDefaults
{
	public:
		typedef TensorImplementation< BasicHeapAllocator<T>,BasicTensorAccessor<R> >	ImplementationType;
		
		typedef BasicHeapAllocator<T>		AllocatorType;
		//typedef StdVectorHeapAllocator<T>	AllocatorType;
		//typedef ValarrayHeapAllocator<T>	AllocatorType;
		
		typedef BasicTensorAccessor<R>		AccessorType;
};

__DECLARE_EXCEPTION_CLASS__(TensorException,Exception)

template<class T,Uint8 R,class Imp = typename TensorDefaults<T,R>::ImplementationType>
class Tensor : 	public Imp
{
	public:
		enum { Rank = R };
		
		typedef Imp										ImplementationType;
		
		typedef typename Imp::AccessorType				AccessorType;
		typedef typename Imp::AllocatorType				AllocatorType;
		
		typedef T										ElementType;
		typedef typename IndexRange<Rank>::IndexType	IndexType;
		typedef IndexRange<Rank>						RangeType;
		
		typedef Tensor<T,R,Imp>							TensorType;
		typedef TensorIndirection<R>					IndirectionType;
		typedef Tensor<Binary,R>						MaskType;
		
		typedef Tensor< T,R,AliasImplementation<TensorType> >				AliasType;
		typedef Tensor< T,R,IndirectionSliceImplementation<TensorType> >	IndirectionSliceType;
		typedef Tensor< T,R,MaskSliceImplementation<TensorType> >			MaskSliceType;
		typedef Tensor< T,R,RangeSliceImplementation<TensorType> >			RangeSliceType;
		
		class iterator;
		class const_iterator;
		class reverse_iterator;
		class const_reverse_iterator;
		
		Tensor();
		explicit Tensor(const RangeType&);
		Tensor(const RangeType&,const ElementType&);
		Tensor(const RangeType&,ElementType*);
		Tensor(const RangeType&,const std::string&);
		Tensor(const ImplementationType&);
		template<class Imp2> explicit Tensor(Tensor<T,R,Imp2>&);
		Tensor(const TensorType&);
		~Tensor() { }
		
		ElementType& operator()(const IndexType& idx);
		const ElementType& operator()(const IndexType& idx) const;
		
		TensorType& operator=(const ElementType&);
		TensorType& operator=(const std::string&);
		TensorType& operator=(const TensorType&);
		
		// slice creation
		MaskSliceType operator()(const MaskType& msk);
		RangeSliceType operator()(const RangeType& rng);
		IndirectionSliceType operator()(const IndirectionType& ind);
		
		// slice assignment
		TensorType& operator=(const MaskSliceType&);
		TensorType& operator=(const RangeSliceType&);
		TensorType& operator=(const IndirectionSliceType&);
		
		#ifdef USE_PETE_OPERATORS
		// PETE copy constructor
		template<class RHS> explicit Tensor(const Expression<RHS>& rhs);
		
		// PETE assignment
		template<class RHS> TensorType& operator=(const Expression<RHS>& rhs);
		#endif
		
		// arithmetic
		TensorType& operator+=(const ElementType&);
		TensorType& operator-=(const ElementType&);
		TensorType& operator*=(const ElementType&);
		TensorType& operator/=(const ElementType&);

		TensorType& operator+=(const TensorType&);
		TensorType& operator-=(const TensorType&);
		TensorType& operator*=(const TensorType&);
		TensorType& operator/=(const TensorType&);
		
		const_iterator begin() const;
		const_iterator end() const;
	 	const_reverse_iterator rbegin() const;
	 	const_reverse_iterator rend() const;
	 	
		iterator begin();
		iterator end();
	 	reverse_iterator rbegin();
	 	reverse_iterator rend();
	 	
		void deserialize(std::istream& is);
	 	void serialize(std::ostream& os) const;
	 	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,TensorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
	 
	private:
		friend class iterator;
		friend class const_iterator;
		friend class reverse_iterator;
		friend class const_reverse_iterator;
		
		void set(const ElementType&);
		void set(const std::string&);
		void set(const TensorType&);
};

// class for locally changing tensor offset
template<class TensorType> class Offset
{
	public:
		typedef typename TensorType::IndexType	IndexType;
		
		Offset(TensorType&,const IndexType&);
		~Offset();
		
		const IndexType& currentOffset() const		{ return tensor_.first(); }
		const IndexType& originalOffset() const		{ return original_offset_; }
		
	private:
		TensorType&	tensor_;
		IndexType	original_offset_;
};

template<class TensorType>
inline Offset<TensorType>::Offset(TensorType& tensor,const IndexType& idx) :
	tensor_(tensor),
	original_offset_(tensor.first())
{
	tensor_.setOffset(idx);
}

template<class TensorType>
inline Offset<TensorType>::~Offset()
{
	tensor_.setOffset(original_offset_);
}

// implementation
#include "MCS_TensorIterators.cpp"

#include "MCS_Tensor.cpp"

#ifndef USE_PETE_OPERATORS
#include "MCS_TensorFunctions.cpp"
#include "MCS_TensorOperators.cpp"
#endif

} // namespace mcs

#ifdef USE_PETE_OPERATORS

// PETE specializations

// evaluate leaves using offsets
template<class T,Uint8 R,class Imp>
struct EvalTensorOffset
{
  typedef Int32 offset;
  
  offset offs_;

  inline EvalTensorOffset(offset offs) : offs_(offs) { }
  inline offset val1() const { return offs_; }
};
  
template<class T,Uint8 R,class Imp>
struct LeafFunctor< mcs::Tensor<T,R,Imp>, EvalTensorOffset<T,R,Imp> >
{
  typedef T Type_t;
  
  static Type_t apply(const mcs::Tensor<T,R,Imp>& a,const EvalTensorOffset<T,R,Imp>& f) { return a[f.val1()]; }
};

// evaluate leaves using iterator
//
//	-> seems a little slower than using offsets
template<class T,Uint8 R,class Imp>
struct EvalTensorIterator
{
  typedef typename mcs::Tensor<T,R,Imp>::iterator	iterator;
  
  iterator& it_;

  inline EvalTensorIterator(iterator& it) : it_(it) { }
  inline iterator& val1() const { return it_; }
};
  
template<class T,Uint8 R,class Imp>
struct LeafFunctor< mcs::Tensor<T,R,Imp>, EvalTensorIterator<T,R,Imp> >
{
  typedef T Type_t;
  
  //static Type_t apply(const mcs::Tensor<T,R,Imp>& a,const EvalTensorIterator<T,R,Imp>& f) { return a(f.val1().index()); }
  static Type_t apply(const mcs::Tensor<T,R,Imp>&,const EvalTensorIterator<T,R,Imp>& f) { return *(f.val1()); }
};

template<class T,Uint8 R,class Imp>
struct CreateLeaf< mcs::Tensor<T,R,Imp> >
{
	typedef Reference< mcs::Tensor<T,R,Imp> > Leaf_t;

	inline static Leaf_t make(const mcs::Tensor<T,R,Imp>& a) { return Leaf_t(a); }
};

// use to check that all tensor ranges in expression are commensurate
template<Uint8 R>
class RangeLeaf
{
	public:
		typedef mcs::IndexRange<R>	RangeType;
		
		RangeLeaf(const RangeType& rng) : rng_(rng) { }
		RangeLeaf(const RangeLeaf<R>& source) : rng_(source.rng_) { }
		~RangeLeaf() { }
		
		bool operator()(const RangeType& rng) const { return rng == rng_; }
		
	private:
		RangeType	rng_;
};

template<class T,Uint8 R,class Imp>
struct LeafFunctor< Scalar<T>, RangeLeaf<R> >
{
	typedef bool Type_t;
  
	static Type_t apply(const mcs::Tensor<T,R,Imp>& a,const RangeLeaf<R>& f) { return true; }
};

template<class T,Uint8 R,class Imp>
struct LeafFunctor< mcs::Tensor<T,R,Imp>, RangeLeaf<R> >
{
	typedef bool Type_t;
  
	static Type_t apply(const mcs::Tensor<T,R,Imp>& a,const RangeLeaf<R>& f) { return f(a.range()); }
};

#include "MCS_TensorOperators_PETE.h"

#endif

#endif // MCS_TENSOR_H