#ifndef MCS_TENSORACCESSORS_H
#define MCS_TENSORACCESSORS_H

// Tensor Accessors keep track of tensor dimension and convert between indices and linear element offset, and must provide the following member functions:
//
//	const RangeType& range() const								-> return range of tensor
//	const IndexType& signature() const							-> return signature describing order in which ranks are traversed
//																	e.g. column-major = (0,1), row-major = (1,0), etc...
//		
//	void transpose(const Uint8& rank1,const Uint8& rank2)		-> transpose two ranks
//		
//	Int32 offsetFromIndex(const IndexType& idx) const			-> return offset given an index
//	IndexType indexFromOffset(const Int32& offs) const			-> return index given an offset

#include "MCS_Exceptions.h"
#include "MCS_IndexRange.h"
#include "MCS_Index.h"
#include "MCS_Typedefs.h"

namespace mcs {

class NoAccessorRangeChecking
{
	public:
		template<Uint8 R> static bool check(const IndexRange<R>&,const typename IndexRange<R>::IndexType&) { return true; }
		template<Uint8 R> static bool compare(const IndexRange<R>&,const IndexRange<R>&) { return true; }
};

class AccessorThrowRangeChecking
{
	public:
		template<Uint8 R> static bool check(const IndexRange<R>& rng,const typename IndexRange<R>::IndexType& idx);
		template<Uint8 R> static bool compare(const IndexRange<R>& rng1,const IndexRange<R>& rng2);
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
};

template<Uint8 R>
inline bool AccessorThrowRangeChecking::check(const IndexRange<R>& rng,const typename IndexRange<R>::IndexType& idx)	
{ 
	if (any(idx<rng.first() || idx>rng.last())) 
	{
		std::stringstream sstr;
		
		sstr << idx;
		
		throw Err(sstr.str()); 
	}
	
	return true;
}

template<Uint8 R>
inline bool AccessorThrowRangeChecking::compare(const IndexRange<R>& rng1,const IndexRange<R>& rng2)	
{ 
	if (rng1 != rng2) 
	{
		std::stringstream sstr;
		
		sstr << rng1 << " != " << rng2;
		
		throw Err(sstr.str()); 
	}
	
	return true;
}

// basic tensor accessor 
template<Uint8 R,class CP = NoAccessorRangeChecking> class BasicTensorAccessor
{
	public:
		enum { Rank = R };
		
		typedef BasicTensorAccessor<R,CP>			this_type;
		
		typedef CP									CheckingPolicyType;
		typedef typename IndexRange<R>::IndexType	IndexType;
		typedef IndexRange<R>						RangeType;
		typedef Signature<R>						SignatureType;
		
		BasicTensorAccessor();
		explicit BasicTensorAccessor(const RangeType& rng);
		BasicTensorAccessor(const BasicTensorAccessor<R,CP>&);
		~BasicTensorAccessor() { }
		
		BasicTensorAccessor<R,CP>& operator=(const BasicTensorAccessor<R,CP>&);
		
		const RangeType& range() const					{ return rng_; }
		const SignatureType& signature() const			{ return sig_; }
	
		Int32 elem() const								{ return rng_.elem(); }

		const IndexType& first() const					{ return rng_.first(); }
		const IndexType& last() const					{ return rng_.last(); }
		const IndexType& dim() const					{ return rng_.dim(); }

		Int32 first(Int32 rnk) const					{ return rng_.first(rnk); }
		Int32 last(Int32 rnk) const						{ return rng_.last(rnk); }
		Int32 dim(Int32 rnk) const						{ return rng_.dim(rnk); }

		void setOffset(const IndexType& idx)			{ rng_.setOffset(idx); original_rng_.setOffset(sig_.inverse_permute(idx)); }
		
		void permute(const SignatureType& sig);
		void inverse_permute(const SignatureType& sig);
		void transpose(Int32 rank1,Int32 rank2);
		
		Int32 offsetFromIndex(const IndexType& idx) const;
		IndexType indexFromOffset(Int32 offs) const;
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		RangeType		rng_,
						original_rng_;
		SignatureType	sig_;
		IndexType		sz_;
};

template<Uint8 R,class CP>
inline BasicTensorAccessor<R,CP>::BasicTensorAccessor() :
	rng_(),
	original_rng_(),
	sig_(),
	sz_()
{ }

template<Uint8 R,class CP>
inline BasicTensorAccessor<R,CP>::BasicTensorAccessor(const RangeType& rng) :
	rng_(rng),
	original_rng_(rng),
	sig_(),
	sz_()
{ 
	sz_[R-1] = original_rng_.dim(R-1);
	
	for (Int32 i=R-2;i>=0;i--)
		sz_[i] = sz_[i+1]*original_rng_.dim(i);	
}

template<Uint8 R,class CP>
inline BasicTensorAccessor<R,CP>::BasicTensorAccessor(const BasicTensorAccessor<R,CP>& source) :
	rng_(source.rng_),
	original_rng_(source.original_rng_),
	sig_(source.sig_),
	sz_(source.sz_)
{ }

template<Uint8 R,class CP>
inline BasicTensorAccessor<R,CP>& BasicTensorAccessor<R,CP>::operator=(const BasicTensorAccessor<R,CP>& source)
{
	if (this == &source) return *this;
	
	rng_ = source.rng_;
	original_rng_ = source.original_rng_;
	sig_ = source.sig_;
	sz_ = source.sz_;
	
	return *this;
}
		
template<Uint8 R,class CP>
inline void BasicTensorAccessor<R,CP>::permute(const SignatureType& sig)
{
	sig_ = sig;
	
	rng_.permute(sig);
}

template<Uint8 R,class CP>
inline void BasicTensorAccessor<R,CP>::inverse_permute(const SignatureType& sig)
{
	sig_ = sig;

	rng_.inverse_permute(sig);
}

template<Uint8 R,class CP>
inline void BasicTensorAccessor<R,CP>::transpose(Int32 rank1,Int32 rank2)
{
	sig_.transpose(rank1,rank2);
	rng_.transpose(rank1,rank2);
}

template<Uint8 R,class CP>
inline Int32 BasicTensorAccessor<R,CP>::offsetFromIndex(const IndexType& idx) const
{
	CP::check(rng_,idx);
	
	Int32	outidx = 0;
	
	for (Int32 i=0;i<R-1;i++)
		outidx += (idx[sig_[i]]-original_rng_.first(i))*sz_[i+1];
		
	return (outidx += (idx[sig_[R-1]]-original_rng_.first(R-1)));
}

template<Uint8 R,class CP>
inline typename BasicTensorAccessor<R,CP>::IndexType BasicTensorAccessor<R,CP>::indexFromOffset(Int32 offs) const
{
	Int32		tmpoffs = offs;
	IndexType	page;
	
	for (Int32 i=0;i<R-1;++i)
	{
		page[i] = tmpoffs/sz_[i+1];
		tmpoffs -= page[i]*sz_[i+1];
	}
		
	page[R-1] = tmpoffs;
	
	IndexType	idx;
	
	for (Int32 j=0;j<R;++j)
		idx[sig_[j]] = page[j]+original_rng_.first(j);
							
	return idx;
}

template<Uint8 R,class CP>
inline void BasicTensorAccessor<R,CP>::deserialize(std::istream& is)
{
	IndexRange<R>	rng;
	Signature<R>	sig;
	
	rng.deserialize(is);
	sig.deserialize(is);
	
	rng.inverse_permute(sig);
	
	*this = BasicTensorAccessor<R,CP>(rng);
	
	permute(sig);
}

template<Uint8 R,class CP>
inline void BasicTensorAccessor<R,CP>::serialize(std::ostream& os) const
{
	rng_.serialize(os);
	sig_.serialize(os);
}

// specializations for small R

// vector : R==1
template<class CP> class BasicTensorAccessor<1,CP>
{
	public:
		enum { Rank = 1 };
		
		typedef BasicTensorAccessor<1,CP>	this_type;
		
		typedef CP							CheckingPolicyType;
		typedef Index<1>					IndexType;
		typedef IndexRange<1>				RangeType;
		typedef Signature<1>				SignatureType;
		
		BasicTensorAccessor();
		BasicTensorAccessor(const RangeType& rng);
		BasicTensorAccessor(const BasicTensorAccessor<1,CP>&);
		~BasicTensorAccessor() { }
		
		BasicTensorAccessor<1,CP>& operator=(const BasicTensorAccessor<1,CP>&);
		
		const RangeType& range() const					{ return rng_; }
		const SignatureType& signature() const			{ return sig_; }
	
		Int32 elem() const								{ return rng_.elem(); }

		const IndexType& first() const					{ return rng_.first(); }
		const IndexType& last() const					{ return rng_.last(); }
		const IndexType& dim() const					{ return rng_.dim(); }

		Int32 first(Int32 rnk) const					{ return rng_.first(rnk); }
		Int32 last(Int32 rnk) const						{ return rng_.last(rnk); }
		Int32 dim(Int32 rnk) const						{ return rng_.dim(rnk); }

		void setOffset(const IndexType& idx)			{ rng_.setOffset(idx); }
		
		void permute(const SignatureType&) { }
		void inverse_permute(const SignatureType&) { }		
		void transpose(Int32,Int32) { }
		
		Int32 offsetFromIndex(const IndexType& idx) const;
		IndexType indexFromOffset(Int32 offs) const;
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	private:
		RangeType		rng_;
		SignatureType	sig_;
};

template<class CP>
inline BasicTensorAccessor<1,CP>::BasicTensorAccessor() :
	rng_(),
	sig_()
{ }

template<class CP>
inline BasicTensorAccessor<1,CP>::BasicTensorAccessor(const IndexRange<1>& rng) :
	rng_(rng),
	sig_()
{ }

template<class CP>
inline BasicTensorAccessor<1,CP>::BasicTensorAccessor(const BasicTensorAccessor<1,CP>& source) :
	rng_(source.rng_),
	sig(source.sig_)
{ }

template<class CP>
inline BasicTensorAccessor<1,CP>& BasicTensorAccessor<1,CP>::operator=(const BasicTensorAccessor<1,CP>& source)
{
	if (this == &source) return *this;
	
	rng_ = source.rng_;
	sig_ = source.sig_;
	
	return *this;
}
		
template<class CP>
inline Int32 BasicTensorAccessor<1,CP>::offsetFromIndex(const IndexType& idx) const
{
	CP::check(rng_,idx);
	
	return idx-first();
}

template<class CP>
inline typename BasicTensorAccessor<1,CP>::IndexType BasicTensorAccessor<1,CP>::indexFromOffset(Int32 offs) const
{
	return offs+first();
}

template<class CP>
inline void BasicTensorAccessor<1,CP>::deserialize(std::istream& is)
{
	rng_.deserialize(is);
}

template<class CP>
inline void BasicTensorAccessor<1,CP>::serialize(std::ostream& os) const
{
	rng_.serialize(os);
}

// matrix : R==2
template<class CP> class BasicTensorAccessor<2,CP>
{
	public:
		enum { Rank = 2 };
		
		typedef BasicTensorAccessor<2,CP>	this_type;
		
		typedef CP							CheckingPolicyType;
		typedef Index<2>					IndexType;
		typedef IndexRange<2>				RangeType;
		typedef Signature<2>				SignatureType;
		
		BasicTensorAccessor();
		explicit BasicTensorAccessor(const RangeType& rng);
		BasicTensorAccessor(const BasicTensorAccessor<2,CP>&);
		
		BasicTensorAccessor<2,CP>& operator=(const BasicTensorAccessor<2,CP>&);
		
		const RangeType& range() const					{ return rng_; }
		const SignatureType& signature() const			{ return sig_; }
	
		Int32 elem() const								{ return rng_.elem(); }

		const IndexType& first() const					{ return rng_.first(); }
		const IndexType& last() const					{ return rng_.last(); }
		const IndexType& dim() const					{ return rng_.dim(); }

		Int32 first(Int32 rnk) const					{ return rng_.first(rnk); }
		Int32 last(Int32 rnk) const						{ return rng_.last(rnk); }
		Int32 dim(Int32 rnk) const						{ return rng_.dim(rnk); }

		void setOffset(const IndexType& idx)			{ rng_.setOffset(idx); original_rng_.setOffset(sig_.inverse_permute(idx)); }
				
		void permute(const SignatureType& sig);
		void inverse_permute(const SignatureType& sig);
		void transpose(Int32 rank1,Int32 rank2);
		
		Int32 offsetFromIndex(const IndexType& idx) const;
		IndexType indexFromOffset(Int32 offs) const;
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~BasicTensorAccessor() { }
		
	private:
		RangeType		rng_,
						original_rng_;
		SignatureType	sig_;
};

template<class CP>
inline BasicTensorAccessor<2,CP>::BasicTensorAccessor() :
	rng_(),
	original_rng_(),
	sig_()
{ }

template<class CP>
inline BasicTensorAccessor<2,CP>::BasicTensorAccessor(const RangeType& rng) :
	rng_(rng),
	original_rng_(rng),
	sig_()
{ }

template<class CP>
inline BasicTensorAccessor<2,CP>::BasicTensorAccessor(const BasicTensorAccessor<2,CP>& source) :
	rng_(source.rng_),
	original_rng_(source.original_rng_),
	sig_(source.sig_)
{ }

template<class CP>
inline BasicTensorAccessor<2,CP>& BasicTensorAccessor<2,CP>::operator=(const BasicTensorAccessor<2,CP>& source)
{
	if (this == &source) return *this;
	
	rng_ = source.rng_;
	original_rng_ = source.original_rng_;
	sig_ = source.sig_;
	
	return *this;
}
		
template<class CP>
inline void BasicTensorAccessor<2,CP>::permute(const SignatureType& sig)
{
	sig_ = sig;
	
	rng_.permute(sig);
}

template<class CP>
inline void BasicTensorAccessor<2,CP>::inverse_permute(const SignatureType& sig)
{
	sig_ = sig;
	
	rng_.inverse_permute(sig);
}

template<class CP>
inline void BasicTensorAccessor<2,CP>::transpose(Int32 rank1,Int32 rank2)
{
	sig_.transpose(rank1,rank2);
	rng_.transpose(rank1,rank2);
}

template<class CP>
inline Int32 BasicTensorAccessor<2,CP>::offsetFromIndex(const IndexType& idx) const
{
	CP::check(rng_,idx);
	
	return (idx[sig_[0]]-original_rng_.first(0))*original_rng_.dim(1) + 
		   (idx[sig_[1]]-original_rng_.first(1));
}

template<class CP>
inline typename BasicTensorAccessor<2,CP>::IndexType BasicTensorAccessor<2,CP>::indexFromOffset(Int32 offs) const
{
	IndexType	page;
	
	page[0] = offs/original_rng_.dim(1);
	page[1] = offs - page[0]*original_rng_.dim(1);
	
	IndexType	idx;
	
	idx[sig_[0]] = page[0]+original_rng_.first(0);
	idx[sig_[1]] = page[1]+original_rng_.first(1);
							
	return idx;
}

template<class CP>
inline void BasicTensorAccessor<2,CP>::deserialize(std::istream& is)
{
	IndexRange<2>	rng;
	Signature<2>	sig;
	
	rng.deserialize(is);
	sig.deserialize(is);
	
	rng.inverse_permute(sig);
	
	*this = BasicTensorAccessor<2,CP>(rng);
	
	permute(sig);
}

template<class CP>
inline void BasicTensorAccessor<2,CP>::serialize(std::ostream& os) const
{
	rng_.serialize(os);
	sig_.serialize(os);
}

// tensor alias accessor
template<class Accessor> class AliasAccessor
{
	public:
		enum { Rank = Accessor::Rank };
		
		typedef typename Accessor::CheckingPolicyType	CheckingPolicyType;
		typedef typename Accessor::IndexType			IndexType;
		typedef typename Accessor::RangeType			RangeType;
		typedef typename Accessor::SignatureType		SignatureType;
		
		AliasAccessor(Accessor& source) : ref_(source) { }
		~AliasAccessor() { }
		
		const RangeType& range() const							{ return ref_.range(); }
		const SignatureType& signature() const					{ return ref_.signature(); }
		
		Int32 elem() const										{ return ref_.elem(); }

		const IndexType& first() const							{ return ref_.first(); }
		const IndexType& last() const							{ return ref_.last(); }
		const IndexType& dim() const							{ return ref_.dim(); }

		Int32 first(Int32 rnk) const							{ return ref_.first(rnk); }
		Int32 last(Int32 rnk) const								{ return ref_.last(rnk); }
		Int32 dim(Int32 rnk) const								{ return ref_.dim(rnk); }

		void setOffset(const IndexType& idx)					{ ref_.setOffset(idx); }
		
		void permute(const SignatureType& sig)					{ ref_.permute(sig); }
		void inverse_permute(const SignatureType& sig)			{ ref_.inverse_permute(sig); }
		void transpose(Int32 rank1,Int32 rank2)					{ ref_.transpose(rank1,rank2); }
		
		Int32 offsetFromIndex(const IndexType& idx) const		{ return ref_.offsetFromIndex(idx); }
		IndexType indexFromOffset(Int32 offs) const				{ return ref_.indexFromOffset(offs); }
		
		void deserialize(std::istream& is)						{ return ref_.deserialize(is); }
		void serialize(std::ostream& os) const					{ return ref_.serialize(os); }

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		AliasAccessor();
		AliasAccessor<Accessor>& operator=(const AliasAccessor<Accessor>&);
		
		Accessor&	ref_;
};

template<class TensorType>
class MaskSliceAccessor
{
	public:
		enum { Rank = TensorType::Rank };
		
		typedef typename TensorType::CheckingPolicyType		CheckingPolicyType;
		typedef typename TensorType::IndexType				IndexType;
		typedef typename TensorType::RangeType				RangeType;
		typedef typename TensorType::SignatureType			SignatureType;
		
		typedef typename TensorType::MaskType		MaskType;
		
		MaskSliceAccessor(TensorType& source,const MaskType& msk) : ref_(source),msk_(msk) { }
		~MaskSliceAccessor() { }
		
		const RangeType& range() const							{ return ref_.range(); }
		const SignatureType& signature() const					{ return ref_.signature(); }
		
		Int32 elem() const										{ return ref_.elem(); }

		const IndexType& first() const							{ return ref_.first(); }
		const IndexType& last() const							{ return ref_.last(); }
		const IndexType& dim() const							{ return ref_.dim(); }

		Int32 first(Int32 rnk) const							{ return ref_.first(rnk); }
		Int32 last(Int32 rnk) const								{ return ref_.last(rnk); }
		Int32 dim(Int32 rnk) const								{ return ref_.dim(rnk); }

		void setOffset(const IndexType& idx)					{ ref_.setOffset(idx); }
		
		Int32 offsetFromIndex(const IndexType& idx) const		{ return ref_.offsetFromIndex(idx); }
		IndexType indexFromOffset(Int32 offs) const				{ return ref_.indexFromOffset(offs); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		MaskSliceAccessor();
		MaskSliceAccessor<TensorType>& operator=(const MaskSliceAccessor<TensorType>&);
		
		TensorType&		ref_;
		const MaskType&	msk_;
};

template<class TensorType>
class RangeSliceAccessor
{
	public:
		enum { Rank = TensorType::Rank };
		
		typedef typename TensorType::CheckingPolicyType		CheckingPolicyType;
		typedef typename TensorType::IndexType				IndexType;
		typedef typename TensorType::RangeType				RangeType;
		typedef typename TensorType::SignatureType			SignatureType;
		
		RangeSliceAccessor(TensorType& source,const RangeType& rng) : ref_(source),rng_(rng) { }
		~RangeSliceAccessor() { }
		
		const RangeType& range() const							{ return ref_.range(); }
		const SignatureType& signature() const					{ return ref_.signature(); }
		
		Int32 elem() const										{ return ref_.elem(); }

		const IndexType& first() const							{ return ref_.first(); }
		const IndexType& last() const							{ return ref_.last(); }
		const IndexType& dim() const							{ return ref_.dim(); }

		Int32 first(Int32 rnk) const							{ return ref_.first(rnk); }
		Int32 last(Int32 rnk) const								{ return ref_.last(rnk); }
		Int32 dim(Int32 rnk) const								{ return ref_.dim(rnk); }

		void setOffset(const IndexType& idx)					{ ref_.setOffset(idx); }
		
		Int32 offsetFromIndex(const IndexType& idx) const;
		IndexType indexFromOffset(Int32 offs) const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		RangeSliceAccessor();
		RangeSliceAccessor<TensorType>& operator=(const RangeSliceAccessor<TensorType>&);
		
		TensorType&			ref_;
		const RangeType&	rng_;
};

template<class TensorType>
class IndirectionSliceAccessor
{
	public:
		enum { Rank = TensorType::Rank };
		
		typedef typename TensorType::CheckingPolicyType		CheckingPolicyType;
		typedef typename TensorType::IndexType				IndexType;
		typedef typename TensorType::RangeType				RangeType;
		typedef typename TensorType::SignatureType			SignatureType;

		typedef typename TensorType::IndirectionType	IndirectionType;
		
		IndirectionSliceAccessor(TensorType& source,const IndirectionType& ind) : ref_(source),ind_(ind) { }
		~IndirectionSliceAccessor() { }
		
		const RangeType& range() const							{ return ref_.range(); }
		const SignatureType& signature() const					{ return ref_.signature(); }
		
		Int32 elem() const										{ return ref_.elem(); }

		const IndexType& first() const							{ return ref_.first(); }
		const IndexType& last() const							{ return ref_.last(); }
		const IndexType& dim() const							{ return ref_.dim(); }

		Int32 first(Int32 rnk) const							{ return ref_.first(rnk); }
		Int32 last(Int32 rnk) const								{ return ref_.last(rnk); }
		Int32 dim(Int32 rnk) const								{ return ref_.dim(rnk); }

		void setOffset(const IndexType& idx)					{ ref_.setOffset(idx); }
		
		Int32 offsetFromIndex(const IndexType& idx) const;
		IndexType indexFromOffset(Int32 offs) const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		IndirectionSliceAccessor();
		IndirectionSliceAccessor<TensorType>& operator=(const IndirectionSliceAccessor<TensorType>&);
		
		TensorType&				ref_;
		const IndirectionType&	ind_;
};

template<class TensorType>
inline Int32 IndirectionSliceAccessor<TensorType>::offsetFromIndex(const IndexType& idx) const
{
	const typename IndirectionType::const_iterator	it=std::find(ind_.begin(),ind_.end(),idx);
	
	return it-ind_.begin();
}

template<class TensorType>
inline typename IndirectionSliceAccessor<TensorType>::IndexType IndirectionSliceAccessor<TensorType>::indexFromOffset(Int32 offs) const
{
	return ind_[offs];
}

} // namespace mcs

#endif // MCS_TENSORACCESSORS_H