#ifndef MCS_TENSORALLOCATORS_H
#define MCS_TENSORALLOCATORS_H

// Tensor Allocators provide access to a linearly ordered sequence of elements, and must provide the following member functions:
//
//	void set(const ElementType&)						-> set all elements to a specified value
//	void set(const Allocator&)							-> copy all elements
//
//	Int32 size() const									-> return total number of elements
//
//	ElementType& elementFromOffset(const Int32&)					-> return specified element as offset (0..size-1)
//	const ElementType& elementFromOffset(const Int32&) const		-> return specified element as offset (0..size-1)

#include <cstring>
#include <map>
#include <valarray>
#include <vector>

#include "MCS_TensorIndirection.h"
#include "MCS_Typedefs.h"

namespace mcs {

class NoAllocatorRangeChecking
{
	public:
		static bool check(const Int32,const Int32) { return true; }
};

class AllocatorThrowRangeChecking
{
	public:
		static bool check(const Int32 sz,const Int32 iter) { if (iter < 0 || iter >= sz) throw Err(iter); return true; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
};

// store elements in a contiguous block on the heap
template<class T,class CP = NoAllocatorRangeChecking> class BasicHeapAllocator
{
	public:
		typedef T		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		typedef CP		CheckingPolicyType;
		
		BasicHeapAllocator() : data_(NULL),size_(0) { }
		explicit BasicHeapAllocator(const Int32& sz) : data_(NULL),size_(0) { alloc(sz); set(T()); }
		BasicHeapAllocator(const Int32& sz,const ElementType& val) : data_(NULL),size_(0) { alloc(sz); set(val); }
		BasicHeapAllocator(const BasicHeapAllocator<T,CP>& source) : data_(NULL),size_(0) { set(source); }
		~BasicHeapAllocator() { dealloc(); }
		
		void set(const ElementType&);
		void set(const BasicHeapAllocator<T,CP>&);
		
		Int32 size() const														{ return size_; }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return iter; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return offs; }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ CP::check(size(),iter); return data_[iter]; }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ CP::check(size(),iter); return data_[iter]; }

		ElementType& elementFromOffset(const OffsetType& offs)					{ return elementFromIterator(offsetToIterator(offs)); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return elementFromIterator(offsetToIterator(offs)); }
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	protected:
		BasicHeapAllocator<T,CP>& operator=(const BasicHeapAllocator<T,CP>&);
		
	private:
		void alloc(const Int32& sz);
		void dealloc();
		
		ElementType*	data_;
		Int32			size_;
};

template<class T,class CP>
inline BasicHeapAllocator<T,CP>& BasicHeapAllocator<T,CP>::operator=(const BasicHeapAllocator<T,CP>& source)
{
	if (this == &source) return *this;
	
	set(source);
	
	return *this;
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::alloc(const Int32& sz)
{
	if (size_ != sz) 
	{
		dealloc();
	
		size_ = sz;
		data_ = new ElementType[sz];
	}
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::dealloc()
{
	if (data_ != NULL) delete[] data_;
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::set(const ElementType& val) 
{
	for (Int32 i=0;i<size();++i) elementFromOffset(i) = val;
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::set(const BasicHeapAllocator<T,CP>& source) 
{
	if (size() != source.size())
	{
		dealloc();
		alloc(source.size());
	}

	#ifdef __MWERKS__
	std::memcpy(reinterpret_cast<char*>(data_),reinterpret_cast<const char*>(source.data_),size_*sizeof(ElementType));
	#else
	memcpy(reinterpret_cast<char*>(data_),reinterpret_cast<const char*>(source.data_),size_*sizeof(ElementType));
	#endif
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::deserialize(std::istream& is)
{
	Int32	sz;
	
	is.read(reinterpret_cast<char*>(&sz),sizeof(sz));
	
	alloc(sz);
	
	for (Int32 i=0;i<size();++i)
		is.read(reinterpret_cast<char*>(&elementFromIterator(i)),sizeof(ElementType));
}

template<class T,class CP>
inline void BasicHeapAllocator<T,CP>::serialize(std::ostream& os) const
{
	os.write(reinterpret_cast<const char*>(&size_),sizeof(size_));
	
	for (Int32 i=0;i<size();++i)
		os.write(reinterpret_cast<const char*>(&elementFromIterator(i)),sizeof(ElementType));
}
/*
// specialize for pointer types, with allocator taking ownership
//
//	N.B. BUGGY
template<class T,class CP> class BasicHeapAllocator<T*,CP>
{
	public:
		typedef T		PointeeType;
		typedef T*		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		
		BasicHeapAllocator() : data_(NULL),size_(0) { }
		explicit BasicHeapAllocator(const Int32& sz) : data_(NULL),size_(0) { alloc(sz); }
		
		Int32 size() const														{ return size_; }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return iter; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return offs; }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ CP::check(size(),iter); return data_[iter]; }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ CP::check(size(),iter); return data_[iter]; }

		ElementType& elementFromOffset(const OffsetType& offs)					{ return elementFromIterator(offsetToIterator(offs)); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return elementFromIterator(offsetToIterator(offs)); }
		
		void deserialize(std::istream&);
		void serialize(std::ostream&) const;

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~BasicHeapAllocator() { dealloc(); }
		
	private:
		BasicHeapAllocator(const BasicHeapAllocator<T*,CP>&);
		BasicHeapAllocator<T*,CP>& operator=(const BasicHeapAllocator<T*,CP>&);
		
		void alloc(const Int32& sz);
		void dealloc();
		
		ElementType*	data_;
		Int32			size_;
};

template<class T,class CP>
inline void BasicHeapAllocator<T*,CP>::alloc(const Int32& sz)
{
	if (size_ != sz) 
	{
		dealloc();
	
		size_ = sz;
		data_ = new ElementType[sz];
		
		for (Int32 i=0;i<size();++i) data_[i] = NULL;
	}
}

template<class T,class CP>
inline void BasicHeapAllocator<T*,CP>::dealloc()
{
	if (data_ != NULL) 
	{
		// delete all allocated elements first
		for (Int32 i=0;i<size();++i) if (data_[i] != NULL) delete data_[i];
				
		delete[] data_;
	}
}

template<class T,class CP>
inline void BasicHeapAllocator<T*,CP>::deserialize(std::istream& is)
{
	Int32	sz;
	
	is.read(reinterpret_cast<char*>(&sz),sizeof(sz));
	
	alloc(sz);
	
	for (Int32 i=0;i<size();++i)
	{
		elementFromIterator(i) = new PointeeType();
		
		is.read(reinterpret_cast<char*>(elementFromIterator(i)),sizeof(PointeeType));
	}
}

template<class T,class CP>
inline void BasicHeapAllocator<T*,CP>::serialize(std::ostream& os) const
{
	os.write(reinterpret_cast<const char*>(&size_),sizeof(size_));
	
	for (Int32 i=0;i<size();++i)
	{
		if (elementFromIterator(i) == NULL)
			elementFromIterator(i) = new PointeeType();
			
		os.write(reinterpret_cast<const char*>(elementFromIterator(i)),sizeof(PointeeType));
	}
}
*/
// store elements in std::vector
template<class T,class CP = NoAllocatorRangeChecking> class StdVectorHeapAllocator
{
	public:
		typedef T		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		typedef CP		CheckingPolicyType;
		
		StdVectorHeapAllocator() : data_() { }
		explicit StdVectorHeapAllocator(const Int32& sz) { alloc(sz); set(T()); }
		StdVectorHeapAllocator(const Int32& sz,const ElementType& val) { alloc(sz); set(val); }
		StdVectorHeapAllocator(const StdVectorHeapAllocator<T,CP>& source) { set(source); }
		
		void set(const ElementType&);
		void set(const StdVectorHeapAllocator<T,CP>&);
		
		Int32 size() const														{ return data_.size(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return iter; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return offs; }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ CP::check(size(),iter); return data_[iter]; }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ CP::check(size(),iter); last_val_ = data_[iter]; return last_val_; }

		ElementType& elementFromOffset(const OffsetType& offs)					{ return elementFromIterator(offsetToIterator(offs)); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return elementFromIterator(offsetToIterator(offs)); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	protected:
		~StdVectorHeapAllocator() { dealloc(); }

	private:
		StdVectorHeapAllocator<T,CP>& operator=(const StdVectorHeapAllocator<T,CP>&);
		
		void alloc(const Int32& sz);
		void dealloc();
		
		std::vector<ElementType>	data_;
		mutable ElementType			last_val_;
};

template<class T,class CP>
inline void StdVectorHeapAllocator<T,CP>::alloc(const Int32& sz)
{
	data_ = std::vector<ElementType>(sz);
}

template<class T,class CP>
inline void StdVectorHeapAllocator<T,CP>::dealloc()
{
	data_.clear();
}

template<class T,class CP>
inline void StdVectorHeapAllocator<T,CP>::set(const ElementType& val) 
{
	for (typename std::vector<ElementType>::iterator it=data_.begin();it!=data_.end();++it) *it = val;
}

template<class T,class CP>
inline void StdVectorHeapAllocator<T,CP>::set(const StdVectorHeapAllocator<T,CP>& source) 
{
	data_ = source.data_;
}

// specialize for bool
template<class CP> class StdVectorHeapAllocator<bool,CP>;

// store elements in std::valarray
template<class T,class CP = NoAllocatorRangeChecking> class ValarrayHeapAllocator
{
	public:
		typedef T		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		typedef CP		CheckingPolicyType;
		
		ValarrayHeapAllocator() : data_() { }
		explicit ValarrayHeapAllocator(const Int32& sz) { alloc(sz); set(T()); }
		ValarrayHeapAllocator(const Int32& sz,const ElementType& val) { alloc(sz); set(val); }
		ValarrayHeapAllocator(const ValarrayHeapAllocator<T,CP>& source) { set(source); }
		
		void set(const ElementType&);
		void set(const ValarrayHeapAllocator<T,CP>&);
		
		Int32 size() const														{ return data_.size(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return iter; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return offs; }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ CP::check(size(),iter); return data_[iter]; }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ CP::check(size(),iter); last_val_ = data_[iter]; return last_val_; }

		ElementType& elementFromOffset(const OffsetType& offs)					{ return elementFromIterator(offsetToIterator(offs)); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return elementFromIterator(offsetToIterator(offs)); }
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	protected:	
		~ValarrayHeapAllocator() { dealloc(); }

	private:
		ValarrayHeapAllocator<T,CP>& operator=(const ValarrayHeapAllocator<T,CP>&);
		
		void alloc(const Int32& sz);
		void dealloc();
		
		std::valarray<ElementType>	data_;
		mutable ElementType			last_val_;
};

template<class T,class CP>
inline void ValarrayHeapAllocator<T,CP>::alloc(const Int32& sz)
{
	data_ = std::valarray<ElementType>(sz);
}

template<class T,class CP>
inline void ValarrayHeapAllocator<T,CP>::dealloc()
{
	data_ = std::valarray<ElementType>();
}

template<class T,class CP>
inline void ValarrayHeapAllocator<T,CP>::set(const ElementType& val) 
{
	data_ = val;
}

template<class T,class CP>
inline void ValarrayHeapAllocator<T,CP>::set(const ValarrayHeapAllocator<T,CP>& source) 
{
	data_ = source.data_;
}

// allocate individual elements as needed, storing in an associative array
template<class T,class CP = NoAllocatorRangeChecking> class SparseAllocator
{
	public:
		typedef T		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		typedef CP		CheckingPolicyType;
		
		SparseAllocator() : data_() { }
		explicit SparseAllocator(const Int32& sz) : data_() { alloc(sz); set(T()); }
		SparseAllocator(const Int32& sz,const ElementType& val) : data_() { alloc(sz); set(val); }
		SparseAllocator(const SparseAllocator<T,CP>& source) { set(source); }
		
		void set(const ElementType&);
		void set(const SparseAllocator<T,CP>&);
		
		Int32 size() const														{ return data_.size(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return otoi_[iter]; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return otoi_.find(offs)-otoi_.begin(); }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ return elementFromOffset(iteratorToOffset(iter)); }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ return elementFromOffset(iteratorToOffset(iter)); }

		ElementType& elementFromOffset(const OffsetType& offs);
		const ElementType& elementFromOffset(const OffsetType& offs) const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~SparseAllocator() { dealloc(); }
	
	private:
		SparseAllocator<T,CP>& operator=(const SparseAllocator<T,CP>&);
		
		void alloc(const Int32& sz);
		void dealloc();
		
		std::vector<Int32>			otoi_;
		std::map<Int32,ElementType>	data_;
};

template<class T,class CP>
inline void SparseAllocator<T,CP>::alloc(const Int32& sz)
{
	if (size() != sz) dealloc();
}

template<class T,class CP>
inline void SparseAllocator<T,CP>::dealloc()
{
	otoi_.clear();
	data_.clear();
}

template<class T,class CP>
inline void SparseAllocator<T,CP>::set(const ElementType& val) 
{
	for (typename std::map<Int32,ElementType>::iterator it = data_.begin();it!=data_.end();++it) 
		it->second = val;
}

template<class T,class CP>
inline void SparseAllocator<T,CP>::set(const SparseAllocator<T,CP>& source) 
{
	otoi_ = source.data_;
	data_ = source.data_;
}

template<class T,class CP>
inline typename SparseAllocator<T,CP>::ElementType& SparseAllocator<T,CP>::elementFromOffset(const Int32& offs)					
{ 
	const typename std::map<Int32,ElementType>::const_iterator	it = data_.find(offs);
	
	// don't perform range check since sparse allocator creates needed element
	//CP::check(size(),it-data_.begin());
	
	if (it == data_.end()) 
	{
		otoi_.push_back(offs);
		data_[offs] = ElementType();
	}
	
	return data_[offs];
}

template<class T,class CP>
inline const typename SparseAllocator<T,CP>::ElementType& SparseAllocator<T,CP>::elementFromOffset(const Int32& offs) const			
{ 
	const typename std::map<Int32,ElementType>::const_iterator	it = data_.find(offs);
	
	CP::check(size(),it-data_.begin());
	
	return it->second; 
}

// alias ownership of memory at specified pointer
template<class T,class CP = NoAllocatorRangeChecking> class PtrAliasAllocator
{
	public:
		typedef T		ElementType;
		typedef Int32	IteratorType;
		typedef Int32	OffsetType;
		typedef CP		CheckingPolicyType;

		PtrAliasAllocator() : size_(0),data_(NULL) { }
		PtrAliasAllocator(const Int32& sz,ElementType* ptr);
		PtrAliasAllocator(const PtrAliasAllocator<T,CP>& source) { set(source); }
		
		void set(const ElementType&);
		void set(const PtrAliasAllocator<T,CP>&);
		
		Int32 size() const														{ return size_; }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return iter; }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return offs; }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ CP::check(size(),iter); return data_[iter]; }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ CP::check(size(),iter); return data_[iter]; }

		ElementType& elementFromOffset(const OffsetType& offs)					{ return elementFromIterator(offsetToIterator(offs)); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return elementFromIterator(offsetToIterator(offs)); }

		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	protected:	
		~PtrAliasAllocator() { dealloc(); }

	private:
		PtrAliasAllocator<T,CP>& operator=(const PtrAliasAllocator<T,CP>&);
		
		void alloc(const Int32& sz);
		void dealloc();
		
		Int32				size_;
		ElementType*	data_;
};

template<class T,class CP>
inline PtrAliasAllocator<T,CP>::PtrAliasAllocator(const Int32& sz,ElementType* ptr) :
	size_(sz),data_(ptr)
{ }

template<class T,class CP>
inline void PtrAliasAllocator<T,CP>::alloc(const Int32& sz)
{
	if (size_ != sz) size_ = sz;
}

template<class T,class CP>
inline void PtrAliasAllocator<T,CP>::dealloc()
{ 
	data_ = NULL;
	size_ = 0;
}

template<class T,class CP>
inline void PtrAliasAllocator<T,CP>::set(const ElementType& val) 
{
	for (Int32 i=0;i<size();++i) elementFromOffset(i) = val;
}

template<class T,class CP>
inline void PtrAliasAllocator<T,CP>::set(const PtrAliasAllocator<T,CP>& source) 
{
	size_ = source.size_;
	data_ = source.data_;
}

// alias to a different allocator
template<class Allocator> class AliasAllocator
{
	public:
		typedef typename Allocator::ElementType			ElementType;
		typedef typename Allocator::IteratorType		IteratorType;
		typedef typename Allocator::OffsetType			OffsetType;
		typedef typename Allocator::CheckingPolicyType	CheckingPolicyType;

		AliasAllocator(Allocator& source) : ref_(source) { }
		AliasAllocator(const AliasAllocator<Allocator>& source) : ref_(source.ref_) { }
		
		void set(const ElementType& val)										{ ref_.set(val); }
		void set(const AliasAllocator<Allocator>& source) 						{ ref_.set(source.ref_); }
		
		Int32 size() const														{ return ref_.size(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return ref_.iteratorToOffset(iter); }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return ref_.offsetToIterator(offs); }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ return ref_.elementFromIterator(iter); }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ return ref_.elementFromIterator(iter); }
		
		ElementType& elementFromOffset(const OffsetType& offs)					{ return ref_.elementFromOffset(offs); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return ref_.elementFromOffset(offs); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~AliasAllocator() { }
	
	private:
		AliasAllocator();
		AliasAllocator<Allocator>& operator=(const AliasAllocator<Allocator>&);
		
		Allocator&	ref_;
};

// alias to a tensor mask slice
template<class TensorType> class MaskSliceAllocator
{
	public:	
		typedef typename TensorType::AllocatorType::ElementType			ElementType;
		typedef typename TensorType::AllocatorType::IteratorType		IteratorType;
		typedef typename TensorType::AllocatorType::OffsetType			OffsetType;
		typedef typename TensorType::AllocatorType::CheckingPolicyType	CheckingPolicyType;

		enum { Rank = TensorType::Rank };

		typedef typename TensorType::MaskType		MaskType;
		
		MaskSliceAllocator(TensorType& source,const MaskType& msk) : ref_(source),msk_(msk) { }
		
		void set(const ElementType&);
		void set(const MaskSliceAllocator<TensorType>&);
		
		Int32 size() const														{ return msk_.elem(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return ref_.iteratorToOffset(iter); }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return ref_.offsetToIterator(offs); }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ return ref_.elementFromIterator(iter); }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ return ref_.elementFromIterator(iter); }
		
		ElementType& elementFromOffset(const OffsetType& offs)					{ return ref_.elementFromOffset(offs); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return ref_.elementFromOffset(offs); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~MaskSliceAllocator() { }
	
	private:
		MaskSliceAllocator();
		MaskSliceAllocator<TensorType>& operator=(const MaskSliceAllocator<TensorType>&);
		
		TensorType&		ref_;
		const MaskType&	msk_;
};

template<class TensorType>
inline void MaskSliceAllocator<TensorType>::set(const ElementType& val)
{
	typename MaskType::const_iterator	mit=msk_.begin();
	typename TensorType::iterator		rit=ref_.begin();
	
	for (;mit!=msk_.end();++mit,++rit) 
		if (*mit == true)
			*rit = val;
}

// alias to a tensor range slice
template<class TensorType> class RangeSliceAllocator
{
	public:
		typedef typename TensorType::AllocatorType::ElementType			ElementType;
		typedef typename TensorType::AllocatorType::IteratorType		IteratorType;
		typedef typename TensorType::AllocatorType::OffsetType			OffsetType;
		typedef typename TensorType::AllocatorType::CheckingPolicyType	CheckingPolicyType;

		enum { Rank = TensorType::Rank };

		typedef typename TensorType::IndexType		IndexType;
		typedef typename TensorType::RangeType		RangeType;
		
		RangeSliceAllocator(TensorType& source,const RangeType& rng) : ref_(source),rng_(rng) { }
		
		void set(const ElementType&);
		void set(const RangeSliceAllocator<TensorType>&);
		
		Int32 size() const														{ return rng_.elem(); }
		
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return ref_.iteratorToOffset(iter); }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return ref_.offsetToIterator(offs); }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ return ref_.elementFromIterator(iter); }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ return ref_.elementFromIterator(iter); }
		
		ElementType& elementFromOffset(const OffsetType& offs)					{ return ref_.elementFromOffset(offs); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return ref_.elementFromOffset(offs); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~RangeSliceAllocator() { }
	
	private:
		RangeSliceAllocator();
		RangeSliceAllocator<TensorType>& operator=(const RangeSliceAllocator<TensorType>&);
		
		IndexType transformElement(const Int32& el) const;
		
		TensorType&			ref_;
		const RangeType&	rng_;
};

// transform from original range to new range
/*
template<class TensorType>
inline typename RangeSliceAllocator<TensorType>::IndexType RangeSliceAllocator<TensorType>::transformElement(const Int32&) const
{
	// incomplete implementation
	throw Err();
	
	return IndexType();
}
*/
template<class TensorType>
inline void RangeSliceAllocator<TensorType>::set(const ElementType& val)
{
	for (Int32 i=0;i<size();++i) elementFromOffset(i) = val;
}

// alias to a tensor indirection slice
template<class TensorType> class IndirectionSliceAllocator
{
	public:
		typedef typename TensorType::AllocatorType::ElementType			ElementType;
		typedef typename TensorType::AllocatorType::IteratorType		IteratorType;
		typedef typename TensorType::AllocatorType::OffsetType			OffsetType;
		typedef typename TensorType::AllocatorType::CheckingPolicyType	CheckingPolicyType;

		enum { Rank = TensorType::Rank };

		typedef typename TensorType::IndexType			IndexType;
		typedef typename TensorType::IndirectionType	IndirectionType;
		
		IndirectionSliceAllocator(TensorType& source,const IndirectionType& ind) : ref_(source),ind_(ind) { }
		
		void set(const ElementType&);
		void set(const IndirectionSliceAllocator<TensorType>&);
		
		Int32 size() const														{ return ind_.size(); }
	
		OffsetType iteratorToOffset(const IteratorType& iter) const				{ return ref_.iteratorToOffset(iter); }
		IteratorType offsetToIterator(const OffsetType& offs) const				{ return ref_.offsetToIterator(offs); }
		
		ElementType& elementFromIterator(const IteratorType& iter)				{ return ref_.elementFromIterator(iter); }
		const ElementType& elementFromIterator(const IteratorType& iter) const	{ return ref_.elementFromIterator(iter); }
		
		ElementType& elementFromOffset(const OffsetType& offs)					{ return ref_.elementFromOffset(offs); }
		const ElementType& elementFromOffset(const OffsetType& offs) const		{ return ref_.elementFromOffset(offs); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		~IndirectionSliceAllocator() { }
		
	private:
		IndirectionSliceAllocator();
		IndirectionSliceAllocator<TensorType>& operator=(const IndirectionSliceAllocator<TensorType>&);
		
		TensorType&				ref_;
		const IndirectionType&	ind_;
};

template<class TensorType>
inline void IndirectionSliceAllocator<TensorType>::set(const ElementType& val)
{
	for (typename IndirectionType::const_iterator it=ind_.begin();it!=ind_.end();++it)
		ref_(*it) = val;
}

} // namespace mcs

#endif // MCS_TENSORALLOCATORS_H