#ifndef MCS_TENSORBOUNDS_H
#define MCS_TENSORBOUNDS_H

#include <limits>
#include <utility>
#include <vector>

#include "MCS_PlatformDependent.h"

#include "MCS_Constants.h"
#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_IndexRange.h"
#include "MCS_Typedefs.h"

namespace mcs {

class Regular { };
class Irregular { };
		
template<class G> class Simple { };
template<class G> class Extended { };

template<class T,Uint8 R,class Type = Simple<Regular> > class Bounds;

// evenly spaced bins
template<class T,Uint8 R>
class Bounds<T,R,Simple<Regular> >
{
	public:
		typedef typename IndexRange<R>::IndexType	IndexType;
		
		Bounds(const FixedVector<T,R>& lo,const FixedVector<T,R>& hi,const FixedVector<Int32,R>& nbins);
		Bounds(const Bounds<T,R,Simple<Regular> >& source);
		~Bounds() { }
		
		IndexType find(const FixedVector<T,R>& val) const;
			
		const IndexRange<R>& range() const		{ return rng_; }
		IndexRange<1> range(Int32 rnk) const	{ return rng_(rnk); }
		
		const FixedVector<T,R>& lo() const		{ return lo_; }
		const FixedVector<T,R>& hi() const		{ return hi_; }
		const FixedVector<T,R>& delta() const	{ return del_; }
		
		T lo(Int32 rnk) const		{ return lo_[rnk]; }
		T hi(Int32 rnk) const		{ return hi_[rnk]; }
		T delta(Int32 rnk) const	{ return del_[rnk]; }
		
		const std::vector<T>& centers(const Uint8& rank) const	{ return bds_[rank]; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		Bounds();
		Bounds<T,R,Simple<Regular> >& operator=(const Bounds<T,R,Simple<Regular> >&);
		
		typedef std::vector< std::vector<T> >	BoundsVectorType;
		
		static const EnsureReal<T>	dummy_;

		BoundsVectorType	bds_;
		FixedVector<T,R>	lo_,
							hi_,
							del_;
		IndexRange<R>		rng_;
};

// implementation
template<class T,Uint8 R> const EnsureReal<T> Bounds<T,R,Simple<Regular> >::dummy_;

template<class T,Uint8 R> 
inline Bounds<T,R,Simple<Regular> >::Bounds(const FixedVector<T,R>& lo,const FixedVector<T,R>& hi,const FixedVector<Int32,R>& nbins) : 
	bds_(R),
	lo_(lo),
	hi_(hi),
	del_(),
	rng_(nbins)
{
	if (any(hi <= lo)) throw Err();
	
	for (typename BoundsVectorType::iterator it=bds_.begin();it!=bds_.end();++it)
	{
		const Uint8	rankidx = it-bds_.begin();
		
		del_[rankidx] = (hi_[rankidx]-lo_[rankidx])/nbins[rankidx];
		
		*it = std::vector<T>(nbins[rankidx]);
		
		// get bin centers
		for (typename std::vector<T>::iterator jt=it->begin();jt!=it->end();++jt)
		{
			const Int32	intidx = jt-it->begin();
			
			*jt = lo_[rankidx]+(intidx+0.5)*del_[rankidx];
		}
	}
}

template<class T,Uint8 R> 
inline Bounds<T,R,Simple<Regular> >::Bounds(const Bounds<T,R,Simple<Regular> >& source) : 
	bds_(source.bds_),
	lo_(source.lo_),
	hi_(source.hi_),
	del_(source.del_),
	rng_(source.rng_)
{ }

template<class T,Uint8 R> 
inline typename Bounds<T,R,Simple<Regular> >::IndexType Bounds<T,R,Simple<Regular> >::find(const FixedVector<T,R>& val) const
{
	// val is out of bounds
	if (any(val < lo_ || val > hi_)) throw Err();
	
	IndexType	idx;
	
	for (typename BoundsVectorType::const_iterator it=bds_.begin();it!=bds_.end();++it)
	{
		const Uint8	rankidx = it-bds_.begin();

		const Int32	tmpidx = std::floor((val[rankidx]-lo_[rankidx])/del_[rankidx]);
		
		// make sure returned index is in allowable range 
		if (tmpidx < rng_.first(rankidx) || tmpidx > rng_.last(rankidx)) 
			throw Err();
		else
			idx[rankidx] = tmpidx;
	}
	
	return idx;
}

// evenly spaced bins with extra bins for OOB values
template<class T,Uint8 R>
class Bounds<T,R,Extended<Regular> >
{
	public:
		typedef typename IndexRange<R>::IndexType	IndexType;
		
		Bounds(const FixedVector<T,R>& lo,const FixedVector<T,R>& hi,const FixedVector<Int32,R>& nbins);
		Bounds(const Bounds<T,R,Extended<Regular> >& source);
		~Bounds() { }
		
		IndexType find(const FixedVector<T,R>& val) const;
		
		const IndexRange<R> range() const		{ return rng_; }
		
		const FixedVector<T,R>& lo() const		{ return lo_; }
		const FixedVector<T,R>& hi() const		{ return hi_; }
		const FixedVector<T,R>& delta() const	{ return del_; }
		
		const std::vector<T>& centers(const Uint8& rank) const	{ return bds_[rank]; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		Bounds();
		Bounds<T,R,Extended<Regular> >& operator=(const Bounds<T,R,Extended<Regular> >&);
		
		typedef std::vector< std::vector<T> >	BoundsVectorType;
		
		static const EnsureReal<T>	dummy_;
		static const T				MIN_VAL,
									MAX_VAL;
									
		BoundsVectorType	bds_;
		FixedVector<T,R>	lo_,
							hi_,
							del_;
		IndexRange<R>		rng_;
};

// implementation
template<class T,Uint8 R> const EnsureReal<T>	Bounds<T,R,Extended<Regular> >::dummy_;
template<class T,Uint8 R> const T				Bounds<T,R,Extended<Regular> >::MIN_VAL = std::numeric_limits<T>::is_integer ? std::numeric_limits<T>::min() : _minus_Inf_;
template<class T,Uint8 R> const T				Bounds<T,R,Extended<Regular> >::MAX_VAL = std::numeric_limits<T>::is_integer ? std::numeric_limits<T>::max() : _plus_Inf_;
					
template<class T,Uint8 R> 
inline Bounds<T,R,Extended<Regular> >::Bounds(const FixedVector<T,R>& lo,const FixedVector<T,R>& hi,const FixedVector<Int32,R>& nbins) : 
	bds_(R),
	lo_(lo),
	hi_(hi),
	del_(),
	rng_(nbins+2)
{
	//if (any(hi <= lo)) throw Err();
	
	for (typename BoundsVectorType::iterator it=bds_.begin();it!=bds_.end();++it)
	{
		const Uint8	rankidx = it-bds_.begin();
		
		del_[rankidx] = (hi_[rankidx]-lo_[rankidx])/nbins[rankidx];
		
		*it = std::vector<T>(nbins[rankidx]+2);
		
		// get bin centers
		*(it->begin()) = MIN_VAL;
		
		for (typename std::vector<T>::iterator jt=it->begin()+1;jt<=it->end()-1;++jt)
		{
			// -1 offset because of OOB bins
			const Int32	intidx = jt-it->begin()-1;
			
			*jt = lo_[rankidx]+(intidx+0.5)*del_[rankidx];
		}
		
		*(it->rbegin()) = MAX_VAL;
	}
}

template<class T,Uint8 R> 
inline Bounds<T,R,Extended<Regular> >::Bounds(const Bounds<T,R,Extended<Regular> >& source) : 
	bds_(source.bds_),
	lo_(source.lo_),
	hi_(source.hi_),
	del_(source.del_),
	rng_(source.rng_)
{ }

template<class T,Uint8 R> 
inline typename Bounds<T,R,Extended<Regular> >::IndexType Bounds<T,R,Extended<Regular> >::find(const FixedVector<T,R>& val) const
{
	IndexType	idx;
	
	for (typename BoundsVectorType::const_iterator it=bds_.begin();it!=bds_.end();++it)
	{
		const Uint8	rankidx = it-bds_.begin();
		
		if (val[rankidx] < lo_[rankidx])
			idx[rankidx] = rng_.first(rankidx);
		else if (val[rankidx] > hi_[rankidx])
			idx[rankidx] = rng_.last(rankidx);
		else
			// +1 offset because of OOB bins
			idx[rankidx] =  std::floor((val[rankidx]-lo_[rankidx])/del_[rankidx]) + 1;
	}
	
	return idx;
}

} // namespace mcs

#endif // MCS_TENSORBOUNDS_H