#ifndef MCS_TENSORIMPLEMENTATION_H
#define MCS_TENSORIMPLEMENTATION_H

#include <iostream>

#include "MCS_Exceptions.h"
#include "MCS_IndexRange.h"
#include "MCS_Index.h"
#include "MCS_TensorAccessors.h"
#include "MCS_TensorAllocators.h"
#include "MCS_TensorIndirection.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class All,class Acc>
class TensorImplementation :
	public All,
	public Acc
{
	public:
		typedef Acc	AccessorType;
		typedef All	AllocatorType;

		enum { Rank = AccessorType::Rank };

		typedef	typename AllocatorType::ElementType	ElementType;
		typedef typename AccessorType::IndexType	IndexType;
		typedef typename AccessorType::RangeType	RangeType;

		TensorImplementation();
		explicit TensorImplementation(const RangeType& rng);
		explicit TensorImplementation(const RangeType&,const ElementType&);
		explicit TensorImplementation(const RangeType&,ElementType*);
		explicit TensorImplementation(const All&,const Acc&);
		template<class All2,class Acc2> explicit TensorImplementation(const TensorImplementation<All2,Acc2>&);
		TensorImplementation(const TensorImplementation<All,Acc>&);

		TensorImplementation<All,Acc>& operator=(const TensorImplementation<All,Acc>&);

		void set(const ElementType&);
	
		ElementType& operator[](const Int32& el)					{ return elementFromOffset(el); }
		const ElementType& operator[](const Int32& el) const		{ return elementFromOffset(el); }
		
		void deserialize(std::istream& is)							{ Acc::deserialize(is); All::deserialize(is); }
		void serialize(std::ostream& os) const						{ Acc::serialize(os); All::serialize(os); }

	protected:
		~TensorImplementation() { }
};

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation() :
	All(),
	Acc()
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation(const RangeType& rng) :
	All(rng.elem()),
	Acc(rng)
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation(const RangeType& rng,const ElementType& val) :
	All(rng.elem(),val),
	Acc(rng)
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation(const RangeType& rng,ElementType* ptr) :
	All(rng.elem(),ptr),
	Acc(rng)
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation(const All& all,const Acc& acc) :
	All(all),
	Acc(acc)
{ }

template<class All,class Acc>
template<class All2,class Acc2>
inline TensorImplementation<All,Acc>::TensorImplementation(const TensorImplementation<All2,Acc2>& source) :
	All(source),
	Acc(source)
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>::TensorImplementation(const TensorImplementation<All,Acc>& source) :
	All(source),
	Acc(source)
{ }

template<class All,class Acc>
inline TensorImplementation<All,Acc>& TensorImplementation<All,Acc>::operator=(const TensorImplementation<All,Acc>& source) 
{
	if (this == &source) return *this;

	All::operator=(source);
	Acc::operator=(source);

	return *this;
}

template<class All,class Acc>
inline void TensorImplementation<All,Acc>::set(const ElementType& val)
{
	All::set(val);
}

template<class TensorType>
class AliasImplementation :
	public AliasAccessor<typename TensorType::AccessorType>,
	public AliasAllocator<typename TensorType::AllocatorType>
{
	public:
		typedef AliasAccessor<typename TensorType::AccessorType>	AccessorType;
		typedef AliasAllocator<typename TensorType::AllocatorType>	AllocatorType;

		enum { Rank = AccessorType::Rank };

		typedef	typename AllocatorType::ElementType		ElementType;
		typedef typename AccessorType::IndexType		IndexType;
		typedef typename AccessorType::RangeType		RangeType;

		AliasImplementation(TensorType&);
		AliasImplementation(const AliasImplementation<TensorType>&);
		~AliasImplementation() { }

		AliasImplementation<TensorType>& operator=(const AliasImplementation<TensorType>&);

		void set(const ElementType&);
		
		ElementType& operator[](const Int32& el)					{ return element(el); }
		const ElementType& operator[](const Int32& el) const		{ return element(el); }

	 	// IO
	 	void get(std::istream& is)									{ AccessorType::get(is); AllocatorType::get(is); }
	 	void read(std::istream& is)									{ AccessorType::read(is); AllocatorType::read(is); }
	 	void put(std::ostream& os) const							{ AccessorType::put(os); AllocatorType::put(os); }
	 	void write(std::ostream& os) const							{ AccessorType::write(os); AllocatorType::write(os); } 	
};

template<class TensorType>
inline AliasImplementation<TensorType>::AliasImplementation(TensorType& source) :
	AccessorType(source),
	AllocatorType(source)
{ }

template<class TensorType>
inline void AliasImplementation<TensorType>::set(const ElementType& val)
{
	AllocatorType::set(val);
}

template<class TensorType>
class IndirectionSliceImplementation :
	public IndirectionSliceAllocator<TensorType>,
	public TensorType::AccessorType
{
	public:
		typedef typename TensorType::AccessorType		AccessorType;
		typedef IndirectionSliceAllocator<TensorType>	AllocatorType;

		enum { Rank = AccessorType::Rank };

		typedef	typename AllocatorType::ElementType		ElementType;
		typedef typename AccessorType::IndexType		IndexType;
		typedef typename AccessorType::RangeType		RangeType;

		IndirectionSliceImplementation(TensorType&,const TensorIndirection<Rank>&);
		IndirectionSliceImplementation(const IndirectionSliceImplementation<TensorType>&);
		~IndirectionSliceImplementation() { }

		IndirectionSliceImplementation<TensorType>& operator=(const IndirectionSliceImplementation<TensorType>&);

		void set(const ElementType&);

		// IO
	 	void get(std::istream& is)									{ AccessorType::get(is); AllocatorType::get(is); }
	 	void read(std::istream& is)									{ AccessorType::read(is); AllocatorType::read(is); }
	 	void put(std::ostream& os) const							{ AccessorType::put(os); AllocatorType::put(os); }
	 	void write(std::ostream& os) const							{ AccessorType::write(os); AllocatorType::write(os); } 	
};

template<class TensorType>
inline IndirectionSliceImplementation<TensorType>::IndirectionSliceImplementation(TensorType& source,const TensorIndirection<Rank>& ind) :
	AllocatorType(source,ind),
	AccessorType(source.range())
{ }

template<class TensorType>
inline IndirectionSliceImplementation<TensorType>::IndirectionSliceImplementation(const IndirectionSliceImplementation<TensorType>& source) :
	AllocatorType(source),
	AccessorType(source)
{ }

template<class TensorType>
inline void IndirectionSliceImplementation<TensorType>::set(const ElementType& val)
{
	AllocatorType::set(val);
}

template<class TensorType>
class MaskSliceImplementation :
	public MaskSliceAllocator<TensorType>,
	public TensorType::AccessorType
{
	public:
		typedef typename TensorType::AccessorType		AccessorType;
		typedef MaskSliceAllocator<TensorType>			AllocatorType;

		enum { Rank = AccessorType::Rank };

		typedef	typename AllocatorType::ElementType		ElementType;
		typedef typename AccessorType::IndexType		IndexType;
		typedef typename AccessorType::RangeType		RangeType;

		MaskSliceImplementation(TensorType&,const typename TensorType::MaskType&);
		MaskSliceImplementation(const MaskSliceImplementation<TensorType>&);
		~MaskSliceImplementation() { }

		MaskSliceImplementation<TensorType>& operator=(const MaskSliceImplementation<TensorType>&);

		void set(const ElementType&);

		// IO
	 	void get(std::istream& is)									{ AccessorType::get(is); AllocatorType::get(is); }
	 	void read(std::istream& is)									{ AccessorType::read(is); AllocatorType::read(is); }
	 	void put(std::ostream& os) const							{ AccessorType::put(os); AllocatorType::put(os); }
	 	void write(std::ostream& os) const							{ AccessorType::write(os); AllocatorType::write(os); } 	
};

template<class TensorType>
inline MaskSliceImplementation<TensorType>::MaskSliceImplementation(TensorType& source,const typename TensorType::MaskType& msk) :
	AllocatorType(source,msk),
	AccessorType(source.range())
{ }

template<class TensorType>
inline MaskSliceImplementation<TensorType>::MaskSliceImplementation(const MaskSliceImplementation<TensorType>& source) :
	AllocatorType(source),
	AccessorType(source)
{ }

template<class TensorType>
inline void MaskSliceImplementation<TensorType>::set(const ElementType& val)
{
	AllocatorType::set(val);
}

template<class TensorType>
class RangeSliceImplementation :
	public RangeSliceAllocator<TensorType>,
	public TensorType::AccessorType
{
	public:
		typedef typename TensorType::AccessorType		AccessorType;
		typedef RangeSliceAllocator<TensorType>			AllocatorType;

		enum { Rank = AccessorType::Rank };

		typedef	typename AllocatorType::ElementType		ElementType;
		typedef typename AccessorType::IndexType		IndexType;
		typedef typename AccessorType::RangeType		RangeType;

		RangeSliceImplementation(TensorType&,const RangeType&);
		RangeSliceImplementation(const RangeSliceImplementation<TensorType>&);
		~RangeSliceImplementation() { }

		RangeSliceImplementation<TensorType>& operator=(const RangeSliceImplementation<TensorType>&);

		void set(const ElementType&);

		// IO
	 	void get(std::istream& is)									{ AccessorType::get(is); AllocatorType::get(is); }
	 	void read(std::istream& is)									{ AccessorType::read(is); AllocatorType::read(is); }
	 	void put(std::ostream& os) const							{ AccessorType::put(os); AllocatorType::put(os); }
	 	void write(std::ostream& os) const							{ AccessorType::write(os); AllocatorType::write(os); } 	
};

template<class TensorType>
inline RangeSliceImplementation<TensorType>::RangeSliceImplementation(TensorType& source,const RangeType& rng) :
	AllocatorType(source,rng),
	AccessorType(source.range())
{ }

template<class TensorType>
inline RangeSliceImplementation<TensorType>::RangeSliceImplementation(const RangeSliceImplementation<TensorType>& source) :
	AllocatorType(source),
	AccessorType(source)
{ }

template<class TensorType>
inline void RangeSliceImplementation<TensorType>::set(const ElementType& val)
{
	AllocatorType::set(val);
}

} // namespace mcs

#endif // MCS_TENSORIMPLEMENTATION_H