#ifndef MCS_VECTOR_H
#define MCS_VECTOR_H

#include <cmath>
#include <cstring>

#include "MCS_PlatformDependent.h"

#include "MCS_Ensure.h"
#include "MCS_Exceptions.h"
#include "MCS_IndexRange.h"
#include "MCS_Promote.h"
#include "MCS_Typedefs.h"

#define USE_INTEL_SPL 1

namespace mcs {

class VectorIndirection;
class VectorMask;

template<class T> class VectorIndirectionSlice;
template<class T> class VectorMaskSlice;
template<class T> class VectorRangeSlice;

__DECLARE_EXCEPTION_CLASS__(VectorException,Exception)

// vector
template<class T> class Vector
{
	public:	
		typedef Vector<T>&			reference;
		typedef const Vector<T>&	const_reference;
		typedef Vector<T>*			pointer;
		typedef const Vector<T>*	const_pointer;
		
		// ctor/dtor		
		Vector();
		explicit Vector(const IndexRange<1>&);
		explicit Vector(const IndexRange<1>&,const T&);
		explicit Vector(const IndexRange<1>&,const T*);
		explicit Vector(const IndexRange<1>&,const std::string&);
		explicit Vector(const VectorIndirectionSlice<T>&);
		explicit Vector(const VectorMaskSlice<T>&);
		explicit Vector(const VectorRangeSlice<T>&);
		Vector(const Vector<T>&);
		virtual ~Vector();
		
		// range accessors		
				bool			isValid() const		{ return valid_; }
				
				Int32			elem() const		{ return rng_.elem(); }
					
				Int32			first() const		{ return rng_.first(0); }
				Int32		 	last() const		{ return rng_.last(0); }
				Int32			dim() const			{ return rng_.dim(0); }
				Int32			offset() const		{ return rng_.first(0); }
		
		const	IndexRange<1>&	range() const		{ return rng_; }
		
		// range assignors
		const	Vector<T>& 		offset(const Int32&) const;
		const	Vector<T>& 		range(const IndexRange<1>&) const;
		
		// assignment
				Vector<T>& 		operator=(const T&);
				Vector<T>& 		operator=(const std::string&);
				Vector<T>&		operator=(const IndexRange<1>&);
				Vector<T>& 		operator=(const Vector<T>&);
				 
				Vector<T>& 		operator=(const VectorIndirectionSlice<T>&);
				Vector<T>&		operator=(const VectorMaskSlice<T>&);
				Vector<T>& 		operator=(const VectorRangeSlice<T>&);
		
		// element accessors/assignors		
				T& 				operator[](Int32);
		const 	T& 				operator[](Int32) const;
		
			 	VectorRangeSlice<T>			operator[](const IndexRange<1>&);
			    VectorMaskSlice<T>			operator[](const VectorMask&);
			    VectorIndirectionSlice<T>	operator[](const VectorIndirection&);
		
		const	VectorRangeSlice<T>			operator[](const IndexRange<1>&) const;
		const	VectorMaskSlice<T> 			operator[](const VectorMask&) const;
		const	VectorIndirectionSlice<T>	operator[](const VectorIndirection&) const;

		// arithmetic operations		
			 	Vector<T>& 		operator+=(const T&);		
			 	Vector<T>& 		operator-=(const T&);
			 	Vector<T>& 		operator*=(const T&);
			 	Vector<T>& 		operator/=(const T&);

			 	Vector<T>& 		operator+=(const Vector<T>&);
			 	Vector<T>& 		operator-=(const Vector<T>&);
			 	Vector<T>& 		operator*=(const Vector<T>&);
			 	Vector<T>& 		operator/=(const Vector<T>&);
							
		// low level access
				T* const		dataPointer() const		{ return vec_ptr_; }

		// set vec_ptr_ to specified pointer - potentially dangerous
				void			place(T*,const IndexRange<1>&);
				void			unplace();
				
		// copy data from specified pointer - safe
				void			clone(const T*);
				
		// element assignors			
				void			set(const T&);
				void			set(const T*);
				void			set(const std::string&);	
				
		// IO
				void			print(std::ostream& os) const;
				
		virtual	Vector<T>& 		get(std::istream& ifs);
		virtual	Vector<T>&		getNext(std::istream& ifs,Int32 N = 1);
		virtual	Vector<T>&		getNth(std::istream& ifs,Int32 N = 1);
				
		virtual	Vector<T>& 		read(std::istream& ifs);
		virtual	Vector<T>&		readNext(std::istream& ifs,Int32 N = 1);
		virtual	Vector<T>&		readNth(std::istream& ifs,Int32 N = 1);
				
		virtual const Vector<T>& 	put(std::ostream& ofs) const;
		virtual const Vector<T>& 	write(std::ostream& ofs) const;
		
		// offset class to allow offset change within local block with cleanup
		class Offset;
			
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	protected:
		// element assignors			
				void			set(const VectorIndirectionSlice<T>&);
				void			set(const VectorMaskSlice<T>&);
				void			set(const VectorRangeSlice<T>&);

		// allocators	
				void			alloc(const IndexRange<1>& rng = IndexRange<1>());
				void 			dealloc();
				void			realloc(const IndexRange<1>& rng = IndexRange<1>());
				
	private:
				bool	valid_;
				T*		vec_ptr_;			// pointer to actual location of start of data
		
		mutable IndexRange<1>	rng_;		// vector range
		mutable T*				vec_data_;	// pointer to virtual location of start of data wrt offset
};

/*
	Utility class for Vectors which allows the offset index to be temporarily changed 
		within a local block of code, with the destructor resetting it to its original value
		on exit.  Usage :
		
			if (true)
			{
				Vector<T>::Offset	offs(v1,o1);
				
				...do some stuff...
			}
*/

template<class T> class Vector<T>::Offset
{
	public:
		Offset(const Vector<T>& vec,const Int32& off) : vec_(vec) { ooff_ = vec_.first(); vec_.offset(off); }
		~Offset() { vec_.offset(ooff_); }
		
		Int32 currentOffset() const		{ return vec_.first(); }
		Int32 originalOffset() const	{ return ooff_; }
		
	private:
		Offset();
		Offset(const Offset&);
		Offset& operator=(const Offset&);
		
		const Vector<T>&	vec_;
		Int32				ooff_;
};

#include "MCS_Vector.cpp"

} // namespace mcs

// implementation
#define USE_INTEL_SPL 1

#ifdef __INTEL__
	#if USE_INTEL_SPL
		#include "MCS_Vector_IntelSpecialized.h"
	#endif
#endif

#include "MCS_VectorIndirection.h"
#include "MCS_VectorMask.h"

#include "MCS_VectorIndirectionSlice.h"
#include "MCS_VectorMaskSlice.h"
#include "MCS_VectorRangeSlice.h"

#include "MCS_VectorFunctions.h"
#include "MCS_VectorIO.h"
#include "MCS_VectorOperators.h"
#include "MCS_VectorStatistics.h"
#include "MCS_VectorTypecasts.h"
#include "MCS_VectorUtilities.h"

#endif // MCS_VECTOR_H