#ifndef MCS_VECTORIO_H
#define MCS_VECTORIO_H

#include <iostream>
#include <string>
#include <valarray>

#include "MCS_ByteSwap.h"
#include "MCS_IO.h"
#include "MCS_TypeFlags.h"
#include "MCS_Vector.h"

namespace mcs {

template<class T> std::istream& operator>>(std::istream& is,Vector<T>& dest);
template<class T> std::ostream& operator<<(std::ostream& os,const Vector<T>& source);

// stream input
template<class T> inline std::istream& operator>>(std::istream& is,Vector<T>& dest)
{ 
	//const std::string	lineread(is.getline());
	
	dest = Vector<T>(is.getline());
	
	return is; 
}

// stream output
template<class T> inline std::ostream& operator<<(std::ostream& os,const Vector<T>& source)
{
	source.print(os);
	
	return os;
}

// I/O
// output in text format
template<class T> inline void Vector<T>::print(std::ostream& os) const
{
	// allow this to be changed
	static const int 	MAX_OUT_ELEM = 20;
	
	os << "[ ";

	if (isValid())
	{
		if (dim() > MAX_OUT_ELEM)
		{
			for (Int32 i=first();i<=first()+MAX_OUT_ELEM-1;i++)
				os << (*this)[i] << "  ";
				
			os << " ... " << (*this)[last()] << " ";
		}
		else
		{
			for (Int32 i=first();i<=last()-1;i++)
				os << (*this)[i] << "  ";
				
			os << (*this)[last()] << " ";
		}
	}

	os << "]";
	
	return;
}

// read from file stream
template<class T> inline Vector<T>& Vector<T>::get(std::istream& ifs)
{
	// if exception handling is desired, need to have following in calling program:
	// 	
	//	ios_base::iostate	ifsState = ifs.exceptions();							// save IO state
	//	ifs.exceptions(ios_base::badbit | ios_base::eofbit | ios_base::failbit);	// throw exception on IO error
	//
	//	...IO operations...
	//
	//	ifs.exceptions(ifsState);													// restore IO state
	
	if (!isValid()) throw IOErr();
	
	try 
	{
		if (!ifs.read(reinterpret_cast<char*>(dataPointer()),static_cast<Int32>(dim())*sizeof(T)))
			throw IOErr();
	} 
	catch (std::ios_base::failure& ex) 
	{ 
		throw IOErr(); 
	}
	
	return (*this);
}

// read nth vector
template<class T> inline Vector<T>& Vector<T>::getNth(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(dim()*sizeof(T)));
	
		ifs.seekg(offs,std::ios::beg);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(dim()*sizeof(T)));
	
		ifs.seekg(offs,std::ios::end);
	}
	
	return get(ifs);
}

// read nth vector after skipping n-1 vectors
template<class T> inline Vector<T>& Vector<T>::getNext(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(dim()*sizeof(T)));
	
		ifs.seekg(offs,std::ios::cur);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(dim()*sizeof(T)));
	
		ifs.seekg(offs,std::ios::cur);
	}
	
	return get(ifs);
}

// read from file stream, with dimensions
template<class T> Vector<T>& Vector<T>::read(std::istream& ifs)
{
	if (!ifs.good()) throw IOErr();
	
	Int32	lo_idx,hi_idx;
	Uint8	flags[4];
	
	try
	{
		if (!ifs.read(reinterpret_cast<char *>(&lo_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char *>(&hi_idx),sizeof(Int32)) ||
			!ifs.read(reinterpret_cast<char *>(&flags),4*sizeof(Uint8)))
			throw IOErr();
		
		if (!ifs.good()) throw IOErr();
	}
	catch (std::ios_base::failure& ex)
	{
		throw IOErr();
	}
		
	if (TypeFlags<T>::flags().to_ulong() != flags[1]) throw IOErr();

	realloc(IndexRange<1>(lo_idx,hi_idx));
	
	return get(ifs);
}

// read nth vector, with dimensions
template<class T> inline Vector<T>& Vector<T>::readNth(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(dim()*sizeof(T)+3*sizeof(Int32)));
		
		ifs.seekg(offs,std::ios::beg);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(dim()*sizeof(T)+3*sizeof(Int32)));
		
		ifs.seekg(offs,std::ios::end);
	}
			
	return read(ifs);
}

// read nth vector, with dimensions, after skipping n-1 vectors
template<class T> inline Vector<T>& Vector<T>::readNext(std::istream& ifs,Int32 N)
{
	if (!isValid() || N == 0) throw IllegalOperationErr();
	
	if (N > 0)
	{
		const std::streamoff	offs((N-1)*(dim()*sizeof(T)+3*sizeof(Int32)));
	
		ifs.seekg(offs,std::ios::cur);
	}
	else if (N < 0)
	{
		const std::streamoff	offs(N*(dim()*sizeof(T)+3*sizeof(Int32)));
	
		ifs.seekg(offs,std::ios::cur);
	}
	
	return read(ifs);
}

// write to file stream
template<class T> const Vector<T>& Vector<T>::put(std::ostream& ofs) const
{
	if (!isValid() || !ofs.good()) throw IOErr();
	
	try
	{
		if (!ofs.write(reinterpret_cast<char*>(dataPointer()),static_cast<Uint32>(dim())*sizeof(T)))
			throw IOErr();
	}
	catch (std::ios_base::failure& ex)
	{
		throw IOErr();
	}
			
	return (*this);
}

// write to file stream, with dimensions
template<class T> const Vector<T>& Vector<T>::write(std::ostream& ofs) const
{
	if (!isValid() || !ofs.good()) throw IOErr();
				
	Int32 	lo_idx = rng_.first(),hi_idx = rng_.last();
	Int32		flags = TypeFlags<T>::flags().to_ulong() << 8;
	
	try
	{
		bool	failed = false;
		
		// output lo and hi indices
		if (!ofs.write(reinterpret_cast<char *>(&lo_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char *>(&hi_idx),sizeof(Int32)) ||
			!ofs.write(reinterpret_cast<char *>(&flags),sizeof(Uint32)))
			throw IOErr();
	}
	catch (std::ios_base::failure& ex)
	{
		throw IOErr();
	}
	
	// output data
	return put(ofs);
}

template<class T> void byteSwap(Vector<T>& vec)
{
	for (Int32 i=vec.first();i<=vec.last();i++)
		byteSwap(vec[i]);
}

} // namespace mcs

#endif // MCS_VECTORIO_H

