#ifndef MCS_VECTORINDIRECTIONSLICE_H
#define MCS_VECTORINDIRECTIONSLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> class Vector;

// VectorIndirectionSlice
template<class T> class VectorIndirectionSlice 
{
	public:			
		// dtor
		~VectorIndirectionSlice();
		
		// range accessors		
				bool			isValid() const		{ return ind_.isValid(); }
				
				Int32			elem() const		{ return ind_.elem(); }
					
				Int32	 		first() const		{ return ind_.first(); }
				Int32	 		last() const		{ return ind_.last(); }
				Int32	 		dim() const			{ return ind_.dim(); }
				Int32			offset() const		{ return ind_.offset(); }
		
		const	IndexRange<1>&	range() const		{ return ind_.range(); }
		
		// assignment
		VectorIndirectionSlice<T>& 	operator=(const T&);
		VectorIndirectionSlice<T>& 	operator=(const Vector<T>&);
		VectorIndirectionSlice<T>&	operator=(const VectorIndirectionSlice<T>&);
		
		T& operator[](const Int32& idx)				{ return vec_[ind_[idx]]; }
		const T& operator[](const Int32& idx) const	{ return vec_[ind_[idx]]; }
		
		// logical operations	
		VectorMask	 				operator==(const T&) const;
	 	VectorMask 					operator!=(const T&) const;
	 	VectorMask 					operator<(const T&) const;
	 	VectorMask 					operator<=(const T&) const;
	 	VectorMask 					operator>(const T&) const;
	 	VectorMask 					operator>=(const T&) const;
		
	 	VectorMask 					operator==(const VectorIndirectionSlice<T>&) const;
	 	VectorMask 					operator!=(const VectorIndirectionSlice<T>&) const;
	 	VectorMask 					operator<(const VectorIndirectionSlice<T>&) const;
	 	VectorMask 					operator<=(const VectorIndirectionSlice<T>&) const;
	 	VectorMask 					operator>(const VectorIndirectionSlice<T>&) const;
	 	VectorMask 					operator>=(const VectorIndirectionSlice<T>&) const;

		// arithmetic operations		
	 	VectorIndirectionSlice<T>& 	operator+=(const T&);		
	 	VectorIndirectionSlice<T>& 	operator-=(const T&);
	 	VectorIndirectionSlice<T>& 	operator*=(const T&);
	 	VectorIndirectionSlice<T>& 	operator/=(const T&);

	 	VectorIndirectionSlice<T>& 	operator+=(const VectorIndirectionSlice<T>&);
	 	VectorIndirectionSlice<T>& 	operator-=(const VectorIndirectionSlice<T>&);
	 	VectorIndirectionSlice<T>& 	operator*=(const VectorIndirectionSlice<T>&);
	 	VectorIndirectionSlice<T>& 	operator/=(const VectorIndirectionSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Vector<T>;
	
		// ctor	
		explicit VectorIndirectionSlice(Vector<T>&,const VectorIndirection&);
		explicit VectorIndirectionSlice(const Vector<T>&,const VectorIndirection&);
		
		VectorIndirectionSlice(const VectorIndirectionSlice<T>&);

		void checkBounds() const;
		
				Vector<T>&			vec_;
		const 	VectorIndirection&	ind_;
};

// constructor
// zero offset vector
template<class T> VectorIndirectionSlice<T>::VectorIndirectionSlice(Vector<T>& vec,const VectorIndirection& ind) : 
	vec_(vec),ind_(ind)
{ 
	// ensure all indirected indices are in bounds
	checkBounds();
}

// dangerous const_cast - kludge central
//	intended to allow things like m = mean(T[find(Z<1000)]);
//
template<class T> VectorIndirectionSlice<T>::VectorIndirectionSlice(const Vector<T>& vec,const VectorIndirection& ind) : 
	vec_(const_cast<Vector<T>&>(vec)),ind_(ind)
{ 
	// ensure all indirected indices are in bounds
	checkBounds();
}

template<class T> void VectorIndirectionSlice<T>::checkBounds() const
{
#if VECTOR_RANGE_CHECKING
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		if (ind_[i] < vec_.first() || ind_[i] > vec_.last())
			throw RangeErr();
#endif
	
	return;
}

// destructor
template<class T> VectorIndirectionSlice<T>::~VectorIndirectionSlice()
{ }

// assignment
template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator=(const T& val)
{
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		vec_[ind_[i]] = val;
		
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator=(const Vector<T>& source)
{
	if (ind_.dim() != source.dim()) throw RangeErr();
	
	if (&vec_ == &source) return *this;
		
	for (Int32 i=ind_.first(),ii=source.first();i<=ind_.last();i++,ii++)
		vec_[ind_[i]] = source[ii];
		
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator=(const VectorIndirectionSlice<T>& source)
{
	if (ind_.dim() != source.ind_.dim()) throw RangeErr();
	
	if (&vec_ == &source.vec_) return *this;
		
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++)
		vec_[ind_[i]] = source.vec_[source.ind_[ii]];
		
	return *this;
}

// logical operators
template<class T> VectorMask VectorIndirectionSlice<T>::operator==(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[i] == val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator!=(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[i] != val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator<(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[i] < val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator<=(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[i] <= val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator>(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[i] > val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator>=(const T& val) const
{
	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first();i<=ind_.last();i++)
		tmp[i] = (vec_[ind_[i]] >= val);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator==(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] == source.vec_[source.ind_[ii]]);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator!=(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] != source.vec_[source.ind_[ii]]);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator<(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] < source.vec_[source.ind_[ii]]);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator<=(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] <= source.vec_[source.ind_[ii]]);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator>(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] > source.vec_[source.ind_[ii]]);
		
	return tmp;
}

template<class T> VectorMask VectorIndirectionSlice<T>::operator>=(const VectorIndirectionSlice<T>& source) const
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();

	VectorMask tmp(ind_.range());
	
	for (Int32 i=ind_.first(),ii=source.ind_.dim();i<=ind_.last();i++,ii++)
		tmp[i] = (vec_[ind_[i]] >= source.vec_[source.ind_[ii]]);
		
	return tmp;
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=ind_.first();i<=ind_.last();vec_[ind_[i++]]+=val) { };
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=ind_.first();i<=ind_.last();vec_[ind_[i++]]-=val) { };
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
		
	for (Int32 i=ind_.first();i<=ind_.last();vec_[ind_[i++]]*=val) { };
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator/=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
	
	for (Int32 i=ind_.first();i<=ind_.last();vec_[ind_[i++]]/=val) { };
	
	return *this;
}

// VectorIndirectionSlice member assign/equals operators
template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator+=(const VectorIndirectionSlice<T>& source)
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++) 
		vec_[ind_[i]] += source.vec_[source.ind_[ii]];
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator-=(const VectorIndirectionSlice<T>& source)
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++) 
		vec_[ind_[i]] -= source.vec_[source.ind_[ii]];
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator*=(const VectorIndirectionSlice<T>& source)
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++) 
		vec_[ind_[i]] *= source.vec_[source.ind_[ii]];
	
	return *this;
}

template<class T> VectorIndirectionSlice<T>& VectorIndirectionSlice<T>::operator/=(const VectorIndirectionSlice<T>& source)
{
	if (source.ind_.dim() != ind_.dim()) throw RangeErr();
	
	for (Int32 i=ind_.first(),ii=source.ind_.first();i<=ind_.last();i++,ii++) 
		vec_[ind_[i]] /= source.vec_[source.ind_[ii]];
	
	return *this;
}

} // namespace mcs

#include "MCS_VectorIndirectionSliceFunctions.h"
#include "MCS_VectorIndirectionSliceOperators.h"
#include "MCS_VectorIndirectionSliceStatistics.h"

#endif // MCS_VECTORINDIRECTIONSLICE_H