#ifndef MCS_VECTORMASKSLICE_H
#define MCS_VECTORMASKSLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> class Vector;

// VectorMaskSlice
template<class T> class VectorMaskSlice
{
	public:			
		// dtor
		~VectorMaskSlice();
		
		// assignment
		VectorMaskSlice<T>& operator=(const T&);
		VectorMaskSlice<T>& operator=(const Vector<T>&);
		
		// logical operations	
			 	VectorMask 		operator==(const T&) const;
			 	VectorMask 		operator!=(const T&) const;
			 	VectorMask 		operator<(const T&) const;
			 	VectorMask 		operator<=(const T&) const;
			 	VectorMask 		operator>(const T&) const;
			 	VectorMask 		operator>=(const T&) const;
		
			 	VectorMask 		operator==(const VectorMaskSlice<T>&) const;
			 	VectorMask 		operator!=(const VectorMaskSlice<T>&) const;
			 	VectorMask 		operator<(const VectorMaskSlice<T>&) const;
			 	VectorMask 		operator<=(const VectorMaskSlice<T>&) const;
			 	VectorMask 		operator>(const VectorMaskSlice<T>&) const;
			 	VectorMask 		operator>=(const VectorMaskSlice<T>&) const;

		// arithmetic operations		
	 	VectorMaskSlice<T>& 	operator+=(const T&);		
	 	VectorMaskSlice<T>& 	operator-=(const T&);
	 	VectorMaskSlice<T>& 	operator*=(const T&);
	 	VectorMaskSlice<T>& 	operator/=(const T&);

	 	VectorMaskSlice<T>& 	operator+=(const VectorMaskSlice<T>&);
	 	VectorMaskSlice<T>& 	operator-=(const VectorMaskSlice<T>&);
	 	VectorMaskSlice<T>& 	operator*=(const VectorMaskSlice<T>&);
	 	VectorMaskSlice<T>& 	operator/=(const VectorMaskSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Vector<T>;
	
		// ctor		
		explicit VectorMaskSlice(Vector<T>&,const VectorMask&);
		explicit VectorMaskSlice(const Vector<T>&,const VectorMask&);
		
		VectorMaskSlice(const VectorMaskSlice<T>&);
		VectorMaskSlice<T>& operator=(const VectorMaskSlice<T>&);

				Vector<T>&		vec_;
		const 	VectorMask&		msk_;
};

// constructor
// zero offset vector
template<class T> VectorMaskSlice<T>::VectorMaskSlice(Vector<T>& vec,const VectorMask& msk) : 
	vec_(vec),msk_(msk)
{ }

// dangerous const_cast - kludge central
//	intended to allow things like m = mean(T[rng]);
//
template<class T> VectorMaskSlice<T>::VectorMaskSlice(const Vector<T>& vec,const VectorMask& msk) : 
	vec_(const_cast<Vector<T>&>(vec)),msk_(msk)
{ }

// destructor
template<class T> VectorMaskSlice<T>::~VectorMaskSlice()
{ }

// assignment
template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator=(const T& val)
{
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] = val;
		
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator=(const Vector<T>& source)
{
	if (msk_.range() != source.range()) throw VectorMaskSlice<T>::SizeMismatchErr();
	
	if (&vec_ == &source) return *this;
		
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] = source[i];
		
	return *this;
}

// logical operators
template<class T> VectorMask VectorMaskSlice<T>::operator==(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] == val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator!=(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] != val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator<(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] < val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator<=(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] <= val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator>(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] > val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator>=(const T& val) const
{
	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[msk_[i]] >= val);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator==(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] == source.vec_[i]);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator!=(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] != source.vec_[i]);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator<(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] < source.vec_[i]);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator<=(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] <= source.vec_[i]);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator>(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] > source.vec_[i]);
		
	return tmp;
}

template<class T> VectorMask VectorMaskSlice<T>::operator>=(const VectorMaskSlice<T>& source) const
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();

	VectorMask tmp(msk_.range());
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) tmp[i] = (vec_[i] >= source.vec_[i]);
		
	return tmp;
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] += val;
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] -= val;
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
		
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] *= val;
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator/=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
	
	for (Int32 i=msk_.first();i<=msk_.last();i++)
		if (msk_[i]) vec_[i] /= val;
	
	return *this;
}

// VectorMaskSlice member assign/equals operators
template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator+=(const VectorMaskSlice<T>& source)
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();
	
	for (Int32 i=msk_.first();i<=msk_.last();i++) 
		if (msk_[i]) vec_[i] += source.vec_[i];
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator-=(const VectorMaskSlice<T>& source)
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();
	
	for (Int32 i=msk_.first();i<=msk_.last();i++) 
		if (msk_[i]) vec_[i] -= source.vec_[i];
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator*=(const VectorMaskSlice<T>& source)
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();
	
	for (Int32 i=msk_.first();i<=msk_.last();i++) 
		if (msk_[i]) vec_[i] *= source.vec_[i];
	
	return *this;
}

template<class T> VectorMaskSlice<T>& VectorMaskSlice<T>::operator/=(const VectorMaskSlice<T>& source)
{
	if (source.msk_.range() != msk_.range()) throw VectorMaskSlice<T>::SizeMismatchErr();
	
	for (Int32 i=msk_.first();i<=msk_.last();i++) 
		if (msk_[i]) vec_[i] /= source.vec_[i];
	
	return *this;
}

} // namespace mcs

#endif // MCS_VECTORMASKSLICE_H