#ifndef MCS_VECTORRANGESLICE_H
#define MCS_VECTORRANGESLICE_H

#include "MCS_Ensure.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> class Vector;

// VectorRangeSlice
template<class T> class VectorRangeSlice
{
	public:			
		// dtor
		~VectorRangeSlice();
		
		// range accessors		
				bool			isValid() const		{ return vec_alias_.isValid(); }
				
				Int32			elem() const		{ return vec_alias_.elem(); }
					
				Int32	 		first() const		{ return vec_alias_.first(); }
				Int32	 		last() const		{ return vec_alias_.last(); }
				Int32	 		dim() const			{ return vec_alias_.dim(); }
				Int32			offset() const		{ return vec_alias_.offset(); }
		
		const	IndexRange<1>&	range() const		{ return vec_alias_.range(); }
		
		// assignment
		VectorRangeSlice<T>& 	operator=(const T&);
		VectorRangeSlice<T>& 	operator=(const Vector<T>&);
		
		T& operator[](const Int32& idx)				{ return vec_alias_[idx]; }
		const T& operator[](const Int32& idx) const	{ return vec_alias_[idx]; }
		
		// logical operations	
		VectorMask 				operator==(const T&) const;
		VectorMask 				operator!=(const T&) const;
	 	VectorMask 				operator<(const T&) const;
	 	VectorMask 				operator<=(const T&) const;
	 	VectorMask 				operator>(const T&) const;
	 	VectorMask 				operator>=(const T&) const;
		
	 	VectorMask 				operator==(const VectorRangeSlice<T>&) const;
	 	VectorMask 				operator!=(const VectorRangeSlice<T>&) const;
	 	VectorMask 				operator<(const VectorRangeSlice<T>&) const;
	 	VectorMask 				operator<=(const VectorRangeSlice<T>&) const;
	 	VectorMask 				operator>(const VectorRangeSlice<T>&) const;
	 	VectorMask 				operator>=(const VectorRangeSlice<T>&) const;

		// arithmetic operations		
	 	VectorRangeSlice<T>& 	operator+=(const T&);		
	 	VectorRangeSlice<T>& 	operator-=(const T&);
	 	VectorRangeSlice<T>& 	operator*=(const T&);
	 	VectorRangeSlice<T>& 	operator/=(const T&);

	 	VectorRangeSlice<T>& 	operator+=(const VectorRangeSlice<T>&);
	 	VectorRangeSlice<T>& 	operator-=(const VectorRangeSlice<T>&);
	 	VectorRangeSlice<T>& 	operator*=(const VectorRangeSlice<T>&);
	 	VectorRangeSlice<T>& 	operator/=(const VectorRangeSlice<T>&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
		
	private:
		friend class Vector<T>;
			
		// ctor		
		explicit VectorRangeSlice(Vector<T>&,const IndexRange<1>&);
		explicit VectorRangeSlice(const Vector<T>&,const IndexRange<1>&);
		
		VectorRangeSlice(const VectorRangeSlice<T>&);
		VectorRangeSlice<T>& operator=(const VectorRangeSlice<T>&);

				Vector<T>&		vec_;
				Vector<T>		vec_alias_;
		const 	IndexRange<1>&	rng_;
};

// constructor
// zero offset vector
template<class T> inline VectorRangeSlice<T>::VectorRangeSlice(Vector<T>& vec,const IndexRange<1>& rng) : 
	vec_(vec),vec_alias_(),rng_(rng)
{
	vec_alias_.place(&vec_[rng_.first()],rng); 
}

// dangerous const_cast - kludge central
//	intended to allow things like m = mean(T[rng]);
//
template<class T> inline VectorRangeSlice<T>::VectorRangeSlice(const Vector<T>& vec,const IndexRange<1>& rng) : 
	vec_(const_cast<Vector<T>&>(vec)),vec_alias_(),rng_(rng)
{
	vec_alias_.place(&vec_[rng_.first()],rng); 
}

// destructor
template<class T> inline VectorRangeSlice<T>::~VectorRangeSlice()
{ 
	vec_alias_.unplace();
}

// assignment
template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator=(const T& val)
{
	vec_alias_ = val;
		
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator=(const Vector<T>& source)
{
	if (rng_.dim() != source.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	if (&vec_ == &source) return *this;
		
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	vec_alias_ = source;	
		
	return *this;
}

// logical operators
template<class T> inline VectorMask VectorRangeSlice<T>::operator==(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ == val);
	
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator!=(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ != val);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator<(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ < val);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator<=(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ <= val);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator>(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ > val);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator>=(const T& val) const
{
	VectorMask tmp(rng_);
	
	tmp = (vec_alias_ >= val);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator==(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ == source);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator!=(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ != source);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator<(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ < source);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator<=(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ <= source);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator>(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ > source);
		
	return tmp;
}

template<class T> inline VectorMask VectorRangeSlice<T>::operator>=(const VectorRangeSlice<T>& source) const
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorMask tmp(rng_);
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	tmp = (vec_ >= source);
		
	return tmp;
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	vec_alias_ += val;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	vec_alias_ -= val;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
		
	vec_alias_ *= val;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator/=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
	
	vec_alias_ /= val;
	
	return *this;
}

// VectorRangeSlice member assign/equals operators
template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator+=(const VectorRangeSlice<T>& source)
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	vec_alias_ += source;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator-=(const VectorRangeSlice<T>& source)
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	vec_alias_ -= source;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator*=(const VectorRangeSlice<T>& source)
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	vec_alias_ *= source;
	
	return *this;
}

template<class T> inline VectorRangeSlice<T>& VectorRangeSlice<T>::operator/=(const VectorRangeSlice<T>& source)
{
	if (source.dim() != rng_.dim()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	const typename Vector<T>::Offset	offs(vec_alias_,source.offset());
	
	vec_alias_ /= source;
	
	return *this;
}

} // namespace mcs

#include "MCS_VectorRangeSliceFunctions.h"
#include "MCS_VectorRangeSliceOperators.h"
#include "MCS_VectorRangeSliceStatistics.h"

#endif // MCS_VECTORRANGESLICE_H