#ifndef MCS_VECTORSTATISTICS_H
#define MCS_VECTORSTATISTICS_H

#include "MCS_Promote.h"
#include "MCS_Vector.h"

namespace mcs {

namespace statistics {

template<class T> T 			max(const Vector<T>& source);
template<class T> Vector<T> 	max(const Vector<T>& lhs,const T& rhs);
template<class T> Vector<T> 	max(const T& lhs,const Vector<T>& rhs);
template<class T> Vector<T> 	max(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> T				min(const Vector<T>& source);
template<class T> Vector<T> 	min(const Vector<T>& lhs,const T& rhs);
template<class T> Vector<T> 	min(const T& lhs,const Vector<T>& rhs);
template<class T> Vector<T> 	min(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> T				product(const Vector<T>& source);
template<class T> T				sum(const Vector<T>& source); 		

template<class T> typename PromoteFloat<T>::promote_t	mean(const Vector<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const Vector<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const Vector<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const Vector<T>& source,const Vector<T>& weight);

template<class T> Vector<typename PromoteFloat<T>::promote_t>	moments(const Vector<T>& source,Int32 num_moments);

// min/max
template<class T> T max(const Vector<T>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T> Vector<T> max(const Vector<T>& lhs,const T& rhs)
{
	Vector<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs);
	
	return tmp;
}

template<class T> Vector<T> max(const T& lhs,const Vector<T>& rhs)
{
	Vector<T> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(rhs[i],lhs);
	
	return tmp;
}

template<class T> Vector<T> max(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Vector<T>::RangeErr();
	
	Vector<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs[i]);
	
	return tmp;
}

template<class T> T min(const Vector<T>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T> Vector<T> min(const Vector<T>& lhs,const T& rhs)
{
	Vector<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs);
	
	return tmp;
}

template<class T> Vector<T> min(const T& lhs,const Vector<T>& rhs)
{
	Vector<T> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs,rhs[i]);
	
	return tmp;
}

template<class T> Vector<T> min(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Vector<T>::RangeErr();
		
	Vector<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs[i]);
	
	return tmp;
}

// product of all elements
template<class T> T product(const Vector<T>& source)
{
	return mcs::prod(source);
}

// sum of all elements
template<class T> T sum(const Vector<T>& source)
{
	return mcs::sum(source);
}

// other statistics
template<class T> typename PromoteFloat<T>::promote_t mean(const Vector<T>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
			
	return tsum/source.dim();
}

// standard deviation
template<class T> typename PromoteFloat<T>::promote_t standardDeviation(const Vector<T>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T> typename PromoteFloat<T>::promote_t variance(const Vector<T>& source)
{ 
	if (source.dim() < 2) throw Vector<T>::IllegalOperationErr();
		
	typename PromoteFloat<T>::promote_t xbar = mean(source),
										sum1(0),
										sum2(0);
	
	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t	tmp(source[i]-xbar);
		
		sum2 += tmp*tmp;
		sum1 += tmp;
	}
		
	return (sum2 - sum1*sum1/source.dim())/(source.dim()-1); 
}

template<class T> typename PromoteFloat<T>::promote_t weightedMean(const Vector<T>& source,const Vector<T>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		tsum += source[i]*weight[i];
		norm += weight[i];
	}
				
	return tsum/norm;
}
/*
template<class T> Vector<typename PromoteFloat<T>::promote_t> moments(const Vector<T>& source,Int32 num_moments)
{
	Vector<typename PromoteFloat<T>::promote_t>	mom(IndexRange<1>(num_moments));

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 m=mom.first();m<=mom.last();m++)
			mom[m] += std::pow(source[i],m);
			
	return mom;
}
*/
namespace ignoreNaN {

template<class T> T 			max(const Vector<T>& source);
template<class T> Vector<T> 	max(const Vector<T>& lhs,const T& rhs);
template<class T> Vector<T> 	max(const T& lhs,const Vector<T>& rhs);
template<class T> Vector<T> 	max(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> T 			min(const Vector<T>& source);
template<class T> Vector<T> 	min(const Vector<T>& lhs,const T& rhs);
template<class T> Vector<T> 	min(const T& lhs,const Vector<T>& rhs);
template<class T> Vector<T> 	min(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> T				product(const Vector<T>& source);
template<class T> T 			sum(const Vector<T>& source); 
		
template<class T> typename PromoteFloat<T>::promote_t	mean(const Vector<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const Vector<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const Vector<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const Vector<T>& source,const Vector<T>& weight);

template<class T> Vector<typename PromoteFloat<T>::promote_t>	moments(const Vector<T>& source,Int32 num_moments);

// min/max
template<class T> T max(const Vector<T>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T> Vector<T> max(const Vector<T>& lhs,const T& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> Vector<T> max(const T& lhs,const Vector<T>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> Vector<T> max(const Vector<T>& lhs,const Vector<T>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> T min(const Vector<T>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T> Vector<T> min(const Vector<T>& lhs,const T& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T> Vector<T> min(const T& lhs,const Vector<T>& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T> Vector<T> min(const Vector<T>& lhs,const Vector<T>& rhs)
{
	return normal::min(lhs,rhs);
}

// product of all elements
template<class T> T product(const Vector<T>& source)
{
	T tprod(1);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tprod *= source[i];
				
	return tprod;
}

// sum of all elements
template<class T> T sum(const Vector<T>& source)
{
	T tsum(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tsum += source[i];
				
	return tsum;
}

// other statistics
template<class T> typename PromoteFloat<T>::promote_t mean(const Vector<T>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		if (isfinite(source[i]))
		{
			tsum += source[i];
			norm += 1;
		}
	}
		
	return tsum/norm;
}

// standard deviation
template<class T> typename PromoteFloat<T>::promote_t standardDeviation(const Vector<T>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T> typename PromoteFloat<T>::promote_t variance(const Vector<T>& source)
{ 
	if (source.dim() < 2) throw Vector<T>::IllegalOperationErr();
		
	typename PromoteFloat<T>::promote_t xbar(mean(source)),
										sum1(0),
										sum2(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t tmp(source[i]-xbar);
		
		if (isfinite(tmp))
		{
			sum2 += tmp*tmp;
			sum1 += tmp;
			norm += 1;
		}
	}
	
	return (sum2 - sum1*sum1/norm)/(norm-1); 
}

template<class T> typename PromoteFloat<T>::promote_t weightedMean(const Vector<T>& source,const Vector<T>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i]) && isfinite(weight[i]))
		{
			tsum += source[i]*weight[i];
			norm += weight[i];
		}

	return tsum/norm;
}
/*
template<class T> Vector<typename PromoteFloat<T>::promote_t> moments(const Vector<T>& source,Int32 num_moments)
{
	Vector<typename PromoteFloat<T>::promote_t>	mom(IndexRange<1>(num_moments));

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 m=mom.first();m<=mom.last();m++)
			if (isfinite(source[i])) 
				mom[m] += std::pow(source[i],m);
			
	return mom;
}
*/
} // namespace ignoreNaN

} // namespace statistics

} // namespace mcs

#ifdef __INTEL__
	#if USE_INTEL_SPL
		#include "MCS_VectorStatistics_IntelSpecialized.h"
	#endif
#endif

#endif // MCS_VECTORSTATISTICS_H
