#ifndef MCS_VECTORUTILITIES_H
#define MCS_VECTORUTILITIES_H

#include "MCS_Vector.h"
#include "MCS_VectorMask.h"

namespace mcs {

// miscellaneous utility functions
template<class T> const T& nthValue(const Vector<T>& source);
template<class T> const T& firstValue(const Vector<T>& source);
template<class T> const T& lastValue(const Vector<T>& source);

template<class T,class F> Vector<T> apply(const F& func,const Vector<T>& source);
template<class T,class F> Vector<T> apply(const F& func,const Vector<T>& vec1,const Vector<T>& vec2);

template<class T> VectorIndirection find(const Vector<T>& source);

template<class T> bool isMonotonic(const Vector<T>& source);
template<class T> bool isMonotonicallyIncreasing(const Vector<T>& source);
template<class T> bool isMonotonicallyDecreasing(const Vector<T>& source);

template<class T> VectorMask isFinite(const Vector<T>& source);
template<class T> VectorMask isNaN(const Vector<T>& source);

template<class T> const Vector<T>& append(Vector<T>& source,const Vector<T>& add);
template<class T> const Vector<T>& normalize(Vector<T>& source,const T& normalization);
template<class T> const Vector<T>& normalize(Vector<T>& source,const Vector<T>& normalization);
template<class T> const Vector<T>& remove(Vector<T>& source,const VectorMask& mask);
template<class T> const Vector<T>& reorder(Vector<T>& source,const VectorIndirection& ind);
template<class T> const Vector<T>& reverse(Vector<T>& source);

template<class T> Vector<T> concatenate(const Vector<T>& source,const Vector<T>& add);
template<class T> Vector<T> difference(const Vector<T>& source);
template<class T> Vector<T> sample(const Vector<T>& source,Int32 scal);

//template<class T> Matrix<T> outer(const Vector<T>& lhs,const Vector<T>& rhs);

// miscellaneous operators
template<class T> inline const T& nthValue(const Vector<T>& source,Int32 n)
{
	return source[n-1+source.first()];
}

template<class T> inline const T& firstValue(const Vector<T>& source)
{
	return nthValue(source,1);
}

template<class T> inline const T& lastValue(const Vector<T>& source)
{ 
	return nthValue(source,source.dim());
}

template<class T,class F> Vector<T> apply(const F& func,const Vector<T>& source)
{
	Vector<T>	tmp(source.range());
	
	for (Int32 i=source.first();i<=source.last();i++)
		tmp[i] = func(source[i]);
		
	return tmp;
}

template<class T,class F> Vector<T> apply(const F& func,const Vector<T>& vec1,const Vector<T>& vec2)
{
	if (vec1.range() != vec2.range()) throw Vector<T>::RangeErr();
	
	Vector<T>	tmp(vec1.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = func(vec1[i],vec2[i]);
		
	return tmp;
}

template<class T> VectorIndirection find(const Vector<T>& source)
{
	Int32	num_elem = 0;
	
	for (Int32 i=source.first();i<=source.last();i++)
		if (source[i]) num_elem++;
		
	if (num_elem == 0) return VectorIndirection();
	
	VectorIndirection	ind(IndexRange<1>(0,num_elem-1));
	
	for (Int32 i=source.first(),j=ind.first();i<=source.last();i++)
		if (source[i]) ind[j++] = i;
	
	return ind;
}

// check for monotonicity
template<class T> inline bool isMonotonic(const Vector<T>& source)
{
	const VectorMask	dx(difference(source)>T(0));
	const Uint32		numTrue = dx.trueElements();
	
	if (numTrue == dx.dim() || numTrue == 0)
		return true;
	else
		return false;
}

// check that input data are monotonically increasing
template<class T> inline bool isMonotonicallyIncreasing(const Vector<T>& source)
{
	return (!isMonotonic(source) || (nthValue(source,2)-nthValue(source,1)) > T(0));
}

// check that input data are monotonically decreasing
template<class T> inline bool isMonotonicallyDecreasing(const Vector<T>& source)
{
	return (!isMonotonic(source) || (nthValue(source,2)-nthValue(source,1)) < T(0));
}

// check for finite values
template<class T> inline VectorMask isFinite(const Vector<T>& source)
{
	return VectorMask(source.range(),true);
}

template<> inline VectorMask isFinite(const Vector<Float32>& source)
{
	VectorMask	mask(source.range(),false);
	
	for (Int32 i=source.first();i<=source.last();++i)
		if (isfinite(source[i])) mask[i] = true;
		
	return mask;
}

template<> inline VectorMask isFinite(const Vector<Float64>& source)
{
	VectorMask	mask(source.range(),false);
	
	for (Int32 i=source.first();i<=source.last();++i)
		if (isfinite(source[i])) mask[i] = true;
		
	return mask;
}

// check for NaN values
template<class T> inline VectorMask isNaN(const Vector<T>& source)
{
	return VectorMask(source.range(),false);
}

template<> inline VectorMask isNaN(const Vector<Float32>& source)
{
	VectorMask	mask(source.range(),false);
	
	for (Int32 i=source.first();i<=source.last();++i)
		if (!isfinite(source[i])) mask[i] = true;
		
	return mask;
}

template<> inline VectorMask isNaN(const Vector<Float64>& source)
{
	VectorMask	mask(source.range(),false);
	
	for (Int32 i=source.first();i<=source.last();++i)
		if (!isfinite(source[i])) mask[i] = true;
		
	return mask;
}

// append
template<class T> const Vector<T>& append(Vector<T>& source,const Vector<T>& add)
{
	if (!source.isValid()) 
	{
		source = add;
		return source;
	}
	
	const IndexRange<1>	rng1(source.range()),
						rng2(source.last()+1,source.last()+add.dim());
						
	Vector<T> tmp(IndexRange<1>(source.first(),source.first()+source.dim()-1+add.dim()));
	
	tmp[rng1] = source;
	tmp[rng2] = add;
	
	source = tmp;
	
	return source;
}	

// normalize vector
template<class T> inline const Vector<T>& normalize(Vector<T>& source,const T& normalization)
{
	return (source /= normalization);
}

template<class T> inline const Vector<T>& normalize(Vector<T>& source,const Vector<T>& normalization)
{
	return (source /= normalization);
}

// remove elements
template<class T> const Vector<T>& remove(Vector<T>& source,const VectorMask& mask)
{
	if (mask.range() != source.range()) throw Vector<T>::SizeMismatchErr();
	
	Int32	elem = 0;
	
	for (Int32 i=mask.first();i<=mask.last();i++)
		if (mask[i] == false) elem++;
		
	Vector<T>	tmp(IndexRange<1>(source.first(),source.first()+elem-1));
	
	for (Int32 i=mask.first(),ii=tmp.first();i<=mask.last();i++)
		if (mask[i] == false)
			tmp[ii++] = source[i];
		
	return (source = tmp);
}

// reorder elements
template<class T> const Vector<T>& reorder(Vector<T>& source,const VectorIndirection& ind)
{
	const Vector<T>	tmp(source[ind]);
	
	return (source = tmp);
}

// reverse vector
template<class T> const Vector<T>& reverse(Vector<T>& source)
{ 
	for (Int32 i=source.first(),j=source.last();i<=source.first()+(source.dim()-2)/2;swap(source[i++],source[j--])) { };
	
	return source;
}

// concatenate two vectors
template<class T> Vector<T> concatenate(const Vector<T>& source,const Vector<T>& add)
{
	if (!source.isValid()) return add;
	
	const IndexRange<1>	rng1(source.range()),
						rng2(source.last()+1,source.last()+add.dim());
						
	Vector<T> tmp(IndexRange<1>(source.first(),source.first()+source.dim()-1+add.dim()));
	
	tmp[rng1] = source;
	tmp[rng2] = add;
	
	return tmp;
}	

// first difference
template<class T> inline Vector<T> difference(const Vector<T>& source)
{
	Vector<T>	tmp(IndexRange<1>(source.first(),source.last()-1));
	
	for (Int32 i=source.first();i<=source.last()-1;++i)
		tmp[i] = source[i+1]-source[i];
		
	return tmp;
}

// scale vector
/*
template<class T> inline Vector<T> sample(const Vector<T>& source,Int32 scal)
{
	if (scal <= 0 || source.dim() % scal != 0) throw Exception::IllegalOperationErr();
	
	Int32 new_dim = source.dim()/scal;
	
	Vector<T> tmp(v_range(source.first(),source.first()+new_dim-1));
	
	if (opf.ignoreNaN())
	{
		for (Int32 i=tmp.first();i<=tmp.last();i++)
		{
			Int32 	idx_lo = source.first() + (i-tmp.first())*scal,
					idx_hi = idx_lo + scal - 1,
					norm = 0;
				
			for (Int32 j=idx_lo;j<=idx_hi;j++)
				if (is_finite(source[j]))
				{
					tmp[i] += source[j];
					norm++;
				}
				
			tmp[i] /= norm;
		}
	}
	else
	{
		for (Int32 i=tmp.first();i<=tmp.last();i++)
		{
			Int32 	idx_lo = source.first() + (i-tmp.first())*scal,
					idx_hi = idx_lo + scal - 1;
				
			for (Int32 j=idx_lo;j<=idx_hi;j++)
				tmp[i] += source[j];
		}
	}	
	
	return tmp/T(scal);
}
*/
/*
template<class T> Matrix<T> outer(const Vector<T>& lhs,const Vector<T>& rhs)
{
	Matrix<T> tmp(m_range(lhs.first(),lhs.last(),rhs.first(),rhs.last()));
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		for (Int32 j=rhs.first();j<=rhs.last();j++)
			tmp[i][j] = lhs[i]*rhs[j];
			
	return tmp;
}
*/

} // namespace mcs

#endif // MCS_VECTORUTILITIES_H