$freeform
!@***m* MLT_RSS/AMSU_L1B_to_L2A.f90
!  NAME
!    AMSU_L1B_to_L2A
! 
!  PURPOSE
!    Module for converting L1B data to L2A data
! 
!  DESCRIPTION
!    Module for converting L1B data to L2A data.  This involves converting the raw counts to
!    brightness temperatures, as well as calculating the diurnal and Earth incidence angle corrections.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_L1B_to_L2A
!@*****
module AMSU_L1B_to_L2A
	use NAN_support
	use date_and_time
	use interpolate
	use planck
	use AMSU_Constants
	use AMSU_Types
	use AMSU_inventory
	use AMSU_equator_crossing_times
	use AMSU_File_Locations
	use AMSU_Level_1B
	use AMSU_Level_2A
	use AMSU_SAA
	use AMSU_nadir
	use AMSU_diurnal
	use AMSU_Quality

	implicit none


	! header data list declarations

	type(AMSU_A_Data_to_Keep_from_Header),dimension(20)			:: AMSU_header_data_list
	integer(4)													:: num_in_header_data_list

	character(len = 120),parameter		:: LOG_FILE_PATH = '\\dell-p1700d\j\AMSU-A_L2A\L1B_to_L2A_logs\'


contains

	subroutine convert_L1B_orbits_to_L2A_orbits(sat_num,channel,num_orbits,orbit_list)
!@***s* MLT_RSS/AMSU_L1B_to_L2A.f90/convert_L1B_orbits_to_L2A_orbits
!  NAME
!    convert_L1B_orbits_to_L2A_orbits
! 
!  PURPOSE
!    converts a number of L1B_orbits to L2A_orbits
! 
!  INPUTS
!    integer(4)				:: sat_num  ! AMSU satellite number (NOAA-15 = 10, etc)
!    integer(4)             :: channel         ! AMSU channel
!    integer(4)			:: num_orbits      ! number of orbits to process
!	 integer(4),dimension(MAX_ORBITS_TO_PROCESS)			:: orbit_list  ! list of orbit number to process
!  OUTPUTS
!     integer(4)                    :: error             ! error flag,  0 means everyting OK
!  USAGE
!     call convert_L1B_orbits_to_L2A_orbits(sat_num,channel,num_orbits,orbit_list)
!@*****
		integer(4)			:: sat_num
		integer(4)			:: channel
		integer(4)			:: num_orbits
		integer(4),dimension(MAX_ORBITS_TO_PROCESS)			:: orbit_list

		! errors

		integer(4)			:: error
		integer(4)			:: read_error
		integer(4)			:: convert_error
		integer(4)			:: write_error

		! inventory file stuff

		type(amsu_inventory_str)									:: L1B_inventory
		type(amsu_inventory_str)									:: L2A_inventory

		! orbit declarations

		integer(4)													:: orbit_index
		integer(4)													:: orbit_num
		type(amsu_L1B_orbit_data)									:: L1B_orbit
		type(amsu_L1B_orbit_header)									:: L1B_orbit_header

		type(amsu_L2A_orbit_data)									:: L2A_orbit
		type(amsu_L2A_orbit_header)									:: L2A_orbit_header

		character(len = 7)											:: sat_name

		! log file stuff

		type(date_str)												:: cur_date,temp_date
		character(len = 20)											:: cur_date_String
		character(len = 120)										:: log_file


		if ((channel .ge. 3) .and. (channel .le. 14)) then
			call read_amsu_ccm3_diurnal(channel,error)
		endif


		! open the log files

		cur_date		= current_date(error)
		cur_date_string = date_string_for_file(cur_date)

		log_file = trim(LOG_FILE_PATH)//'L2A_log_file_'//trim(cur_date_String)//'.txt'
		open(unit=LOG_unit,file = log_file)


		sat_name = amsu_sat_names_2(sat_num)

		! read in L1B inventory file

		call read_amsu_l1b_inventory_file(sat_num,L1B_inventory,read_error)

		! read in any existing L2A inventory file

		call read_amsu_L2A_inventory_file_text(sat_num,channel,L2A_inventory,read_error)

		! read in header list file

		call read_AMSU_Header_Data_List(sat_name,num_in_header_data_list,AMSU_header_data_list)

		! loop through orbit numbers

		do orbit_index = 1,num_orbits
			orbit_num = orbit_list(orbit_index)			
			if (L1B_inventory%orbit_valid(orbit_num)) then
				
				call read_AMSU_L1B_orbit_file(	sat_name,					&
												orbit_num,					&
												L1B_orbit_header,			&
												L1B_orbit,					&				
												error)
				if (error .ne. 0) cycle
				temp_date = set_date_2000(L1B_orbit_header%start_rd_2000,error)
				if (error .ne. 0) cycle




				call convert_L1B_orbit_to_L2A_orbit(L1B_orbit,				&
													L2A_orbit,				&
													channel,				&
													orbit_num,				&
													sat_name,				&
													convert_error) 

				if (convert_error .ge. 0) then

				    ! need to write some stuff to assign proper data to the L2A header structure
					
					call zero_amsu_L2A_orbit_file_header(L2A_orbit_header)
					L2A_orbit_header%eqx_Rd_2000 = L1B_orbit_header%eqx_Rd_2000 

					call write_AMSU_L2A_orbit_file(	amsu_Sat_names_2(sat_num),		&
													orbit_num,						& 
													channel,						&
													L2A_orbit_header,				&
													L2A_orbit,						&
													error)

					call update_AMSU_L2A_inventory(	L2A_inventory,			&
													L2A_orbit,				&
													sat_num,				&
													orbit_num,				&
													error)

				endif
			endif
		enddo

		call  write_AMSU_L2A_Inventory_file(sat_num,channel,L2A_inventory,write_error)
		call  write_AMSU_L2A_Inventory_file_text(sat_num,channel,L2A_inventory,write_error)

	end  subroutine convert_L1B_orbits_to_L2A_orbits



	subroutine convert_L1B_orbit_to_L2A_orbit(  L1B_orbit,				&
												L2A_orbit,				&
												channel,				&
												orbit_num,				&
												sat_name,				&
												convert_error) 

!@***s* MLT_RSS/AMSU_L1B_to_L2A.f90/convert_L1B_orbit_to_L2A_orbit
!  NAME
!    convert_L1B_orbit_to_L2A_orbit
! 
!  PURPOSE
!    converts a L1B orbit to a L2A orbit.
!  DESCRIPTION
!    converts a L1B orbit to a L2A orbit. Calculates brightness temperatures and diurnal and 
!    incidence angle corrections.  compares brightness temperature to an expected range to remove outliers.
!  INPUTS
!		type(AMSU_L1B_orbit_data)							:: L1B_orbit
!		type(AMSU_L2A_orbit_data)							:: L2A_orbit
!		integer(4)											:: channel
!		integer(4)											:: orbit_num
!		character(len = 7)									:: sat_name
!  OUTPUTS
!     integer(4)                    :: convert_error             ! error flag,  0 means everyting OK
!  USAGE
!     call convert_L1B_orbit_to_L2A_orbit(L1B_orbit,L2A_orbit,channel,orbit_num,sat_name,convert_error) 
!@*****
		type(AMSU_L1B_orbit_data)							:: L1B_orbit
		type(AMSU_L2A_orbit_data)							:: L2A_orbit
		type(AMSU_L2A_orbit_data)							:: L2A_orbit2
		integer(4)											:: channel
		integer(4)											:: orbit_num
		character(len = 7)									:: sat_name

		integer(4)											:: convert_Error

		integer(4)											:: scan_num
		integer(4)											:: error
		type(AMSU_A_Data_to_Keep_from_Header)				:: Calibration_Data
		type(AMSU_A_L1B_Calibration_Data)					:: decoded_calibration_data


		integer(4)												:: cold_counts1
		integer(4)												:: cold_counts2
		integer(4)												:: warm_counts1
		integer(4)												:: warm_counts2

		real(4)													:: a0,a1,a2
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: warm_counts
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: cold_counts
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: warm_counts_smoothed
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: cold_counts_smoothed

		real(4)												:: tot_cold_counts
		real(4)												:: tot_warm_counts
		real(4)												:: tot_cold_wt
		real(4)												:: tot_warm_wt
		integer(4)											:: smooth_half_width
		integer(4)											:: j
		real(4)												:: wt

		real(4)												:: nadir_lon
		real(4)												:: nadir_lat
		real(4)												:: altitude
		real(8)												:: rd_2000

		real(4),dimension(3)					:: calibration_coeff1
		real(4),dimension(3)					:: calibration_coeff2





		convert_Error = 0

		warm_counts = 0
		cold_counts = 0

		! get cold counts and warm counts and smooth them for use in the "full" calibration

		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			! get warm and cold counts
			call get_info_from_L1B_scan(L1B_orbit%SCAN_DATA(scan_num),	&
										channel,						&
										rd_2000,						&
										nadir_lat,						&
										nadir_lon,						&
										altitude,						&
										cold_counts1,					&
										cold_counts2,					&
										warm_counts1,					&
										warm_counts2,					&
										error)

			if ((abs(cold_counts1 - cold_counts2) .lt. 10) .and. (cold_counts1 .lt. 16000))then
				cold_counts(scan_num) = 0.5*(cold_counts1+cold_counts2)
			else
				cold_counts(scan_num) = cold_counts(scan_num-1)
			endif

			if (abs(warm_counts1 - warm_counts2) .lt. 10) then
				warm_counts(scan_num) = 0.5*(warm_counts1+warm_counts2)
			else
				warm_counts(scan_num) = warm_counts(scan_num-1)
			endif
		enddo

		smooth_half_width = 3
		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			! perform the smooth
			tot_cold_wt		= 0.0
			tot_warm_wt		= 0.0
			tot_cold_counts	= 0.0
			tot_warm_counts	= 0.0
			do j = -smooth_half_width,smooth_half_width
				if ((j+scan_num .ge. 1) .and. (j+scan_num .le. AMSU_MAX_SCANS_PER_ORBIT)) then 
					if (cold_counts(j+scan_num) .gt. 1000) then
						wt = 1.0 -abs(j)/(smooth_half_width+1.0)
						tot_cold_counts = tot_cold_counts + wt*cold_counts(j+scan_num)
						tot_cold_wt = tot_cold_wt + wt
					endif
					if (warm_counts(j+scan_num) .gt. 1000) then
						wt = 1.0 -abs(j)/(smooth_half_width+1.0)
						tot_warm_counts = tot_warm_counts + wt*warm_counts(j+scan_num)
						tot_warm_wt = tot_warm_wt + wt
					endif
				endif
			enddo
			if (tot_cold_wt .ge. 0.99) then
				cold_counts_smoothed(scan_num) = tot_cold_counts/tot_cold_wt
			else
				cold_counts_smoothed(scan_num) = cold_counts(scan_num)
			endif

			if (tot_warm_wt .ge. 0.99) then
				warm_counts_smoothed(scan_num) = tot_warm_counts/tot_warm_wt
			else
				warm_counts_smoothed(scan_num) = warm_counts(scan_num)
			endif
		enddo

		! set all scans to not valid

		L2A_orbit%scan_valid = 0

		! zero the L2A_scan_data

		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			call zero_amsu_L2A_scan(L2A_orbit%scan_data(scan_num))
			L2A_orbit%scan_valid(scan_num) = 0  
		enddo

		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			!select proper header data to use for calibrating this scan

			if ((L1B_orbit%header_num_used(scan_num) .gt. 0) .and. &
			   	(L1B_orbit%header_num_used(scan_num) .le. num_in_header_data_list)) then
					Calibration_Data = AMSU_Header_Data_List(L1B_orbit%header_num_used(scan_num))
			else
				cycle
			endif


			!call decode_calibration_data(calibration_data%calibration,decoded_calibration_data)

			! perform scan calibration

			if (L1B_orbit%scan_valid(scan_num) .gt. 0) then
                call convert_L1B_scan_to_L2A_scan(L1B_orbit%scan_data(scan_num), &
												  L2A_orbit%scan_data(scan_num), &
												  channel,						 &
												  orbit_num,					 &
												  scan_num,						 &
												  sat_name,						 &
												  Calibration_data,				 &
												  calibration_coeff1,			 &
												  error)
				

				if (error .ge. 0) then
					L2A_orbit%scan_valid(scan_num) = 2
				endif
				
				if ((channel .ge. 3) .and. (channel .le. 9)) then 
					call calculate_nadir_correction(  L2A_orbit%scan_data(scan_num),	&
													  channel,							&
													  error)
				endif
				if ((channel .ge. 3) .and. (channel .le. 9)) then 
					call calculate_diurnal_correction(L2A_orbit%scan_data(scan_num),	&
													  channel,							&
													  error)
				endif
			endif
		enddo
	end subroutine convert_L1B_orbit_to_L2A_orbit

	subroutine convert_L1B_scan_to_L2A_scan(L1B_scan,						&
											L2A_scan,						&
											channel,						&
											orbit_num,						&
											scan_num,						&
											sat_name,						&
											calibration_data,				&
											calibration_coeff,				&
											error)
!@***s* MLT_RSS/AMSU_L1B_to_L2A.f90/convert_L1B_scan_to_L2A_scan
!  NAME
!    convert_L1B_scan_to_L2A_scan
! 
!  PURPOSE
!    converts a L1B scan to a L2A scan.
!  DESCRIPTION
!    converts a L1B scan to a L2A scan. Calculates thermistor temperatures and brightness temperatures from counts
!    using NOAA operation calibration.
!  INPUTS
!		type(AMSU_A_L1B_Swath_Data)				:: L1B_scan        ! AMSU L1B scan
!		integer(4)								:: channel         ! AMSU channel
!		integer(4)								:: orbit_num       ! orbit number
!		integer(4)								:: scan_num        ! scan number
!		character(len = 7)						:: sat_name        ! satellite name
!		type(AMSU_A_Data_to_Keep_from_Header)	:: Calibration_Data  ! calibration data
!  OUTPUTS
!     type(AMSU_A_L2A_Swath_Data)				:: L2A_scan        ! AMSU L2A scan
!     integer(4)                                :: error           ! error flag,  0 means everyting OK
!  USAGE
!     call convert_L1B_scan_to_L2A_scan(L1B_scan,L2A_scan,channel,orbit_num,scan_num,sat_name,calibration_data,calibration_coeff,error)
!@*****
		use type_conversion

		type(AMSU_A_L1B_Swath_Data)				:: L1B_scan
		type(AMSU_A_L2A_Swath_Data)				:: L2A_scan
		integer(4)								:: channel
		integer(4)								:: orbit_num
		integer(4)								:: scan_num
		character(len = 7)						:: sat_name
		type(AMSU_A_Data_to_Keep_from_Header)	:: Calibration_Data
		integer(4)								:: Error

		type(date_str)							:: scan_date


		integer(4)								:: fov
		integer(4)								:: offset
		logical									:: A1_1
		logical									:: A1_2
		logical									:: A2

		real(4),dimension(3)					:: calibration_coeff
		integer(4),dimension(3)					:: calibration_coeff_int

		integer(4)								:: scene_counts

		real(4)									:: wavenumber
		real(4)									:: constant
		real(4)									:: slope
		real(4)									:: Tb_star


		
		real(4),dimension(30)					:: radiance

		integer(4)								:: date_error

		integer(4)								:: therm_num
		integer(4)								:: num_therms

		character(Len = 4)						:: coeff_str

		real(4),dimension(7,4)					:: warm_load_calibration_coeff
		integer(4),dimension(7)					:: warm_load_thermistor_counts
		real(4),dimension(7)					:: warm_load_thermistor_temperature
		real(4)									:: total_temperature
		integer(4)								:: num_temperature
		real(4)									:: mean_warm_load_temperature


		logical									:: scan_ok




		! transfer basic scan line info

		L2A_scan%channel_num		= channel
		L2A_scan%orbit_number		= orbit_num
		L2A_scan%scan_line_in_orbit = scan_num

		scan_date = set_date(  1.0d-3*L1B_scan%scan_line_utc_time_of_day,		&
								1*L1B_scan%scan_line_day_of_year,			&
								1*L1B_scan%scan_line_year,				&
								date_error)
								
		L2A_scan%scan_rd_2000 = scan_date%rd_2000
		L2A_scan%satellite_name = sat_name	

		if (btest(L1b_scan%scan_line_bit_field,15)) then
			L2A_scan%asc_desc = 2
		else
			L2A_scan%asc_desc = 1
		endif
			

			
		! extract earth location data

		do fov = 1,30
			L2A_scan%fov_latitude(fov)	= L1B_scan%earth_location(1,fov)*1.0e-4
			L2A_scan%fov_longitude(fov) = modulo(L1B_scan%earth_location(2,fov)*1.0e-4,360.0)
		enddo

		L2A_scan%spacecraft_altitude = L1B_scan%spacecraft_altitude*1.0e-1

		! find appropriate target temperature

		! get the warm load calibration coefficients for each of the
		! thermistors (5 for AMSU_A1-1 (channels 6,7,9-15) and 
		! AMSU_A1-2 (Channels 3,4,5, and 8)
		! and 7 for AMSU A2 (channels 1 and 2)

		! check quality flags

		L2A_scan%fov_quality = 0

		call evaluate_amsu_quality_indicator(L1B_Scan%quality_indicator_bit_field,scan_ok)

		if ( .not. (scan_ok)) then
			L2A_scan%fov_quality = -1
			return
		endif



		A1_1 = .false.
		A1_2 = .false.
		A2   = .false.
				
		if ((channel .gt. 2) .and. (channel .lt. 16)) then   ! This is AMSU A1
			select case (channel)
				Case (3:5)
					A1_2 = .true.
				Case (6:7) 
					A1_1 = .true.
				Case (8)			 
					A1_2 = .true.
				Case (9:15)
					A1_1 = .true.
				Case Default
			end select
		else if ((channel .eq. 1) .or. (channel .eq. 2))  then    ! AMSU_A2
			A2 = .true.
		else
			print *,'channel out of range',channel
			stop
		endif

		
		if ((A1_1) .or. (A1_2)) then
			
			! get appropriate calibration coefficients

			calibration_coeff_int = L1B_scan%primary_calibration(:,channel)
			calibration_coeff(1) = calibration_coeff_int(1)*1.0e-19 
			calibration_coeff(2) = calibration_coeff_int(2)*1.0e-13 
			calibration_coeff(3) = calibration_coeff_int(3)*1.0e-9 

			! calc radiance (in mW/(m^2-sr-cm^-1)
			!print *
			!print *,calibration_coeff(3),calibration_coeff(2),calibration_coeff(1)

			do fov = 1,30
				scene_counts = L1B_scan%A1_scene_telemetry(2+channel,fov)
				radiance(fov) = 1.0e-3*(calibration_coeff(3) + scene_counts*calibration_coeff(2) + &
								scene_counts*scene_counts*calibration_coeff(1))
			enddo

			! get temp to radiance data

			offset = 1+(channel-1)*12

			wavenumber	= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset:offset+3)))
			constant	= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+4:offset+7)))
			slope		= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+8:offset+11)))

			!calc Tbs from radiances

			do fov = 1,30
				Tb_star = radiance_to_Tb(radiance(fov),wavenumber)
				L2A_scan%Tbs(fov) = constant + Tb_star*slope
			
			enddo
		endif


		if (A2) then
			
			! get appropriate calibration coefficients

			calibration_coeff_int = L1B_scan%primary_calibration(:,channel)
			calibration_coeff(1) = calibration_coeff_int(1)*1.0e-19 
			calibration_coeff(2) = calibration_coeff_int(2)*1.0e-13 
			calibration_coeff(3) = calibration_coeff_int(3)*1.0e-9 

			! calc radiance (in mW/(m^2-sr-cm^-1)

			do fov = 1,30
				scene_counts = L1B_scan%A2_scene_telemetry(2+channel,fov)
				radiance(fov) = 1.0e-3*(calibration_coeff(3) + scene_counts*calibration_coeff(2) + &
								scene_counts*scene_counts*calibration_coeff(1))
			enddo

			! get temp to radiance data

			offset = 1+(channel-1)*12

			wavenumber	= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset:offset+3)))
			constant	= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+4:offset+7)))
			slope		= 1.0e-06*byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+8:offset+11)))

			!calc Tbs from radiances

			do fov = 1,30
				Tb_star = radiance_to_Tb(radiance(fov),wavenumber)
				L2A_scan%Tbs(fov) = constant + Tb_star*slope
			enddo
		endif
		! find appropriate target temperature

		! get the warm load calibration coefficients for each of the
		! thermistors (5 for AMSU_A1-1 (channels 6,7,9-15) and 
		! AMSU_A1-2 (Channels 3,4,5, and 8)
		! and 7 for AMSU A2 (channels 1 and 2)

		A1_1 = .false.
		A1_2 = .false.
		A2   = .false.
				
		if ((channel .gt. 2) .and. (channel .lt. 16)) then   ! This is AMSU A1

			num_therms = 5

			select case (channel)
				Case (3:5)
					offset = 640	! A1-2
					A1_2 = .true.
				Case (6:7) 
					offset = 560	! A1-1
					A1_1 = .true.
				Case (8)			 
					offset = 640	! A1-2
					A1_2 = .true.
				Case (9:15)
					offset = 560	! A1-1
					A1_1 = .true.
				Case Default
					offset = 560
			end select

			do therm_num = 1,5

				coeff_str = calibration_data%A1_Digital_Conv(offset+1:offset+4)
				warm_load_calibration_coeff(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

				coeff_str = calibration_data%A1_Digital_Conv(offset+5:offset+8)
				warm_load_calibration_coeff(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
			
				coeff_str = calibration_data%A1_Digital_Conv(offset+9:offset+12)
				warm_load_calibration_coeff(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
			
				coeff_str = calibration_data%A1_Digital_Conv(offset+13:offset+16)
				warm_load_calibration_coeff(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))

				offset = offset + 16

				if (offset .eq. 560) then	! A1-1
					warm_load_thermistor_counts(therm_num) = L1B_scan%A1_temperature_sensor_telemetry(35+therm_num)
				else						! A1-2
					warm_load_thermistor_counts(therm_num) = L1B_scan%A1_temperature_sensor_telemetry(40+therm_num)
				endif
			enddo
		else if ((channel .eq. 1) .or. (channel .eq. 2))  then    ! AMSU_A2

			offset = 212
			offset = 192
			num_therms = 7
			A2 = .true.

			do therm_num = 1,num_therms  ! for AMSU A2, the center therm comes first, and the 
								         ! subsequent thermistor numbers are offset by 1 in our
							           	 ! numbering relative to the NOAA numbering scheme

				coeff_str = calibration_data%A2_Digital_Conv(offset+1:offset+4)
				warm_load_calibration_coeff(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

				coeff_str = calibration_data%A2_Digital_Conv(offset+5:offset+8)
				warm_load_calibration_coeff(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
			
				coeff_str = calibration_data%A2_Digital_Conv(offset+9:offset+12)
				warm_load_calibration_coeff(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
			
				coeff_str = calibration_data%A2_Digital_Conv(offset+13:offset+16)
				warm_load_calibration_coeff(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))

				offset = offset + 16

				warm_load_thermistor_counts(therm_num) = L1B_Scan%A2_temperature_sensor_telemetry(12+therm_num)
			enddo
		else
			print *,'channel out of range',channel
			stop

		endif

		! calculate thermistor temperature for each thermistor

		do therm_num = 1,num_therms	
			warm_load_thermistor_temperature(therm_num) = warm_load_calibration_coeff(therm_num,1) +					  &
				(warm_load_thermistor_counts(therm_num) * (warm_load_calibration_coeff(therm_num,2) + & 
				(warm_load_thermistor_counts(therm_num) * (warm_load_calibration_coeff(therm_num,3) + & 
				(warm_load_thermistor_counts(therm_num) * (warm_load_calibration_coeff(therm_num,4)))))))
		enddo

		! calculate mean thermistor temperature

		total_temperature = 0.0
		num_temperature = 0

		do therm_num = 1,num_therms
			total_temperature = total_temperature + warm_load_thermistor_temperature(therm_num)
			num_temperature = num_temperature + 1
		enddo

		mean_warm_load_temperature = total_temperature/num_temperature

		L2A_scan%warm_load_temp = mean_warm_load_temperature

		call check_tbs_against_bounds(L2A_scan%Tbs,channel,L2A_scan%fov_quality)


	end subroutine convert_L1B_scan_to_L2A_scan 
end module AMSU_L1B_to_L2A