module AMSU_L2C_Update_Monthlies_v3_2


use AMSU_constants
use AMSU_TYPES
use date_and_time
use NAN_support
use AMSU_zonal
use topography
use portlib
use AMSU_Level_2A
use AMSU_Inventory
use AMSU_File_Locations
use AMSU_quality
use AMSU_diurnal_CCM3




implicit none

integer(4),parameter	:: GRPT_TB_REC_LENGTH     = 4 + (7*2*144*72*4)

contains

subroutine update_all_amsu_L2C_monthlies_v3_2(sat_num,channel,first_date_to_process,last_date_to_process,view_str,error)

!@***s* MLT_RSS/update_all_amsu_L2C_monthlies_v3_2.f90
!  NAME
!    update_all_amsu_L2C_monthlies_v3_2
!  PURPOSE
!    Calculates Monthly AMSU L2C data, stores to file
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel		amsu channel number (5,7, or 9)
!   first_date_to_process	Date structure containing first dat to process
!   last_date_to_process	Date structure containing the last date to process
!   view_str                string containing FOV subset to use 
!   error					:: = 0 if everything is OK, no zero otherwise
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call update_all_amsu_L2C_monthlies_v3_2_CD(sat_num,					&
!								channel,					&
!								first_date_to_process,		&
!								last_date_to_process,		&
!								view_str,					&
!								diurnal_str,				&
!								error)
!
!@*****
	integer(4)			:: sat_num
	integer(4)			:: channel
	type(date_str)			:: first_date_to_process
	type(date_str)			:: last_date_to_process
	character(len=3)		:: view_str
	character(len=4)		:: diurnal_str
	integer(4)				:: error

    diurnal_str = 'CCM3'

!   This procedure updates the zonal month and daily grid point files for a given satellite and channel
	
!   Parameters
!
!	sat num					:: satellite number 10 = NOAA-15, 2 = NOAA-16, etc
!   channel					:: amsu channel number (5,7, or 9)
!   first_date_to_process	:: Date structure containing first dat to process
!   last_date_to_process	:: Date structure containing the last date to process
!   view_str                :: string containing FOV subset to use
!   error					:: = 0 if everything is OK, no zero otherwise
!
!	Input Files:
!		This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!		An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!		orbit and inventory files is contained in the AMSU_File_Locations module.

!	Output Files:
!		months:
!			month Files 
!
!


	real(4),dimension(NUM_AMSU_FOVS)	:: Tbs
	
	integer(4)							:: start_month
	logical								:: start_found
	integer(4)							:: inv_rec_num
	type(date_str)						:: strt_date
	type(date_str)						::  rec_date
	integer(4)							::  rec_month
	integer(4)							::  strt_orbit
	integer(4)							::  last_orbit
	integer(4)							::  last_month
	integer(4)							::  last_valid_rec
	logical							    ::  last_found
	integer(4)							::  date_Error
	
	type(amsu_inventory_str)			:: inv			


	! declaration of input data structures
	
	type(AMSU_L2A_orbit_header)							:: L2A_orbit_header
	type(AMSU_L2A_orbit_data)							:: L2A_orbit

	type(date_str)				 :: swath_date

	integer(4)					 :: orbit_num
	character(len = 7)           :: sat_name
	integer(4)					 :: n_swath
	integer(4)					 :: orbit_error

	real(4)						 :: remainder

	real(4)							  :: target_temp
	real(4),dimension(NUM_AMSU_FOVS)  :: longitude
	real(4),dimension(NUM_AMSU_FOVS)  :: latitude
	integer(1),dimension(NUM_AMSU_FOVS)  :: fov_qual

	real(4),dimension(NUM_AMSU_FOVS)  :: diurnal_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: nadir_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: incidence_correction
	real(4),parameter				  :: precip_thres = 1.5
	real(4)							  :: altitude
	real(4),parameter				  :: altitude_thres_for_zonal = 1500.0

	type(Date_Str)					  :: curr_Date

	! declaration of gridpoint structures

	real(4),dimension(7,2,144,72)	    :: hist	 ! (num/tot_tb/tot_diur/tot_inc/tot_limb/tot_tar/tot_samp,asc/desc,,ilon,ilat)


	integer(4)			:: ilon
	integer(4)			:: ilat

	! declaration of zonal average structures

	type(amsu_zonal_ave_rec)			:: zonal_tbs
	type(amsu_zonal_ave_rec)			:: zonal_diur
	type(amsu_zonal_ave_rec)			:: zonal_inc
	type(amsu_zonal_ave_rec)			:: zonal_nadir
	type(amsu_zonal_ave_rec)			:: zonal_target


	! Set logical unit numbers for the various files

	integer(4),parameter			:: lu_raw_Tb		= 29	! orbit_files
	integer(4),parameter			:: lu_diurnal		= 30	! diurnal correction
	integer(4),parameter			:: lu_nadir			= 32	! nadir corrections (limb 
	integer(4),parameter			:: lu_inc			= 34	! incidence corrections
	integer(4),parameter			:: lu_target		= 28	! target temperature

	integer(4),parameter			:: lu_Tbs_grpt		= 50	! Tbs
	integer(4),parameter			:: lu_Tbs_month     = 51

	integer(4),parameter			:: lu_log			= 70	! log 
	integer(4),parameter			:: lu_bad_orbit		= 80    ! list of bad orbits for each satellite
	integer(4),parameter			:: lu_inventory     = 100	! orbit inventory file

	integer(4)			:: i
	integer(4)			:: fov
	integer(4)			:: asc_desc
	logical(4)			:: scan_ok
	logical(4)			:: data_ok
	integer(4)			:: zone
	integer(4)			:: land_sea
	integer(4)			:: surf

	integer(4)			:: month
	integer(4)          :: month_last
	integer(4)			:: year
	integer(4)          :: month_num 
	integer(4)			:: grpt_rec_num

	! log file name

	character(len=150)  :: log_file_name
	character(20)		:: ds

	! bad orbit file variables
	character(len=150)  :: bad_orbit_file_name
	integer(4)			:: num_bad_orbits
	integer(4)			:: bad_orbit_num
    integer(4)			:: bad_file_error

	! output file names

	character(len=150)  :: tb_grpt_name
	character(len=150)  :: monthly_tb_name
	character(len=150)  :: tb_zonal_name
	character(len=150)  :: diurnal_zonal_name
	character(len=150)  :: incidence_zonal_name
	character(len=150)  :: limb_zonal_name
	character(len=150)  :: target_zonal_name

	! averaging variables

	real(4)				:: tot_tb 
	real(4)				:: tot_diur 
	real(4)				:: tot_incidence 
	real(4)				:: tot_limb
	real(4)				:: mn_tb 
	real(4)				:: mn_diur 
	real(4)				:: mn_incidence 
	real(4)				:: mn_limb

	real(4)				:: tot_wt 

	! rain detection stuff

	real(4)				:: diff1
	real(4),parameter	:: rain_thres = 2.0
	integer(4)			:: rain_flag




	integer(4),parameter			:: NUM_WT = 2
	integer(4)						:: wt_num
	real(4),dimension(30,NUM_WT)	:: weights 
	integer(4),dimension(30,NUM_WT)	:: onto_map 
	!character(len = 4)				:: view_str

	integer(4)						:: num_used
	integer(4)						:: num_Tb_not_finite 
	integer(4)						:: num_Tb_bad_qual 
	integer(4)						:: num_diur_not_finite 
	integer(4)						:: num_nadir_not_finite

	real(4),dimension(8),parameter	:: tlt_wts   = (/-2.64,-1.14,0.44,1.41,1.61,1.17,0.40,-0.25/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)
	real(4),dimension(8),parameter	:: tlt_wts_r = (/-0.25,0.40,1.17,1.61,1.41,0.44,-1.14,-2.64/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)
						


	!----------------------------------------------



		call init_NAN

		! open log file
		ds = curr_date_string(error)
		write(log_file_name,110) channel,channel,view_str,diurnal_str,sat_num,trim(ds)
110     format('M:\msu_amsu_data\AMSU-A_L2C\L2A_to_L2C_Logs\month_update\ch',i2.2,'\log_ch',i2.2,'_views_',a,'_diur_',a,'_sat_',i2.2,'_',a,'.txt')
		open(unit = lu_log,file = log_file_name)
		print *,'Log File = ',log_file_name

		write(lu_log,140)'Info: Begin Execution: ',ds
		write(lu_log,141)'Info: Sat_num = ',sat_num
		write(lu_log,140)'Info: Satellite = ',amsu_sat_names_2(sat_num)
		write(lu_log,141)'Info: Channel = ',channel
		write(lu_log,140)'Info: Start_date = ',date_string(first_date_to_process)
		write(lu_log,140)'Info: End_date = ',date_string(last_date_to_process)
		write(lu_log,140)'Info: View Set = ',view_str
		write(lu_log,140)'Info: Diurnal Adj = ',diurnal_str

140     format(a,1x,a)
141     format(a,1x,i3)


		error = 0


		select case (view_str)
			case('L8 ')
				weights(7:10,1) = 0.25
				weights(21:24,2) = 0.25
				onto_map(7:10,1) = 1
				onto_map(21:24,2) = 1
			case('N12')
				weights(10:15,1) = 0.16666
				weights(16:21,2) = 0.16666
				onto_map(10:15,1) = 1
				onto_map(16:21,2) = 1
			case('TLT')
				weights(1:8,1) = tlt_wts
				weights(23:30,2) = tlt_wts_r
				onto_map(1:8,1) = 1
				onto_map(23:30,2) = 1
				print *,'Error  - do not use this routine for TLT'
				stop
			case default
				print *,'Case Does Not Match'
				print *,view_str
				stop
		endselect

		sat_name = amsu_sat_names_2(sat_num)

	    ! read in inventory file

		call read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)

		! check the bad orbit list

		write(bad_orbit_file_name,115) trim(AMSU_L2A_Path),sat_name,sat_name
115		format(a,a,'\',a,'_bad_orbits.txt')

		open(unit = lu_bad_orbit,file = bad_orbit_file_name,status = 'OLD',iostat = bad_file_error)
		if (bad_file_error .eq. 0) then 
			do i = 1,1234567
				read(lu_bad_orbit,116,end=118) bad_orbit_num
116				format(i6)
				if ((bad_orbit_num .gt. 0) .and. (bad_orbit_num .le. inv.max_orbit)) then
					inv.orbit_valid(bad_orbit_num) = .false.
					num_bad_orbits = num_bad_orbits + 1
				endif
			enddo
118         write(6,119) num_bad_orbits,bad_orbit_file_name
119			format(i5,' orbits flagged as bad in ',a)
		endif 



		! determine the first orbit and month to use
		! we use the first month that contains the start date, and the last month that contains the
		! end date.  Thus the region of computation is expanded somewhat.

		start_month = calc_month(first_date_to_process,remainder)
		start_found = .false.

		last_valid_rec = -1
		do inv_rec_num = 1,MX_ORBITS
			
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date   = set_date_2000(0.07+inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			
			if (rec_month .ge. start_month) then
				start_found = .true.
				if (last_valid_rec .gt. 0) then
					strt_orbit = last_valid_rec
				else
					strt_orbit = inv_rec_num
				endif
				strt_date = set_date_2000(inv%start_rd_2000(strt_orbit),error)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo
			 
		if (.not.(start_found)) then 
			print *, 'start orbit not found for sat num ',sat_num
			write(lu_log,*)'start orbit not found for sat num ',sat_num
			return
		endif

		! determine the last orbit to use

		last_month = calc_month(last_date_to_process,remainder)
		last_found = .false.
		last_valid_rec = -1

		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			if (rec_month .gt. last_month) then
				last_found = .true.
				if (last_valid_rec .gt. 0) then
					last_orbit = inv_rec_num
				else
					last_orbit = inv_rec_num
				endif
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo

		if (.not.(last_found)) then 
			last_orbit = inv%max_orbit
			last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
		endif

		write(6,155)strt_orbit,start_month,date_string(strt_Date)
		write(lu_log,155)strt_orbit,start_month,date_string(strt_Date)
155     format('Info: Start_orbit = ',i6,'  Start month = ',i4,'  Start Date = ',a)

		write(6,156)last_orbit,last_month,date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))
		write(lu_log,156)last_orbit,last_month,date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))
156     format('Info: Last_orbit = ',i6,'  Last month = ',i4,'  Last Date = ',a)

		! open the output files

			write(6,3351)
			write(lu_log,3351)
	3351	format('Info: Computing Tb GrPt Maps')
			write(tb_grpt_name,351)trim(AMSU_Monthly_Map_Path),channel,sat_num,channel,trim(view_str),trim(diurnal_str)
    351		format(a,'AMSU_Channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Monthly_Hist_144_72_',a,'_V3_02.',a,'.dat')
			open(unit = lu_tbs_grpt,file = tb_grpt_name,form='binary',access = 'direct',recl = GRPT_TB_REC_LENGTH)
			write(6,3352)tb_grpt_name
			write(lu_log,3352)tb_grpt_name
	3352    format('Info: GrPt File = ',a)

			write(6,3345)
			write(lu_log,3345)
	3345	format('Info: Computing Raw Tb zonal')
			write(tb_zonal_name,345)trim(AMSU_ZONAL_MONTHLY_PATH),channel,sat_num,channel,trim(diurnal_str)
	345		format(a,'AMSU_channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Zonal_month_36_v3_02.',a,'.dat')
			open(unit = lu_raw_tb,file = tb_zonal_name,form='binary',access = 'direct',recl = ZONAL_REC_LENGTH)
			write(6,3445)tb_zonal_name
			write(lu_log,3445)tb_zonal_name
	3445    format('Info: Zonal Tb File = ',a)		
		
			write(6,3346)
			write(lu_log,3346)
	3346	format('Info: Computing Diurnal Correction zonal')
			write(diurnal_zonal_name,346)trim(AMSU_ZONAL_MONTHLY_PATH),channel,sat_num,channel,trim(diurnal_str)
	346		format(a,'AMSU_channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Zonal_month_36_diurnal_v3_02.',a,'.dat')
			open(unit = lu_diurnal,file = diurnal_zonal_name,form='binary',access = 'direct',recl = ZONAL_REC_LENGTH)																  
			write(6,3446)diurnal_zonal_name
			write(lu_log,3446)diurnal_zonal_name
	3446    format('Info: Zonal Diurnal File = ',a)		


			write(6,3347)
			write(lu_log,3347)
	3347	format('Info: Computing Nadir Correction zonal')
			write(limb_zonal_name,347)trim(AMSU_ZONAL_MONTHLY_PATH),channel,sat_num,channel,trim(diurnal_str)
	347		format(a,'AMSU_channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Zonal_month_36_nadir_v3_02.',a,'.dat')
			open(unit = lu_nadir,file = limb_zonal_name,form='binary',access = 'direct',recl = ZONAL_REC_LENGTH)
			write(6,3447)limb_zonal_name
			write(lu_log,3447)limb_zonal_name
	3447    format('Info: Zonal Limb File = ',a)		



			write(6,3348)
			write(lu_log,3348)
	3348	format('Info: Computing incidence Correction zonal')
			write(incidence_zonal_name,348)trim(AMSU_ZONAL_MONTHLY_PATH),channel,sat_num,channel,trim(diurnal_str)
	348		format(a,'AMSU_channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Zonal_month_36_inc_v3_02.',a,'.dat')
			open(unit = lu_inc,file = incidence_zonal_name,form='binary',access = 'direct',recl = ZONAL_REC_LENGTH)
			write(6,3448)incidence_zonal_name
			write(lu_log,3448)incidence_zonal_name
	3448    format('Info: Zonal Inc File = ',a)		


			write(6,3349)
			write(lu_log,3349)
	3349	format('Info: Computing Target zonal')
			write(target_zonal_name,349)trim(AMSU_ZONAL_MONTHLY_PATH),sat_num,channel,trim(diurnal_str)
	349		format(a,'AMSU_Target\AMSU_',i2.2,'_Channel_',i2.2,'_Target_Zonal_month_36_v3_02.',a,'.dat')
			open(unit = lu_target,file = target_zonal_name,form='binary',access = 'direct',recl = ZONAL_REC_LENGTH)											 
			write(6,3449)target_zonal_name
			write(lu_log,3449)target_zonal_name
	3449    format('Info: Zonal Target File = ',a)		

		! Initialize a few variables before beginning to read

		i = 1
		error = 0

		call amsu_zero_zonal_ave_rec(zonal_tbs)
		call amsu_zero_zonal_ave_rec(zonal_diur)
		call amsu_zero_zonal_ave_rec(zonal_inc)
		call amsu_zero_zonal_ave_rec(zonal_nadir)
		call amsu_zero_zonal_ave_rec(zonal_target)

		hist = 0.0

		num_used = 0
		num_Tb_not_finite = 0 
		num_Tb_bad_qual = 0 
		num_diur_not_finite = 0 
		num_nadir_not_finite = 0

							
		do orbit_num = strt_orbit,last_orbit
		
			if (.not. inv%orbit_valid(orbit_num)) then
			    write(6,1123)orbit_num
			    write(lu_log,1123)orbit_num
1123			format('Info: Skipping: ',i6)
			    cycle	
			endif

			orbit_error = 0

			call read_AMSU_L2A_orbit_file(sat_name,					&
										  orbit_num,				& 
										  channel,					&
										  L2A_orbit_header,			&
										  L2A_orbit,				&
										  orbit_error)
!			write(lu_log,444)orbit_num
			if (modulo(orbit_num,200) .eq. 0) then
				write(6,444)orbit_num
				write(lu_log,444)orbit_num
444				format('Info: Processing Orbit ',i6.6)
			endif
			
			if ((orbit_error .eq. 0)) then 
				
				do n_swath = L2A_orbit_header%start_swath_num,L2A_orbit_header%end_swath_num
                    
					if (L2A_orbit%scan_valid(n_swath) .le. 0) cycle  ! skip rest if scan not valid
                    if (.not.(finite(L2A_orbit%scan_data(n_swath)%scan_rd_2000))) cycle 
					!if (orbit_num  .eq. 41747) print *,'n_swath = ',n_swath
					asc_desc = L2A_orbit%scan_data(n_swath)%asc_desc
                    !if (orbit_num  .eq. 41747) print *,asc_desc
						select case (diurnal_str)
							case('CMAM')
								call calculate_AMSU_diurnal_correction_CMAM(L2A_orbit%scan_data(n_swath),channel,lu_log,error)
							case('MERR')
								call calculate_AMSU_diurnal_correction_MERRA(L2A_orbit%scan_data(n_swath),channel,lu_log,error)
							case('HG1S')
								call calculate_AMSU_diurnal_correction_HADGEM_SAANE(L2A_orbit%scan_data(n_swath),channel,lu_log,error)
							case('CCM3')
								call calculate_AMSU_diurnal_correction_CCM3(L2A_orbit%scan_data(n_swath),channel,error)
							case default
								print *,'Error: -- diurnal type ',diurnal_str,' not supported'
								write(lu_log,*)'Error: -- diurnal type ',diurnal_str,' not supported'
								error = -1
								return
						end select
					
                    !if (orbit_num  .eq. 41747) print *,'finished diurnal',n_swath

					! set date
					if (.not. finite(L2A_orbit%scan_Data(n_swath)%scan_rd_2000)) cycle	 ! skip rest if bad time
					swath_date = set_date_2000(L2A_orbit%scan_Data(n_swath)%scan_rd_2000,date_error)
					month = calc_month(swath_date,remainder)
                    !if (orbit_num  .eq. 41747) print *,'set date done',n_swath

					if (month .lt. start_month) cycle	  ! only process swaths in the correct months
					if (i .gt. 2) then						
						if ((month .gt. month_last) .or. ((orbit_num .eq. last_orbit) .and. (n_swath .eq. L2A_orbit_header%end_swath_num)) ) then !write out zonal ave record for this month
							if (month_last .le. last_month) then
								write(6,999)      month_last,num_used,num_Tb_not_finite,num_Tb_bad_qual,orbit_num
								write(lu_log,999) month_last,num_used,num_Tb_not_finite,num_Tb_bad_qual,orbit_num
							999 format('Info: Month_last = ',i4,' Num used = ',i10,' Num NAN = ',i10,' Num Bad = ',i10,' Current Orbit = ',i6)
								num_used = 0
								num_Tb_not_finite = 0 
								num_Tb_bad_qual = 0 
								num_diur_not_finite = 0 
								num_nadir_not_finite = 0

							
								call amsu_update_zonal_ave_rec(zonal_tbs)
								call amsu_update_zonal_ave_rec(zonal_diur)
								call amsu_update_zonal_ave_rec(zonal_inc)
								call amsu_update_zonal_ave_rec(zonal_nadir)
								call amsu_update_zonal_ave_rec(zonal_target)

								
								zonal_tbs%end_date		= swath_date
								zonal_diur%end_date		= swath_date
								zonal_inc%end_date	    = swath_date
								zonal_nadir%end_date	= swath_date
								zonal_target%end_date   = swath_date

								data_ok = .false.

								do zone = 1, NUM_ZONES
									do fov = 1,NUM_AMSU_FOVS
									  do asc_desc = 1,NUM_NODES
										do land_sea = 1,NUM_SURFACES
											if ((finite(zonal_tbs%dat(zone,fov,asc_desc,land_sea)%mean)) .and. &
													   (zonal_tbs%dat(zone,fov,asc_desc,land_sea)%mean .gt. 0.0)) then
												data_ok = .true.
											endif
										 enddo
									  enddo
									enddo
								enddo

								if (data_ok) then  ! go ahead an write out a record
									call amsu_write_zonal_ave_rec_direct(month_last,AMSU_FIRST_month(sat_num)-1,zonal_tbs,lu_raw_tb,error)
									call amsu_write_zonal_ave_rec_direct(month_last,AMSU_FIRST_month(sat_num)-1,zonal_diur,lu_diurnal,error)
									call amsu_write_zonal_ave_rec_direct(month_last,AMSU_FIRST_month(sat_num)-1,zonal_inc,lu_inc,error)
									call amsu_write_zonal_ave_rec_direct(month_last,AMSU_FIRST_month(sat_num)-1,zonal_nadir,lu_nadir,error)
									call amsu_write_zonal_ave_rec_direct(month_last,AMSU_FIRST_month(sat_num)-1,zonal_target,lu_target,error)
								endif

								call amsu_zero_zonal_ave_rec(zonal_tbs)
								call amsu_zero_zonal_ave_rec(zonal_diur)
								call amsu_zero_zonal_ave_rec(zonal_inc)
								call amsu_zero_zonal_ave_rec(zonal_nadir)
								call amsu_zero_zonal_ave_rec(zonal_target)

								zonal_tbs%begin_date = swath_date
								zonal_diur%begin_date = swath_date
								zonal_inc%begin_date = swath_date
								zonal_nadir%begin_date = swath_date
								zonal_target%begin_date = swath_date

								curr_date = current_date(error)

								call flush(lu_log)

	987							format('Info: Processing Channel ',i2.2,1x,a,2x,i2.2,'/',i2.2,'/',i4.4,' ',i2.2,':',i2.2,':',i2.2,2x,'Orbit = ',i6,' Month Num = ',i6)
							
								grpt_rec_num = 1 + month_last - AMSU_FIRST_MONTH(sat_num)

								! encode current date into map so we can see when each month was processed
								! this is a part of the map (south pole) that never has real data

								hist(7,1,1,1) = curr_date%year
								hist(7,1,2,1) = curr_date%oday
								hist(7,1,3,1) = curr_date%time%hour
								hist(7,1,4,1) = curr_date%time%minute

								write(lu_Tbs_grpt,rec = grpt_rec_num) month_last,hist
                                year = 1978 + (month_last-1)/12
								month_num = 1 + modulo((month_last-1),12)

								!write(monthly_tb_name,391)trim(AMSU_Monthly_Map_Path),channel,trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel,year,month_num,trim(view_str),trim(diurnal_str)
								write(monthly_tb_name,391)trim(AMSU_Monthly_Map_Path_Dev2),channel,trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel,year,month_num,trim(view_str),trim(diurnal_str)
    391		                    format(a,'AMSU_Channel_',i2.2,'\',a,'\',a,'_Chan_',i2.2,'_',i4.4,'_',i2.2,'_',a,'_V3_02.',a,'.dat')
                                write(6,392)monthly_tb_name
								write(lu_log,392)monthly_tb_name
	392				            format('Info: Writing File ',a)
	                            call flush(lu_log)

								open(unit=lu_Tbs_month,file = monthly_tb_name,form='binary')
								write(lu_Tbs_month) month_last,hist
 						        close(unit=lu_Tbs_month)

								hist  = 0.0
								month_last = month
							endif
						endif
					else 
						zonal_tbs%begin_date = swath_date
						zonal_diur%begin_date = swath_date
						zonal_inc%begin_date = swath_date
						zonal_nadir%begin_date = swath_date
						zonal_target%begin_date = swath_date
						month_last = month
					endif  ! i gt 2


					i = i+1
					
					scan_ok = .true.

					Tbs						= L2A_orbit%scan_data(n_swath)%Tbs
					longitude				= L2A_orbit%scan_data(n_swath)%fov_longitude
					latitude				= L2A_orbit%scan_data(n_swath)%fov_latitude
					diurnal_correction		= L2A_orbit%scan_data(n_swath)%diurnal_corr
					nadir_correction		= L2A_orbit%scan_data(n_swath)%nadir_corr 
					incidence_correction    = L2A_orbit%scan_data(n_swath)%incidence_corr 
					target_temp				= L2A_orbit%scan_data(n_swath)%warm_load_temp
					fov_qual                = L2A_orbit%scan_data(n_swath)%fov_quality
	
	                fov_qual = 0
					call check_tbs_against_bounds(Tbs,channel,fov_qual)
                    !if (orbit_num  .eq. 41747) print *,'bounds check done',n_swath

					if ((scan_ok) .and. (month .eq. month_last)) then  ! this ignores data from bried backwards time steps

						do fov = 1,30

							altitude = topo_height_ncep_scale(longitude(fov),latitude(fov),error)
						
							if ((finite(Tbs(fov))) .and.				&
								(fov_qual(fov) .ge. 0) .and.			&
								!(finite(diurnal_correction(fov))) .and.	&
								!(finite(nadir_correction(fov)))	.and.	&
								(altitude .lt. altitude_thres_for_zonal) .and.				&
								(finite(longitude(fov))) .and.			&
								(finite(latitude(fov)))) then
								
							    num_used = num_used+1
								call amsu_put_in_zone(	Tbs(fov),						&
														longitude(fov),					&
														latitude(fov),					&
														swath_date,						&
														zone,							&
														fov,							&
														asc_desc,						&
														surf,							&
														zonal_tbs,						&
														lu_log)

								call amsu_put_in_zone(diurnal_correction(fov),		&
													 longitude(fov),				&
													 latitude(fov),					&
													 swath_date,					&
													 zone,							&
													 fov,							&
													 asc_desc,						&
													 surf,							&
													 zonal_diur,						&
													 lu_log)

								call amsu_put_in_zone(incidence_correction(fov),		&
													 longitude(fov),		&
													 latitude(fov),			&
													 swath_date,					&
													 zone,							&
													 fov,							&
													 asc_desc,						&
													 surf,							&
													 zonal_inc,						&
													 lu_log)								
								 
								call amsu_put_in_zone(nadir_correction(fov),		&
													 longitude(fov),		&
													 latitude(fov),			&
													 swath_date,					&
													 zone,							&
													 fov,							&
													 asc_desc,						&
													 surf,							&
													 zonal_nadir,						&
													 lu_log)
								call amsu_put_in_zone(target_temp,		&
													 longitude(fov),		&
													 latitude(fov),			&
													 swath_date,					&
													 zone,							&
													 fov,							&
													 asc_desc,						&
													 surf,							&
													 zonal_target,						&
													 lu_log)
								
							else
								if (.not. finite(Tbs(fov))) num_Tb_not_finite = num_Tb_not_finite + 1
								if (fov_qual(fov) .ne. 0)   num_Tb_bad_qual = num_Tb_bad_qual + 1
								if (.not. finite(diurnal_correction(fov))) num_diur_not_finite = num_diur_not_finite + 1
								if (.not. finite(nadir_correction(fov))) num_nadir_not_finite = num_nadir_not_finite + 1

							

							endif
						enddo	! fov
						do wt_num = 1,NUM_WT
							tot_tb = 0.0
							tot_diur = 0.0
							tot_incidence = 0.0
							tot_limb = 0.0
							tot_wt = 0.0

							rain_flag = 0

							do fov = 1,30
								if (abs(weights(fov,wt_num)) .lt. 0.001) cycle
									
								
								if ((fov .gt. 1) .and. (fov .lt. 30)) then
								    diff1 = tbs(fov) - (tbs(fov-1)+tbs(fov+1))/2.0
									if (abs(diff1) .gt. rain_thres) then
										rain_flag = 1
									endif
								endif

								if ((finite(Tbs(fov))) .and.				&
									(fov_qual(fov) .ge. 0) .and.			&
								!	(finite(diurnal_correction(fov))) .and.	&
								!	(finite(nadir_correction(fov)))	.and.	&
									(finite(longitude(fov))) .and.			&
									(finite(latitude(fov)))) then
									
								
									tot_tb =		tot_tb + weights(fov,wt_num)*tbs(fov)
									tot_diur =		tot_diur + weights(fov,wt_num)*diurnal_correction(fov)
									tot_incidence = tot_incidence + weights(fov,wt_num)*incidence_correction(fov)
									tot_limb =		tot_limb + weights(fov,wt_num)*nadir_correction(fov)
									tot_wt =		tot_wt + weights(fov,wt_num)


								endif
							enddo	! fov


							if (abs(1.0-tot_wt) .lt. 0.01) then
								mn_tb			= tot_tb/tot_wt
								mn_diur			= tot_diur/tot_wt
								mn_incidence	= tot_incidence/tot_wt
								mn_limb			= tot_limb/tot_wt

								if (view_str .eq. 'TLT') then
									if (mn_tb .lt. 170.0) cycle
									if (mn_tb .gt. 330.0) cycle
								endif
								do fov = 1,30
									if ((onto_map(fov,wt_num) .gt. 0)     .and.  &
										(finite(longitude(fov)))          .and.  &
										(finite(latitude(fov)))) then ! 
										ilon = floor(longitude(fov)/2.5) + 1
										ilat = floor((latitude(fov)+90.0)/2.5) + 1	
										if ((ilon .ge. 1) .and. (ilon .le. 144) .and. (ilat .ge. 1) .and. (ilat .le. 72)) then
											hist(1,asc_desc,ilon,ilat) =  hist(1,asc_desc,ilon,ilat) + 1
											hist(2,asc_desc,ilon,ilat) =  hist(2,asc_desc,ilon,ilat) + mn_tb
											hist(3,asc_desc,ilon,ilat) =  hist(3,asc_desc,ilon,ilat) + mn_diur
											hist(4,asc_desc,ilon,ilat) =  hist(4,asc_desc,ilon,ilat) + mn_incidence
											hist(5,asc_desc,ilon,ilat) =  hist(5,asc_desc,ilon,ilat) + mn_limb
											hist(6,asc_desc,ilon,ilat) =  hist(6,asc_desc,ilon,ilat) + target_temp
											!hist(7,asc_desc,ilon,ilat) =  hist(7,asc_desc,ilon,ilat) + swath_date%day
											hist(7,asc_desc,ilon,ilat) =  hist(7,asc_desc,ilon,ilat) + rain_flag
										endif
									endif
								enddo
							endif
						enddo	! wt_num
					endif		! scan_ok
				enddo			! n_swath
			endif				! orbit error & diurnal error
		enddo					! orbit num
		
		close(lu_raw_tb)			
		close(lu_diurnal)
		close(lu_nadir)
		close(lu_inc)
		close(lu_target)
		close(lu_tbs_grpt)

		ds = curr_date_string(error)
		write(lu_log,140) 'Info: Normal Completion ',ds
	    close(lu_log)
end subroutine update_all_amsu_L2C_monthlies_v3_2





end module AMSU_L2C_Update_Monthlies_v3_2

