module AMSU_L2C_Update_Monthlies_v3_2_TLT_CD


use AMSU_constants
use AMSU_TYPES
use date_and_time
use NAN_support
use topography
use portlib
use AMSU_Level_2A
use AMSU_Inventory
use AMSU_File_Locations
use AMSU_quality
use AMSU_diurnal_CCM3
use AMSU_diurnal_CMAM
use AMSU_diurnal_HADGEM_SAANE


implicit none

integer(4),parameter	:: GRPT_TB_REC_LENGTH     = 4 + (7*2*144*72*4)

contains

subroutine update_all_amsu_L2C_monthlies_v3_2_TLT_CD(	sat_num,					&
														channel,					&
														first_date_to_process,		&
														last_date_to_process,		&
														view_str,					&
														diurnal_str,				&
														error)
!@***s* MLT_RSS/update_all_amsu_L2C_monthlies_v3_2_TLT.f90
!  NAME
!    update_all_amsu_L2C_monthlies_v3_2 for TLT
!  PURPOSE
!    Calculates Monthly AMSU L2C data, stores to file.  Special TLT version that
!    separates right from left instead of ascending from descending
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel		amsu channel number (5,7, or 9)
!   first_date_to_process	Date structure containing first dat to process
!   last_date_to_process	Date structure containing the last date to process
!   view_str                string containing FOV subset to use 
!   error					:: = 0 if everything is OK, no zero otherwise
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call update_all_amsu_L2C_monthlies_v3_2_TLT_CD(sat_num,					&
!							channel,					&
!							first_date_to_process,		&
!							last_date_to_process,		&
!							view_str,					&
!							diurnal_str,				&
!							error)
!
!@*****
	integer(4)				:: sat_num
	integer(4)				:: channel
	type(date_str)			:: first_date_to_process
	type(date_str)			:: last_date_to_process
	type(date_str)			:: strt_date
	character(len=3)		:: view_str
	character(len=4)		:: diurnal_str
	integer(4)				:: error

!   This procedure updates the zonal month and daily grid point files for a given satellite and channel
!   The HADGEM diurnal cycle is substituted for the usual CCM3 diurnal cycle


!   Parameters
!
!	sat num					:: satellite number 10 = NOAA-15, 2 = NOAA-16, etc
!   channel					:: amsu channel number 1 - 16
!   first_date_to_process	:: Date structure containing first dat to process
!   last_date_to_process	:: Date structure containing the last date to process
!   process control			:: Logical array (20).  Controls which corrections are used and ancilliary data are calulated
!   error					:: = 0 if everything is OK, no zero otherwise
!
!	Input Files:
!		This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!		An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!		orbit and inventory files is contained in the AMSU_File_Locations module.

!	Output Files:
!		months:
!			month Files 
!
!


	real(4),dimension(NUM_AMSU_FOVS)	:: Tbs
	
	integer(4)							:: start_month
	logical								:: start_found
	integer(4)							:: inv_rec_num
	
	type(date_str)						::  rec_date
	integer(4)							::  rec_month
	integer(4)							::  strt_orbit
	integer(4)							::  last_orbit
	integer(4)							::  last_month
	integer(4)							::  last_valid_rec
	logical							    ::  last_found
	integer(4)							::  date_Error
	
	type(amsu_inventory_str)			:: inv			


	! declaration of input data structures
	
	type(AMSU_L2A_orbit_header)							:: L2A_orbit_header
	type(AMSU_L2A_orbit_data)							:: L2A_orbit

	type(date_str)				 :: swath_date

	integer(4)					 :: orbit_num
	character(len = 7)           :: sat_name
	integer(4)					 :: n_swath
	integer(4)					 :: orbit_error

	real(4)						 :: remainder

	real(4)							  :: target_temp
	real(4),dimension(NUM_AMSU_FOVS)  :: longitude
	real(4),dimension(NUM_AMSU_FOVS)  :: latitude
	integer(1),dimension(NUM_AMSU_FOVS)  :: fov_qual

	real(4),dimension(NUM_AMSU_FOVS)  :: diurnal_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: nadir_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: incidence_correction
	real(4),parameter				  :: precip_thres = 1.5

	type(Date_Str)					  :: curr_Date

	! declaration of gridpoint structures

	real(4),dimension(7,2,144,72)	    :: hist	 ! (num/tot_tb/tot_diur/tot_inc/tot_limb/tot_tar/tot_samp,left/right,ilon,ilat)

	integer(4)			:: ilon
	integer(4)			:: ilat

	! Set logical unit numbers for the various files

	integer(4),parameter			:: lu_raw_Tb		= 29	! orbit_files
	integer(4),parameter			:: lu_Tbs_grpt		= 50	! Tbs
    integer(4),parameter			:: lu_Tbs_month     = 51
	integer(4),parameter			:: lu_log			= 70	! log 
	integer(4),parameter			:: lu_bad_orbit		= 80    ! list of bad orbits for each satellite
	integer(4),parameter			:: lu_inventory     = 100	! orbit inventory file

	integer(4)			:: i
	integer(4)			:: fov
	integer(4)			:: asc_desc
	logical(4)			:: scan_ok

	integer(4)			:: month
	integer(4)          :: month_last	
	integer(4)			:: month_num
	integer(4)          :: year
	integer(4)			:: grpt_rec_num

	! log file name

	character(len=150)  :: log_file_name
	character(20)		:: ds

	! bad orbit file variables

	character(len=150)  :: bad_orbit_file_name
	integer(4)			:: num_bad_orbits
	integer(4)			:: bad_orbit_num
    integer(4)			:: bad_file_error

	! output file names

	character(len=150)  :: tb_grpt_name
	character(len=150)  :: monthly_tb_name

	! averaging variables

	real(4)				:: tot_tb 
	real(4)				:: tot_diur 
	real(4)				:: tot_incidence 
	real(4)				:: tot_limb
	real(4)				:: mn_tb 
	real(4)				:: mn_diur 
	real(4)				:: mn_incidence 
	real(4)				:: mn_limb

	real(4)				:: tot_wt 

	! rain detection stuff

	real(4)				:: diff1
	real(4),parameter	:: rain_thres = 2.0
	integer(4)			:: rain_flag




	integer(4),parameter			:: NUM_WT = 2
	integer(4)						:: wt_num
	real(4),dimension(30,NUM_WT)	:: weights 
	integer(4),dimension(30,NUM_WT)	:: onto_map 
	!character(len = 4)				:: view_str

	integer(4)						:: num_used
	integer(4)						:: num_Tb_not_finite 
	integer(4)						:: num_Tb_bad_qual 
	integer(4)						:: num_diur_not_finite 
	integer(4)						:: num_nadir_not_finite

	real(4),dimension(8),parameter	:: tlt_wts   = (/-2.64,-1.14,0.44,1.41,1.61,1.17,0.40,-0.25/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)
	real(4),dimension(8),parameter	:: tlt_wts_r = (/-0.25,0.40,1.17,1.61,1.41,0.44,-1.14,-2.64/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)
						


	!----------------------------------------------



		call init_NAN
		error = 0


		! open log file
		ds = curr_date_string(error)
		!print *,'making log file',channel,sat_num,trim(ds)
		write(log_file_name,110) sat_num,trim(ds)
110     format('M:\msu_amsu_data\AMSU-A_L2C\L2A_to_L2C_Logs\month_update\ch_tlt\log_ch_tlt_sat_',i2.2,'_',a,'_TLT.txt')
		open(unit = lu_log,file = log_file_name)
		print *,'Log File = ',log_file_name

		!write summary of calling parameters to log

		write(lu_log,140)'Info: Begin Execution: ',ds
		write(lu_log,141)'Info: Sat_num = ',sat_num
		write(lu_log,140)'Info: Satellite = ',amsu_sat_names_2(sat_num)
		write(lu_log,141)'Info: Channel = ',channel
		write(lu_log,140)'Info: Start_date = ',date_string(first_date_to_process)
		write(lu_log,140)'Info: End_date = ',date_string(last_date_to_process)
		write(lu_log,140)'Info: View Set = ',view_str
		write(lu_log,140)'Info: Diurnal Adj = ',diurnal_str

140     format(a,1x,a)
141     format(a,1x,i3)

		!make sure view set is 'TLT' and set view weights
		select case (view_str)
			case('TLT')
				weights(1:8,1) = tlt_wts
				weights(23:30,2) = tlt_wts_r
				onto_map(1:8,1) = 1
				onto_map(23:30,2) = 1
			case default
				write(lu_log,*)'Case Does Not Match ',view_str
				write(6,*)'Case Does Not Match ',view_str
				stop
		endselect

		sat_name = amsu_sat_names_2(sat_num)

	    ! read in inventory file

		call read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)

		! check the bad orbit list

		write(bad_orbit_file_name,115) trim(AMSU_L2A_Path),sat_name,sat_name
115		format(a,a,'\',a,'_bad_orbits.txt')

		open(unit = lu_bad_orbit,file = bad_orbit_file_name,status = 'OLD',iostat = bad_file_error)
		if (bad_file_error .eq. 0) then 
			do i = 1,1234567
				read(lu_bad_orbit,116,end=118) bad_orbit_num
116				format(i6)
				if ((bad_orbit_num .gt. 0) .and. (bad_orbit_num .le. inv.max_orbit)) then
					inv.orbit_valid(bad_orbit_num) = .false.
					num_bad_orbits = num_bad_orbits + 1
				endif
			enddo
118         write(6,119) num_bad_orbits,bad_orbit_file_name
119			format(i5,' orbits flagged as bad in ',a)
		endif 

		! determine the first orbit and month to use
		! we use the first month that contains the start date, and the last month that contains the
		! end date.  Thus the region of computation is expanded somewhat.

		start_month = calc_month(first_date_to_process,remainder)
		start_found = .false.

		last_valid_rec = -1
		do inv_rec_num = 1,MX_ORBITS
			
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle
			rec_date   = set_date_2000(0.07+inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			
			if (rec_month .ge. start_month) then
				start_found = .true.
				if (last_valid_rec .gt. 0) then
					strt_orbit = last_valid_rec
				else
					strt_orbit = inv_rec_num
				endif
				strt_date = set_date_2000(inv%start_rd_2000(strt_orbit),error)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo
			 
		if (.not.(start_found)) then 
			write(lu_log,*) 'Error: start orbit not found for sat num ',sat_num
			write(6,*) 'Error: start orbit not found for sat num ',sat_num
			return
		endif

		! determine the last orbit to use

		last_month = calc_month(last_date_to_process,remainder)
		last_found = .false.
		last_valid_rec = -1

		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			if (rec_month .gt. last_month) then
				last_found = .true.
				if (last_valid_rec .gt. 0) then
					last_orbit = inv_rec_num
				else
					last_orbit = inv_rec_num
				endif
				!last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo

		if (.not.(last_found)) then 
			last_orbit = inv%max_orbit
			last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
		endif

		! write info about first and last orbits to process to the log

		write(6,155)strt_orbit,start_month,date_string(strt_Date)
		write(lu_log,155)strt_orbit,start_month,date_string(strt_Date)
155     format('Info: Start_orbit = ',i6,'  Start month = ',i4,'  Start Date = ',a)

		write(6,156)last_orbit,last_month,date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))
		write(lu_log,156)last_orbit,last_month,date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))
156     format('Info: Last_orbit = ',i6,'  Last month = ',i4,'  Last Date = ',a)

		! open the output file

		write(lu_log,3351)
3351	format('Info: Computing Tb GrPt Maps')
		write(tb_grpt_name,351)trim(AMSU_Monthly_Map_Path),channel,sat_num,channel,trim(view_str),trim(diurnal_str)
351		format(a,'AMSU_Channel_',i2.2,'\AMSU_',i2.2,'_Chan_',i2.2,'_Monthly_Hist_144_72_',a,'_V3_02.',a,'.dat')
		open(unit = lu_tbs_grpt,file = tb_grpt_name,form='binary',access = 'direct',recl = GRPT_TB_REC_LENGTH)
		write(6,3352)tb_grpt_name
		write(lu_log,3352)tb_grpt_name
3352    format('Info: GrPt File = ',a)

		! Initialize a few variables before beginning to read

		i = 1
		error = 0
		hist = 0.0

		num_used = 0
		num_Tb_not_finite = 0 
		num_Tb_bad_qual = 0 
		num_diur_not_finite = 0 
		num_nadir_not_finite = 0

	    ! loop over orbits
							
		do orbit_num = strt_orbit,last_orbit
		    
			! check for invalid orbits

			if (.not. inv%orbit_valid(orbit_num)) then
			    write(6,1123)orbit_num
			    write(lu_log,1123)orbit_num
1123			format('Info: Skipping: ',i6)
			    cycle	
			endif


			! read in L@a orbit data

			orbit_error = 0
			call read_AMSU_L2A_orbit_file(sat_name,					&
										  orbit_num,				& 
										  channel,					&
										  L2A_orbit_header,			&
										  L2A_orbit,				&
										  orbit_error)

			if (modulo(orbit_num,200) .eq. 0) then
				write(lu_log,444)orbit_num
				write(6,444)orbit_num,sat_name,last_date_to_process%year
444				format('Info: Processing Orbit ',i6.6,'  ',a,2x,i4)
			endif
			
			if ((orbit_error .eq. 0)) then 
				
				do n_swath = L2A_orbit_header%start_swath_num,L2A_orbit_header%end_swath_num
                    
					if (L2A_orbit%scan_valid(n_swath) .le. 0) cycle  ! skip rest if scan not valid

					asc_desc = L2A_orbit%scan_data(n_swath)%asc_desc

					select case (diurnal_str)
						case('HG1S')
							call calculate_AMSU_diurnal_correction_HADGEM_SAANE(L2A_orbit%scan_data(n_swath),channel,lu_log,error)
						case('CCM3')
							call calculate_AMSU_diurnal_correction_CCM3(L2A_orbit%scan_data(n_swath),channel,error)
						case default
							write(lu_log,*)'Error: -- diurnal type ',diurnal_str,' not supported'
							error = -1
							return
					end select

					! set date
					if (.not. finite(L2A_orbit%scan_Data(n_swath)%scan_rd_2000)) cycle	 ! skip rest if bad time
					swath_date = set_date_2000(L2A_orbit%scan_Data(n_swath)%scan_rd_2000,date_error)
					month = calc_month(swath_date,remainder)

					if (month .lt. start_month) cycle	  ! only process swaths in the correct months
					if (i .gt. 2) then
						if ((month .gt. month_last) .or. ((orbit_num .eq. last_orbit) .and. (n_swath .eq. L2A_orbit_header%end_swath_num)) ) then !write out zonal ave record for this month
							if (month_last .le. last_month) then
								write(6,999)      month_last,num_used,num_Tb_not_finite,num_Tb_bad_qual,orbit_num
								write(lu_log,999) month_last,num_used,num_Tb_not_finite,num_Tb_bad_qual,orbit_num
							999 format('Info: Month_last = ',i4,' Num used = ',i10,' Num NAN = ',i10,' Num Bad = ',i10,' Current Orbit = ',i6)
								num_used = 0
								num_Tb_not_finite = 0 
								num_Tb_bad_qual = 0 
								num_diur_not_finite = 0 
								num_nadir_not_finite = 0

								curr_date = current_date(error)
								write(6,987) channel,sat_name,swath_date%month,swath_date%day,  &
									 swath_date%year,curr_date%time%hour,curr_date%time%minute,&
									 floor(curr_date%time%sec),orbit_num,month_last

								write(lu_log,987) channel,sat_name,swath_date%month,swath_date%day,  &
									 swath_date%year,curr_date%time%hour,curr_date%time%minute,&
									 floor(curr_date%time%sec),orbit_num,month_last

								call flush(lu_log)

	987							format(' Processing Channel ',i1,1x,a,2x,i2.2,'/',i2.2,'/',i4.4,'     ',i2.2,':',i2.2,':',i2.2,2x,i6,2x,i6)
							
								grpt_rec_num = 1 + month_last - AMSU_FIRST_MONTH(sat_num)

								! encode current date into map so we can see when each month was processed
								! this is a part of the map (south pole) that never has real data

								hist(7,1,1,1) = curr_date%year
								hist(7,1,2,1) = curr_date%oday
								hist(7,1,3,1) = curr_date%time%hour
								hist(7,1,4,1) = curr_date%time%minute

								write(lu_Tbs_grpt,rec = grpt_rec_num) month_last,hist
								                                year = 1978 + (month_last-1)/12
								month_num = 1 + modulo((month_last-1),12)
								write(monthly_tb_name,391)trim(AMSU_Monthly_Map_Path),channel,trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel,year,month_num,trim(view_str),trim(diurnal_str)
    391		                    format(a,'AMSU_Channel_',i2.2,'\',a,'\',a,'_Chan_',i2.2,'_',i4.4,'_',i2.2,'_',a,'_V3_02.',a,'.dat')
                                write(6,392)monthly_tb_name
								write(lu_log,392)monthly_tb_name
	392				            format('Info: Writing File ',a)

								open(unit=lu_Tbs_month,file = monthly_tb_name,form='binary')
								write(lu_Tbs_month) month_last,hist
 						        close(unit=lu_Tbs_month)
								hist  = 0.0
								month_last = month
							endif
						endif
					else 
							month_last = month
					endif  ! i gt 2


					i = i+1
					
					scan_ok = .true.

					Tbs						= L2A_orbit%scan_data(n_swath)%Tbs
					longitude				= L2A_orbit%scan_data(n_swath)%fov_longitude
					latitude				= L2A_orbit%scan_data(n_swath)%fov_latitude
					diurnal_correction		= L2A_orbit%scan_data(n_swath)%diurnal_corr
					nadir_correction		= L2A_orbit%scan_data(n_swath)%nadir_corr 
					incidence_correction    = L2A_orbit%scan_data(n_swath)%incidence_corr 
					target_temp				= L2A_orbit%scan_data(n_swath)%warm_load_temp
					fov_qual                = L2A_orbit%scan_data(n_swath)%fov_quality
	
					call check_tbs_against_bounds(Tbs,channel,fov_qual)

					if ((scan_ok) .and. (month .eq. month_last)) then  ! this ignores data from bried backwards time steps

						do wt_num = 1,NUM_WT
							tot_tb = 0.0
							tot_diur = 0.0
							tot_incidence = 0.0
							tot_limb = 0.0
							tot_wt = 0.0

							rain_flag = 0

							do fov = 1,30
								if (abs(weights(fov,wt_num)) .lt. 0.001) cycle
									
								diff1 = tbs(fov) - (tbs(fov-1)+tbs(fov+1))/2.0
								if ((fov .gt. 1) .and. (fov .lt. 30) .and. (abs(diff1) .gt. rain_thres)) then
									rain_flag = 1
								endif

								if ((finite(Tbs(fov))) .and.				&
									(fov_qual(fov) .ge. 0) .and.			&
									(finite(diurnal_correction(fov))) .and.	&
									(finite(nadir_correction(fov)))	.and.	&
									(finite(longitude(fov))) .and.			&
									(finite(latitude(fov)))) then
									
								
									tot_tb =		tot_tb + weights(fov,wt_num)*tbs(fov)
									tot_diur =		tot_diur + weights(fov,wt_num)*diurnal_correction(fov)
									tot_incidence = tot_incidence + weights(fov,wt_num)*incidence_correction(fov)
									tot_limb =		tot_limb + weights(fov,wt_num)*nadir_correction(fov)
									tot_wt =		tot_wt + weights(fov,wt_num)


								endif
							enddo	! fov

							if (abs(1.0-tot_wt) .lt. 0.01) then
								mn_tb			= tot_tb/tot_wt
								mn_diur			= tot_diur/tot_wt
								mn_incidence	= tot_incidence/tot_wt
								mn_limb			= tot_limb/tot_wt

								if (view_str .eq. 'TLT') then
									if (mn_tb .lt. 170.0) cycle
									if (mn_tb .gt. 330.0) cycle
								endif
								do fov = 1,30
									if ((onto_map(fov,wt_num) .gt. 0)     .and.  &
										(finite(longitude(fov)))          .and.  &
										(finite(latitude(fov)))) then ! 
										ilon = floor(longitude(fov)/2.5) + 1
										ilat = floor((latitude(fov)+90.0)/2.5) + 1	
										if ((ilon .ge. 1) .and. (ilon .le. 144) .and. (ilat .ge. 1) .and. (ilat .le. 72)) then
											if (rain_flag .eq. 0) then
												hist(1,wt_num,ilon,ilat) =  hist(1,wt_num,ilon,ilat) + 1
												hist(2,wt_num,ilon,ilat) =  hist(2,wt_num,ilon,ilat) + mn_tb
												hist(3,wt_num,ilon,ilat) =  hist(3,wt_num,ilon,ilat) + mn_diur
												hist(4,wt_num,ilon,ilat) =  hist(4,wt_num,ilon,ilat) + mn_incidence
												hist(5,wt_num,ilon,ilat) =  hist(5,wt_num,ilon,ilat) + mn_limb
												hist(6,wt_num,ilon,ilat) =  hist(6,wt_num,ilon,ilat) + target_temp
											endif
											hist(7,wt_num,ilon,ilat) =  hist(7,wt_num,ilon,ilat) + rain_flag
										endif
									endif
								enddo
							endif
						enddo	! wt_num
					endif		! scan_ok
				enddo			! n_swath
			endif				! orbit error & diurnal error
		enddo					! orbit num
		
		close(lu_tbs_grpt)
		ds = curr_date_string(error)
		write(lu_log,140) 'Info: Normal Completion ',ds
	    close(lu_log)
end subroutine update_all_amsu_L2C_monthlies_v3_2_TLT_CD
end module AMSU_L2C_Update_Monthlies_v3_2_TLT_CD
