

$freeform

module AMSU_L1A_Ingest

	use AMSU_constants
	use AMSU_file_locations
	use AMSU_types
	use AMSU_SAA
	use AMSU_Level_1B
	use nan_support
	use date_and_time
	use msu_date
	use interpolate
	use get_file_list
	use numerical_recipes_sorting
	use amsu_inventory 
	use AMSU_Equator_Crossing_Times 
	use expand_to_bits 
	use AMSU_quality
	use logging

	implicit none


	integer(4),parameter  :: AMSU_NRECMAX=6000000
	integer(4),parameter  :: AMSU_NUM_CROSS_MAX=6000
	integer(4),parameter  :: AMSU_MAX_FILES = 7000
	integer(4),parameter  :: AMSU_NUM_ORBITS_IN_STACK = 3

	character(len = 120)		:: LOG_FILE_PATH 

contains

subroutine Ingest_AMSU_L1A(sat_num,amsu_incoming_path,wrt)

!@***h* MLT_RSS/AMSU_L1A_Ingest.f90/Ingest_AMSU_L1A
!  NAME
!    Ingest_AMSU_L1A
!
!  PURPOSE
!    reads incoming AMSU L1A files and cuts them up into orbits
!  DESCRIPTION
!    reads incoming AMSU L1A files and cuts them up into orbits.  Writes out
!    orbit files, updates inventory files.  Keeps track of embedded calibration
!    information.
!
!  INPUT
!     integer(4)   :: sat_num,
!     character(len=120)  ::amsu_incoming_path
!     logical  :: wrt
!
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call Ingest_AMSU_L1A(sat_num,amsu_incoming_path,wrt)
!@*****

	character(len = 120)							::	amsu_incoming_path	! outgoing file name
	logical											::  wrt
	character(len = 120),dimension(AMSU_MAX_FILES)	::	amsu_raw_l1b_file		! Incoming file name
	character(len = 220)							::  file
	character(len = 3)								::  file_begin
	character(len = 3)								::  file_begin2
	character(len = 3)								::  file_begin3

	character(len = AMSU_LEVEL_1_RECORD_LENGTH)			:: amsu_L1A_scan
	character(len = AMSU_LEVEL_1_RECORD_LENGTH)			:: amsu_raw_header

	type(AMSU_A_Data_to_Keep_from_Header)						:: AMSU_header_data
	type(AMSU_A_Data_to_Keep_from_Header)						:: AMSU_header_data_last_file

	integer(4),parameter										:: MAX_NUM_HEADERS = 15
	type(AMSU_A_Data_to_Keep_from_Header),dimension(MAX_NUM_HEADERS)	:: AMSU_header_data_list
	integer(4)													:: num_in_header_data_list
	logical														:: header_changed,header_match_found
	logical														:: header_is_nonsense,change_critical
	integer(4)													:: header_num,current_header_num


	! stuff for the orbit data stack

	type(amsu_L1A_orbit_data),dimension(AMSU_NUM_ORBITS_IN_STACK)		:: L1A_orbit_Stack
	integer(4),dimension(AMSU_NUM_ORBITS_IN_STACK)						:: stack_orbit_num
	real(8),dimension(AMSU_NUM_ORBITS_IN_STACK)							:: stack_eqx
	integer(4),dimension(AMSU_NUM_ORBITS_IN_STACK)						:: stack_last_accessed
	integer(4)															:: orbit_index
	logical																:: orbit_found
	integer(4)															:: min_stack_last_accessed
	integer(4)															:: stack_pos_to_change

	type(amsu_L1B_orbit_header)											:: amsu_orbit_header

	

	! end stuff for the orbit data stack

	! errors

	integer(4)							:: read_Error
	integer(4)							:: err
	integer(4)							:: date_error

	integer(4)							:: sat_num
		

	real(8)			:: scan_r_d	        ! r_d for this scan
															! i.e. the r_d of the ith actual scan lives in scan_r_d(scan_num_index(i))
	real(4)			:: scan_lat			! nadir latitude for this scan
	real(4)			:: scan_lon	        ! nadir longitude for this scan
	real(4)									:: nadir_longitude
	real(4)									:: nadir_latitude
	integer(4)								:: scan_index
	real(8)									:: scan_rd_2000
	type(date_str)							:: scan_date


								 
	! crossing time stuff
	
	integer(4)									:: num_scan
	integer(4)									:: num_scan_this_file



	! inventory file stuff

	type(amsu_inventory_str)				:: amsu_inventory

	! orbit assignment stuff

	integer(4)		::  scan_orbit_num	! orbit number for this scan
	integer(2)		::  orbit_scan_num  ! scan number in orbit

	character(len = 2) :: scan_num_str

	integer(4),dimension(10)				:: tot_err									
	integer(4),dimension(10,AMSU_NUM_ORBITS_IN_STACK)	:: tot_err_this_output_file
	integer(4),dimension(AMSU_NUM_ORBITS_IN_STACK)		:: num_scans_this_output_file									

	integer(4)								:: file_num
	integer(4)								:: num_files
	integer(4)								:: num_files_to_process 
	integer(4)								:: files_processed
	integer(4)								:: num_orbits_written



	character(len = 120)					::	log_file	
	character(len = 120)					::	gap_file
	character(len = 512)					::  char_512
	
	type(date_str)	  :: cur_date
	character(len=20) :: cur_date_string
	integer(4)		  :: last_slash_pos


	integer(4)				 :: quality_indicator
	logical(4)				 :: scan_ok
	real(8)				     :: eq_cross_time
	real(8)					 :: delta_t

	logical					 :: log_verbose = .false.


		call init_NAN

		! open the log files

		cur_date		= current_date(err)
		cur_date_string = date_string_for_file_year_first(cur_date)

        LOG_FILE_PATH = trim(AMSU_L1A_Path)//'amsu_orbit_logs\'//trim(amsu_sat_names_2(sat_num))//'\'
		log_file = trim(LOG_FILE_PATH)//'amsu_orbitification_log_file_'//trim(cur_date_String)//'.txt'
		
		print *,'Open_log_file ',log_file
		open(unit=LOG_unit,file = log_file)

		gap_file = trim(LOG_FILE_PATH)//'amsu_gaps_'//trim(cur_date_String)//'.txt'
		print *, gap_file
		open(unit=GAP_unit,file = gap_file)

		! read the directory in the incoming file list
        print *,'Opened Gap File'
		print *, 'AMSU_INCOMING_PATH'
		print *, '*'//AMSU_INCOMING_PATH//'*'

		call get_files_in_path(AMSU_INCOMING_PATH,AMSU_MAX_FILES,num_files,amsu_raw_l1b_file,err)

		write(log_str,101)num_files,trim(AMSU_INCOMING_PATH)
101		format(' Found ',i6,' files in ',a)
		call log(log_str,log_err)
		! sort the files according to date using the appropriate part of the file name

		call sort_amsu_l1b_file_list(num_files,amsu_raw_l1b_file)

		! read in any previous inventory file

		call read_amsu_l1b_inventory_file_text(sat_num,amsu_inventory,read_error)

		! read in any previous header file

		call read_AMSU_Header_Data_List(amsu_sat_names_2(sat_num),num_in_header_data_list,AMSU_header_data_list)

										
		
			
!---------------------------------------------------------------------------
! read through all the files, and keep scan times and crossing times
!---------------------------------------------------------------------------

		num_Scan  = 0
		num_scan_this_file = 0

	!	num_files_to_process = 7
		num_files_to_process = num_files
		files_processed = 0
		num_orbits_written = 0

		do file_num = 1,num_files_to_process
			file = amsu_raw_l1b_file(file_num)
			file_begin = file(1:3)
			file_begin2 = file(10:12)
			file_begin3 = file(11:13)
			if ((file_begin .ne. 'NSS') .and. (file_begin2 .ne. 'NSS') .and. (file_begin3 .ne. 'NSS')) cycle

			last_slash_pos = scan(amsu_incoming_path,'\/',back=.true.)
			file  = AMSU_INCOMING_PATH(1:last_slash_pos)//amsu_raw_l1b_file(file_num)
			open(unit = AMSU_SAA_LU,file = file,status= 'OLD',form = 'binary')

		;!	if ((len(trim(amsu_raw_l1b_file(file_num))) .gt. 44) .or. (sat_num .eq.14)) then
			!	read(AMSU_SAA_LU) char_512
			!endif

			call read_raw_AMSU_header(	AMSU_SAA_LU,				&
										AMSU_raw_Header,			&
										read_error)
            
			call get_AMSU_calibration_data_from_header(AMSU_raw_Header, &
													   AMSU_header_data)

			call compare_header_calibration_data_for_critical_changes(AMSU_header_data,AMSU_header_data_list(header_num),change_critical,header_is_nonsense,.false.)
            if (header_is_nonsense) then 
			    rewind(AMSU_SAA_LU)
				read(AMSU_SAA_LU) char_512
				call read_raw_AMSU_header(	AMSU_SAA_LU,				&
										AMSU_raw_Header,			&
										read_error)
            
				call get_AMSU_calibration_data_from_header(AMSU_raw_Header, &
														   AMSU_header_data)
			endif

			if (num_in_header_Data_list .eq. 0) then
				AMSU_header_data_list(1) = AMSU_Header_data
				num_in_header_Data_list = 1
				current_header_num = 1
			endif

			header_changed = .false.

			if (files_processed .eq. 0) then
			   	header_match_found = .false.
				do header_num = 1,num_in_header_Data_list
					call compare_header_calibration_data_for_critical_changes(AMSU_header_data,AMSU_header_data_list(header_num),change_critical,header_is_nonsense,.false.)

					if (.not. (change_critical)) then
						header_match_found = .true.
						current_header_num = header_num
					endif
				enddo
				if (.not. header_match_found) then ! add to list
					num_in_header_Data_list = num_in_header_data_list + 1
					
					if (num_in_header_data_list .le. MAX_NUM_HEADERS) then
						AMSU_header_data_list(num_in_header_Data_list) = AMSU_Header_data
						current_header_num =num_in_header_Data_list
						write(log_str,*)'IMPORTANT:  New header values found in',file
						call log(log_str,log_err)
					else
						call log('Fatal Error: Out of Room in header data list',log_err)
						stop
					endif
				endif
				write(log_str,774)current_header_num
774				format(1x,'Using Header Number ',i3,' for first file')
				call log(log_str,log_err)


			endif

			if (files_processed .ge. 1) then
				if (AMSU_header_data%calibration .ne. AMSU_header_data_last_file%calibration) then
					write(log_str,554) file
554					format(' Header Calibration Data Changed: ',a)
					call log(log_str,log_err)
					header_changed = .true.
				endif

				if (AMSU_header_data%Temp_to_rad .ne. AMSU_header_data_last_file%temp_to_rad) then
					write(log_str,555) file
555					format(' Header Temp to Radiance Data Changed: ',a)
					call log(log_str,log_err)
					header_changed = .true.

				endif

				if (AMSU_header_data%A1_Digital_Conv .ne. AMSU_header_data_last_file%A1_Digital_Conv) then
					write(log_str,556) file
556					format(' Header A1_Digital_Conv Data Changed: ',a)
					call log(log_str,log_err)
					header_changed = .true.
				endif

				if (AMSU_header_data%A2_Digital_Conv .ne. AMSU_header_data_last_file%A2_Digital_Conv) then
					write(log_str,557) file
557					format(' Header A2_Digital_Conv Data Changed: ',a)
					call log(log_str,log_err)
					header_changed = .true.
				endif
			endif
			header_is_nonsense = .false.
			change_critical = .false.
			if (header_changed) then 
				call compare_header_calibration_data_for_critical_changes(AMSU_header_data,AMSU_header_data_last_file,change_critical,header_is_nonsense,.true.)
			endif

			if (.not. (header_is_nonsense)) then
				if (change_critical) then  ! search list for match
					header_match_found = .false.
					do header_num = 1,num_in_header_Data_list
						call compare_header_calibration_data_for_critical_changes(AMSU_header_data,AMSU_header_data_list(header_num),change_critical,header_is_nonsense,.false.)

						if (.not. (change_critical)) then
							header_match_found = .true.
							current_header_num = header_num
						endif
					enddo
					if (.not. header_match_found) then ! add to list
						num_in_header_Data_list = num_in_header_data_list + 1
						
						if (num_in_header_data_list .le. MAX_NUM_HEADERS) then
							AMSU_header_data_list(num_in_header_Data_list) = AMSU_Header_data
							current_header_num =num_in_header_Data_list
						else
							call log('Fatal Error: Out of Room in header data list',log_Err)
							stop
						endif
					endif

					write(log_str,778)current_header_num
778					format(1x,'Now Using Header Number ',i3)
					call log(log_str,log_err)

				endif
				AMSU_header_data_last_file = AMSU_header_data

			else
				write(log_str,779) current_header_num
779				format(1x,'New Header is nonsense -- still using header number ',i3)
				call log(log_str,log_err)
				AMSU_header_data_last_file = AMSU_header_data_list(current_header_num)

			endif

			read_error = 0
			num_Scan_this_file = 0
			do while (read_Error .eq. 0)
				call read_SAA_AMSU_swath(AMSU_SAA_LU,amsu_L1A_scan,read_error)

				if (read_error .ne. 0) then
					cycle
				endif

				call decode_date_and_earth_loc_from_SAA_AMSU_swath(amsu_L1A_scan,&
	                                                         scan_rd_2000,		 &
															 nadir_latitude,	 &
															 nadir_longitude,	 &
															 quality_indicator,  &
															 err)

				if (err .ne. 0) then
					cycle
				endif

				scan_date = set_date_2000(scan_rd_2000,date_error)
				if (scan_date%year .lt. 1997) cycle
				if (scan_date%year .gt. 2020) cycle


				call evaluate_amsu_quality_indicator(quality_indicator,scan_ok)

				if (.not. scan_ok) then
					cycle
				endif

				num_scan = num_Scan + 1
				num_scan_this_file = num_Scan_this_file + 1

				scan_r_d			= scan_rd_2000
				scan_lat			= nadir_latitude  
				scan_lon			= nadir_longitude 
				scan_orbit_num		= AMSU_rev_num_from_rd_2000(sat_num,scan_rd_2000,err)
				if (err .ne. 0) then
					Write(log_str,*),'RD_2000 greater than RD_2000 for last known orbit'
					call log(log_str,log_err)
				endif
					

				eq_cross_time = equator_crossing_rd_2000_from_AMSU_rev_num(sat_num,scan_orbit_num,err)
				if (err .ne. 0) cycle

				delta_t = 86400.0D0*(scan_rd_2000 - eq_cross_time)
				orbit_scan_num = floor(delta_t/8.0) + 1
				

				if (log_verbose) then

					write(log_str,1239) scan_rd_2000,nadir_latitude,nadir_longitude,scan_orbit_num,delta_t,orbit_scan_num
1239				format(1x,f12.5,2(1x,f8.3),1x,i6,f10.2,1x,i6)
					call log(log_str,log_err)

				endif

				! change first 2 bytes to be the scan number in the current orbit

				scan_num_str =  int_2_to_string(byte_swap(orbit_scan_num))
				amsu_L1A_scan(1:2) = scan_num_str

				! we know the orbit number, scan number, and crossing time -- add data to 
				! appropriate orbit

				orbit_found = .false.
				do orbit_index = 1,AMSU_NUM_ORBITS_IN_STACK
					if (scan_orbit_num .eq. stack_orbit_num(orbit_index)) then
						! we have a match to an existing orbit stack member

						! add the scan to the data for this orbit
				
						call add_amsu_L1A_scan_to_orbit(amsu_L1A_scan,					&
														L1A_orbit_stack(orbit_index),	&
														orbit_scan_num,		&
														current_header_num, &
														err)
				
						tot_err(err) = tot_err(err) + 1
							 
						num_scans_this_output_file(orbit_index) = num_scans_this_output_file(orbit_index) + 1 
						tot_err_this_output_file(err,orbit_index) = tot_err_this_output_file(err,orbit_index) + 1 
 
						orbit_found = .true.

						stack_last_accessed(orbit_index) = num_scan
						exit
					endif
				enddo

				if (.not. orbit_found) then
					! didn't find a matching orbit -- we need to write the least most recently used
					! orbit and assign its place to a new orbit, check for an existing orbit file
					! and read it in if necessary.
					min_stack_last_accessed = minval(stack_last_accessed)
					stack_pos_to_change = 0
					do orbit_index = 1,AMSU_NUM_ORBITS_IN_STACK
						if (min_stack_last_accessed .eq. stack_last_accessed(orbit_index)) then
							stack_pos_to_change = orbit_index
							exit
						endif
					enddo
					
					if (stack_pos_to_change .le. 0) then
						! problem
						write(log_str,*),'Fatal Error: Bizarre Stack Error'
						call log(log_str,log_err)
						stop
					endif

					! write out old orbit, if it has any data

					if (stack_orbit_num(stack_pos_to_change) .gt. 0) then
						if (amsu_new_data_present(L1A_orbit_stack(stack_pos_to_change))) then
							if (wrt) &
							call write_AMSU_L1A_orbit_file(amsu_sat_names_2(sat_num),						&
														   stack_orbit_num(stack_pos_to_change),			&
														   L1A_orbit_stack(stack_pos_to_change),			&
														   num_Scans_this_output_file(stack_pos_to_change), &
														   stack_eqx(stack_pos_to_change),					&
														   AMSU_header_data_list,							&
														   tot_err_this_output_file(:,stack_pos_to_change),	&
														   err)
						
							if (err .eq. 0) num_orbits_written = num_orbits_written+1
	
						!	Update the inventory data
							
							if (wrt) call update_amsu_inventory_data(amsu_inventory,							&
															L1A_orbit_stack(stack_pos_to_change),	&
															sat_num, &
															stack_orbit_num(stack_pos_to_change))
								
						endif
					endif

					! zero the orbit data

					do scan_index = 1,MAX_SCANS_PER_ORBIT
						L1A_orbit_stack(stack_pos_to_change)%scan_data(scan_index) = ''
						L1A_orbit_stack(stack_pos_to_change)%scan_valid(scan_index) = 0
					enddo

					! zero scan and error counters

					num_scans_this_output_file(stack_pos_to_change) = 0 
					tot_err_this_output_file(:,stack_pos_to_change) = 0 


					! read in orbit file, if it exists

					call read_amsu_L1A_orbit_file(amsu_sat_names_2(sat_num),			&
												  scan_orbit_num,						&
												  amsu_orbit_header,					&
												  L1A_orbit_stack(stack_pos_to_change),	&
												  0,									&
												  err)

					! set orbit number in the stack

					stack_orbit_num(stack_pos_to_change)	= scan_orbit_num
					stack_eqx(stack_pos_to_change)			= eq_cross_time 

					! add the current scan to this new stack pos

					call add_amsu_L1A_scan_to_orbit(amsu_L1A_scan,							&
													L1A_orbit_stack(stack_pos_to_change),	&
													orbit_scan_num,				&
													current_header_num, &
													err)
				
					tot_err(err) = tot_err(err) + 1 
					num_scans_this_output_file(stack_pos_to_change) = num_scans_this_output_file(stack_pos_to_change) + 1 
					tot_err_this_output_file(err,stack_pos_to_change) = tot_err_this_output_file(err,stack_pos_to_change) + 1 
					orbit_found = .true.
					stack_last_accessed(stack_pos_to_change) = num_scan
				endif ! not orbit found

				
			enddo

			files_processed = files_processed + 1
			write(log_unit,1212)file_num,current_header_num,trim(amsu_raw_l1b_file(file_num)),num_Scan_this_file
			write(log_str,1212)file_num,current_header_num,trim(amsu_raw_l1b_file(file_num)),num_Scan_this_file
1212		format(1x,i7,1x,i2,1x,a,2x,i5)
			call log(log_str,log_err)
			close(ILU)
		enddo

		! all files processed -- flush the orbit data stack

		do orbit_index = 1,AMSU_NUM_ORBITS_IN_STACK
			if (stack_orbit_num(stack_pos_to_change) .gt. 0) then
				if (amsu_new_data_present(L1A_orbit_stack(stack_pos_to_change))) then
					if (wrt) then
						call write_amsu_L1A_orbit_file(amsu_sat_names_2(sat_num),		&
													stack_orbit_num(orbit_index),			&
													L1A_orbit_stack(orbit_index),			&
													num_Scans_this_output_file(orbit_index),&
													stack_eqx(orbit_index),					&
 													AMSU_header_data_list,					&
													tot_err_this_output_file(:,orbit_index),&
													err  )
						if (err .eq. 0) num_orbits_written = num_orbits_written+1
					endif
										
					if (wrt) call update_amsu_inventory_data(	amsu_inventory,				&
												L1A_orbit_Stack(orbit_index),				&
												sat_num,									&
												stack_orbit_num(orbit_index))

				endif
			endif
		enddo

		! update the inventory files

 		call write_amsu_l1b_inventory_file_text(sat_num,amsu_inventory,err)

		call write_amsu_l1b_inventory_file(sat_num,amsu_inventory,err)

		call write_AMSU_Header_Data_List(amsu_sat_names_2(sat_num),num_in_header_data_list,AMSU_header_data_list)





!-----------------------------------------------
! end write out orbit files
!-----------------------------------------------


		write(log_str,2323)num_scan,tot_err(1:6)
2323	format(1x,10i12)
		call log(log_str,log_err)

		write(log_unit,3434),files_processed,num_orbits_written
3434    format('Files Processed: ',i6,' Orbits Written: ',i6)
		close(log_unit)
		close(gap_unit)

end subroutine ingest_AMSU_L1A

	function almost_equal(t1,t2)

		! compares two real*8's -- true is their absolute difference is less than 2.0*10-5 (about 2 seconds)

		real(8)		:: t1
		real(8)		:: t2
		logical     :: almost_equal

		almost_equal = .false.
		if (abs(t1 - t2) .lt. 2.0e-5) then
			almost_equal	= .true.
		endif

		return
	end function almost_equal
 

	subroutine sort_amsu_l1b_file_list(num_files,amsu_raw_l1b_file)

		integer(4)											::  num_files
		character(len = 120),dimension(AMSU_MAX_FILES)		::	amsu_raw_l1b_file

		character(len = 120),dimension(AMSU_MAX_FILES)		::	temp_file_list

		character(len = 6)              :: temp0
		character(len = 5)				:: temp1
		character(len = 4)				:: temp2
		character(len = 11)				:: temp3

		real(8),dimension(num_files) :: file_index
		integer(4),dimension(num_files) :: indx
		integer(4)						:: i
		integer(4)                      :: nss_loc
		
		
		! first extract real(8) related to date and time

		do i = 1,num_files
		    
			temp1 = ' '
			temp2 = ' '
		    nss_loc = index(amsu_raw_l1b_file(i),'NSS.AM')
			if (nss_loc .ge. 1) then
				temp1 = amsu_raw_l1b_file(i)(nss_loc+13:nss_loc+17)
				temp2 = amsu_raw_l1b_file(i)(nss_loc+20:nss_loc+23)
			endif

			temp3 = ' '
			if (temp1(1:1) .eq. '1') temp3 = '20' // temp1 // temp2
			if (temp1(1:1) .eq. '0') temp3 = '20' // temp1 // temp2
			if (temp1(1:1) .eq. '9') temp3 = '19' // temp1 // temp2
			
			read(temp3,111) file_index(i)
111			format(f11.0)
            print *,temp1,temp2,temp3,file_index(i)

		enddo

		! sort this array

		call indexx_r8(num_files,file_index,indx)

		! put sorted file names in temp variable

		do i = 1, num_files
			temp_file_list(i) = amsu_raw_l1b_file(indx(i))
		enddo

		! put back into original array

		amsu_raw_l1b_file = temp_file_list

		return

	end subroutine sort_amsu_l1b_file_list

	subroutine add_amsu_L1A_scan_to_orbit(	scan_to_add,					&
											L1A_orbit_data,					&
											scan_num,						&
											current_header_num,				&
											error)

		character(len = AMSU_LEVEL_1_RECORD_LENGTH)	:: scan_to_add
		type(amsu_L1A_orbit_data)					:: L1A_orbit_data
		integer(2)									:: scan_num
		integer(4)									:: current_header_num
		integer(4)									:: error

		! this routine puts the data in scan into the L1A_orbit_data array
		!
		! eventually, we need to make decisions about with version of the scan data is
		! better, if one already exists, and they are different

		! errors:  
		!   Error = 6 means something weird happened
		!	Error = 1 means match found in correct place, no replacement made
		!	Error = 2 means empty spot in correct place, replacement made
		!   Error = 3 means match found, replacement made


		! check to see if r_d_2000s match up -- if not, look on appropriate side

		error = 6
		if ((scan_num .gt. AMSU_MAX_SCANS_PER_ORBIT) .or. (scan_num .lt. 1)) then
			return
		endif
		if (L1A_orbit_data%scan_valid(scan_num) .eq. 0) then
			! we have an empty spot == put this scan in

			L1A_orbit_data%scan_data(scan_num) = scan_to_add
			L1A_orbit_data%scan_valid(scan_num) = 2
			L1A_orbit_data%header_num_used(scan_num) = current_header_num
			error = 2
			return
		endif

		! hmmm there is already a scan in this spot
		! Since both scans must have passed preliminary quality control
		! we'll assume the new one is better for now

		L1A_orbit_data%scan_data(scan_num) = scan_to_add
		L1A_orbit_data%scan_valid(scan_num) = 2
		L1A_orbit_data%header_num_used(scan_num) = current_header_num

		error = 3


	end subroutine add_amsu_L1A_scan_to_orbit
											 
end module AMSU_L1A_ingest