$freeform
module AQUA_AMSU_L2A_Corrections

	use NAN_support
	use date_and_time
	use interpolate
	use planck
	use AMSU_Constants
	use AMSU_Types
	use AMSU_inventory
	use AMSU_equator_crossing_times
	use AMSU_File_Locations
	use AMSU_Level_2A
	use AMSU_nadir
	use AMSU_diurnal
	use AMSU_Quality

	implicit none


	! header data list declarations

	type(AMSU_A_Data_to_Keep_from_Header),dimension(20)			:: AMSU_header_data_list
	integer(4)													:: num_in_header_data_list

	character(len = 120),parameter		:: LOG_FILE_PATH = trim(AMSU_L2A_Path) // 'L1B_to_L2A_logs\'
    character(len = 20),parameter       :: L2A_processor_ID = 'L2A ID:AQUA_ADD v1.0'

contains

	subroutine add_corrections_to_AQUA_L2A_orbits(sat_num,channel,num_orbits,orbit_list)
!@***s* MLT_RSS/AMSU_Level_2A.f90/add_corrections_to_AQUA_L2A_orbits
!  NAME
!    add_corrections_to_AQUA_L2A_orbits
!
!  PURPOSE
!    Adds corrections to AMSU-AQUA L2A orbit files.
!
!  DESCRIPTION
!   Adds corrections to AMSU-AQUA L2A orbit files.  AQUA files arrive as brightness temperatures, so L1B to L2A
!   is not needed.  For NOAA and EUMETSAT satellites, diurnal and angle adjustments are performed as this conversion
!   is done.  So we need a separate routine for AQUA to calculate adjustments.
!
!  INPUTS 
!    integer(4)                     :: sat_num                  ! satellite name (NOAA-15, etc)
!    integer(4)                     :: channel                  ! AMSU channel
!    integer(4)                     :: num_orbits               ! number of orbits to process
!    integer(4),dimension(MAX_ORBITS_TO_PROCESS) :: orbit_list   ! list of orbits to process
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call add_corrections_to_AQUA_L2A_orbits(sat_num,channel,num_orbits,orbit_list)
!@*****
		integer(4)			:: sat_num
		integer(4)			:: channel
		integer(4)			:: num_orbits
		integer(4),dimension(MAX_ORBITS_TO_PROCESS)			:: orbit_list
	
		! errors

		integer(4)			:: error
		integer(4)			:: read_error
		integer(4)			:: convert_error
		integer(4)			:: write_error

		! inventory file stuff

		type(amsu_inventory_str)									:: L2A_inv

		! orbit declarations

		integer(4)													:: orbit_index
		integer(4)													:: orbit_num

		type(amsu_L2A_orbit_data)									:: L2A_orbit
		type(amsu_L2A_orbit_header)									:: L2A_orbit_header

		character(len = 7)											:: sat_name

		! log file stuff

		type(date_str)												:: cur_date,temp_date
		character(len = 20)											:: cur_date_String
		character(len = 120)										:: log_file


		if ((channel .ge. 3) .and. (channel .le. 14)) then
			call read_amsu_ccm3_diurnal(channel,error)
		endif

		! open the log files

		cur_date		= current_date(error)
		cur_date_string = date_string_for_file(cur_date)

		log_file = trim(LOG_FILE_PATH)//'L2A_log_file_'//trim(cur_date_String)//'.txt'
		open(unit=LOG_unit,file = log_file)

		! read in the L2A inventory

		call read_amsu_L2A_inventory_file_text(sat_num,channel,L2A_inv,error)

		! loop through orbit numbers

		do orbit_index = 1,num_orbits
			orbit_num = orbit_list(orbit_index)			
			
			if (reprocess) then 
				 call read_AMSU_L2A_orbit_file(                  &
									amsu_Sat_names_2(sat_num),	&
									orbit_num,					& 
									channel,					&
									L2A_orbit_header,			&
									L2A_orbit,					&
									error)
			else
				call read_AMSU_L2A_orbit_file_unadjusted(                   &
												amsu_Sat_names_2(sat_num),	&
												orbit_num,					& 
												channel,					&
												L2A_orbit_header,			&
												L2A_orbit,					&
												error)
			endif
			if (error .ne. 0) cycle
			temp_date = set_date_2000(L2A_orbit_header%start_rd_2000,error)
			if (error .ne. 0) cycle


			call add_corrections_to_L2A_orbit(  L2A_orbit,				&
												channel,				&
												orbit_num,				&
												amsu_Sat_names_2(sat_num),				&
												convert_error)
												 
			L2A_orbit_header.cross_track_correction = residual_cross_track_correction
			L2A_orbit_header.L2A_processor_ID = L2A_processor_ID 
			if (convert_error .ge. 0) then

				call write_AMSU_L2A_orbit_file(	amsu_Sat_names_2(sat_num),		&
												orbit_num,						& 
												channel,						&
												L2A_orbit_header,				&
												L2A_orbit,						&
												error)

				call update_AMSU_L2A_inventory(	L2A_inv,			    &
												L2A_orbit,				&
												sat_num,				&
												orbit_num,				&
												error)

			endif
		enddo

		call  write_AMSU_L2A_Inventory_file(sat_num,channel,L2A_inv,error)
		call  write_AMSU_L2A_Inventory_file_text(sat_num,channel,L2A_inv,error)

	end  subroutine add_corrections_to_AQUA_L2A_orbits



	subroutine add_corrections_to_L2A_orbit(   L2A_orbit,				&
												channel,				&
												orbit_num,				&
												sat_name,				&
												convert_error) 
!@***s* MLT_RSS/AMSU_Level_2A.f90/add_corrections_to_L2A_orbit
!  NAME
!    add_corrections_to_L2A_orbit
!
!  PURPOSE
!    Adds corrections to a AMSU L2A orbit.
!
!  DESCRIPTION
!   Adds corrections to a single AMSU-AQUA L2A. 
!
!  INPUTS
!   type(AMSU_L2A_orbit_data)							:: L2A_orbit
!   integer(4)											:: channel
!   integer(4)											:: orbit_num
!   character(len = 7)									:: sat_name
!  OUTPUTS
!    integer(4)											:: convert_Error
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call add_corrections_to_L2A_orbit(L2A_orbit,channel,orbit_num,sat_name,convert_error)

		type(AMSU_L2A_orbit_data)							:: L2A_orbit
		integer(4)											:: channel
		integer(4)											:: orbit_num
		character(len = 7)									:: sat_name

		integer(4)											:: convert_Error

		integer(4)											:: scan_num
		integer(4)											:: error
		type(AMSU_A_Data_to_Keep_from_Header)				:: Calibration_Data
		type(AMSU_A_L1B_Calibration_Data)					:: decoded_calibration_data


		integer(4)												:: cold_counts1
		integer(4)												:: cold_counts2
		integer(4)												:: warm_counts1
		integer(4)												:: warm_counts2

		real(4)													:: a0,a1,a2
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: warm_counts
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: cold_counts
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: warm_counts_smoothed
		real(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)			:: cold_counts_smoothed

		real(4)												:: tot_cold_counts
		real(4)												:: tot_warm_counts
		real(4)												:: tot_cold_wt
		real(4)												:: tot_warm_wt
		integer(4)											:: smooth_half_width
		integer(4)											:: j
		real(4)												:: wt

		real(4)												:: nadir_lon
		real(4)												:: nadir_lat
		real(4)												:: altitude
		real(8)												:: rd_2000

		real(4),dimension(3)					:: calibration_coeff1
		real(4),dimension(3)					:: calibration_coeff2

		convert_Error = 0

		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
		

			if ((L2A_orbit%scan_valid(scan_num) .gt. 0) .and. &
			    finite(l2a_orbit%scan_data(scan_num)%scan_rd_2000)) then
				if ((channel .eq. 4) .or. &
				    (channel .eq. 5) .or. &
					(channel .eq. 6) .or. &
					(channel .eq. 7) .or. &
					(channel .eq. 8) .or. &
					(channel .eq. 9)) then 
					call calculate_nadir_correction(  L2A_orbit%scan_data(scan_num),	&
													  channel,							&
													  error)
				endif
				if ((channel .eq. 4) .or. &
				    (channel .eq. 5) .or. &
					(channel .eq. 6) .or. &
					(channel .eq. 7) .or. &
					(channel .eq. 8) .or. &
					(channel .eq. 9)) then 					
					call calculate_diurnal_correction(L2A_orbit%scan_data(scan_num),	&
													  channel,							&
													  error)
				endif
				L2A_orbit%scan_valid(scan_num) = 2
			else 
				L2A_orbit%scan_valid(scan_num) = 0
			endif
				
		enddo

	end subroutine add_corrections_to_L2A_orbit

end module AQUA_AMSU_L2A_Corrections
