module AMSU_diurnal
$freeform
!@***m* MLT_RSS/AMSU_Diurnal.f90
!  NAME
!    AMSU_Diurnal.f90
! 
!  PURPOSE
!    Module for performing diurnal adjustment to AMSU measurements
! 
!  DESCRIPTION
!    Module for performing diurnal adjustment to AMSU measurements
!    Uses a model derived diurnal climatology to make the adjustment
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_Diurnal
!@*****
!@***s* MLT_RSS/AMSU_Diurnal.f90/calculate_diurnal_correction
!     NAME
!      calculate_diurnal_correction
!     PURPOSE
!      Calculate Diurnal Correction for a single AMSU scan line
!     INPUTS
!      channel     AMSU channel number
!     INPUTS_OUTPUTS
!      L2A_Scan    Structure containing AMSU L2A Scan data
!     OUTPUTS
!      Error       Error Flag.  0 if everything OK, not zero otherwise
!     USAGE
!      Call calculate_diurnal_correction(L2A_scan,channel,error)
!@*****
!@***s* MLT_RSS/AMSU_Diurnal.f90/read_amsu_ccm3_diurnal
!     NAME
!        read_amsu_ccm3_diurnal
!     PURPOSE
!        reads CCM3-derived AMSU Diurnal Climatology for the given AMSU channel
!     INPUTS
!        chan    AMSU channel
!     OUTPUTS
!        error   error flag.  0 if everything OK, -1 if invalid channel passed
!     REMARKS
!        loads climatology into the module variable diur_tb_sim. The number of the channel currently loaded is contained in channel_loaded
!     USAGE
!        call read_amsu_ccm3_diurnal(5,error)
!@*****
!@***s* MLT_RSS/AMSU_Diurnal.f90/amsu_ccm3_diurnal_tb
!     NAME
!       amsu_ccm3_diurnal_tb
!     PURPOSE
!       calculates diurnal value from diurnal climatology for given channel,fov,location,date and local_time
!     INPUTS
!       chan    AMSU_CHANNEL
!       fov     AMSU Field of View
!       lon     longitude
!       lat     latitude
!       date    date, to find time of year, in date structure (date_str) defined in date_and_time.f90
!       local_time  local time, defined in time structure (time_str) defined in date_and_time.f90
!     RETURNED VALUE
!       diur_tb  interpolated value of dirunal Tb
!     USAGE
!       tb_diurnal = amsu_ccm3_diurnal_tb(chan,fov,lon,lat,date,local_time)
!@*****
	use NAN_support
	use date_and_time
	use maps
	use mcs
	use AMSU_constants
	use AMSU_types

	type(map_str),dimension(15,24,12)		:: diur_tb_sim	   !AMSU fov, hour, month

	integer(4)								:: channel_loaded = 0

contains

    subroutine calculate_diurnal_correction(L2A_scan,						&
											channel,						&
											error)
		use AMSU_Types

		type(AMSU_A_L2A_Swath_Data)			:: L2A_scan
		integer(4)							:: channel
		integer(4)							:: error

		integer(4)							:: date_error
		integer(4)							:: fov

		type(date_Str)						:: scan_date
		type(time_str)						:: local_time

		real(4)								:: diur_tb_local_time
		real(4)								:: diur_tb_noon

		scan_date = set_date_2000(L2A_scan%scan_rd_2000,date_error)

		do fov = 1,30 

			local_time = convert_to_local_time(scan_date%time,L2a_scan%fov_longitude(fov),error)

			diur_tb_local_time = amsu_ccm3_diurnal_tb(channel,fov,L2A_scan%fov_longitude(fov),L2A_scan%fov_latitude(fov),scan_date,local_time,error)
			diur_tb_noon       = amsu_ccm3_diurnal_tb(channel,fov,L2A_scan%fov_longitude(fov),L2A_scan%fov_latitude(fov),scan_date,NOON,error)

			L2A_scan%diurnal_corr(fov) = diur_tb_noon - diur_tb_local_time
		enddo
		
	end subroutine calculate_diurnal_correction 



	subroutine read_amsu_ccm3_diurnal(chan,error)

		use AMSU_File_Locations

		integer(4)					:: chan
		integer(4)					:: error

		real(4),dimension(144,72)	:: mat

		integer(4)					:: AMSU_chan
		integer(4)					:: fov
		integer(4)					:: month
		integer(4)					:: hour

		character(len = 120)		:: diurnal_clim_file


		
		select case (chan) 
			case (4)
				AMSU_chan = 4
			case (5)
				AMSU_chan = 5
			case (6)
				AMSU_chan = 6
			case (7)
				AMSU_chan = 7
			case (8)
				AMSU_chan = 8
			case (9)
				AMSU_chan = 9
			case default
				write(6,111)chan
111             format(' Channel ',i2,' not implemented')
				error = -1
				return
		end select


		write(diurnal_clim_file,9001) trim(AMSU_ADJUSTMENT_PATH),AMSU_chan
9001	format(a,'AMSU_Diurnal\CCM3_Diurnal_Climatology\mn_ccm3_diur_cycles_amsu_chan_',i2.2'.dat')

		print *, 'AMSU Diurnal Adjustment:'
		print *, '   Loading: ',diurnal_clim_file
		open(unit = 3,file = diurnal_clim_file,form = 'binary')

		do fov = 1,15
			do month = 1,12
				do hour = 1, 24
                 
				    ! 16-fov reverses order so that fov = 1 is the nader position

					read(3) mat

					if (error .ne. 0) return

					call set_up_map(0.0, 2.5,144,CELL_CENTERED, &				
					               -90.0,2.5,72,CELL_CENTERED,  &
						     	    SPHERICAL,diur_tb_sim(16-fov,hour,month),error)

					if (error .ne. 0) then
						print *,'Error setting up map'
						return
					endif

					diur_tb_sim(16-fov,hour,month)%dat = mat

				enddo
			enddo
		enddo

		channel_loaded = chan

	end subroutine read_amsu_ccm3_diurnal

	real(4) function amsu_ccm3_diurnal_tb(chan,fov,lon,lat,date,local_time,error)

		integer(4)				:: chan
		integer(4)				:: fov
		real(4)					:: lon
		real(4)					:: lat
		type(date_str)			:: date
		type(time_str)			:: local_time
		integer(4)				:: error

		! returns interpolated value of CCM3 diurnal cycle climatology
		! date is used to do the month interpolation
		! local time is used to do the hourly interpolation

		real(4),dimension(12)		:: month_wts
		real(4),dimension(24)       :: hour_wts
		real(4),dimension(6)		:: fov_wts

		integer(4),dimension(2)     :: month_indices
		integer(4),dimension(2)		:: hour_indices
		integer(4),dimension(2)		:: fov_indices

		real(4)						:: diur_value
		real(4)						:: diur_tb

		integer(4)					:: hour_index,ihour
		integeR(4)					:: month_index,imonth
		integer(4)					:: fov_index,ifov
		integer(4)					:: view_index

		real(4)						:: hour_wt
		real(4)						:: month_wt
		real(4)						:: fov_wt


		
		if (chan .ne. channel_loaded) then
			call read_amsu_ccm3_diurnal(chan,error)
			if (error .ne. 0) then
				print *,'Error:  could not read diurnal file'
				return
			endif
		endif

		call month_weights(date,month_wts,month_indices,error)
		call hour_weights(local_time,hour_wts,hour_indices,error)

		view_index = nint(0.5 + abs(15.5 - fov))

		diur_tb = 0.0

		do imonth = 1,2
			month_index = month_indices(imonth)
			month_wt    = month_wts(month_index)
			do ihour = 1,2
				hour_index = hour_indices(ihour)
				hour_wt    = hour_wts(hour_index)
				call get_map_value(diur_tb_sim(view_index,hour_index,month_index),lon,lat,diur_value,error)
				diur_tb = diur_tb+month_wt*hour_wt*diur_value
			enddo
		enddo

		amsu_ccm3_diurnal_tb = diur_tb
		return

	end function amsu_ccm3_diurnal_tb
end module AMSU_diurnal