$freeform

!@***m* MLT_RSS/AMSU_equator_crossing_times.f90
!  NAME
!    AMSU_equator_crossing_times.f90
! 
!  PURPOSE
!    Module for finding ascending node equator crossing times for NOAA and EUMETSAT satellites
! 
!  DESCRIPTION
!    Module for finding ascending node equator crossing times for NOAA and EUMETSAT satellites
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_equator_crossing_times
!@*****

!
!@***s* MLT_RSS/AMSU_equator_crossing_times.f90/equator_crossing_rd_2000_from_AMSU_rev_num
!     NAME
!      equator_crossing_rd_2000_from_AMSU_rev_num
!     PURPOSE
!      Calculate Diurnal Correction for a single AMSU scan line
!     INPUTS
!       sat_num     AMSU satellite number (10 = NOAA-15, 11 = NOAA-16, etc)
!       rev_num     AMSU orbit number.    
!     OUTPUTS
!       Error       Error Flag.  0 if everything OK, not zero otherwise
!     RETURNED VALUE
!       ascending node eqaotor crossing time in rd_2000 format (days since midnight, jan 1, 2000) (real*8)
!     USAGE
!      crossing_time = equator_crossing_rd_2000_from_AMSU_rev_num(sat_num,rev_num,error)
!@*****
!
!@***s* MLT_RSS/AMSU_equator_crossing_times.f90/AMSU_rev_num_from_rd_2000
!     NAME
!      AMSU_rev_num_from_rd_2000
!     PURPOSE
!      Calculates AMSU rev num from time in rd_2000 format
!     INPUTS
!       sat_num     AMSU satellite number (10 = NOAA-15, 11 = NOAA-16, etc)
!       rd_2000     Time in rd_2000 format 
!     OUTPUTS
!       Error       Error Flag.  0 if everything OK, not zero otherwise
!     RETURNED VALUE
!       AMSU orbit number corresponding to given time
!     USAGE
!      orbit_num = AMSU_rev_num_from_rd_2000(sat_num,rd_2000,error)
!@*****
!
!@***s* MLT_RSS/AMSU_equator_crossing_times.f90/load_NOAA_15_eqx_data
!     NAME
!        load_NOAA_15_eqx_data
!     PURPOSE
!        reads equator crossing time data for NOAA-15
!     SIDE EFFECTS
!        loads climatology into the module variable eq_x_time_NOAA_15, and the equator crossing longitude into eq_x_longitude_NOAA_15. 
!     REMARKS
!        exactly analogous routines exist for the other NOAA and EUMETSAT satellites. (NOAA-16, etc.)  These are:
!        - load_NOAA_16_eqx_data
!        - load_NOAA_17_eqx_data
!        - load_NOAA_18_eqx_data
!        - load_NOAA_19_eqx_data
!        - load_METOP_A_eqx_data
!     USAGE
!        load_NOAA_15_eqx_data
! 
!@*****
module AMSU_equator_crossing_times

	implicit none

	integer(4),parameter   :: AMSU_MAX_REVS = 100000


	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_NOAA_15
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_NOAA_15
	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_NOAA_16
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_NOAA_16
	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_NOAA_17
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_NOAA_17
	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_NOAA_18
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_NOAA_18
	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_NOAA_19
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_NOAA_19
	real(8),dimension(AMSU_MAX_REVS)		:: eq_x_time_METOP_A
	real(4),dimension(AMSU_MAX_REVS)		:: eq_x_longitude_METOP_A
	integer(4)								:: max_rev_NOAA_15
	integer(4)								:: max_rev_NOAA_16
	integer(4)								:: max_rev_NOAA_17
	integer(4)								:: max_rev_NOAA_18
	integer(4)								:: max_rev_NOAA_19
	integer(4)								:: max_rev_METOP_A
	logical									:: eq_x_loaded_NOAA_15 = .false.
	logical									:: eq_x_loaded_NOAA_16 = .false.
	logical									:: eq_x_loaded_NOAA_17 = .false.
	logical									:: eq_x_loaded_NOAA_18 = .false.	
	logical									:: eq_x_loaded_NOAA_19 = .false.
	logical									:: eq_x_loaded_METOP_A = .false.

	character(len = 120)					:: NOAA_15_eq_x_time_file = 'NOAA_K.eqx'
	character(len = 120)					:: NOAA_16_eq_x_time_file = 'NOAA_L.eqx'
	character(len = 120)					:: NOAA_17_eq_x_time_file = 'NOAA_M.eqx'
	character(len = 120)					:: NOAA_18_eq_x_time_file = 'NOAA_N.eqx'
	character(len = 120)					:: METOP_A_eq_x_time_file = 'METOP_A.eqx'
	character(len = 120)					:: NOAA_19_eq_x_time_file = 'NOAA_P.eqx'

										

contains

	real(8) function equator_crossing_rd_2000_from_AMSU_rev_num(sat_num,rev_num,error)

		use NAN_support

		integer(4)			:: sat_num
		integer(4)			:: rev_num
		integer(4)			:: error

		select case(sat_num)

			case(10)
				if (.not. (eq_x_loaded_NOAA_15)) call load_NOAA_15_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_NOAA_15) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_NOAA_15(rev_num)
				
				return

			case(11)
				if (.not. (eq_x_loaded_NOAA_16)) call load_NOAA_16_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_NOAA_16) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_NOAA_16(rev_num)
				
				return

			case(12)
				if (.not. (eq_x_loaded_NOAA_17)) call load_NOAA_17_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_NOAA_17) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_NOAA_17(rev_num)
				
				return

						
			case(13)
				if (.not. (eq_x_loaded_NOAA_18)) call load_NOAA_18_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_NOAA_18) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_NOAA_18(rev_num)
				
				return		
					
			case(16)
				if (.not. (eq_x_loaded_NOAA_19)) call load_NOAA_19_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_NOAA_19) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_NOAA_19(rev_num)
				
				return

			case(14)
				if (.not. (eq_x_loaded_METOP_A)) call load_METOP_A_eqx_data

				if (rev_num .lt. 1) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				if (rev_num .gt. max_rev_METOP_A) then 
					equator_crossing_rd_2000_from_AMSU_rev_num = NAN
					error = -1
					return
				endif

				equator_crossing_rd_2000_from_AMSU_rev_num = &
					eq_x_time_METOP_A(rev_num)
				
				return



			case default
				equator_crossing_rd_2000_from_AMSU_rev_num = NAN
				error = -1
				return

			end select

	end function equator_crossing_rd_2000_from_AMSU_rev_num






	integer(4) function AMSU_rev_num_from_rd_2000(sat_num,rd_2000,error)

		use binary_search

		integer(4)			:: sat_num
		real(8)				:: rd_2000
		integer(4)			:: error

		integer(4)			:: rev
		
		error = 0
		select case(sat_num)
			case(10)
				if (.not. (eq_x_loaded_NOAA_15)) call load_NOAA_15_eqx_data
					

				if (rd_2000 .lt. eq_x_time_NOAA_15(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_NOAA_15(max_rev_NOAA_15)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_NOAA_15,max_rev_NOAA_15,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

			case(11)
			 	if (.not. (eq_x_loaded_NOAA_16)) call load_NOAA_16_eqx_data

				if (rd_2000 .lt. eq_x_time_NOAA_16(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_NOAA_16(max_rev_NOAA_16)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_NOAA_16,max_rev_NOAA_16,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

			case(12)
			 	if (.not. (eq_x_loaded_NOAA_17)) call load_NOAA_17_eqx_data

				if (rd_2000 .lt. eq_x_time_NOAA_17(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_NOAA_17(max_rev_NOAA_17)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_NOAA_17,max_rev_NOAA_17,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

			case(13)
			 	if (.not. (eq_x_loaded_NOAA_18)) call load_NOAA_18_eqx_data

				if (rd_2000 .lt. eq_x_time_NOAA_18(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_NOAA_18(max_rev_NOAA_18)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_NOAA_18,max_rev_NOAA_18,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

			case(14)
			 	if (.not. (eq_x_loaded_METOP_A)) call load_METOP_A_eqx_data

				if (rd_2000 .lt. eq_x_time_METOP_A(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_METOP_A(max_rev_METOP_A)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_METOP_A,max_rev_METOP_A,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

			case(16)
			 	if (.not. (eq_x_loaded_NOAA_19)) call load_NOAA_19_eqx_data

				if (rd_2000 .lt. eq_x_time_NOAA_19(1)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				if (rd_2000 .gt. eq_x_time_NOAA_19(max_rev_NOAA_19)) then
					AMSU_rev_num_from_rd_2000 = -999
					error = -1
					return
				endif

				call locate8(eq_x_time_NOAA_19,max_rev_NOAA_19,rd_2000,rev)

				AMSU_rev_num_from_rd_2000 = rev

				return

		case default

			AMSU_rev_num_from_rd_2000 = -999
			error = -1
			return
		end select

	end function AMSU_rev_num_from_rd_2000

	subroutine load_NOAA_15_eqx_data

		use AMSU_File_Locations

		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev


		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//NOAA_15_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_NOAA_15(rev_num),eq_x_longitude_NOAA_15(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_NOAA_15(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 3) then 
						delta_rd_rev = eq_x_time_NOAA_15(rev_num-1)-eq_x_time_NOAA_15(rev_num-2)
						eq_x_time_NOAA_15(rev_num) = eq_x_time_NOAA_15(rev_num-1) + delta_rd_rev
					else
						! this is a bigger problemmm
						stop
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_NOAA_15 = .true.
			max_rev_NOAA_15 = num_revs_to_read
	end subroutine load_NOAA_15_eqx_data

	subroutine load_NOAA_16_eqx_data

		use AMSU_File_Locations

		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev

		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//NOAA_16_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_NOAA_16(rev_num),eq_x_longitude_NOAA_16(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_NOAA_16(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 3) then 
						delta_rd_rev = eq_x_time_NOAA_16(rev_num-1)-eq_x_time_NOAA_16(rev_num-2)
						eq_x_time_NOAA_16(rev_num) = eq_x_time_NOAA_16(rev_num-1) + delta_rd_rev
					else
						! this is a bigger problemmm
						stop
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_NOAA_16 = .true.
			max_rev_NOAA_16 = num_revs_to_read
	end subroutine load_NOAA_16_eqx_data

	subroutine load_NOAA_17_eqx_data

		use AMSU_File_Locations

		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev



		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//NOAA_17_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_NOAA_17(rev_num),eq_x_longitude_NOAA_17(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_NOAA_17(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 3) then 
						delta_rd_rev = eq_x_time_NOAA_17(rev_num-1)-eq_x_time_NOAA_17(rev_num-2)
						eq_x_time_NOAA_17(rev_num) = eq_x_time_NOAA_17(rev_num-1) + delta_rd_rev
					else
						! this is a bigger problemmm
						stop
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_NOAA_17 = .true.
			max_rev_NOAA_17 = num_revs_to_read


	end subroutine load_NOAA_17_eqx_data





	subroutine load_NOAA_18_eqx_data

		use AMSU_File_Locations

		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev



		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//NOAA_18_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_NOAA_18(rev_num),eq_x_longitude_NOAA_18(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_NOAA_18(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 3) then 
						delta_rd_rev = eq_x_time_NOAA_18(rev_num-1)-eq_x_time_NOAA_18(rev_num-2)
						eq_x_time_NOAA_18(rev_num) = eq_x_time_NOAA_18(rev_num-1) + delta_rd_rev
					else
						! this is a bigger problemmm
						stop
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_NOAA_18 = .true.
			max_rev_NOAA_18 = num_revs_to_read
	end subroutine load_NOAA_18_eqx_data


	subroutine load_NOAA_19_eqx_data

		use AMSU_File_Locations

		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev



		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//NOAA_19_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_NOAA_19(rev_num),eq_x_longitude_NOAA_19(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_NOAA_19(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 3) then 
						delta_rd_rev = eq_x_time_NOAA_19(rev_num-1)-eq_x_time_NOAA_19(rev_num-2)
						eq_x_time_NOAA_19(rev_num) = eq_x_time_NOAA_19(rev_num-1) + delta_rd_rev
					else
						! this is a bigger problemmm
						stop
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_NOAA_19 = .true.
			max_rev_NOAA_19 = num_revs_to_read
	end subroutine load_NOAA_19_eqx_data


	subroutine load_METOP_A_eqx_data

		use NAN_support
		use AMSU_File_Locations


		integer(4)  :: num_revs_to_read
		integer(4)  :: rev_num,rev
		real(8)		:: delta_rd_rev



		! load the data
			open(unit=30,file = trim(AMSU_Navigation_Path)//METOP_A_eq_x_time_file,status = 'old')
			read(30,111) num_revs_to_read
			do rev_num = 1,num_revs_to_read
				read(30,112) rev,eq_x_time_METOP_A(rev_num),eq_x_longitude_METOP_A(rev_num)
				
				!fill in missing crossing times by extrapolating from below

				if (eq_x_time_METOP_A(rev_num) .lt. -9990.0) then
					if (rev_num .ge. 6) then 
						delta_rd_rev = eq_x_time_METOP_A(rev_num-1)-eq_x_time_METOP_A(rev_num-2)
						eq_x_time_METOP_A(rev_num) = eq_x_time_METOP_A(rev_num-1) + delta_rd_rev
						print *,eq_x_time_METOP_A(rev_num),delta_rd_rev
					else
						! this is a bigger problemmm
						eq_x_time_METOP_A(rev_num) = NAN
					endif
				endif
			enddo
111					format(1x,i7)
112                 format(1x,i7,1x,f20.12,1x,f9.4)
			eq_X_loaded_METOP_A = .true.
			max_rev_METOP_A = num_revs_to_read
	end subroutine load_METOP_A_eqx_data

end module AMSU_equator_crossing_times