$freeform
!@***m* MLT_RSS/AMSU_Level_1B.f90
!  NAME
!    AMSU_Level_1B.f90
! 
!  PURPOSE
!    Module for read, writing and storing AMSU L1B data
! 
!  DESCRIPTION
!    Module for read, writing and storing AMSU L1B data
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_Level_1B
!@*****

module AMSU_Level_1B

		use amsu_types 
		use amsu_constants
		use AMSU_file_locations
		use AMSU_SAA
		use NAN_support
		use Date_and_Time
		use type_conversion
		use byte_swapping
		use msflib
		use gzip
		use logging

		implicit none



contains



subroutine encode_amsu_l1b_orbit_file_header(amsu_orbit_header,		&
										 	 header)
!@***s* MLT_RSS/AMSU_Level_1B.f90/encode_amsu_l1b_orbit_file_header
!  NAME
!    encode_amsu_l1b_orbit_file_header
! 
!  PURPOSE
!    Encodes header data into binary form for writing
! 
!  INPUTS
!    type(amsu_L1B_orbit_header)		:: amsu_orbit_header  ! orbit header data in structured form
!  OUTPUTS
!    character(len=2560)				:: header             ! orbit header data in binary form
!  USAGE
!     call encode_amsu_l1b_orbit_file_header(amsu_orbit_header,header)
!@*****
		type(amsu_L1B_orbit_header)		:: amsu_orbit_header
		character(len=2560)				:: header

		integer(4)						:: j

		header(1:7)			= amsu_orbit_header%satellite_name 
		header(9:12)		= int_4_to_string(amsu_orbit_header%orbit_number)		
		header(13:16)		= int_4_to_string(amsu_orbit_header%number_of_swaths)
		header(17:20)		= int_4_to_string(amsu_orbit_header%start_swath_num)
		header(21:24)		= int_4_to_string(amsu_orbit_header%end_swath_num)
		header(25:32)		= real_8_to_string(amsu_orbit_header%start_rd_2000)
		header(33:40)		= real_8_to_string(amsu_orbit_header%end_rd_2000)
		header(41:48)		= real_8_to_string(amsu_orbit_header%eqx_rd_2000)
		header(49:100)		= amsu_orbit_header%pad2

		header(101:562)		= amsu_orbit_header%calibration
		header(563:742)		= amsu_orbit_header%Temp_to_Rad
		header(743:1462)	= amsu_orbit_header%A1_Digital_Conv
		header(1463:1766)	= amsu_orbit_header%A2_Digital_Conv

		do j = 1,AMSU_MAX_SCANS_PER_ORBIT
			header(1766 + j:1766 + j) = int_1_to_char(amsu_orbit_header%scan_valid(j))
		enddo

		header(2557:2560) = ' END'


		
end subroutine encode_amsu_l1b_orbit_file_header

subroutine decode_amsu_l1b_orbit_file_header(header,amsu_orbit_header)

!@***s* MLT_RSS/AMSU_Level_1B.f90/decode_amsu_l1b_orbit_file_header
!  NAME
!    decode_amsu_l1b_orbit_file_header
! 
!  PURPOSE
!    Decodes header data from binary form to structured form
! 
!  INPUTS
!    character(len=2560)				:: header             ! orbit header data in binary form
!  OUTPUTS
!    type(amsu_L1B_orbit_header)		:: amsu_orbit_header  ! orbit header data in structured form
!    
!  USAGE
!     call encode_amsu_l1b_orbit_file_header(header,amsu_orbit_header)
!@*****

		type(amsu_L1B_orbit_header)		:: amsu_orbit_header
		character(len=2560)				:: header

		integer(4)						:: j

		amsu_orbit_header%satellite_name	=	header(1:7)		 
		amsu_orbit_header%orbit_number		=	string_to_int_4(header(9:12))	
		amsu_orbit_header%number_of_swaths	=	string_to_int_4(header(13:16))
		amsu_orbit_header%start_swath_num	=	string_to_int_4(header(17:20))
		amsu_orbit_header%end_swath_num		=	string_to_int_4(header(21:24))
		amsu_orbit_header%start_rd_2000		=	string_to_real_8(header(25:32))
		amsu_orbit_header%end_rd_2000		=	string_to_real_8(header(33:40))
		amsu_orbit_header%eqx_rd_2000		=	string_to_real_8(header(41:48))
		amsu_orbit_header%pad2				=   header(49:100)

		amsu_orbit_header%calibration		=	header(101:562) 
		amsu_orbit_header%Temp_to_Rad		=	header(563:742) 
		amsu_orbit_header%A1_Digital_Conv	=	header(743:1462) 
		amsu_orbit_header%A2_Digital_Conv	=	header(1463:1766) 

		do j = 1,AMSU_MAX_SCANS_PER_ORBIT
			amsu_orbit_header%scan_valid(j) = char_to_int_1(header(1766 + j:1766 + j))
		enddo
		
end subroutine decode_amsu_l1b_orbit_file_header


subroutine zero_amsu_l1b_orbit_file_header(	amsu_orbit_header)
!@***s* MLT_RSS/AMSU_Level_1B.f90/zero_amsu_l1b_orbit_file_header
!  NAME
!    zero_amsu_l1b_orbit_file_header
! 
!  PURPOSE
!    Zeros header data from binary form to structured form
! 
!  INPUTS_OUTPUTS
!    type(amsu_L1B_orbit_header)		:: amsu_orbit_header  ! orbit header data in structured form
!    
!  USAGE
!     call zero_amsu_l1b_orbit_file_header(amsu_orbit_header)
!@*****
		type(amsu_L1B_orbit_header)		:: amsu_orbit_header

		amsu_orbit_header%satellite_name		= '       '
		amsu_orbit_header%orbit_number			= 0
		amsu_orbit_header%number_of_swaths		= 0
		amsu_orbit_header%start_swath_num		= 0
		amsu_orbit_header%end_swath_num			= 0
		amsu_orbit_header%start_rd_2000			= 0.0D0
		amsu_orbit_header%end_rd_2000			= 0.0D0
		amsu_orbit_header%eqx_rd_2000			= 0.0D0

end subroutine zero_amsu_l1b_orbit_file_header


subroutine decode_amsu_L1B_orbit_scan(raw_scan,						&
								      AMSU_L1B_scan,					&
								      error)
!@***s* MLT_RSS/AMSU_Level_1B.f90/decode_amsu_l1b_orbit_scan
!  NAME
!    decode_amsu_l1b_orbit_scan
! 
!  PURPOSE
!    Decodes scan data from binary form to structured form
! 
!  INPUTS
!    character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_scan	  ! orbit scan data in binary form
!  OUTPUTS
!    type(AMSU_A_L1B_swath_Data)				 :: AMSU_L1B_scan ! orbit scan data in structured form
!    integer(4)                                  :: error         1 error flag, set to 0 if all OK
!  USAGE
!     call decode_amsu_L1B_orbit_scan(raw_scan,AMSU_L1B_scan,error)
!@*****
		character(len = AMSU_LEVEL_1_RECORD_LENGTH)					:: raw_scan			
		type(AMSU_A_L1B_swath_Data)									:: AMSU_L1B_scan
		integer(4)													:: error

		integer(4)													:: k
		integer(4)													:: j
		integer(4)													:: offset

		error = 0

		! Scan Line Information

		AMSU_L1B_scan%scan_line_number				= byte_swap(string_to_int_2(raw_scan(1:2)))
		AMSU_L1B_scan%scan_line_year				= byte_swap(string_to_int_2(raw_scan(3:4)))
		AMSU_L1B_scan%scan_line_day_of_year			= byte_swap(string_to_int_2(raw_scan(5:6)))
		AMSU_L1B_scan%satellite_clock_drift_delta	= byte_swap(string_to_int_2(raw_scan(7:8)))
		AMSU_L1B_scan%scan_line_UTC_time_of_day		= byte_swap(string_to_int_4(raw_scan(9:12)))
		AMSU_L1B_scan%scan_line_bit_field			= byte_swap(string_to_int_2(raw_scan(13:14)))
		AMSU_L1B_scan%major_frame_count				= byte_swap(string_to_int_2(raw_scan(15:16)))
		
		! Quality Indicators

		AMSU_L1B_scan%quality_indicator_bit_field	= byte_swap(string_to_int_4(raw_scan(25:28)))
		AMSU_L1B_scan%scan_line_quality_flags		= byte_swap(string_to_int_4(raw_scan(29:32)))

		do j = 1,16
			offset = 2*(j-1)
			AMSU_L1B_scan%calibration_quality_flags(j) = byte_swap(string_to_int_2(raw_scan(33+offset:34+offset)))
		enddo

		! calibration coefficients

		do j = 1,15
			do k = 1,3
			    offset = 81 + (12*(j-1))+ 4*(k-1)
				AMSU_L1B_scan%primary_calibration(k,j)		= byte_swap(string_to_int_4(raw_scan(offset+81:offset+84)))
				AMSU_L1B_scan%secondary_calibration(k,j)	= byte_swap(string_to_int_4(raw_scan(offset+261:offset+264)))
			enddo
		enddo
		
		AMSU_L1B_scan%navigation_status_bit_field			= byte_swap(string_to_int_4(raw_scan(457:460)))
		AMSU_L1B_scan%time_associated_with_TIP_euler_angles = byte_swap(string_to_int_4(raw_scan(461:464)))

		
		do j = 1,3
			offset = 465 + (j-1)*4
			AMSU_L1B_scan%TIP_euler_angles(j)				= byte_swap(string_to_int_2(raw_scan(offset:offset+3)))
		enddo
		
		AMSU_L1B_scan%spacecraft_altitude					= byte_swap(string_to_int_2(raw_scan(471:472)))
		
		do j = 1,30
			do k = 1,3
				offset = 473 + (6*(j-1)) + 2*(k-1)
				AMSU_L1B_scan%angular_relationships(k,j)	= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
			enddo
		enddo	
		
		do j = 1,30
			do k = 1,2
				offset = 653 + (8*(j-1)) + 4*(k-1)
				AMSU_L1B_scan%earth_location(k,j)			= byte_swap(string_to_int_4(raw_scan(offset:offset+3)))
			enddo
		enddo

		do j = 1,30
			do k = 1,17
				offset = 905 + (34*(j-1)) + 2*(k-1)
				AMSU_L1B_scan%A1_scene_telemetry(k,j)		= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
			enddo
		enddo

		do j = 1,30
			offset = 1925 + 2*(j-1)
			AMSU_L1B_scan%A1_cold_calibration_telemetry(j)	= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
		enddo


		do j = 1,46
		    offset = 1985 + 2*(j-1)
			AMSU_L1B_scan%A1_temperature_sensor_telemetry(j)= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
		enddo
		
		do j = 1,30
			offset = 2077 + 2*(j-1)
			AMSU_L1B_scan%A1_warm_calibration_telemetry(j)	= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
		enddo
		
		AMSU_L1B_scan%A1_invalid_word_bit_flags				= byte_swap(string_to_int_2(raw_scan(2141:2142)))		
		AMSU_L1B_scan%A1_digital_B_telemetry				= byte_swap(string_to_int_2(raw_scan(2143:2144)))
		
		AMSU_L1B_scan%A1_invalid_data_bit_flags				= byte_swap(string_to_int_2(raw_scan(2143:2144)))


		do j = 1,30
			do k = 1,4
				offset = 2193 + 8*(j-1)+ 2*(k-1)
				AMSU_L1B_scan%A2_scene_telemetry(k,j)	= byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
			enddo
		enddo	

		do j = 1,6
		    offset = 2433 + 2*(j-1)
			AMSU_L1B_scan%A2_cold_calibration_telemetry(j) = byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
		enddo																										 
		
		do j = 1,20
			offset = 2445 + 2*(j-1)
			AMSU_L1B_scan%A2_temperature_sensor_telemetry(j) = 	byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
	
		enddo

		do j = 1,6
		    offset = 2485 + 2*(j-1)
			AMSU_L1B_scan%A2_warm_calibration_telemetry(j) = byte_swap(string_to_int_2(raw_scan(offset:offset+1)))
		enddo																										 

		return

end subroutine decode_AMSU_L1B_orbit_scan


subroutine decode_calibration_data(calibration_data,decoded_calibration_data)
!@***s* MLT_RSS/AMSU_Level_1B.f90/decode_calibration_data
!  NAME
!    decode_calibration_data
! 
!  PURPOSE
!    Decodes AMSU calibration data from binary form to structured form
! 
!  INPUTS
!    character(len = 462)				:: calibration_data	  ! calibration data in binary form
!  OUTPUTS
!    type(AMSU_A_L1B_Calibration_Data)	:: decoded_calibration_data ! calibration data in structured form
!  USAGE
!     call decode_calibration_data(calibration_data,decoded_calibration_data)
!@*****
	character(len = 462)				:: calibration_data
	type(AMSU_A_L1B_Calibration_Data)	:: decoded_calibration_data

	integer(4)							:: i,chan,offset


	do i = 1,3 
		offset = (i-1)*2
		decoded_calibration_data%instrument_temperature_sensor_ID(i) = &
			byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 6+2*(i-1)
		decoded_calibration_data%RF_shelf_A1_1_reference_temperature_1(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 12+2*(i-1)
		decoded_calibration_data%RF_shelf_A1_2_reference_temperature(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 18+2*(i-1)
		decoded_calibration_data%RF_shelf_A2_reference_temperature(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 24+2*(i-1)
		decoded_calibration_data%RF_shelf_A1_1_reference_temperature_2(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 30+2*(i-1)
		decoded_calibration_data%RF_mux_A1_1_reference_temperature_1(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 36+2*(i-1)
		decoded_calibration_data%RF_mux_A1_2_reference_temperature(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 42+2*(i-1)
		decoded_calibration_data%RF_mux_A2_reference_temperature(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do i = 1,3
		offset = 48+2*(i-1)
		decoded_calibration_data%RF_mux_A1_1_reference_temperature_2(i) = &
			1.0e-02*byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
	enddo

	do chan = 1,15
		do i = 1,4
			offset = 54+8*(chan-1)+2*(i-1)
			decoded_calibration_data%fixed_bias_corr(chan,i) = &
				1.0e-3 * byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
		enddo
	enddo

	do chan = 9,14
		do i = 1,3
			offset = 174+6*(chan-9) + 2*(i-1)
			decoded_calibration_data%warm_target_bias_corr(chan-8,i) = &
				1.0e-3 * byte_swap(string_to_int_2(calibration_data(offset+1:offset+2)))
		enddo
	enddo

	do chan = 1,15
		do i = 1,3
			offset = 210+12*(chan-1) + 4*(i-1)
			decoded_calibration_data%nonlinearity_coef(chan,i) = 1.0e-6*byte_swap(string_to_int_4(calibration_data(offset+1:offset+4)))
		enddo
	enddo

	do chan = 9,14
		do i = 1,3
			offset = 390+12*(chan-9) + 4*(i-1)
			decoded_calibration_data%nonlinearity_coef_2(chan-8,i) = &
				1.0e-6 * byte_swap(string_to_int_4(calibration_data(offset+1:offset+4)))
		enddo
	enddo
end subroutine decode_calibration_data


subroutine zero_amsu_l1b_scan(amsu_L1B_scan)
!@***s* MLT_RSS/AMSU_Level_1B.f90/zero_amsu_l1b_scan
!  NAME
!    zero_amsu_l1b_scan
! 
!  PURPOSE
!    Zeros AMSU L1B scan data
! 
!  INPUTS_OUTPUTS
!    type(AMSU_A_L1B_Swath_Data)	:: amsu_L1B_scan  ! L1B scan data in structured form
!    
!  USAGE
!     zero_amsu_l1b_scan(amsu_L1B_scan)
!@*****
		type(AMSU_A_L1B_Swath_Data)	:: amsu_L1B_scan

		amsu_L1B_scan%scan_line_number						= 0
		amsu_L1B_scan%scan_line_year						= 0
		amsu_L1B_scan%scan_line_day_of_year					= 0
		amsu_L1B_scan%satellite_clock_drift_delta			= 0			    
		amsu_L1B_scan%scan_line_UTC_time_of_day				= 0			
		amsu_L1B_scan%scan_line_bit_field					= 0
		amsu_L1B_scan%major_frame_count						= 0
		amsu_L1B_scan%quality_indicator_bit_field			= 0
		amsu_L1B_scan%scan_line_quality_flags				= 0
		amsu_L1B_scan%calibration_quality_flags				= 0
	 	amsu_L1B_scan%primary_calibration					= 0				
	 	amsu_L1B_scan%secondary_calibration					= 0			   
	 	amsu_L1B_scan%navigation_status_bit_field			= 0
	 	amsu_L1B_scan%time_associated_with_TIP_euler_angles = 0	
	 	amsu_L1B_scan%TIP_euler_angles						= 0				
	 	amsu_L1B_scan%spacecraft_altitude					= 0				
	 	amsu_L1B_scan%angular_relationships					= 0		
	 	amsu_L1B_scan%earth_location						= 0			
	 	amsu_L1B_scan%A1_synchronization_sequence			= 0
	 	amsu_L1B_scan%A1_unit_identification				= 0
	 	amsu_L1B_scan%A1_digital_housekeeping				= 0
	 	amsu_L1B_scan%A1_scene_telemetry					= 0	
	 	amsu_L1B_scan%A1_cold_calibration_telemetry			= 0	
	 	amsu_L1B_scan%A1_temperature_sensor_telemetry		= 0
	 	amsu_L1B_scan%A1_warm_calibration_telemetry			= 0	
	 	amsu_L1B_scan%A1_invalid_word_bit_flags				= 0
	 	amsu_L1B_scan%A1_digital_B_telemetry				= 0
	 	amsu_L1B_scan%A1_invalid_data_bit_flags				= 0
	 	amsu_L1B_scan%A1_housekeeping						= 0
	 	amsu_L1B_scan%A2_synchronization_sequence			= 0
	 	amsu_L1B_scan%A2_unit_identification				= 0
	 	amsu_L1B_scan%A2_digital_housekeeping				= 0
	 	amsu_L1B_scan%A2_scene_telemetry					= 0	
	 	amsu_L1B_scan%A2_cold_calibration_telemetry			= 0
	 	amsu_L1B_scan%A2_temperature_sensor_telemetry		= 0	
	 	amsu_L1B_scan%A2_warm_calibration_telemetry			= 0	
	 	amsu_L1B_scan%A2_invalid_word_bit_flags				= 0
	 	amsu_L1B_scan%A2_digital_B_telemetry				= 0
	 	amsu_L1B_scan%A2_invalid_data_bit_flags				= 0
	 	amsu_L1B_scan%A2_housekeeping						= 0

end subroutine zero_AMSU_l1b_scan



subroutine read_AMSU_L1A_orbit_file(	satellite_name,			&
										orbit_number,			&
										amsu_orbit_header,		&
										amsu_orbit_data,		&
										error)
										
!@***s* MLT_RSS/AMSU_Level_1B.f90/read_AMSU_L1A_orbit_file
!  NAME
!    read_AMSU_L1A_orbit_file
! 
!  PURPOSE
!    reads AMSU_L1A_orbit_file
! 
!  INPUTS
!    character(len=7)									:: satellite_name
!    integer(4)											:: orbit_number
!  OUTPUTS
!	 type(amsu_L1B_orbit_header)						:: amsu_orbit_header ! AMSU L1B header in structured form
!	 type(amsu_L1A_orbit_data)						    :: amsu_orbit_data   ! AMSU L1A orbit data 
!    integer(4)                                         :: error       ! error flag
!  USAGE
!     call read_AMSU_L1A_orbit_file(satellite_name,orbit_number,amsu_orbit_header,amsu_orbit_data,error)
!@*****

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(amsu_L1B_orbit_header)							:: amsu_orbit_header
		type(amsu_L1A_orbit_data)						    :: amsu_orbit_data
		integer(4)											:: error

		character(len=AMSU_LEVEL_1_RECORD_LENGTH),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: scan_num
		integer(4)						:: scan_line_number
		integer(4)						:: sat_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fuz_err


		type  (FILE$INFO)				:: info

		! find satellite_number

		do sat_num = 10,16
			if (amsu_sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16))  then
			call log('Fatal Error: No Data for that satellite '//satellite_name,log_err)
			stop
		endif

		error = 0

		! zero the amsu_orbit_Scan_data

		call zero_amsu_l1b_orbit_file_header(amsu_orbit_header)
		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			amsu_orbit_data%scan_data(scan_num) = ''
			amsu_orbit_data%scan_valid(scan_num) = 0  
		enddo

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		write(filename,9001)trim(AMSU_L1B_PATH),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
9001	format(a,a,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L0.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)

		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		fuz_err = 0
		if (error == 0) then
			call fastunzip(filename,size,buffer,fuz_err)
			if (fuz_err == 0) then
				call decode_amsu_l1b_orbit_file_header(buffer(1),amsu_orbit_header)

				do scan_num = 2,amsu_orbit_header%number_of_swaths+1

					scan_line_number = byte_swap(string_to_int_2(buffer(scan_num)(1:2)))
					if ((scan_line_number .ge. 1) .and. (scan_line_number .le. AMSU_MAX_SCANS_PER_ORBIT)) then
						amsu_orbit_data%scan_data(scan_line_number)   = buffer(scan_num)
						amsu_orbit_data%scan_valid(scan_line_number)  = 1 ! 1 = from old data
						amsu_orbit_data%header_num_used(scan_line_number)  = amsu_orbit_header%scan_valid(scan_line_number)/8
					endif
					
				enddo
			else
				error = 2
			endif
		endif


  end subroutine read_amsu_L1A_orbit_file


subroutine read_AMSU_L1B_orbit_file(	satellite_name,			&
										orbit_number,			&
										amsu_orbit_header,		&
										amsu_orbit_data,	    &
										error)
!@***s* MLT_RSS/AMSU_Level_1B.f90/read_AMSU_L1B_orbit_file
!  NAME
!    read_AMSU_L1B_orbit_file
! 
!  PURPOSE
!    reads AMSU_L1B_orbit_file
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!  OUTPUTS
!	  type(amsu_L1B_orbit_header)	:: amsu_orbit_header ! orbit header in structured form 
!     type(amsu_L1B_orbit_data)	    :: amsu_orbit_data   ! orbit data in structured form 
!     integer(4)                    :: error             ! error flag
!  USAGE
!     call read_AMSU_L1A_orbit_file(satellite_name,orbit_number,amsu_orbit_header,amsu_orbit_data,error)
!@*****
		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(amsu_L1B_orbit_header)							:: amsu_orbit_header
		type(amsu_L1B_orbit_data)						    :: amsu_orbit_data
		type(AMSU_A_L1B_Swath_Data)							:: amsu_orbit_scan
		integer(4)											:: error

		character(len=AMSU_LEVEL_1_RECORD_LENGTH),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: scan_num
		integer(4)						:: sat_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fuz_err


		type  (FILE$INFO)				:: info

		! find satellite_number

		do sat_num = 10,16
			if (amsu_sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16).or. (sat_num .eq. 15))  then
			call log('Fatal Error: No Data for that satellite '//satellite_name,log_err)
			stop
		endif

		error = 0

		! zero the amsu_orbit_Scan_data

		call zero_amsu_l1b_orbit_file_header(amsu_orbit_header)
		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			call zero_amsu_l1b_scan(amsu_orbit_data%scan_data(scan_num))
			amsu_orbit_data%scan_valid(scan_num) = 0  
		enddo

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		write(filename,9001)trim(AMSU_L1B_PATH),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
9001	format(a,a,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L0.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)

		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		fuz_err = 0
		if (error == 0) then
			call fastunzip(filename,size,buffer,fuz_err)
			call decode_amsu_l1b_orbit_file_header(buffer(1),amsu_orbit_header)

			do scan_num = 2,amsu_orbit_header%number_of_swaths+1

				call decode_SAA_AMSU_swath(buffer(scan_num),amsu_orbit_scan,error)
				amsu_orbit_data%scan_data(amsu_orbit_scan%scan_line_number)   = amsu_orbit_scan
				amsu_orbit_data%scan_valid(amsu_orbit_scan%scan_line_number)  = iand(amsu_orbit_header%scan_valid(amsu_orbit_scan%scan_line_number),7)
				amsu_orbit_data%header_num_used(amsu_orbit_scan%scan_line_number)  = amsu_orbit_header%scan_valid(amsu_orbit_scan%scan_line_number)/8
			enddo
		endif


  end subroutine read_amsu_L1B_orbit_file


subroutine read_AMSU_L1B_orbit_file_header(	satellite_name,			&
											orbit_number,			&
											amsu_orbit_header,		&
											error)
!@***s* MLT_RSS/AMSU_Level_1B.f90/read_AMSU_L1B_orbit_file_header
!  NAME
!    read_AMSU_L1B_orbit_file_header
! 
!  PURPOSE
!    reads header from AMSU_L1B_orbit_file
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!  OUTPUTS
!	  type(amsu_L1B_orbit_header)	:: amsu_orbit_header ! orbit header in structured form 
!     integer(4)                    :: error             ! error flag
!  USAGE
!     call read_AMSU_L1A_orbit_file_header(satellite_name,orbit_number,amsu_orbit_header,error)
!@*****		
        character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(amsu_L1B_orbit_header)							:: amsu_orbit_header
		type(AMSU_A_L1B_Swath_Data)							:: amsu_orbit_scan
		integer(4)											:: test  ! = 0 for operational directory, = 1 for test directory
		integer(4)											:: error

		character(len=AMSU_LEVEL_1_RECORD_LENGTH),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: scan_num
		integer(4)						:: sat_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fuz_err


		type  (FILE$INFO)				:: info

		! find satellite_number

		do sat_num = NOAA_15,AQUA
			if (amsu_sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if ((sat_num .lt. NOAA_15) .or. (sat_num .gt. AQUA))  then
			call log('Fatal Error: No Data for that satellite '//satellite_name,log_err)
			stop
		endif

		error = 0

		! zero the amsu_orbit_Scan_data

		call zero_amsu_l1b_orbit_file_header(amsu_orbit_header)

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		write(filename,9001)trim(AMSU_L1B_PATH),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
		
9001	format(a,a,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L0.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)

		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		
		fuz_err = 0
		if (error == 0) then
			call fastunzip(filename,size,buffer,fuz_err)
			call decode_amsu_l1b_orbit_file_header(buffer(1),amsu_orbit_header)
		endif


  end subroutine read_amsu_L1B_orbit_file_header

  logical function amsu_new_data_present(amsu_orbit_data)

		type(amsu_L1A_orbit_data)		:: amsu_orbit_data
		logical							:: new_data
		integer(4)						:: scan_index

		new_Data = .false.

		do scan_index = 1,AMSU_MAX_SCANS_PER_ORBIT
			if (amsu_orbit_data%scan_valid(scan_index) .ge. 2)	then
				new_data = .true.
			endif
		enddo

		amsu_new_Data_present = new_data
 
  end function amsu_new_data_present

		
  subroutine write_AMSU_L1A_orbit_file(	satellite_name,			&
										orbit_number,			&
										amsu_orbit_data,		&
										num_scans,				&
										eqx_rd_2000,			&
										AMSU_header_data_list,	&
										tot_err,				&
										error)
!@***s* MLT_RSS/AMSU_Level_1B.f90/write_AMSU_L1B_orbit_file
!  NAME
!    write_AMSU_L1B_orbit_file
! 
!  PURPOSE
!    writes AMSU_L1B_orbit_file
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!    type(amsu_L1B_orbit_data)	    :: amsu_orbit_data   ! orbit data in structured form 
!	 integer(4)						:: num_scans    ! number of scans (not used)
!	 real(8)						:: eqx_rd_2000  ! ascending node crossing time
!	 type(AMSU_A_Data_to_Keep_from_Header),dimension(15)	:: AMSU_header_data_list  ! list of header data, including calibration data
!  OUTPUTS
!	  integer(4),dimension(10)      :: tot_err           ! accumulated errors, sorted by type.
!     integer(4)                    :: error             ! error flag
!  USAGE
!     call write_AMSU_L1A_orbit_file(	satellite_name,orbit_number,amsu_orbit_data,num_scans,eqx_rd_2000,AMSU_header_data_list,tot_err,error)
!@*****		

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(amsu_L1B_orbit_header)							:: amsu_orbit_header
		type(amsu_L1A_orbit_data)							:: amsu_orbit_data
		integer(4)											:: num_scans
		real(8)												:: eqx_rd_2000
		type(AMSU_A_Data_to_Keep_from_Header),dimension(15)	:: AMSU_header_data_list
		integer(4),dimension(10)							:: tot_Err
		integer(4)											:: error,date_error

		character(len=AMSU_LEVEL_1_RECORD_LENGTH ),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer
		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: num_scans_valid
		integer(4)						:: num_scans_new
		integer(4),dimension(20)		:: header_data_histogram
		integer(4)						:: header_data_num
		integer(4)						:: max_hist
		integer(4)						:: num_headers
		integer(4)						:: header_to_use

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fz_err
		integer(4)						:: scan_index
		integer(4)						:: first_valid_scan
		integer(4)						:: last_valid_scan
		real(8)							:: start_rd_2000
		real(8)							:: end_rd_2000
		character(len = 20)				:: begin_date_String
		character(len = 20)				:: end_date_String

		real(4)							:: junk1
		real(4)							:: junk2
		integer(4)						:: junk3

		type  (FILE$INFO)				:: info

		! encode all the data into a large array of characters, keeping track of
		! important header info

		error = 0

		num_scans_valid = 0
		num_scans_new   = 0
		header_data_histogram = 0
		do scan_index = 1,AMSU_MAX_SCANS_PER_ORBIT
			if (amsu_orbit_data%scan_valid(scan_index) .gt. 0)	then
				num_scans_valid = num_scans_valid + 1
				if (num_scans_valid .eq. 1) first_valid_scan = scan_index
				last_valid_scan = scan_index 
				buffer(num_scans_valid+1) = amsu_orbit_data%scan_data(scan_index)
				if (amsu_orbit_data%scan_valid(scan_index) .ge. 2) num_scans_new = num_scans_new+1
				header_data_histogram(amsu_orbit_data%header_num_used(scan_index)) =  &
					header_data_histogram(amsu_orbit_data%header_num_used(scan_index)) + 1
			endif
		enddo
		
		if (num_scans_valid .le. 0) then
			error = 1
			return
		endif

		num_headers = 0
		max_hist = 0
		do header_data_num = 1,20
			if (header_data_histogram(header_Data_num) .gt. 0) then
				num_headers = num_headers+1
				if (header_data_histogram(header_Data_num) .gt. max_hist) then
					header_to_use = header_Data_num
					max_hist = header_data_histogram(header_data_num)
				endif
			endif
		enddo

		if (num_headers .gt. 1) then
			write(log_str,112)orbit_number
112			format(' Warning:  Multiple headers for this orbit:',1x,i7)
			call log(log_Str,log_err)
		endif



		amsu_orbit_header%satellite_name	= satellite_name
		amsu_orbit_header%orbit_number		= orbit_number
		amsu_orbit_header%number_of_swaths	= num_scans_valid
		amsu_orbit_header%start_swath_num	= first_valid_scan
		amsu_orbit_header%end_swath_num		= last_valid_Scan
		
		call decode_date_and_earth_loc_from_SAA_AMSU_swath(amsu_orbit_data%scan_data(first_valid_scan),			&
	                                                         start_rd_2000,		&
															 junk1,junk2,junk3,  &
															 error)

		amsu_orbit_header%start_rd_2000		= start_rd_2000

		call decode_date_and_earth_loc_from_SAA_AMSU_swath(amsu_orbit_data%scan_data(last_valid_scan),			&
	                                                         end_rd_2000,		&
															 junk1,junk2,junk3,  &
															 error)

		amsu_orbit_header%end_rd_2000		= end_rd_2000
			    
		amsu_orbit_header%eqx_rd_2000       = eqx_rd_2000

		amsu_orbit_header%Calibration		= AMSU_header_data_list(header_to_use)%Calibration	! byte 101:562
		amsu_orbit_header%Temp_to_Rad		= AMSU_header_data_list(header_to_use)%Temp_to_Rad 	! byte 563:742
		amsu_orbit_header%A1_Digital_Conv	= AMSU_header_data_list(header_to_use)%A1_Digital_Conv !byte 743:1462
		amsu_orbit_header%A2_Digital_Conv	= AMSU_header_data_list(header_to_use)%A2_Digital_Conv  !byte 1463:1766	
		amsu_orbit_header%scan_valid		= amsu_orbit_data%scan_valid  + (8*amsu_orbit_data%header_num_used)


		call encode_amsu_L1b_orbit_file_header(amsu_orbit_header,buffer(1))

		!determine the file name

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		
		write(filename,9001)trim(AMSU_L1B_PATH),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
9001	format(a,a,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L0.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)
	    if (info.length .gt. 0) then
				error = 2				
		endif


		begin_date_string	= date_string(set_date_2000(start_rd_2000,date_error)) 
		end_date_string		= date_string(set_date_2000(end_rd_2000,date_error)) 
		write(log_str,9004)orbit_number,							&
								num_scans_valid,						&
								num_scans_new,							&
								amsu_orbit_header%start_swath_num,		&
								amsu_orbit_header%end_swath_num,		&
								begin_date_string,						&
								end_date_string,tot_err(1:5),num_scans
9004	format(1x,i6,1x,i4,1x,i4,1x,i4,1x,i4,1x,a20,1x,a20,1x,6i6)
		call log(log_str,log_err)

!		if (num_scans_new .le. 0) then
!			error = 4
!			return
!		endif

		
		fz_err = 0
	
		if (error .ge. 0) then
			size = AMSU_LEVEL_1_RECORD_LENGTH*(num_scans_valid+1)
			call fastgzip(filename,size,buffer(1:num_scans_valid+1),fz_err)
		endif
	 	write(log_str,555)num_scans_valid,error,fz_err,trim(filename)
		555 format(i5,1x,i2,1x,i2,1x,a)
		call log(log_str,log_err)


	end subroutine write_AMSU_L1A_orbit_file


	subroutine write_AMSU_Header_Data_List(satellite_name,num_in_list,header_list)
!@***s* MLT_RSS/AMSU_Level_1B.f90/write_AMSU_Header_Data_List
!  NAME
!    write_AMSU_Header_Data_List
! 
!  PURPOSE
!    writes list of unique header data (includes calibration data) 
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)                     :: num_in_list     ! number of unique headers
!	 type(AMSU_A_Data_to_Keep_from_Header),dimension(15)	:: AMSU_header_data_list  ! list of header data, including calibration data
!  USAGE
!     write_AMSU_Header_Data_List(satellite_name,num_in_list,header_list)
!@*****	
		character(len=7)											:: satellite_name
		integer(4)													:: num_in_list
		type(AMSU_A_Data_to_Keep_from_Header),dimension(15)			:: header_list

		character(len = 120)										:: filename
		integer(4)													:: j

		write(filename,9001)trim(AMSU_L1B_PATH),satellite_name,satellite_name
9001	format(a,a,'\',a,'_Header_Data_List.dat')

		open(unit = AMSU_HEADER_LIST_UNIT,file = filename,form = 'binary')
		write(log_str,*)'writing ',num_in_list,' headers into ',filename
		call log(log_str,log_err)
		write(AMSU_HEADER_LIST_UNIT) num_in_list

		do j = 1,num_in_list
			write(AMSU_HEADER_LIST_UNIT)	header_list(j)%Calibration,			&
									header_list(j)%Temp_to_Rad,			& 
									header_list(j)%A1_Digital_Conv,		&
									header_list(j)%A2_Digital_Conv
		enddo

		close(AMSU_HEADER_LIST_UNIT)

	end subroutine write_AMSU_Header_Data_List

	subroutine read_AMSU_Header_Data_List(satellite_name,num_in_list,header_list)
!@***s* MLT_RSS/AMSU_Level_1B.f90/read_AMSU_Header_Data_List
!  NAME
!    read_AMSU_Header_Data_List
! 
!  PURPOSE
!    reads list of unique header data (includes calibration data) 
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!  OUTPUTS
!    integer(4)                     :: num_in_list     ! number of unique headers
!	 type(AMSU_A_Data_to_Keep_from_Header),dimension(15)	:: AMSU_header_data_list  ! list of header data, including calibration data
!  USAGE
!     read_AMSU_Header_Data_List(satellite_name,num_in_list,header_list)
!@*****	

		character(len=7)											:: satellite_name
		integer(4)													:: num_in_list
		type(AMSU_A_Data_to_Keep_from_Header),dimension(15)			:: header_list

		integer(4)													:: j
		integer(4)													:: iostat

		character(len = 120)										:: filename


		write(filename,9001)trim(AMSU_L1B_PATH),trim(satellite_name),trim(satellite_name)
9001	format(a,a,'\',a,'_Header_Data_List.dat')

		open(unit = AMSU_HEADER_LIST_UNIT,file = filename,form = 'binary',status = 'old',iostat = iostat)

		if (iostat .eq. 0) then
			write(log_str,*)'reading header list file ',filename
			call log(log_str,log_err)
			read(AMSU_HEADER_LIST_UNIT) num_in_list
			write(log_str,*)'reading ',num_in_list,' headers'
			call log(log_str,log_err)
				do j = 1,num_in_list
					read(AMSU_HEADER_LIST_UNIT)	header_list(j)%Calibration,			&
												header_list(j)%Temp_to_Rad,			& 
												header_list(j)%A1_Digital_Conv,		&
												header_list(j)%A2_Digital_Conv
				enddo
		else
			num_in_list = 0
		endif


		close(AMSU_HEADER_LIST_UNIT)

	end subroutine read_AMSU_Header_Data_List

	subroutine compare_header_calibration_data_for_critical_changes(calibration_data,calibration_data_last_file,change_critical,header_is_nonsense,write_log)
!@***s* MLT_RSS/AMSU_Level_1B.f90/compare_header_calibration_data_for_critical_changes
!  NAME
!    compare_header_calibration_data_for_critical_changes
! 
!  PURPOSE
!    compares two sets of header data for important (i.e. calibration related) changes 
! 
!  INPUTS
!       type(AMSU_A_Data_to_Keep_from_Header)						:: calibration_data           ! calibration data from new file    
!		type(AMSU_A_Data_to_Keep_from_Header)						:: calibration_data_last_file ! calibration data from last file  
!		logical														:: write_log  ! set to true to write results to log file
!  OUTPUTS
!		logical														:: change_critical   ! set to true if important change is detected
!		logical														:: header_is_nonsense ! set to true if new header is garbled
!  USAGE
!     call compare_header_calibration_data_for_critical_changes(calibration_data,calibration_data_last_file,change_critical,header_is_nonsense,write_log)
!@*****		
	
        type(AMSU_A_Data_to_Keep_from_Header)						:: calibration_data
		type(AMSU_A_Data_to_Keep_from_Header)						:: calibration_data_last_file
		logical														:: change_critical
		logical														:: header_is_nonsense
		logical														:: write_log

		integer(4)			:: wavenumber_int
		integer(4)			:: constant_int
		integer(4)			:: slope_int

		integer(4)			:: wavenumber_int_last
		integer(4)			:: constant_int_last
		integer(4)			:: slope_int_last

		character(len = 4)	:: coeff_str
		real(4),dimension(7,4)	:: warm_load_calibration_coeff
		real(4),dimension(7,4)	:: warm_load_calibration_coeff_last

		integer(4)				:: channel
		integer(4)				:: offset
		integer(4)				:: therm_num
		integer(4)				:: coeff_num




		change_critical = .false.
		header_is_nonsense = .false.

		! At the present (1/2004), we are doing the simple calibration using the scan by scan
		! calibration coefficients

		! The only critical data in the header record is the Temperature to Radiance
		! correction coefficients.  These really should not change, since they depend
		! on physical constants and bandwidths.

		! We also are calculating the warm load temperature to check for hot load
		! calibration problems a la MSU.  I'm not sure why these would change either.

		! I think any changes in these header data should be viewed with suspicion.....


		do  channel = 1,15

			offset = 1+(channel-1)*12

			wavenumber_int		= byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset:offset+3)))
			constant_int		= byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+4:offset+7)))
			slope_int			= byte_swap(string_to_int_4(Calibration_Data%Temp_to_Rad(offset+8:offset+11)))

			wavenumber_int_last	= byte_swap(string_to_int_4(Calibration_Data_last_file%Temp_to_Rad(offset:offset+3)))
			constant_int_last	= byte_swap(string_to_int_4(Calibration_Data_last_file%Temp_to_Rad(offset+4:offset+7)))
			slope_int_last		= byte_swap(string_to_int_4(Calibration_Data_last_file%Temp_to_Rad(offset+8:offset+11)))

			if (wavenumber_int .ne. wavenumber_int_last) then
				write(log_str,111) channel,wavenumber_int,wavenumber_int_last
				!call log(log_str,log_err)
111				format(' Wavenumber for Channel ',i2.2,' Changed ',i12,1x,i12)
				change_critical = .true.
				if ((wavenumber_int .gt. 3000000) .or. (wavenumber_int .lt. 700000)) header_is_nonsense = .true.
			endif

			if (constant_int .ne. constant_int_last) then

				write(log_str,112) channel,constant_int,constant_int_last
112				format(' Constant for Channel ',i2.2,' Changed ',i12,1x,i12)
				!call log(log_str,log_err)
				change_critical = .true.
			endif
	
			if (slope_int .ne. slope_int_last) then
				write(log_str,113) channel,slope_int,slope_int_last
113				format(' Slope for Channel ',i2.2,' Changed ',i12,1x,i12)
				!call log(log_str,log_err)

				change_critical = .true.
				if ((slope_int .gt. 1100000) .or. (slope_int .lt. 900000)) header_is_nonsense = .true.
			endif
		enddo

			! compare the warm load calibration coefficients for each of the
			! thermistors (5 for AMSU_A1-1 (channels 6,7,9-15) and 
			! AMSU_A1-2 (Channels 3,4,5, and 8)
			! and 7 for AMSU A2 (channels 1 and 2)
				

				offset = 560  ! A1-1

				do therm_num = 1,5

					coeff_str = calibration_data%A1_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data%A1_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A1_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A1_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff_last(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff_last(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff_last(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff_last(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					offset = offset + 16

					do coeff_num = 1,4
						if (warm_load_calibration_coeff(therm_num,coeff_num) .ne.   &
						    warm_load_calibration_coeff_last(therm_num,coeff_num)) then
						
							write(log_str,211) coeff_num,therm_num, &
												warm_load_calibration_coeff(therm_num,coeff_num), &
												warm_load_calibration_coeff_last(therm_num,coeff_num) 
211							format(' A1-1 Thermometer Calibration Coeff #',i1,' Changed for Thermometer # ',i1,1x,e15.5,1x,e15.5)
							!call log(log_str,log_err)

						endif
					enddo
				enddo

				offset = 640  ! A1-1

				do therm_num = 1,5

					coeff_str = calibration_data%A1_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data%A1_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A1_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A1_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff_last(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff_last(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff_last(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A1_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff_last(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					offset = offset + 16

					do coeff_num = 1,4
						if (warm_load_calibration_coeff(therm_num,coeff_num) .ne.   &
						    warm_load_calibration_coeff_last(therm_num,coeff_num)) then
							
							write(log_str,212) coeff_num,therm_num,warm_load_calibration_coeff(therm_num,coeff_num),warm_load_calibration_coeff_last(therm_num,coeff_num) 
212							format(' A1-2 Thermometer Calibration Coeff #',i1,' Changed for Thermometer # ',i1,1x,e15.5,1x,e15.5)
							!call log(log_str,log_err)

						endif
					enddo
				enddo

				offset = 212  ! A1-1

				do therm_num = 1,7

					coeff_str = calibration_data%A2_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data%A2_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A2_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data%A2_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					coeff_str = calibration_data_last_file%A2_Digital_Conv(offset+1:offset+4)
					warm_load_calibration_coeff_last(therm_num,1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

					coeff_str = calibration_data_last_file%A2_Digital_Conv(offset+5:offset+8)
					warm_load_calibration_coeff_last(therm_num,2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A2_Digital_Conv(offset+9:offset+12)
					warm_load_calibration_coeff_last(therm_num,3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))
				
					coeff_str = calibration_data_last_file%A2_Digital_Conv(offset+13:offset+16)
					warm_load_calibration_coeff_last(therm_num,4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))


					offset = offset + 16

					do coeff_num = 1,4
						if (warm_load_calibration_coeff(therm_num,coeff_num) .ne.   &
						    warm_load_calibration_coeff_last(therm_num,coeff_num)) then
							write(log_str,213) coeff_num,therm_num,warm_load_calibration_coeff(therm_num,coeff_num),warm_load_calibration_coeff_last(therm_num,coeff_num) 
213							format(' A2 Thermometer Calibration Coeff #',i1,' Changed for Thermometer # ',i1,1x,e15.5,1x,e15.5)
							!call log(log_str,log_err)
						endif
					enddo
				enddo

		end subroutine compare_header_calibration_data_for_critical_changes


    subroutine get_temps_from_L1B_scan(	L1B_scan,							&
										Calibration_data,					&
										instrument_temperatures,			&
										error)

!@***s* MLT_RSS/AMSU_Level_1B.f90/get_temps_from_L1B_scan
!  NAME
!    get_temps_from_L1B_scan
! 
!  PURPOSE
!    extracts intrument temperature data from L1B scan
!  INPUTS
!     type(AMSU_A_L1B_Swath_Data)				:: L1B_scan
!	  type(AMSU_A_Data_to_Keep_from_Header)	:: calibration_data														:: write_log  ! set to true to write results to log file
!  OUTPUTS
!		real(4),dimension(45)					:: instrument_temperatures   ! array of instrument temperatures
!		integer(4)								:: error                     ! error flag
!  USAGE
!     call get_temps_from_L1B_scan(	L1B_scan,Calibration_data,instrument_temperatures,error)
!@*****	
		use type_conversion

		type(AMSU_A_L1B_Swath_Data)				:: L1B_scan
		type(AMSU_A_Data_to_Keep_from_Header)	:: calibration_data
		real(4),dimension(45)					:: instrument_temperatures
		integer(4)								:: error


		integer(4)								:: offset
		integer(4)								:: offset2
		integer(4)								:: temp_num


		real(4),dimension(4)					:: calibration_coeff
		character(len = 4)						:: coeff_str
		

		
		do temp_num = 1,45 
			offset = (temp_num-1)*16
			coeff_str = calibration_data%A1_Digital_conv(offset+1:offset+4)		! SF 4/9/16/20
			calibration_coeff(1) = 1.0e-4*byte_swap(string_to_int_4(coeff_str))

			coeff_str = calibration_data%A1_Digital_conv(offset+5:offset+8)		! SF 4/9/16/20
			calibration_coeff(2) = 1.0e-9*byte_swap(string_to_int_4(coeff_str))

			coeff_str = calibration_data%A1_Digital_conv(offset+9:offset+12)		! SF 4/9/16/20
			calibration_coeff(3) = 1.0e-16*byte_swap(string_to_int_4(coeff_str))

			coeff_str = calibration_data%A1_Digital_conv(offset+13:offset+16)		! SF 4/9/16/20
			calibration_coeff(4) = 1.0e-20*byte_swap(string_to_int_4(coeff_str))

			instrument_temperatures(temp_num)  = calibration_coeff(1) +							&
				(L1B_scan%A1_temperature_sensor_telemetry(temp_num) * (calibration_coeff(2) +	& 
				(L1B_scan%A1_temperature_sensor_telemetry(temp_num) * (calibration_coeff(3) +	& 
				(L1B_scan%A1_temperature_sensor_telemetry(temp_num) * (calibration_coeff(4)))))))
		enddo
	end subroutine get_temps_from_L1B_scan


end module AMSU_Level_1B

