$freeform
!@***m* MLT_RSS/AMSU_Level_2A.f90
!  NAME
!    AMSU_Level_2A.f90
! 
!  PURPOSE
!    Module for reading, writing and storing AMSU L2A data
! 
!  DESCRIPTION
!    Module for reading, writing and storing AMSU L2A data.  L2A files are stored in flat binary form,
!    so there is quite a bit of code that converts from the flat binary form to easier to understand
!    fortran structures.  Structures are defined in AMSU_types.f90
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_Level_2A
!@*****
module AMSU_Level_2A

		use amsu_types 
		use amsu_constants
		use AMSU_file_locations
		use NAN_support
		use Date_and_Time
		use type_conversion
		use byte_swapping
		use msflib
		use gzip

		implicit none


contains


	subroutine write_AMSU_L2A_orbit_file(	satellite_name,				&
											orbit_num,					& 
											channel,					&
											L2A_orbit_header,			&
											L2A_orbit,					&
											error)
!@***s* MLT_RSS/AMSU_Level_2A.f90/write_AMSU_L2A_orbit_file
!  NAME
!    write_AMSU_L2A_orbit_file
! 
!  PURPOSE
!    writes AMSU_L2A_orbit_file
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!    integer(4)                     :: channel         ! AMSU channel
!    type(AMSU_L2A_orbit_header)    :: L2A_orbit_header  ! in structured from
!	 type(AMSU_L2A_orbit_data)		:: L2A_orbit         ! in structured from
!  OUTPUTS
!     integer(4)                    :: error             ! error flag
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!     call write_AMSU_L1A_orbit_file(	satellite_name,orbit_num,channel,L2A_orbit_header,L2A_orbit,error)
!@*****

		character(len = 7)					:: satellite_name
		integer(4)							:: orbit_num
		integer(4)							:: channel
		type(AMSU_L2A_orbit_header)			:: L2A_orbit_header
		type(AMSU_L2A_orbit_data)			:: L2A_orbit
		integer(4)							:: error

		character(len=AMSU_LEVEL_2A_RECORD_LENGTH ),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer
		character(len=AMSU_LEVEL_2A_RECORD_LENGTH ) :: scan_string
		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: num_scans_valid
		integer(4)						:: num_scans_new

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fz_err
		integer(4),save					:: scan_index
		integer(4)						:: first_valid_scan
		integer(4)						:: last_valid_scan
		real(8)							:: start_rd_2000
		real(8)							:: end_rd_2000
		character(len = 20)				:: begin_date_String
		character(len = 20)				:: end_date_String
		integer(4)						:: date_error

		type  (FILE$INFO)				:: info

		! encode all the data into a large array of characters, keeping track of
		! important header info

		error = 0

		num_scans_valid = 0
		num_scans_new   = 0
		do scan_index = 1,AMSU_MAX_SCANS_PER_ORBIT
			if ((L2A_orbit%scan_valid(scan_index) .gt. 0) .and. &
				(finite(L2A_orbit%scan_data(scan_index)%scan_rd_2000)))	then
				num_scans_valid = num_scans_valid + 1
				if (num_scans_valid .eq. 1) then
					first_valid_scan = scan_index
					start_rd_2000 = L2A_orbit%scan_data(scan_index)%scan_rd_2000
				endif
				last_valid_scan = scan_index

				call encode_AMSU_L2A_scan(L2A_orbit%scan_data(scan_index),scan_string)
				 
				buffer(num_scans_valid+1) = scan_string
				if (L2A_orbit%scan_valid(scan_index) .ge. 2) num_scans_new = num_scans_new+1
			endif
		enddo
		
		if (num_scans_valid .le. 0) then
			error = 1
			return
		endif

		end_rd_2000 = L2A_orbit%scan_data(last_valid_scan)%scan_rd_2000

		L2A_orbit_header%satellite_name		 = satellite_name
		L2A_orbit_header%orbit_number		 = orbit_num
		L2A_orbit_header%number_of_swaths    = num_scans_valid
		L2A_orbit_header%start_swath_num     = first_valid_scan
		L2A_orbit_header%end_swath_num       = last_valid_scan
		L2A_orbit_header%start_rd_2000		 = start_rd_2000
		L2A_orbit_header%end_rd_2000		 = end_rd_2000
		!L2A_orbit_header%eqx_rd_2000 is already set from next level up
		L2A_orbit_header%channel			 = channel
		print *,'L2A ',orbit_num,L2A_orbit_header%start_rd_2000,L2A_orbit_header%end_rd_2000
		print *
		call encode_AMSU_L2A_header(L2A_orbit_header,				&
									L2A_orbit%scan_valid,			&
									buffer(1)) 

			!determine the file name

		range_num = orbit_num/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

			
		write(filename,9001)trim(AMSU_L2A_PATH),trim(satellite_name),channel,orbit_range_lower,orbit_range_upper,trim(satellite_name),orbit_num
9001	format(a,a,'\Channel_',i2.2,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L2A.gz')
	
	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)
	    if (info.length .gt. 0) then
				error = 2				
		endif

		if (log_unit .gt. 0) then
			begin_date_string	= date_string(set_date_2000(start_rd_2000,date_error)) 
			end_date_string		= date_string(set_date_2000(end_rd_2000,date_error)) 
			write(log_unit,9004)orbit_num,								&
								num_scans_valid,						&
								num_scans_new,							&
								L2A_orbit_header%start_swath_num,		&
								L2A_orbit_header%end_swath_num,			&
								begin_date_string,						&
								end_date_string,num_scans_valid
9004		format(1x,i6,1x,i4,1x,i4,1x,i4,1x,i4,1x,a20,1x,a20,1x,6i6)
		endif

		if (num_scans_new .le. 0) then
			error = 4
			return
		endif

		
		fz_err = 0
	
		if (error .ge. 0) then
			size = AMSU_LEVEL_2A_RECORD_LENGTH*(num_scans_valid+1)
			call fastgzip(filename,size,buffer(1:num_scans_valid+1),fz_err)
		endif
		if (fz_err .ne. 0) then 
			error = -1
		else
			write(log_unit,9005) trim(filename)
 			write(*,9005) trim(filename)
 9005       format('Successfully wrote ',a)
		endif


	end subroutine write_AMSU_L2A_orbit_file

	subroutine read_AMSU_L2A_orbit_file(satellite_name,				&
										orbit_number,					& 
										channel,					&
										L2A_orbit_header,			&
										L2A_orbit,					&
										error)
!@***s* MLT_RSS/AMSU_Level_2A.f90/read_AMSU_L2A_orbit_file
!  NAME
!    read_AMSU_L2A_orbit_file
! 
!  PURPOSE
!    reads AMSU_L2A_orbit_file
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!    integer(4)                     :: channel         ! AMSU channel
!  OUTPUTS
!    type(AMSU_L2A_orbit_header)    :: L2A_orbit_header  ! in structured from
!	 type(AMSU_L2A_orbit_data)		:: L2A_orbit         ! in structured from
!    integer(4)                    :: error             ! error flag
!  USAGE
!     call write_AMSU_L1A_orbit_file(	satellite_name,orbit_num,channel,L2A_orbit_header,L2A_orbit,error)
!@*****

		character(len = 7)					:: satellite_name
		integer(4)							:: orbit_number
		integer(4)							:: channel
		type(AMSU_L2A_orbit_header)			:: L2A_orbit_header
		type(AMSU_L2A_orbit_data)			:: L2A_orbit
		integer(4)							:: error


		character(len=AMSU_LEVEL_2A_RECORD_LENGTH),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer

		type(AMSU_A_L2A_Swath_Data)		:: L2A_Scan

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: scan_num
		integer(4)						:: sat_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fuz_err


		type  (FILE$INFO)				:: info

		! find satellite_number

		do sat_num = NOAA_15,AQUA
			if (amsu_sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if ((sat_num .lt. NOAA_15) .or. (sat_num .gt. NOAA_19))  then
			print *,'No Data for that satellite'
			print *, satellite_name
			stop
		endif

		error = 0

		! zero the L2A_scan_data

		buffer = ' '

		call zero_amsu_L2A_orbit_file_header(L2A_orbit_header)
		do scan_num = 1,AMSU_MAX_SCANS_PER_ORBIT
			call zero_amsu_L2A_scan(L2A_orbit%scan_data(scan_num))
			L2A_orbit%scan_valid(scan_num) = 0  
		enddo

		! determine the file name

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		write(filename,9001)trim(AMSU_L2A_PATH),trim(satellite_name),channel,orbit_range_lower,orbit_range_upper,trim(satellite_name),orbit_number
9001	format(a,a,'\Channel_',i2.2,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L2A.gz')

        if ((channel .ge. 10) .and. (sat_num .eq. AQUA)) then 
			write(filename,9002)trim(AMSU_L2A_PATH),trim(satellite_name),channel,trim(satellite_name),orbit_number
9002	    format(a,a,'\Channel_',i2.2,'\unadjusted\',a,'_r',i5.5,'.L2A.unadj.gz')
        endif

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)
		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		fuz_err = 0
		if (error == 0) then	
		!	print *,'reading ',filename
			call fastunzip(filename,size,buffer,fuz_err)
			call decode_amsu_L2A_header(buffer(1),L2A_orbit_header,L2A_orbit%scan_valid)

			do scan_num = 2,L2A_orbit_header%number_of_swaths+1
				call decode_amsu_L2A_scan(buffer(scan_num),L2A_scan)
				if ((L2A_scan%scan_line_in_orbit .gt. 0) .and.  &
				    (L2A_scan%scan_line_in_orbit .le. AMSU_MAX_SCANS_PER_ORBIT)) then
					L2A_orbit%scan_data(L2A_scan%scan_line_in_orbit)   = L2A_scan
				endif
			enddo
		endif


	end subroutine read_amsu_L2A_orbit_file



	subroutine read_AMSU_L2A_orbit_file_header(satellite_name,		&
										orbit_number,				& 
										channel,					&
										L2A_orbit_header,			&
										error)
!@***s* MLT_RSS/AMSU_Level_2A.f90/read_AMSU_L2A_orbit_file_header
!  NAME
!    read_AMSU_L2A_orbit_file_header 
! 
!  PURPOSE
!    reads AMSU_L2A orbit file header
! 
!  INPUTS
!    character(len=7)				:: satellite_name  ! satellite name (NOAA-15, etc)
!    integer(4)						:: orbit_number    ! AMSU orbit number
!    integer(4)                     :: channel         ! AMSU channel
!  OUTPUTS
!    type(AMSU_L2A_orbit_header)    :: L2A_orbit_header  ! in structured from
!    integer(4)                    :: error             ! error flag
!  USAGE
!     call write_AMSU_L1A_orbit_file_header(satellite_name,orbit_num,channel,L2A_orbit_header,error)
!@*****
		character(len = 7)					:: satellite_name
		integer(4)							:: orbit_number
		integer(4)							:: channel
		type(AMSU_L2A_orbit_header)			:: L2A_orbit_header
		integer(4)							:: error


		character(len=AMSU_LEVEL_2A_RECORD_LENGTH),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: sat_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fuz_err

		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: scan_valid



		type  (FILE$INFO)				:: info

		! find satellite_number

		do sat_num = NOAA_15,AQUA
			if (amsu_sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if ((sat_num .lt. NOAA_15) .or. (sat_num .gt. AQUA))  then
			print *,'No Data for that satellite'
			print *, satellite_name
			stop
		endif

		error = 0

		! zero the L2A Header

		call zero_amsu_L2A_orbit_file_header(L2A_orbit_header)

		! determine the file name

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999
		write(filename,9001)trim(AMSU_L2A_PATH),trim(satellite_name),channel,orbit_range_lower,orbit_range_upper,trim(satellite_name),orbit_number
9001	format(a,a,'\Channel_',i2.2,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L2A.gz')
			

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)
		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		fuz_err = 0
		buffer(1) = ' '
		if (error == 0) then
			call fastunzip(filename,size,buffer,fuz_err)
			call decode_amsu_L2A_header(buffer(1),L2A_orbit_header,scan_valid)
		endif
	end subroutine read_amsu_L2A_orbit_file_header


	subroutine encode_AMSU_L2A_Header(	L2A_orbit_header,		&
										scan_valid,				&
										scan_string) 
!@***s* MLT_RSS/AMSU_Level_2A.f90/encode_AMSU_L2A_Header
!  NAME
!    encode_AMSU_L2A_Header
! 
!  PURPOSE
!    encodes AMSU L2A Header from structured to binary form
! 
!  INPUTS
!    type(AMSU_L2A_orbit_header)                     :: L2A_orbit_header  ! in structured from
!    integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)	 :: scan_valid        ! positive value means scan is valid
!    integer(4)                     :: channel         ! AMSU channel
!  OUTPUTS
!    character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string    ! header and scan valid data in bianry form for writing to L2A file
!  USAGE
!     call encode_AMSU_L2A_Header(	L2A_orbit_header,scan_valid,scan_string) 
!@*****

		type(AMSU_L2A_orbit_header)						:: L2A_orbit_header
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: scan_valid
		character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string
		integer(1)										:: packed_scan_valid_byte
		integer(4)										:: j
		
		scan_string = ''
		scan_string(1:7)	= L2A_orbit_header%satellite_name
		scan_string(9:12)	= int_4_to_string(L2A_orbit_header%orbit_number)
		scan_string(13:16)	= int_4_to_string(L2A_orbit_header%number_of_swaths) 
		scan_string(17:20)	= int_4_to_string(L2A_orbit_header%start_swath_num) 
		scan_string(21:24)	= int_4_to_string(L2A_orbit_header%end_swath_num) 
		scan_string(25:32)	= real_8_to_string(L2A_orbit_header%start_rd_2000)
		scan_string(33:40)	= real_8_to_string(L2A_orbit_header%end_rd_2000)
		scan_string(41:48)	= real_8_to_string(L2A_orbit_header%eqx_rd_2000)
		scan_string(49:52)  = int_4_to_string(L2A_orbit_header%channel)

		!pack scan valid infor into half bytes

		do j = 1,AMSU_MAX_SCANS_PER_ORBIT/2
			packed_scan_valid_byte =   iand(scan_valid((2*j)-1),15) + &
			                       16* iand(scan_valid((2*j)),15)
			scan_string(j+100:j+100) = int_1_to_char(packed_scan_valid_byte)
		enddo
        do j = 1,30 
			scan_string(492+(j*4):495+(j*4)) = real_4_to_string(L2A_orbit_header%cross_track_correction(j))
		enddo
		scan_string(616:635) = L2A_orbit_header%L2A_processor_ID
		scan_string(636:655) = L2A_orbit_header%L1B_processor_ID

	end subroutine encode_AMSU_L2A_Header

	subroutine decode_AMSU_L2A_Header(	scan_string,			&
										L2A_orbit_header,		&
										scan_valid)   
!@***s* MLT_RSS/AMSU_Level_2A.f90/decode_AMSU_L2A_Header
!  NAME
!    decode_AMSU_L2A_Header
! 
!  PURPOSE
!    decodes AMSU L2A Header in binary form to structured form
! 
!  INPUTS
!    character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string    ! header and scan valid data in bianry form for writing to L2A file
!  OUTPUTS
!    type(AMSU_L2A_orbit_header)                     :: L2A_orbit_header  ! in structured from
!    integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)	 :: scan_valid        ! positive value means scan is valid
!  USAGE
!     call decode_AMSU_L2A_Header(scan_string,L2A_orbit_header,scan_valid)
!@*****
 		character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string
		type(AMSU_L2A_orbit_header)						:: L2A_orbit_header
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: scan_valid
		integer(1)										:: packed_scan_valid_byte
		integer(4)										:: j
		
		L2A_orbit_header%satellite_name = scan_string(1:7)	 
		L2A_orbit_header%orbit_number   = string_to_int_4(scan_string(9:12))
		L2A_orbit_header%number_of_swaths = string_to_int_4(scan_string(13:16)) 
		L2A_orbit_header%start_swath_num = string_to_int_4(scan_string(17:20)) 
		L2A_orbit_header%end_swath_num = string_to_int_4(scan_string(21:24)) 
		L2A_orbit_header%start_rd_2000 = string_to_real_8(scan_string(25:32))
		L2A_orbit_header%end_rd_2000 = string_to_real_8(scan_string(33:40))
		L2A_orbit_header%eqx_rd_2000 = string_to_real_8(scan_string(41:48))
		L2A_orbit_header%channel = string_to_int_4(scan_string(49:52))

		!pack scan valid infor into half bytes

		do j = 1,AMSU_MAX_SCANS_PER_ORBIT/2
			packed_scan_valid_byte =   char_to_int_1(scan_string(j+100:j+100))
			
			scan_valid((2*j)-1) = iand(packed_scan_valid_byte,15)
			scan_valid(2*j)     = packed_scan_valid_byte/16
		enddo

	end subroutine decode_AMSU_L2A_Header

	subroutine encode_AMSU_L2A_scan(AMSU_L2A_scan,scan_string)
!@***s* MLT_RSS/AMSU_Level_2A.f90/encode_AMSU_L2A_scan
!  NAME
!    encode_AMSU_L2A_scan
! 
!  PURPOSE
!    encodes AMSU L2A scan from structured to binary form
! 
!  INPUTS
!    type(AMSU_A_L2A_Swath_Data)						:: AMSU_L2A_scan
!  OUTPUTS
!    character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string    ! header and scan valid data in bianry form for writing to L2A file
!  USAGE
!     call encode_AMSU_L2A_scan(AMSU_L2A_scan,scan_string)
!@*****
		type(AMSU_A_L2A_Swath_Data)						:: AMSU_L2A_scan
		character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string

		integer(4)	:: fov,fov_4
		
		scan_string = ''

		scan_string(1:7)	= AMSU_L2A_scan%satellite_name							! byte 1:7
		scan_string(8:8)	= int_1_to_char(AMSU_L2A_scan%channel_num)				! byte 8:8
		scan_string(9:12)	= int_4_to_string(AMSU_L2A_scan%orbit_number)			! byte 9:12
		scan_string(13:16)	= int_4_to_string(AMSU_L2A_scan%scan_line_in_orbit)		! byte 13:16
		scan_string(17:24)	= real_8_to_string(AMSU_L2A_scan%scan_rd_2000)			! byte 16:24

		do fov = 1,30
		    fov_4 = 4*fov
			scan_string(21+fov_4:24+fov_4)		= real_4_to_string(AMSU_L2A_scan%fov_latitude(fov))
			scan_string(141+fov_4:144+fov_4)	= real_4_to_string(AMSU_L2A_scan%fov_longitude(fov))
			scan_string(264+fov:264+fov)		= int_1_to_char(AMSU_L2A_scan%fov_quality(fov))
			scan_string(297+fov_4:300+fov_4)	= real_4_to_string(AMSU_L2A_scan%Tbs(fov))
			scan_string(417+fov_4:420+fov_4)	= real_4_to_string(AMSU_L2A_scan%Diurnal_Corr(fov))
			scan_string(537+fov_4:540+fov_4)	= real_4_to_string(AMSU_L2A_scan%Nadir_Corr(fov))
			scan_string(657+fov_4:660+fov_4)	= real_4_to_string(AMSU_L2A_scan%Incidence_Corr(fov))
		enddo
		
		scan_string(295:298) = real_4_to_string(AMSU_L2A_scan%warm_load_temp)
		scan_String(299:300) = int_2_to_string(AMSU_L2A_scan%asc_desc)
		scan_string(781:784) = real_4_to_string(AMSU_L2A_scan%spacecraft_altitude)			

	end subroutine encode_AMSU_L2A_scan

	subroutine decode_AMSU_L2A_scan(scan_string,AMSU_L2A_scan)
!@***s* MLT_RSS/AMSU_Level_2A.f90/decode_AMSU_L2A_scan
!  NAME
!    decode_AMSU_L2A_scan
!  PURPOSE
!    decodes AMSU L2A scan to structured from binary form
!  INPUTS
!    character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string    ! header and scan valid data in bianry form for writing to L2A file
!  OUTPUTS
!    type(AMSU_A_L2A_Swath_Data)						:: AMSU_L2A_scan
!  USAGE
!     call encode_AMSU_L2A_scan(AMSU_L2A_scan,scan_string)
!@*****
		character(len = AMSU_LEVEL_2A_RECORD_LENGTH)		:: scan_string
		type(AMSU_A_L2A_Swath_Data)						:: AMSU_L2A_scan

		integer(4)	:: fov,fov_4
		
		AMSU_L2A_scan%satellite_name		= scan_string(1:7) 							! byte 1:7
		AMSU_L2A_scan%channel_num			= char_to_int_1(scan_string(8:8))			! byte 8:8
		AMSU_L2A_scan%orbit_number			= string_to_int_4(scan_string(9:12))		! byte 9:12
		AMSU_L2A_scan%scan_line_in_orbit	= string_to_int_4(scan_string(13:16))		! byte 13:16
		AMSU_L2A_scan%scan_rd_2000			= string_to_real_8(scan_string(17:24))		! byte 16:24

		do fov = 1,30
		    fov_4 = 4*fov
			AMSU_L2A_scan%fov_latitude(fov)		= string_to_real_4(scan_string(21+fov_4:24+fov_4))	
			AMSU_L2A_scan%fov_longitude(fov)	= string_to_real_4(scan_string(141+fov_4:144+fov_4))
			AMSU_L2A_scan%fov_quality(fov)		= char_to_int_1(scan_string(264+fov:264+fov))
			AMSU_L2A_scan%Tbs(fov)				= string_to_real_4(scan_string(297+fov_4:300+fov_4))
			AMSU_L2A_scan%Diurnal_Corr(fov)		= string_to_real_4(scan_string(417+fov_4:420+fov_4))
			AMSU_L2A_scan%Nadir_Corr(fov)		= string_to_real_4(scan_string(537+fov_4:540+fov_4))
			AMSU_L2A_scan%Incidence_Corr(fov)	= string_to_real_4(scan_string(657+fov_4:660+fov_4))
		enddo
		
		AMSU_L2A_scan%warm_load_temp     		= string_to_real_4(scan_string(295:298))
		AMSU_L2A_scan%asc_desc					= string_to_int_2(scan_string(299:300))
		AMSU_L2A_scan%spacecraft_altitude       = string_to_real_4(scan_string(781:784))			
			

	end subroutine decode_AMSU_L2A_scan


	
	subroutine zero_amsu_L2A_scan(L2A_Scan)
!@***s* MLT_RSS/AMSU_Level_2A.f90/zero_amsu_L2A_scan
!  NAME
!    zero_amsu_L2A_scan
!  PURPOSE
!    zeros AMSU L2A scan in structured form
!  INPUTS_OUTPUTS
!    type(AMSU_A_L2A_Swath_Data)						:: AMSU_L2A_scan
!  USAGE
!     call zero_amsu_L2A_scan(L2A_Scan)
!@*****
		type(AMSU_A_L2A_Swath_Data)  :: L2A_Scan

		L2A_Scan%satellite_name = '       '	  ! byte 1:7
		L2A_Scan%channel_num			= -1  ! byte 8:8
		L2A_Scan%orbit_number			= -1  ! byte 9:12
		L2A_Scan%scan_line_in_orbit	    = -1  ! byte 13:16
		L2A_Scan%scan_rd_2000			= NAN ! byte 16:24
		L2A_Scan%fov_latitude			= NAN ! byte 25:144
		L2A_Scan%fov_longitude			= NAN ! byte 145:264
		L2A_Scan%fov_quality			= -1  ! byte 265:294
		L2A_Scan%warm_load_temp			= NAN ! byte 295:298
		L2A_Scan%asc_desc				= 0   !byte 299:300  (1 = asc, 2 = desc, 0 = undetermined)
		L2A_Scan%Tbs					= NAN ! byte 301:420
		L2A_Scan%Diurnal_Corr			= NAN ! byte 421:540
		L2A_Scan%Nadir_Corr				= NAN ! byte 541:660
		L2A_Scan%Incidence_Corr			= NAN ! byte 6611:780


   	end subroutine zero_amsu_L2A_scan

	subroutine zero_amsu_L2A_orbit_file_header(L2A_orbit_header)
!@***s* MLT_RSS/AMSU_Level_2A.f90/ero_amsu_L2A_orbit_file_header
!  NAME
!    zero_amsu_L2A_orbit_file_header
!  PURPOSE
!    zeros AMSU L2A scan in structured form
!  INPUTS_OUTPUTS
!		type(AMSU_L2A_orbit_header)			:: L2A_orbit_header
!  USAGE
!     callzero_amsu_L2A_orbit_file_header(L2A_orbit_header)
!@*****
		type(AMSU_L2A_orbit_header)			:: L2A_orbit_header

		L2A_orbit_header%satellite_name			= '       '	! byte 1:7
		L2A_orbit_header%orbit_number			= -1		! byte 9:12
		L2A_orbit_header%number_of_swaths		= -1		! byte 13:16
		L2A_orbit_header%start_swath_num		= -1		! byte 17:20
		L2A_orbit_header%end_swath_num			= -1		! byte 21:24
		L2A_orbit_header%start_rd_2000			= NAN		! byte 25:32
		L2A_orbit_header%end_rd_2000			= NAN		! byte 33:40
		L2A_orbit_header%eqx_rd_2000			= NAN		! byte 41:48
		L2A_orbit_header%channel				= - 1		! byte 49:52
		L2A_orbit_header%scan_valid				= 0			!byte 101:495  packed 2 per byte

	end subroutine zero_amsu_L2A_orbit_file_header








end module AMSU_Level_2A

