$freeform
!@***m* MLT_RSS/AMSU_Quality.f90
!  NAME
!    AMSU_Quality.f90
! 
!  PURPOSE
!    Module for performing rudimentary quality control for AMSU brightness temperatures
! 
!  DESCRIPTION
!    Module for performing rudimentary quality control for AMSU brightness temperatures
!
!  ROUTINES
!     subroutine evaluate_amsu_quality_indicator(quality_indicator,scan_ok)  ! evaluates NOAA quality bits for fatal problems
!     subrouinte check_tbs_against_bounds(Tbs,chan,fov_qual)                 ! checks TBs against reasonableness bounds for each channel
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_quality
!@*****

module AMSU_quality

	use NAN_support

	implicit none

	real(4),parameter,dimension(16)   :: lower_tb_bound = (/100.0,100.0,100.0,100.0,		&
															170.0,100.0,100.0,100.0,		&
															160.0,160.0,160.0,100.0,		&
															100.0,100.0,100.0,100.0/)

	real(4),parameter,dimension(16)   :: upper_tb_bound = (/320.0,320.0,320.0,320.0,		&
															290.0,290.0,280.0,270.0,		&
															270.0,290.0,290.0,320.0,		&
															300.0,300.0,320.0,260.0/)


	

contains


	subroutine evaluate_amsu_quality_indicator(quality_indicator,scan_ok)
!@***m* MLT_RSS/AMSU_Quality.f90/evaluate_amsu_quality_indicator
!  NAME
!    evaluate_amsu_quality_indicator
! 
!  PURPOSE
!    Evaluates NOAA provided quality flags for fatal errors
! 
!  DESCRIPTION
!    Evaluates NOAA provided quality flags for fatal errors
!
!  INPUTS
!    integer(4)   :: quality indicator   ! 4 byte integer of quality indicator flags
!    
!  OUTPUTS
!    logical(4)   :: scan_ok  !  flag -- set true of scan appears to be OK
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    call evaluate_amsu_quality_indicator(quality_indicator,scan_ok)
!@*****

		integer(4)		:: quality_indicator
		logical(4)		:: scan_ok

		scan_ok = .true.

		if (btest(quality_indicator,31)) then ! do not use for product
			scan_ok = .false.
			return
		endif

		if (btest(quality_indicator,30)) then ! time sequence error
			scan_ok = .false.
			return
		endif

		if (btest(quality_indicator,28)) then ! insufficient data for calibration
			scan_ok = .false.
			return
		endif
		if (btest(quality_indicator,27)) then ! earth loc not available
			scan_ok = .false.
			return
		endif
		if (btest(quality_indicator,1)) then  ! major frame error
			scan_ok = .false.
			return
		endif
		if (btest(quality_indicator,0)) then  ! parity error
			scan_ok = .false.
			return
		endif

	end subroutine evaluate_amsu_quality_indicator 

	subroutine check_tbs_against_bounds(Tbs,chan,fov_qual)

!@***m* MLT_RSS/AMSU_Quality.f90/check_tbs_against_bounds
!  NAME
!    check_tbs_against_bounds
! 
!  PURPOSE
!    Checks brightness temperatures agains reasonableness bounds.
! 
!  DESCRIPTION
!    Checks brightness temperatures agains reasonableness bounds.
!
!  INPUTS
!    integer(4)   :: quality indicator   ! 4 byte integer of quality indicator flags
!    
!  OUTPUTS
!    integer(1),dimension(30)        :: fov_qual           !  field of view flag -- set to negative number if out of bounds
!                                                          !  or not finite
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    call check_tbs_against_bounds(Tbs,chan,fov_qual)
!@*****
		real(4),dimension(30)			:: Tbs
		integer(4)						:: chan
		integer(1),dimension(30)        :: fov_qual

		integer(4)						:: fov

		do fov = 1,30
			if (.not. finite(Tbs(fov))) then
			!	PRINT *,'Finite Error'
				fov_qual(fov) = -1
				cycle
			endif
			if (tbs(fov) < lower_tb_bound(chan)) then
			!	PRINT *,'Lower Bounds Error'
				fov_qual(fov) = -2
				cycle
			endif
			if (tbs(fov) > upper_tb_bound(chan)) then
			!	PRINT *,'Upper Bounds Error'
				fov_qual(fov) = -2
				cycle
			endif
		enddo
		return
	end subroutine check_tbs_against_bounds

				


end module AMSU_quality