$freeform
module AMSU_SAA
!@***m* MLT_RSS/AMSU_SAA.f90
!  NAME
!    module AMSU_SAA
!
!  PURPOSE
!    module containing routines for reading and decoding AMSU L1A files from
!    CLASS
!
!  DESCRIPTION
!    module containing routines for reading and decoding AMSU L1A files from
!    CLASS.  
!
!  ROUTINES
!      read_SAA_AMSU_archive_header
!      read_raw_AMSU_header
!      get_AMSU_calibration_data_from_header
!      read_SAA_AMSU_header
!      read_SAA_AMSU_swath
!      decode_SAA_AMSU_swath
!      find_rd_2000_from_AMSU_L1A_Swath
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    use msu_date
!
!@*****
		use AMSU_constants
		use AMSU_file_locations
		use AMSU_types
		use date_And_time
		use NAN_support
		use byte_swapping

		implicit none

		integer(4),parameter	:: LENGTH_JUNK_BEFORE =	22
		integer(4),parameter	:: LENGTH_JUNK_AFTER =	400

contains


	subroutine read_SAA_AMSU_archive_header(AMSU_SAA_LU,			&
											file_name,				&	
											error)
!@***s* MLT_RSS/AMSU_SAA.f90/read_SAA_AMSU_archive_header
!  NAME
!    read_SAA_AMSU_archive_header
!
!  PURPOSE
!    reads SAA Archive header on AMSU files
!  DESCRIPTION
!    reads SAA Archive header on AMSU files
!
!  INPUT
!     AMSU_SAA_LU   logical unit number for AMSU SAA file
!
!  OUTPUT
!    character(len = 42)  :: file_name  ! file name from header
!    integer(4)			  :: error
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call read_SAA_AMSU_archive_header(AMSU_SAA_LU,file_name,error)
!
!@*****

		integer(4)							:: AMSU_SAA_LU
		character(len = 42)					:: file_name
		integer(4)							:: error

		character(len = LENGTH_JUNK_BEFORE) :: junk_1
		character(len = LENGTH_JUNK_AFTER)  :: junk_2

		error = 0

		read(AMSU_SAA_LU,err = 111)junk_1
		read(AMSU_SAA_LU,err = 111)file_name
		read(AMSU_SAA_LU,err = 111)junk_2

		return
	
	111 error = -1
		return

	end subroutine read_SAA_AMSU_archive_header

	subroutine read_raw_AMSU_header(AMSU_SAA_LU,				&
									raw_Header,			&
									error)
!@***s* MLT_RSS/AMSU_SAA.f90/read_raw_AMSU_header
!  NAME
!    read_raw_AMSU_header
!
!  PURPOSE
!    reads header on raw AMSU files
!  DESCRIPTION
!    reads header on raw AMSU files
!
!  INPUT
!     AMSU_SAA_LU   logical unit number for AMSU SAA file
!
!  OUTPUT
!    character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_header
!    integer(4)			  :: error
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call read_raw_AMSU_header(AMSU_SAA_LU,raw_Header,error)
!
!@*****
	 	integer(4)									:: AMSU_SAA_LU
		character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_header
		integer(4)									:: error

		read(AMSU_SAA_LU,err = 111)raw_header
		error = 0

		return

	111 error = -1
		raw_Header = ' '


	end subroutine read_raw_AMSU_Header

	subroutine get_AMSU_calibration_data_from_header(raw_Header,  &
													 AMSU_header_data)
!@***s* MLT_RSS/AMSU_SAA.f90/get_AMSU_calibration_data_from_header
!  NAME
!    get_AMSU_calibration_data_from_header
!
!  PURPOSE
!    parses calibration data out of header
!  DESCRIPTION
!    parses calibration data out of header
!
!  INPUT
!    character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_header
!  OUTPUT
!    type(AMSU_A_Data_to_Keep_from_Header)		:: AMSU_header_data
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call get_AMSU_calibration_data_from_header(raw_Header,AMSU_header_data)
!@*****

		character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_header
		type(AMSU_A_Data_to_Keep_from_Header)		:: AMSU_header_data

		AMSU_header_data%Calibration		= raw_header(211:672)
		AMSU_header_data%Temp_to_Rad		= raw_header(689:868)
		AMSU_header_data%A1_Digital_Conv	= raw_header(977:1696)
		AMSU_header_data%A2_Digital_Conv    = raw_header(1921:2224)

	end subroutine get_AMSU_calibration_data_from_header


	subroutine read_SAA_AMSU_header(AMSU_SAA_LU,					&
									AMSU_L1A_Header,				&
									error)

!@***s* MLT_RSS/AMSU_SAA.f90/read_SAA_AMSU_header
!  NAME
!    read_SAA_AMSU_header
!
!  PURPOSE
!    reads and decodes AMSU L1A header
!  DESCRIPTION
!    reads and decodes AMSU L1A header
!
!  INPUT
!     AMSU_SAA_LU   !logical unit number for AMSU SAA file
!  OUTPUT
!     type(AMSU_A_L1A_File_Header)		:: AMSU_L1A_Header  ! decoded header info
!     integer(4)					    :: error  ! error flag
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call read_SAA_AMSU_header(AMSU_SAA_LU,AMSU_L1A_Header,error)
!@*****

		integer(4)							:: AMSU_SAA_LU
		type(AMSU_A_L1A_File_Header)		:: AMSU_L1A_Header
		integer(4)							:: error

		integer(2)							:: j,k
		character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_header
		type(AMSU_A_L1A_File_Header)		:: AMSU_L1A_Header_local

		equivalence(raw_header,AMSU_L1A_Header_Local)


		error = 0

		read(AMSU_SAA_LU,err = 111)raw_header
		AMSU_L1A_Header = AMSU_L1A_Header_Local

		print *,AMSU_L1A_Header%data_set_creation_Site_ID

		AMSU_L1A_Header%version_number=byte_swap(AMSU_L1A_Header%version_number)
		AMSU_L1A_Header%version_year=byte_swap(AMSU_L1A_Header%version_year)
		AMSU_L1A_Header%version_day_of_year=byte_swap(AMSU_L1A_Header%version_day_of_year)
		AMSU_L1A_Header%logical_record_length=byte_swap(AMSU_L1A_Header%logical_record_length)
		AMSU_L1A_Header%block_size=byte_swap(AMSU_L1A_Header%block_size)
		AMSU_L1A_Header%count_of_header_records=byte_swap(AMSU_L1A_Header%count_of_header_records)
		AMSU_L1A_Header%NOAA_spacecraft_identification_code=byte_swap(AMSU_L1A_Header%NOAA_spacecraft_identification_code)
		AMSU_L1A_Header%instrument_ID=byte_swap(AMSU_L1A_Header%instrument_ID)
		AMSU_L1A_Header%data_type_code=byte_swap(AMSU_L1A_Header%data_type_code)
		AMSU_L1A_Header%TIP_source_code=byte_swap(AMSU_L1A_Header%TIP_source_code)
		AMSU_L1A_Header%start_of_data_set_day_count=byte_swap(AMSU_L1A_Header%start_of_data_set_day_count)
		AMSU_L1A_Header%start_of_data_set_year=byte_swap(AMSU_L1A_Header%start_of_data_set_year)
		AMSU_L1A_Header%start_of_data_set_day_of_year=byte_swap(AMSU_L1A_Header%start_of_data_set_day_of_year)
		AMSU_L1A_Header%start_of_data_set_UTC_time_of_day=byte_swap(AMSU_L1A_Header%start_of_data_set_UTC_time_of_day)
		AMSU_L1A_Header%end_of_data_set_day_count=byte_swap(AMSU_L1A_Header%end_of_data_set_day_count)
		AMSU_L1A_Header%end_of_data_set_year=byte_swap(AMSU_L1A_Header%end_of_data_set_year)
		AMSU_L1A_Header%end_of_data_set_day_of_year=byte_swap(AMSU_L1A_Header%end_of_data_set_day_of_year)
		AMSU_L1A_Header%end_of_data_set_UTC_time_of_day=byte_swap(AMSU_L1A_Header%end_of_data_set_UTC_time_of_day)
		AMSU_L1A_Header%year_of_last_CPIDS_update=byte_swap(AMSU_L1A_Header%year_of_last_CPIDS_update)

		!byte swap important data set quality indicators

  		AMSU_L1A_Header%instrument_status_A2=byte_swap(AMSU_L1A_Header%instrument_status_A2)
  		AMSU_L1A_Header%instrument_status_A1=byte_swap(AMSU_L1A_Header%instrument_status_A1)
  		AMSU_L1A_Header%count_of_data_records=byte_swap(AMSU_L1A_Header%count_of_data_records)
  		AMSU_L1A_Header%count_of_scan_lines=byte_swap(AMSU_L1A_Header%count_of_scan_lines)
  		AMSU_L1A_Header%count_of_data_gaps=byte_swap(AMSU_L1A_Header%count_of_data_gaps)
  		AMSU_L1A_Header%count_of_AMSU_frames=byte_swap(AMSU_L1A_Header%count_of_AMSU_frames)
  		AMSU_L1A_Header%count_of_PACS_detected_TIP_errors=byte_swap(AMSU_L1A_Header%count_of_PACS_detected_TIP_errors)
  		AMSU_L1A_Header%earth_location_error_indicator=byte_swap(AMSU_L1A_Header%earth_location_error_indicator)
  	
	
		!byte swap  calbration data

		do j = 1,3

			AMSU_L1A_Header%RF_shelf_A1_1_reference_temperature_1(j)=byte_swap(AMSU_L1A_Header%RF_shelf_A1_1_reference_temperature_1(j))
			AMSU_L1A_Header%RF_shelf_A1_1_reference_temperature_2(j)=byte_swap(AMSU_L1A_Header%RF_shelf_A1_1_reference_temperature_2(j))
  			AMSU_L1A_Header%RF_shelf_A1_2_reference_temperature(j)=byte_swap(AMSU_L1A_Header%RF_shelf_A1_2_reference_temperature(j))
  			AMSU_L1A_Header%RF_shelf_A2_reference_temperature(j)=byte_swap(AMSU_L1A_Header%RF_shelf_A2_reference_temperature(j))
			AMSU_L1A_Header%RF_Mux_A1_1_reference_temperature_1(j)=byte_swap(AMSU_L1A_Header%RF_Mux_A1_1_reference_temperature_1(j))
			AMSU_L1A_Header%RF_Mux_A1_1_reference_temperature_2(j)=byte_swap(AMSU_L1A_Header%RF_Mux_A1_1_reference_temperature_2(j))
  			AMSU_L1A_Header%RF_Mux_A1_2_reference_temperature(j)=byte_swap(AMSU_L1A_Header%RF_Mux_A1_2_reference_temperature(j))
  			AMSU_L1A_Header%RF_Mux_A2_reference_temperature(j)=byte_swap(AMSU_L1A_Header%RF_Mux_A2_reference_temperature(j))
  		enddo

		do j = 1,15
			do k = 1,4
				AMSU_L1A_Header%fixed_bias_Corr(j,k) =byte_swap(AMSU_L1A_Header%fixed_bias_Corr(j,k))
			enddo
		enddo

		do j = 1,6
			do k = 1,3
				AMSU_L1A_Header%warm_target_bias_Corr(j,k) =byte_swap(AMSU_L1A_Header%warm_target_bias_Corr(j,k))
			enddo
		enddo

		do j = 1,15
			do k = 1,3			
				AMSU_L1A_Header%nonlinearity_coef(j,k) =byte_swap(AMSU_L1A_Header%nonlinearity_coef(j,k))
			enddo
		enddo

		do j = 1,6
			do k = 1,3
				AMSU_L1A_Header%nonlinearity_coef_2(j,k) =byte_swap(AMSU_L1A_Header%nonlinearity_coef_2(j,k))
			enddo
		enddo

		do j = 1,15
			do k = 1,3
				AMSU_L1A_Header%temperature_radiance(j,k) =byte_swap(AMSU_L1A_Header%temperature_radiance(j,k))
			enddo
		enddo


		! byte swap navigation stuff

		AMSU_L1A_Header%nadir_earth_location_tolerance=byte_swap(AMSU_L1A_Header%nadir_earth_location_tolerance) 					! SF 1 in km
		AMSU_L1A_Header%constant_roll_attitude_error=byte_swap(AMSU_L1A_Header%constant_roll_attitude_error)				! SF 3 in deg
		AMSU_L1A_Header%constant_pitch_attitude_error=byte_swap(AMSU_L1A_Header%constant_pitch_attitude_error)					! SF 3 in deg
		AMSU_L1A_Header%constant_yaw_attitude_error	=byte_swap(AMSU_L1A_Header%constant_yaw_attitude_error)						! SF 3 in deg
		AMSU_L1A_Header%epoch_year_for_orbit_vector =byte_swap(AMSU_L1A_Header%epoch_year_for_orbit_vector)

		AMSU_L1A_Header%day_of_epoch_year_for_orbit_vector =byte_swap(AMSU_L1A_Header%day_of_epoch_year_for_orbit_vector)
		AMSU_L1A_Header%epoch_UTC_time_of_day_for_orbit_vector =byte_swap(AMSU_L1A_Header%epoch_UTC_time_of_day_for_orbit_vector)			! in msec
		AMSU_L1A_Header%semi_major_axis	 =byte_swap(AMSU_L1A_Header%semi_major_axis)									! SF 5 in km
		AMSU_L1A_Header%eccentricity =byte_swap(AMSU_L1A_Header%eccentricity)									! SF 8
		AMSU_L1A_Header%inclination =byte_swap(AMSU_L1A_Header%inclination)										! SF 5 in deg
		AMSU_L1A_Header%argument_of_perigee	 =byte_swap(AMSU_L1A_Header%argument_of_perigee)								! SF 5 in deg
		AMSU_L1A_Header%right_ascension_of_the_ascending_node =byte_swap(AMSU_L1A_Header%right_ascension_of_the_ascending_node)			! SF 5 in deg
		AMSU_L1A_Header%mean_anomaly =byte_swap(AMSU_L1A_Header%mean_anomaly)									! SF 5 in deg
		AMSU_L1A_Header%earth_sun_distance_ratio =byte_swap(AMSU_L1A_Header%earth_sun_distance_ratio)						! SF 6

		do j = 1,3
			AMSU_L1A_Header%position(j) =byte_swap(AMSU_L1A_Header%position(j))										! SF 5 in km
			AMSU_L1A_Header%velocity(j) =byte_swap(AMSU_L1A_Header%velocity(j))
		enddo										! SF 8 in km/sec


		
			






		return

	111	error = -1
		return

	end subroutine read_SAA_AMSU_header

	subroutine read_SAA_AMSU_swath(AMSU_SAA_LU,			&
									Raw_Swath,			&
									read_error)


		integer(4)										:: AMSU_SAA_LU
		character(len = AMSU_LEVEL_1_RECORD_LENGTH)		:: raw_swath
		integer(4)										:: read_error
		
		read_error = 0

		read(AMSU_SAA_LU,err = 111,end = 111) Raw_Swath

		return

111		read_error = -1
		raw_swath = ' '
		return
	end subroutine read_SAA_AMSU_swath

		
					
	subroutine decode_date_and_earth_loc_from_SAA_AMSU_swath(raw_scan,			&
	                                                         scan_rd_2000,		&
															 nadir_latitude,	&
															 nadir_longitude,	&
															 quality_indicator,  &
															 error)
!@***s* MLT_RSS/AMSU_SAA.f90/decode_date_and_earth_loc_from_SAA_AMSU_swath
!  NAME
!    decode_date_and_earth_loc_from_SAA_AMSU_swath
!
!  PURPOSE
!    decodes earth location data from AMSU L1A swath
!  DESCRIPTION
!    decodes earth location data from AMSU L1A swath
!
!  INPUT
!    character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_scan
!
!  OUTPUT
!    real(8)								:: scan_rd_2000    !time of scan, in RD2000 format
!    real(4)								:: nadir_latitude  !nadir lat of scan
!    real(4)								:: nadir_longitude !nadir lon of scan
!    integer(4)							    :: quality_indicator ! scan quality flag
!    integer(4)					            :: error  ! error flag

!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call rdecode_date_and_earth_loc_from_SAA_AMSU_swath(raw_scan,scan_rd_2000,	nadir_latitude,nadir_longitude,quality_indicator, error)
!@*****
		character(len = AMSU_LEVEL_1_RECORD_LENGTH) :: raw_scan
		real(8)								:: scan_rd_2000
		real(4)								:: nadir_latitude
		real(4)								:: nadir_longitude
		integer(4)							:: quality_indicator
		integer(4)							:: error

		character(len = 2)			:: raw_scan_line
		character(len = 2)			:: raw_year
		character(len = 2)			:: raw_day_of_year
		character(len = 4)			:: raw_scan_line_UTC
		character(len = 240)		:: raw_earth_loc
		character(len = 4)			:: raw_quality_indicator

		integer(2)							:: scan_line_number
		integer(2)							:: year
		integer(2)							:: day_of_year
		integer(4)							:: scan_line_UTC
		integer(4)							:: quality_indicator_temp

		integer(4)							:: year_i4
		integer(4)							:: day_i4
		real(8)								:: scan_line_UTC_r8

		integer(4),dimension(2,30)			:: earth_location_i4
		real(4)								:: lat_fov_15
		real(4)								:: lon_fov_15
		real(4)								:: lat_fov_16
		real(4)								:: lon_fov_16


		type(date_Str)						:: scan_date

		equivalence (raw_scan_line_UTC, scan_line_utc)
		equivalence (raw_year, year)
		equivalence (raw_day_of_year, day_of_year)
        equivalence (raw_earth_loc, earth_location_i4)
		equivalence (raw_quality_indicator,quality_indicator_temp)
		equivalence (raw_Scan_line,scan_line_number)

		error = 0

		raw_scan_line = raw_scan(1:2)
		raw_year = raw_scan(3:4)
		raw_day_of_year = raw_scan(5:6)
		raw_scan_line_UTC = raw_scan(9:12)
		raw_quality_indicator = raw_scan(25:28)
		raw_earth_loc = raw_scan(653:892)

		scan_line_number = byte_swap(scan_line_number)
		year_i4 = byte_swap(year)
		day_i4 =  byte_swap(day_of_year)
		scan_line_UTC_r8 = 0.001D0 * byte_swap(scan_line_utc)
		quality_indicator = byte_swap(quality_indicator_temp)

		scan_date = set_date(scan_line_UTC_r8,day_i4,year_i4,error)
		scan_rd_2000 = rd_2000(scan_Date)

		lat_fov_15 = 1.0e-4*byte_swap(earth_location_i4(1,15))
		lon_fov_15 = 1.0e-4*byte_swap(earth_location_i4(2,15))
		lat_fov_16 = 1.0e-4*byte_swap(earth_location_i4(1,16))
		lon_fov_16 = 1.0e-4*byte_swap(earth_location_i4(2,16))


		if (abs(lon_fov_15 - lon_fov_16) .gt. 90.0) then
			if (lon_fov_15 .gt. 0.0) then
				if (lon_fov_16 .lt. 0.0) then
					lon_fov_16 = lon_fov_16 + 360.0
				else
					print *,lat_fov_15,lat_fov_16,lon_fov_15,lon_fov_16
					error = 1
				endif
			endif

			if (lon_fov_15 .lt. 0.0) then
				if (lon_fov_16 .gt. 0.0) then
					lon_fov_16 = lon_fov_16 - 360.0
				else
					print *,lat_fov_15,lat_fov_16,lon_fov_15,lon_fov_16
					error = 1
				endif
			endif
		endif

		nadir_latitude =         (lat_fov_15 + lat_fov_16)/2.0
		nadir_longitude = modulo((lon_fov_15 + lon_fov_16)/2.0,360.0)




	end subroutine decode_date_and_earth_loc_from_SAA_AMSU_swath


	subroutine decode_SAA_AMSU_swath(raw_swath,						&
								     AMSU_L1A_Swath,					&
								     error)
!@***s* MLT_RSS/AMSU_SAA.f90/decode_SAA_AMSU_swath
!  NAME
!    decode_SAA_AMSU_swath
!
!  PURPOSE
!    fully decodes AMSU L1A swath
!  DESCRIPTION
!     fully decodes AMSU L1A swath
!  INPUT
!    character*1,dimension(2560)			:: raw_swath
!  OUTPUT
!    type(AMSU_A_L1B_Swath_Data)			:: AMSU_L1A_Swath
!    integer(4)					            :: error  ! error flag

!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call decode_SAA_AMSU_swath(raw_swath,AMSU_L1A_Swath,error)					
!
!@*****
		character*1,dimension(2560)			:: raw_swath
		character*1,dimension(2560)			:: raw_swath_local			
		type(AMSU_A_L1B_Swath_Data)			:: AMSU_L1A_Swath
		integer(4)							:: error

		integer(4)							:: k
		integer(4)							:: j
		type(AMSU_A_L1B_Swath_Data)			:: Input_Swath

		equivalence(raw_swath_local,Input_swath)


		error = 0


		raw_swath_local = raw_swath
		AMSU_L1A_Swath = Input_Swath

		AMSU_L1A_Swath%scan_line_number = byte_swap(AMSU_L1A_Swath%scan_line_number)
		AMSU_L1A_Swath%scan_line_year = byte_swap(AMSU_L1A_Swath%scan_line_year)
		AMSU_L1A_Swath%scan_line_day_of_year = byte_swap(AMSU_L1A_Swath%scan_line_day_of_year)
		AMSU_L1A_Swath%satellite_clock_drift_delta = byte_swap(AMSU_L1A_Swath%satellite_clock_drift_delta)
		AMSU_L1A_Swath%scan_line_UTC_time_of_day = byte_swap(AMSU_L1A_Swath%scan_line_UTC_time_of_day)
		AMSU_L1A_Swath%scan_line_bit_field = byte_swap(AMSU_L1A_Swath%scan_line_bit_field)
		AMSU_L1A_Swath%major_frame_count = byte_swap(AMSU_L1A_Swath%major_frame_count)
		AMSU_L1A_Swath%quality_indicator_bit_field = byte_swap(AMSU_L1A_Swath%quality_indicator_bit_field)
		AMSU_L1A_Swath%scan_line_quality_flags = byte_swap(AMSU_L1A_Swath%scan_line_quality_flags)
		do j = 1,16
			AMSU_L1A_Swath%calibration_quality_flags(j) = byte_swap(AMSU_L1A_Swath%calibration_quality_flags(j))
		enddo

		do j = 1,15
			do k = 1,3
				AMSU_L1A_Swath%primary_calibration(k,j) = byte_swap(AMSU_L1A_Swath%primary_calibration(k,j))
				AMSU_L1A_Swath%secondary_calibration(k,j) = byte_swap(AMSU_L1A_Swath%secondary_calibration(k,j))
			enddo
		enddo
		
		AMSU_L1A_Swath%navigation_status_bit_field = byte_swap(AMSU_L1A_Swath%navigation_status_bit_field)
		AMSU_L1A_Swath%time_associated_with_TIP_euler_angles = byte_swap(AMSU_L1A_Swath%time_associated_with_TIP_euler_angles)
		
		do j = 1,3
			AMSU_L1A_Swath%TIP_euler_angles(j) = byte_swap(AMSU_L1A_Swath%TIP_euler_angles(j))
		enddo
		
		AMSU_L1A_Swath%spacecraft_altitude = byte_swap(AMSU_L1A_swath%spacecraft_altitude)
		
		do j = 1,30
			do k = 1,3
				AMSU_L1A_Swath%angular_relationships(k,j) = byte_swap(AMSU_L1A_Swath%angular_relationships(k,j))
			enddo
		enddo	
		
		do j = 1,30
			do k = 1,2
				AMSU_L1A_Swath%earth_location(k,j) = byte_swap(AMSU_L1A_Swath%earth_location(k,j))
			enddo
		enddo

		do j = 1,30
			do k = 1,17
				AMSU_L1A_Swath%A1_scene_telemetry(k,j) = byte_swap(AMSU_L1A_Swath%A1_scene_telemetry(k,j))
			enddo
		enddo

		do j = 1,30
			AMSU_L1A_Swath%A1_cold_calibration_telemetry(j) = byte_swap(AMSU_L1A_Swath%A1_cold_calibration_telemetry(j))	
		enddo

		do j = 1,46
			AMSU_L1A_Swath%A1_temperature_sensor_telemetry(j) = byte_swap(AMSU_L1A_Swath%A1_temperature_sensor_telemetry(j))
		enddo
		
		do j = 1,30
			AMSU_L1A_Swath%A1_warm_calibration_telemetry(j) = byte_swap(AMSU_L1A_Swath%A1_warm_calibration_telemetry(j))
		enddo
		
		AMSU_L1A_Swath%A1_invalid_word_bit_flags = byte_swap(AMSU_L1A_Swath%A1_invalid_word_bit_flags)		
		AMSU_L1A_Swath%A1_digital_B_telemetry = byte_swap(AMSU_L1A_Swath%A1_digital_B_telemetry)
		
		AMSU_L1A_Swath%A1_invalid_word_bit_flags = byte_swap(AMSU_L1A_Swath%A1_invalid_word_bit_flags)

		do j = 1,30
			do k = 1,4
				AMSU_L1A_Swath%A2_scene_telemetry(k,j) = byte_swap(AMSU_L1A_Swath%A2_scene_telemetry(k,j))
			enddo
		enddo	

		do j = 1,6
			AMSU_L1A_Swath%A2_cold_calibration_telemetry(j) = byte_swap(AMSU_L1A_Swath%A2_cold_calibration_telemetry(j))
		enddo																										 
		
		do j = 1,20
			AMSU_L1A_Swath%A2_temperature_sensor_telemetry(j) = 	byte_swap(AMSU_L1A_Swath%A2_temperature_sensor_telemetry(j))	
		enddo

		do j = 1,6
			AMSU_L1A_Swath%A2_warm_calibration_telemetry(j) = byte_swap(AMSU_L1A_Swath%A2_warm_calibration_telemetry(j))
		enddo																										 

		return

	111 error = -1
		return

	end subroutine decode_SAA_AMSU_swath

	subroutine find_rd_2000_from_AMSU_L1A_Swath(AMSU_L1A_Swath,	&
												amsu_rd_2000,		&
												error)
!@***s* MLT_RSS/AMSU_SAA.f90/find_rd_2000_from_AMSU_L1A_Swath
!  NAME
!    find_rd_2000_from_AMSU_L1A_Swath
!
!  PURPOSE
!    calculates scan time in RD2000 format from decoded AMSU L1A swath
!  DESCRIPTION
!    calculates scan time in RD2000 format from decoded AMSU L1A swath
!  INPUT
!    type(AMSU_A_L1B_Swath_Data)			:: AMSU_L1A_Swath
!  OUTPUT
!    real(8)								:: amsu_rd_2000
!    integer(4)							    :: error

!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call find_rd_2000_from_AMSU_L1A_Swath(AMSU_L1A_Swath,amsu_rd_2000,error)
!
!@*****

		type(AMSU_A_L1B_Swath_Data)			:: AMSU_L1A_Swath
		real(8)								:: amsu_rd_2000
		integer(4)							:: error


		amsu_rd_2000 = rd_2000(set_Date(AMSU_L1A_Swath%scan_line_UTC_time_of_day/1000.0D0,	&
								 1*AMSU_L1A_Swath%scan_line_day_of_year,				& !1's convert to int*4						
								 1*AMSU_L1A_Swath%scan_line_year,						&
								 error))
		return




	end subroutine find_rd_2000_from_AMSU_L1A_Swath




end module AMSU_SAA