$freeform
!@***m* MLT_RSS/AMSU_inventory.f90
!  NAME
!    AMSU_inventory
! 
!  PURPOSE
!    AMSU_inventory psuedoclass
! 
!  DESCRIPTION
!    pseudoclass for reading and writing AMSU orbit inventories
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use msu_L2A_inventory
!@***** 


module AMSU_inventory

	use nan_support
	use date_and_time
	use amsu_types
	use amsu_constants
	use AMSU_SAA
	use AMSU_file_locations
	use AMSU_equator_crossing_times

	implicit none

	integer(4),parameter   :: MX_ORBITS = 100000

	type amsu_inventory_str
	    integer(4)						:: max_orbit
		logical(4),dimension(MX_ORBITS) :: orbit_valid
		integer(4),dimension(MX_ORBITS)	:: start_swath
		real(8),dimension(MX_ORBITS)	:: start_rd_2000
		integer(4),dimension(MX_ORBITS)	:: end_swath
		real(8),dimension(MX_ORBITS)	:: end_rd_2000
		real(8),dimension(MX_ORBITS)    :: eqx_rd_2000
	end type


contains

	subroutine read_amsu_l1b_inventory_file(sat_num,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/read_amsu_l1b_inventory_file
!  NAME
!      read_amsu_l1b_inventory_file(sat_num,inv,error)
!  PURPOSE
!    reads AMSU L1B Inventory file
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!  OUTPUTS
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    read_amsu_l1b_inventory_file(sat_num,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN)	:: inv_file
		integer(4)					:: io_error
		integer(4)					:: j
		integer(4)					:: orbit_num

		error = 0

		! zero the inventory

		inv%max_orbit		= 0
		inv%orbit_valid		= .false.
		inv%start_swath		= 0
		inv%end_swath		= 0
		inv%start_rd_2000	= NAN
		inv%end_rd_2000		= NAN
		inv%eqx_rd_2000 = NAN

		if ((sat_num .lt. 10) .or. (sat_num .gt. 14)) then
			error = -1
			return
		endif

	

		! make the file name

		write(inv_file,111) trim(AMSU_L1B_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num))
111		format(a,a,'\',a,'_L1B.inv')

		! open the file

		open(unit = 3, file = inv_file,form='binary',iostat = io_error)
		
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open inventory file'
			print *,inv_file
			error = -1
			stop
		endif
       
		do j = 1,MX_ORBITS
			read(3,end=999) orbit_num
			if ((orbit_num .gt. 0) .and. (orbit_num .le. MX_ORBITS)) then
				read(3,end=999) inv%start_swath(orbit_num),	&
					inv%start_rd_2000(orbit_num), inv%end_swath(orbit_num),	&
					inv%end_rd_2000(orbit_num),inv%eqx_rd_2000(orbit_num)
				inv%orbit_valid(orbit_num) = .true.
			endif
		enddo
		
999		inv%max_orbit = orbit_num-1

		return
		
	end subroutine read_amsu_l1b_inventory_file 

	subroutine read_amsu_l1b_inventory_file_text(sat_num,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/read_amsu_l1b_inventory_file_text
!  NAME
!      read_amsu_l1b_inventory_file_text
!  PURPOSE
!    reads AMSU L1B Inventory file from text format
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!  OUTPUTS
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call read_amsu_l1b_inventory_file_text(sat_num,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN)	:: inv_file
		integer(4)					:: io_error
		integer(4)					:: j
		integer(4)					:: orbit_num

		integer(4)					:: first_Scan
		integer(4)					:: last_Scan
		real(8)						:: first_rd_2000
		real(8)						:: last_rd_2000
		real(8)						:: eqx_rd_2000


		error = 0

		! zero the inventory

		inv%max_orbit		= 0
		inv%orbit_valid		= .false.
		inv%start_swath		= 0
		inv%end_swath		= 0
		inv%start_rd_2000	= NAN
		inv%end_rd_2000		= NAN
		inv%eqx_rd_2000 = NAN

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16) .or. (sat_num .eq. 15)) then
			call log('Error: sat_num out of range',log_err)
			error = -1
			return
		endif

	

		! make the file name

		write(inv_file,111) trim(AMSU_L1B_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num))
111		format(a,a,'\',a,'_L1B.inv.txt')

		! open the file

		open(unit = 3, file = inv_file,iostat = io_error)
		
		if(io_error .ne. 0) then
			call log('Fatal Error:  Could not open L1B inventory file: '//inv_file,log_err)
			error = -1
			stop
		endif

		do j = 1,MX_ORBITS
			read(3,888,end=999) orbit_num,first_Scan,first_rd_2000,last_scan,	&
					last_rd_2000,eqx_rd_2000
			
			if ((orbit_num .gt. 0) .and. (orbit_num .lt. MX_ORBITS)) then
				inv%start_swath(orbit_num)		= first_Scan
				inv%end_swath(orbit_num)		= last_Scan
				inv%start_rd_2000(orbit_num)	= first_rd_2000
				inv%end_rd_2000(orbit_num)		= last_rd_2000
				inv%eqx_rd_2000(orbit_num)		= eqx_rd_2000
				inv%orbit_valid(orbit_num)		= .true.
888				format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)
			endif
		enddo
		
999		inv%max_orbit = orbit_num
		call log('Finished Reading L1B inventory file: '//trim(inv_file),error)

		close(3)

		return
		
	end subroutine read_amsu_l1b_inventory_file_text



	subroutine write_amsu_l1b_inventory_file(sat_num,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/write_amsu_l1b_inventory_file
!  NAME
!      write_amsu_l1b_inventory_file
!  PURPOSE
!    writes AMSU L1B Inventory file in binary format
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  USAGE
!    write_amsu_l1b_inventory_file(sat_num,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN)	:: inv_file
		integer(4)					:: io_error
		integer(4)					:: j

		error = 0

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16) .or. (sat_num .eq. 15)) then
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L1B_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num))
111		format(a,a,'\',a,'_L1B.inv')

		! open the file

		open(unit = 3, file = inv_file,form='binary',iostat = io_error)
		
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open inventory file'
			print *,inv_file
			error = -1
			stop
		endif

		do j = 1,inv%max_orbit
			if (inv%orbit_valid(j)) then
				write(3) j,inv%start_swath(j),	&
					inv%start_rd_2000(j),inv%end_swath(j),	&
					inv%end_rd_2000(j),inv%eqx_rd_2000(j)
			endif
		enddo
		
		close(3)

		return
		
	end subroutine write_amsu_l1b_inventory_file

	subroutine write_amsu_l1b_inventory_file_text(sat_num,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/write_amsu_l1b_inventory_file_text
!  NAME
!      write_amsu_l1b_inventory_file_text
!  PURPOSE
!    writes AMSU L1B Inventory file in text format
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  USAGE
!    write_amsu_l1b_inventory_file_text(sat_num,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN) :: inv_file
		integer(4)				  :: io_error
		integer(4)				  :: j

		error = 0

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16) .or. (sat_num .eq. 15)) then
			call log('Error: sat_num out of range in write L1B inventory',log_err)
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L1B_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num))
111		format(a,a,'\',a,'_L1B.inv.txt')

		! open the file

		open(unit = 3, file = inv_file,iostat = io_error)
		
		if(io_error .ne. 0) then
			call log('Fatal Error -- could not open L1B inventory file for writing: '//inv_file,error)
			error = -1
			stop
		endif

		do j = 1,inv%max_orbit
			if (inv%orbit_valid(j)) then
			write(3,888) j,inv%start_swath(j),	&
					inv%start_rd_2000(j),inv%end_swath(j),	&
					inv%end_rd_2000(j),inv%eqx_rd_2000(j)
888					format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)
			endif
		enddo
		
		close(3)
		call log('Finished Writing L1B inventory file: '//inv_file,error)

		return
		
	end subroutine write_amsu_l1b_inventory_file_text

	subroutine update_amsu_inventory_data(	amsu_inventory,				&
											L1A_orbit_data,				&
											sat_num,					&
											orbit_num)
!@***s* MLT_RSS/AMSU_inventory.f90/update_amsu_inventory_data
!  NAME
!      update_amsu_inventory_data
!  PURPOSE
!    updates an amsu L1B inventory data record using data from AMSU L1A orbit structure
!  INPUTS
!    type(amsu_inventory_str)	           :: inv   ! inventory of AMSU orbits in structured variable.
!    type(amsu_L1A_orbit_data)			   :: L1A_orbit_data  !L1A orbit data
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)  ! satellite number (10 == NOAA-15, etc)
!    integer(4) :: orbit_num   ! orbit numbers
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call update_amsu_inventory_data(amsu_inventory,L1A_orbit_data,sat_num,orbit_num)
!
!@*****
		use logging

		type(amsu_inventory_str)			:: amsu_inventory
		type(amsu_L1A_orbit_data)			:: L1A_orbit_data
		integer(4)							:: sat_num
		integer(4)							:: orbit_num
		real(8)								:: eqx_rd_2000

		integer(4)			:: scan
		integer(4)			:: num_scans_valid

		integer(4)			:: first_scan
		integer(4)			:: last_scan

		real(8)				:: first_rd_2000
		real(8)				:: last_rd_2000

		integer(4)			:: error,junk3
		real(4)				:: junk1,junk2

		num_scans_valid = 0

		do scan = 1,AMSU_MAX_SCANS_PER_ORBIT
			if (L1A_orbit_data%scan_valid(scan) .gt. 0) then
				num_scans_valid = num_Scans_valid + 1
				if (num_scans_valid .eq. 1) first_scan = scan
				last_scan = scan
			endif
		enddo


		call decode_date_and_earth_loc_from_SAA_AMSU_swath(L1A_orbit_data%scan_data(first_scan),			&
	                                                         first_rd_2000,		&
															 junk1,junk2,junk3,  &
															 error)

		call decode_date_and_earth_loc_from_SAA_AMSU_swath(L1A_orbit_data%scan_data(last_scan),			&
	                                                         last_rd_2000,		&
															 junk1,junk2,junk3,  &
															 error)	

		eqx_rd_2000 = equator_crossing_rd_2000_from_AMSU_rev_num(sat_num,orbit_num,error)

		if ((orbit_num .gt. 0) .and. (orbit_num .lt. MX_ORBITS)) then
			 amsu_inventory%orbit_valid(orbit_num) = .true.

			if (orbit_num .gt. amsu_inventory%max_orbit) then
				amsu_inventory%max_orbit = orbit_num
			endif

			if (amsu_inventory%orbit_valid(orbit_num)) then 
				amsu_inventory%start_swath(orbit_num) = first_Scan
				amsu_inventory%end_swath(orbit_num) =   last_Scan
				amsu_inventory%start_rd_2000(orbit_num) = first_rd_2000
				amsu_inventory%end_rd_2000(orbit_num)   = last_rd_2000
				amsu_inventory%eqx_rd_2000(orbit_num) = eqx_rd_2000
			else
				write(log_str,*)'Fatal Error: inventory -- attemp to assign scan data to invalid orbit =',orbit_num
				call log(log_str,log_err)
				return
			endif
		else
			write(log_str,*)'Fatal Error: orbit num gt mx_orbit in L1B inventory',orbit_num
			call log(log_str,log_err)
		endif
			
	end subroutine update_amsu_inventory_data

	subroutine update_AMSU_L2A_inventory(	L2A_inventory,			&
											L2A_orbit,				&
											sat_num,				&
											orbit_num,				&
											error)
!@***s* MLT_RSS/AMSU_inventory.f90/update_AMSU_L2A_inventory
!  NAME
!      update_amsu_L2A_inventory
!  PURPOSE
!    updates an amsu L2A inventory data record using data from AMSU L2A orbit structure
!  INPUTS
!    type(amsu_inventory_str)	           :: L2A_inventory   ! inventory of AMSU orbits in structured variable.
!    type(amsu_L1A_orbit_data)			   :: L2A_orbit       ! L2A orbit data
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)  ! satellite number (10 == NOAA-15, etc)
!    integer(4) :: orbit_num   ! orbit numbers
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call update_AMSU_L2A_inventory(amsu_inventory,L2A_orbit,sat_num,orbit_num)
!
!@*****

		type(amsu_inventory_str)			:: L2A_inventory
		type(amsu_L2A_orbit_data)			:: L2A_orbit
		integer(4)							:: sat_num
		integer(4)							:: orbit_num
		integer(4)							:: error

		integer(4)			:: scan
		integer(4)			:: num_scans_valid

		integer(4)			:: first_scan
		integer(4)			:: last_scan

		real(8)				:: first_rd_2000
		real(8)				:: last_rd_2000
		real(8)				:: eqx_rd_2000

		num_scans_valid = 0
		
		do scan = 1,AMSU_MAX_SCANS_PER_ORBIT
			if (L2A_orbit%scan_valid(scan) .gt. 0) then
				num_scans_valid = num_Scans_valid + 1
				if (num_scans_valid .eq. 1) first_scan = scan
				last_scan = scan
			endif
		enddo

		if (num_scans_valid .gt. 0) then

			! find time from L2a_scans

			first_rd_2000 = L2A_orbit%scan_data(first_scan)%scan_rd_2000
			last_rd_2000  = L2A_orbit%scan_data(last_scan)%scan_rd_2000

			! find eqx time

			if (sat_num .ne. 15) eqx_rd_2000 = equator_crossing_rd_2000_from_AMSU_rev_num(sat_num,orbit_num,error)
            if (sat_num .eq. 15) eqx_rd_2000 =L2A_inventory%eqx_rd_2000(orbit_num) 
		
			if ((orbit_num .gt. 0) .and. (orbit_num .lt. MX_ORBITS)) then
				 L2A_inventory%orbit_valid(orbit_num) = .true.

				if (orbit_num .gt. L2A_inventory%max_orbit) then
					L2A_inventory%max_orbit = orbit_num
				endif

				if (L2A_inventory%orbit_valid(orbit_num)) then 
					L2A_inventory%start_swath(orbit_num) = first_Scan
					L2A_inventory%end_swath(orbit_num) =   last_Scan
					L2A_inventory%start_rd_2000(orbit_num) = first_rd_2000
					L2A_inventory%end_rd_2000(orbit_num)   = last_rd_2000
					L2A_inventory%eqx_rd_2000(orbit_num) = eqx_rd_2000
				else
					write(6,*)'inventory error -- attemp to assign scan data to invalid orbit =',orbit_num
					!write(21,*)'inventory error -- attemp to assign scan data to invalid orbit =',orbit_num
					return
				endif
			else
				write(6,*)'orbit num gt mx_orbit in L2A inventory',orbit_num
			endif
		endif
	end subroutine update_AMSU_L2A_inventory




	subroutine read_amsu_L2A_inventory_file(sat_num,channel,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/read_amsu_L2A_inventory_file
!  NAME
!      read_amsu_L2A_inventory_file(sat_num,inv,error)
!  PURPOSE
!    reads AMSU L1B Inventory file in binary form
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel    AMSU channel number
!  OUTPUTS
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call read_amsu_L2A_inventory_file(sat_num,channel,inv,error)
!
!@*****
		use AMSU_constants
		use logging

		integer(4)					:: sat_num
		integer(4)					:: channel
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN)	:: inv_file
		integer(4)					:: io_error
		integer(4)					:: j,k
		integer(4)					:: orbit_num

		error = 0

		if ((sat_num .lt. 10) .or. (sat_num .gt. 14)) then
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L2A_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel
111		format(a,a,'\',a,'_Channel_',i2.2,'_L2A.inv')

		! open the file

		open(unit = 3, file = inv_file,form='binary',iostat = io_error)
		
		if(io_error .ne. 0) then
			inv%max_orbit = 0
			return
		endif

		do j = 1,12345678
				read(3,end = 999) orbit_num
				read(3,end = 999)	inv%start_swath(orbit_num),		&
									inv%start_rd_2000(orbit_num),	&
									inv%end_swath(orbit_num),		&
									inv%end_rd_2000(orbit_num),		&
									inv%eqx_rd_2000(orbit_num)

				inv%orbit_valid(orbit_num) = .true.
				if ((j == 1) .and. (orbit_num .gt. 1)) then
					do k = 1,orbit_num-1
						inv%start_rd_2000(k) = NAN	
						inv%end_rd_2000(k) = NAN	
						inv%eqx_rd_2000(k) = NAN
					enddo
				endif	
					
			
		enddo
		
999		close(3)
		inv%max_orbit = orbit_num

		return
		
	end subroutine read_amsu_L2A_inventory_file
					  	
						
	subroutine write_amsu_L2A_inventory_file(sat_num,channel,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/writes_amsu_L2A_inventory_file
!  NAME
!      write_amsu_L2A_inventory_file(sat_num,inv,error)
!  PURPOSE
!    writes AMSU L1B Inventory file in binary form
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel    AMSU channel number
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call write_amsu_L2A_inventory_file(sat_num,channel,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		integer(4)					:: channel
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = 140)	    :: inv_file
		integer(4)					:: io_error
		integer(4)					:: j

		error = 0

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16)) then
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L2A_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel
111		format(a,a,'\',a,'_Channel_',i2.2,'_L2A.inv')

		! open the file

		open(unit = 3, file = inv_file,form='binary',iostat = io_error)
		
		if(io_error .ne. 0) then
			call log('Fatal Error -- could not open inventory file',error)
			call log(inv_file,error)
			error = -1
			stop
		else
			call log('Opened L2A inventory file: '//inv_file,error)
		endif

		do j = 1,inv%max_orbit
			if (inv%orbit_valid(j)) then
				write(3) j,inv%start_swath(j),	&
					inv%start_rd_2000(j),inv%end_swath(j),	&
					inv%end_rd_2000(j),inv%eqx_rd_2000(j)
			endif
		enddo
		
		close(3)

		call log('Finished Writing L2A inventory file: '//inv_file,error)

		return
		
	end subroutine write_amsu_L2A_inventory_file

	subroutine read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/read_amsu_L2A_inventory_file_text
!  NAME
!      read_amsu_L2A_inventory_file_text
!  PURPOSE
!    reads AMSU L1B Inventory file in text form
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel    AMSU channel number
!  OUTPUTS
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)
!
!@*****
		use AMSU_constants
		use logging

		integer(4)					:: sat_num
		integer(4)					:: channel
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN)	:: inv_file
		integer(4)					:: io_error
		integer(4)					:: j

		integer(4)					:: start_swath
		integer(4)					:: end_swath
		real(8)						:: start_rd_2000
		real(8)						:: end_rd_2000
		real(8)						:: eqx_rd_2000
		integer(4)					:: orbit_num
		integer(4)					:: io_err

		error = 0



		if ((sat_num .lt. 10) .or. (sat_num .gt. 16)) then
			call log('Error: sat_num out of range',log_err)
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L2A_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel
111		format(a,a,'\',a,'_Channel_',i2.2,'_L2A.inv.txt')

		! open the file

		open(unit = 3, file = inv_file,iostat = io_error,readonly)

		! zero the inventory

		inv%max_orbit = 0
		inv%start_swath		= 0
		inv%start_rd_2000	= NAN
		inv%end_swath		= 0	
		inv%end_rd_2000		= NAN
		inv%eqx_rd_2000		= NAN
		inv%orbit_valid		= .false.

		if(io_error .ne. 0) then
			call log('Fatal Error:  Could not open L2A inventory file: '//inv_file,log_err)
			error = -1			
			return
		endif

		do j = 1,12345678
				read(3,888,end=999,iostat = io_err) orbit_num,			&
							start_swath,		&
							start_rd_2000,		&
							end_swath,			&
							end_rd_2000,		&
							eqx_rd_2000
888				format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)

				if (io_err .eq. 0) then

					inv%start_swath(orbit_num)		= start_swath
					inv%start_rd_2000(orbit_num)	= start_rd_2000
					inv%end_swath(orbit_num)		= end_swath
					inv%end_rd_2000(orbit_num)		= end_rd_2000
					inv%eqx_rd_2000(orbit_num)		= eqx_rd_2000
					inv%orbit_valid(orbit_num)		= .true.
				else
					inv%start_swath(orbit_num)		= start_swath
					inv%start_rd_2000(orbit_num)	= start_rd_2000
					inv%end_swath(orbit_num)		= end_swath
					inv%end_rd_2000(orbit_num)		= end_rd_2000
					inv%eqx_rd_2000(orbit_num)		= eqx_rd_2000
					inv%orbit_valid(orbit_num)		= .true.
				endif
					
					
		enddo
		
999		close(3)
		inv%max_orbit = orbit_num
		call log('Finished Reading L2A inventory file: '//trim(inv_file),error)

		return
		
	end subroutine read_amsu_L2A_inventory_file_text


	subroutine write_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)
!@***s* MLT_RSS/AMSU_inventory.f90/write_amsu_L2A_inventory_file_text
!  NAME
!      write_amsu_L2A_inventory_file_text
!  PURPOSE
!    reads AMSU L1B Inventory file in text form
!  INPUTS
!    sat_num    AMSU satellite number (10 = NOAA-15, etc)
!    channel    AMSU channel number
!    type(amsu_inventory_str)	:: inv   ! inventory of AMSU orbits in structured variable.
!  OUTPUTS
!    integer(4) :: error                 ! error flag.  0 if all OK, -1 if out of bounds satellite number passed
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  USAGE
!    call write_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)
!
!@*****
		use logging

		integer(4)					:: sat_num
		integer(4)					:: channel
		type(amsu_inventory_str)	:: inv
		integer(4)					:: error

		character(len = FILE_LEN) :: inv_file
		integer(4)				  :: io_error
		integer(4)				  :: j

		error = 0

		if ((sat_num .lt. 10) .or. (sat_num .gt. 16)) then
			call log('Error: sat_num out of range in write L2A inventory',log_err)
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(AMSU_L2A_PATH),trim(amsu_sat_names_2(sat_num)),trim(amsu_sat_names_2(sat_num)),channel
111		format(a,a,'\',a,'_Channel_',i2.2,'_L2A.inv.txt')

		! open the file

		open(unit = 3, file = inv_file,iostat = io_error)
		
		if(io_error .ne. 0) then
			call log('Fatal Error -- could not open L2A inventory file for writing: '//inv_file,error)
			error = -1
			stop
		else
			call log('Opened L2A inventory file for writing: '//inv_file,error)
		endif


		do j = 1,inv%max_orbit
			if (inv%orbit_valid(j)) then
			write(3,888) j,inv%start_swath(j),	&
					inv%start_rd_2000(j),inv%end_swath(j),	&
					inv%end_rd_2000(j),inv%eqx_rd_2000(j)
888					format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)
			endif
		enddo
		
		close(3)
		call log('Finished Writing L2A inventory file: '//inv_file,error)

		return
		
	end subroutine write_amsu_L2A_inventory_file_text
		
end module amsu_inventory        	 







		