
!@***m* MLT_RSS/AMSU_Nadir.f90
!  NAME
!    AMSU_Nadir.f90
! 
!  PURPOSE
!    Module for performing incidence angle adjustments to AMSU measurements
! 
!  DESCRIPTION
!    Module for performing incidence angle adjustments to AMSU measurements
!    Uses a model derived fov and angle perturbation climatology for the calculations.
!    The climatology is caclulated separately for each AMSU channel
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_Nadir
!@*****

module AMSU_nadir

	use NAN_support
	use Date_and_Time
	use maps
	use landmask
	use AMSU_Constants

	implicit none
	type(map_str),dimension(3,15,2,12)		:: tb_sim
	logical,dimension(12)					:: month_loaded = .false.
	integer(4)								:: channel_loaded
	integer(4)								:: channel_loaded_cross_track_corr
	real(4),dimension(NUM_AMSU_FOVS)		:: residual_cross_track_correction
	logical									:: cross_track_correction_loaded = .false.
	character(len=7)						:: sat_name_loaded

contains

    subroutine calculate_nadir_correction(  L2A_scan,						&
											channel,						&
											error)
											
!@***s* MLT_RSS/AMSU_nadir.f90/calculate_nadir_correction
!     NAME
!      calculate_nadir_correction
!     PURPOSE
!      Calculate nadir and incidence angle Corrections for a single AMSU scan line
!     INPUTS
!      channel     AMSU channel number
!     INPUTS_OUTPUTS
!      L2A_Scan    Structure containing AMSU L2A Scan data
!     OUTPUTS
!      Error       Error Flag.  0 if everything OK, not zero otherwise
!     USAGE
!      Call calculate_nadir_correction(L2A_scan,channel,error)
!@*****
		use AMSU_Types
		use AMSU_File_Locations

		type(AMSU_A_L2A_Swath_Data)			:: L2A_scan
		integer(4)							:: channel
		integer(4)							:: error

		integer(4)							:: date_error
		real(4)								:: roll
		integer(4)							:: view_index
		integer(4)							:: fov

		type(date_Str)						:: scan_date
		real(4)								:: altitude

		real(4)								:: theta_view
		real(4)								:: theta_nom
		real(4)								:: eia_nom
		real(4)								:: theta
		real(4)								:: nadir_lat

		real(4)								:: Tb_sim_fov
		real(4)								:: Tb_sim_nadir
		real(4)								:: Tb_sim_nom_eia

		character(len=150)					:: cross_Track_corr_file
		integer(4)							:: satnum
		logical(4)							:: satnum_found



		roll = 0.0

		if ((sat_name_loaded .ne. L2A_Scan%satellite_name) .or. &
		    (channel_loaded_cross_Track_corr .ne. L2A_Scan%channel_num)) then
			! read in residual cross_track Correction

			satnum_found = .false.

			do satnum = 1,NUM_SATS
				print *,satnum,amsu_sat_names_2(satnum),L2A_Scan%satellite_name
				if (amsu_sat_names_2(satnum) .eq. L2A_Scan%satellite_name) then
					satnum_found = .true.
					exit
				endif
			enddo
			if (satnum_found) then
				if ((channel .eq. 5) .or. (channel .eq. 7).or. (channel .eq. 9)) then   ! only channels 5 and 9 have had this analysis done so far
					write(cross_Track_corr_file,345)trim(AMSU_ADJUSTMENT_PATH),satnum,channel
345					format(a,'cross_track_corrections\Sat_',i2.2,'_Channel_',i2.2,'_Cross_track_corrections.txt')
					open(unit=3,file= cross_Track_corr_file)
					print *,'Reading ',cross_track_corr_file
					do fov = 1,NUM_AMSU_FOVS
						read(3,567)residual_cross_track_correction(fov)
	567					format(f10.5)
					enddo
					close(3)
					write(*,444)residual_cross_track_correction
444                 format(30(f6.2,1x))
					
					channel_loaded_cross_Track_corr = channel
					sat_name_loaded = L2A_Scan%satellite_name
				else
					residual_cross_Track_correction = 0.0
					channel_loaded_cross_Track_corr = channel
					sat_name_loaded = L2A_Scan%satellite_name
				endif
			endif ! if satnum not found dont bother -- this scan is not valid
		endif

		scan_date = set_date_2000(L2A_scan%scan_rd_2000,date_error)
		nadir_lat = 0.5*(L2a_scan%fov_latitude(15)+L2a_scan%fov_latitude(16))
		altitude = L2A_scan%spacecraft_altitude

		do fov = 1,30 
			view_index = nint(0.5 + abs(15.5 - fov))

			theta_nom = AMSU_VIEW_ANGLES(view_index)
			if (fov >= 16) theta_nom = -theta_nom
			theta_view = theta_nom + roll
			call geometry(theta_view,nadir_lat,altitude,theta)
			
			Tb_sim_fov     = amsu_simulated_tb(channel,fov,theta,           L2a_scan%fov_longitude(fov),L2a_scan%fov_latitude(fov),scan_date,error) 
			Tb_sim_nadir   = amsu_simulated_tb(channel,15, AMSU_NOM_EIAs(1),L2a_scan%fov_longitude(fov),L2a_scan%fov_latitude(fov),scan_date,error)
			eia_nom = AMSU_NOM_EIAs(view_index)
			if (fov >= 16) eia_nom = -eia_nom
			Tb_sim_nom_EIA = amsu_simulated_tb(channel,fov,eia_nom,         L2a_scan%fov_longitude(fov),L2a_scan%fov_latitude(fov),scan_date,error)
			
			L2A_scan%nadir_corr(fov)	 = Tb_sim_nadir   - Tb_sim_fov - residual_cross_track_correction(fov)
			L2A_scan%incidence_corr(fov) = Tb_sim_nom_EIA - Tb_sim_fov - residual_cross_track_correction(fov)
			enddo
		
	end subroutine calculate_nadir_correction 

	real(4) function amsu_simulated_tb(chan,fov,theta,lon,lat,date,error)
!@***s* MLT_RSS/AMSU_nadir.f90/amsu_simulated_tb
!     NAME
!      amsu_simulated_tb
!     PURPOSE
!      Calculates simulated AMSU Tb for a given channel, incidence angle, field of view, latitude, longitude and date.
!     INPUTS
!		integer(4)				:: chan    ! AMSU channel
!		integer(4)				:: fov     ! field of view index
!		real(4)					:: theta   ! earth incidence angle (degrees)
!		real(4)					:: lon     ! longitude
!		real(4)					:: lat     ! latitude
!		type(date_Str)			:: date    ! date, in date structure form
!     OUTPUTS
!       Error       Error Flag.  0 if everything OK, not zero otherwise
!     RETURNED VALUE
!       climatological Tb value for the given parameters
!     USAGE
!      Tb = amsu_simulated_tb(chan,fov,theta,lon,lat,date,error)
!@*****

		integer(4)				:: chan
		integer(4)				:: fov
		real(4)					:: theta
		real(4)					:: lon
		real(4)					:: lat
		type(date_Str)			:: date
		integer(4)				:: error


		integer(4),dimension(2)	:: month_indices
		real(4),dimension(12)	:: month_wts
		real(4),dimension(3)	:: tb_coeffs
		real(4),dimension(2)    :: tbs
		integer(4)				:: month_num
		integer(4)				:: surf
		integer(4)				:: view_index
		integer(4)				:: par
		real(4)					:: delta_theta_view


		call month_weights(date,month_wts,month_indices,error)
	    error = 0
		do month_num = 1,2
			if ((channel_loaded .ne. chan) .or. (.not. month_loaded(month_indices(month_num)))) then
				call read_amsu_tb_sim(chan,month_indices(month_num),1996,error)
			endif
		enddo
        if (error .ne. 0) then
			amsu_simulated_tb = NAN
			return
		endif

		surf = find_land_sea(lat,lon)

		view_index = nint(0.5 + abs(15.5 - fov))



		do month_num = 1,2
			do par = 1,3
				call get_map_value(tb_sim(par,view_index,surf,month_indices(month_num)),lon,lat,tb_coeffs(par),error)
			enddo
  			delta_theta_view = abs(theta) - AMSU_NOM_EIAs(view_index)
			tbs(month_num) = tb_coeffs(1) + tb_coeffs(2)*delta_theta_view + 0.5*tb_coeffs(3)*delta_theta_view*delta_theta_view
		enddo
		
		amsu_simulated_tb = month_wts(month_indices(1))*tbs(1) + month_wts(month_indices(2))*tbs(2)

	end function amsu_simulated_tb



	subroutine read_amsu_tb_sim(chan,month,year,error)
!@***s* MLT_RSS/AMSU_nadir.f90/read_amsu_tb_sim
!     NAME
!      read_amsu_tb_sim
!     PURPOSE
!      reads in climatological Tb data for a given channel and month and year
!     INPUTS
!		integer(4)								:: chan
!		integer(4)								:: month
!		integer(4)								:: year
!     OUTPUTS
!       Error       Error Flag.  0 if everything OK, not zero otherwise
!     REMARKS
!       loads climatological Tb value into module variable tb_sim
!     USAGE
!      call read_amsu_tb_sim(chan,month,year,error)
!@*****		
		use AMSU_File_Locations

		integer(4)								:: chan
		integer(4)								:: month
		integer(4)								:: year
		integer(4)								:: error

		real(4),dimension(3,15,144,73,2)		:: tb_dat
		character(len = 120)					:: infile
		
		integer(4)								:: par
		integer(4)								:: fov
		integer(4)								:: surf
		
		integer(4)								:: allocate_error 
		integer(4)								:: err


		if ((chan .eq. 4) .or. &
		    (chan .eq. 6) .or. &
			(chan .eq. 8)) then
			write(infile,9002)trim(AMSU_ADJUSTMENT_PATH),chan,month
		else
			write(infile,9003)trim(AMSU_ADJUSTMENT_PATH),chan,year,month
		endif

		!write(infile,9002)trim(AMSU_ADJUSTMENT_PATH),chan,month

9002    format(a,'NCEP_Modeled_AMSU_Tbs\Channel_',i2.2,'\amsu_tbs_from_ncep_LTM_month_',i2.2,'.dat')
9003    format(a,'NCEP_Modeled_AMSU_Tbs\Channel_',i2.2,'\ncep_amsu_tbs_',i4.4,'_',i2.2,'.dat')
	
		print *,'Angle Adjustments'
		print *,'Loading: ',infile
		open(unit=3,file=infile,form='binary',err = 9999)
		read(3,err = 9999)tb_dat

		close(3)

		allocate_error = 0
		do par = 1,3
			do fov = 1,15
				do surf = 1,2
					call set_up_map(0.0,2.5,144,EDGE_CENTERED,-90.0,2.5,73,EDGE_CENTERED,SPHERICAL,tb_sim(par,fov,surf,month),err)
					tb_sim(par,fov,surf,month)%dat = tb_dat(par,fov,:,:,surf)
					if (err .ne. 0) allocate_error = -1
				enddo !surf
			enddo	! fov
		enddo ! par

		if (allocate_error .eq. 0) then
			month_loaded(month) = .true.
			channel_loaded		=  chan
			error = 0
		endif

		return

9999    error = -1
		return

	end subroutine read_amsu_tb_sim




	subroutine geometry (tht_nadir,xlat,z,tht)
!@***s* MLT_RSS/AMSU_nadir.f90/geometry
!     NAME
!      geometry
!     PURPOSE
!       calculates EIA tht for given nadir looking angle tht_nadir 
!     DESCRIPTION
!       calculates EIA tht for given nadir looking angle tht_nadir
!	    at geodesic latitude xlat and height z above ground
!	 	approximates the surface of the earth by a sphere with
!	    radius of curvature rho (c.f. RSS SSM/I Users Manual, DEC1, 1991)
!     INPUTS
!		real(4)     tht_nadir   ! look angle
!       real(4)     xlat        ! latitude of point on surface
!       real(4)     z           ! height of satellite above surface
!     OUTPUTS
!       real(4)     tht         ! earth incidence angle
!     USAGE
!      call geometry (tht_nadir,xlat,z,tht)
!@*****		

		implicit none
		real tht_nadir,xlat,z,tht,rho
	
		real, parameter :: rho0 = 6345.7, a1 = 55.0 
	
		rho = rho0 + a1*(sind(xlat))**2 
			! earth radius of curvature

		tht = asind(sind(tht_nadir) * (z+rho)/rho )

		return
	end subroutine geometry


		













end module AMSU_nadir