$freeform
module maps
!@***m* MLT_RSS/maps.f90
!  NAME
!    module maps
! 
!  PURPOSE
!    psuedoclass that defines and manipulates a rectangular gridded map type
! 
!  DESCRIPTION
!    psuedoclass that defines and manipulates a rectangular gridded map type
!    
!  VARIABLES
!    the pseudoclass defines a map type
!
!	type map_str
!		real(4)							:: min_x		 ! minimum x value
!		real(4)							:: max_x         ! maximum x value
!		real(4)							:: delta_x       ! spacing points in x direction
!		integer(4)						:: num_x		 ! number of x points
!		integer(4)						:: type_x        ! type of grid in x direction (see explantion below)
!		real(4)							:: min_y		 ! minimum y value
!		real(4)							:: max_y		 ! maximum x value
!		real(4)							:: delta_y		 ! spacing points in x direction
!		integer(4)						:: num_y   		 ! number of x points
!		integer(4)						:: type_y		 ! type of grid in x direction (see explantion below)
!		integer(4)						:: type_bc		 ! type of boundary conditions
!		real(4),dimension(:,:),pointer	:: dat		     ! map data
!	end type map_str
!
! REMARKS 
!  explanation of the grid type flag:
!
!  There are basically two types of evenly spaced rectangular gridded map --
!
!		1. Those for which the value represents the value in a cell - I'll call this cell_centered
!		      for these maps, the lower left value represents the value in the cell defined
!             by long = (0.0:delta_x), lat = (-90,-90+delta_y), and is associated with at point at
!			  (delta_x/2.0, -90+delta_x/2.0)
!		2. Those for which the value represents the value on the vertex between cells - I'll call this edge_centered
!			  for these maps, the lower left value represents the value at the point 
!			  by long = 0.0, lat = (-90).  A peculiar feature of this sort of grid is
!			  that the row at -90.0 must be identical, since it all refers to the same point!
!		For now, all routines in the module assume that maps obey spherical boundary conditions
!		For now, all maps are not redundant at the vertical edges
!		
!		This means that for both types of grids, 360.0/(delta_x) longitude points are needed, and the only different is
!		whether the values are offset by 0.5*delta_x
!
!		For cell_centered grids, 180/(delta_y) latitude points are needed, and for 
!		Edge centered grids 180/(delta_y) + 1 latitude points are needed
!
!		note that mixed grid types are allowed, with the lat and long grids being different types
!
!  ROUTINES
!    set_up_map(min_x,delta_x,num_x,type_x,min_y,delta_y,num_y,type_y,type_bc,mp,error)
!    zero_map(mp,error)
!    deallocate_map(mp,error)
!    get_map_value(mp,xx,yy,value,error)
!    get_map_value_no_interp(mp,xx,yy,value,error)
!    wrap_indices(mp,ix,iy,ix_wrapped,iy_wrapped,error)
!    map_value(mp,ix,iy,error)
!    get_grid_location(mp,ix,iy,x,y,error)
!    get_vertex_location(mp,x,y,ix,iy,error)
!    get_grid_indices(mp,x,y,ix,iy,error)
!    get_vertex_indices(mp,x,y,ix,iy,error)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use maps
! 
!@*****


	use NAN_support

	implicit none


	type map_str
		real(4)							:: min_x		 ! minimum x value
		real(4)							:: max_x         ! maximum x value
		real(4)							:: delta_x       ! spacing points in x direction
		integer(4)						:: num_x		 ! number of x points
		integer(4)						:: type_x        ! type of grid in x direction (see explantion below)
		real(4)							:: min_y		 ! minimum y value
		real(4)							:: max_y		 ! maximum x value
		real(4)							:: delta_y		 ! spacing points in x direction
		integer(4)						:: num_y   		 ! number of x points
		integer(4)						:: type_y		 ! type of grid in x direction (see explantion below)
		integer(4)						:: type_bc		 ! type of boundary conditions
		real(4),dimension(:,:),pointer	:: dat		     ! map data
	end type map_str

	integer(4),parameter				:: CELL_CENTERED = 1
	integer(4),parameter				:: EDGE_CENTERED = 2

	integer(4),parameter				:: SPHERICAL     = 1
	integer(4),parameter				:: RECTANGULAR   = 2
	integer(4),parameter				:: CYLINDRICAL   = 3 ! not implemented yet

	real(4)								:: SMALL_NUMBER  = 1.0e-3    

contains
	
	subroutine set_up_map(min_x,delta_x,num_x,type_x,min_y,delta_y,num_y,type_y,type_bc,mp,error)
!@***m* MLT_RSS/maps.f90/set_up_map
!  NAME
!    set_up_map
! 
!  PURPOSE
!    sets up an instance of the map type
! 
!  DESCRIPTION
!    sets up an instance of the map type
!    
! INPUTS
!   min_x    ! minimum value of x (longitude) coordinate
!   delta_x  ! size of grid cell in longitude direction (degrees)
!   num_x    ! number of grid cells in longitude direction
!   type_x   ! type of grid in x direction(cell centered or edge defined)
!   min_y    ! minimum value of y (latitude) coordinate
!   delta_y  ! size of grid cell in latitude direction (degrees)
!   num_y    ! number of grid cells in latitude direction
!   type_y   ! type of grid in x direction(cell centered or edge defined)
!   type_bc, ! type of boundary conditions (spherical, cylindrical, or no wrapping)
! OUTPUTS
!   mp       ! new instance of map type
!   error    ! error flag
!
! REMARKS 
!  explanation of the grid type flag:
!
!  There are basically two types of evenly spaced rectangular gridded map --
!
!		1. Those for which the value represents the value in a cell - I'll call this cell_centered
!		      for these maps, the lower left value represents the value in the cell defined
!             by long = (0.0:delta_x), lat = (-90,-90+delta_y), and is associated with at point at
!			  (delta_x/2.0, -90+delta_x/2.0)
!		2. Those for which the value represents the value on the vertex between cells - I'll call this edge_centered
!			  for these maps, the lower left value represents the value at the point 
!			  by long = 0.0, lat = (-90).  A peculiar feature of this sort of grid is
!			  that the row at -90.0 must be identical, since it all refers to the same point!
!		For now, all routines in the module assume that maps obey spherical boundary conditions
!		For now, all maps are not redundant at the vertical edges
!		
!		This means that for both types of grids, 360.0/(delta_x) longitude points are needed, and the only different is
!		whether the values are offset by 0.5*delta_x
!
!		For cell_centered grids, 180/(delta_y) latitude points are needed, and for 
!		Edge centered grids 180/(delta_y) + 1 latitude points are needed
!
!		note that mixed grid types are allowed, with the lat and long grids being different types
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call set_up_map(min_x,delta_x,num_x,type_x,min_y,delta_y,num_y,type_y,type_bc,mp,error)
! 
!@*****
		use NAN_support

		implicit none

		real(4),intent(IN)			:: min_x
		real(4),intent(IN)			:: delta_x
		integer(4),intent(IN)		:: num_x
		integer(4),intent(IN)		:: type_x
		real(4),intent(IN)			:: min_y
		real(4),intent(IN)			:: delta_y
		integer(4),intent(IN)		:: num_y 
		integer(4),intent(IN)		:: type_y
		integer(4),intent(IN)		:: type_bc
		type(map_str),intent(INOUT)	:: mp
		integer(4),intent(OUT)		:: error

		if (finite(min_x)) then
			mp%min_x = min_x
		else
			mp%min_x = NAN
			error = 1
			return
		endif

		if ((finite(delta_x)) .and.(delta_x .gt. 0.0)) then
			mp%delta_x = delta_x
		else
			mp%delta_x = NAN
			error = 1
			return
		endif

		if (num_x .gt. 0) then
			mp%num_x = num_x
		else
			mp%num_x = 0
			error = 1
			return
		endif

		if (type_x .eq. CELL_CENTERED) then
			mp%type_x = type_x
		else
			if (type_x .eq. EDGE_CENTERED) then
				mp%type_x = type_x
			else
				mp%type_x = 0
				error = 1
				return
			endif
		endif


		if (finite(min_y)) then
			mp%min_y = min_y
		else
			mp%min_y = NAN
			error = 1
			return
		endif

		if ((finite(delta_y)) .and.(delta_y .gt. 0.0)) then
			mp%delta_y = delta_y
		else
			mp%delta_y = NAN
			error = 1
			return
		endif

		if (num_y .gt. 0) then
			mp%num_y = num_y
		else
			mp%num_y = 0
			error = 1
			return
		endif

		if (type_y .eq. CELL_CENTERED) then
			mp%type_y = type_y
		else
			if (type_y .eq. EDGE_CENTERED) then
				mp%type_y = type_y
			else
				mp%type_y = 0
				error = 1
				return
			endif
		endif

		select case(type_bc)	
			case(SPHERICAL)
				mp%type_bc = SPHERICAL
			case(RECTANGULAR)
				mp%type_bc = RECTANGULAR
			case default
				mp%type_bc = 0
				error = 1
				return
		end select

		
		if (mp%type_bc .eq. SPHERICAL) then 

			! make sure things make sense for spherical boundaries

			if (modulo(mp%num_x,2) .eq. 1) then
				print *,'Error, must have an even number of x value for spherical boundaries'
				Error = -4
				return
			endif

			if (mp%min_x .ne. 0.0) then
				print *,'Error, must have x_min = 0 for spherical boundaries'
				Error = -1
				return
			endif

			if (abs(mp%delta_x*mp%num_x - 360.0) .gt. SMALL_NUMBER) then
				print *,'Error, delta_x and num_x doesnt make sense for spherical boundaries'
				Error = -8
				return
			endif

			if (abs(mp%min_y+90.0) .gt. SMALL_NUMBER) then
				print *,'Error, must have y_min = -90.0 for spherical boundaries '
				Error = -1
				return
			endif
			select case(type_y)
				case (CELL_CENTERED)
					if (abs(mp%delta_y*mp%num_y-180.0) .gt. SMALL_NUMBER) then
						print *,'Error, delta_y and num_y make no sense for spherical boundaries'
						error = -8
						return
					endif
				case (EDGE_CENTERED)
					if (abs(mp%delta_y*(mp%num_y-1)-180.0) .gt. SMALL_NUMBER) then
						print *,'Error, delta_y and num_y make no sense for spherical boundaries'
						error = -8
						return
					endif
			end select
			

		endif


		select case(mp%type_bc)
			case(SPHERICAL)
				mp%max_x = 360.0 
				mp%max_y = 90.0
			case(RECTANGULAR)
				select case(mp%type_x)
					case(EDGE_CENTERED)
						mp%max_x = min_x+delta_x*(num_x -1)
					case(CELL_CENTERED)
						mp%max_x = min_x+delta_x*num_x
					case default
						mp%max_x = NAN
				end select

				select case(mp%type_y)
					case(EDGE_CENTERED)
						mp%max_y = min_y +delta_y*(num_y-1)
					case(CELL_CENTERED)
						mp%max_y = min_y + delta_y*num_y
					case default
						mp%max_y = NAN
				end select
			case default
				mp%max_x = NAN
				mp%max_y = NAN
		end select

		! allocate space for the map

		allocate(mp%dat(mp%num_x,mp%num_y),STAT = error)

		return

	end subroutine set_up_map

	subroutine zero_map(mp,error)
!@***m* MLT_RSS/maps.f90/zero_map
!  NAME
!    zero_map
! 
!  PURPOSE
!    zeros data in variable of map type
! 
!  DESCRIPTION
!    zeros data in variable of map type
!    
! INPUTS
!   mp       ! map to be zeroed
!
! OUTPUTS
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call zero_map(mp,error)
! 
!@*****
		implicit none

		type(map_str),intent(INOUT)		:: mp
		integer(4),intent(INOUT)			:: error

		mp%dat = 0.0
		error = ior(error,0)

	end subroutine zero_map




	subroutine deallocate_map(mp,error)
!@***m* MLT_RSS/maps.f90/deallocate_map
!  NAME
!    deallocate_map
! 
!  PURPOSE
!    deallocates data space in variable of map type
! 
!  DESCRIPTION
!    deallocates data space in variable of map type
!    
! INPUTS
!   mp       ! map to be deallocated
!
! OUTPUTS
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call deallocate_map(mp,error)
! 
!@*****		
		implicit none

		type(map_str),intent(INOUT)		:: mp
		integer(4),intent(OUT)			:: error

		mp%min_x	= NAN
		mp%max_x	= NAN
		mp%delta_x  = NAN
		mp%num_x    = 0
		mp%type_x   = 0
	    mp%min_y    = NAN
		mp%max_y	= NAN
		mp%delta_y	= NAN
		mp%num_y   	= 0
		mp%type_y   = 0
		mp%type_bc  = 0

		deallocate(mp%dat,STAT = error)

		return

	end subroutine deallocate_map

	subroutine get_map_value(mp,xx,yy,value,error)
!@***m* MLT_RSS/maps.f90/get_map_value
!  NAME
!    get_map_value
! 
!  PURPOSE
!    this subroutine returns the interpolated value at a point x,y
! 
!  DESCRIPTION
!    this subroutine returns the interpolated value at a point x,y
!    note that the interpolation is somewhat different depending on 
!    what type of grid is used for the map structure
!    
! INPUTS
!   mp       ! map to be sampled
!   xx       ! x location
!   yy       ! y location
!
! OUTPUTS
!   value    ! interpolated value
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_map_value(mp,xx,yy,value,error)
! 
!@*****
		
		implicit none

		type(map_str),intent(IN)		:: mp       ! map structure to be interpolated
		real(4),intent(IN)				:: xx	    ! longitude
		real(4),intent(IN)				:: yy		! latitude
		real(4),intent(OUT)				:: value	! the interpolated value
		integer(4),intent(OUT)			:: error

		integer(4)						:: ix_left,ix_right		! map indices of points to the right (left) of desired point
		integer(4)						:: iy_lower,iy_upper    ! map indices of points above (below) the desired point
		real(4)							:: dx,dy				! distance of the desired point to the
																! right (above) the lower left grid point

		real(4)							:: wt_left,wt_right
		real(4)							:: wt_lower,wt_upper
		real(4)							:: tot,tot_wt,wt
		real(4)							:: x,y,v

		error = 0

		! modulo x and y to be within bounds

		if ((.not. finite(xx)) .or. (.not. finite(yy)) .or. &
		    (mp%num_x .eq. 0) .or. (mp%num_y .eq. 0)) then
			value = NAN
			return
		endif

		x = modulo(xx-mp%min_x,mp%max_x - mp%min_x) + mp%min_x
		y = modulo(yy-mp%min_y,mp%max_y - mp%min_y) + mp%min_y

		! find x indices and distance
		select case (mp%type_x)

			case (CELL_CENTERED) 

				ix_left		= floor((x-mp%min_x)/mp%delta_x - 0.5) + 1
				ix_right	= ix_left + 1
				dx			= x - mp%min_x - ((ix_left - 0.5) * mp%delta_x)

			case (EDGE_CENTERED) 
				ix_left  = floor((x-mp%min_x)/mp%delta_x) + 1
				ix_right = ix_left + 1
				dx       = x - mp%min_x - ((ix_left - 1) * mp%delta_x)		 
			case default
				error = 1
				value = NAN
				return
		end select

		! find y indices and distance

		select case(mp%type_y)

			case (CELL_CENTERED)
				
				iy_lower	= floor((y-mp%min_y)/mp%delta_y - 0.5) + 1
				iy_upper	= iy_lower + 1
				dy			= y - mp%min_y - ((iy_lower - 0.5) * mp%delta_y)
			case (EDGE_CENTERED)
				iy_lower	= floor((y-mp%min_y)/mp%delta_y) + 1
				iy_upper	= iy_lower + 1
				dy			= y - mp%min_y - ((iy_lower - 1) * mp%delta_y)
			case default
				error = 1
				return
		end select

		! now perform the interpolation

		wt_right = dx/mp%delta_x
		wt_left  = 1.0 - wt_right

		wt_upper = dy/mp%delta_y
		wt_lower = 1.0 - wt_upper

		tot = 0.0 
		tot_wt = 0.0

		v = map_value(mp,ix_left,iy_lower,error) 
		if (finite(v)) then
			wt = wt_left * wt_lower
			tot = tot +( v * wt)
			tot_wt = tot_wt+wt
		endif 

		v = map_value(mp,ix_left,iy_upper,error) 
		if (finite(v)) then
			wt = wt_left * wt_upper
			tot = tot + (v * wt)
			tot_wt = tot_wt+wt
		endif

		v = map_value(mp,ix_right,iy_upper,error) 
		if (finite(v)) then
			wt = wt_right*wt_upper
			tot = tot + (v * wt)
			tot_wt = tot_wt + wt
		endif
		
		v = map_value(mp,ix_right,iy_lower,error) 
		if (finite(v)) then
			wt = wt_right*wt_lower 
			tot = tot + (v * wt)
			tot_wt = tot_wt+wt
		endif 

		value = tot/tot_wt

		return

	end subroutine get_map_value

	subroutine get_map_value_no_interp(mp,xx,yy,value,error)

!@***m* MLT_RSS/maps.f90/get_map_value_no_interp
!  NAME
!    get_map_value_no_interp
! 
!  PURPOSE
!    this subroutine returns the non-interpolated value at grid point closest to xx,yy
! 
!  DESCRIPTION
!    this subroutine returns the non-interpolated value at grid point closest to xx,yy
!    
! INPUTS
!   mp       ! map to be sampled
!   xx       ! x location
!   yy       ! y location
!
! OUTPUTS
!   value    ! interpolated value
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_map_value_no_interp(mp,xx,yy,value,error)
! 
!@*****
		implicit none

		type(map_str),intent(IN)		:: mp       ! map structure to be interpolated
		real(4),intent(IN)				:: xx	    ! longitude
		real(4),intent(IN)				:: yy		! latitude
		real(4),intent(OUT)				:: value	! the interpolated value
		integer(4),intent(OUT)			:: error

		integer(4)						:: ix					! map index of point closest to the desired point
		integer(4)						:: iy					! map index of point closest to the desired point

		real(4)							:: x,y
		error = 0


		if ((.not. finite(xx)) .or. (.not. finite(yy)) .or. &
		    (mp%num_x .eq. 0) .or. (mp%num_y .eq. 0)) then
			value = NAN
			return
		endif

		! modulo x and y to be within bounds

		x = modulo(xx-mp%min_x,mp%max_x - mp%min_x) + mp%min_x
		y = modulo(yy-mp%min_y,mp%max_y - mp%min_y) + mp%min_y

		! find x indices and distance
		select case (mp%type_x)

			case (CELL_CENTERED) 
				ix		= floor((x-mp%min_x)/mp%delta_x) + 1
			case (EDGE_CENTERED) 
				ix  = floor((x-mp%min_x)/mp%delta_x + 0.5) + 1
			case default
				error = 1
				value = NAN
				return
		end select

		! find y indices and distance

		select case(mp%type_y)

			case (CELL_CENTERED)
				iy	= floor((y-mp%min_y)/mp%delta_y) + 1
			case (EDGE_CENTERED)
				iy	= floor((y-mp%min_y)/mp%delta_y + 0.5) + 1
			case default
				error = 1
				return
		end select

		! now get the data

		value = map_value(mp,ix,iy,error)
		 
		return

	end subroutine get_map_value_no_interp


	subroutine wrap_indices(mp,ix,iy,ix_wrapped,iy_wrapped,error)

!@***m* MLT_RSS/maps.f90/wrap_indices
!  NAME
!    wrap_indices
! 
!  PURPOSE
!    wraps map indiced according to boundary conditions
! 
!  DESCRIPTION
!    wraps map indiced according to boundary conditions
!    
! INPUTS
!   mp       ! map to be sampled
!   ix       ! x index
!   iy       ! y index
!
! OUTPUTS
!   ix_wrapped       ! x index, wrapped
!   iy_wrapped       ! y index, wrapped
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call wrap_indices(mp,ix,iy,ix_wrapped,iy_wrapped,error)
! 
!@*****
		implicit none

		type(map_str),intent(IN)		:: mp
		integer(4),intent(IN)			:: ix
		integer(4),intent(IN)			:: iy
		integer(4),intent(OUT)			:: ix_wrapped
		integer(4),intent(OUT)			:: iy_wrapped
		integer(4),intent(OUT)			:: error

		logical							:: reversed

		select case (mp%type_bc)

			case(SPHERICAL)

				! y values are reflected across poles

				iy_wrapped = modulo(((iy-1)+4*mp%num_y),(2*mp%num_y))+1
				reversed = .false.
				if (iy_wrapped .gt. mp%num_y) then  ! fold   
  					iy_wrapped = (2*mp%num_y)+1 - iy_wrapped
  					reversed = .true.
				endif

				if (reversed) then
					ix_wrapped = ix + mp%num_x/2    ! reflect across pole --  modulo will be taken care of in next part
				else
					ix_wrapped = ix
				endif


				! x values are wrapped 

				ix_wrapped = modulo(ix_wrapped-1,mp%num_x) + 1

			case (RECTANGULAR)

				if ((ix .lt. 1) .or. (ix .gt. mp%num_x) .or. &
				    (iy .lt. 1) .or. (iy .gt. mp%num_y)) then
				   ix_wrapped = 0
				   iy_wrapped = 0	 
				   error = 1
				else
					ix_wrapped = ix
					iy_wrapped = iy
				endif
				return

			case default
			    print *,'Error: Map Boundary Conditions Undefined'
				error = -1
				ix_wrapped = 0
				iy_wrapped = 0	 
		end select

	end subroutine wrap_indices


	real(4)  function map_value(mp,ix,iy,error)
!@***m* MLT_RSS/maps.f90/map_value
!  NAME
!    map_value
! 
!  PURPOSE
		! this subroutine returns the value of the map at xi,yi even
		! if xi and yi are out of range by applying the appropriate
		! boundary conditions
! 
!  DESCRIPTION
		! this subroutine returns the value of the map at xi,yi even
		! if xi and yi are out of range by applying the appropriate
		! boundary conditions
!    
! INPUTS
!   mp       ! map to be sampled
!   ix       ! x index
!   iy       ! y index
!
! OUTPUTS
!   error    ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    z = map_value(mp,ix,iy,error)
! 
!@*****

		implicit none

		type(map_str),intent(IN):: mp
		integer(4),intent(IN)	:: ix
		integer(4),intent(IN)	:: iy
		integer(4),intent(OUT)	:: error

		integer(4)				:: ix_wrapped
		integer(4)				:: iy_wrapped

		error = 0

		call wrap_indices(mp,ix,iy,ix_wrapped,iy_wrapped,error) 

		if (error .eq. 0) then
			map_value = mp%dat(ix_wrapped,iy_wrapped)
		else
			map_value = NAN
		endif


		return

	end function map_value



	subroutine get_grid_location(mp,ix,iy,x,y,error)
!@***m* MLT_RSS/maps.f90/get_grid_location
!  NAME
!    get_grid_location
! 
!  PURPOSE
		! this subroutine returns the location in real map space of
		! the grid point ix,iy
! 
!  DESCRIPTION
		! this subroutine returns the location in real map space of
		! the grid point ix,iy
!    
! INPUTS
!   mp       ! map to be sampled
!   ix       ! x index
!   iy       ! y index
!
! OUTPUTS
!   x       ! x location in real space
!   y       ! x location in real space
!   error   ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_grid_location(mp,ix,iy,x,y,error)
! 
!@*****


		implicit none

		type(map_str),intent(IN)			:: mp		! map structure
		integer(4),intent(IN)				:: ix		! x grid position
		integer(4),intent(IN)				:: iy	    ! y grid position
		real(4),intent(OUT)					:: x        ! x position in real space
		real(4),intent(OUT)					:: y		! y position in real space
		integer(4),intent(OUT)				:: error	! error flag

		error = 0

		if ((ix .lt. 1) .or. (ix .gt. mp%num_x) .or. &
		    (iy .lt.	1) .or. (iy .gt. mp%num_y)) then
		   error = 1
		   x = NAN
		   y = NAN
		   return
		endif

		select case (mp%type_x)

			case(CELL_CENTERED)
				x = (ix - 0.5)*mp%delta_x+mp%min_x
			case(EDGE_CENTERED)
				x = (ix - 1)*mp%delta_x+mp%min_x
			case default
				error = 2
				x = NAN
		end select

		select case (mp%type_y)
			
			case(CELL_CENTERED)
				y = (iy - 0.5)*mp%delta_y+mp%min_y
			case(EDGE_CENTERED)
				y = (iy - 1.0)*mp%delta_y+mp%min_y
			case default
				error = 4
				y = NAN
		end select

		return

	end subroutine

	subroutine get_vertex_location(mp,ix,iy,x,y,error)
!@***m* MLT_RSS/maps.f90/get_vertex_location
!  NAME
!   get_vertex_location
! 
!  PURPOSE
		! this subroutine returns the location in real map space of
		! the vertex point ix,iy
! 
!  DESCRIPTION
		! this subroutine returns the location in real map space of
		! the vertex point ix,iy
!    
! INPUTS
!   mp       ! map to be sampled
!   ix       ! x index
!   iy       ! y index
!
! OUTPUTS
!   x       ! x location in real space
!   y       ! x location in real space
!   error   ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_vertex_location(mp,ix,iy,x,y,error)
! 
!@*****


		implicit none

		type(map_str),intent(IN)			:: mp		! map structure
		integer(4),intent(IN)				:: ix		! x grid position
		integer(4),intent(IN)				:: iy	    ! y grid position
		real(4),intent(OUT)					:: x        ! x position in real space
		real(4),intent(OUT)					:: y		! y position in real space
		integer(4),intent(OUT)				:: error	! error flag

		error = 0

		if ((ix .lt. 0) .or. (ix .gt. mp%num_x) .or. &
		    (iy .lt.	0) .or. (iy .gt. mp%num_y)) then
		   error = 1
		   x = NAN
		   y = NAN
		   return
		endif

		select case (mp%type_x)

			case(CELL_CENTERED)
				x = ix*mp%delta_x+mp%min_x
			case(EDGE_CENTERED)
				x = (ix - 0.5)*mp%delta_x+mp%min_x
			case default
				error = 2
				x = NAN
		end select

		select case (mp%type_y)
			
			case(CELL_CENTERED)
				y = iy*mp%delta_y+mp%min_y
			case(EDGE_CENTERED)
				if (iy .gt. 0) then
					y = (iy - 0.5)*mp%delta_y+mp%min_y
				else
					y = NAN
					error = 1
					return
				endif
			case default
				error = 4
				y = NAN
		end select

		return

	end subroutine get_vertex_location



	subroutine get_grid_indices(mp,x,y,ix,iy,error)
!@***m* MLT_RSS/maps.f90/get_grid_indices
!  NAME
!   get_grid_indices
! 
!  PURPOSE
		! returns the indices of the grid point in whose cell the
		! point x,y is located
! 
!  DESCRIPTION
		! returns the indices of the grid point in whose cell the
		! point x,y is located
!    
! INPUTS
!   mp       ! map to be sampled
!   x       ! x location in real space
!   y       ! x location in real space
! OUTPUTS
!   ix       ! x index
!   iy       ! y index
!   error   ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_grid_indices(mp,x,y,ix,iy,error)
! 
!@*****


		implicit none

		type(map_str),intent(IN)			:: mp   ! map structure
		real(4),intent(IN)					:: x
		real(4),intent(IN)					:: y
		integer(4),intent(OUT)				:: ix
		integer(4),intent(OUT)				:: iy
		integer(4),intent(OUT)				:: error

		! check ranges

		if ((x .lt. mp%min_x) .or. (x .gt. mp%max_x) .or. &
		    (y .lt. mp%min_y) .or. (y .gt. mp%max_y)) then
			error = 1
			ix = 0
			iy = 0
			return
		endif

		! find x index

		select case (mp%type_x)

			case(CELL_CENTERED)
				ix = floor((x-mp%min_x)/mp%delta_x)+1
				if (ix .eq. mp%num_x+1) ix = mp%num_x  ! this is really only for x = max_x
			case(EDGE_CENTERED)
				ix = floor((x-mp%min_x)/mp%delta_x+0.5)+1
				if (ix .eq. mp%num_x+1) then
					select case (mp%type_bc)
						case (SPHERICAL)
							ix = 1
						case (RECTANGULAR)
							ix = mp%num_x
							! I don't think this should evere happen
						case default
							ix = 0
							error = 1
					end select
				endif
			case default
				error = 2
				ix = 0
		end select

		select case (mp%type_y)

			case(CELL_CENTERED)
				iy = floor((y-mp%min_y)/mp%delta_y)+1
				if (iy .eq. mp%num_y+1) iy = mp%num_y  ! this is really only for x = max_x
			case(EDGE_CENTERED)
				iy = floor((y-mp%min_y)/mp%delta_y+0.5)+1
				if (iy .eq. mp%num_y+1) then
					iy = mp%num_y
				endif
			case default
				error = 2
				iy = -1
		end select

		return

	end subroutine get_grid_indices

	subroutine get_vertex_indices(mp,x,y,ix,iy,error)
!@***m* MLT_RSS/maps.f90/get_vertex_indices
!  NAME
!   get_vertex_indices
! 
!  PURPOSE
		! returns the indices of the vertex point nearest x,y.  The vertex point is
		! where the edges of the cells cross
!  DESCRIPTION
		! returns the indices of the vertex point nearest x,y.  The vertex point is
		! where the edges of the cells cross
!    
! INPUTS
!   mp       ! map to be sampled
!   x       ! x location in real space
!   y       ! x location in real space
! OUTPUTS
!   ix       ! x index
!   iy       ! y index
!   error   ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_vertex_indices(mp,x,y,ix,iy,error)
! 
!@*****


		implicit none

		type(map_str),intent(IN)			:: mp   ! map structure
		real(4),intent(IN)					:: x
		real(4),intent(IN)					:: y
		integer(4),intent(OUT)				:: ix
		integer(4),intent(OUT)				:: iy
		integer(4),intent(OUT)				:: error

		! check ranges

		if ((x .lt. mp%min_x) .or. (x .gt. mp%max_x) .or. &
		    (y .lt. mp%min_y) .or. (y .gt. mp%max_y)) then
			error = 1
			ix = 0
			iy = 0
			return
		endif

		! find x vertex index

		select case (mp%type_x)

			case(CELL_CENTERED)
				ix = floor((x-mp%min_x)/mp%delta_x-0.5)+1
			case(EDGE_CENTERED)
				ix = floor((x-mp%min_x)/mp%delta_x)+1
			case default
				error = 2
				ix = 0
		end select

		select case (mp%type_y)

			case(CELL_CENTERED)
				iy = floor((y-mp%min_y)/mp%delta_y-0.5)+1
			case(EDGE_CENTERED)
				iy = floor((y-mp%min_y)/mp%delta_y)+1
			case default
				error = 2
				iy = -1
		end select

		return

	end subroutine get_vertex_indices

	subroutine calc_relative_position(mp,x,y,ix,iy,x_rel,y_rel,error)
!@***m* MLT_RSS/maps.f90/calc_relative_position
!  NAME
!   calc_relative_position
! 
!  PURPOSE
! calculates the relative positon of the point x,y relative to the
! vertex ix,iy.  
!  DESCRIPTION
! calculates the relative positon of the point x,y relative to the
! vertex ix,iy.  Since x, y is constrained to be within the
! bounds of the map, the modulo arithmatic is relatively simple,
! as long as x,y id within half a delta of the vertex.  This is
! assumed to be the case here
!    
! INPUTS
!   mp       ! map to be sampled
!   x       ! x location in real space
!   y       ! x location in real space
!   ix       ! x index
!   iy       ! y index
! OUTPUTS
!   x_rel   ! relative x distance to vertex
!   y_rel   ! relative y distance to vertex
!   error   ! error flag
!
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call calc_relative_position(mp,x,y,ix,iy,x_rel,y_rel,error)
! 
!@*****		


		implicit none

		type(map_str),intent(IN)			:: mp   ! map structure
		real(4),intent(IN)					:: x
		real(4),intent(IN)					:: y
		integer(4),intent(IN)				:: ix
		integer(4),intent(IN)				:: iy
		real(4),intent(OUT)					:: x_rel
		real(4),intent(OUT)					:: y_rel
		integer(4),intent(OUT)				:: error

		real(4)								:: xv,yv

		! check ranges

		if ((x .lt. mp%min_x) .or. (x .gt. mp%max_x) .or. &
		    (y .lt. mp%min_y) .or. (y .gt. mp%max_y)) then
			error = 1
			x_rel = NAN 
			y_rel = NAN
			return
		endif

		! find location of vertex in real space

		call get_vertex_location(mp,ix,iy,xv,yv,error)

		select case(mp%type_bc)

			case(SPHERICAL)
				x_rel = modulo(x-xv+180,360.0) - 180.0
				y_rel = y - yv

			case(RECTANGULAR)
				x_rel = x - xv
				y_rel = y - yv

			case default
				x_rel = NAN
				y_rel = NAN
		end select

		if ((x_rel .gt. mp%delta_x/2.0) .or. (abs(y_rel) .gt. mp%delta_y/2.0)) then
			print *,'Warning rel_x, or rel_y in calc_rel_position too big'
			print *,'Rel X = ',x_rel,' Rel Y = ',y_rel
		endif

		return

	end subroutine calc_relative_position


	subroutine write_map(mp,unit_num,error)
!@***m* MLT_RSS/maps.f90/write_map
!  NAME
!   write_map
! 
!  PURPOSE
!    writes out map in a standard, self describing format
!  DESCRIPTION
!    writes out map in a standard, self describing format
!    
! INPUTS
!   mp       ! map to be written
!   unit_num ! logical unit to write to
!
! OUTPUTS
!   error   ! error flag
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call write_map(mp,unit_num,error)
! 
!@*****
		implicit none

		type(map_str),intent(IN)			:: mp
		integer(4),intent(IN)				:: unit_num
		integer(4),intent(OUT)				:: error

		error = 0

		write(unit_num) mp%min_x
		write(unit_num) mp%max_x
		write(unit_num) mp%delta_x
		write(unit_num) mp%num_x
		write(unit_num) mp%min_y
		write(unit_num) mp%max_y
		write(unit_num) mp%delta_y
		write(unit_num) mp%num_y
		write(unit_num) mp%dat

		return
	end subroutine write_map

	subroutine read_map(unit_num,mp,error)
!@***m* MLT_RSS/maps.f90/read_map
!  NAME
!   read_map
! 
!  PURPOSE
!    reads in map from a standard, self describing format
!  DESCRIPTION
!    reads in map from a standard, self describing format
!    allocated map structure according to description in file
!    
! INPUTS
!   unit_num ! logical unit to write to
!
! OUTPUTS
!   mp       ! map variable to be read into
!   error   ! error flag
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call ead_map(unit_num,mp,error)
! 
!@*****
		implicit none

		type(map_str),intent(OUT)			:: mp
		integer(4),intent(IN)				:: unit_num
		integer(4),intent(OUT)				:: error

		real(4)								:: min_x
		real(4)								:: max_x
		real(4)								:: delta_x
		integer(4)							:: num_x
		real(4)								:: min_y
		real(4)								:: max_y
		real(4)								:: delta_y
		integer(4)							:: num_y
		integer(4)							:: type_x
		integer(4)							:: type_y

		! try to figure out axis type based on limits

		read(unit_num) min_x
		read(unit_num) max_x
		read(unit_num) delta_x
		read(unit_num) num_x
		read(unit_num) min_y
		read(unit_num) max_y
		read(unit_num) delta_y
		read(unit_num) num_y


		if (abs(max_x - (min_x+delta_x*num_x)) .lt. SMALL_NUMBER) then
			type_x = CELL_CENTERED
		else
			if (abs(max_x - (min_x+delta_x*(num_x-1))) .lt. SMALL_NUMBER) then
				type_x = EDGE_CENTERED
			else
				print *,'Error -- Unable to determine x axis type'
				type_x = 0
				error = 1
				return
			endif
		endif

		if (abs(max_y - (min_y+delta_y*num_y)) .lt. SMALL_NUMBER) then
			type_y = CELL_CENTERED
		else
			if (abs(max_y - (min_y+delta_y*(num_y-1))) .lt. SMALL_NUMBER) then
				type_y = EDGE_CENTERED
				type_x = EDGE_CENTERED
			else
				print *,'Error -- Unable to determine y axis type'
				type_y = 0
				error = 1
				return
			endif
		endif

		error = 0



		call set_up_map(min_x,delta_x,num_x,type_x,min_y,delta_y,num_y,type_y,SPHERICAL,mp,error)

		read(unit_num) mp%dat

		return
	end subroutine read_map


end module maps






