$freeform
! @***h* MLT_RSS/MSU_CCM3_diurnal.f90
!  NAME
!    Module MSU_CCM3_diurnal
! 
!  PURPOSE
!    Module containing CCM3 diurnal pseudoclass for MSU
! 
!  DESCRIPTION
!    Module containing CCM3 diurnal pseudoclass for MSU
!    contains for loading the CCM3 diurnal climatology, and for retrieving
!    interpolated values from the climatology.  CCM3 data is stored in the 
!    derived type map_str, and routines from map.f90 are used for interpolating.
!
!  ROUTINES
!   allocate_CCM3_Diurnal_Tbs       allocates space for diurnal climatology
!   load_CCM3_Diurnal_Tbs           loads diurnal climatology for all 4 channels
!   CCM3_diurnal_value              calculates interpolated value for diurnal climatology
!   CCM3_diurnal_value_local        calculated interpolated value using LOCAL time as input
!   calculate_diurnal_correction    calculates the diurnal correction for a MSU scan
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use MSU_CCM3_diurnal
! @*****
module msu_CCM3_diurnal

	use msu_constants
	use NAN_support 
	use msu_types
	use maps
	use date_and_time
	use msu_date
	use interpolate

	implicit none

	! declarations for the array that contains the modeled Tb's as a function of
	! lat,lon,height,nadir angle, sea or land, and declaration of associated 
	! constants. 

	character(100),parameter	:: CCM3_DIURNAL_PATH = '\\dell-p1700d\E\MSU_Processing\Module_Data\CCM3_Diurnal_Climatology\'

	integer,parameter	::	NUM_LONS_DIURNAL		= 128
	integer,parameter	::	NUM_LATS_DIURNAL		= 64
	integer,parameter	::	NUM_HOURS				= 24  

	type(map_str),dimension(NUM_CHANNELS,N_FOV,NUM_MONTHS,NUM_HOURS)	 ::	CCM3_Diur
	logical				:: CCM3_Diur_allocated	= .false.
	logical				:: CCM3_Diur_loaded     = .false.


																								 

contains

	subroutine allocate_CCM3_Diurnal_Tbs(Error)
! @***h* MLT_RSS/MSU_CCM3_diurnal.f90
!  NAME
!    allocate_CCM3_Diurnal_Tbs
! 
!  PURPOSE
!    Allocates space for CCM3 diurnal climatology
! 
!  DESCRIPTION
!    Allocates space for CCM3 diurnal climatolog
!
!  INPUTS
!    none
!
!  OUTPUTS
!    error     integer(4), set equal to 0 if allocation successful
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use allocate_CCM3_Diurnal_Tbs(Error)
! @*****	
		integer(4)			::  error

		integer(4)			:: chan
		integer(4)			:: fov
		integer(4)			:: month
		integer(4)			:: hour

			do chan = 1,NUM_CHANNELS
				do fov = 1, N_FOV
					do month = 1,NUM_MONTHS
						do hour = 1,NUM_HOURS
							call set_up_map( 0.0,							&
											 2.8125,						&
											 128,							&
											 EDGE_CENTERED,					&
											 -90.0,							&
											 2.8125,						&
											 64,							&
											 CELL_CENTERED,					&
											 SPHERICAL,						&
											 CCM3_diur(chan,fov,month,hour),&
											 error)
							if (error .ne. 0) then
								return
							endif
						enddo
					enddo
				enddo
			enddo

			CCM3_Diur_allocated = .true.

		return

	end subroutine allocate_CCM3_Diurnal_Tbs

	subroutine load_CCM3_Diurnal_Tbs(error)

		use mcs

		integer(4)			::  error

		integer(4)			:: chan
		integer(4)			:: fov
		integer(4)			:: month
		integer(4)			:: hour
		integer(4)			:: mcs_Error


		character(len=120)			:: file_name
		type(real_4_matrix_str)		:: temp

		error = 0

		

		do chan = 1,NUM_CHANNELS

			write(file_name,111) trim(CCM3_DIURNAL_PATH),chan
111			format(a,'mn_diur_cycles_chan_',i1,'_ccm3_128x64_local_time.mcs')
			print *,file_name
			open(unit=3,form='binary',file=file_name,status='old')

			do fov = 1,6
				do month = 1,12
					do hour = 1,24
						call read_real_4_mcs(3,temp,mcs_error)
						if (mcs_error .eq. 0) then
							ccm3_diur(chan,fov,month,hour)%dat = temp%dat
						else
							call deallocate_mcs(temp,error)
							error = -1	
							return
						endif
						call deallocate_mcs(temp,error)
					enddo
				enddo
			enddo

			close(unit = 3)
		enddo

		CCM3_Diur_loaded = .true.

	end subroutine load_CCM3_Diurnal_Tbs

	real(4) function CCM3_diurnal_value(chan,fov,date,lat,lon,error)

		integer(4)		:: chan
		integer(4)		:: fov
		type(date_str)	:: date
		real(4)			:: lat
		real(4)			:: lon
		integer(4)		:: error

		integer(4)		:: hour_1
		integer(4)		:: hour_2
		integer(4)		:: month_1
		integer(4)		:: month_2
		integer(4)		:: month_to_get_1
		integer(4)		:: month_to_get_2
		integer(4)		:: leap
		integer(4)		:: fv

		real(4),dimension(2,2)			:: Tb
		type(time_str)					:: local_time

		real(4)							:: y,y_min,y_max
		real(4)							:: x,x_min,x_max


		if (.not. CCM3_Diur_Allocated) call allocate_CCM3_Diurnal_Tbs(Error)
		if (.not. CCM3_Diur_loaded) call load_CCM3_Diurnal_Tbs(Error)

				fv = 6 - iabs(fov-6)


		call find_bracket_months(date,month_1,month_2)
		month_to_get_1 = month_1
		month_to_get_2 = month_2
		if (month_1 .eq. 0)  month_to_get_1 = 12
		if (month_2 .eq. 13) month_to_get_2 = 1

		local_time = convert_to_local_time(date%time,lon,error)
 

		hour_1 = modulo(floor(local_time%m_secs/3600.0 - 0.5),24)+1
		hour_2 = modulo(hour_1,24)+1

		call get_map_value(ccm3_diur(chan,fv,month_to_get_1,hour_1),lon,lat,Tb(1,1),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_1,hour_2),lon,lat,Tb(1,2),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_2,hour_1),lon,lat,Tb(2,1),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_2,hour_2),lon,lat,Tb(2,2),error)

		! x is months, y is hours

		y_min = hour_1 - 0.5
		y_max = hour_2 - 0.5
		y = local_time%m_secs/3600.0
		if (y_min .gt. y_max) y_min = y_min - 24.0
		if (y .gt. y_max) y = y - 24.0

		leap = 1
		if (gregorian_leap_year(date%year)) leap = 2

		x_min = month_centers(month_1,leap)
		x_max = month_centers(month_2,leap)
		x = date%oday

		CCM3_diurnal_value = interpolate_2D(x,y,tb,x_min,x_max,y_min,y_max)

	end function CCM3_diurnal_value

	real(4) function CCM3_diurnal_value_local(chan,fov,date,local_time,lat,lon,error)

		integer(4)		:: chan
		integer(4)		:: fov
		type(date_str)	:: date
		type(time_str)	:: local_time
		real(4)			:: lat
		real(4)			:: lon
		integer(4)		:: error

		integer(4)		:: hour_1
		integer(4)		:: hour_2
		integer(4)		:: month_1
		integer(4)		:: month_2
		integer(4)		:: month_to_get_1
		integer(4)		:: month_to_get_2
		integer(4)		:: leap
		integer(4)		:: fv

		real(4),dimension(2,2)			:: Tb

		real(4)							:: y,y_min,y_max
		real(4)							:: x,x_min,x_max

		fv = 6 - iabs(fov-6)


		if (.not. CCM3_Diur_Allocated) call allocate_CCM3_Diurnal_Tbs(Error)
		if (.not. CCM3_Diur_loaded) call load_CCM3_Diurnal_Tbs(Error)

		call find_bracket_months(date,month_1,month_2)
		month_to_get_1 = month_1
		month_to_get_2 = month_2
		if (month_1 .eq. 0)  month_to_get_1 = 12
		if (month_2 .eq. 13) month_to_get_2 = 1

		hour_1 = modulo(floor(local_time%m_secs/3600.0 - 0.5),24)+1
		hour_2 = modulo(hour_1,24)+1

		call get_map_value(ccm3_diur(chan,fv,month_to_get_1,hour_1),lon,lat,Tb(1,1),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_1,hour_2),lon,lat,Tb(1,2),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_2,hour_1),lon,lat,Tb(2,1),error)
		call get_map_value(ccm3_diur(chan,fv,month_to_get_2,hour_2),lon,lat,Tb(2,2),error)

		! x is months, y is hours

		y_min = hour_1 - 0.5
		y_max = hour_2 - 0.5
		y = local_time%m_secs/3600.0
		if (y_min .gt. y_max) y_min = y_min - 24.0
		if (y .gt. y_max) y = y - 24.0

		leap = 1
		if (gregorian_leap_year(date%year)) leap = 2

		x_min = month_centers(month_1,leap)
		x_max = month_centers(month_2,leap)
		x = date%oday

		CCM3_diurnal_value_local = interpolate_2D(x,y,tb,x_min,x_max,y_min,y_max)

	end function CCM3_diurnal_value_local
		
	subroutine calculate_diurnal_correction(L2A_scan,error) 

		type(msu_L2A_scan)			:: L2A_scan
		integer(4)					:: error

		integer(4)					:: chan
		integer(4)					:: fov

		real(4)						:: Tb_Diur_Noon
		real(4)						:: Tb_Diur_Ref

		error = 0

		do chan = 1,4
			do fov = 1,11
				Tb_diur_noon = CCM3_diurnal_value_local(chan,fov,L2A_Scan%date,NOON,		&
													L2A_scan%fov_latitude(fov),		&
													L2A_scan%fov_longitude(fov),	&
													error)

				Tb_diur_ref  = CCM3_diurnal_value(chan,fov,L2A_Scan%date,		&
													L2A_scan%fov_latitude(fov),		&
													L2A_scan%fov_longitude(fov),	&
													error)

				L2A_scan%MSU_TBs_diurnal_corr(chan,fov) = Tb_diur_noon - Tb_diur_ref
			enddo
		enddo
	end subroutine calculate_diurnal_correction
end module msu_CCM3_diurnal
