$freeform

! @***h* MLT_RSS/MSU_Level_2A.f90
!  NAME
!    msu_L2A_inventory
! 
!  PURPOSE
!    msu_L2A psuedoclass
! 
!  DESCRIPTION
!    pseudoclass for reading and writing MSU L2A orbit data
!    
!  ROUTINES

!      zero_msu_L2A_scan
!      zero_msu_L2A_header
!      zero_L2A_orbit
!      encode_msu_L2A_scan
!      decode_msu_L2A_scan
!      encode_msu_L2A_header
!      decode_msu_L2A_header
!      write_L2A_orbit_file
!      read_L2A_orbit_file
!      encode_msu_L2A_Corr_header
!      decode_msu_L2A_Corr_header
!      encode_msu_L2A_Corr_scan
!      decode_msu_L2A_Corr_scan
!      write_L2A_corr_file
!      read_L2A_corr_file
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use msu_L2A_inventory
! 
! @*****

module MSU_Level_2A

	use NAN_support
	use Date_and_Time
	use MSU_Types
	use MSU_constants
	use MSU_date
	use msu_calibration
	use msu_CCM3_diurnal
	use gzip

	implicit none

	integer(4),parameter  :: INCIDENCE = 1
	integer(4),parameter  :: LIMB	   = 2
	integer(4),parameter  :: DIURNAL   = 3

contains

    

    subroutine zero_msu_L2A_scan(L2A_scan)
        ! @***h* zero_msu_L2A_scan
        !  NAME
        !    zero_msu_L2A_scan
        ! 
        !  PURPOSE
        !    zeros data in a MSU L2A scan
        ! 
        !  DESCRIPTION
        !    zeros data in a MSU L2A scan
        !    
        !  INPUTS
        !    type(msu_L2A_scan)			:: L2A_scan
        !  OUTPUTS
        !   type(msu_L2A_scan)			:: L2A_scan (now zeroed)
        ! 
        !  AUTHOR
        !    Carl Mears, Remote Sensing Systems
        ! 
        ! COPYRIGHT
        !   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
        !   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
        !   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
        !   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
        !   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
        !   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
        !   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
        !   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
        !   TO USERS.
        ! 
        !  HISTORY
        !    2/21/2012 Initial Version prepared for NCDC
        !  
        !  USAGE
        !    call zero_msu_L2A_scan(L2A_scan)
        !
        ! @*****

		use NAN_support

		type(msu_L2A_scan)			:: L2A_scan


		L2A_scan%scan_number			= 0		
		L2A_scan%date%rd_2000			= NAN
		L2A_scan%scan_qual				= 0
		L2A_scan%fov_qual				= 0 
		L2A_scan%height					= NAN
		L2A_scan%fov_latitude			= NAN
		L2A_scan%fov_longitude			= NAN
		L2A_scan%fov_incidence			= NAN
		L2A_scan%T%R_PRT_A1				= NAN
		L2A_scan%T%R_PRT_A2				= NAN
		L2A_scan%T%R_PRT_B1				= NAN
		L2A_scan%T%R_PRT_B2				= NAN
		L2A_scan%T%Target1_temp			= NAN
		L2A_scan%T%Target2_temp			= NAN
		L2A_scan%T%Target1_Temp_A		= NAN		
		L2A_scan%T%Target1_Temp_B		= NAN
		L2A_scan%T%Target2_Temp_A		= NAN
		L2A_scan%T%Target2_Temp_B		= NAN
		L2A_scan%MSU_Tbs_raw			= NAN
		L2A_scan%MSU_TBs_nom_EIA_Corr	= 0.0
		L2A_scan%MSU_TBs_diurnal_corr	= 0.0
		L2A_scan%MSU_TBs_nadir_corr		= 0.0
		L2A_scan%asc_desc				= 0
		L2A_scan%roll_err				= NAN									!

end subroutine zero_msu_L2A_scan


subroutine zero_msu_L2A_header(L2A_header)
        ! @***h* zero_msu_L2A_header
        !  NAME
        !    zero_msu_L2A_header
        ! 
        !  PURPOSE
        !    zeros data in a MSU L2A header
        ! 
        !  DESCRIPTION
        !    zeros data in a MSU L2A header
        !    
        !  INPUTS
        !    type(msu_L2A_orbit_header)			:: L2A_header
        !  OUTPUTS
        !   type(msu_L2A_orbit_header)			:: L2A_header (now zeroed)
        ! 
        !  AUTHOR
        !    Carl Mears, Remote Sensing Systems
        ! 
        ! COPYRIGHT
        !   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
        !   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
        !   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
        !   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
        !   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
        !   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
        !   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
        !   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
        !   TO USERS.
        ! 
        !  HISTORY
        !    2/21/2012 Initial Version prepared for NCDC
        !  
        !  USAGE
        !    call zero_msu_L2A_header(L2A_header)
        !
        ! @*****

		type(MSU_L2A_orbit_header)		:: L2A_header

	 	
		
		L2A_header%satellite_name		= '       '
		L2A_header%orbit_number			= 0
		L2A_header%number_of_swaths		= 0
		L2A_header%start_swath_num		= 0
		L2A_header%end_swath_num		= 0
		L2A_header%start_rd_2000		= 0
		L2A_header%end_rd_2000			= 0

end subroutine zero_msu_L2A_header


subroutine zero_L2A_orbit(orbit_scan_data)
! @***h* zero_msu_L2A_orbit
!  NAME
!    zero_msu_L2A_orbit
! 
!  PURPOSE
!    zeros data in a MSU L2A orbit
! 
!  DESCRIPTION
!    zeros data in a MSU L2A orbit
!    
!  INPUTS
!    type(msu_L2A_orbit_data)							:: orbit_scan_data
!  OUTPUTS
!    type(msu_L2A_orbit_data)							:: orbit_scan_data (now zeroed)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call zero_msu_L2A_header(L2A_header)
!
! @*****
	type(msu_L2A_orbit_data)							:: orbit_scan_data
	integer(4)											:: scan_number

	do scan_number = 1,MAX_SCANS_PER_ORBIT
		call zero_msu_L2A_scan(orbit_scan_data%scan_data(scan_number))
	enddo
	orbit_scan_data%scan_valid = 0

end subroutine zero_L2A_orbit

subroutine encode_msu_L2A_scan(	L2A_scan,scan_char)
									
! @***h* encode_msu_L2A_scan
!  NAME
!    encode_msu_L2A_scan
! 
!  PURPOSE
!    converts data in a MSU L2A orbit to binary form
! 
!  DESCRIPTION
!    converts data in a MSU L2A orbit to binary form 348 bytes long.  This form is used to
!    store L2A data in the RSS MSU L2A files.
!    
!  INPUTS
!    type(msu_L2A_scan)			:: L2A_scan  ! data from single L2A scan

!  OUTPUTS
!    character(len = 348)		:: scan_char ! same data converted to flat binaru form for writing to file
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call encode_msu_L2A_scan(L2A_scan,scan_char)
!
! @*****									
				 
		use type_conversion


		character(len = 348)		:: scan_char
		type(msu_L2A_scan)			:: L2A_scan

		integer(4)					:: fov
		integer(4)					:: offset
		integer(4)					:: chan


		scan_char(1:4)		= int_4_to_string(L2A_scan%scan_number)
		scan_char(5:12)		= real_8_to_string(L2A_scan%date%rd_2000)
		scan_char(13:16)	= int_4_to_string(L2A_scan%scan_qual)
		do fov = 1,11
			scan_char(16+fov:16+fov) = char(L2A_scan%fov_qual(fov))
		enddo
		scan_char(28:28) = char(0)
		scan_char(29:32) = real_4_to_string(L2A_scan%height)
		do fov = 1,11
			offset = 29 + fov*4
			scan_char(offset:offset+3) = real_4_to_string(L2A_scan%fov_latitude(fov))
			offset = 73 + fov*4
			scan_char(offset:offset+3) = real_4_to_string(L2A_scan%fov_longitude(fov))
			offset = 117 + fov*4
			scan_char(offset:offset+3) = real_4_to_string(L2A_scan%fov_incidence(2,fov))
		enddo

		scan_char(165:168) = real_4_to_string(real(L2A_scan%T%Target1_temp))
		scan_char(169:172) = real_4_to_string(real(L2A_scan%T%Target2_temp))

		do chan = 1,4
			do fov = 1,11
				offset = 125 + (chan*44) + (fov*4)
				scan_char(offset:offset+3) = real_4_to_string(real(L2A_scan%MSU_Tbs_raw(chan,fov)))
			enddo
		enddo 

end subroutine encode_msu_L2A_scan


subroutine decode_msu_L2A_scan(	scan_char,L2A_scan)
								
! @***h* decode_msu_L2A_scan
!  NAME
!    decode_msu_L2A_scan
! 
!  PURPOSE
!    converts data in a MSU L2A orbit to binary form
! 
!  DESCRIPTION
!    converts data from binary form (348 bytes) to data in a structured type.  The binary form is used to
!    store L2A data in the RSS MSU L2A files.
!    
!  INPUTS
!    character(len = 348)		:: scan_char ! same data converted to flat binaru form for writing to file

!  OUTPUTS
!    type(msu_L2A_scan)			:: L2A_scan  ! data from single L2A scan

!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call decode_msu_L2A_scan(scan_char,L2A_scan)
!
! @*****

		use type_conversion


		character(len = 348)		:: scan_char
		type(msu_L2A_scan)			:: L2A_scan

		integer(4)					:: fov
		integer(4)					:: offset
		integer(4)					:: chan

		real(4)						:: temp_long


		L2A_scan%scan_number = string_to_int_4(scan_char(1:4))		
		L2A_scan%date%rd_2000 = string_to_real_8(scan_char(5:12))
		L2A_scan%scan_qual = string_to_int_4(scan_char(13:16))

		do fov = 1,11
			L2A_scan%fov_qual(fov) = char_to_int_1(scan_char(16+fov:16+fov)) 
		enddo

		L2A_scan%height = string_to_real_4(scan_char(29:32))
		do fov = 1,11
			offset = 29 + fov*4
			L2A_scan%fov_latitude(fov) = string_to_real_4(scan_char(offset:offset+3))
			offset = 73 + fov*4
			temp_long = string_to_real_4(scan_char(offset:offset+3)) 
			if (finite(temp_long)) then
				L2A_scan%fov_longitude(fov) = modulo(720.0+temp_long,360.0)
			else
				L2A_scan%fov_longitude = NAN
			endif
			offset = 117 + fov*4
			L2A_scan%fov_incidence(2,fov) = string_to_real_4(scan_char(offset:offset+3))
		enddo

		L2A_scan%T%Target1_temp = string_to_real_4(scan_char(165:168)) 
		L2A_scan%T%Target2_temp = string_to_real_4(scan_char(169:172)) 

		do chan = 1,4
			do fov = 1,11
				offset = 125 + (chan*44) + (fov*4)
				L2A_scan%MSU_Tbs_raw(chan,fov) = string_to_real_4(scan_char(offset:offset+3))
			enddo
		enddo 

end subroutine decode_msu_L2A_scan

subroutine decode_msu_L2A_header(	header,		&
									L2A_header)
! @***h* decode_msu_L2A_header
!  NAME
!    decode_msu_L2A_header
! 
!  PURPOSE
!    converts data in a MSU L2A orbit to binary form
! 
!  DESCRIPTION
!    converts data from binary form (348 bytes) to data in a structured type.  The binary form is used to
!    store L2A data in the RSS MSU L2A files.
!    
!  INPUTS
!    character(len = 348)		:: header ! same data converted to flat binaru form for writing to file

!  OUTPUTS
!    type(MSU_L2A_orbit_header)			:: L2A_header  ! data from L2A orbit
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call decode_msu_L2A_header(header,L2A_header)
!
! @*****
		use type_conversion


		character(len = 348)			:: header
		type(MSU_L2A_orbit_header)		:: L2A_header


		L2A_header%satellite_name		= header(1:7)
		L2A_header%orbit_number			= string_to_int_4(header(9:12))
		L2A_header%number_of_swaths		= string_to_int_4(header(13:16))
		L2A_header%start_swath_num		= string_to_int_4(header(17:20))
		L2A_header%end_swath_num		= string_to_int_4(header(21:24))
		L2A_header%start_rd_2000		= string_to_real_8(header(25:32))
		L2A_header%end_rd_2000			= string_to_real_8(header(33:40))
		L2A_header%pad= header(41:348)

end subroutine decode_msu_L2A_header

subroutine encode_msu_L2A_header(L2A_Header,		&
								 header)
! @***h* encode_msu_L2A_header
!  NAME
!    encode_msu_L2A_header
! 
!  PURPOSE
!    converts data in a MSU L2A orbit to binary form
! 
!  DESCRIPTION
!    converts data from structured type to binary form (348 bytes).  The binary form is used to
!    store L2A data in the RSS MSU L2A files.  The header record is the first record in each file.
!    
!  INPUT
!    type(MSU_L2A_orbit_header)			:: L2A_header  ! data from L2A orbit
!
!  OUTPUT
!    character(len = 348)		:: header ! same data converted to flat binaru form for writing to RSS L2A file 
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call encode_msu_L2A_header(header,L2A_header)
!
! @*****

		use type_conversion

		type(msu_L2A_orbit_header)		:: L2A_Header
		character(len=348)				:: header

		header(1:7)		= L2A_header%satellite_name 
		header(9:12)	= int_4_to_string(L2A_header%orbit_number)		
		header(13:16)	= int_4_to_string(L2A_header%number_of_swaths)
		header(17:20)	= int_4_to_string(L2A_header%start_swath_num)
		header(21:24)	= int_4_to_string(L2A_header%end_swath_num)
		header(25:32)	= real_8_to_string(L2A_header%start_rd_2000)
		header(33:40)	= real_8_to_string(L2A_header%end_rd_2000)
		header(41:348)	= L2A_header%pad 

end subroutine encode_msu_L2A_orbit_header



subroutine write_L2A_orbit_file(	satellite_name,			&
									orbit_number,			&
									orbit_scan_data,		&
									log_unit,				&
									error)
! @***h* write_L2A_orbit_file
!  NAME
!    write_L2A_orbit_file
! 
!  PURPOSE
!    writes data to RSS MSU L2A orbit file
! 
!  DESCRIPTION
!    writes data to RSS MSU L2A orbit file.  data is compressed using gzip.
!    
!  INPUT
!  character(len=7)					:: satellite_name   ! satellite name (TIROS-N, etc)
!  integer(4)						:: orbit_number     ! orbit number ot write
!  type(msu_L2A_orbit_header)		:: msu_orbit_header ! header data to write
!  type(msu_L2A_orbit_data)			:: orbit_scan_data  ! data to write
!  integer(4)						:: log_unit  ! logical unit to write to
!
!  OUTPUT
!   integer(4)                      :: error   ! = 0 is all is OK, non zero if error encountered.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call write_L2A_orbit_file(header,L2A_header)
!
! @*****
		use msflib

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(msu_L2A_orbit_header)							:: msu_orbit_header
		type(msu_L2A_orbit_data)							:: orbit_scan_data
		integer(4)											:: log_unit
		integer(4)											:: error

		character(len=348),dimension(MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: num_scans_valid
		integer(4)						:: num_scans_new

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fz_err
		integer(4)						:: scan_index
		integer(4)						:: first_valid_scan
		integer(4)						:: last_valid_scan
		character(len = 20)				:: begin_date_String
		character(len = 20)				:: end_date_String

		type  (FILE$INFO)				:: info

		! encode all the data into a large array of characters, keeping track of
		! important header info

		error = 0

		num_scans_valid = 0
		num_scans_new   = 0
		do scan_index = 1,MAX_SCANS_PER_ORBIT
			if (orbit_scan_data%scan_valid(scan_index) .gt. 0)	then
				num_scans_valid = num_scans_valid + 1
				if (num_scans_valid .eq. 1) first_valid_scan = scan_index
				last_valid_scan = scan_index 
				call encode_msu_L2A_scan(orbit_scan_data%scan_data(scan_index),buffer(num_scans_valid+1))
			endif
		enddo
		
		! don't bother to write the file if no valid scans

		if (num_scans_valid .le. 0) then
			error = 1
			return
		endif

		msu_orbit_header%satellite_name		= satellite_name
		msu_orbit_header%orbit_number		= orbit_number
		msu_orbit_header%number_of_swaths	= num_scans_valid
		msu_orbit_header%start_swath_num	= orbit_scan_data%scan_data(first_valid_scan)%scan_number
		msu_orbit_header%end_swath_num		= orbit_scan_data%scan_data(last_valid_scan)%scan_number
		msu_orbit_header%start_rd_2000		= orbit_scan_data%scan_data(first_valid_scan)%date%rd_2000
		msu_orbit_header%end_rd_2000		= orbit_scan_data%scan_data(last_valid_scan)%date%rd_2000

		! put header in position 1 in buffer

		call encode_msu_L2A_header(msu_orbit_header,buffer(1))

		!determine the file name (hardwired to Mo1995 for now!)

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999
		write(filename,9001)trim(MSU_L2A_PATH_TEST),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
9001	format(a,a,'\Mo1995\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L1A.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)
	    if (info.length .gt. 0) then
				error = 2				
		endif

		if (log_unit .gt. 0) then
			begin_date_string = date_string(orbit_scan_data%scan_data(first_valid_scan)%date) 
			end_date_string   = date_string(orbit_scan_data%scan_data(last_valid_scan)%date) 
			write(log_unit,9004)orbit_number,							&
								num_scans_valid,						&
								num_scans_new,							&
								msu_orbit_header%start_swath_num,		&
								msu_orbit_header%end_swath_num,			&
								begin_date_string,						&
								end_date_string,filename
9004		format(1x,i6,1x,i4,1x,i4,1x,i4,1x,i4,1x,a20,1x,a20,1x,a100)
		endif

		fz_err = 0
		if (error .ge. 0) then
			size = 348*(num_scans_valid+1)
			call fastgzip(filename,size,buffer(1:num_scans_valid+1), fz_err)
		endif

	end subroutine write_L2A_orbit_file

	subroutine read_L2A_orbit_file(	satellite_name,			&
									orbit_number,			&
									msu_orbit_header,		&
									orbit_scan_data,		&
									error,					&
									path)
! @***h* read_L2A_orbit_file
!  NAME
!    read_L2A_orbit_file
! 
!  PURPOSE
!    reads data from RSS MSU L2A orbit file
! 
!  DESCRIPTION
!    reads data from RSS MSU L2A orbit file.  data is uncompressed using gzip and 
!    stored in the appropriate structures
!    
!  INPUT
!   character(len=7)				:: satellite_name   ! satellite name (TIROS-N, etc)
!   integer(4)						:: orbit_number     ! orbit number to read
!   integer(4)						:: log_unit         ! logical unit to write to
!   character(len =100),optional	:: path             ! path to read from -- if not present, default path is used.
!
!  OUTPUT
!   type(msu_L2A_orbit_header)		:: msu_orbit_header ! header data to write
!   type(msu_L2A_orbit_data)		:: orbit_scan_data  ! structure to put data in
!   integer(4)                      :: error   ! = 0 is all is OK, non zero if error encountered.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    read_L2A_orbit_file(	satellite_name,orbit_number,msu_orbit_header,orbit_scan_data,error,path)
!
! @*****			
		use msflib

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(msu_L2A_orbit_header)							:: msu_orbit_header
		type(msu_L2A_orbit_data)							:: orbit_scan_data
		type(msu_L2A_scan)									:: msu_orbit_scan
		integer(4)											:: error
		character(len =100),optional						:: path

		character(len = 100)								:: path_to_use

		character(len=348),dimension(MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num

		character(len=100)				:: filename
		integer(4)						:: size
		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag
		integer(4)						:: fz_err

		integer(4)						:: sat_num
		integer(4)						:: scan_num
		integer(4)						:: date_err

		type  (FILE$INFO)				:: info

		if (present(path)) then
			path_to_use = path
		else 
			path_to_use = 	MSU_L2A_PATH
		endif


		do sat_num = 1,9
			if (sat_names_2(sat_num) .eq. satellite_name) exit
		enddo

		if (sat_num .gt. 9) stop

		error = 0

		! zero the msu_orbit_Scan_data

		call zero_msu_L2A_header(msu_orbit_header)
		call zero_L2A_orbit(orbit_scan_data)
		orbit_scan_data%scan_valid = 0  
		

		!determine the file name (hardwired to Mo1995 for now!)

		range_num = (orbit_number-1)/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999
		write(filename,9001)trim(path_to_use),satellite_name,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number
9001	format(a,a,'\'Mo1995'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'.L1A.gz')

	!	Determine if file exists

		Handle = file$first
		length = getfileinfoqq(filename,info,handle)

		if (info.length .gt. 3000000) then
				error = 8
		endif
	    if (info.length .le. 0) then
				error = 1				
		endif

		!print *,filename
		fz_err = 0
		if (error == 0) then							   
			call fastunzip(filename,size,buffer,fz_err)
			call decode_msu_L2A_header(buffer(1),msu_orbit_header)		
					
			do scan_num = 2,msu_orbit_header%number_of_swaths+1
			
				call decode_msu_L2A_scan(buffer(scan_num),msu_orbit_scan)

				! set the date structure
			 
				msu_orbit_scan%date = set_date_2000(msu_orbit_scan%date%rd_2000,date_err)

				! set the sat_name and orbit_number

				msu_orbit_scan%orbit_num = orbit_number
				msu_orbit_scan%sat_num = sat_num

				orbit_scan_data%scan_data(msu_orbit_scan%scan_number) = msu_orbit_scan

				orbit_scan_data%scan_valid(msu_orbit_scan%scan_number) = 1 ! 1 = from old data
				
			enddo
		endif

	end subroutine read_L2A_orbit_file

	subroutine encode_msu_L2A_Corr_header(L2A_Corr_Header,header)
! @***h* encode_msu_L2A_header
!  NAME
!    encode_msu_L2A_Corr_header
! 
!  PURPOSE
!    converts data in a MSU L2A Correction Data header to binary form
! 
!  DESCRIPTION
!    converts data from structured type to binary form (48 bytes).  The binary form is used to
!    store L2A data in the RSS MSU L2A Correction files.  The header record is the first record 
!    in each file.  Correction files contain diurnal and angle adjustments for each scan.
!    
!  INPUT
!    type(L2A_Corr_header)		:: L2A_Corr_header  ! Correction Header data from L2A orbit
!
!  OUTPUT
!    character(len = 48)		:: header ! same data converted to flat binaru form for writing to RSS L2A file 
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call encode_msu_L2A_Corr_header(L2A_Corr_Header,header)
!
! @*****
		use type_conversion

		type(msu_L2A_corr_header)		:: L2A_Corr_Header
		character(len=48)				:: header

		header(1:7)		= L2A_Corr_header%satellite_name 
		header(9:12)	= int_4_to_string(L2A_Corr_header%orbit_number)		
		header(13:16)	= int_4_to_string(L2A_Corr_header%number_of_swaths)
		header(17:20)	= int_4_to_string(L2A_Corr_header%start_swath_num)
		header(21:24)	= int_4_to_string(L2A_Corr_header%end_swath_num)
		header(25:32)	= real_8_to_string(L2A_Corr_header%start_rd_2000)
		header(33:40)	= real_8_to_string(L2A_Corr_header%end_rd_2000)

	end subroutine encode_msu_L2A_Corr_header

	subroutine decode_msu_L2A_Corr_header(	header,				&
											L2A_Corr_Header)
! @***h* decode_msu_L2A_header
!  NAME
!    decode_msu_L2A_Corr_header
! 
!  PURPOSE
!    converts correction header data in binary form to a structured variable.
!
!  DESCRIPTION
!    converts data from binary form (48 bytes) to structured data.  The binary form is used to
!    store L2A data in the RSS MSU L2A Correction files.  The header record is the first record 
!    in each file.  Correction files contain diurnal and angle adjustments for each scan.
!    
!  INPUT
!    type(L2A_Corr_header)		:: L2A_Corr_header  ! Correction Header data from L2A orbit
!
!  OUTPUT
!    character(len = 48)		:: header ! same data converted to flat binaru form for writing to RSS L2A file 
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call decode_msu_L2A_Corr_header(L2A_Corr_Header,header)
!
! @*****
		use type_conversion

		type(msu_L2A_corr_header)		:: L2A_Corr_Header
		character(len=48)				:: header

		L2A_Corr_header%satellite_name			= header(1:7)		
		L2A_Corr_header%orbit_number			= string_to_int_4(header(9:12))		
		L2A_Corr_header%number_of_swaths		= string_to_int_4(header(13:16))
		L2A_Corr_header%start_swath_num 		= string_to_int_4(header(17:20))
		L2A_Corr_header%end_swath_num			= string_to_int_4(header(21:24))
		L2A_Corr_header%start_rd_2000			= string_to_real_8(header(25:32))
		L2A_Corr_header%end_rd_2000			    = string_to_real_8(header(33:40))
 
	end subroutine decode_msu_L2A_Corr_header

	subroutine encode_msu_L2A_Corr_scan(	chan,				&
											scan_number,		&
											L2A_Corr_Data,		&
											scan_char)
! @***h* encode_msu_L2A_scan
!  NAME
!    encode_msu_L2A_Corr_scan
! 
!  PURPOSE
!    converts data in a MSU L2A Correction Data scan to binary form
! 
!  DESCRIPTION
!    converts data from structured type to binary form (48 bytes).  The binary form is used to
!    store L2A data in the RSS MSU L2A Correction files.  The scan record is the first record 
!    in each file.  Correction files contain diurnal and angle adjustments for each scan.
!    
!  INPUT
!		integer(4)				:: chan          ! MSU channel
!		integer(4)				:: scan_number   ! Scan number in orbit
!		real(8)					:: L2A_Corr_Data(NUM_CHANNELS,NUM_FOVS)
!
!  OUTPUT
!		character(len = 48)		:: scan_char     ! correction data in character form
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call encode_msu_L2A_Corr_scan(L2A_Corr_scan,scan)
!
! @*****

		use type_conversion

		integer(4)				:: chan
		integer(4)				:: scan_number
		character(len = 48)		:: scan_char
		real(8)					:: L2A_Corr_Data(NUM_CHANNELS,NUM_FOVS)

		integer(4)					:: fov
		integer(4)					:: offset


		scan_char(1:4)		= int_4_to_string(scan_number)
		do fov = 1,11
			offset = 1 + (fov*4)
			scan_char(offset:offset+3) = real_4_to_string(real(L2A_Corr_Data(chan,fov)))
		enddo
		 

	end subroutine encode_msu_L2A_Corr_scan

	subroutine decode_msu_L2A_Corr_scan(	scan_char ,			&
											chan,				&
											L2A_Corr_Data,		&
											scan_number)
! @***h* decode_msu_L2A_scan
!  NAME
!    decode_msu_L2A_Corr_scan
! 
!  PURPOSE
!    converts data in binary form to MSU L2A Correction Data 
! 
!  DESCRIPTION
!    converts data from binary form (48 bytes) to structured data.  The binary form is used to
!    store L2A data in the RSS MSU L2A Correction files.  The scan record is the first record 
!    in each file.  Correction files contain diurnal and angle adjustments for each scan.
!    
!  INPUT
!		integer(4)				:: chan          ! MSU channel
!		character(len = 48)		:: scan_char     ! correction data in character form
!  OUTPUT
!		integer(4)				:: scan_number   ! Scan number in orbit
!		real(8)					:: L2A_Corr_Data(NUM_CHANNELS,NUM_FOVS)
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call encode_msu_L2A_Corr_scan(L2A_Corr_scan,scan)
!
! @*****
		use type_conversion

		character(len = 48)		:: scan_char
		integer(4)				:: chan
		real(8)					:: L2A_Corr_Data(NUM_CHANNELS,NUM_FOVS)
		integer(4)				:: scan_number

		integer(4)					:: fov
		integer(4)					:: offset


		scan_number = string_to_int_4(scan_char(1:4))
		do fov = 1,11
			offset = 1 + (fov*4)
			L2A_Corr_Data(chan,fov) = string_to_real_4(scan_char(offset:offset+3)) 
		enddo
		 

	end subroutine decode_msu_L2A_Corr_scan


	subroutine write_L2A_Corr_file(	satellite_name,			&
									orbit_number,			&
									orbit_scan_data,		&
									corr_type,				&
									log_unit,				&
									error)
									
! @***h* write_L2A_Corr_file
!  NAME
!    write_L2A_Corr_file
! 
!  PURPOSE
!    writes correction data to RSS MSU L2A correction file
! 
!  DESCRIPTION
!    writes data to RSS MSU L2A correction file.  data is compressed using gzip.
!    
!  INPUT
!  character(len=7)					:: satellite_name   ! satellite name (TIROS-N, etc)
!  integer(4)						:: orbit_number     ! orbit number ot write
!  type(msu_L2A_orbit_header)		:: msu_corr_header ! header data to write
!  type(msu_L2A_orbit_data)			:: orbit_scan_data  ! data to write
!  integer(4)                       :: corr_type  ! 	code for correction type  (INCIDENCE=1,LIMB=2,DIURNAL=3)
!  integer(4)						:: log_unit  ! logical unit to write to
!
!  OUTPUT
!   integer(4)                      :: error   ! = 0 is all is OK, non zero if error encountered.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call write_L2A_Corr_file(satellite_name,orbit_number,orbit_scan_data,corr_type,log_unit,error)
!
! @*****		
		use msflib

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(msu_L2A_corr_header)							:: msu_corr_header
		type(msu_L2A_orbit_data)							:: orbit_scan_data
		integer(4)											:: corr_type
		integer(4)											:: log_unit
		integer(4)											:: error

		character(len=48),dimension(MAX_SCANS_PER_ORBIT)	:: buffer

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: num_scans_valid
		integer(4)						:: num_scans_new

		character(len=150)				:: filename
		integer(4)						:: chan
		integer(4)						:: scan_index
		integer(4)						:: first_valid_scan
		integer(4)						:: last_valid_scan
		character(len = 20)				:: begin_date_String
		character(len = 20)				:: end_date_String

		real(8),dimension(NUM_CHANNELS,NUM_FOVS)	:: Corr_data



		! loop over channels

		do chan = 1,NUM_CHANNELS

					

			! encode all the data into a large array of characters, keeping track of
			! important header info

			error = 0

			num_scans_valid = 0
			num_scans_new   = 0
			do scan_index = 1,MAX_SCANS_PER_ORBIT
				if (orbit_scan_data%scan_valid(scan_index) .gt. 0)	then
					num_scans_valid = num_scans_valid + 1
					if (num_scans_valid .eq. 1) first_valid_scan = scan_index
					last_valid_scan = scan_index 
					select case(corr_type)
						case(INCIDENCE)
							corr_data = orbit_scan_data%scan_data(scan_index)%MSU_TBs_nom_EIA_corr
						case(LIMB)
							corr_data = orbit_scan_data%scan_data(scan_index)%MSU_TBs_nadir_corr
						case(DIURNAL)
							corr_data = orbit_scan_data%scan_data(scan_index)%MSU_TBs_diurnal_corr
						case default
							error = -1
							stop
					end select

					call encode_msu_L2A_Corr_scan(	chan,				&
													scan_index,			&
													corr_data,			&
													buffer(num_scans_valid + 1))
				endif
			enddo
			
			! don't bother to write the file if no valid scans

			if (num_scans_valid .le. 0) then
				error = 1
				return
			endif

			msu_corr_header%satellite_name		= satellite_name
			msu_corr_header%orbit_number		= orbit_number
			msu_corr_header%number_of_swaths	= num_scans_valid
			msu_corr_header%start_swath_num	    = orbit_scan_data%scan_data(first_valid_scan)%scan_number
			msu_corr_header%end_swath_num		= orbit_scan_data%scan_data(last_valid_scan)%scan_number
			msu_corr_header%start_rd_2000		= orbit_scan_data%scan_data(first_valid_scan)%date%rd_2000
			msu_corr_header%end_rd_2000		    = orbit_scan_data%scan_data(last_valid_scan)%date%rd_2000

			! put header in position 1 in buffer

			call encode_msu_L2A_corr_header(msu_corr_header,buffer(1))

			!determine the file name

			range_num = orbit_number/5000
			orbit_Range_lower = (range_num*5000)+1
			orbit_range_upper = orbit_range_lower + 4999

			select case(corr_type)
				case(INCIDENCE)
					write(filename,9001)trim(MSU_L2A_PATH_TEST),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9001			format(a,a,'\Incidence_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.inc')
				case(LIMB)
					write(filename,9002)trim(MSU_L2A_PATH_TEST),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9002			format(a,a,'\Limb_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.lmb')
				case(DIURNAL)
					write(filename,9003)trim(MSU_L2A_PATH_TEST),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9003			format(a,a,'\Diurnal_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.dnl')
				case default
					error = -1
					stop
			end select 


			open(unit = 3,file = filename,form='binary')
			write(3) buffer(1:num_scans_valid + 1)
			close(3)

			if (log_unit .gt. 0) then
				begin_date_string = date_string(orbit_scan_data%scan_data(first_valid_scan)%date) 
				end_date_string   = date_string(orbit_scan_data%scan_data(last_valid_scan)%date) 
				write(log_unit,9004)orbit_number,							&
									num_scans_valid,						&
									num_scans_new,							&
									msu_corr_header%start_swath_num,		&
									msu_corr_header%end_swath_num,			&
									begin_date_string,						&
									end_date_string,filename
	9004		format(1x,i6,1x,i4,1x,i4,1x,i4,1x,i4,1x,a20,1x,a20,1x,a100)
			endif
		enddo

	end subroutine write_L2A_corr_file

	subroutine read_L2A_Corr_file(	satellite_name,			&
									orbit_number,			&
									orbit_scan_data,		&
									corr_type,				&
									error,					&
									path)
! @***h* read_L2A_Corr_file
!  NAME
!    read_L2A_Corr_file
! 
!  PURPOSE
!    reads correction data from RSS MSU L2A correction file
! 
!  DESCRIPTION
!    writes data to RSS MSU L2A correction file.  data is compressed using gzip.
!    
!  INPUT
!  character(len=7)					:: satellite_name   ! satellite name (TIROS-N, etc)
!  integer(4)						:: orbit_number     ! orbit number to read
!  type(msu_L2A_orbit_header)		:: msu_corr_header  ! header data to read
!  integer(4)                       :: corr_type        ! code for correction type  (INCIDENCE=1,LIMB=2,DIURNAL=3)
!  integer(4)						:: log_unit         ! logical unit to read from
!   character(len =100),optional	:: path             ! optional path to read from
!
!  OUTPUT
!   type(msu_L2A_orbit_data)		:: orbit_scan_data  ! data read in
!   integer(4)                      :: error   ! = 0 is all is OK, non zero if error encountered.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call read_L2A_Corr_file(satellite_name,orbit_number,orbit_scan_data,corr_type,log_unit,error)
!
! @*****
		use msflib

		character(len=7)									:: satellite_name
		integer(4)											:: orbit_number
		type(msu_L2A_corr_header)							:: msu_corr_header
		type(msu_L2A_orbit_data)							:: orbit_scan_data
		integer(4)											:: corr_type
		integer(4)											:: log_unit
		integer(4)											:: error
		character(len =100),optional						:: path

		character(len = 100)								:: path_to_use


		character(len=48)									:: buffer
		character(len=48)									:: header

		integer(4)						:: orbit_range_lower
		integer(4)						:: orbit_range_upper
		integer(4)						:: range_num
		integer(4)						:: num_scans_valid
		integer(4)						:: num_scans_new

		character(len=150)				:: filename
		integer(4)						:: chan
		integer(4)						:: scan_index
		integer(4)						:: scan_number
		integer(4)						:: first_valid_scan
		integer(4)						:: last_valid_scan
		character(len = 20)				:: begin_date_String
		character(len = 20)				:: end_date_String

		real(8),dimension(NUM_CHANNELS,NUM_FOVS)	:: Corr_data

		integer(4)						:: status

		integer(4)						:: handle				!file handle
		integer(4)						:: length				!length flag

		type  (FILE$INFO)				:: info


		if (present(path)) then
			path_to_use = path
		else 
			path_to_use = 	MSU_L2A_PATH
		endif


		! loop over channels

		corr_data = NAN

		do chan = 1,NUM_CHANNELS

				! set appropriate channel to NAN
	
			do scan_number = 1,MAX_SCANS_PER_ORBIT
				select case(corr_type)
					case(INCIDENCE)
						orbit_scan_data%scan_data(scan_number)%MSU_TBs_nom_EIA_corr(chan,:) = NAN 
					case(LIMB)
						orbit_scan_data%scan_data(scan_number)%MSU_TBs_nadir_corr(chan,:)	= NAN
					case(DIURNAL)
						orbit_scan_data%scan_data(scan_number)%MSU_TBs_diurnal_corr(chan,:) = NAN 
					case default
						error = -1
						stop
				end select
			enddo



		
			range_num = orbit_number/5000
			orbit_Range_lower = (range_num*5000)+1
			orbit_range_upper = orbit_range_lower + 4999



			select case(corr_type)
				case(INCIDENCE)
					write(filename,9001)trim(path_to_use),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9001			format(a,a,'\Incidence_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.inc')
				case(LIMB)
					write(filename,9002)trim(path_to_use),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9002			format(a,a,'\Limb_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.lmb')
				case(DIURNAL)
					write(filename,9003)trim(path_to_use),satellite_name,chan,orbit_range_lower,orbit_range_upper,satellite_name,orbit_number,chan
	9003			format(a,a,'\Diurnal_Correction\Ch_',i1,'\r',i5.5,'_',i5.5,'\',a,'_r',i5.5,'_Ch',i1,'.dnl')
				case default
					error = -1
					stop
			end select 

			Handle = file$first
			length = getfileinfoqq(filename,info,handle)

			if (info.length .le. 0) then
					error = 1
					cycle				
			endif

			open(unit = 3,file = filename,form='binary',iostat=status)
			if (status .eq. 0) then 
			    
				read(3) header

				call decode_msu_L2A_Corr_header(header,msu_corr_header)

				do scan_index = 2,msu_corr_header%number_of_swaths+1
					read(3) buffer
					call decode_msu_L2A_corr_scan(	buffer,			&
													chan,						&
													Corr_Data,					&
													scan_number)
					select case(corr_type)
						case(INCIDENCE)
							orbit_scan_data%scan_data(scan_number)%MSU_TBs_nom_EIA_corr(chan,:) = corr_data(chan,:) 
						case(LIMB)
							orbit_scan_data%scan_data(scan_number)%MSU_TBs_nadir_corr(chan,:)	= corr_data(chan,:) 
						case(DIURNAL)
							orbit_scan_data%scan_data(scan_number)%MSU_TBs_diurnal_corr(chan,:) = corr_data(chan,:) 
						case default
							error = -1
							stop
					end select
				enddo
				close(3)
			else
				do scan_index = 1,MAX_SCANS_PER_ORBIT

					select case(corr_type)
						case(INCIDENCE)
							orbit_scan_data%scan_data(scan_index)%MSU_TBs_nom_EIA_corr(chan,:) = NAN
						case(LIMB)
							orbit_scan_data%scan_data(scan_index)%MSU_TBs_nadir_corr(chan,:)	= NAN 
						case(DIURNAL)
							orbit_scan_data%scan_data(scan_index)%MSU_TBs_diurnal_corr(chan,:) = NAN 
						case default
							error = -1
							stop
					end select	
				enddo
			endif


		enddo
	end subroutine read_L2A_corr_file

    subroutine eval_swath_qual(swath_qual,fov_qual,fov_ok)
! @***h* eval_swath_qual
!  NAME
!    eval_swath_qual
! 
!  PURPOSE
!    evaluates swath and fov quality flags
! 
!  DESCRIPTION
!    evaluates swath and fov quality flags.  sets fov qulaity flags if swath qulity flags so indicate.  
!    sets overall logical flag FOV OK that controls further processing of that FOV.
!    
!  INPUT
!  	    integer(4)						:: swath_qual
!       integer(1),dimension(NUM_FOVS)	:: fov_qual
!  OUTPUT
!   	logical(4),dimension(NUM_FOVS)	:: fov_ok   ! true is further processing warrented...

!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/18/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call eval_swath_qual(satellite_name,orbit_number,orbit_scan_data,corr_type,log_unit,error)
!
! @*****
	    integer(4)						:: swath_qual
	    integer(1),dimension(NUM_FOVS)	:: fov_qual
	    logical(4),dimension(NUM_FOVS)	:: fov_ok

	    integer(4)						:: fov

	    fov_ok = .true.   !innocent until proven guilty

	    do fov = 1,NUM_FOVS
		    if (btest(fov_qual(fov),0) .or.  &
		        btest(fov_qual(fov),2) .or.  &
			    btest(swath_qual,fov+15)) fov_ok(fov) = .false.

	    enddo

	    if ( btest(swath_qual,1) .or. &
	         btest(swath_qual,3) .or. &
		     btest(swath_qual,12) .or. &
		     btest(swath_qual,15) .or. &
		     (btest(swath_qual,13) .and. btest(swath_qual,14))) fov_ok = .false.

	    return
    end subroutine eval_swath_qual

end module MSU_Level_2A