$freeform
! @***h* MLT_RSS/MSU_types.f90
!  NAME
!    Module MSU_types
! 
!  PURPOSE
!    Module containing structured types for MSU processing
! 
!  DESCRIPTION
!    Module containing structured types for MSU processing
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use MSU_Types
! @*****
module msu_types

	use NAN_support
	use date_and_time
	use msu_constants

	implicit none

	type msu_temps
		real(8)			:: R_PRT_A1						! resistance of PRT#A, Hot Load 1
		real(8)			:: R_PRT_A2						! resistance of PRT#A, Hot Load 2
		real(8)			:: R_PRT_B1						! resistance of PRT#B, Hot Load 1
		real(8)			:: R_PRT_B2						! resistance of PRT#B, Hot Load 2
		real(8)			:: Target1_temp					! temperature of Hot Load #1, used for Channel 1&2
		real(8)         :: Target2_temp					! temperature of Hot Load #2, used for Channel 3&4
		real(8)			:: Target1_Temp_A				
		real(8)			:: Target1_Temp_B
		real(8)			:: Target2_Temp_A
		real(8)			:: Target2_Temp_B
		real(8)			:: R_Other_1_Temp
		real(8)			:: R_Other_2_Temp
		real(8)			:: R_LO_1_Temp
		real(8)			:: R_LO_2_Temp
		real(8)			:: R_LO_3_Temp
		real(8)			:: R_LO_4_Temp
		real(8)			:: R_Dicke_1_Temp	
		real(8)			:: R_Dicke_2_Temp
		real(8)			:: R_Dicke_3_Temp
		real(8)			:: R_Dicke_4_Temp
		real(8)			:: R_Antenna_1_Temp
		real(8)			:: R_Antenna_2_Temp
		real(8)			:: R_Motor_1_Temp
		real(8)			:: R_Motor_2_Temp
		real(8)			:: R_Chassis_1_Temp		
		real(8)			:: R_Chassis_2_Temp

		real(8)			:: Other_1_Temp
		real(8)			:: Other_2_Temp
		real(8)			:: LO_1_Temp
		real(8)			:: LO_2_Temp
		real(8)			:: LO_3_Temp
		real(8)			:: LO_4_Temp
		real(8)			:: Dicke_1_Temp	
		real(8)			:: Dicke_2_Temp
		real(8)			:: Dicke_3_Temp
		real(8)			:: Dicke_4_Temp
		real(8)			:: Antenna_1_Temp
		real(8)			:: Antenna_2_Temp
		real(8)			:: Motor_1_Temp
		real(8)			:: Motor_2_Temp
		real(8)			:: Chassis_1_Temp		
		real(8)			:: Chassis_2_Temp
	end type msu_temps

	! calibration coefficients....

	type msu_calib_coeffs
		integer(4)						::  sat_num
		integer(4)						::  set_coeff_num
		real(4),dimension(NUM_CHANNELS)	::  A0
		real(4),dimension(NUM_CHANNELS)	::  A1
		real(8),dimension(NUM_CHANNELS)	::  B0
		real(8),dimension(NUM_CHANNELS)	::  B1
		real(8),dimension(NUM_CHANNELS)	::  B2
		real(8),dimension(NUM_CHANNELS)	::  B3
	end type msu_calib_coeffs


	type msu_L1B_orbit_header
		character(len=7)	:: satellite_name
		integer(4)			:: orbit_number
		integer(4)			:: number_of_swaths
		integer(4)			:: start_swath_num
		integer(4)			:: end_swath_num
		real(8)				:: start_rd_2000
		real(8)				:: end_rd_2000
		character(len=300)  :: pad
	end type msu_L1B_orbit_header


	! variables of type msu_raw contain the raw data in each msu scan		

	type msu_raw
		integer(2)		::	lyear					! year of observation
		integer(2)		::	jday					! julian day of observation
		real(4)			::	secdy					! # of seconds since midnight for observation
		real(4)         ::  height					! height, in kilometers wrt geoid
		real(4)			::	loc_zen_angle			! local zenith angle
		real(4)			::	FOV_latitude(NUM_FOVS)	! longitude of each measurement
		real(4)			::	FOV_longitude(NUM_FOVS)	! latitude of each measurement
		integer(2)		::	MSU_counts(NUM_CHANNELS+3,NUM_SCAN_POSITIONS)  ! Raw MSU counts, and other data
		integer(2)		::  set_coeff
		real(4)			::	earth_location_delta	! Some sort of Earth location error (could be clock error)
		integer(2)		::	scan_quality
		integer(2)		::	FOV_quality(14)
		integer(4)		::  chksum
		integer(4)		::  sat_num
		integer(4)		::  orbit_num
		integer(4)		::  scan_number		
		type(date_str)  ::  date
		type(msu_calib_coeffs) :: calib
	end type msu_raw

	type msu_L2A_scan
		integer(4)		:: sat_num
		integer(4)		:: orbit_num
		integer(4)		:: scan_number
		real(4)			:: fov_latitude(NUM_FOVS)		! fov location
		real(4)			:: fov_longitude(NUM_FOVS)		! FOV location
		real(4)			:: height
		integer(4)		:: scan_qual					! new quality flag, with additional derived values for each channel
		integer(1)		:: fov_qual(NUM_FOVS)			! new quality flag, with values for each Channel and FOV
		type(date_str)	:: date							! observation date converted to date structure
		type(msu_temps) :: T							! all of the various temperatures
	    real(8)			:: MSU_counts_corr(NUM_CHANNELS,NUM_SCAN_POSITIONS) ! MSU counts with the non-linear correction applied (note now are real)
		real(8)			:: MSU_TBs_raw(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temps for the 11 FOVs
		real(8)			:: MSU_TBs_nom_EIA_corr(NUM_CHANNELS,NUM_FOVS)	! Calculated Brightness Temp, corrected for orbital decay
 		real(8)			:: MSU_TBs_diurnal_corr(NUM_CHANNELS,NUM_FOVS)	! Calculated Brightness Temp, corrected for orbital decay and to the center latitude
		real(8)			:: MSU_TBs_nadir_corr(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temp, corrected for cross-swath diurnal effect
		integer(4)		:: asc_desc									! ascending/descending flag
		real(8)			:: roll_err									! estimated roll from fit
		real(4)			:: fov_incidence(NUM_CHANNELS,NUM_FOVS)
	end type msu_L2A_scan

	type MSU_L2A_orbit_header
		character(len=7)	:: satellite_name
		integer(4)			:: orbit_number
		integer(4)			:: number_of_swaths
		integer(4)			:: start_swath_num
		integer(4)			:: end_swath_num
		real(8)				:: start_rd_2000
		real(8)				:: end_rd_2000
		character(len=308)  :: pad
	end type msu_L2A_orbit_header

	type MSU_L2A_Corr_header
		character(len=7)	:: satellite_name
		integer(4)			:: orbit_number
		integer(4)			:: number_of_swaths
		integer(4)			:: start_swath_num
		integer(4)			:: end_swath_num
		real(8)				:: start_rd_2000
		real(8)				:: end_rd_2000
		character(len=8)  :: pad
	end type MSU_L2A_Corr_header



	type msu_L1B_raw_orbit_data
		type(msu_raw),dimension(MAX_SCANS_PER_ORBIT)		:: scan_data
		integer(4),dimension(MAX_SCANS_PER_ORBIT)			:: scan_valid
	end type msu_L1B_raw_orbit_data 

	type msu_L2A_orbit_data
		type(msu_L2A_scan),dimension(MAX_SCANS_PER_ORBIT)		:: scan_data
		integer(4),dimension(MAX_SCANS_PER_ORBIT)				:: scan_valid
	end type msu_L2A_orbit_data 


	! variables of type msu_derived contain values of derived data for each msu scan

	type msu_derived
		integer(4)		:: sat_num
		integer(4)		:: scan_qual(NUM_CHANNELS)		! new quality flag, with additional derived values for each channel
		integer(4)		:: fov_qual(NUM_CHANNELS,NUM_FOVS) ! new quality flag, with values for each Channel and FOV
		type(date_str)	:: date							! observation date converted to date structure
		type(msu_temps) :: T							! all of the various temperatures
	    real(8)			:: MSU_counts_corr(NUM_CHANNELS,NUM_SCAN_POSITIONS) ! MSU counts with the non-linear correction applied (note now are real)
		real(8)			:: MSU_TBs_raw(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temps for the 11 FOVs
		real(8)			:: MSU_TBs_orb_dec(NUM_CHANNELS,NUM_FOVS)	! Calculated Brightness Temp, corrected for orbital decay
 		real(8)			:: MSU_TBs_orb_dec_lat(NUM_CHANNELS,NUM_FOVS)	! Calculated Brightness Temp, corrected for orbital decay and to the center latitude
		real(8)			:: MSU_TBs_diur(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temp, corrected for cross-swath diurnal effect
		real(8)			:: MSU_TBs_sat_temp(NUM_CHANNELS,NUM_FOVS)	! Calculated Brightness Temp, corrected for drift in space craft temperature
		real(8)			:: Model_TBs_NCEP(NUM_CHANNELS,NUM_FOVS)    ! Modeled brightness temperatures
		real(8)			:: MSU_TBs_fov(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temp, corrected to NADIR
		real(8)			:: MSU_TBs_EIA(NUM_CHANNELS,NUM_FOVS)		! Calculated Brightness Temp, corrected to nominal Earth Incidence Angles
		real(8)			:: FOV_incidence(NUM_FOVS)
		integer(4)		:: land_sea(NUM_FOVS)						! land sea flag
		integer(4)		:: asc_desc									! ascending/descending flag
		real(8)			:: roll_err
	end type msu_derived

	! description of msu_derived%scan_qual(channel number)
	! and            msu_derived%fov_qual(Channel,FOV)
	! 
	! scan_qual is an internal quality flag for each scan for each channel number.  It's
	! a 4-byte integer so there's lots of room for future expansion.  Basically, if it is
	! not zero, then something is wrong.  The bits are assigned as follows.  The Checks are
	! performed in the order indicated, so if a scan fails QC at some level n, no further
	! processing is done.  This makes sense, since in many cases, failure at a lower level will
	! certainly lead to a failure at a higher level.  For example, a bad target temp will certainly
	! lead to out of bounds Tbs.
	!
	!	0	Summary of raw%scan_quality flag.  Set to 1 for all channels if msu_raw%scan_quality is non-zero,
	!		or if date fields are somehow messed up and can't be interpreted
	!	1	Geolocation Error.  Set to 1 for all channels if geolocation error is found
	!   2   Height Error.	Set to 1 if height is below MSU_CONSTANTS::MIN_HEIGHT or above MSU_CONSTANTS::MAX_HEIGHT
	!   3   Target Temperature Error.  Set to one for appropriate channels if Target temperature is below 
	!			MSU_CONSTANTS::LOWER_TARGET_TEMP_LIMIT, or above MSU_CONSTANTS::UPPER_TARGET_TEMP_LIMIT or if the
	!			Temperature difference between the two target thermometers is greater than 
	!			MSU_CONSTANTS::TARGET_TEMP_DIFF_LIMIT
	!	4	Counts at limit.  Set to one if the counts are pegged at either limit for any scan position (1-14)
	!			for a given channel
	!	5	Tb out of range. Set to one for a given channel if any of the Tbs in the scan are out of range.  The
	!			range is given by MIN_REASONABLE_TB(channel#) to MAX_REASONABLE_TB(channel#)	
	!	6	Possible Hydrometeors.  Set to 1 for a channel if the difference between a given scan position and the mean of the
	!			two adjacent scan positions	is greater than MAX_REASONABLE_NEAR_NEIGHBOR_DIFF for of scan positions
	!			3 through 9.  The outer scan positions were deemed by Spencer and Christy to be too variable.  S+C use
	!		    MAX_REASONABLE_NEAR_NEIGHBOR_DIFF = 1.5
	!	7	Height calculated using interpolated Two Line Element



	type msu_scan
		type(msu_raw)		:: raw
		type(msu_derived)	:: der
	end type msu_scan
end module msu_types