$freeform
module zonal

!@***h* MLT_RSS/zonal.f90
!  NAME
!    module zonal
! 
!  PURPOSE
!    psuedoclass that defines and manipulates zonal means of MSU/AMSU data
! 
!  DESCRIPTION
!    psuedoclass that defines and manipulates zonal means of MSU/AMSU data
!    
!  Components
!    the pseudoclass defines a zonal average type
!
!		type zonal_ave_rec
!		type (date_str)														:: begin_date
!		type (date_str)														:: end_date
!		type(ave),dimension(NUM_ZONES,NUM_FOVS,NUM_NODES,NUM_SURFACES+1)	:: dat
!	end type zonal_ave_rec
!  
! explanation of the grid type flag:
!
!
!  ROUTINES
!    zero_zonal_ave_rec(z)
!        fills zonal average record z with initial values.
!    put_in_zone(x,lon,lat,date,zone,fov,node,surf,z)  
!        adds a single observation to a zonal average record z
!        INPUTS
!           lon         longitude of observation
!           lat         latitude of observation
!           date        date of observation, in data_str form
!           fov         field of view
!           node        ascending or descending
!           z           zonal average record to add to
!        OUTPUTS
!           zone        zonal index of choosen zone
!           surf        surface type (land or ocean) of location
!           z           updated zonal average record
!
!    put_scan_in_zone(scan,ok,z,chan,corr_level)
!        adds a single MSU scan to a zonal average record
!        INPUTS
!           scan        MSU scan structure
!           ok          array of fov quality flags
!           z           zonal average record to add to
!           chan        msu_channel
!           corr_level  msu correction level
!        OUTPUTS
!           z           updated zonal average record

!    put_scan_in_zone_xswath(scan,ok,z,chan,corr_level)
!        adds a single cross-swath difference to a zonal average.  deprecated.
!        adds a single MSU scan to a zonal average record
!        INPUTS
!           scan        MSU scan structure
!           ok          array of fov quality flags
!           z           zonal average record to add to
!           chan        msu_channel
!           corr_level  msu correction level
!        OUTPUTS
!           z           updated zonal average record
!
!    update_zonal_ave_rec(z)
!        updates a zonal average record  -- i.e computed averages from totals so they are consistent
!        INPUTS
!           z           zonal average record to add to
!        OUTPUTS
!           z           updated zonal average record

!    write_zonal_ave_rec(z,lu,full,error)
!        writes a zonal average record in standard binary form.
!        INPUTS
!           z           zonal average record to write
!           lu          logical unit to write to
!           full        set to true to write "full" set of data
!        OUTPUTS
!           error       error flag -- if non zero, write was not successful

!    write_zonal_ave_rec_direct(pentad,offset,z,lu,error)
!        writes a zonal average record in standard binary form to fortran direct access file.
!        INPUTS
!           pentad      pentad number to write data to.
!           offset      beginning pentad number in file
!           z           zonal average record to write
!           lu          logical unit to write to
!        OUTPUTS
!           error       error flag -- if non zero, write was not successful
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use zonal
! 
!@*****


	use msu_constants
	use msu_types
	use landmask
	use NAN_support
	use msflib
	use date_and_time  
	use average


	implicit none

	! type to contain arrays of zonal-mean informations

	integer,parameter		:: NUM_ZONES   = 36
	real(8),parameter		:: ZONE_SPACING = 180.0d0/NUM_ZONES
	integer,parameter		:: ZONAL_REC_LENGTH = 8 + 8 + 8 + (4*NUM_ZONES*NUM_FOVS*NUM_NODES*(NUM_SURFACES+1)*8)

	type zonal_ave_rec
		type (date_str)														:: begin_date
		type (date_str)														:: end_date
		type(ave),dimension(NUM_ZONES,NUM_FOVS,NUM_NODES,NUM_SURFACES+1)	:: dat
	end type zonal_ave_rec

	type zonal_output_structure 
		integer(4)						:: pentad_num
		real(8)							:: begin_rd
		real(8)                         :: end_rd
		real(8),dimension(4,NUM_ZONES,NUM_FOVS,NUM_NODES,NUM_SURFACES+1) :: dat
	end type 



contains

	subroutine zero_zonal_ave_rec(z)
		
		use msu_constants
		use NAN_support
		use msu_types

		implicit none

		type(zonal_ave_rec),intent(INOUT)		::	z

		integer(4)		:: surf
		integer(4)		:: node
		integer(4)		:: fov
		integer(4)		:: zone

		do surf = 1,NUM_SURFACES+1
			do node = 1,NUM_NODES
				do fov = 1,NUM_FOVS
					do zone = 1,NUM_ZONES
						call zero_ave(z%dat(zone,fov,node,surf))
					enddo
				enddo
			enddo
		enddo
	end subroutine zero_zonal_ave_rec


	subroutine put_in_zone(x,lon,lat,date,zone,fov,node,surf,z)

		use msu_constants
		use NAN_support
		use msu_types

		implicit none

		real(8),intent(IN)						:: x			! The number to be added to average
		real(8),intent(IN)						:: lon          ! The longitude of the observation
		real(8),intent(IN)						:: lat	        ! The latitude of the observation
		type(date_str),intent(IN)				:: date		    ! Time and Date of the observation
		integer(4),intent(OUT)					:: zone	        ! zonal index
		integer(4),intent(IN)					:: fov	        ! field of view index
		integer(4),intent(IN)					:: node		    ! nodal index (asc = 1, desc = 2)
		integer(4),intent(OUT)					:: surf		    ! surface type index (1 = sea, 2 = land 3 = both)
		type(zonal_ave_rec),intent(INOUT)		:: z	        ! Structure containing the zonal averages
										   
		real(8)									:: loc_time

		if (finite(x)) then
			zone = floor((lat +90.0D0)/ZONE_SPACING)+1
			surf = find_land_sea(real(lat),real(lon))
			if ((zone .lt. 1) .or. (zone .gt. 36)) then
			 print *,zone,fov,node,surf
			endif
			z%dat(zone,fov,node,surf)%total =   &
			z%dat(zone,fov,node,surf)%total + x

			z%dat(zone,fov,node,surf)%tot_sqr=  &
			z%dat(zone,fov,node,surf)%tot_sqr + x**2

			z%dat(zone,fov,node,surf)%num=  &
			z%dat(zone,fov,node,surf)%num + 1

			loc_time = date%r_d + (lon/360.0D0)
			loc_time = loc_time - floor(loc_time)

			z%dat(zone,fov,node,surf)%tot_sin_loc_time =  &
			z%dat(zone,fov,node,surf)%tot_sin_loc_time + &
				  sin(loc_time * TWO_PI)
			z%dat(zone,fov,node,surf)%tot_cos_loc_time =  &
			z%dat(zone,fov,node,surf)%tot_cos_loc_time + &
				  cos(loc_time * TWO_PI)
		endif
	
	end subroutine put_in_zone

	subroutine put_scan_in_zone(scan,ok,z,chan,corr_level)

		implicit none

		type(msu_scan),intent(IN)					:: scan
		logical,dimension(NUM_FOVS,NUM_CHANNELS)	:: ok
		type(zonal_ave_rec)							:: z
		integer(4)									:: chan
		integer(4)									:: corr_level


		real(8),dimension(NUM_CHANNELS,NUM_FOVS)				:: Tbs			! array of chosen brightness temperatures
		integer(4)												:: fov			! Scan index
		integer(4)												:: surf		! Latitude bin index
		integer(4)												:: zone		! Longitude bin index
		integer(4)												:: error
		! select the data depending on chosen correction level

		call select_Tbs(scan,corr_level,Tbs,error)

		if (error .eq. 0) then

			do fov = 1,NUM_FOVS

					! check to make sure observation exists and is finite

					if ((finite(Tbs(chan,fov))) .and. (ok(fov,chan))) then ! 

						call put_in_zone(Tbs(chan,fov),		&
							 dble(scan%raw%FOV_longitude(fov)),	&
							 dble(scan%raw%FOV_latitude(fov)),	&
							 scan%der%date,					&
							 zone,							&
							 fov,							&
							 scan%der%asc_desc,				&
							 surf,							&
							 z)
					endif
			  enddo

			endif

		return

	end subroutine put_scan_in_zone

	subroutine put_scan_in_zone_xswath(scan,ok,z,chan,corr_level)

		implicit none

		type(msu_scan),intent(IN)					:: scan
		logical,dimension(NUM_FOVS,NUM_CHANNELS)	:: ok
		type(zonal_ave_rec)							:: z
		integer(4)									:: chan
		integer(4)									:: corr_level


		real(8),dimension(NUM_CHANNELS,NUM_FOVS)				:: Tbs			! array of chosen brightness temperatures
		integer(4)												:: fov			! Scan index
		integer(4)												:: surf		! Latitude bin index
		integer(4)												:: zone		! Longitude bin index
		integer(4)												:: error
		! select the data depending on chosen correction level

		call select_Tbs(scan,corr_level,Tbs,error)

		if (error .eq. 0) then

			do fov = 1,NUM_FOVS

					! check to make sure observation exists and is finite

					if ((finite(Tbs(chan,fov))) .and. (finite(Tbs(chan,12-fov))) .and. (ok(fov,chan))) then ! 

						call put_in_zone(Tbs(chan,fov)-Tbs(chan,12-fov),		&
							 dble(scan%raw%FOV_longitude(6)),	&
							 dble(scan%raw%FOV_latitude(6)),	&
							 scan%der%date,					&
							 zone,							&
							 fov,							&
							 scan%der%asc_desc,				&
							 surf,							&
							 z)
					endif
			  enddo

			endif

		return

	end subroutine put_scan_in_zone_xswath



	subroutine update_zonal_ave_rec(z)
		
		use msu_constants
		use NAN_support
		use average
		use msu_types

		implicit none

		type(zonal_ave_rec),intent(INOUT)		::	z

		integer(4)		:: surf
		integer(4)		:: node
		integer(4)		:: fov
		integer(4)		:: zone

		do surf = 1,NUM_SURFACES
			do node = 1,NUM_NODES
				do fov = 1,NUM_FOVS
					do zone = 1,NUM_ZONES
						call update_ave(z%dat(zone,fov,node,surf))
					enddo
				enddo
			enddo
		enddo

		do node = 1,NUM_NODES
			do fov = 1,NUM_FOVS
				do zone = 1,NUM_ZONES
					z%dat(zone,fov,node,BOTH_SURFS) =  &
						combine_ave(z%dat(zone,fov,node,SEA), &
									z%dat(zone,fov,node,LAND))
				enddo
			enddo
		enddo

	end subroutine update_zonal_ave_rec

	subroutine write_zonal_ave_rec(z,lu,full,error)

		use msu_constants
		use NAN_support
		use average
		use msu_types

		implicit none

		type(zonal_ave_rec),intent(IN)		    :: z
		integer(4),intent(IN)					:: lu		! logical unit to be written
		logical(4),intent(IN)					:: full	    ! if true, write full record, otherwise, write num,mean,stddev,loc_time only			
		integer(4),intent(OUT)					:: error

		integer(4)		:: surf
		integer(4)		:: node
		integer(4)		:: fov
		integer(4)		:: zone

		write(lu)z%begin_date%r_d
		write(lu)z%end_date%r_d
		do surf = 1,NUM_SURFACES+1
			do node = 1,NUM_NODES
				do fov = 1,NUM_FOVS
					do zone = 1,NUM_ZONES
						call write_ave(z%dat(zone,fov,node,surf),lu,full,error)
					enddo
				enddo
			enddo
		enddo

		return

	end subroutine write_zonal_ave_rec

	subroutine write_zonal_ave_rec_direct(pentad,offset,z,lu,error)

		use msu_constants
		use NAN_support
		use average
		use msu_types

		implicit none

		integer(4)								:: pentad
		integer(4)								:: offset
		type(zonal_ave_rec),intent(IN)		    :: z
		integer(4),intent(IN)					:: lu		! logical unit to be written
		integer(4),intent(OUT)					:: error

		integer(4)		:: surf
		integer(4)		:: node
		integer(4)		:: fov
		integer(4)		:: zone

		type (zonal_output_structure)			:: output_str 
		
		output_str%pentad_num = pentad
		output_str%begin_rd = z%begin_date%r_d
		output_str%end_rd = z%end_date%r_d


		do surf = 1,NUM_SURFACES+1
			do node = 1,NUM_NODES
				do fov = 1,NUM_FOVS
					do zone = 1,NUM_ZONES
						output_str%dat(1,zone,fov,node,surf) = dfloat(z%dat(zone,fov,node,surf)%num)
						output_str%dat(2,zone,fov,node,surf) = z%dat(zone,fov,node,surf)%mean
						output_str%dat(3,zone,fov,node,surf) = z%dat(zone,fov,node,surf)%std_dev
						output_str%dat(4,zone,fov,node,surf) = z%dat(zone,fov,node,surf)%local_time
					enddo
				enddo
			enddo
		enddo
		print *,'writing record = ',pentad-offset,' Unit = ',lu,' Pentad = ',pentad
		write(unit = lu,rec=pentad-offset) output_str 
		error = 0
		return

	end subroutine write_zonal_ave_rec_direct


end module zonal

