


function init_L2a_orbit
;@***h* MLT_RSS/orbitify_aqua_data.pro/init_L2a_orbit

; NAME
;   init_L2a_orbit
;
; PURPOSE
;   Initializes L2A orbit structure
;
; DESCRIPTION
;   Initializes L2A orbit structure
;
; INPUTS:
;   none
;
; RETURN VALUES:
;   IDL structure containing L2A orbit data, set to initial values
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   L2a_orbit = init_L2a_orbit()
;
;
;@*****
compile_opt idl2

		L2A_header = {satellite_name:'       ', 		$
		              fill0:' ',						$
		              orbit_number:0L, 					$
		              number_of_swaths:0L,				$
		              start_swath_num:0L,				$
		              end_swath_num:0L,					$
		              start_rd_2000:0.0D0,				$
   		              end_rd_2000:0.0D0,				$
   		              eqx_rd_2000:0.0D0,				$
   		              channel:0L,						$
   		              fill:bytarr(48),					$
   		              packed_scan_valid:bytarr(395), 	$
   		              residual_cross_track_Corrections:fltarr(30), $
   		              L2A_processor_ID:bytArr(20),    $
   		              L1B_processor_ID:bytarr(20),      $
   		              fill2:bytarr(129)}

			scan_str_templ = {satellite_name:'       ',	$
						channel_num:0B,					$
						orbit_number:0L,				$
						scan_line_in_orbit:0L,			$
						scan_rd_2000:0.0D0,				$
						fov_latitude:fltarr(30),		$
						fov_longitude:fltarr(30),		$
						fov_quality:bytarr(30),			$
						warm_load_temp:0.0, 			$
						asc_desc:0s,						$	;(1 = asc, 2 = desc, 0 = undetermined)
						Tbs:fltarr(30),					$
						Diurnal_Corr:fltarr(30),		$
						Nadir_Corr:fltarr(30),			$
						Incidence_Corr:fltarr(30),		$
						spacecraft_altitude:0.0}

			data = replicate(scan_str_templ,790)

	L2a_orbit = {header:L2a_header,    $
				 data:data}
	return,l2a_orbit

end

function rd_2000_from_tai1993,tai
;@***h* MLT_RSS/orbitify_aqua_data.pro/rd_2000_from_tai1993
;
; NAME
;   rd_2000_from_tai1993
;
; PURPOSE
;   converts date in tai1993 form to rd_2000 form
;
; DESCRIPTION
;   converts date in tai1993 form to rd_2000 form
;
; INPUTS:
;   tai   ; date in tai form
;
; RETURN VALUES:
;   date in rd_2000 form
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   rd_2000 = rd_2000_from_tai1993(tai)
;
;
;@*****
  compile_opt idl2
	jan_1_1993 = julday(1,1,1993,0,0,0.0)
	jan_1_2000 = julday(1,1,2000,0,0,0.0)

	rd_2000 = (tai/(86400.0D0))+jan_1_1993-jan_1_2000

	return,rd_2000
end





pro orbitify_aqua_data,year,month,day,channel,mode,num_missing_files,num_read_errors,last_rd_2000 = last_rd_2000,last_orbit=last_orbit,debug = debug
;@***h* MLT_RSS/orbitify_aqua_data.pro/orbitify_aqua_data

; NAME
;   orbitify_aqua_data
;
; PURPOSE
;   This routine obitifies 1 day of aqua data.
;
; DESCRIPTION
;   This routine obitifies 1 day of aqua data.
;   the input data is what NASA called L1b, which is brightness temperature data
;   in granules.  The output data is what I call L2A, which is brightness temperature
;   data in orbits, plus various corrections....
;
; INPUTS:
;   year      ; year to process
;   month     ; month to process
;   day       ; day in month to process
;   channel   ; AMSU channel to process
;   mode      ; mode to run in -- if set to APPEND, data is appended to previous orbit
;   debug=debug ; if set, more verbose output is written to screen
;
; OUTPUTS:
;   num_missing_files  ; number of missing files in data
;   num_read_errors    ; number of read erros encountered
;   last_rd_2000 = last_rd_2000   ; date of last scan processed, in rd_2000 format
;   last_orbit=last_orbit         ; orbit number of last orbit processed
;   
; FILES READ
;   NASA L1B files
;   e.g. AIRS.2003.01.01.001.L1B.AMSU_Rad.v5.0.0.0.G07073101304.hdf
; 
; FILES WRITTEN
;   RSS L2A files, with no adjustments applied
;   e.g. AQUA_r52742.L2A.unadj.gz
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   orbitify_aqua_data,2007,3,4,7,'APPEND',num_missing_files,num_read_errors,last_rd_2000 = last_rd_2000,last_orbit=last_orbit
;
;
;@*****
  compile_opt idl2
	; this routine obitifies 1 day of aqua data.
	; the input data is what NASA called L1b, which is brightness temperature data
	; in granules
	;
	; the output data is what I call L2A, which is brightness temperature
	; data in orbits, plus various corrections....
	;
	; the first step is to write the part that puts the NASA data into
	; orbits, then I'll worry about the corrections.  For consistency,
	; the corrections will be done in fortran, I think.

	; at all times, there is one orbit structure in use.  When we need to
	; start using a new orbit, the older orbit is written out and zeroed.

	; current_orbit is the orbit that is currently being processed
	; active_orbit is the orbit whose structure is loaded.

	common aqua1,scan_last,active_orbit
	common aqua2,time_scan1


	satellite_name = 'AQUA'
	sat_num = 15
	first_L2A_orbit_loaded = 0

	get_AMSU_File_Locations,AMSU_L2A_Path = AMSU_L2A_Path

	; read in the current inventory file

	read_L2A_inventory,sat_num,channel,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000


	close,10
	scan_Error_file = string(year,month,day,format = '("\\msu-ops\c\idlworkspace\orbitify_aqua_v5\scan_num_error_logs\scan_num_errors_",i4,"_",i2.2,"_",i2.2,".txt")')
	openw,10,scan_error_file

	num_read_errors = 0
	num_missing_files = 0
	scans_processed = 0
	eqx = 0.0
	pushd,'M:\msu_amsu_data\amsu-a_L1A\'
	for granule_num = 1,240 do begin
		file = aqua_amsu_file(year,month,day,granule_num,error)
		if error eq 0 then begin
			log,'Reading.... ',file
			read_AIRS_AQUA_L1B, file, x, read_error
			if read_error eq 0 then begin
				; figure out which orbits we need
				start_orbit = x.l1b_amsu.attr.start_orbit[0]
				end_orbit   = x.l1b_amsu.attr.end_orbit[0]

				if first_L2A_orbit_loaded eq 0 then begin
					; if in APPEND mode, read in previous version of the orbit

					active_orbit = start_orbit
					current_orbit = start_orbit

					orbit = init_L2a_orbit()
					L2A_scan_valid = intarr(790)

					if mode eq 'APPEND' then begin
						; first try the unadjusted orbit, then the adjusted one
						read_AMSU_L2A_orbit_unadjusted,satellite_name,	$
							 current_orbit,					$
							 channel,						$
							 L2A_orbit_header,				$
							 L2A_orbit,						$
							 L2A_scan_valid,				$
							 AMSU_L2A_Path = AMSU_L2A_Path, $
							 error = error
						if error eq 0 then begin
							orbit.header = L2a_orbit_header
							orbit.data   = L2A_orbit
						endif else begin
							read_AMSU_L2A_orbit,satellite_name,	$
								 current_orbit,					$
								 channel,						$
								 L2A_orbit_header,				$
								 L2A_orbit,						$
								 L2A_scan_valid,				$
								 AMSU_L2A_Path = AMSU_L2A_Path, $
								 error = error2
							if error2 eq 0 then begin
								orbit.header = L2a_orbit_header
								orbit.data   = L2A_orbit
							endif
						endelse
					endif ; mode eq 'APPEND'
					first_L2A_orbit_loaded = 1
				endif

				switched_orbits = 0

				for line = 0,44 do begin

					;determine ascending/descending

					if line gt 0 then begin
						if x.l1b_amsu.field.sat_lat[line] gt x.l1b_amsu.field.sat_lat[line-1] then node = 'ascending' else node = 'descending'
					endif else begin
						if x.l1b_amsu.field.sat_lat[line+1] gt x.l1b_amsu.field.sat_lat[line] then node = 'ascending' else node = 'descending'
					endelse

					asc_desc = 0
					if node eq 'ascending' then asc_desc =1
					if node eq 'descending' then asc_desc = 2

					scan_num_float = -999.99

					if finite(time_scan1[current_orbit]) eq 0 then begin   ;try to guess the time offset for this orbit
						scan_num_float = 371.05 + (x.l1b_amsu.field.nadirtai[line] - x.l1b_amsu.attr.eq_x_tai[0])/8.000
						scan_num_guess = floor(scan_num_float)
						time_scan1[current_orbit] = x.l1b_amsu.field.nadirtai[line] - 8.0*(scan_num_guess - 1.0)
					endif

					scan_num = 1+round((x.l1b_amsu.field.nadirtai[line] - time_scan1[current_orbit])/8.0)

					; when the granule goes across the equator, ascending node, the EQX time is not right for
					; scans above the equator... so perform this adjustment

					if ((node eq 'ascending') and (x.l1b_amsu.field.sat_lat[line] gt 0.0) and (scan_num gt 400)) then begin
						if switched_orbits eq 0 then begin
							scan_num_float = 371.05 + (x.l1b_amsu.field.nadirtai[line] - (x.l1b_amsu.attr.eq_x_tai[0]+5934.0))/8.000
							scan_num_guess = floor(scan_num_float)
							if (x.l1b_amsu.field.sat_lat[line] lt 0.4835) then scan_num_guess = 1
							current_orbit = current_orbit + 1
							time_scan1[current_orbit] = x.l1b_amsu.field.nadirtai[line] - 8.0*(scan_num_guess - 1.0)
							switched_orbits = 1
							scan_num = 1+round((x.l1b_amsu.field.nadirtai[line] - time_scan1[current_orbit])/8.0)
							log,'Crossed equator, switched orbit num to ',current_orbit,' Lat = ',x.l1b_amsu.field.sat_lat[line],' Scan = ',scan_num,' Time of Scan 1 = ',time_scan1[current_orbit]
						endif
					endif

                    if ((node eq 'ascending') and (x.l1b_amsu.field.sat_lat[line] gt 0.0) and (x.l1b_amsu.field.sat_lat[line] lt 0.4835) and (scan_num ne 1)) then begin
                    	printf,10,x.l1b_amsu.field.sat_lat[line],' ',scan_num,' ',scan_num_float
                    	print,x.l1b_amsu.field.sat_lat[line],' ',scan_num,' ',scan_num_float
                        log,'Scan Num Error ',x.l1b_amsu.field.sat_lat[line],' ',scan_num,' ',scan_num_float

                    endif
                    if ((node eq 'ascending') and (x.l1b_amsu.field.sat_lat[line] lt 0.0) and (scan_num lt 200)) then begin
                    	printf,10,x.l1b_amsu.field.sat_lat[line],' ',scan_num,' ',scan_num_float
                    	print,x.l1b_amsu.field.sat_lat[line],' ',scan_num,' ',scan_num_float
                        log,'Scan Num Error ',x.l1b_amsu.field.sat_lat[line],' ',scan_num_float

                    endif

					;if (scans_processed gt 1) then begin
						if ((scan_num - scan_last) ne 1) then begin
							log,'Scan Num Jump: Current Scan = ',scan_num,' Last Scan = ',scan_last
						endif
					;endif

					; if we have changed orbits for any reason, write out the active orbit
					; and zero the orbit structure

					if active_orbit ne current_orbit then begin  ;

						; write out the loaded orbit

						if keyword_set(L2a_scan_valid) then begin
							ok = where(L2a_scan_valid gt 0,ct)
						endif else begin
							ct = 0
						endelse

						if ((ct gt 0) and (active_orbit gt 0)) then begin

							 ; update the information in the header

							 orbit.header.satellite_name 	= strpad(satellite_name,7)
							 orbit.header.orbit_number 		= active_orbit
							 orbit.header.number_of_swaths  = ct
							 orbit.header.start_swath_num   = min(ok)+1
							 orbit.header.end_swath_num     = max(ok)+1
							 orbit.header.start_rd_2000     = orbit.data[ok[0]].scan_rd_2000
							 orbit.header.end_rd_2000       = orbit.data[ok[ct-1]].scan_rd_2000
							 orbit.header.eqx_rd_2000       = rd_2000_from_tai1993(x.l1b_amsu.attr.eq_x_tai[0])  ; need to offset by 1/2 orbit to agree with NOAA data
							 orbit.header.channel			= channel
							 ; write out the orbit file

							write_AMSU_L2A_orbit_unadjusted,				$
													 'AQUA',				$
													 active_orbit,			$
													 channel,				$
													 orbit.header,			$
													 orbit.data,			$
													 L2A_scan_valid,		$
													 AMSU_L2A_Path = AMSU_L2A_Path, $
													 error = error

							if (finite(orbit.header.end_rd_2000) gt 0) then begin
								last_rd_2000 = orbit.header.end_rd_2000
							endif



							exists[active_orbit] = 1
							start_scan[active_orbit] 	= orbit.header.start_swath_num
							end_scan[active_orbit] 		= orbit.header.end_swath_num
							start_rd_2000[active_orbit] = orbit.header.start_rd_2000
							end_rd_2000[active_orbit] 	= orbit.header.end_rd_2000
							eqx_rd_2000[active_orbit] 	= orbit.header.eqx_rd_2000
							last_orbit = active_orbit

						endif  ; ((ct gt 0) and active_orbit gt 0))

						;zero the orbit structure

						orbit = init_L2a_orbit()
						L2A_scan_valid = intarr(790)

						; if in APPEND mode, read in previous version of the orbit

						if mode eq 'APPEND' then begin
							read_AMSU_L2A_orbit,satellite_name,	$
								 current_orbit,					$
								 channel,						$
								 L2A_orbit_header,				$
								 L2A_orbit,						$
								 L2A_scan_valid,				$
								 AMSU_L2A_Path = AMSU_L2A_Path, $
								 error = error
							if error eq 0 then begin
								orbit.header = L2a_orbit_header
								orbit.data   = L2A_orbit
							endif
						endif ; mode eq 'APPEND'

						active_orbit = current_orbit

						last_scan_last_orbit = scan_last

					endif ; active_loaded ne current_orbit

					; quality control steps

					fov_quality = replicate(0B,30)


					scan_last = scan_num
					if keyword_set(debug) then log,scan_num,x.l1b_amsu.field.sat_lat[line],371.35 + (x.l1b_amsu.field.nadirtai[line] - x.l1b_amsu.attr.eq_x_tai[0])/8.000


					; select appropriate target temperature

					case channel of
						1:  target_temp = x.l1b_amsu.field.a2_target_temp[line]
						2:  target_temp = x.l1b_amsu.field.a2_target_temp[line]
						3:  target_temp = x.l1b_amsu.field.a12_target_temp[line]
						4:  target_temp = x.l1b_amsu.field.a12_target_temp[line]
						5:  target_temp = x.l1b_amsu.field.a12_target_temp[line]
						6:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						7:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						8:  target_temp = x.l1b_amsu.field.a12_target_temp[line]
						9:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						10:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						11:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						12:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						13:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						14:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
						15:  target_temp = x.l1b_amsu.field.a11_target_temp[line]
					else : begin
						log,'Error, invalid channel: ',channel
						return
						end
					endcase

					; move data into the orbit structure for this scan

					if ((scan_num ge 1) and (scan_num le 789)) then begin
						orbit.data[scan_num-1].satellite_name 			= strpad(satellite_name,7)
						orbit.data[scan_num-1].channel_num    			= channel
						orbit.data[scan_num-1].orbit_number    			= current_orbit
						orbit.data[scan_num-1].scan_line_in_orbit 		= scan_num
						orbit.data[scan_num-1].scan_rd_2000 			= rd_2000_from_tai1993(x.l1b_amsu.field.nadirtai[line])
						orbit.data[scan_num-1].fov_latitude             = x.l1b_amsu.geo_field.latitude[*,line]
						orbit.data[scan_num-1].fov_longitude            = (720.0+x.l1b_amsu.geo_field.longitude[*,line]) mod 360
						orbit.data[scan_num-1].fov_quality				= fov_quality
						orbit.data[scan_num-1].warm_load_temp			= target_temp + 273.15  ; convert to K
						orbit.data[scan_num-1].asc_desc					= asc_desc
						orbit.data[scan_num-1].tbs						= x.l1b_amsu.field.brightness_temp[channel-1,*,line]
						orbit.data[scan_num-1].diurnal_corr				= replicate(!values.f_nan,30)
						orbit.data[scan_num-1].nadir_corr				= replicate(!values.f_nan,30)
						orbit.data[scan_num-1].incidence_corr			= replicate(!values.f_nan,30)
						orbit.data[scan_num-1].spacecraft_altitude		= x.l1b_amsu.field.satheight[line]
	                    l2a_scan_valid[scan_num-1] = 1
	                    ;print,scan_num-1,target_temp + 273.15
	                 endif

					scans_processed = scans_processed + 1
				endfor  ;line
			endif else begin
				log,'Error reading '+file
				num_read_errors = num_read_errors + 1
			endelse
		endif else begin
			;log, string(day,month,year,granule_num, format = '("File for ",i2.2,"/",i2.2,"/",i4.4," Gr: ",i4," does not exist")')
			num_missing_files = num_missing_files + 1
		endelse
	endfor

	; write out the current orbit

	if keyword_set(L2a_scan_valid) then begin
		ok = where(L2a_scan_valid gt 0,ct)
	endif else begin
		ct = 0
	endelse

	if ((ct gt 0) and (active_orbit gt 0)) then begin

		 ; update the information in the header

		 orbit.header.satellite_name 	= strpad(satellite_name,7)
		 orbit.header.orbit_number 		= active_orbit
		 orbit.header.number_of_swaths  = ct
		 orbit.header.start_swath_num   = min(ok)+1
		 orbit.header.end_swath_num     = max(ok)+1
		 orbit.header.start_rd_2000     = orbit.data[ok[0]].scan_rd_2000
		 orbit.header.end_rd_2000       = orbit.data[ok[ct-1]].scan_rd_2000
		 orbit.header.eqx_rd_2000       = rd_2000_from_tai1993(x.l1b_amsu.attr.eq_x_tai[0])  ; need to offset by 1/2 orbit to agree with NOAA data
		 orbit.header.channel			= channel
		 ; write out the orbit file


;		 write_AMSU_L2A_orbit,	 'AQUA',				$
;								 active_orbit,			$
;								 channel,				$
;								 orbit.header,			$
;								 orbit.data,			$
;								 L2A_scan_valid,		$
;								 AMSU_L2A_Path = AMSU_L2A_Path, $
;								 error = error

		 write_AMSU_L2A_orbit_unadjusted,				$
		 						'AQUA',					$
								 active_orbit,			$
								 channel,				$
								 orbit.header,			$
								 orbit.data,			$
								 L2A_scan_valid,		$
								 AMSU_L2A_Path = AMSU_L2A_Path, $
								 error = error


		; update the inventory data

		exists[active_orbit] = 1
		start_scan[active_orbit] 	= orbit.header.start_swath_num
		end_scan[active_orbit] 		= orbit.header.end_swath_num
		start_rd_2000[active_orbit] = orbit.header.start_rd_2000
		end_rd_2000[active_orbit] 	= orbit.header.end_rd_2000
		eqx_rd_2000[active_orbit] 	= orbit.header.eqx_rd_2000
		last_orbit = active_orbit
	endif  ; ((ct gt 0) and active_orbit gt 0))
	close,10
	if n_elements(orbit) gt 0 then begin
		if (finite(orbit.header.end_rd_2000) gt 0) then last_rd_2000 = orbit.header.end_rd_2000
	endif

	write_L2A_inventory,sat_num,channel,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000
  popd

end