pro calc_offsets_from_grpt_maps_AMSU,tb_arr_adj,months_to_use,offsets

;@***h* MLT_RSS/calc_offsets_from_grpt_maps_AMSU.pro
; NAME
;   calc_offsets_from_grpt_maps_AMSU.pro
;
; PURPOSE
;   calculates AMSU zonal intersatellite offsets directly from the adjusted monthly averaged grid point maps. 
;
; DESCRIPTION
;   Calculates the offsets using the method described in 
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
;   
;   and 
;   
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the RSS V3.2 lower tropospheric dataset from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1493-1509.
;
;   The routine calculates the offsets directly from the monthly averaged grid point maps. 
; INPUTS
;   tb_arr_adj          (144,72,num_months,num_sats) real(4) array of adjusted(for target temperature changes) brightness temperatures
;   months_to_use   (num_sats,num_months) array of integers -- 1 means use this month, 0 mean ignore)
; OUTPUTS
;   offsets         (num_lats,num_sats) real(4) array of offsets

;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
; 
; USAGE
;   calc_offsets_from_grpt_maps_AMSU,tb_arr_adj,months_to_use,offsets
;
;@*****
;
; compute the offsets separately for each latitude
	sigma_est = 0.3
	sz = size(tb_arr_adj,/dimensions)

	num_lons     = sz[0]
	num_lats     = sz[1]
	num_months   = sz[2]
	num_sats     = sz[3]

	bad = where(months_to_use eq 0)

	offsets = replicate(!values.f_nan,num_lats,num_sats)
	for ilat = 3,68 do begin
		tb_arr_lat = replicate(!values.f_nan,num_months,num_sats)
		for month = 0,num_months-1 do begin
			for satnum = 0,num_sats-1 do begin
				tb_arr_lat[month,satnum] = mean(tb_arr_adj[*,ilat,month,satnum],/nan)
			endfor
		endfor
		tb_arr_lat[bad] = !values.f_nan
		num_col = num_sats
		X = fltarr(num_col,10000L)
		Y = fltarr(10000L)
		row_num = 0

		for month = 0,num_months-1 do begin
			tbs = reform(tb_arr_lat[month,*])
			ok = where(finite(tbs),ct)
			
			if ct ge 2 then begin
				for j = 0,ct-2 do begin
				  row = fltarr(num_col)
					sat1 = ok[j]
					sat2 = ok[j+1]
					row[sat1] = 1.0
					row[sat2] = -1.0
					X[*,row_num] = row
					Y[row_num] = tbs[sat1]-tbs[sat2]
					row_num = row_num+1
				endfor
			endif
		endfor

		if row_num gt 10 then begin
			X = X[*,0:row_num-1]
			Y = Y[0:row_num-1]
			sigma_in = replicate(sigma_est,row_num)
			lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct
			offsets[ilat,*] = float(a)
		endif else begin
			offsets[ilat,*] = replicate(!values.f_nan,num_sats)
		endelse
	endfor
end