pro calc_TF_from_grpt_maps_AMSU_v3_3,tb_arr,tar_arr,months_to_use,channel,target_factor,target_means

;@***h* MLT_RSS/calc_TF_from_grpt_maps_AMSU_v3_3.pro
; NAME
;   calc_TF_from_grpt_maps_AMSU_v3_3.pro
;
; PURPOSE
;   calculates AMSU target factors directly from the monthly averaged grid point maps. 
;
; DESCRIPTION
;   Calculates the target factors using the method described in 
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
;   
;   and 
;   
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the RSS V3.2 lower tropospheric dataset from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1493-1509.
;
;   The routine calculates the target factors directly from the monthly averaged grid point maps.  The in put data is
; INPUTS
;   tb_arr          (144,72,num_months,num_sats) real(4) array of brightness temperatures
;   tar_arr         (144,72,num_months,num_sats) real(4) array of target temperatures
;   months_to_use   (num_sats,num_months) array of integers -- 1 means use this month, 0 mean ignore)
;   channel          3 character channel string (TLT,TMT, etc)
; OUTPUTS
;   target_factor   (num_sats) real(4) array of target factors
;   target_means    (num_sats) real(4) array of global mean traget temperatures

;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
; 
; USAGE
;   calc_TF_from_grpt_maps_AMSU_v3_3,tb_arr,tar_arr,months_to_use,channel,target_factor,target_means
;
;@*****


  ; set estimated error on input brightness temperatures
	sigma_est = 0.3
	
	; set latitude range to use for "global" averages used to calculate target factors
	lat_range=[-50,50]
	
	; find number of months and number of satellite from size of tb_Arr
	sz = size(tb_arr,/dimensions)
	num_months = sz[2]
	num_sats   = sz[3]
	
	;calculate temperate global averages for the tb_arr,tar_arr, and the num_arr
	
	mean_tb_arr = replicate(!values.f_nan,num_months,num_sats)
	mean_tar_arr = replicate(!values.f_nan,num_months,num_sats)

	lat_wts = latitude_wts(72,lat_range)

	surf_str = 'Land_and_Sea'
	
	;remove high altitude data (grid point average altitude above 3000 m) for TLT.
	max_alt = 10000
	if channel eq 'TLT' then max_alt = 3000

	for satnum = 0,num_sats-1 do begin
		for month = 0,num_months-1 do begin	
		  if months_to_use[month,satnum] gt 0 then begin
  			map = tb_arr[*,*,month,satnum]
  			ok = where(finite(map) eq 1,ct)		
  			mean_tb_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)
  
  			map = tar_arr[*,*,month,satnum]
  			mean_tar_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)
      endif
		endfor
	endfor

  ;remove means from target factors, and save means for passing back up

	target_means = fltarr(num_sats)
	for satnum = 0,num_sats-1 do begin
		tar = mean_tar_arr[*,satnum]
		target_means[satnum] = mean(tar,/nan)
		mean_tar_arr[*,satnum] = mean_tar_arr[*,satnum]-target_means[satnum]
	endfor

  ; if more than one satellite is used, then we can calculate the target factors.
  ; if not, we set the target factor to 0
 
  ;construct the regression array using months when 2 or more satellites have valid data
  ; Y = AX
  ; Y is the measured intersatellite Tb differences
  ; X is the array of satellite numbers and target temperature anomalies
  ; A is the array of global offsets (discarded) and target factors
  ; (latitude dependent offsets are calculated in a different routine (calc_offsets_from_grpt_maps_AMSU)
  
  ; define arrays way too big
  num_col = num_sats*2
  X = fltarr(num_col,10000L)
  Y = fltarr(10000L)
  s1 = intarr(10000L)
  s2 = intarr(10000L)
  m  = intarr(10000L)
  
  ;initialize row number
  row_num = 0
  zero_wt = 1.0
  
  ;loop over months looking for months with data from 2 or more satellites
    
  for month = 0,num_months-1 do begin
		tbs = reform(mean_tb_arr[month,*])
		ok = where(finite(tbs),ct)
		if ct ge 2 then begin
			for j = 0,ct-2 do begin
			  ; construct a row of X
			  row = fltarr(num_col)
				sat1 = ok[j]
				sat2 = ok[j+1]
				row[sat1] = 1.0
				row[sat2] = -1.0
				row[sat1+num_sats] = mean_tar_arr[month,sat1]
				row[sat2+num_sats] = -mean_tar_arr[month,sat2]
				X[*,row_num] = row
				Y[row_num] = tbs[sat1]-tbs[sat2]
				row_num = row_num+1
			endfor
		endif
	endfor
  	
  if row_num gt 10 then begin
    log,string(row_num-1,format='("Info: calculating Target Factors from ",i4," equations")')
  	X = X[*,0:row_num-1]
  	Y = Y[0:row_num-1]
  	sigma_in = replicate(sigma_est,row_num)
  	
    ;calculate A using singular value decomposition
  	lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error
  
  	; the offsets calculated here offset = float(a[0:num_sats-1]) are discarded.
  	
  	;save the target factors 
  	target_factor = float(a[num_sats:num_col-1])
  	
  	; set target factors and target means for unused satellite to NAN
    bad = where(total(months_to_use,1) eq 0,ct)
    if ct gt 0 then begin
      target_factor[bad] = !values.f_nan
      target_means[bad]  = !values.f_nan
    endif
  endif else begin
    ; this is the case of insufficient overlap, or only 1 satellite
    log,string(row_num-1,format='("Info: not calculating Target Factors, only ",i4," equations")')
    log,'Info: Target Factors set to zero'
    target_factor = replicate(!values.f_nan,num_sats)
    ok = where(total(months_to_use,1) gt 0,ct)
    if ct gt 0 then target_factor[ok] = 0.0
  endelse
end
