
pro grpt_merge_amsu_only_3_3

;@***h* MLT_RSS/grpt_merge_AMSU_only_3_3.pro

; NAME
;   grpt_merge_AMSU_only_3_3.pro
;
; PURPOSE
;   Top-level program that performs adjustments to AMSU monthyl gridded data, and then 
;   merges the data together into a single AMSU derived brightness temperature dataset
;
; DESCRIPTION
;   Reads gridded monthly averages from individual files.  We call these L2C files.
;   Calculates and applies target factors using singluar value decomposition
;   Calculates and applies latitude dependendent offsets using singlular value decompostion
;   Scene temeprature factors are set to zero for AMSU
;   Merges data from the chosen set of satellite together using equal weighting
;   Writes the results to a netcdf file.  We call these files L3.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   grpt_merge_amsu_only_3_3


;@*****

  ; set some basic constants
  defsysv,'!AMSU_year_range',[1998,2011]
  defsysv,'!num_AMSUs',7
  defsysv,'!num_AMSU_months', 12*(1+!AMSU_year_Range[1]-!AMSU_year_range[0])
  defsysv,'!num_years',!num_AMSU_months/12
  defsysv,'!Num_GrPt_Lats',72
  defsysv,'!Num_GrPt_Lons',144
  defsysv,'!Num_Nodes',2
  defsysv,'!Log_lun',9
  defsysv,'!Num_Hist_Datatypes',7
  
  ; set output paths for merged data and evaluation plots
  defsysv,'!AMSU_L2C_Path','M:\MSU_AMSU_Data\AMSU-A_L2C\'
  defsysv,'!AMSU_L3_Path', 'M:\MSU_AMSU_Data\AMSU-A_L3\'  
  defsysv,'!AMSU_L3_Plot_Path','C:\IDLWorkspace\Merge_Monthly_AMSU_Data_v3_3_OP\plots\'
    
  ; set channels to merge
  channels = ['TLT','TMT','TTS','TLS']
  process  = [ 1,    1,    1,    1]  ; 1 means process this channel
  
  ; set some constant that depend on the channel being processed.
  
  for channel_index = 0,n_elements(channels) -1 do begin
  	if process[channel_index] gt 0 then begin
      channel = channels[channel_index]
  		case channel of
  			'TLT': begin
  				channel_str = '05'
  				view_str = 'TLT'
          no_nadir=1
          num_thres = 300
  				end
  			'TMT': begin
  				channel_str = '05'
  				view_str = 'N12'
  				num_thres = 300
  				no_nadir = 0
  				end
  			'TTS': begin
  				merge_parameters = fltarr(2,12,4,2)
  				channel_str = '07'
  				view_str = 'N12'
  				no_nadir = 0
  				num_thres = 300
  				end
  			'TLS': begin
  				merge_parameters = fltarr(2,12,4,2)
  				channel_str = '09'
  				view_str = 'L8'
  				no_nadir = 0
  				num_thres = 220
  			end
  		endcase
  
      ; internal subversion num is 11 for standard processing (diurnal model = CCM3 and use N15,N18,METOP-A,AQUA)
      ; mapping for sats_to_use is [N15,N16,N17,N18,METOP-A,AQUA,N19]
      
      version_num = 1 & diurnal_model = 'CCM3'
      sats_to_use = [1,0,0,1,1,1,0] & sats_to_check = [1,0,0,1,1,1,0] & version_num = version_num+10
      if channel eq 'TTS' then begin
        sats_to_use = sats_to_use and [1,0,0,1,0,1,0]  ; channel 7 is bad on Metop-A -- do not use
        sats_to_check = sats_to_check and [1,0,0,1,0,1,0]
      endif
       
      ; open a log file
      defsysv,'!log_lun',2
      log_file = string(channel,version_num,format='("C:\IDLWorkspace\Merge_Monthly_AMSU_Data_v3_3_op\logs\merge_log_amsu_channel_",a,"_v3.3.",i3.3,".log.txt")')
      close,!log_lun
      ok = check_path(log_file,/create)
      openw,!log_lun,log_file
      
      ; write some information to the log file
      log,'Info: AMSU Only V3.3 log file'
      log,'Info:','Channel = ',channel_str
      log,'Info:','Diurnal model =',diurnal_model
      log,'Info:','EIA_model = ','NCEP'
      log,'Info:','Version Num = 3.3.'+string(version_num,format = '(i3.3)')
      log,'Info:','Number of observations threshold = ',string(num_thres,format='(i5)')
      if no_nadir eq 0 then begin
        log,'Info: Adjusting Tbs to refer to nadir'
      endif else begin
        log,'Info: No Nadir Adjustment'
      endelse
      
      sats_used_str = ''
      for satnum = 0,!num_AMSUs - 1 do begin
        if sats_to_use[satnum] gt 0 then sats_used_str = sats_used_str + sat_name(satnum + 10)+' '
      endfor
      log,'Info:','Satellites Used: ',sats_used_Str
  	  
  	  
  	  ; read in L2C data.
  		read_all_monthly_maps_amsu_only_v3_3, $
  			                  channel, 	$              
    											view_str,	$
    											sats_to_use, $
  												tb_arr,		$
  												tar_arr,	$
  												num_arr,	$
  												file_list_all_sats=file_list_all_sats, $
  												diurnal_model = diurnal_model, $
  												date_processed=date_processed,  $
  												no_nadir = no_nadir
  												
      ; find the number of months from the size of the returned array
      
      sz = size(tb_arr,/dimensions)
      num_months = sz[2]
      if num_months ne !num_AMSU_months then begin
        log,'Warning: num_months (',string(num_months),') not equal to !num_AMSU_months (',string(!num_AMSU_months),')'
      endif
      
      ; initialize adjustment parameter arrays
      offset_arr =  replicate(!values.f_nan,!Num_GrPt_Lats,!num_AMSUs)
      tarmult_arr = replicate(!values.f_nan,!Num_GrPt_Lats,!num_AMSUs)
      tbmult_arr =  replicate(!values.f_nan,!Num_GrPt_Lats,!num_AMSUs)
        

  		ok = where(sats_to_use gt 0,num_sats_to_use)
  		
  		; remove some bad months from processing for TLT, even though they pass data QC
  		; 
      ; if months_to_use_mask[month,sat_num] = -1, it means do not use no matter what
      ; if months_to_use_mask[month,sat_num] = +1, it means use even if it fails data number QC
      ; if months_to_use_mask[month,sat_num] = 0   it means use data number QC to determine of that month, sat_num is used
      
  		months_to_use_mask = intarr(!num_AMSU_months,!num_AMSUs)
  		if channel eq 'TLT' then begin
  		  months_to_use_mask[111:122,3] = -1
  		  months_to_use_mask[94,3] = -1
  		endif
  		
  		; perform data number QC to find good months
  	  check_grpt_maps_AMSU_v3_3,num_arr,sats_to_use,num_thres,months_to_use_mask,months_to_use
    
      ; check to make sure last month has good data
      last_month = current_month_number()-241
      
      stop_processing = 0
      for satnum = 0,!num_AMSUs -1 do begin
        if ((sats_to_check[satnum] ge 1) and (months_to_use[last_month,satnum] eq 0)) then begin
          message_text = 'Last Monthly Average for Satellite '+sat_name(satnum+10)+' failed QC, Yes to Continue, No to Abort'
          Result = DIALOG_MESSAGE(message_text,/question)
          log,'Warning: '+message_text
          if result eq 'No' then begin
            stop_processing = 1
            break
          endif
        endif
      endfor
      
      if stop_processing eq 1 then begin
        log,'Warning: skipping channel '+channel
        continue
      endif  ; move to next channel
      
      ; calculate target multipliers for each satellite.
      
    	calc_TF_from_grpt_maps_amsu_v3_3,tb_arr,					$
    								tar_arr,				    $
    								months_to_use,		  $
    								channel,				    $
    								target_factors,			$
    								target_means,			  $
    								offsets
   
      log,string(target_factors,format = '("Info: Target Factors = [",f9.4,6(",",f9.4),"]")')
      ; expand target multipliers to an array with a target multiplier for each latitude.
  		for ilat = 0,!Num_GrPt_Lats-1 do begin
  		  tarmult_arr[ilat,*] = target_factors
  		endfor
  		
  		; set up merged evaulation postscript file
  	  ps_file =string(!AMSU_L3_Plot_Path,channel,channel,version_num,format='(a,a,"\",a,"_amsu_grpt_merge_3_3_report.",i3.3,".ps")')
  	  log,'Info: Postscript file = '+ps_file
  	  set_up_ps,file = ps_file,xsize=6.5,ysize=7.0
  	  
  	  ; apply target factors to input data to produce tb_arr_adj
      apply_TF_to_grpt_maps_AMSU,tb_arr,tar_arr,channel,target_factors,target_means,months_to_use,tb_arr_adj
      
      
      if total(sats_to_use) gt 1 then begin  ; need to perform the merge
        ; calculate the offsets as a funtion of latitude from tb_arr_adj
        calc_offsets_from_grpt_maps_AMSU,tb_arr_adj,months_to_use,offsets
        
        ; smooth the offset in the N-S direction
        smooth_offsets,offsets,offsets_smooth
        offsets = offsets_smooth
        ;apply the smoothed offsets to tb_arr_adj
        apply_offsets_to_grpt_maps_AMSU,tb_arr_adj,offsets
      endif else begin
        offsets = fltarr(!Num_GrPt_Lats,!num_AMSUs)
      endelse
      
      ; tb_factors (scene temperature factors) are set to zero for AMSU
      tb_factors = fltarr(!num_AMSUs)
      bad = where(finite(target_factors) eq 0,ct)
      if ct gt 0 then begin
        tb_factors[bad] = !values.f_nan
        offsets[*,bad] = !values.f_nan
      endif
    
      offset_arr = offsets
      ; expand tb factors to one for each latitude (all are zero for AMSU)
      for ilat = 0,!Num_GrPt_Lats-1 do begin
        tbmult_arr[ilat,*] = tb_factors
      endfor
        
      ; merge data from different satellites together
  		tb_arr_merged = replicate(!values.f_nan,144,72,num_months)
  		for i = 0,num_months-1 do begin
  			tot_tb = fltarr(144,72)
  			tot_num = fltarr(144,72)
  			for satnum = 0,!num_AMSUs-1 do begin
  				if months_to_use[i,satnum] gt 0 then begin
  					tb = tb_arr_adj[*,*,i,satnum]
  					num = num_arr[*,*,i,satnum]
  					ok = where((finite(tb) gt 0) and (num gt 100),ct)
  					if ct gt 100 then begin
  						tot_tb[ok] = tot_tb[ok] + tb[ok]
  						tot_num[ok] = tot_num[ok] + 1
  					endif
  				endif
  			endfor ; satnum
  			tb_merged = tot_tb/tot_num
  			tb_arr_merged[*,*,i] = tb_merged
  		endfor
  
      ok = where(months_to_use gt 0)
      source_files = file_list_all_sats[ok]
  
      ; write the ourput netcdf file
  		if view_str eq 'TLT' then channel_str = 'TLT'
  		write_amsu_only_merged_Tbs_144_72_netcdf_3_3,		$
  							Tbs_144_72 = tb_arr_merged ,				  $
  							channel_str=channel_str,  					  $
  		          version_num=version_num, 					    $
  							source_files = source_files,					$
  							offsets = offset_arr,						      $
  							target_factors = tarmult_arr,				  $
  							tb_Factors = tbmult_arr,					    $
  							target_means = target_means,          $
  							months_used = months_to_use,					$
  							date_L2C_processed=date_processed,    $
  							output_path = !AMSU_L3_Path + 'AMSU_ONLY\'
  							
      num_files_used = n_elements(source_files)
      for i = 0,num_files_used - 1 do begin
        log,'Info: Source File ',source_files[i]
      endfor
      log,'Info: Normal End for channel ',channel       
   			
   		; set large arrays to zero to free up some space.
        tb_arr           = 0
        tb_arr_adj       = 0
        tb_arr_merged    = 0
        num_arr          = 0
        tar_arr          = 0
  	endif
  	; close the log file
  	close,!log_lun
  endfor
end