pro read_monthly_histograms_V3_0,lu,month_num,monthly_hist,error
  
;@***h* MLT_RSS/read_monthly_histograms_V3_0.pro
;
; NAME
;   read_monthly_histograms_V3_0.pro
;
; PURPOSE
;   reads a file (already open) containing a single monthly gridded average for a 
;   single satellite and channel
;
; DESCRIPTION
;   Reads a flat binary file with the monthly averaged data.  This file is
;   real(4) little endian.
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_monthly_histograms_V3_0,lu,month_num,monthly_hist,error
;   
;     lu          is the (already open) logical unit that contains the monthly average data
;     month_num   is the month number corresponding to the month requested.  0 is Jan 1978
;     monthly_hist[!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats] is a float(4) array
;     error       is set to 0 if the read was successful, -1 otherwise
;   
;
;@*****

  month_num = 0L
  monthly_hist = fltarr(!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats)

  on_ioerror,problem
  readu,lu,month_num
  readu,lu,monthly_hist
  error = 0
  return

  problem:
  ;print,'Problem reading unit lu'
  error = -1
  return

end

pro read_amsu_monthly_single_month,channel,satnum,year,month,view_str,diurnal_str,monthly_hist,error,file=file

;@***h* MLT_RSS/read_amsu_monthly_single_month.pro
;
; NAME
;   read_amsu_monthly_single_month.pro
;
; PURPOSE
;   finds and reads a file containing a single monthly gridded average for a 
;   single satellite and channel
;
; DESCRIPTION
;   Reads a flat binary file with the monthly averaged data.  This file is
;   real(4) little endian.
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !AMSU_L2C_Path
; INPUTS
;     channel          is the AMSU channel to be read (5,7,9)
;     satnum           is the satellite number of the AMSU satellite to be read (10 = NOAA-15, 11 = NOAA-16, 12 = NOAA-17, 13 = NOAA-18, 14 = METOP-A, 15 = AQUA, 16 = NOAA-19)
;     year             is the year of the month to be read
;     month            is the month to be read
;     view_str         is a string that describes the FOV set used for the montly average (N12 = near-nadir 12 views, L8 = near limb 8 views, TLT = TLT weighted views)  
;     diurnal_str      is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 cliamte model)
;
; OUTPUTS    
;     monthly_hist[!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats] is a float(4) array
;     error            is set to 0 if the read was successful, -1 otherwise
;     file             is the name of the file that was read
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_amsu_monthly_single_month,channel,satnum,year,month,view_str,diurnal_str,monthly_hist,error,file=file
;   
;
;@*****

single_month_file_size = 580612L
name = sat_name(satnum,/fill)
file = string(!AMSU_L2C_Path,channel,name,name,channel,year,month,view_str,diurnal_str, $
format='(a,"Monthly_Maps\AMSU_Channel_",i2.2,"\",a,"\",a,"_Chan_",i2.2,"_",i4.4,"_",i2.2,"_",a,"_V3_02.",a,".dat")')

info = file_info(file)
if info.exists eq 1 then begin
  if info.size ge single_month_file_size then begin
    close,3
    openr,3,file
    read_monthly_histograms_V3_0,3,month_num,monthly_hist,error
    close,3
  endif else begin
    error = -1 ; file is to small
  endelse
endif else begin
  error = -1  ; file does not exist
endelse

end

pro read_amsu_monthly_histograms_one_sat_ind_files_v3_3,channel,satnum,view_str,diurnal_str,monthly_histograms_one_sat,error,file_list=file_list,date_processed=date_processed

;@***h* MLT_RSS/read_amsu_monthly_histograms_one_sat_ind_files_v3_3.pro
;
; NAME
;   read_amsu_monthly_histograms_one_sat_ind_files_v3_3.pro
;
; PURPOSE
;   finds and reads all files containing monthly gridded averages for a 
;   single satellite and channel
;
; DESCRIPTION
;   Reads a number of flat binary file with the monthly averaged data.  Each file is
;   real(4) little endian.
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !AMSU_L2C_Path
;     !Num_AMSU_months
;     !AMSU_year_range
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
; 
; INPUTS
;     channel          is the AMSU channel to be read (5,7,9)
;     satnum           is the satellite number of the AMSU satellite to be read (10 = NOAA-15, 11 = NOAA-16, 12 = NOAA-17, 13 = NOAA-18, 14 = METOP-A, 15 = AQUA, 16 = NOAA-19)
;     view_str         is a string that describes the FOV set used for the montly average (N12 = near-nadir 12 views, L8 = near limb 8 views, TLT = TLT weighted views)  
;     diurnal_str      is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 cliamte model)
;
; OUTPUTS
;     monthly_histgrams_one_sat[!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats] is a float(4) array
;     error            is set to 0 if the read was successful, -1 otherwise
;     file_list        is the a list of file names that were read
;     date_processed   is an array of dates (in IDL julday form) that each monthly file was processed

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_amsu_monthly_histograms_one_sat_ind_files_v3_3,channel,satnum,view_str,diurnal_str,monthly_histograms_one_sat,error,file_list=file_list,date_processed=date_processed
;   
;
;@*****

  monthly_histograms_one_sat = fltarr(!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months)
  date_processed = replicate(!values.d_nan,!num_AMSU_months)
  file_list = replicate(' ',!num_AMSU_months)
  num_found = 0
  get_AMSU_File_Locations,AMSU_L2C_Path = AMSU_L2C_Path

  for year = !AMSU_year_range[0],!AMSU_year_range[1] do begin
    for month = 1,12 do begin
      month_index = 12*(year-!AMSU_year_range[0])+month-1
      read_amsu_monthly_single_month,channel,satnum,year,month,view_str,diurnal_str,monthly_hist,error2,file=file
      if error2 eq 0 then begin
        file_list[month_index] = file
        num_found = num_found+1
        monthly_histograms_one_sat[*,*,*,*,month_index] = monthly_hist
        if ((round(monthly_hist(6,0,0,0)) ge 2008) and (monthly_hist(6,0,0,0) le 2020)) then begin
            year_processed = round(monthly_hist(6,0,0,0))
            oday_processed = round(monthly_hist(6,0,1,0))
            hour_processed = round(monthly_hist(6,0,2,0))
            minute_processed = round(monthly_hist(6,0,3,0))
            date_processed[month_index] = JULDAY(1, oday_processed, Year_processed, Hour_processed, Minute_processed, 0.0)
        endif else begin
          date_processed[month_index] = !values.f_nan
        endelse
      endif
    endfor
  endfor
  
  error = 0
  if num_found lt 1 then error = -1 
  log,string(num_found,sat_name(satnum,/fill),format = '("Info: Loaded ",i4," months for ",a)')

end

pro read_all_monthly_maps_amsu_only_v3_3,       $
                            channel_str,        $
                            view_str,           $
                            sats_to_use,        $
                            tb_arr,             $
                            tar_arr,            $
                            num_arr,            $
                            rain_Arr=rain_Arr,  $
                            file_list_all_sats=file_list_all_sats, $
                            no_diurnal=no_diurnal,                 $
                            diurnal_only = diurnal_only,           $
                            date_processed=date_processed,         $
                            diurnal_model=diurnal_model,           $
                            no_nadir=no_nadir

;@***h* MLT_RSS/read_all_monthly_maps_amsu_only_v3_3.pro
;
; NAME
;   read_all_monthly_maps_amsu_only_v3_3.pro
;
; PURPOSE
;   finds and reads all files containing monthly gridded averages for a 
;   channel
;
; DESCRIPTION
;   Reads a number of flat binary file with the monthly averaged data.  Each file is
;   real(4) little endian.
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !AMSU_L2C_Path
;     !Num_AMSU_months
;     !Num_AMSUs
;     !AMSU_year_range
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
;     
;  INPUTS
;     channel_str      is the layer begin processed ('TLT','TMT','TTS','TLS')
;     view_str         is a string that describes the FOV set used for the montly average (N12 = near-nadir 12 views, L8 = near limb 8 views, TLT = TLT weighted views)  
;     sats_to_use      is an int32 array that tell the routine which satellite to read 1 means read the data, 0 means skip this satellite
;     diurnal_model    is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 cliamte model)
;     no_nadir         is a flag.  If set to 1, no nadir adjustment is performed
;     no_diurnal       is a flag.  If set to 1, no diurnal adjustment is performed
;     diurnal_only     is a flag.  If set to 1, only the adjustment is returned

;  OUTPUTS   
;     tb_arr           is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs] of brightness temperatures.
;     tar_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs] of calibration target temperatures.
;     num_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs] of the number of obs in each average.
;     rain_arr         is an optional float32 array of rain flag occurences
;     file_list_all_sats   is the a list of file names that were read   
;     date_processed   is an array of dates (in IDL julday form) that each monthly file was processed
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_all_monthly_maps_amsu_only_v3_3,       $
;                            channel_str,        $
;                            view_str,           $
;                            sats_to_use,        $
;                            tb_arr,             $
;                            tar_arr,            $
;                            num_arr,            $
;                            rain_Arr=rain_Arr,  $
;                            file_list_all_sats=file_list_all_sats, $
;                            no_diurnal=no_diurnal,                 $
;                            diurnal_only = diurnal_only,           $
;                            date_processed=date_processed,         $
;                            diurnal_model=diurnal_model,           $
;                            no_nadir=no_nadir
;                            

;
;@*****
	 case channel_str of
    'TLT' : begin
      implemented_models = ['CCM3','HG1S','NO_DIURNAL']
      end
    'TMT' : begin
      implemented_models = ['CCM3','HG1S','HADGEM','NO_DIURNAL']
    end
    'TTS' : begin
      implemented_models = ['CCM3','HG1S','CMAM','NO_DIURNAL']
    end
    'TLS' : begin
        implemented_models = ['CCM3','HG1S','CMAM','NO_DIURNAL']
    end
  endcase

  if n_elements(diurnal_model) eq 1 then begin
    diurnal_model = strupcase(diurnal_model)
    ok = where(diurnal_model eq implemented_models,ct)
    if ct ne 1 then begin
      log,'Error: Model: ' + diurnal_model + ' not implemented'
      diurnal_model = implemented_models[0]
      log,'Info: Model set to: '+diurnal_model
    endif
  endif else begin
    log,'Error: No (or too many) model specified'
    diurnal_model = implemented_models[0]
    log,'Info: Model set to: '+diurnal_model
  endelse
  
	date_processed = replicate(!values.d_nan,!num_AMSU_months,!num_AMSUs)

	tb_arr 		= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs)
	tar_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs)
	mn_day_arr= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs)
	num_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs)
	rain_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!num_AMSU_months,!num_AMSUs)

	case channel_str of
		'TLT' : begin
			msu_channel = 2
			amsu_channel = 5
			view_str = 'TLT'
			end
		'TMT' : begin
			msu_channel = 2
			amsu_channel = 5
			view_str = 'N12'
		end
		'TTS' : begin
			msu_channel = 3
			amsu_channel = 7
			view_str = 'N12'
		end
		'TLS' : begin
			msu_channel = 4
			amsu_channel = 9
			view_str = 'L8'
    end
	endcase
  diurnal_str = diurnal_model
  if diurnal_model eq 'NO_DIURNAL' then diurnal_str = implemented_models[0]
  
	file_list_all_sats = replicate(' ',!num_AMSU_months,!num_amsus)
	for satnum = 10,10+!num_AMSUS-1 do begin
	  if sats_to_use[satnum-10] gt 0 then begin
		  error = 0
      read_amsu_monthly_histograms_one_sat_ind_files_v3_3,  $
                          amsu_channel,           $
                          satnum,                 $
                          view_str,               $
                          diurnal_str,            $
                          mh1,                    $
                          error,                  $
                          file_list=file_list,    $
                          date_processed=dt_processed
    
        if error eq 0 then begin
	        file_list_all_sats[*,satnum-10] = file_list
	        if keyword_set(dt_processed) then begin
	        	date_processed[*,satnum-10] = dt_processed
	        endif

				  if view_str eq 'TLT' then begin
				      ;zero part of histgrams that contains big errors due to inadvertent spatial derivatives for TLT.  See 
				      ;Mears et al., "Construction of the RSS V3.2 Lower-Tropospheric Temperature Dataset from the MSU and AMSU Microwave Sounders"
				      ;JTECH, 2009, DOI: 10.1175/2009JTECHA1237.1, page 1497 for more details
              mh1[*,0,*,60:71,*] = 0.0
              mh1[*,0,*,59,*] = mh1[*,0,*,59,*]*0.25
              mh1[*,0,*,58,*] = mh1[*,0,*,58,*]*0.5
              mh1[*,0,*,57,*] = mh1[*,0,*,57,*]*0.75
              mh1[*,1,*,0:11,*] = 0.0
              mh1[*,1,*,12,*] = mh1[*,1,*,12,*]*0.25
              mh1[*,1,*,13,*] = mh1[*,1,*,13,*]*0.5
              mh1[*,1,*,14,*] = mh1[*,1,*,14,*]*0.75		
        	endif
        	
        	mh2 = total(mh1,2)  ; combine ascending and descending nodes (or right and left for TLT)
					
					if ((keyword_set(no_diurnal) eq 1) or (diurnal_model eq 'NO_DIURNAL')) then begin
					  tbs = reform((mh2[1,*,*,*])/mh2[0,*,*,*])
					endif else begin
					  if (keyword_set(diurnal_only) eq 1) then begin
					    tbs = reform(mh2[2,*,*,*]/mh2[0,*,*,*])
					  endif else begin
					    if  (keyword_set(no_nadir) eq 1) then begin
					      tbs = reform((mh2[1,*,*,*]+mh2[2,*,*,*])/mh2[0,*,*,*])
              endif else begin
                tbs = reform((mh2[1,*,*,*]+mh2[2,*,*,*]+mh2[3,*,*,*])/mh2[0,*,*,*])
              endelse
					  endelse
					endelse
					tar = reform(mh2[5,*,*,*]/mh2[0,*,*,*])
					tb_Arr[*,*,*,satnum-10] = tbs
					tar_arr[*,*,*,satnum-10] = tar
					num_arr[*,*,*,satnum-10] = mh2[0,*,*,*]
					rain_arr[*,*,*,satnum-10] = mh2[6,*,*,*]
				endif  ; error eq 0
		endif  ; sats_to_use gt 0
	endfor   ; satnum
	

end



