pro calc_offsets_from_grpt_maps,tb_arr_adj,months_to_use,offsets,   $
    plt_offsets=plt_offsets,covar = covar, $
    stddev_resids = stddev_resids

;@***h* MLT_RSS/smooth_offsets.pro
;
; NAME
;   calc_offsets_from_grpt_maps.pro
;
; PURPOSE
;   Calculates latitude-dependent offsets from gridded monthly maps of MSU data.
;
; DESCRIPTION
;   begins with gridded monthly maps of MSU data.  Typically, the target factor adjustment 
;   adjustment has already been applied. Singular value decomposition is used to calculate  
;   offsets for each 2.5 degree latitude band separately from a system of equations 
;   derived from inter satellite differences.
;
; INPUTS
;   tb_arr_adj         (num_lons,num_lats,num_months) float32 array of brightness temperatures
;   months_to_use      (num_sats,num_months) int32 array. 1 means use that month, 0 mean don't use.
; OUTPUTS
;   offsets  	       (num_lats,num_sats) float32 array of fitted offsets
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   calc_offsets_from_grpt_maps,tb_arr_adj,months_to_use,offsets,/plt_offsets
;@*****


	; compute the offsets separately for each latitude

	sigma_est = 1.0
	sz = size(tb_arr_adj,/dimensions)

	num_lons = sz[0]
	num_lats = sz[1]
	num_months = sz[2]
	num_sats = sz[3]

	bad = where(months_to_use eq 0)

	offsets = replicate(!values.f_nan,72,9)
	stddev_resids = replicate(!values.f_nan,72)
	for ilat = 3,68 do begin
		tb_arr_lat = replicate(!values.f_nan,num_months,num_sats)
		for month = 0,num_months-1 do begin
			for satnum = 0,num_sats-1 do begin
				tb_arr_lat[month,satnum] = mean(tb_arr_adj[*,ilat,month,satnum],/nan)
			endfor
		endfor
		tb_arr_lat[bad] = !values.f_nan
		num_col = num_sats
		X = fltarr(num_col,10000L)
		Y = fltarr(10000L)
		row_num = 0

		for month = 0,num_months-1 do begin
			tbs = reform(tb_arr_lat[month,*])
			ok = where(finite(tbs),ct)
			if ct ge 2 then begin
				for j = 0,ct-2 do begin
				  row = fltarr(num_col)
					sat1 = ok[0]
					sat2 = ok[1]
					row[sat1] = 1.0
					row[sat2] = -1.0
					X[*,row_num] = row
					Y[row_num] = tbs[sat1]-tbs[sat2]
					row_num = row_num+1
				endfor
			endif
		endfor

		if row_num gt 10 then begin
			X = X[*,0:row_num-1]
			Y = Y[0:row_num-1]
			sigma_in = replicate(sigma_est,row_num)

			lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct
			offsets[ilat,*] = float(a)
			stddev_resids[ilat] = stddev(y-fit)
		endif else begin
			offsets[ilat,*] = replicate(!values.f_nan,num_sats)
		endelse
	endfor
	if keyword_set(plt_offsets) then begin
		if !D.name eq 'WIN' then window,0
		!p.multi = [0,1,1]
		ok = where(finite(offsets))
		mx = max(offsets[ok])
		mn = min(offsets[ok])
		lats = -88.75+ findgen(72)*2.5
		plot,lats,fltarr(72),yrange=[mn,mx],title='Offsets of a function of latitude, RAW'
		for i = 0,8 do begin
			oplot,lats,offsets[*,i],color=1+i*30
		endfor
	endif
end