pro calc_single_TbF_from_grpt_map,tb_arr_adj,tb_clim_arr,months_to_use,tb_factors,remove_global_mean=remove_global_mean

;@***h* MLT_RSS/smooth_offsets.pro
;
; NAME
;   calc_offsets_from_grpt_maps.pro
;
; PURPOSE
;   Calculates single scene temperature factors for each satellite from gridded monthly maps of MSU data.
;   It then applies this adjustment to tb_arr_adj.
;
; DESCRIPTION
;   begins with gridded monthly maps of MSU data.  Typically, the target factor adjustment 
;   adjustment and offsets have already been applied. Singular value decomposition is used to calculate  
;   a scene temperature factor for each satellite.  The adjustment is the applied to tb_arr_adj.
;
; INPUTS
;   tb_arr_adj         (num_lons,num_lats,num_months) float32 array of brightness temperatures
;   tb_clim_arr        (num_lons,num_lats,12) float32 array of average brightness temperature for each month
;   months_to_use      (num_sats,num_months) int32 array. 1 means use that month, 0 mean don't use.
; OUTPUTS
;   tb_factors         (num_sats) float32 array of scene temperature factors
;   tb_arr_adj         (num_lons,num_lats,num_months) float32 array of brightness temperatures
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   calc_single_TbF_from_grpt_map,tb_arr_adj,tb_clim_arr,months_to_use,tb_factors
;@*****

	sigma_est = 0.3
	sz = size(tb_arr_adj,/dimensions)

	num_lons = sz[0]
	num_lats = sz[1]
	num_months = sz[2]
	num_sats = sz[3]
	tb_clim_amp = replicate(!values.f_nan,72)

	bad = where(months_to_use eq 0)

	tb_factors = replicate(!values.f_nan,9)
	tb_stddev = replicate(!values.f_nan,72)

	num_col = num_sats

	X = fltarr(num_col,20000L)
	Y = fltarr(20000L)
	row_num = 0

	tb_clim_arr_lat = replicate(!values.f_nan,72,12)
	mn_tb_clim_arr = mean(tb_clim_arr,/nan)

	for ilat = 0,71 do begin

		tb_arr_lat = replicate(!values.f_nan,num_months,num_sats)
		for month = 0,num_months-1 do begin
			for satnum = 0,num_sats-1 do begin
				tb_arr_lat[month,satnum] = mean(tb_arr_adj[*,ilat,month,satnum],/nan)
			endfor
		endfor
		tb_arr_lat[bad] = !values.f_nan

		for month = 0,11 do begin
			tb_clim_arr_lat[ilat,month] = mean(tb_clim_arr[*,ilat,month],/nan)
		endfor

		if keyword_set(remove_global_mean) then begin
			tb_clim_arr_lat[ilat,*] = tb_clim_arr_lat[ilat,*] - mn_tb_clim_arr
		endif else begin
			tb_clim_arr_lat[ilat,*] = tb_clim_arr_lat[ilat,*] - mean(tb_clim_arr_lat[ilat,*])
                      ; this is the typical case
		endelse

		tb_clim_amp[ilat] = max(tb_clim_arr_lat[ilat,*]) - min(tb_clim_arr_lat[ilat,*])


		for month = 0,num_months-1 do begin
			tbs = reform(tb_arr_lat[month,*])
			ok = where(finite(tbs),ct)
			if ct ge 2 then begin
				row = fltarr(num_col)
				for j = 0,ct-2 do begin
					sat1 = ok[0]
					sat2 = ok[1]
					if finite(tb_clim_arr_lat[ilat,month mod 12]) then begin
						row[sat1] = tb_clim_arr_lat[ilat,month mod 12]
						row[sat2] = -tb_clim_arr_lat[ilat,month mod 12]
						X[*,row_num] = row
						Y[row_num] = tbs[sat1]-tbs[sat2]
						row_num = row_num+1
					endif
				endfor
			endif
		endfor
	endfor

	X = X[*,0:row_num-1]
	Y = Y[0:row_num-1]
	sigma_in = replicate(sigma_est,row_num)

	lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct
	tb_factors = float(a)

	for ilat = 0,71 do begin

		num_yrs = num_months/12
		tb_clim_ts = reform(tb_clim_arr_lat[ilat,*])
		for yr = 1,num_yrs-1 do begin
			tb_clim_ts = [tb_clim_ts,reform(tb_clim_arr_lat[ilat,*])]
		endfor
		tb_clim_arr_expanded = fltarr(num_lons,num_months)
		for i = 0,num_lons-1 do begin
			tb_clim_arr_expanded[i,*] = tb_clim_ts
		endfor

		for satnum = 0,8 do begin
			tb_arr_adj[*,ilat,*,satnum] = reform(tb_arr_adj[*,ilat,*,satnum])-tb_factors[satnum]*tb_clim_arr_expanded
		endfor
	endfor
end
