pro calc_TF_from_grpt_maps,tb_arr,tar_arr,num_arr,channel,sats_to_use,   $
            target_factor,target_means,offset,months_to_use,             $
            num_thres=num_thres
            
;@***h* MLT_RSS/calc_TF_from_grpt_maps.pro
; NAME
;   calc_TF_from_grpt_maps.pro
;
; PURPOSE
;   calculates MSU target factors directly from the monthly averaged grid point maps. 
;
; DESCRIPTION
;   Calculates the target factors using the method described in 
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
;   
;   and 
;   
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the RSS V3.2 lower tropospheric dataset from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1493-1509.
;
;   The routine calculates the target factors directly from the monthly averaged grid point maps.  The in put data is
; INPUTS
;   tb_arr          (144,72,num_months,num_sats) real(4) array of brightness temperatures
;   tar_arr         (144,72,num_months,num_sats) real(4) array of target temperatures
;   sats_to_use     (9) int32 array, 1 means use this satellite, 0 mean ignore)
;   months_to_use   (num_sats,num_months) array of integers -- 1 means use this month, 0 mean ignore)
;   num_thres        number of observations per cell to make a valid cell
;   channel          3 character channel string (TLT,TMT, etc)
; OUTPUTS
;   target_factor   (num_sats) real(4) array of target factors
;   target_means    (num_sats) real(4) array of global mean traget temperatures
;   offset          (num_sats) float32 array of globa offsets (discarded)
;   months_to_use   (num_sats,num_months) array of integers -- 1 means use this month, 0 mean ignore)


;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
; 
; USAGE
;   calc_TF_from_grpt_maps,tb_arr,tar_arr,num_arr,channel,sats_to_use,   $
;             target_factor,target_means,offset,months_to_use,             $
;             num_thres=num_thres
;
;@*****


	if keyword_set(num_thres) eq 0 then num_thres = 34.0    ; average number of obs per cell must be at least this big
	
	
	sigma_est = 1.0
	lat_range=[-50,50]
	sz = size(tb_arr,/dimensions)

	num_months = sz[2]
	num_sats = sz[3]

	;calculate tropical/global averages for the tb_arr,tar_arr, and the num_arr

	trop_tb_arr = fltarr(num_months,num_sats)
	trop_tar_arr = fltarr(num_months,num_sats)
	trop_num_arr = fltarr(num_months,num_sats)
	months_to_use = intarr(num_months,num_sats)

	lat_wts = latitude_wts(72,lat_range)

  surf_str = 'Land_and_Sea'
	max_alt = 10000
	if channel eq 'TLT' then max_alt = 3000

	for satnum = 0,num_sats-1 do begin
		if sats_to_use[satnum] lt 1 then continue
		for month = 0,num_months-1 do begin
			map = tb_arr[*,*,month,satnum]
			trop_tb_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

			map = tar_arr[*,*,month,satnum]
			trop_tar_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

			map = num_arr[*,*,month,satnum]
			trop_num_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)
		endfor
	endfor

	;remove points with not enough data

	months_to_use = replicate(1,num_months,num_sats)

  bad = where(trop_num_arr lt num_thres,ct)
  months_to_use[bad] = 0

	; remove some other months that seem to be bad

	months_to_use[26,0]  = 0
	months_to_use[324:335,8] = 0
	months_to_use[39,1] = 1

	if channel eq 'TTS' then months_to_use[0:107,*] = 0



	if channel eq 'TLT' then begin
		months_to_use[63,1]  = 0
		;months_to_use[95,1]  = 0
		months_to_use[160,7] = 0
		months_to_use[161,7] = 0
		months_to_use[162,7] = 0
		months_to_use[163,7] = 0
		months_to_use[164,7] = 0
		months_to_use[200,6] = 0
		months_to_use[201,6] = 0
		months_to_use[202,6] = 0
		months_to_use[203,6] = 0
	endif

	if channel eq 'TMT' then begin
		months_to_use[63,1]  = 0
		months_to_use[160,7] = 0
		months_to_use[161,7] = 0
		months_to_use[162,7] = 0
		months_to_use[163,7] = 0
		months_to_use[164,7] = 0
		months_to_use[201,6] = 0
		months_to_use[202,6] = 0
		months_to_use[203,6] = 0
	endif

	if channel eq 'TLS' then begin
		months_to_use[63,1] = 0
		months_to_use[0:168,7] = 0
		months_to_use[200,6] = 0
		months_to_use[201,6] = 0
		months_to_use[202,6] = 0
		months_to_use[203,6] = 0
	endif

    ; now use months to use to remove bad data

	bad = where(months_to_use eq 0)

	;display_months_to_use,months_To_use
	;plot_num_arr,trop_num_Arr,threshold=num_thres

    if ct gt 0 then begin
    	trop_tb_arr[bad] = !values.f_nan
    	trop_tar_arr[bad] = !values.f_nan
    endif

	;remove means from target factors, and save means for passing back up

	target_means = fltarr(9)
	for satnum = 0,num_sats-1 do begin
		tar = trop_tar_arr[*,satnum]
		target_means[satnum] = mean(tar,/nan)
		trop_tar_arr[*,satnum] = trop_tar_arr[*,satnum]-target_means[satnum]
	endfor

	;construct the regression array

	num_col = num_sats*2
	X = fltarr(num_col,10000L)
	Y = fltarr(10000L)
	s1 = intarr(10000L)
	s2 = intarr(10000L)
	m  = intarr(10000L)
	row_num = 0
    zero_wt = 1.0
	for month = 0,num_months-1 do begin
		tbs = reform(trop_tb_arr[month,*])
		ok = where(finite(tbs),ct)
		if ct ge 2 then begin
			for j = 0,ct-2 do begin
			  row = fltarr(num_col)
				sat1 = ok[0]
				sat2 = ok[1]
				row[sat1] = 1.0
				row[sat2] = -1.0
				row[sat1+num_sats] = trop_tar_arr[month,sat1]
				row[sat2+num_sats] = -trop_tar_arr[month,sat2]
				X[*,row_num] = row
				Y[row_num] = tbs[sat1]-tbs[sat2]
				s1[row_num] = sat1
				s2[row_num] = sat2
				m[row_num] = month
				row_num = row_num+1
			endfor
			; experimental == add rows that hold the target factors to
			; lower absolute values
		endif
	endfor

	if channel eq 'TLT' then begin
		for j = 0,8 do begin
			row = fltarr(num_col)
			row[j+num_sats] = zero_wt
			X[*,row_num] = row
			Y[row_num] = 0.0
			m[row_num] = -1
			row_num=row_num+1
		endfor
	endif
	X = X[*,0:row_num-1]
	Y = Y[0:row_num-1]
	s1 = s1[0:row_num-1]
	s2 = s2[0:row_num-1]
	m  = m[0:row_num-1]
	sigma_in = replicate(sigma_est,row_num)

	lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct

	offset = float(a[0:8])
	target_factor = float(a[9:17])
	
	; scale the covariance matrix
	; the original covariance matrix was calculated using a sigma est of 1.0
	; we find a new sigma est from the after the fit residuals
	
	sigma_est_ATF = stddev(y-fit)
	covar = covar*sigma_est_ATF*sigma_est_ATF

end
