

pro read_all_monthly_maps_msu_only_TLT_V3_3,channel_str,tb_arr,tar_arr,mn_day_arr,tb_clim_arr,num_arr,file_list=file_list,  $
        diurnal_only = diurnal_only,no_diurnal=no_diurnal,diurnal_model = diurnal_model

;@***h* MLT_RSS/read_all_monthly_maps_msu_only_v3_3.pro
;
; NAME
;   read_all_monthly_maps_msu_only_v3_3.pro
;
; PURPOSE
;   finds and reads all files containing monthly gridded averages for a 
;   channel
;
; DESCRIPTION
;   Reads a number of flat binary files with the monthly averaged data.  Each file is
;   real(4) little endian.
;
; INPUTS
;   channel_str     3 character string describing the layer begin processed ('TLT','TMT','TTS','TLS')
;   diurnal_model   is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 climate model)  CCM3 is the default model
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !AMSU_L2C_Path
;     !Num_Months
;     !Num_MSUs
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
;   typically, these are set by the routine initialize_msu_variables_w_amsu.
;
; OUTPUTS
;     tb_arr           is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of brightness temperatures.
;     tar_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of calibration target temperatures.
;     num_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of the number of obs in each average.
;     mn_day_arr       is an float32 array of mean day num in month
;     tb_clim_arr      is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,12] of mean climatological brightness temperatures for each month
;     file_list        is the a list of file names that were read   


;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_all_monthly_maps_msu_only_v3_3,channel_str,tb_arr,tar_arr,mn_day_arr,tb_clim_arr,num_arr,file_list=file_list,diurnal_model = diurnal_model

;                            
;
;@*****
	;!num_months = 348
	tb_arr 		= fltarr(144,72,336,9)
	tar_arr 	= fltarr(144,72,336,9)
	mn_day_arr 	= fltarr(144,72,336,9)
	num_arr 	= fltarr(144,72,336,9)
	
	implemented_models = ['CCM3','HADGEM_SAANE','MERRA','NO_DIURNAL']
  if n_elements(diurnal_model) eq 1 then begin
    diurnal_model = strupcase(diurnal_model)
    ok = where(diurnal_model eq implemented_models,ct)
    if ct ne 1 then begin
      print,'Model: ' + diurnal_model + ' not implemented'
      diurnal_model = 'CCM3'
      print,'Model set to: '+diurnal_model
    endif
  endif else begin
    print,'No model specified'
    diurnal_model = 'CCM3'
    print,'Model set to: '+diurnal_model
  endelse

	if channel_str ne 'TLT' then begin
		print,'V3_2 L2C data only meaningful for TLT'
		stop
	endif

	msu_channel = 'TLT'
	restore,file='u:\AMSU-A_L3\Climatology\TLT_monthly_climatology.sav'
	tb_clim_arr = tlt_monthly_clim

    file_list = ' '
	for satnum = 1,9 do begin
    case diurnal_model of 
      'CCM3': begin
          read_msu_monthly_histograms_one_sat_v3_2,msu_channel,satnum,mh1,error,file=file
          end
      'NO_DIURNAL': begin
          read_msu_monthly_histograms_one_sat_v3_2,msu_channel,satnum,mh1,error,file=file
          end 
      'HADGEM_SAANE' : begin
          read_msu_monthly_histograms_one_sat_v3_2_hadgem_saane,msu_channel,satnum,mh1,error,file=file
          end
      'MERRA' : begin
          read_msu_monthly_histograms_one_sat_v3_2_merra,msu_channel,satnum,mh1,error,file=file
          end
      
    endcase
		if error eq 0 then begin
		
			; for V3_2, the second index is left/right, not asc_desc

			file_list = [file_list,file]

			;zero part of histgrams that contains big error

			mh1[*,0,*,60:71,*] = 0.0
			mh1[*,0,*,59,*] = mh1[*,0,*,59,*]*0.25
			mh1[*,0,*,58,*] = mh1[*,0,*,58,*]*0.5
			mh1[*,0,*,57,*] = mh1[*,0,*,57,*]*0.75
			mh1[*,1,*,0:11,*] = 0.0
			mh1[*,1,*,12,*] = mh1[*,1,*,12,*]*0.25
			mh1[*,1,*,13,*] = mh1[*,1,*,13,*]*0.5
			mh1[*,1,*,14,*] = mh1[*,1,*,14,*]*0.75

			mh2 = total(mh1,2)
	        ; 0 -- num obs
	        ; 1 -- tbs
	        ; 2 -- diurnal corrections
	        ; 3 -- incidence correction
	        ; 4 -- nadir correction
	        ; 5 -- target temperature
	        ; 6 -- day of month

			; we do not include the nadir correction here --

	    tbs = reform((mh2[1,*,*,*]+mh2[2,*,*,*]+mh2[3,*,*,*])/mh2[0,*,*,*])

			if keyword_Set(diurnal_only) then begin
	    		tbs = reform(mh2[2,*,*,*]/mh2[0,*,*,*])
	    	endif

	    	if keyword_set(no_diurnal) then begin
	    		tbs = reform((mh2[1,*,*,*]+mh2[3,*,*,*])/mh2[0,*,*,*])
	    	endif

			tar = reform(mh2[5,*,*,*]/mh2[0,*,*,*])
			mn_day = reform(mh2[6,*,*,*]/mh2[0,*,*,*])

			tb_Arr[*,*,*,satnum-1] = tbs
			tar_arr[*,*,*,satnum-1] = tar
			mn_day_arr[*,*,*,satnum-1] = mn_day
			num_arr[*,*,*,satnum-1] = mh2[0,*,*,*]
		endif
	endfor
	file_list= file_list[1:*]
end



