

pro read_all_monthly_maps_msu_only_v3_3,channel_str,tb_arr,tar_arr,mn_day_arr,tb_clim_arr,num_arr,file_list=file_list,diurnal_model = diurnal_model

;@***h* MLT_RSS/read_all_monthly_maps_msu_only_v3_3.pro
;
; NAME
;   read_all_monthly_maps_msu_only_v3_3.pro
;
; PURPOSE
;   finds and reads all files containing monthly gridded averages for a 
;   channel
;
; DESCRIPTION
;   Reads a number of flat binary file with the monthly averaged data.  Each file is
;   real(4) little endian.
;
; INPUTS
;   channel_str     3 character string describing the layer begin processed ('TLT','TMT','TTS','TLS')
;   diurnal_model   is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 climate model)  CCM3 is the default model
;   
;   The global variables the describe the size of this file must already be set.  These are:
;     !AMSU_L2C_Path
;     !Num_Months
;     !Num_MSUs
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
;
; OUTPUTS
;     tb_arr           is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of brightness temperatures.
;     tar_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of calibration target temperatures.
;     num_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of the number of obs in each average.
;     mn_day_arr       is an float32 array of mean day num in month
;     tb_clim_arr      is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,12] of mean climatological brightness temperatures for each month
;     file_list        is the a list of file names that were read 
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_all_monthly_maps_msu_only_v3_3,channel_str,tb_arr,tar_arr,mn_day_arr,tb_clim_arr,num_arr,file_list=file_list,diurnal_model = diurnal_model

;                            
;     channel_str      is the layer begin processed ('TLT','TMT','TTS','TLS')
;     tb_arr           is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of brightness temperatures.
;     tar_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of calibration target temperatures.
;     num_arr          is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,!num_months,!num_MSUs] of the number of obs in each average.
;     mn_day_arr       is an float32 array of mean day num in month
;     tb_clim_arr      is a float32 array[!Num_GrPt_Lons,!Num_GrPt_Lats,12] of mean climatological brightness temperatures for each month
;     file_list        is the a list of file names that were read   
;     diurnal_model    is a string that describes the diurnal adjustment used ('CCM3' means the CCM3 climate model)
;
;@*****

	tb_arr 		= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!Num_Months,!Num_MSUs)
	tar_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!Num_Months,!Num_MSUs)
	mn_day_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!Num_Months,!Num_MSUs)
	num_arr 	= fltarr(!Num_GrPt_Lons,!Num_GrPt_Lats,!Num_Months,!Num_MSUs)
	
	diurnal_model = 'CCM3'

	case channel_str of
		'TMT' : begin
			msu_channel = 2
			amsu_channel = 5
			restore,file='M:\MSU_AMSU_Data\AMSU-A_L3\Climatology\TMT_monthly_climatology.sav'
			tb_clim_arr = tmt_monthly_clim
		end
		'TLS' : begin
			msu_channel = 4
			amsu_channel = 9
			restore,file='M:\MSU_AMSU_Data\AMSU-A_L3\Climatology\TLS_monthly_climatology.sav'
			tb_clim_arr = tls_monthly_clim
		end
		'TTS' : begin
			msu_channel = 3
			amsu_channel = 7
			restore,file='M:\MSU_AMSU_Data\AMSU-A_L3\Climatology\TTS_monthly_climatology.sav'
			tb_clim_arr = tts_monthly_clim

		end
	endcase
	
	bad = where(tb_clim_Arr lt 100.0,ct)
	if ct gt 0 then tb_clim_arr[bad] = !values.f_nan

  file_list = ' '
	for satnum = 1,!Num_MSUs do begin
    read_msu_monthly_histograms_one_sat,msu_channel,satnum,mh1,error,file=file

			file_list = [file_list,file]
			mh2 = total(mh1,2)
			    ; First index:
	        ; 0 -- num obs
	        ; 1 -- tbs
	        ; 2 -- diurnal corrections
	        ; 3 -- incidence correction
	        ; 4 -- nadir correction
	        ; 5 -- target temperature
	        ; 6 -- day of month

			; we do not include the nadir correction here --
			num = reform(mh2[0,*,*,*])
	    tb = reform((mh2[1,*,*,*]+mh2[2,*,*,*]+mh2[3,*,*,*])/mh2[0,*,*,*])
			tar = reform(mh2[5,*,*,*]/mh2[0,*,*,*])
			mn_day = reform(mh2[6,*,*,*]/mh2[0,*,*,*])

			tb_Arr[*,*,*,satnum-1] = tb
			tar_arr[*,*,*,satnum-1] = tar
			mn_day_arr[*,*,*,satnum-1] = mn_day
			num_arr[*,*,*,satnum-1] = num
	endfor
	file_list= file_list[1:*]
end



