function compute_global_average_from_144_72_map,map,lat_wts,surf_str = surf_str,no_cosine = no_cosine,max_alt=max_alt

;@***h* MLT_RSS/compute_global_average_from_144_72_map.pro
; NAME
;   compute_global_average_from_144_72_map.pro
;
; PURPOSE
;   calculates the global average of a 144 x 72 map
;   
; DESCRIPTION
;   calculates the global average of a 144 x 72 map, with parameters
;   for latitude weights, surface_type, or cosine lat weighting
;   
; INPUTS
;   map             (144,72) float32 array.  This is assume to be a 2.5x2.5 degree map, RSS conventions.
;   lat_wts         (72) float32 array of latitude weights.
;   surf_str        string that describes the surface tpye to average over -- choices are 'Land','Sea','Land_and_Sea'
;                   default is 'Land_and_Sea'
;   no_cosine       keyword if set do not apply cosine latitude weighting (for example, if this weighting is already included in lat_wts)
;   channel         max_alt.  if set to a positive number, only use grid points with average altitudes below that number.
; RETURN VALUE
;   global average value

;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
; 
; USAGE
;   global_average = compute_global_average_from_144_72_map(map,lat_wts=replicate(1.0,72),surf_str = 'Land',/no_cosine,max_alt=3000
;
;@*****
	common compute_global_average_from_144_72_map_static,lf,hgt

	if keyword_set(lf) eq 0 then begin
		read_land_fraction_144_72,lf
		read_topography_144x72,hgt
	endif
    compute_cos = 1
	if (keyword_set(surf_str) eq 0) then surf_str = 'Land_and_Sea'
	if (keyword_set(no_cosine) eq 1) then compute_cos = 0


	lat = (indgen(72)-35.5)*2.5
	cos_wt = cos(!pi*lat/180.0)

	wt_map = fltarr(144,72)
	num_lons = 144

	for ilon = 0,num_lons -1 do begin
		if compute_cos then begin
			wt_map[ilon,*] = lat_wts*cos_wt
		endif else begin
			wt_map[ilon,*] = lat_wts
		endelse
	endfor

	case surf_str of
		'Sea' 		:	begin
							wt_map = wt_map*(1.0 - lf)
						end
		'Land'		:	begin
						   wt_map = wt_map*lf
						end
		'Land_and_Sea'	:	begin

						end
	endcase

	if keyword_set(max_alt) then begin
		bad = where(hgt gt max_alt,ct)
		if ct gt 0 then begin
			wt_map[bad] = 0.0
		endif
	endif

	ok = where((finite(map) eq 1) and (abs(wt_map) gt 0.0),ct)

	if ct gt 0 then begin
		wt_mean = total(map[ok]*wt_map[ok])/total(wt_map[ok])
	endif else begin
		wt_mean = !values.f_nan
	endelse

	return,wt_mean

end



