pro lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,$
                    w_thres = w_thres,trunc_ct = trunc_ct

;@***h* MLT_RSS/lin_regress_SVD.pro
;
; NAME
;   lin_regress_SVD
; 
; PURPOSE
;   solves a set of linear equations using singular value decomposition
;
; DESCRIPTION
;   this routine used singular value decomposition to solve the set of linear
;   equations given by
;
;    X . A = Y
;    
;    X is a m by n array representing n equations in m unknowns
;    Y is a vector of length n
;    A is the least squares svd solution to the system of equations
;
;   sigma_in  	: expected errors in Y
;   sigma out		: errors in A
;   fit			    : best fit to Y
;   covar			  : covariance matrix for A
;   error			  : value *not* equal to zero if SVD failed, 0 if successful
;   w_thres		  : lower threshold for truncating singlular values (default value = 1.0e-6)
;   trunc_ct		: # of singular values set to zero
;
;   calculations are done in double precision.
;   results are returned in double precision complex.
;   
;   Based on Press, Teukolsky, Vetterling, Flaanery, Numerical Recipes in C, Cambridge University Press,
;   Cambridge, UK, 1997
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   log,'Info: AMSU Only V3.3 log file',/quiet,lun=23
;
;@***** MLT_RSS/lin_regress_SVD.pro

	if keyword_set(w_thres) eq 0 then w_thres = 1.0e-6

	sigma_in = double(sigma_in)

	size_y = size(Y)
	size_x = size(X)

	num_terms = size_x[1]
	num_pts   = size_x[2]

	design_mat = double(X)
	B = Y/sigma_in
	for n = 0,num_terms-1 do begin
		design_mat[n,*] = design_mat[n,*]/sigma_in
	endfor

	LA_SVD,design_mat,W,U,V,/double,status = error

	; look for small values in w, if too small, set to zero

	small = where(w lt 1.0e-6,trunc_ct)
	if trunc_ct gt 0 then w[small] = 0.0

	; back substitute to recover the solution

	; calc Utrans * b
	Utransb = dcomplexarr(n_elements(w))
	for i = 0,n_elements(w)-1 do begin
		if w[i] gt 0.0 then begin
			Utransb[i] = (1.0/w[i])*conj(u[i,*]) # b
		endif
	endfor

	a = v ## utransb

	;evaluate covariance matrix

	w_inv = 1.0/w
	if trunc_ct gt 0 then w_inv[small] = 0.0
	w_inv = diag_matrix(w_inv)
	covar = transpose(w_inv#V)#(w_inv#V)

	; calculate fitted value for Y

  fit = reform(x ## a)

	; calculate errors in A

  sigma_out = sqrt(DIAG_MATRIX(covar))

end
