;@***h* MLT_RSS/convert_to_netcdf4.pro
;
; NAME
;   convert_to_netcdf4.pro
;
; PURPOSE
;   Top level driver program for converting output files to netcdf4 with approved metadata.
;
; DESCRIPTION
;   Top level driver program for converting output files to netcdf4 with approved metadata.
;   Reads RSS internal version (in netcdf3), computes climatology and anomaly maps, and writes
;   separate netcdf4 files for means and anomalies.
;
; FILE READ:
;   merged msu/amsu data file in netcdf format
;   e.g. RSS_Tb_Maps_ch_TLT_V3_3_001.nc
;        RSS_Tb_Maps_ch_TMT_V3_3_001.nc
;        RSS_Tb_Maps_ch_TTS_V3_3_001.nc
;        RSS_Tb_Maps_ch_TLS_V3_3_001.nc
;        
; FILES WRITTEN:
;   netcdf files containing msu amsu merged data
;   e.g.  uat4_tb_v03r03_anom_chTLT_197812_201212.nc
;         uat4_tb_v03r03_avrg_chTLT_197812_201212.nc
;         uat4_tb_v03r03_anom_chTMT_197812_201212.nc
;         uat4_tb_v03r03_avrg_chTMT_197812_201212.nc
;         uat4_tb_v03r03_anom_chTTS_197812_201212.nc
;         uat4_tb_v03r03_avrg_chTTS_197812_201212.nc
;         uat4_tb_v03r03_anom_chTLS_197812_201212.nc
;         uat4_tb_v03r03_avrg_chTLS_197812_201212.nc
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/24/2012 Initial Version prepared for NCDC
;
; USAGE
;   top level program invoked by .r command in IDL environment
;
;
;@*****
  MSU_AMSU_L3_path = 'M:\MSU_AMSU_Data\AMSU-A_L3\'
  output_path = 'M:\MSU_TB_CDR\'

  print,"output_path = "+output_path

  input_files = [MSU_AMSU_L3_path +'MSU_AMSU\RSS_Tb_Maps_ch_TLT_V3_3_001.nc', $ ; Channel TLT file
                 MSU_AMSU_L3_path +'MSU_AMSU\RSS_Tb_Maps_ch_TMT_V3_3_001.nc', $ ; Channel TMT file
                 MSU_AMSU_L3_path +'MSU_AMSU\RSS_Tb_Maps_ch_TTS_V3_3_001.nc', $ ; Channel TTS file
                 MSU_AMSU_L3_path +'MSU_AMSU\RSS_Tb_Maps_ch_TLS_V3_3_001.nc']  ; Channel TLS File
  
  channel_strings = ['TLT','TMT','TTS','TLS']
  
  for channel_index = 0,3 do begin
      netcdf_file = input_files[channel_index]

      read_monthly_msu_tbs_netcdf,    file = netcdf_file,             $
                        longitude = longitude,          $
                        latitude  = latitude,         $
                        long_bounds = long_bounds,        $
                        lat_bounds  = lat_bounds,       $
                        months = months,            $
                        years  = years,             $
                        mean_tb_maps = mean_tb_maps,      $
                        offsets=offsets,            $
                        target_factors=target_factors,      $
                        tb_Factors=tb_factors,          $
                        satellites_used = satellites_used,    $
                        date_L2C_processed = date_L2C_processed,$
                        global_Attributes = glb_attr,       $
                        msu_amsu_diff = msu_amsu_diff
     
          calc_monthly_climatology,     monthly_tb_maps=mean_tb_maps,       $
                      begin_month=12,        $
                      end_month=371,         $
                      monthly_clim=clim_tb_maps
          
          sz = size(mean_tb_maps,/dimensions)
          
          anom_tb_maps = replicate(!values.f_nan,sz[0],sz[1],sz[2])
          
          for m = 0,sz[2]-1 do begin
            anom_tb_maps[*,*,m] = mean_tb_maps[*,*,m] - clim_tb_maps[*,*,m mod 12]
          endfor
          
          ; figure out first and last month of data
          first_valid_month = -99
          last_valid_month = -99
          help,satellites_used
          for m = 0,sz[2]-1 do begin
              mp = mean_tb_maps[*,*,m]
              ok = where(finite(mp) eq 1,ct)
              if ct gt 1000 then begin
                if first_valid_month eq -99 then begin
                  first_valid_month = m
                endif
                last_valid_month = m
              endif
          endfor
          
          start_year = 1978 + first_valid_month/12
          start_month = 1+(first_valid_month mod 12)
          end_year = 1978 + last_valid_month/12
          end_month = 1+(last_valid_month mod 12)
                        
     write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC, Tbs = mean_tb_maps,Clim_Tbs = clim_tb_maps ,     $
                          channel_str=channel_strings[channel_index],     $
                          offsets = offsets,                              $
                          target_factors = target_factors,                $
                          tb_Factors = tb_factors,                        $
                          satellites_used = satellites_used,              $
                          date_L2C_processed = date_L2C_processed,        $
                          global_Attributes = global_attributes,          $
                          msu_global_Attributes=msu_global_Attributes,   $
                          amsu_global_Attributes=amsu_global_Attributes, $
                          msu_amsu_diff=msu_amsu_diff, $
                          ;nc_file = nc_file, $
                          nc4 = 1,data_Type = 'avrg',$
                          start_year=start_year,start_month=start_month,end_year = end_year,end_month=end_month, $
                          first_valid_month=first_valid_month,last_valid_month = last_valid_month
                          
                          
     write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC, Tbs = anom_tb_maps,Clim_Tbs = clim_tb_maps ,     $
                          channel_str=channel_strings[channel_index],      $
                          offsets = offsets,          $
                          target_factors = target_factors,  $
                          tb_Factors = tb_factors,      $
                          satellites_used = satellites_used,  $
                          date_L2C_processed = date_L2C_processed,  $
                          global_Attributes = global_attributes,    $
                          msu_global_Attributes=msu_global_Attributes,   $
                          amsu_global_Attributes=amsu_global_Attributes, $
                          msu_amsu_diff=msu_amsu_diff, $
                          ;nc_file = nc_file, $
                          nc4 = 1,data_Type = 'anom',$
                          start_year=start_year,start_month=start_month,end_year = end_year,end_month=end_month, $
                          first_valid_month=first_valid_month,last_valid_month = last_valid_month    
  
  endfor
  
  end